#!/usr/bin/env python
#
# Generate list of include files for SWIG interface file.
#
# Copyright (C) 2007 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by <aasmund@simula.no>
# Modified by Johan Hake, 2009
# Modified by Kristian B. Oelgaard, 2010
#
# First added:  2007-04-12
# Last changed: 2010-09-03

import os
import re

# List of headers to exclude (add more here)
excludes = ["plot.h", "ParameterSystem.h", "ParameterList.h", \
            "ConvectionMatrix.h", "MassMatrix.h", "StiffnessMatrix.h", "LoadVector.h", \
            "IntersectionOperatorImplementation.h" ]

# Name of SWIG interface file to be generated
interface_file = "kernel_modules.i"

# Extract modules from dolfin.h
modules = []
f = open("../dolfin.h")
for line in f:
    if "#include <dolfin/" in line and line[:17] == "#include <dolfin/":
        module = line.split("/")[1]
        modules += [module]
f.close()

# Extract header files
headers = []
docstring_headers = [] # Added for docstring extraction
for module in modules:
    module_headers = []
    print "Processing dolfin_%s.h..." % module
    f = open("../%s/dolfin_%s.h" % (module, module))
    module_base = "../" + module + "/"  # Added for docstring extraction
    for line in f:
        if re.search("^#include ",line):
            header = line.split()[1].replace("<", "").replace(">", "")
            # get just the file name (after last /) and check against excludes:
            if not header.split("/")[-1] in excludes:
                module_headers += [header]
                docstring_headers += [module_base + os.path.basename(header)]  # Added for docstring extraction
    f.close()
    headers += [(module, module_headers)]

# Generate list of header files
print "Generating file %s" % interface_file
f = open(interface_file, "w")
f.write("// Generated list of include files for PyDOLFIN\n")
for (module, module_headers) in headers:
    f_import = open(os.path.join("import", module + ".i"), "w")
    f_import.write("// Auto generated import statements for the SWIG kernel module: '%s'\n\n"% module)
    f.write("\n// DOLFIN headers included from %s\n" % module)
    if os.path.isfile(module+"_pre.i"):
        f.write("%%include \"dolfin/swig/%s_pre.i\"\n" % module)
    for header in module_headers:
        f.write("%%include \"%s\"\n" % header)
        f_import.write('%%import(module="dolfin.cpp") "%s"\n'%header)
    if os.path.isfile(module+"_post.i"):
        f.write("%%include \"dolfin/swig/%s_post.i\"\n" % module)
    f_import.close()
f.close()

# Create docstrings.i file from docstrings module (only for dolfin.cpp)
from documentation import generate_docstrings
generate_docstrings()

# Extract all shared_ptr stored classes and store them in a pyton module
# and place that under dolfin.compilemodeuls.sharedptrclasses.py
shared_ptr_classes = re.findall("%shared_ptr\(dolfin::(.+)\)", \
                                open("shared_ptr_classes.i").read())

shared_ptr_classes = filter(lambda x: "NAME" not in x, shared_ptr_classes)
template = """
'''
This module contains the names of the classes in DOLFIN that is
stored using shared_ptrs. The file is automatically generated by the
generate.py script in the dolfin/swig directory.
'''

__all__ = ['shared_ptr_classes']

shared_ptr_classes = %s
"""

par = os.path.pardir
open(os.path.join(par, par, "site-packages", "dolfin", \
                  "compilemodules", "sharedptrclasses.py"), "w").write(\
    template%(repr(shared_ptr_classes)))
