// This code conforms with the UFC specification version 2.0.2
// and was automatically generated by FFC version 1.0-beta+.
//
// This code was generated with the option '-l dolfin' and
// contains DOLFIN-specific wrappers that depend on DOLFIN.
// 
// This code was generated with the following parameters:
// 
//   cache_dir:                      ''
//   convert_exceptions_to_warnings: False
//   cpp_optimize:                   False
//   cpp_optimize_flags:             '-O2'
//   epsilon:                        1e-14
//   error_control:                  False
//   form_postfix:                   True
//   format:                         'dolfin'
//   log_level:                      10
//   log_prefix:                     ''
//   no_ferari:                      True
//   optimize:                       True
//   output_dir:                     '.'
//   precision:                      15
//   quadrature_degree:              'auto'
//   quadrature_rule:                'auto'
//   representation:                 'auto'
//   split:                          False
//   swig_binary:                    'swig'
//   swig_path:                      ''

#ifndef __POISSON3D_5_H
#define __POISSON3D_5_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class poisson3d_5_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  poisson3d_5_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the topological dimension of the cell shape
  virtual unsigned int topological_dimension() const
  {
    return 3;
  }

  /// Return the geometric dimension of the cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = x[3][0] + x[2][0] + x[1][0] - x[0][0];
    const double C1 = x[3][1] + x[2][1] + x[1][1] - x[0][1];
    const double C2 = x[3][2] + x[2][2] + x[1][2] - x[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (d_00*(2.0*coordinates[0] - C0) + d_10*(2.0*coordinates[1] - C1) + d_20*(2.0*coordinates[2] - C2)) / detJ;
    double Y = (d_01*(2.0*coordinates[0] - C0) + d_11*(2.0*coordinates[1] - C1) + d_21*(2.0*coordinates[2] - C2)) / detJ;
    double Z = (d_02*(2.0*coordinates[0] - C0) + d_12*(2.0*coordinates[1] - C1) + d_22*(2.0*coordinates[2] - C2)) / detJ;
    
    
    // Reset values.
    *values = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923505, 0.00609788120679253, 0.00352061335622801, 0.00248944957812478, 0.00705946233325035, 0.00546823600997146, 0.0044647960058476, 0.0031570875323494, 0.00257775117585286, 0.00182274533665715, -0.00298267418725715, -0.00252081977982932, -0.00218309396769448, -0.00195261860521384, -0.00169101731601733, -0.00138070985681772, -0.0011273448773449, -0.00097630930260694, -0.000797153207506489, -0.000563672438672444, 0.00278687112460006, 0.00245778931055954, 0.00219831358911339, 0.00207721109304566, 0.00185791408306669, 0.00160900079398463, 0.00160900079398464, 0.00143913406048031, 0.00124632665582739, 0.00101762145320215, 0.000928957041533368, 0.000830884437218251, 0.000719567030240159, 0.000587524019939407, 0.000415442218609133, -0.00705593524717798, -0.00638233357089079, -0.00582624677716565, -0.00562868913761196, -0.00513826668309062, -0.00459580543596521, -0.00475711057319694, -0.00434262794914382, -0.00388416451811044, -0.00336378514516178, -0.00368484200521178, -0.00336378514516176, -0.00300866089851429, -0.00260557676948629, -0.00212744452363027, -0.00212744452363026, -0.00194208225905519, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828762};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923493, -0.00609788120679249, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, -0.00546823600997144, -0.00446479600584757, 0.00315708753234942, 0.00257775117585287, 0.00182274533665715, 0.00298267418725716, -0.00252081977982929, -0.00218309396769448, 0.00195261860521383, 0.00169101731601732, 0.00138070985681771, -0.00112734487734489, -0.000976309302606925, -0.000797153207506475, -0.000563672438672438, 0.00278687112460003, -0.00245778931055953, -0.0021983135891134, 0.00207721109304565, 0.00185791408306669, 0.00160900079398462, -0.00160900079398462, -0.0014391340604803, -0.0012463266558274, -0.00101762145320215, 0.000928957041533351, 0.000830884437218258, 0.000719567030240152, 0.000587524019939402, 0.000415442218609129, 0.00705593524717798, -0.00638233357089081, -0.00582624677716566, 0.00562868913761197, 0.00513826668309062, 0.0045958054359652, -0.00475711057319694, -0.00434262794914383, -0.00388416451811044, -0.00336378514516178, 0.00368484200521179, 0.00336378514516178, 0.0030086608985143, 0.0026055767694863, 0.00212744452363027, -0.00212744452363027, -0.00194208225905521, -0.00173705117965753, -0.00150433044925715, -0.00122828066840393, -0.000868525589828762};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923494, 0.0, -0.00704122671245613, 0.00248944957812474, 0.0, 0.0, 0.0, 0.0094712625970483, -0.00515550235170573, 0.00182274533665714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0045093795093795, -0.00292892790782074, 0.00159430641501297, -0.000563672438672427, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00464478520766671, -0.00332353774887303, 0.00215870109072045, -0.0011750480398788, 0.000415442218609134, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, -0.0097104112952761, 0.00694820471863011, -0.00451299134777143, 0.00245656133680785, -0.000868525589828767};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923491, 0.0, 0.0, -0.00746834873437434, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0109364720199429, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00563672438672436, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00623163327913695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0, 0.03747187483114, -0.0289614183593723, 0.0, 0.0, 0.0, 0.0284137877911448, 0.00515550235170568, -0.00619733414463431, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0315656565656565, -0.0113902751970807, 0.0026571773583549, -0.000187890812890804, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0232239260383337, -0.00166176887443655, -0.00647610327216137, 0.00646276421933346, -0.00290809553026392, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0213629048496074, -0.0277928188745204, 0.0243701532779658, -0.0157219925555703, 0.00607967912880135};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, 0.0, -0.0287574853355261, 0.05360943398437, 0.0, 0.0, 0.0, -0.0189425251940965, 0.0154665070551172, 0.0153110608279201, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00450937950937951, 0.0370997534990628, -0.0281660799985623, 0.0108976671476671, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.00431740218144093, -0.0123380044187275, 0.00872428659079177, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338356, -0.0487403065559316, 0.041270230458372, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 6:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, 0.0, 0.0409576306293853, -0.0449826285156207, 0.0, 0.0, 0.0, 0.0189425251940965, -0.0051555023517058, -0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00455611007883225, 0.0356061766019561, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0302218152700864, 0.00411266813957586, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0379091273212801, -0.0550269739444961, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 7:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, 0.0, -0.0148144621425436, 0.0449826285156208, 0.0, 0.0, 0.0, 0.0, 0.0206220094068229, 0.021143845905223, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0106287094334197, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.020563340697879, 0.0145404776513197, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.03439185871531, -0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 8:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00446479600584752, 0.00947126259704829, -0.00257775117585286, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, -0.00848980987436738, 0.0136683302364968, 0.00657617845117846, 0.00230118309469618, -0.00789141414141413, -0.0037967583990269, -0.00132858867917748, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, -0.00109915679455667, 0.0103860554652283, 0.000928957041533345, -0.00482700238195389, -0.00804500396992315, -0.000719567030240148, 0.0037389799674822, 0.00559691799261188, 0.00464478520766673, 0.000415442218609139, -0.00215870109072047, -0.0032313821096667, -0.00290809553026391, 0.0, 0.0, 0.0128177429097644, 0.0, -0.0113041867027994, -0.0183832217438608, 0.0, 0.0095537814881164, 0.0155366580724417, 0.0181644397838736, 0.0, -0.00740032731935591, -0.0120346435940572, -0.014070114555226, -0.0136156449512337, 0.0, 0.00427258096992147, 0.00694820471863013, 0.00812338442598859, 0.00786099627778514, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 9:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734316, -0.0249047128495238, 0.014378742667763, 0.0536094339843698, -0.0141189246665007, 0.0109364720199429, 0.0133943880175427, -0.00631417506469885, -0.00773325352755854, 0.0153110608279201, -0.00298267418725716, 0.00252081977982933, 0.0276525235907967, -0.00195261860521385, -0.0214195526695527, -0.0243925408037797, 0.00112734487734488, 0.0123665844996876, 0.0140830399992812, 0.0108976671476671, 0.0, 0.0, 0.0197848223020205, 0.0, -0.0167212267476003, -0.00321800158796927, 0.0, 0.0129522065443227, 0.00249265331165477, -0.0106850252586226, 0.0, -0.00747795993496439, -0.00143913406048031, 0.00616900220936372, 0.00872428659079175, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, 0.0135389740433143, 0.0281402291104519, 0.0357410679969885, 0.0, 0.0, -0.00781673030845889, -0.0162467688519772, -0.020635115229186, -0.018239037386404};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 10:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0354703486038674, -0.0204788153146928, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, -0.00446479600584748, 0.00631417506469886, 0.00257775117585284, -0.021143845905223, 0.0, 0.0, 0.00339592394974695, 0.0, -0.00263047138047139, 0.0308358534689291, 0.0, 0.00151870335961076, -0.017803088300978, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, -0.0174485731815836, 0.00356167508620753, 0.0, 0.0, 0.0100739384233621, -0.00205633406978791, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, -0.0218868448636848, -0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 11:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0128297005588457, 0.00740723107127185, 0.0449826285156207, 0.0, 0.0, 0.0178591840233903, 0.0, -0.0103110047034114, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00920473237878477, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489396, 0.0145404776513197, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0297842233308238, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 12:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0324515955311978, -0.0335503995581136, -0.00801060507812418, 0.0211783869997512, 0.00109364720199429, -0.0169662248222207, -0.00568275755822894, 0.00567105258687627, 0.00838462854862288, 0.0208787193108001, -0.0109235523792603, -0.0162519217595033, 0.00499002532443536, 0.00958243145743147, 0.0108922666482287, -0.00187890812890812, -0.00466458889023306, -0.00558007245254536, -0.00469727032227032, 0.0139343556230002, -0.00245778931055953, -0.0120907247401236, -0.00415442218609133, 0.00464478520766674, 0.00965400476390776, 0.00643600317593851, -0.000719567030240141, -0.00498530662330958, -0.006614539445814, -0.00464478520766674, -0.000415442218609131, 0.00215870109072044, 0.00323138210966672, 0.00290809553026393, 0.0, 0.0127646671417816, -0.00116524935543313, -0.0202632808954031, -0.00719357335632686, 0.00183832217438609, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, -0.020635115229186, -0.01143686949355, -0.00481385743762287, -0.000521115353897263, 0.00170195561890421, 0.0127646671417816, 0.00737991258440982, 0.00347410235931507, 0.00090259826955429, -0.000491312267361578, -0.000868525589828769};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 13:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, -0.0249047128495238, 0.0553363732971482, -0.00431340273437461, -0.0141189246665008, 0.0153110608279201, 0.00803663281052558, 0.00883984509057838, -0.0159820572902877, 0.000729098134662845, -0.00298267418725716, 0.0277290175781224, -0.00145539597846296, -0.0301571095694136, -0.0116492303992304, 0.00352848074520083, 0.0191648629148629, 0.00976309302606916, 0.00132858867917747, -0.00263047138047138, 0.0, 0.0196623144844763, -0.00219831358911338, -0.00727023882565981, -0.0148633126645336, 0.00402250198496157, -0.00724050357293084, 0.0129522065443227, 0.00685479660705067, -0.00457929653940971, 0.00928957041533346, -0.00664707549774609, -0.0057565362419212, -0.00029376200996971, 0.00290809553026395, 0.0, 0.0, 0.0, 0.0202632808954031, -0.00411061334647249, 0.000459580543596519, -0.0399597288148543, -0.00303983956440069, 0.00427258096992149, -0.00100913554354853, 0.0464290092656685, 0.00908221989193678, -0.00270779480866285, -0.00234501909253766, 0.00127646671417817, -0.031911667854454, -0.00776832903622087, 0.000868525589828756, 0.00180519653910858, 0.000245656133680792, -0.000868525589828773};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 14:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0354703486038674, -0.0492363006502187, -0.00431340273437462, 0.0141189246665007, -0.00656188321196576, -0.00982255121286465, -0.016416855168217, 0.0149509568199466, 0.00072909813466287, 0.0, 0.0033610930397724, -0.00048513199282101, 0.0282044909641998, -0.0124007936507936, 0.00291483191994851, -0.0353234728234728, 0.00889526253486301, 0.00239145962251945, -0.00263047138047138, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920009, 0.00080450039699231, -0.00402250198496154, -0.014391340604803, 0.00934744991870548, -0.00254405363300537, -0.0092895704153335, 0.0141250354327105, -0.00287826812096064, -0.00381890612960613, 0.00290809553026391, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320202, 0.00233049871086625, -0.000336378514516179, -0.0515877880729651, 0.00840946286290446, 0.00300866089851429, -0.00286613444643492, 0.000850977809452107, 0.0425488904726053, -0.00388416451811045, -0.00347410235931507, 0.00120346435940572, 0.000982624534723146, -0.000868525589828758};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 15:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0128297005588457, 0.0448791059024117, -0.00801060507812427, 0.0, 0.0174983552319086, -0.00357183680467801, 0.0132597676358676, -0.0175287079957995, 0.00838462854862288, 0.0, 0.0, 0.0, 0.00867830491206147, -0.00300625300625301, 0.000613648825252317, 0.0225468975468975, -0.019200749617936, 0.0122230158484327, -0.00469727032227032, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0160900079398463, -0.00719567030240153, 0.0024926533116548, -0.000508810726601086, 0.00464478520766675, -0.0108014976838374, 0.0107935054536023, -0.00734405024924252, 0.00290809553026394, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0257938940364826, -0.0134551405806471, 0.00601732179702859, -0.00208446141558904, 0.000425488904726059, -0.031911667854454, 0.0135945758133865, -0.00347410235931504, -0.000902598269554292, 0.0017195929357655, -0.000868525589828772};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 16:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734313, -0.0324515955311979, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, -0.00446479600584756, 0.00947126259704828, -0.00257775117585285, -0.00619733414463429, -0.0208787193108001, -0.0176457384588052, -0.00848980987436741, -0.0136683302364968, -0.00657617845117842, -0.00230118309469618, -0.00789141414141414, -0.00379675839902689, -0.00132858867917748, -0.000187890812890818, 0.0139343556230002, 0.0122889465527977, 0.00109915679455668, 0.0103860554652283, 0.000928957041533349, -0.0048270023819539, 0.00804500396992312, 0.00071956703024016, -0.0037389799674822, -0.00559691799261187, 0.00464478520766674, 0.000415442218609132, -0.00215870109072046, -0.00323138210966673, -0.00290809553026393, 0.0, 0.0, 0.0128177429097644, 0.0, 0.0113041867027994, 0.0183832217438608, 0.0, 0.0095537814881164, 0.0155366580724417, 0.0181644397838736, 0.0, 0.00740032731935588, 0.0120346435940572, 0.014070114555226, 0.0136156449512337, 0.0, 0.00427258096992147, 0.00694820471863012, 0.00812338442598859, 0.00786099627778515, 0.00607967912880135};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 17:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.0133943880175426, -0.00631417506469884, -0.00773325352755861, 0.0153110608279199, 0.00298267418725715, 0.0025208197798293, 0.0276525235907967, 0.00195261860521382, 0.0214195526695527, 0.0243925408037797, 0.00112734487734485, 0.0123665844996876, 0.0140830399992811, 0.0108976671476671, 0.0, 0.0, -0.0197848223020205, 0.0, -0.0167212267476003, -0.00321800158796923, 0.0, -0.0129522065443228, -0.00249265331165476, 0.0106850252586226, 0.0, -0.00747795993496438, -0.0014391340604803, 0.00616900220936375, 0.00872428659079182, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, -0.0135389740433143, -0.0281402291104519, -0.0357410679969884, 0.0, 0.0, -0.00781673030845887, -0.0162467688519772, -0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 18:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734313, -0.0354703486038674, -0.0204788153146928, -0.0449826285156206, 0.0141189246665007, 0.0109364720199429, 0.00446479600584753, 0.00631417506469885, 0.00257775117585285, -0.0211438459052229, 0.0, 0.0, 0.00339592394974695, 0.0, 0.00263047138047134, -0.0308358534689291, 0.0, 0.00151870335961077, -0.0178030883009781, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, 0.0174485731815835, -0.00356167508620756, 0.0, 0.0, 0.0100739384233621, -0.00205633406978792, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, 0.0218868448636848, 0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 19:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0.0, 0.0, -0.0178591840233903, 0.0, -0.0103110047034115, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00920473237878482, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489395, 0.0145404776513197, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0297842233308237, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 20:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734305, -0.0324515955311978, -0.0335503995581136, -0.00801060507812423, 0.0211783869997512, -0.00109364720199431, 0.0169662248222208, -0.00568275755822893, 0.00567105258687627, 0.00838462854862291, -0.0208787193108001, -0.0109235523792603, -0.0162519217595033, -0.00499002532443534, -0.00958243145743147, -0.0108922666482287, -0.00187890812890815, -0.00466458889023305, -0.00558007245254536, -0.00469727032227032, 0.0139343556230002, 0.00245778931055952, 0.0120907247401236, -0.00415442218609134, 0.00464478520766673, 0.00965400476390777, -0.00643600317593851, 0.000719567030240148, 0.00498530662330958, 0.00661453944581401, -0.00464478520766677, -0.000415442218609124, 0.00215870109072045, 0.00323138210966672, 0.00290809553026392, 0.0, 0.0127646671417816, -0.00116524935543312, 0.0202632808954031, 0.00719357335632687, -0.00183832217438607, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, 0.020635115229186, 0.0114368694935501, 0.00481385743762287, 0.000521115353897252, -0.00170195561890421, 0.0127646671417816, 0.00737991258440981, 0.00347410235931507, 0.000902598269554277, -0.000491312267361576, -0.000868525589828769};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 21:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734306, 0.0249047128495239, 0.0553363732971483, -0.00431340273437459, -0.0141189246665007, -0.0153110608279201, -0.00803663281052554, 0.00883984509057833, -0.0159820572902877, 0.000729098134662816, 0.00298267418725714, 0.0277290175781224, -0.00145539597846298, 0.0301571095694136, 0.0116492303992305, -0.00352848074520083, 0.0191648629148629, 0.00976309302606916, 0.00132858867917748, -0.00263047138047135, 0.0, -0.0196623144844763, 0.00219831358911337, -0.00727023882565974, -0.0148633126645335, 0.00402250198496158, 0.00724050357293086, -0.0129522065443227, -0.00685479660705068, 0.0045792965394097, 0.00928957041533352, -0.0066470754977461, -0.00575653624192119, -0.000293762009969713, 0.00290809553026392, 0.0, 0.0, 0.0, -0.0202632808954031, 0.00411061334647248, -0.00045958054359652, -0.0399597288148542, -0.00303983956440067, 0.00427258096992148, -0.00100913554354853, -0.0464290092656685, -0.00908221989193682, 0.00270779480866286, 0.00234501909253766, -0.00127646671417816, -0.031911667854454, -0.00776832903622085, 0.000868525589828734, 0.00180519653910859, 0.000245656133680784, -0.000868525589828764};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 22:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734315, -0.0354703486038674, -0.0492363006502185, -0.00431340273437457, 0.0141189246665008, 0.00656188321196575, 0.00982255121286466, -0.016416855168217, 0.0149509568199466, 0.000729098134662867, 0.0, 0.00336109303977241, -0.000485131992820998, -0.0282044909641998, 0.0124007936507937, -0.00291483191994851, -0.0353234728234728, 0.008895262534863, 0.00239145962251945, -0.0026304713804714, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920008, 0.000804500396992312, 0.00402250198496156, 0.0143913406048031, -0.00934744991870546, 0.00254405363300538, -0.00928957041533351, 0.0141250354327105, -0.00287826812096063, -0.00381890612960613, 0.00290809553026392, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320202, 0.00233049871086625, -0.000336378514516176, 0.0515877880729651, -0.00840946286290444, -0.00300866089851428, 0.00286613444643492, -0.000850977809452105, 0.0425488904726053, -0.00388416451811045, -0.00347410235931505, 0.00120346435940571, 0.000982624534723157, -0.000868525589828761};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 23:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812429, 0.0, -0.0174983552319087, 0.00357183680467806, 0.0132597676358676, -0.0175287079957995, 0.00838462854862292, 0.0, 0.0, 0.0, -0.00867830491206146, 0.00300625300625302, -0.000613648825252329, 0.0225468975468975, -0.019200749617936, 0.0122230158484327, -0.00469727032227032, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0160900079398463, 0.00719567030240154, -0.00249265331165478, 0.000508810726601091, 0.00464478520766676, -0.0108014976838374, 0.0107935054536023, -0.00734405024924251, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0257938940364825, 0.0134551405806471, -0.00601732179702859, 0.00208446141558904, -0.000425488904726056, -0.031911667854454, 0.0135945758133865, -0.00347410235931504, -0.000902598269554288, 0.00171959293576549, -0.000868525589828771};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 24:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665005, 0.0164047080299144, 0.0133943880175427, 0.0145226026488074, 0.0118576554089232, 0.00838462854862293, -0.00894802256177147, -0.0142846454190327, -0.0123708658169354, -0.014536160727703, -0.0125886844636845, -0.0102786178229764, -0.00939454064454065, -0.00813591085505764, -0.00664294339588734, -0.00469727032227033, -0.00836061337380014, 0.00245778931055955, 0.00219831358911339, 0.00830884437218263, 0.00743165633226679, 0.0064360031759385, 0.00965400476390778, 0.00863480436288184, 0.00747795993496438, 0.00610572871921294, 0.00650269929073342, 0.00581619106052785, 0.00503696921168106, 0.00411266813957582, 0.00290809553026391, 0.0352796762358899, 0.0191470007126724, 0.0174787403314969, 0.00788016479265675, 0.00719357335632689, 0.00643412761035129, 0.000951422114639382, 0.000868525589828755, 0.000776832903622076, 0.000672757029032345, -0.00221090520312707, -0.00201827108709705, -0.00180519653910857, -0.00156334606169177, -0.00127646671417816, -0.00212744452363026, -0.00194208225905521, -0.00173705117965752, -0.00150433044925715, -0.00122828066840393, -0.000868525589828761};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 25:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, 0.0603750614533913, -0.00610007264692978, -0.00431340273437464, -0.00564756986660042, -0.0218729440398858, -0.0178591840233902, 0.00126283501293976, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.016805465198862, 0.0145539597846298, -0.000867830491206139, -0.000751563251563246, -0.00061364882525229, -0.00526094276094275, -0.00455611007883229, -0.00372004830169691, -0.00263047138047138, 0.00557374224920013, -0.0196623144844763, -0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992314, 0.00321800158796925, 0.00287826812096061, 0.00249265331165479, 0.00203524290640428, 0.00650269929073342, 0.00581619106052785, 0.00503696921168108, 0.00411266813957583, 0.00290809553026394, -0.0705593524717798, -0.0127646671417816, -0.0116524935543313, 0.00450295131008955, 0.00411061334647247, 0.00367664434877213, 0.00380568845855755, 0.00347410235931508, 0.00310733161448836, 0.00269102811612943, -0.000736968401042361, -0.000672757029032367, -0.000601732179702866, -0.000521115353897261, -0.000425488904726046, -0.00212744452363026, -0.00194208225905521, -0.00173705117965753, -0.00150433044925714, -0.00122828066840394, -0.000868525589828775};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 26:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, -0.0603750614533913, -0.00610007264692978, -0.00431340273437461, -0.00564756986660028, 0.0218729440398858, 0.0178591840233902, 0.00126283501293978, 0.00103110047034114, 0.00072909813466285, -0.0417574386216002, 0.016805465198862, 0.0145539597846299, 0.00086783049120613, 0.000751563251563235, 0.000613648825252307, -0.00526094276094275, -0.00455611007883227, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, 0.0196623144844763, 0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992316, -0.00321800158796927, -0.00287826812096063, -0.0024926533116548, -0.00203524290640429, 0.00650269929073343, 0.00581619106052785, 0.00503696921168108, 0.00411266813957581, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.0116524935543313, -0.00450295131008957, -0.00411061334647248, -0.00367664434877215, 0.00380568845855754, 0.00347410235931504, 0.00310733161448834, 0.00269102811612942, 0.000736968401042368, 0.000672757029032363, 0.000601732179702872, 0.000521115353897267, 0.000425488904726048, -0.00212744452363027, -0.00194208225905522, -0.00173705117965754, -0.00150433044925714, -0.00122828066840393, -0.000868525589828761};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 27:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0452812960900436, -0.0113287063442981, -0.00801060507812426, -0.00141189246665009, -0.0164047080299144, -0.0133943880175427, 0.0145226026488074, 0.0118576554089232, 0.00838462854862292, 0.00894802256177149, -0.0142846454190328, -0.0123708658169354, 0.014536160727703, 0.0125886844636845, 0.0102786178229763, -0.00939454064454066, -0.00813591085505763, -0.00664294339588734, -0.00469727032227034, -0.0083606133738001, -0.00245778931055954, -0.0021983135891134, 0.00830884437218264, 0.00743165633226679, 0.00643600317593851, -0.00965400476390776, -0.00863480436288183, -0.00747795993496437, -0.00610572871921293, 0.00650269929073342, 0.00581619106052784, 0.00503696921168106, 0.00411266813957582, 0.00290809553026394, -0.0352796762358899, 0.0191470007126724, 0.017478740331497, -0.00788016479265674, -0.00719357335632688, -0.00643412761035128, 0.000951422114639399, 0.000868525589828759, 0.000776832903622089, 0.000672757029032358, 0.00221090520312706, 0.00201827108709707, 0.00180519653910857, 0.00156334606169177, 0.00127646671417816, -0.00212744452363025, -0.00194208225905521, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828767};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 28:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, 0.0679219441350651, -0.00435719474780696, -0.00308100195312468, 0.0705946233325039, -0.0328094160598287, -0.0267887760350854, 0.0126283501293977, 0.0360885164619401, -0.0072909813466286, 0.0298267418725716, 0.0151249186789758, 0.0130985638061668, -0.0325436434202305, -0.0229226791726792, -0.024852777422719, 0.0248015873015873, 0.0169226945785198, 0.0164744996218006, 0.0142797017797018, 0.0, 0.0245778931055954, 0.0219831358911338, -0.0363511941282991, -0.0074316563322668, -0.0345935170706696, 0.0362025178646541, 0.0, 0.0130864298861876, 0.0335815079556711, -0.0232239260383337, 0.00166176887443654, -0.00359783515120076, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0226083734055987, -0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.00908221989193682, 0.0, 0.0333014729371016, 0.0270779480866286, 0.00703505727761299, -0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598858, 0.0019652490694463, 0.00607967912880137};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 29:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, 0.0037734413408369, 0.00217859737390355, -0.0215670136718729, 0.0, 0.0984282481794862, -0.0312535720409329, -0.0852413633734344, -0.0180442582309701, 0.0309866707231717, 0.0, 0.0453747560369276, -0.0116431678277039, 0.0, 0.0236742424242424, -0.00138070985681772, -0.0202922077922078, -0.0335199527228374, -0.000797153207506555, 0.00394570707070709, 0.0, 0.0, 0.0, 0.036351194128299, 0.024152883079867, -0.00724050357293083, -0.0603375297744236, 0.0, -0.039259289658563, 0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064559, 0.0, -0.0555024548951693, 0.0, 0.0164151336477636, -0.00680782247561685, 0.0, 0.0427258096992147, 0.00694820471863017, -0.0108311792346515, -0.00393049813889261, 0.00607967912880133};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 30:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, 0.0301875307266957, 0.0610007264692972, -0.00308100195312469, 0.0, 0.0218729440398859, 0.0178591840233903, 0.0757701007763862, -0.0412440188136457, -0.00729098134662862, 0.0, 0.0, 0.0, 0.0520698294723689, -0.0127765752765753, 0.00184094647575698, -0.00901875901875902, 0.0286384062098028, -0.0297603864135752, 0.0142797017797017, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0402250198496157, 0.014391340604803, -0.0137095932141014, 0.00407048581280861, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0370016365967795, -0.030086608985143, 0.014070114555226, -0.00340391123780845, 0.0, -0.0427258096992148, 0.0243187165152054, 0.0, -0.00982624534723142, 0.00607967912880138};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 31:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.021478804657352, 0.00377344134083693, -0.0196073763651314, 0.00924300585937423, 0.0, -0.0109364720199429, 0.102690308134494, 0.00947126259704826, -0.0695992817480272, -0.060150596109686, 0.0, -0.0050416395596586, 0.0465726713108155, 0.00650872868404612, 0.0214195526695527, -0.00782402252196702, -0.00450937950937954, -0.0289638426440052, -0.0263060558477138, 0.000563672438672401, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, -0.0431740218144091, -0.00249265331165473, -0.0605484764655282, 0.0, 0.0299118397398575, -0.00431740218144093, 0.0185070066280912, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, -0.0451299134777144, -0.0281402291104519, 0.023827378664659, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 32:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, -0.0150937653633478, 0.0130715842434209, 0.00924300585937413, 0.0, -0.0109364720199429, -0.0089295920116951, 0.00947126259704832, 0.123732056440937, -0.0601505961096861, 0.0, 0.0, 0.0, -0.00650872868404607, 0.0574945887445887, -0.0266937238984759, 0.00676406926406925, 0.0126920209338899, 0.00637722566005186, 0.000563672438672445, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0431740218144092, 0.0199412264932383, -0.0142467003448302, 0.0, -0.0448677596097863, 0.0215870109072046, -0.0616900220936373, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0451299134777144, -0.0375203054806026, 0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 33:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0301875307266956, 0.0174287789912279, 0.0585390371093694, 0.0, 0.0218729440398859, 0.0178591840233903, -0.0189425251940966, 0.0103110047034114, 0.083846285486229, 0.0, 0.0, 0.0, 0.0, 0.00526094276094279, 0.0533874477969518, 0.0, -0.0045561100788323, 0.0308232573569172, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.034897146363167, 0.0284934006896603, 0.0, 0.0, -0.0302218152700864, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0437736897273697, -0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 34:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0679219441350652, -0.00435719474780696, -0.00308100195312475, 0.0705946233325039, 0.0328094160598288, 0.0267887760350854, 0.0126283501293977, 0.0360885164619401, -0.00729098134662861, -0.0298267418725716, 0.0151249186789759, 0.0130985638061669, 0.0325436434202305, 0.0229226791726792, 0.0248527774227189, 0.0248015873015873, 0.0169226945785198, 0.0164744996218006, 0.0142797017797017, 0.0, -0.0245778931055953, -0.0219831358911338, -0.036351194128299, -0.00743165633226676, -0.0345935170706695, -0.0362025178646541, 0.0, -0.0130864298861876, -0.0335815079556711, -0.0232239260383336, 0.00166176887443654, -0.00359783515120077, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, -0.0226083734055987, 0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.00908221989193681, 0.0, -0.0333014729371016, -0.0270779480866286, -0.00703505727761297, 0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598856, 0.0019652490694463, 0.00607967912880137};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 35:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573517, -0.0037734413408369, 0.00217859737390357, -0.0215670136718729, 0.0, -0.0984282481794861, 0.0312535720409329, -0.0852413633734345, -0.0180442582309701, 0.0309866707231716, 0.0, 0.0453747560369275, -0.0116431678277039, 0.0, -0.0236742424242424, 0.00138070985681772, -0.0202922077922078, -0.0335199527228374, -0.000797153207506539, 0.0039457070707071, 0.0, 0.0, 0.0, 0.036351194128299, 0.024152883079867, -0.00724050357293081, 0.0603375297744236, 0.0, 0.0392592896585629, -0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064559, 0.0, 0.0555024548951693, 0.0, -0.0164151336477637, 0.00680782247561685, 0.0, 0.0427258096992147, 0.0069482047186302, -0.0108311792346515, -0.0039304981388926, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 36:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, -0.0301875307266958, 0.0610007264692973, -0.00308100195312478, 0.0, -0.0218729440398858, -0.0178591840233903, 0.0757701007763862, -0.0412440188136456, -0.0072909813466286, 0.0, 0.0, 0.0, -0.0520698294723688, 0.0127765752765753, -0.00184094647575691, -0.00901875901875897, 0.0286384062098028, -0.0297603864135751, 0.0142797017797018, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0402250198496157, -0.0143913406048031, 0.0137095932141013, -0.00407048581280864, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0370016365967796, 0.0300866089851429, -0.014070114555226, 0.00340391123780843, 0.0, -0.0427258096992147, 0.0243187165152053, 0.0, -0.00982624534723143, 0.00607967912880138};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 37:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573526, -0.00377344134083707, -0.0196073763651313, 0.00924300585937411, 0.0, 0.0109364720199428, -0.102690308134494, 0.00947126259704835, -0.0695992817480271, -0.0601505961096859, 0.0, -0.00504163955965866, 0.0465726713108155, -0.00650872868404609, -0.0214195526695527, 0.00782402252196703, -0.00450937950937956, -0.0289638426440051, -0.0263060558477138, 0.000563672438672401, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, 0.0431740218144092, 0.00249265331165475, 0.0605484764655282, 0.0, 0.0299118397398574, -0.00431740218144093, 0.0185070066280912, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, 0.0451299134777144, 0.0281402291104519, -0.0238273786646589, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 38:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573518, 0.0150937653633479, 0.013071584243421, 0.00924300585937412, 0.0, 0.0109364720199429, 0.00892959201169515, 0.00947126259704832, 0.123732056440937, -0.0601505961096861, 0.0, 0.0, 0.0, 0.0065087286840461, -0.0574945887445887, 0.0266937238984759, 0.00676406926406924, 0.0126920209338899, 0.0063772256600518, 0.000563672438672482, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0431740218144092, -0.0199412264932383, 0.0142467003448302, 0.0, -0.0448677596097863, 0.0215870109072046, -0.0616900220936373, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0451299134777144, 0.0375203054806026, -0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 39:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0301875307266957, 0.0174287789912278, 0.0585390371093694, 0.0, -0.0218729440398858, -0.0178591840233902, -0.0189425251940966, 0.0103110047034114, 0.083846285486229, 0.0, 0.0, 0.0, 0.0, -0.00526094276094276, -0.0533874477969517, 0.0, -0.00455611007883227, 0.0308232573569172, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0348971463631671, -0.0284934006896603, 0.0, 0.0, -0.0302218152700864, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0437736897273697, 0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 40:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0377344134083696, -0.0566435317214905, -0.00308100195312481, 0.0423567739995023, 0.0546823600997146, 0.0446479600584756, 0.0378850503881931, 0.0051555023517058, -0.00729098134662859, 0.0417574386216002, -0.00504163955965863, 0.0295930515620807, -0.0247331689993752, 0.0101461038961039, 0.0266937238984759, -0.0202922077922078, 0.00216957622801539, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00219831358911336, -0.00519302773261414, -0.0130053985814669, -0.0168945083368385, 0.00402250198496158, -0.0158304746652834, -0.026796023100289, -0.0295110221428625, 0.00464478520766674, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, -0.0226083734055987, -0.0367664434877216, 0.0, -0.0047768907440582, -0.00776832903622084, -0.00908221989193678, 0.0, 0.00370016365967797, 0.00601732179702859, 0.00703505727761299, 0.00680782247561684, 0.0, 0.00427258096992148, 0.0069482047186301, 0.00812338442598856, 0.00786099627778516, 0.00607967912880135};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 41:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, 0.0, -0.00435719474780698, -0.0215670136718729, -0.127070321998507, 0.0, 0.0, 0.0284137877911449, 0.03608851646194, 0.0309866707231716, 0.0, 0.0453747560369275, -0.0422064833754265, 0.0, 0.0, 0.0, -0.0202922077922078, -0.00618329224984381, 0.00159430641501296, 0.00394570707070708, 0.0836061337380012, 0.0, 0.0, -0.0103860554652283, 0.024152883079867, 0.0434430214375849, 0.0, 0.0, 0.0, 0.0, 0.00464478520766673, -0.0108014976838374, -0.0194283098164842, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, 0.0769064574585865, 0.0, 0.0, 0.0, 0.0, -0.00955378148811642, -0.0155366580724417, -0.0181644397838736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00427258096992146, 0.00694820471863015, 0.00812338442598858, 0.00786099627778516, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 42:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0377344134083696, -0.0566435317214904, -0.00308100195312474, 0.0423567739995022, -0.0546823600997145, -0.0446479600584756, 0.037885050388193, 0.00515550235170577, -0.00729098134662853, -0.0417574386216002, -0.00504163955965862, 0.0295930515620806, 0.0247331689993752, -0.0101461038961039, -0.0266937238984759, -0.0202922077922078, 0.00216957622801538, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055953, 0.00219831358911338, -0.00519302773261413, -0.0130053985814669, -0.0168945083368386, -0.00402250198496156, 0.0158304746652834, 0.026796023100289, 0.0295110221428625, 0.00464478520766678, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, 0.0226083734055987, 0.0367664434877216, 0.0, -0.00477689074405818, -0.00776832903622087, -0.0090822198919368, 0.0, -0.00370016365967795, -0.00601732179702859, -0.00703505727761298, -0.00680782247561684, 0.0, 0.00427258096992151, 0.00694820471863008, 0.00812338442598856, 0.00786099627778518, 0.00607967912880136};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 43:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, -0.0188672067041848, 0.00653579212171038, 0.0092430058593742, 0.0141189246665007, 0.0, -0.111619900146189, -0.0031570875323494, -0.0541327746929101, -0.060150596109686, -0.00894802256177143, -0.0025208197798293, -0.0276525235907967, 0.000650872868404634, 0.0360750360750361, -0.0188697013765088, 0.00112734487734487, 0.0374251899332651, 0.0199288301876619, 0.000563672438672437, 0.0, 0.0, 0.0593544669060615, 0.0, 0.0167212267476002, 0.0032180015879692, 0.0, -0.0043174021814409, 0.0224338798048931, 0.0463017761206981, 0.0, -0.00747795993496438, 0.018708742786244, 0.043183015465546, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0620433733855302, 0.0, 0.0, 0.0174787403314969, 0.0363288795677471, 0.0, 0.0, -0.00451299134777145, -0.00938007637015062, -0.0119136893323295, 0.0, 0.0, -0.00781673030845887, -0.0162467688519772, -0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 44:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573519, 0.0188672067041847, 0.00653579212171046, 0.00924300585937428, 0.0141189246665008, 0.0, 0.111619900146189, -0.00315708753234942, -0.0541327746929101, -0.0601505961096861, 0.00894802256177154, -0.00252081977982933, -0.0276525235907967, -0.000650872868404596, -0.0360750360750361, 0.0188697013765088, 0.00112734487734489, 0.0374251899332651, 0.019928830187662, 0.000563672438672452, 0.0, 0.0, -0.0593544669060614, 0.0, 0.0167212267476002, 0.00321800158796926, 0.0, 0.00431740218144093, -0.0224338798048931, -0.0463017761206981, 0.0, -0.00747795993496437, 0.018708742786244, 0.0431830154655461, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0620433733855302, 0.0, 0.0, 0.017478740331497, 0.0363288795677472, 0.0, 0.0, 0.00451299134777144, 0.00938007637015066, 0.0119136893323295, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.0206351152291861, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 45:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, 0.0, -0.0348575579824556, 0.0585390371093695, -0.0282378493330015, 0.0, 0.0, 0.00631417506469884, -0.0206220094068228, 0.083846285486229, 0.0, 0.0, -0.00679184789949389, 0.0, 0.0, 0.0, 0.0, 0.00151870335961078, -0.0616465147138344, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0450520222315696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0100739384233621, -0.0329013451166065, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0565115904387178, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 46:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995024, 0.0546823600997146, 0.0446479600584757, 0.0, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, -0.00921750786359888, 0.0273366604729936, -0.00789141414141414, -0.0248527774227189, 0.0187890812890813, -0.00347132196482459, -0.0143487577351167, -0.015031265031265, -0.0557374224920008, -0.00491557862111906, -0.0241814494802473, -0.0197335053839337, -0.00928957041533348, -0.00241350119097693, -0.0394205194526234, -0.0158304746652834, -0.000623163327913708, 0.00712335017241507, -0.0325134964536671, -0.0141250354327105, -0.00215870109072047, 0.00411266813957583, 0.00540074884191869, 0.0, -0.0510586685671264, 0.00466099742173249, -0.0405265617908061, -0.0143871467126537, 0.00367664434877214, -0.0114170653756726, -0.00564541633388697, -0.00155366580724415, 0.00100913554354854, 0.010317557614593, 0.00571843474677502, 0.00240692871881142, 0.000260557676948618, -0.000850977809452114, 0.0127646671417816, 0.00737991258440983, 0.00347410235931509, 0.000902598269554292, -0.00049131226736157, -0.000868525589828748};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 47:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573517, 0.0, -0.0217859737390347, 0.00308100195312484, -0.127070321998507, 0.0, 0.0, 0.0473563129852413, 0.0257775117585285, 0.0127592173566001, 0.0, -0.0352914769176103, 0.0509388592462045, 0.0, 0.0, 0.0, 0.00225468975468975, -0.00943765659186684, -0.0143487577351166, -0.0129644660894661, 0.0836061337380012, 0.0, 0.0, 0.0477758551400501, 0.00928957041533348, -0.0144810071458617, 0.0, 0.0, 0.0, 0.0, -0.0325134964536671, -0.0141250354327105, -0.00215870109072047, 0.00411266813957584, 0.00540074884191876, 0.0, 0.0765880028506896, -0.00699149613259875, 0.0, 0.0, 0.0, -0.0228341307513453, -0.011290832667774, -0.00310733161448838, 0.00201827108709704, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, 0.00737991258440981, 0.00347410235931506, 0.000902598269554269, -0.000491312267361589, -0.000868525589828803};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 48:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, -0.0446479600584757, 0.0, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.00921750786359893, -0.0273366604729936, 0.00789141414141414, 0.0248527774227189, 0.0187890812890813, -0.00347132196482458, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111907, 0.0241814494802472, -0.0197335053839337, -0.00928957041533349, -0.00241350119097694, 0.0394205194526234, 0.0158304746652834, 0.000623163327913706, -0.00712335017241508, -0.0325134964536671, -0.0141250354327105, -0.00215870109072045, 0.00411266813957581, 0.00540074884191868, 0.0, -0.0510586685671264, 0.0046609974217325, 0.0405265617908061, 0.0143871467126537, -0.00367664434877216, -0.0114170653756727, -0.00564541633388695, -0.00155366580724416, 0.00100913554354855, -0.010317557614593, -0.00571843474677502, -0.00240692871881143, -0.000260557676948629, 0.000850977809452107, 0.0127646671417816, 0.00737991258440984, 0.00347410235931506, 0.000902598269554289, -0.000491312267361559, -0.000868525589828739};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 49:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, -0.0188672067041847, 0.0108929868695175, 0.00308100195312472, 0.0141189246665008, -0.109364720199429, 0.0223239800292378, -0.0789271883087356, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, 0.00145539597846298, -0.00585785581564149, 0.0383297258297258, -0.0124263887113595, 0.0191648629148628, 0.0348216984596467, 0.00717437886755836, -0.0129644660894661, 0.0, 0.0589869434534289, -0.00659494076734012, 0.00727023882565974, 0.0148633126645335, -0.0040225019849616, 0.0506835250105158, -0.00431740218144091, -0.00810112326287806, 0.00356167508620759, 0.0650269929073342, -0.00166176887443651, -0.0115130724838425, -0.00205633406978793, 0.00540074884191875, 0.0, 0.0, 0.0, 0.0607898426862092, -0.0123318400394175, 0.0013787416307896, 0.0399597288148543, 0.00303983956440066, -0.00427258096992151, 0.00100913554354852, -0.0154763364218895, -0.00302740663064561, 0.000902598269554317, 0.000781673030845898, -0.000425488904726052, -0.031911667854454, -0.00776832903622089, 0.000868525589828735, 0.00180519653910857, 0.000245656133680779, -0.000868525589828785};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 50:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.0188672067041847, 0.0108929868695173, 0.00308100195312461, 0.0141189246665007, 0.109364720199429, -0.0223239800292378, -0.0789271883087356, -0.0128887558792643, 0.0127592173566001, 0.00894802256177146, -0.0277290175781224, 0.00145539597846298, 0.00585785581564147, -0.0383297258297259, 0.0124263887113595, 0.0191648629148629, 0.0348216984596467, 0.00717437886755832, -0.0129644660894661, 0.0, -0.0589869434534289, 0.00659494076734013, 0.00727023882565983, 0.0148633126645336, -0.00402250198496153, -0.0506835250105158, 0.00431740218144093, 0.00810112326287806, -0.00356167508620756, 0.0650269929073343, -0.00166176887443652, -0.0115130724838424, -0.00205633406978791, 0.00540074884191871, 0.0, 0.0, 0.0, -0.0607898426862092, 0.0123318400394175, -0.00137874163078959, 0.0399597288148543, 0.00303983956440068, -0.00427258096992146, 0.00100913554354852, 0.0154763364218896, 0.00302740663064561, -0.000902598269554309, -0.000781673030845882, 0.000425488904726061, -0.031911667854454, -0.00776832903622087, 0.000868525589828761, 0.00180519653910858, 0.000245656133680792, -0.000868525589828766};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 51:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.078755617076958, 0.0, 0.0435719474780694, -0.0523770332031201, -0.0282378493330015, 0.0, 0.0, 0.0631417506469885, -0.0515550235170571, 0.0291639253865144, 0.0, -0.00672218607954477, 0.000970263985642002, 0.0, 0.0, 0.0, -0.0601250601250601, -0.0125835421224891, 0.0286975154702333, -0.015031265031265, 0.0, 0.0, 0.0, -0.0436214329539588, 0.0111474844984002, -0.00160900079398465, 0.0, 0.0, 0.0, 0.0, -0.0650269929073343, 0.031573608614294, -0.00287826812096059, -0.00822533627915167, 0.00540074884191872, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0532796384198057, 0.018239037386404, -0.00466099742173253, 0.000672757029032364, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0425488904726053, -0.00388416451811041, -0.00347410235931508, 0.00120346435940574, 0.000982624534723167, -0.000868525589828753};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 52:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997147, -0.0446479600584758, -0.0315708753234942, -0.0257775117585285, -0.0182274533665715, 0.0894802256177148, -0.0151249186789758, -0.0130985638061669, 0.0585785581564149, -0.028183621933622, 0.0690354928408859, 0.0744047619047619, 0.00976309302606916, 0.039857660375324, 0.0597492784992785, 0.0, 0.0737336793167861, 0.0659494076734016, 0.036351194128299, 0.00743165633226677, 0.0345935170706695, -0.0120675059548847, 0.0431740218144092, 0.0155790831978424, -0.0254405363300539, -0.0232239260383337, 0.0465295284842228, 0.0251848460584054, -0.0146881004984851, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, 0.0678251202167962, -0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.00908221989193679, 0.0, -0.0111004909790339, -0.00902598269554285, -0.00234501909253765, 0.0034039112378084, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598858, 0.0019652490694463, 0.00607967912880138};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 53:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281179, -0.0705946233325039, 0.0546823600997147, 0.0446479600584757, -0.0315708753234942, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789759, -0.0130985638061668, -0.0585785581564149, 0.028183621933622, -0.0690354928408859, 0.0744047619047619, 0.00976309302606915, 0.039857660375324, 0.0597492784992785, 0.0, -0.0737336793167861, -0.0659494076734016, 0.036351194128299, 0.00743165633226679, 0.0345935170706696, 0.0120675059548846, -0.0431740218144092, -0.0155790831978425, 0.0254405363300538, -0.0232239260383337, 0.0465295284842228, 0.0251848460584053, -0.014688100498485, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, -0.0678251202167962, 0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.0090822198919368, 0.0, 0.0111004909790338, 0.00902598269554286, 0.00234501909253764, -0.00340391123780842, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598857, 0.00196524906944628, 0.00607967912880138};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 54:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.217859737390348, -0.0770250488281177, 0.0, 0.0, 0.0, -0.0947126259704828, 0.0515550235170571, -0.0182274533665715, 0.0, -0.0907495120738552, 0.0232863356554078, 0.0, 0.0, 0.0, -0.0946969696969697, -0.0227805503941613, -0.079715320750648, 0.0597492784992784, 0.0, 0.0, 0.0, -0.0727023882565981, -0.0483057661597341, 0.0144810071458617, 0.0, 0.0, 0.0, 0.0, 0.0464478520766673, -0.0781031370985168, 0.0431740218144092, 0.0293762009969702, -0.0328199352701214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0668764704168148, 0.0310733161448835, -0.00605481326129118, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0427258096992147, 0.00694820471863014, -0.0108311792346515, -0.0039304981388926, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 55:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.0, 0.231075146484353, 0.0, 0.0, 0.0, 0.0, 0.0, -0.109364720199429, 0.0, 0.0100832791193172, -0.093145342621631, 0.0, 0.0, 0.0, -0.00450937950937958, -0.104139658944738, 0.0, -0.0924422799422799, 0.0, 0.0, 0.0, 0.0, -0.066884906990401, -0.0579240285834466, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.0647610327216138, 0.0, 0.0835038859404355, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0699149613259878, 0.0242192530451648, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338355, 0.0270779480866287, 0.0, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.0;
    
    // Loop dofs and call evaluate_basis.
    for (unsigned int r = 0; r < 56; r++)
    {
      evaluate_basis(r, &dof_values, coordinates, c);
      values[r] = dof_values;
    }// end loop over 'r'
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    const double K_00 = d_00 / detJ;
    const double K_01 = d_10 / detJ;
    const double K_02 = d_20 / detJ;
    const double K_10 = d_01 / detJ;
    const double K_11 = d_11 / detJ;
    const double K_12 = d_21 / detJ;
    const double K_20 = d_02 / detJ;
    const double K_21 = d_12 / detJ;
    const double K_22 = d_22 / detJ;
    
    // Compute constants
    const double C0 = x[3][0] + x[2][0] + x[1][0] - x[0][0];
    const double C1 = x[3][1] + x[2][1] + x[1][1] - x[0][1];
    const double C2 = x[3][2] + x[2][2] + x[1][2] - x[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (d_00*(2.0*coordinates[0] - C0) + d_10*(2.0*coordinates[1] - C1) + d_20*(2.0*coordinates[2] - C2)) / detJ;
    double Y = (d_01*(2.0*coordinates[0] - C0) + d_11*(2.0*coordinates[1] - C1) + d_21*(2.0*coordinates[2] - C2)) / detJ;
    double Z = (d_02*(2.0*coordinates[0] - C0) + d_12*(2.0*coordinates[1] - C1) + d_22*(2.0*coordinates[2] - C2)) / detJ;
    
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 3;
    }// end loop over 'r'
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      combinations[row] = new unsigned int [n];
      for (unsigned int col = 0; col < n; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] = {{K_00, K_01, K_02}, {K_10, K_11, K_12}, {K_20, K_21, K_22}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    }// end loop over 'r'
    
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923505, 0.00609788120679253, 0.00352061335622801, 0.00248944957812478, 0.00705946233325035, 0.00546823600997146, 0.0044647960058476, 0.0031570875323494, 0.00257775117585286, 0.00182274533665715, -0.00298267418725715, -0.00252081977982932, -0.00218309396769448, -0.00195261860521384, -0.00169101731601733, -0.00138070985681772, -0.0011273448773449, -0.00097630930260694, -0.000797153207506489, -0.000563672438672444, 0.00278687112460006, 0.00245778931055954, 0.00219831358911339, 0.00207721109304566, 0.00185791408306669, 0.00160900079398463, 0.00160900079398464, 0.00143913406048031, 0.00124632665582739, 0.00101762145320215, 0.000928957041533368, 0.000830884437218251, 0.000719567030240159, 0.000587524019939407, 0.000415442218609133, -0.00705593524717798, -0.00638233357089079, -0.00582624677716565, -0.00562868913761196, -0.00513826668309062, -0.00459580543596521, -0.00475711057319694, -0.00434262794914382, -0.00388416451811044, -0.00336378514516178, -0.00368484200521178, -0.00336378514516176, -0.00300866089851429, -0.00260557676948629, -0.00212744452363027, -0.00212744452363026, -0.00194208225905519, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828762};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923493, -0.00609788120679249, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, -0.00546823600997144, -0.00446479600584757, 0.00315708753234942, 0.00257775117585287, 0.00182274533665715, 0.00298267418725716, -0.00252081977982929, -0.00218309396769448, 0.00195261860521383, 0.00169101731601732, 0.00138070985681771, -0.00112734487734489, -0.000976309302606925, -0.000797153207506475, -0.000563672438672438, 0.00278687112460003, -0.00245778931055953, -0.0021983135891134, 0.00207721109304565, 0.00185791408306669, 0.00160900079398462, -0.00160900079398462, -0.0014391340604803, -0.0012463266558274, -0.00101762145320215, 0.000928957041533351, 0.000830884437218258, 0.000719567030240152, 0.000587524019939402, 0.000415442218609129, 0.00705593524717798, -0.00638233357089081, -0.00582624677716566, 0.00562868913761197, 0.00513826668309062, 0.0045958054359652, -0.00475711057319694, -0.00434262794914383, -0.00388416451811044, -0.00336378514516178, 0.00368484200521179, 0.00336378514516178, 0.0030086608985143, 0.0026055767694863, 0.00212744452363027, -0.00212744452363027, -0.00194208225905521, -0.00173705117965753, -0.00150433044925715, -0.00122828066840393, -0.000868525589828762};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923494, 0.0, -0.00704122671245613, 0.00248944957812474, 0.0, 0.0, 0.0, 0.0094712625970483, -0.00515550235170573, 0.00182274533665714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0045093795093795, -0.00292892790782074, 0.00159430641501297, -0.000563672438672427, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00464478520766671, -0.00332353774887303, 0.00215870109072045, -0.0011750480398788, 0.000415442218609134, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, -0.0097104112952761, 0.00694820471863011, -0.00451299134777143, 0.00245656133680785, -0.000868525589828767};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 3:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923491, 0.0, 0.0, -0.00746834873437434, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0109364720199429, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00563672438672436, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00623163327913695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 4:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0, 0.03747187483114, -0.0289614183593723, 0.0, 0.0, 0.0, 0.0284137877911448, 0.00515550235170568, -0.00619733414463431, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0315656565656565, -0.0113902751970807, 0.0026571773583549, -0.000187890812890804, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0232239260383337, -0.00166176887443655, -0.00647610327216137, 0.00646276421933346, -0.00290809553026392, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0213629048496074, -0.0277928188745204, 0.0243701532779658, -0.0157219925555703, 0.00607967912880135};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 5:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, 0.0, -0.0287574853355261, 0.05360943398437, 0.0, 0.0, 0.0, -0.0189425251940965, 0.0154665070551172, 0.0153110608279201, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00450937950937951, 0.0370997534990628, -0.0281660799985623, 0.0108976671476671, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.00431740218144093, -0.0123380044187275, 0.00872428659079177, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338356, -0.0487403065559316, 0.041270230458372, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 6:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, 0.0, 0.0409576306293853, -0.0449826285156207, 0.0, 0.0, 0.0, 0.0189425251940965, -0.0051555023517058, -0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00455611007883225, 0.0356061766019561, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0302218152700864, 0.00411266813957586, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0379091273212801, -0.0550269739444961, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 7:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, 0.0, -0.0148144621425436, 0.0449826285156208, 0.0, 0.0, 0.0, 0.0, 0.0206220094068229, 0.021143845905223, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0106287094334197, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.020563340697879, 0.0145404776513197, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.03439185871531, -0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 8:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00446479600584752, 0.00947126259704829, -0.00257775117585286, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, -0.00848980987436738, 0.0136683302364968, 0.00657617845117846, 0.00230118309469618, -0.00789141414141413, -0.0037967583990269, -0.00132858867917748, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, -0.00109915679455667, 0.0103860554652283, 0.000928957041533345, -0.00482700238195389, -0.00804500396992315, -0.000719567030240148, 0.0037389799674822, 0.00559691799261188, 0.00464478520766673, 0.000415442218609139, -0.00215870109072047, -0.0032313821096667, -0.00290809553026391, 0.0, 0.0, 0.0128177429097644, 0.0, -0.0113041867027994, -0.0183832217438608, 0.0, 0.0095537814881164, 0.0155366580724417, 0.0181644397838736, 0.0, -0.00740032731935591, -0.0120346435940572, -0.014070114555226, -0.0136156449512337, 0.0, 0.00427258096992147, 0.00694820471863013, 0.00812338442598859, 0.00786099627778514, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 9:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734316, -0.0249047128495238, 0.014378742667763, 0.0536094339843698, -0.0141189246665007, 0.0109364720199429, 0.0133943880175427, -0.00631417506469885, -0.00773325352755854, 0.0153110608279201, -0.00298267418725716, 0.00252081977982933, 0.0276525235907967, -0.00195261860521385, -0.0214195526695527, -0.0243925408037797, 0.00112734487734488, 0.0123665844996876, 0.0140830399992812, 0.0108976671476671, 0.0, 0.0, 0.0197848223020205, 0.0, -0.0167212267476003, -0.00321800158796927, 0.0, 0.0129522065443227, 0.00249265331165477, -0.0106850252586226, 0.0, -0.00747795993496439, -0.00143913406048031, 0.00616900220936372, 0.00872428659079175, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, 0.0135389740433143, 0.0281402291104519, 0.0357410679969885, 0.0, 0.0, -0.00781673030845889, -0.0162467688519772, -0.020635115229186, -0.018239037386404};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 10:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0354703486038674, -0.0204788153146928, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, -0.00446479600584748, 0.00631417506469886, 0.00257775117585284, -0.021143845905223, 0.0, 0.0, 0.00339592394974695, 0.0, -0.00263047138047139, 0.0308358534689291, 0.0, 0.00151870335961076, -0.017803088300978, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, -0.0174485731815836, 0.00356167508620753, 0.0, 0.0, 0.0100739384233621, -0.00205633406978791, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, -0.0218868448636848, -0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 11:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0128297005588457, 0.00740723107127185, 0.0449826285156207, 0.0, 0.0, 0.0178591840233903, 0.0, -0.0103110047034114, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00920473237878477, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489396, 0.0145404776513197, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0297842233308238, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 12:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0324515955311978, -0.0335503995581136, -0.00801060507812418, 0.0211783869997512, 0.00109364720199429, -0.0169662248222207, -0.00568275755822894, 0.00567105258687627, 0.00838462854862288, 0.0208787193108001, -0.0109235523792603, -0.0162519217595033, 0.00499002532443536, 0.00958243145743147, 0.0108922666482287, -0.00187890812890812, -0.00466458889023306, -0.00558007245254536, -0.00469727032227032, 0.0139343556230002, -0.00245778931055953, -0.0120907247401236, -0.00415442218609133, 0.00464478520766674, 0.00965400476390776, 0.00643600317593851, -0.000719567030240141, -0.00498530662330958, -0.006614539445814, -0.00464478520766674, -0.000415442218609131, 0.00215870109072044, 0.00323138210966672, 0.00290809553026393, 0.0, 0.0127646671417816, -0.00116524935543313, -0.0202632808954031, -0.00719357335632686, 0.00183832217438609, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, -0.020635115229186, -0.01143686949355, -0.00481385743762287, -0.000521115353897263, 0.00170195561890421, 0.0127646671417816, 0.00737991258440982, 0.00347410235931507, 0.00090259826955429, -0.000491312267361578, -0.000868525589828769};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 13:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, -0.0249047128495238, 0.0553363732971482, -0.00431340273437461, -0.0141189246665008, 0.0153110608279201, 0.00803663281052558, 0.00883984509057838, -0.0159820572902877, 0.000729098134662845, -0.00298267418725716, 0.0277290175781224, -0.00145539597846296, -0.0301571095694136, -0.0116492303992304, 0.00352848074520083, 0.0191648629148629, 0.00976309302606916, 0.00132858867917747, -0.00263047138047138, 0.0, 0.0196623144844763, -0.00219831358911338, -0.00727023882565981, -0.0148633126645336, 0.00402250198496157, -0.00724050357293084, 0.0129522065443227, 0.00685479660705067, -0.00457929653940971, 0.00928957041533346, -0.00664707549774609, -0.0057565362419212, -0.00029376200996971, 0.00290809553026395, 0.0, 0.0, 0.0, 0.0202632808954031, -0.00411061334647249, 0.000459580543596519, -0.0399597288148543, -0.00303983956440069, 0.00427258096992149, -0.00100913554354853, 0.0464290092656685, 0.00908221989193678, -0.00270779480866285, -0.00234501909253766, 0.00127646671417817, -0.031911667854454, -0.00776832903622087, 0.000868525589828756, 0.00180519653910858, 0.000245656133680792, -0.000868525589828773};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 14:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0354703486038674, -0.0492363006502187, -0.00431340273437462, 0.0141189246665007, -0.00656188321196576, -0.00982255121286465, -0.016416855168217, 0.0149509568199466, 0.00072909813466287, 0.0, 0.0033610930397724, -0.00048513199282101, 0.0282044909641998, -0.0124007936507936, 0.00291483191994851, -0.0353234728234728, 0.00889526253486301, 0.00239145962251945, -0.00263047138047138, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920009, 0.00080450039699231, -0.00402250198496154, -0.014391340604803, 0.00934744991870548, -0.00254405363300537, -0.0092895704153335, 0.0141250354327105, -0.00287826812096064, -0.00381890612960613, 0.00290809553026391, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320202, 0.00233049871086625, -0.000336378514516179, -0.0515877880729651, 0.00840946286290446, 0.00300866089851429, -0.00286613444643492, 0.000850977809452107, 0.0425488904726053, -0.00388416451811045, -0.00347410235931507, 0.00120346435940572, 0.000982624534723146, -0.000868525589828758};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 15:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0128297005588457, 0.0448791059024117, -0.00801060507812427, 0.0, 0.0174983552319086, -0.00357183680467801, 0.0132597676358676, -0.0175287079957995, 0.00838462854862288, 0.0, 0.0, 0.0, 0.00867830491206147, -0.00300625300625301, 0.000613648825252317, 0.0225468975468975, -0.019200749617936, 0.0122230158484327, -0.00469727032227032, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0160900079398463, -0.00719567030240153, 0.0024926533116548, -0.000508810726601086, 0.00464478520766675, -0.0108014976838374, 0.0107935054536023, -0.00734405024924252, 0.00290809553026394, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0257938940364826, -0.0134551405806471, 0.00601732179702859, -0.00208446141558904, 0.000425488904726059, -0.031911667854454, 0.0135945758133865, -0.00347410235931504, -0.000902598269554292, 0.0017195929357655, -0.000868525589828772};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 16:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734313, -0.0324515955311979, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, -0.00446479600584756, 0.00947126259704828, -0.00257775117585285, -0.00619733414463429, -0.0208787193108001, -0.0176457384588052, -0.00848980987436741, -0.0136683302364968, -0.00657617845117842, -0.00230118309469618, -0.00789141414141414, -0.00379675839902689, -0.00132858867917748, -0.000187890812890818, 0.0139343556230002, 0.0122889465527977, 0.00109915679455668, 0.0103860554652283, 0.000928957041533349, -0.0048270023819539, 0.00804500396992312, 0.00071956703024016, -0.0037389799674822, -0.00559691799261187, 0.00464478520766674, 0.000415442218609132, -0.00215870109072046, -0.00323138210966673, -0.00290809553026393, 0.0, 0.0, 0.0128177429097644, 0.0, 0.0113041867027994, 0.0183832217438608, 0.0, 0.0095537814881164, 0.0155366580724417, 0.0181644397838736, 0.0, 0.00740032731935588, 0.0120346435940572, 0.014070114555226, 0.0136156449512337, 0.0, 0.00427258096992147, 0.00694820471863012, 0.00812338442598859, 0.00786099627778515, 0.00607967912880135};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 17:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.0133943880175426, -0.00631417506469884, -0.00773325352755861, 0.0153110608279199, 0.00298267418725715, 0.0025208197798293, 0.0276525235907967, 0.00195261860521382, 0.0214195526695527, 0.0243925408037797, 0.00112734487734485, 0.0123665844996876, 0.0140830399992811, 0.0108976671476671, 0.0, 0.0, -0.0197848223020205, 0.0, -0.0167212267476003, -0.00321800158796923, 0.0, -0.0129522065443228, -0.00249265331165476, 0.0106850252586226, 0.0, -0.00747795993496438, -0.0014391340604803, 0.00616900220936375, 0.00872428659079182, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, -0.0135389740433143, -0.0281402291104519, -0.0357410679969884, 0.0, 0.0, -0.00781673030845887, -0.0162467688519772, -0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 18:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734313, -0.0354703486038674, -0.0204788153146928, -0.0449826285156206, 0.0141189246665007, 0.0109364720199429, 0.00446479600584753, 0.00631417506469885, 0.00257775117585285, -0.0211438459052229, 0.0, 0.0, 0.00339592394974695, 0.0, 0.00263047138047134, -0.0308358534689291, 0.0, 0.00151870335961077, -0.0178030883009781, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, 0.0174485731815835, -0.00356167508620756, 0.0, 0.0, 0.0100739384233621, -0.00205633406978792, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, 0.0218868448636848, 0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 19:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0.0, 0.0, -0.0178591840233903, 0.0, -0.0103110047034115, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00920473237878482, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489395, 0.0145404776513197, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0297842233308237, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 20:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734305, -0.0324515955311978, -0.0335503995581136, -0.00801060507812423, 0.0211783869997512, -0.00109364720199431, 0.0169662248222208, -0.00568275755822893, 0.00567105258687627, 0.00838462854862291, -0.0208787193108001, -0.0109235523792603, -0.0162519217595033, -0.00499002532443534, -0.00958243145743147, -0.0108922666482287, -0.00187890812890815, -0.00466458889023305, -0.00558007245254536, -0.00469727032227032, 0.0139343556230002, 0.00245778931055952, 0.0120907247401236, -0.00415442218609134, 0.00464478520766673, 0.00965400476390777, -0.00643600317593851, 0.000719567030240148, 0.00498530662330958, 0.00661453944581401, -0.00464478520766677, -0.000415442218609124, 0.00215870109072045, 0.00323138210966672, 0.00290809553026392, 0.0, 0.0127646671417816, -0.00116524935543312, 0.0202632808954031, 0.00719357335632687, -0.00183832217438607, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, 0.020635115229186, 0.0114368694935501, 0.00481385743762287, 0.000521115353897252, -0.00170195561890421, 0.0127646671417816, 0.00737991258440981, 0.00347410235931507, 0.000902598269554277, -0.000491312267361576, -0.000868525589828769};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 21:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734306, 0.0249047128495239, 0.0553363732971483, -0.00431340273437459, -0.0141189246665007, -0.0153110608279201, -0.00803663281052554, 0.00883984509057833, -0.0159820572902877, 0.000729098134662816, 0.00298267418725714, 0.0277290175781224, -0.00145539597846298, 0.0301571095694136, 0.0116492303992305, -0.00352848074520083, 0.0191648629148629, 0.00976309302606916, 0.00132858867917748, -0.00263047138047135, 0.0, -0.0196623144844763, 0.00219831358911337, -0.00727023882565974, -0.0148633126645335, 0.00402250198496158, 0.00724050357293086, -0.0129522065443227, -0.00685479660705068, 0.0045792965394097, 0.00928957041533352, -0.0066470754977461, -0.00575653624192119, -0.000293762009969713, 0.00290809553026392, 0.0, 0.0, 0.0, -0.0202632808954031, 0.00411061334647248, -0.00045958054359652, -0.0399597288148542, -0.00303983956440067, 0.00427258096992148, -0.00100913554354853, -0.0464290092656685, -0.00908221989193682, 0.00270779480866286, 0.00234501909253766, -0.00127646671417816, -0.031911667854454, -0.00776832903622085, 0.000868525589828734, 0.00180519653910859, 0.000245656133680784, -0.000868525589828764};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 22:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734315, -0.0354703486038674, -0.0492363006502185, -0.00431340273437457, 0.0141189246665008, 0.00656188321196575, 0.00982255121286466, -0.016416855168217, 0.0149509568199466, 0.000729098134662867, 0.0, 0.00336109303977241, -0.000485131992820998, -0.0282044909641998, 0.0124007936507937, -0.00291483191994851, -0.0353234728234728, 0.008895262534863, 0.00239145962251945, -0.0026304713804714, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920008, 0.000804500396992312, 0.00402250198496156, 0.0143913406048031, -0.00934744991870546, 0.00254405363300538, -0.00928957041533351, 0.0141250354327105, -0.00287826812096063, -0.00381890612960613, 0.00290809553026392, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320202, 0.00233049871086625, -0.000336378514516176, 0.0515877880729651, -0.00840946286290444, -0.00300866089851428, 0.00286613444643492, -0.000850977809452105, 0.0425488904726053, -0.00388416451811045, -0.00347410235931505, 0.00120346435940571, 0.000982624534723157, -0.000868525589828761};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 23:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812429, 0.0, -0.0174983552319087, 0.00357183680467806, 0.0132597676358676, -0.0175287079957995, 0.00838462854862292, 0.0, 0.0, 0.0, -0.00867830491206146, 0.00300625300625302, -0.000613648825252329, 0.0225468975468975, -0.019200749617936, 0.0122230158484327, -0.00469727032227032, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0160900079398463, 0.00719567030240154, -0.00249265331165478, 0.000508810726601091, 0.00464478520766676, -0.0108014976838374, 0.0107935054536023, -0.00734405024924251, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0257938940364825, 0.0134551405806471, -0.00601732179702859, 0.00208446141558904, -0.000425488904726056, -0.031911667854454, 0.0135945758133865, -0.00347410235931504, -0.000902598269554288, 0.00171959293576549, -0.000868525589828771};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 24:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665005, 0.0164047080299144, 0.0133943880175427, 0.0145226026488074, 0.0118576554089232, 0.00838462854862293, -0.00894802256177147, -0.0142846454190327, -0.0123708658169354, -0.014536160727703, -0.0125886844636845, -0.0102786178229764, -0.00939454064454065, -0.00813591085505764, -0.00664294339588734, -0.00469727032227033, -0.00836061337380014, 0.00245778931055955, 0.00219831358911339, 0.00830884437218263, 0.00743165633226679, 0.0064360031759385, 0.00965400476390778, 0.00863480436288184, 0.00747795993496438, 0.00610572871921294, 0.00650269929073342, 0.00581619106052785, 0.00503696921168106, 0.00411266813957582, 0.00290809553026391, 0.0352796762358899, 0.0191470007126724, 0.0174787403314969, 0.00788016479265675, 0.00719357335632689, 0.00643412761035129, 0.000951422114639382, 0.000868525589828755, 0.000776832903622076, 0.000672757029032345, -0.00221090520312707, -0.00201827108709705, -0.00180519653910857, -0.00156334606169177, -0.00127646671417816, -0.00212744452363026, -0.00194208225905521, -0.00173705117965752, -0.00150433044925715, -0.00122828066840393, -0.000868525589828761};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 25:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, 0.0603750614533913, -0.00610007264692978, -0.00431340273437464, -0.00564756986660042, -0.0218729440398858, -0.0178591840233902, 0.00126283501293976, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.016805465198862, 0.0145539597846298, -0.000867830491206139, -0.000751563251563246, -0.00061364882525229, -0.00526094276094275, -0.00455611007883229, -0.00372004830169691, -0.00263047138047138, 0.00557374224920013, -0.0196623144844763, -0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992314, 0.00321800158796925, 0.00287826812096061, 0.00249265331165479, 0.00203524290640428, 0.00650269929073342, 0.00581619106052785, 0.00503696921168108, 0.00411266813957583, 0.00290809553026394, -0.0705593524717798, -0.0127646671417816, -0.0116524935543313, 0.00450295131008955, 0.00411061334647247, 0.00367664434877213, 0.00380568845855755, 0.00347410235931508, 0.00310733161448836, 0.00269102811612943, -0.000736968401042361, -0.000672757029032367, -0.000601732179702866, -0.000521115353897261, -0.000425488904726046, -0.00212744452363026, -0.00194208225905521, -0.00173705117965753, -0.00150433044925714, -0.00122828066840394, -0.000868525589828775};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 26:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, -0.0603750614533913, -0.00610007264692978, -0.00431340273437461, -0.00564756986660028, 0.0218729440398858, 0.0178591840233902, 0.00126283501293978, 0.00103110047034114, 0.00072909813466285, -0.0417574386216002, 0.016805465198862, 0.0145539597846299, 0.00086783049120613, 0.000751563251563235, 0.000613648825252307, -0.00526094276094275, -0.00455611007883227, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, 0.0196623144844763, 0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992316, -0.00321800158796927, -0.00287826812096063, -0.0024926533116548, -0.00203524290640429, 0.00650269929073343, 0.00581619106052785, 0.00503696921168108, 0.00411266813957581, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.0116524935543313, -0.00450295131008957, -0.00411061334647248, -0.00367664434877215, 0.00380568845855754, 0.00347410235931504, 0.00310733161448834, 0.00269102811612942, 0.000736968401042368, 0.000672757029032363, 0.000601732179702872, 0.000521115353897267, 0.000425488904726048, -0.00212744452363027, -0.00194208225905522, -0.00173705117965754, -0.00150433044925714, -0.00122828066840393, -0.000868525589828761};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 27:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0452812960900436, -0.0113287063442981, -0.00801060507812426, -0.00141189246665009, -0.0164047080299144, -0.0133943880175427, 0.0145226026488074, 0.0118576554089232, 0.00838462854862292, 0.00894802256177149, -0.0142846454190328, -0.0123708658169354, 0.014536160727703, 0.0125886844636845, 0.0102786178229763, -0.00939454064454066, -0.00813591085505763, -0.00664294339588734, -0.00469727032227034, -0.0083606133738001, -0.00245778931055954, -0.0021983135891134, 0.00830884437218264, 0.00743165633226679, 0.00643600317593851, -0.00965400476390776, -0.00863480436288183, -0.00747795993496437, -0.00610572871921293, 0.00650269929073342, 0.00581619106052784, 0.00503696921168106, 0.00411266813957582, 0.00290809553026394, -0.0352796762358899, 0.0191470007126724, 0.017478740331497, -0.00788016479265674, -0.00719357335632688, -0.00643412761035128, 0.000951422114639399, 0.000868525589828759, 0.000776832903622089, 0.000672757029032358, 0.00221090520312706, 0.00201827108709707, 0.00180519653910857, 0.00156334606169177, 0.00127646671417816, -0.00212744452363025, -0.00194208225905521, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828767};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 28:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, 0.0679219441350651, -0.00435719474780696, -0.00308100195312468, 0.0705946233325039, -0.0328094160598287, -0.0267887760350854, 0.0126283501293977, 0.0360885164619401, -0.0072909813466286, 0.0298267418725716, 0.0151249186789758, 0.0130985638061668, -0.0325436434202305, -0.0229226791726792, -0.024852777422719, 0.0248015873015873, 0.0169226945785198, 0.0164744996218006, 0.0142797017797018, 0.0, 0.0245778931055954, 0.0219831358911338, -0.0363511941282991, -0.0074316563322668, -0.0345935170706696, 0.0362025178646541, 0.0, 0.0130864298861876, 0.0335815079556711, -0.0232239260383337, 0.00166176887443654, -0.00359783515120076, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0226083734055987, -0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.00908221989193682, 0.0, 0.0333014729371016, 0.0270779480866286, 0.00703505727761299, -0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598858, 0.0019652490694463, 0.00607967912880137};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 29:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, 0.0037734413408369, 0.00217859737390355, -0.0215670136718729, 0.0, 0.0984282481794862, -0.0312535720409329, -0.0852413633734344, -0.0180442582309701, 0.0309866707231717, 0.0, 0.0453747560369276, -0.0116431678277039, 0.0, 0.0236742424242424, -0.00138070985681772, -0.0202922077922078, -0.0335199527228374, -0.000797153207506555, 0.00394570707070709, 0.0, 0.0, 0.0, 0.036351194128299, 0.024152883079867, -0.00724050357293083, -0.0603375297744236, 0.0, -0.039259289658563, 0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064559, 0.0, -0.0555024548951693, 0.0, 0.0164151336477636, -0.00680782247561685, 0.0, 0.0427258096992147, 0.00694820471863017, -0.0108311792346515, -0.00393049813889261, 0.00607967912880133};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 30:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, 0.0301875307266957, 0.0610007264692972, -0.00308100195312469, 0.0, 0.0218729440398859, 0.0178591840233903, 0.0757701007763862, -0.0412440188136457, -0.00729098134662862, 0.0, 0.0, 0.0, 0.0520698294723689, -0.0127765752765753, 0.00184094647575698, -0.00901875901875902, 0.0286384062098028, -0.0297603864135752, 0.0142797017797017, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0402250198496157, 0.014391340604803, -0.0137095932141014, 0.00407048581280861, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0370016365967795, -0.030086608985143, 0.014070114555226, -0.00340391123780845, 0.0, -0.0427258096992148, 0.0243187165152054, 0.0, -0.00982624534723142, 0.00607967912880138};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 31:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.021478804657352, 0.00377344134083693, -0.0196073763651314, 0.00924300585937423, 0.0, -0.0109364720199429, 0.102690308134494, 0.00947126259704826, -0.0695992817480272, -0.060150596109686, 0.0, -0.0050416395596586, 0.0465726713108155, 0.00650872868404612, 0.0214195526695527, -0.00782402252196702, -0.00450937950937954, -0.0289638426440052, -0.0263060558477138, 0.000563672438672401, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, -0.0431740218144091, -0.00249265331165473, -0.0605484764655282, 0.0, 0.0299118397398575, -0.00431740218144093, 0.0185070066280912, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, -0.0451299134777144, -0.0281402291104519, 0.023827378664659, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 32:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, -0.0150937653633478, 0.0130715842434209, 0.00924300585937413, 0.0, -0.0109364720199429, -0.0089295920116951, 0.00947126259704832, 0.123732056440937, -0.0601505961096861, 0.0, 0.0, 0.0, -0.00650872868404607, 0.0574945887445887, -0.0266937238984759, 0.00676406926406925, 0.0126920209338899, 0.00637722566005186, 0.000563672438672445, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0431740218144092, 0.0199412264932383, -0.0142467003448302, 0.0, -0.0448677596097863, 0.0215870109072046, -0.0616900220936373, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0451299134777144, -0.0375203054806026, 0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 33:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0301875307266956, 0.0174287789912279, 0.0585390371093694, 0.0, 0.0218729440398859, 0.0178591840233903, -0.0189425251940966, 0.0103110047034114, 0.083846285486229, 0.0, 0.0, 0.0, 0.0, 0.00526094276094279, 0.0533874477969518, 0.0, -0.0045561100788323, 0.0308232573569172, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.034897146363167, 0.0284934006896603, 0.0, 0.0, -0.0302218152700864, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0437736897273697, -0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 34:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0679219441350652, -0.00435719474780696, -0.00308100195312475, 0.0705946233325039, 0.0328094160598288, 0.0267887760350854, 0.0126283501293977, 0.0360885164619401, -0.00729098134662861, -0.0298267418725716, 0.0151249186789759, 0.0130985638061669, 0.0325436434202305, 0.0229226791726792, 0.0248527774227189, 0.0248015873015873, 0.0169226945785198, 0.0164744996218006, 0.0142797017797017, 0.0, -0.0245778931055953, -0.0219831358911338, -0.036351194128299, -0.00743165633226676, -0.0345935170706695, -0.0362025178646541, 0.0, -0.0130864298861876, -0.0335815079556711, -0.0232239260383336, 0.00166176887443654, -0.00359783515120077, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, -0.0226083734055987, 0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.00908221989193681, 0.0, -0.0333014729371016, -0.0270779480866286, -0.00703505727761297, 0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598856, 0.0019652490694463, 0.00607967912880137};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 35:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573517, -0.0037734413408369, 0.00217859737390357, -0.0215670136718729, 0.0, -0.0984282481794861, 0.0312535720409329, -0.0852413633734345, -0.0180442582309701, 0.0309866707231716, 0.0, 0.0453747560369275, -0.0116431678277039, 0.0, -0.0236742424242424, 0.00138070985681772, -0.0202922077922078, -0.0335199527228374, -0.000797153207506539, 0.0039457070707071, 0.0, 0.0, 0.0, 0.036351194128299, 0.024152883079867, -0.00724050357293081, 0.0603375297744236, 0.0, 0.0392592896585629, -0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064559, 0.0, 0.0555024548951693, 0.0, -0.0164151336477637, 0.00680782247561685, 0.0, 0.0427258096992147, 0.0069482047186302, -0.0108311792346515, -0.0039304981388926, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 36:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, -0.0301875307266958, 0.0610007264692973, -0.00308100195312478, 0.0, -0.0218729440398858, -0.0178591840233903, 0.0757701007763862, -0.0412440188136456, -0.0072909813466286, 0.0, 0.0, 0.0, -0.0520698294723688, 0.0127765752765753, -0.00184094647575691, -0.00901875901875897, 0.0286384062098028, -0.0297603864135751, 0.0142797017797018, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0402250198496157, -0.0143913406048031, 0.0137095932141013, -0.00407048581280864, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0370016365967796, 0.0300866089851429, -0.014070114555226, 0.00340391123780843, 0.0, -0.0427258096992147, 0.0243187165152053, 0.0, -0.00982624534723143, 0.00607967912880138};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 37:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573526, -0.00377344134083707, -0.0196073763651313, 0.00924300585937411, 0.0, 0.0109364720199428, -0.102690308134494, 0.00947126259704835, -0.0695992817480271, -0.0601505961096859, 0.0, -0.00504163955965866, 0.0465726713108155, -0.00650872868404609, -0.0214195526695527, 0.00782402252196703, -0.00450937950937956, -0.0289638426440051, -0.0263060558477138, 0.000563672438672401, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, 0.0431740218144092, 0.00249265331165475, 0.0605484764655282, 0.0, 0.0299118397398574, -0.00431740218144093, 0.0185070066280912, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, 0.0451299134777144, 0.0281402291104519, -0.0238273786646589, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 38:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573518, 0.0150937653633479, 0.013071584243421, 0.00924300585937412, 0.0, 0.0109364720199429, 0.00892959201169515, 0.00947126259704832, 0.123732056440937, -0.0601505961096861, 0.0, 0.0, 0.0, 0.0065087286840461, -0.0574945887445887, 0.0266937238984759, 0.00676406926406924, 0.0126920209338899, 0.0063772256600518, 0.000563672438672482, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0431740218144092, -0.0199412264932383, 0.0142467003448302, 0.0, -0.0448677596097863, 0.0215870109072046, -0.0616900220936373, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0451299134777144, 0.0375203054806026, -0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 39:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0301875307266957, 0.0174287789912278, 0.0585390371093694, 0.0, -0.0218729440398858, -0.0178591840233902, -0.0189425251940966, 0.0103110047034114, 0.083846285486229, 0.0, 0.0, 0.0, 0.0, -0.00526094276094276, -0.0533874477969517, 0.0, -0.00455611007883227, 0.0308232573569172, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0348971463631671, -0.0284934006896603, 0.0, 0.0, -0.0302218152700864, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0437736897273697, 0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 40:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0377344134083696, -0.0566435317214905, -0.00308100195312481, 0.0423567739995023, 0.0546823600997146, 0.0446479600584756, 0.0378850503881931, 0.0051555023517058, -0.00729098134662859, 0.0417574386216002, -0.00504163955965863, 0.0295930515620807, -0.0247331689993752, 0.0101461038961039, 0.0266937238984759, -0.0202922077922078, 0.00216957622801539, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00219831358911336, -0.00519302773261414, -0.0130053985814669, -0.0168945083368385, 0.00402250198496158, -0.0158304746652834, -0.026796023100289, -0.0295110221428625, 0.00464478520766674, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, -0.0226083734055987, -0.0367664434877216, 0.0, -0.0047768907440582, -0.00776832903622084, -0.00908221989193678, 0.0, 0.00370016365967797, 0.00601732179702859, 0.00703505727761299, 0.00680782247561684, 0.0, 0.00427258096992148, 0.0069482047186301, 0.00812338442598856, 0.00786099627778516, 0.00607967912880135};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 41:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, 0.0, -0.00435719474780698, -0.0215670136718729, -0.127070321998507, 0.0, 0.0, 0.0284137877911449, 0.03608851646194, 0.0309866707231716, 0.0, 0.0453747560369275, -0.0422064833754265, 0.0, 0.0, 0.0, -0.0202922077922078, -0.00618329224984381, 0.00159430641501296, 0.00394570707070708, 0.0836061337380012, 0.0, 0.0, -0.0103860554652283, 0.024152883079867, 0.0434430214375849, 0.0, 0.0, 0.0, 0.0, 0.00464478520766673, -0.0108014976838374, -0.0194283098164842, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, 0.0769064574585865, 0.0, 0.0, 0.0, 0.0, -0.00955378148811642, -0.0155366580724417, -0.0181644397838736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00427258096992146, 0.00694820471863015, 0.00812338442598858, 0.00786099627778516, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 42:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0377344134083696, -0.0566435317214904, -0.00308100195312474, 0.0423567739995022, -0.0546823600997145, -0.0446479600584756, 0.037885050388193, 0.00515550235170577, -0.00729098134662853, -0.0417574386216002, -0.00504163955965862, 0.0295930515620806, 0.0247331689993752, -0.0101461038961039, -0.0266937238984759, -0.0202922077922078, 0.00216957622801538, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055953, 0.00219831358911338, -0.00519302773261413, -0.0130053985814669, -0.0168945083368386, -0.00402250198496156, 0.0158304746652834, 0.026796023100289, 0.0295110221428625, 0.00464478520766678, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, 0.0226083734055987, 0.0367664434877216, 0.0, -0.00477689074405818, -0.00776832903622087, -0.0090822198919368, 0.0, -0.00370016365967795, -0.00601732179702859, -0.00703505727761298, -0.00680782247561684, 0.0, 0.00427258096992151, 0.00694820471863008, 0.00812338442598856, 0.00786099627778518, 0.00607967912880136};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 43:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, -0.0188672067041848, 0.00653579212171038, 0.0092430058593742, 0.0141189246665007, 0.0, -0.111619900146189, -0.0031570875323494, -0.0541327746929101, -0.060150596109686, -0.00894802256177143, -0.0025208197798293, -0.0276525235907967, 0.000650872868404634, 0.0360750360750361, -0.0188697013765088, 0.00112734487734487, 0.0374251899332651, 0.0199288301876619, 0.000563672438672437, 0.0, 0.0, 0.0593544669060615, 0.0, 0.0167212267476002, 0.0032180015879692, 0.0, -0.0043174021814409, 0.0224338798048931, 0.0463017761206981, 0.0, -0.00747795993496438, 0.018708742786244, 0.043183015465546, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0620433733855302, 0.0, 0.0, 0.0174787403314969, 0.0363288795677471, 0.0, 0.0, -0.00451299134777145, -0.00938007637015062, -0.0119136893323295, 0.0, 0.0, -0.00781673030845887, -0.0162467688519772, -0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 44:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573519, 0.0188672067041847, 0.00653579212171046, 0.00924300585937428, 0.0141189246665008, 0.0, 0.111619900146189, -0.00315708753234942, -0.0541327746929101, -0.0601505961096861, 0.00894802256177154, -0.00252081977982933, -0.0276525235907967, -0.000650872868404596, -0.0360750360750361, 0.0188697013765088, 0.00112734487734489, 0.0374251899332651, 0.019928830187662, 0.000563672438672452, 0.0, 0.0, -0.0593544669060614, 0.0, 0.0167212267476002, 0.00321800158796926, 0.0, 0.00431740218144093, -0.0224338798048931, -0.0463017761206981, 0.0, -0.00747795993496437, 0.018708742786244, 0.0431830154655461, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0620433733855302, 0.0, 0.0, 0.017478740331497, 0.0363288795677472, 0.0, 0.0, 0.00451299134777144, 0.00938007637015066, 0.0119136893323295, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.0206351152291861, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 45:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, 0.0, -0.0348575579824556, 0.0585390371093695, -0.0282378493330015, 0.0, 0.0, 0.00631417506469884, -0.0206220094068228, 0.083846285486229, 0.0, 0.0, -0.00679184789949389, 0.0, 0.0, 0.0, 0.0, 0.00151870335961078, -0.0616465147138344, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0450520222315696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0100739384233621, -0.0329013451166065, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0565115904387178, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 46:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995024, 0.0546823600997146, 0.0446479600584757, 0.0, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, -0.00921750786359888, 0.0273366604729936, -0.00789141414141414, -0.0248527774227189, 0.0187890812890813, -0.00347132196482459, -0.0143487577351167, -0.015031265031265, -0.0557374224920008, -0.00491557862111906, -0.0241814494802473, -0.0197335053839337, -0.00928957041533348, -0.00241350119097693, -0.0394205194526234, -0.0158304746652834, -0.000623163327913708, 0.00712335017241507, -0.0325134964536671, -0.0141250354327105, -0.00215870109072047, 0.00411266813957583, 0.00540074884191869, 0.0, -0.0510586685671264, 0.00466099742173249, -0.0405265617908061, -0.0143871467126537, 0.00367664434877214, -0.0114170653756726, -0.00564541633388697, -0.00155366580724415, 0.00100913554354854, 0.010317557614593, 0.00571843474677502, 0.00240692871881142, 0.000260557676948618, -0.000850977809452114, 0.0127646671417816, 0.00737991258440983, 0.00347410235931509, 0.000902598269554292, -0.00049131226736157, -0.000868525589828748};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 47:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573517, 0.0, -0.0217859737390347, 0.00308100195312484, -0.127070321998507, 0.0, 0.0, 0.0473563129852413, 0.0257775117585285, 0.0127592173566001, 0.0, -0.0352914769176103, 0.0509388592462045, 0.0, 0.0, 0.0, 0.00225468975468975, -0.00943765659186684, -0.0143487577351166, -0.0129644660894661, 0.0836061337380012, 0.0, 0.0, 0.0477758551400501, 0.00928957041533348, -0.0144810071458617, 0.0, 0.0, 0.0, 0.0, -0.0325134964536671, -0.0141250354327105, -0.00215870109072047, 0.00411266813957584, 0.00540074884191876, 0.0, 0.0765880028506896, -0.00699149613259875, 0.0, 0.0, 0.0, -0.0228341307513453, -0.011290832667774, -0.00310733161448838, 0.00201827108709704, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, 0.00737991258440981, 0.00347410235931506, 0.000902598269554269, -0.000491312267361589, -0.000868525589828803};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 48:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, -0.0446479600584757, 0.0, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.00921750786359893, -0.0273366604729936, 0.00789141414141414, 0.0248527774227189, 0.0187890812890813, -0.00347132196482458, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111907, 0.0241814494802472, -0.0197335053839337, -0.00928957041533349, -0.00241350119097694, 0.0394205194526234, 0.0158304746652834, 0.000623163327913706, -0.00712335017241508, -0.0325134964536671, -0.0141250354327105, -0.00215870109072045, 0.00411266813957581, 0.00540074884191868, 0.0, -0.0510586685671264, 0.0046609974217325, 0.0405265617908061, 0.0143871467126537, -0.00367664434877216, -0.0114170653756727, -0.00564541633388695, -0.00155366580724416, 0.00100913554354855, -0.010317557614593, -0.00571843474677502, -0.00240692871881143, -0.000260557676948629, 0.000850977809452107, 0.0127646671417816, 0.00737991258440984, 0.00347410235931506, 0.000902598269554289, -0.000491312267361559, -0.000868525589828739};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 49:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, -0.0188672067041847, 0.0108929868695175, 0.00308100195312472, 0.0141189246665008, -0.109364720199429, 0.0223239800292378, -0.0789271883087356, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, 0.00145539597846298, -0.00585785581564149, 0.0383297258297258, -0.0124263887113595, 0.0191648629148628, 0.0348216984596467, 0.00717437886755836, -0.0129644660894661, 0.0, 0.0589869434534289, -0.00659494076734012, 0.00727023882565974, 0.0148633126645335, -0.0040225019849616, 0.0506835250105158, -0.00431740218144091, -0.00810112326287806, 0.00356167508620759, 0.0650269929073342, -0.00166176887443651, -0.0115130724838425, -0.00205633406978793, 0.00540074884191875, 0.0, 0.0, 0.0, 0.0607898426862092, -0.0123318400394175, 0.0013787416307896, 0.0399597288148543, 0.00303983956440066, -0.00427258096992151, 0.00100913554354852, -0.0154763364218895, -0.00302740663064561, 0.000902598269554317, 0.000781673030845898, -0.000425488904726052, -0.031911667854454, -0.00776832903622089, 0.000868525589828735, 0.00180519653910857, 0.000245656133680779, -0.000868525589828785};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 50:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.0188672067041847, 0.0108929868695173, 0.00308100195312461, 0.0141189246665007, 0.109364720199429, -0.0223239800292378, -0.0789271883087356, -0.0128887558792643, 0.0127592173566001, 0.00894802256177146, -0.0277290175781224, 0.00145539597846298, 0.00585785581564147, -0.0383297258297259, 0.0124263887113595, 0.0191648629148629, 0.0348216984596467, 0.00717437886755832, -0.0129644660894661, 0.0, -0.0589869434534289, 0.00659494076734013, 0.00727023882565983, 0.0148633126645336, -0.00402250198496153, -0.0506835250105158, 0.00431740218144093, 0.00810112326287806, -0.00356167508620756, 0.0650269929073343, -0.00166176887443652, -0.0115130724838424, -0.00205633406978791, 0.00540074884191871, 0.0, 0.0, 0.0, -0.0607898426862092, 0.0123318400394175, -0.00137874163078959, 0.0399597288148543, 0.00303983956440068, -0.00427258096992146, 0.00100913554354852, 0.0154763364218896, 0.00302740663064561, -0.000902598269554309, -0.000781673030845882, 0.000425488904726061, -0.031911667854454, -0.00776832903622087, 0.000868525589828761, 0.00180519653910858, 0.000245656133680792, -0.000868525589828766};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 51:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.078755617076958, 0.0, 0.0435719474780694, -0.0523770332031201, -0.0282378493330015, 0.0, 0.0, 0.0631417506469885, -0.0515550235170571, 0.0291639253865144, 0.0, -0.00672218607954477, 0.000970263985642002, 0.0, 0.0, 0.0, -0.0601250601250601, -0.0125835421224891, 0.0286975154702333, -0.015031265031265, 0.0, 0.0, 0.0, -0.0436214329539588, 0.0111474844984002, -0.00160900079398465, 0.0, 0.0, 0.0, 0.0, -0.0650269929073343, 0.031573608614294, -0.00287826812096059, -0.00822533627915167, 0.00540074884191872, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0532796384198057, 0.018239037386404, -0.00466099742173253, 0.000672757029032364, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0425488904726053, -0.00388416451811041, -0.00347410235931508, 0.00120346435940574, 0.000982624534723167, -0.000868525589828753};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 52:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997147, -0.0446479600584758, -0.0315708753234942, -0.0257775117585285, -0.0182274533665715, 0.0894802256177148, -0.0151249186789758, -0.0130985638061669, 0.0585785581564149, -0.028183621933622, 0.0690354928408859, 0.0744047619047619, 0.00976309302606916, 0.039857660375324, 0.0597492784992785, 0.0, 0.0737336793167861, 0.0659494076734016, 0.036351194128299, 0.00743165633226677, 0.0345935170706695, -0.0120675059548847, 0.0431740218144092, 0.0155790831978424, -0.0254405363300539, -0.0232239260383337, 0.0465295284842228, 0.0251848460584054, -0.0146881004984851, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, 0.0678251202167962, -0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.00908221989193679, 0.0, -0.0111004909790339, -0.00902598269554285, -0.00234501909253765, 0.0034039112378084, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598858, 0.0019652490694463, 0.00607967912880138};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 53:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281179, -0.0705946233325039, 0.0546823600997147, 0.0446479600584757, -0.0315708753234942, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789759, -0.0130985638061668, -0.0585785581564149, 0.028183621933622, -0.0690354928408859, 0.0744047619047619, 0.00976309302606915, 0.039857660375324, 0.0597492784992785, 0.0, -0.0737336793167861, -0.0659494076734016, 0.036351194128299, 0.00743165633226679, 0.0345935170706696, 0.0120675059548846, -0.0431740218144092, -0.0155790831978425, 0.0254405363300538, -0.0232239260383337, 0.0465295284842228, 0.0251848460584053, -0.014688100498485, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, -0.0678251202167962, 0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.0090822198919368, 0.0, 0.0111004909790338, 0.00902598269554286, 0.00234501909253764, -0.00340391123780842, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598857, 0.00196524906944628, 0.00607967912880138};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 54:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.217859737390348, -0.0770250488281177, 0.0, 0.0, 0.0, -0.0947126259704828, 0.0515550235170571, -0.0182274533665715, 0.0, -0.0907495120738552, 0.0232863356554078, 0.0, 0.0, 0.0, -0.0946969696969697, -0.0227805503941613, -0.079715320750648, 0.0597492784992784, 0.0, 0.0, 0.0, -0.0727023882565981, -0.0483057661597341, 0.0144810071458617, 0.0, 0.0, 0.0, 0.0, 0.0464478520766673, -0.0781031370985168, 0.0431740218144092, 0.0293762009969702, -0.0328199352701214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0668764704168148, 0.0310733161448835, -0.00605481326129118, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0427258096992147, 0.00694820471863014, -0.0108311792346515, -0.0039304981388926, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 55:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[4] = 1.5*tmp0*basisvalues[1] - 0.5*tmp1*basisvalues[0];
      basisvalues[10] = 1.66666666666667*tmp0*basisvalues[4] - 0.666666666666667*tmp1*basisvalues[1];
      basisvalues[20] = 1.75*tmp0*basisvalues[10] - 0.75*tmp1*basisvalues[4];
      basisvalues[35] = 1.8*tmp0*basisvalues[20] - 0.8*tmp1*basisvalues[10];
      basisvalues[2] = 0.5*(2.0 + 3.0*Y + Z)*basisvalues[0];
      basisvalues[5] = (0.5*(2.0 + 3.0*Y + Z) + 1.0*(1.0 + Y))*basisvalues[1];
      basisvalues[11] = (0.5*(2.0 + 3.0*Y + Z) + 2.0*(1.0 + Y))*basisvalues[4];
      basisvalues[21] = (0.5*(2.0 + 3.0*Y + Z) + 3.0*(1.0 + Y))*basisvalues[10];
      basisvalues[36] = (0.5*(2.0 + 3.0*Y + Z) + 4.0*(1.0 + Y))*basisvalues[20];
      basisvalues[7] = (1.66666666666667*tmp2 + 0.111111111111111*tmp3)*basisvalues[2] - 0.555555555555556*tmp4*basisvalues[0];
      basisvalues[16] = (1.75*tmp2 + 0.05*tmp3)*basisvalues[7] - 0.7*tmp4*basisvalues[2];
      basisvalues[30] = (1.8*tmp2 + 0.0285714285714286*tmp3)*basisvalues[16] - 0.771428571428571*tmp4*basisvalues[7];
      basisvalues[50] = (1.83333333333333*tmp2 + 0.0185185185185185*tmp3)*basisvalues[30] - 0.814814814814815*tmp4*basisvalues[16];
      basisvalues[13] = (2.1*tmp2 + 0.54*tmp3)*basisvalues[5] - 0.56*tmp4*basisvalues[1];
      basisvalues[26] = (2.0*tmp2 + 0.285714285714286*tmp3)*basisvalues[13] - 0.714285714285714*tmp4*basisvalues[5];
      basisvalues[45] = (1.96428571428571*tmp2 + 0.178571428571429*tmp3)*basisvalues[26] - 0.785714285714286*tmp4*basisvalues[13];
      basisvalues[23] = (2.57142857142857*tmp2 + 1.02040816326531*tmp3)*basisvalues[11] - 0.551020408163265*tmp4*basisvalues[4];
      basisvalues[41] = (2.29166666666667*tmp2 + 0.578703703703704*tmp3)*basisvalues[23] - 0.712962962962963*tmp4*basisvalues[11];
      basisvalues[38] = (3.05555555555556*tmp2 + 1.51234567901235*tmp3)*basisvalues[21] - 0.54320987654321*tmp4*basisvalues[10];
      basisvalues[3] = (2.0*Z + 1.0)*basisvalues[0];
      basisvalues[8] = (3.0*Z + 2.0)*basisvalues[2];
      basisvalues[17] = (4.0*Z + 3.0)*basisvalues[7];
      basisvalues[31] = (5.0*Z + 4.0)*basisvalues[16];
      basisvalues[51] = (6.0*Z + 5.0)*basisvalues[30];
      basisvalues[6] = (3.0*Z + 2.0)*basisvalues[1];
      basisvalues[14] = (4.0*Z + 3.0)*basisvalues[5];
      basisvalues[27] = (5.0*Z + 4.0)*basisvalues[13];
      basisvalues[46] = (6.0*Z + 5.0)*basisvalues[26];
      basisvalues[12] = (4.0*Z + 3.0)*basisvalues[4];
      basisvalues[24] = (5.0*Z + 4.0)*basisvalues[11];
      basisvalues[42] = (6.0*Z + 5.0)*basisvalues[23];
      basisvalues[22] = (5.0*Z + 4.0)*basisvalues[10];
      basisvalues[39] = (6.0*Z + 5.0)*basisvalues[21];
      basisvalues[37] = (6.0*Z + 5.0)*basisvalues[20];
      basisvalues[9] = basisvalues[3]*(0.3125 + 1.875*Z) - 0.5625*basisvalues[0];
      basisvalues[19] = basisvalues[9]*(0.155555555555556 + 1.86666666666667*Z) - 0.711111111111111*basisvalues[3];
      basisvalues[34] = basisvalues[19]*(0.09375 + 1.875*Z) - 0.78125*basisvalues[9];
      basisvalues[55] = basisvalues[34]*(0.0628571428571429 + 1.88571428571429*Z) - 0.822857142857143*basisvalues[19];
      basisvalues[18] = basisvalues[8]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[2];
      basisvalues[33] = basisvalues[18]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[8];
      basisvalues[54] = basisvalues[33]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[18];
      basisvalues[32] = basisvalues[17]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[7];
      basisvalues[53] = basisvalues[32]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[17];
      basisvalues[52] = basisvalues[31]*(1.76 + 3.3*Z) - 0.54*basisvalues[16];
      basisvalues[15] = basisvalues[6]*(0.777777777777778 + 2.33333333333333*Z) - 0.555555555555556*basisvalues[1];
      basisvalues[29] = basisvalues[15]*(0.428571428571429 + 2.14285714285714*Z) - 0.714285714285714*basisvalues[6];
      basisvalues[49] = basisvalues[29]*(0.275 + 2.0625*Z) - 0.7875*basisvalues[15];
      basisvalues[28] = basisvalues[14]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[5];
      basisvalues[48] = basisvalues[28]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[14];
      basisvalues[47] = basisvalues[27]*(1.76 + 3.3*Z) - 0.54*basisvalues[13];
      basisvalues[25] = basisvalues[12]*(1.265625 + 2.8125*Z) - 0.546875*basisvalues[4];
      basisvalues[44] = basisvalues[25]*(0.733333333333333 + 2.44444444444444*Z) - 0.711111111111111*basisvalues[12];
      basisvalues[43] = basisvalues[24]*(1.76 + 3.3*Z) - 0.54*basisvalues[11];
      basisvalues[40] = basisvalues[22]*(1.76 + 3.3*Z) - 0.54*basisvalues[10];
      basisvalues[0] *= std::sqrt(0.75);
      basisvalues[3] *= std::sqrt(1.25);
      basisvalues[9] *= std::sqrt(1.75);
      basisvalues[19] *= std::sqrt(2.25);
      basisvalues[34] *= std::sqrt(2.75);
      basisvalues[55] *= std::sqrt(3.25);
      basisvalues[2] *= std::sqrt(2.5);
      basisvalues[8] *= std::sqrt(3.5);
      basisvalues[18] *= std::sqrt(4.5);
      basisvalues[33] *= std::sqrt(5.5);
      basisvalues[54] *= std::sqrt(6.5);
      basisvalues[7] *= std::sqrt(5.25);
      basisvalues[17] *= std::sqrt(6.75);
      basisvalues[32] *= std::sqrt(8.25);
      basisvalues[53] *= std::sqrt(9.75);
      basisvalues[16] *= std::sqrt(9.0);
      basisvalues[31] *= std::sqrt(11.0);
      basisvalues[52] *= std::sqrt(13.0);
      basisvalues[30] *= std::sqrt(13.75);
      basisvalues[51] *= std::sqrt(16.25);
      basisvalues[50] *= std::sqrt(19.5);
      basisvalues[1] *= std::sqrt(7.5);
      basisvalues[6] *= std::sqrt(10.5);
      basisvalues[15] *= std::sqrt(13.5);
      basisvalues[29] *= std::sqrt(16.5);
      basisvalues[49] *= std::sqrt(19.5);
      basisvalues[5] *= std::sqrt(15.75);
      basisvalues[14] *= std::sqrt(20.25);
      basisvalues[28] *= std::sqrt(24.75);
      basisvalues[48] *= std::sqrt(29.25);
      basisvalues[13] *= std::sqrt(27.0);
      basisvalues[27] *= std::sqrt(33.0);
      basisvalues[47] *= std::sqrt(39.0);
      basisvalues[26] *= std::sqrt(41.25);
      basisvalues[46] *= std::sqrt(48.75);
      basisvalues[45] *= std::sqrt(58.5);
      basisvalues[4] *= std::sqrt(26.25);
      basisvalues[12] *= std::sqrt(33.75);
      basisvalues[25] *= std::sqrt(41.25);
      basisvalues[44] *= std::sqrt(48.75);
      basisvalues[11] *= std::sqrt(45.0);
      basisvalues[24] *= std::sqrt(55.0);
      basisvalues[43] *= std::sqrt(65.0);
      basisvalues[23] *= std::sqrt(68.75);
      basisvalues[42] *= std::sqrt(81.25);
      basisvalues[41] *= std::sqrt(97.5);
      basisvalues[10] *= std::sqrt(63.0);
      basisvalues[22] *= std::sqrt(77.0);
      basisvalues[40] *= std::sqrt(91.0);
      basisvalues[21] *= std::sqrt(96.25);
      basisvalues[39] *= std::sqrt(113.75);
      basisvalues[38] *= std::sqrt(136.5);
      basisvalues[20] *= std::sqrt(123.75);
      basisvalues[37] *= std::sqrt(146.25);
      basisvalues[36] *= std::sqrt(175.5);
      basisvalues[35] *= std::sqrt(214.5);
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.0, 0.231075146484353, 0.0, 0.0, 0.0, 0.0, 0.0, -0.109364720199429, 0.0, 0.0100832791193172, -0.093145342621631, 0.0, 0.0, 0.0, -0.00450937950937958, -0.104139658944738, 0.0, -0.0924422799422799, 0.0, 0.0, 0.0, 0.0, -0.066884906990401, -0.0579240285834466, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.0647610327216138, 0.0, 0.0835038859404355, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0699149613259878, 0.0242192530451648, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338355, 0.0270779480866287, 0.0, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033679, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0963825010955e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07182472236942e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.54134980621128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-4.26264312686495e-14, 11.2249721603218, 0.0, 0.0, 1.26199643759963e-14, 0.0, 2.20934381900406e-14, 0.0, -1.27675647831893e-14, 0.0, 0.0, -1.1785469424736e-14, -1.09224868621293e-14, 0.0, -1.36448266108808e-14, -2.62012633811537e-14, 1.19394138887002e-14, 1.09751338184944e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06880604844687e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.87566282508212e-14, 0.0, 0.0, 0.0, -1.54732748419737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495584, 0.0, 8.36660026534078, -1.18321595661995, 0.0, -1.28392932639424e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56216114427701e-14, 0.0, 0.0, 0.0, 0.0, 2.10810325011374e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.50475584837571e-14, 0.0, 1.71654174825898e-14, -1.37279652885435e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.00567859040518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08184959687954e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771366, 0.0, -1.74856858249474e-14, -1.0238544149921e-14, 0.0, 3.38878512135699e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.64575187207114e-14, 1.66947012558271e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23857659559326e-14, 0.0, 0.0, -1.95093666748096e-14, 0.0, 0.0, 0.0, 0.0, 1.14508937720839e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394702, 0.0, -3.34664010613629, -2.36643191323983, 15.4919333848297, 0.0, 0.0, 0.692820323027547, 0.565685424949258, 0.400000000000014, 0.0, -1.28646896990653e-14, 0.0, 1.31292113199272e-14, 0.0, 0.0, 0.0, 0.0, 1.07917873811138e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.12116722044949e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00931743706047e-14, -1.27621425659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.98283431490247e-14, 4.89897948556636, 1.23234755733392e-14, 1.44515352066118e-14, 0.0, 14.1985914794391, -0.828078671210815, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 1.53986796602018e-14, 2.88015020841466e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-5.90398881748247e-14, 4.24264068711927, -1.86517468137026e-14, 0.0, 1.72765844533338e-14, 0.0, 14.3427433120128, 0.0, 0.0, -1.15745997780515e-14, 0.0, 0.0, -2.49692215906733e-14, 0.0, -1.13715454909234e-14, -2.48689957516035e-14, 0.0, 0.0, -1.68753899743024e-14, -1.69731699651606e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82983484340767e-14, 1.11853745822364e-14, 0.0, -1.83885365366384e-14, 0.0, -1.05423101080276e-14, 0.0, 0.0, 0.0, 0.0, -2.66453525910038e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.59999999999998, 0.0, 8.76356092008267, -1.54919333848301, 0.0, -2.38567411889758e-14, 0.0, 9.52470471983254, -1.48131215963609, 0.261861468283198, 0.0, 0.0, 0.0, -1.43665553045169e-14, 0.0, 1.02959924858014e-14, 2.49713702337855e-14, -2.54889747301533e-14, 1.7560335832795e-14, -3.58602036953926e-14, 0.0, 0.0, 0.0, 0.0, -1.11137823752462e-14, 0.0, 0.0, 0.0, -2.70923224185969e-14, 0.0, 1.97030508023093e-14, -1.52709959158275e-14, 1.09128088985221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46747734233077e-14, 1.08707871082455e-14, 0.0, 0.0, -1.28785870856518e-14},
      {3.11769145362397, -1.14489214403993e-14, 3.16227766016839, 4.91934955049952, 0.0, -2.49387456530104e-14, -1.16075338426604e-14, 0.0, 10.690449676497, -2.41897262725906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.04822096683565e-14, 0.0, 0.0, 0.0, 0.0, 1.37935139321432e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.54558441227155, 0.0, -2.03409482370539e-14, 7.66811580507231, 0.0, 0.0, 0.0, 0.0, 1.63572980217868e-14, 10.3691851174526, 0.0, 0.0, 0.0, 0.0, 1.38930326897849e-14, 2.05383528766067e-14, 0.0, 0.0, -1.68932504701822e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0682486967635e-14, -1.06915803460555e-14, 0.0, 0.0, 0.0, 2.15370577812888e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15499345880716e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 11.6057691494799, 0.0, 1.43893582105451e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363963, 0.432521479156767, 1.01374744547013e-14, 1.08422297243272e-14, 1.85407245112401e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36548170217743e-14, 0.0, 0.0, 1.78559914499265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171167, -2.50713268211202, 4.78713553878169, 1.74124097855807e-14, 0.0, -5.5662674507549, -1.04880884817014, 0.741619848709566, 0.0, 18.8034951158402, -0.603122387046452, 0.0, -2.67931485491449e-14, 0.0, 1.68183573174416, 0.70128378121766, 0.132137494528678, -0.0934353184302139, 0.0, 1.03177069223927e-14, 0.0, 0.0, 0.0, 0.0, 1.57341886107826e-14, 0.0, 2.28751383618163e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01335845653222e-14, 0.0, 0.0, 0.0, -1.00561538312514e-14, -1.05033316407144e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095218, 0.0, -1.58564993434418, 4.48489528465112, 4.28174419288837, 1.44444413391423e-14, 0.0, 0.191485421551253, -3.75233260785874, -4.311612227462, 0.0, 0.0, 18.8807171866294, -1.57274211472475e-14, 0.0, 0.0, 0.0, 0.844371341865057, 1.18187368057054, 1.04463861754669, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19177379213116e-14, 0.0, 0.0, 1.54885490901585e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32641960267291e-14, 0.0, -1.7557880399685e-14, -1.80589168489021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {7.42746476816679e-14, 2.59512887494071, 1.36557432028894e-14, 2.54026502394994e-14, 0.0, 10.0285307284481, -0.767649473578754, 0.0, 1.23234755733392e-14, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061158, 0.0967147429123294, 0.0, 2.61668265339638e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09385215324808e-14, 0.0, 0.0, 1.13448661536613e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36450623234388e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.48136050245152e-14, 2.32115382989599, -2.04281036531029e-14, 0.0, 0.0, 3.92428337406971, 5.49285249860478, 0.0, -1.72084568816899e-14, 0.0, 0.0, 0.0, -1.65660109295098e-14, 0.0, 17.3044631365566, -1.73008591662718, 0.0, 1.45156403921063e-14, -4.17443857259059e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20756069426397e-14, 0.0, 0.0, -1.97659376751031e-14, -1.15121348390167e-14, 0.0, 0.0, -1.09492494854124e-14, -1.34065952805688e-14, 1.44790629016451e-14, -1.77635683940025e-14, 0.0, 0.0, -1.66360477851868e-14, 0.0, 0.0, 0.0},
      {-1.38513084179526e-14, 2.01017818278144, 0.0, 1.51158203580928e-14, 0.0, 1.06048187717535e-14, 8.91928088068606, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 1.22685323290123e-14, -1.15257070420338e-14, 0.0, 16.8558564884489, 0.0, 0.0, 0.0, -1.19994280261913e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.09187410067987e-14, 0.0, 0.0, 0.0, -2.30926389122032e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483828, 0.0, 8.12403840463598, -1.6413036132966, 0.0, 0.0, 0.0, 11.7728501222092, -2.40312296813959, 0.485504156227617, 0.0, -1.73622699609175e-14, 0.0, 0.0, 0.0, 0.0, 10.2761860629321, -1.48323969741916, 0.302765035409784, -0.0611677741841419, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.16637502698581e-14, -1.56242142601618e-14, 1.45816858242235e-14, -1.0517682402565e-14, 0.0, 0.0, 0.0, 0.0, -1.1579364932548e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35791994434725e-14, 2.696279444274e-14, 0.0, 0.0, 0.0},
      {2.6532998322843, 0.0, 4.15220619990514, 2.93605316083769, 0.0, -2.25061986372436e-14, 0.0, 2.63248931621766, 9.82591326180784, -2.8226127713774, 0.0, 0.0, 1.39754921677686e-14, 0.0, -2.15559497296283e-14, 0.0, 1.27665076146717e-14, 11.6081867662439, -3.09487177319466, 0.683876505540205, 0.0, 0.0, 0.0, 0.0, -1.06854112020434e-14, 0.0, 0.0, 1.08660056184871e-14, -2.41898167452585e-14, 1.02184030372177e-14, 1.2242283673618e-14, -1.28809038754384e-14, 1.06443785616132e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18794700369427e-14, -1.18605364367499e-14, 0.0, 0.0, 0.0, -1.37059813754499e-14, 0.0, -1.07907956320157e-14, 0.0, -1.48769885299771e-14},
      {2.29782505861519, 0.0, 1.49829835452876, 5.50917601898536, 0.0, 0.0, 0.0, 0.0, 6.64803945320941, 4.13676892548485, 0.0, -1.54557441104667e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -2.35755013490266e-14, 12.5636136423842, -3.55352656109505, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19757697199597e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20890589999235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.62307824705078e-14, 0.0},
      {1.87616630392939, 0.0, -1.57619878448508e-14, 6.22830929985768, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03262891201015e-14, -1.4946943707119e-14, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06925738440755e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14026919094163e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722504, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.54645430581652e-14, 2.13065307820925, 1.73967095350096, 1.23013312825379, 0.0, -4.36653943835009, -3.7815340802378, 0.0, 2.25563515810257e-14, 0.0, -0.557935943461964, -0.483186700722506, -0.394520289089677, -0.278967971731003, 23.6981011897578, 0.0, 0.0, 0.490653381462667, 0.438853725736259, 0.380058475033042, 0.0, 0.0, 0.0, 0.0, 0.062693389390889, 0.0560746721671603, 0.0485620906056366, 0.0396507809422156, 0.0280373360835742, 0.0, 0.0, 0.0, 0.0, 1.01344505238142e-14, 0.0, 0.0, 0.0, 0.0, 1.12177887638196e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.39474250096951e-14, 6.91050664139875, 0.0, 0.0, 0.0, 10.5706241650195, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -3.6506547506255e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365253, 1.37880481739187e-14, 0.0, 2.42028619368284e-14, 0.0, 0.0, 23.1055548876599, -0.469687194424179, 0.0, -1.01996918305813e-14, 0.0, 1.62065746295905, 0.790669146636832, 0.240926884676757, -0.0621208350290903, 0.0, -1.17087873808634e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28374978981536e-14, 0.0, 0.0, -1.2197115318761e-14, 1.59691888895572e-14, 0.0, 0.0, 0.0, 0.0, -1.33608212870904e-14, 0.0, 0.0, 0.0, 0.0, 1.28959284686747e-14, 0.0, 0.0, 0.0},
      {3.00289749974576e-14, 6.3084006188056, 0.0, 0.0, 0.0, -1.5236235005501, 11.1963004093827, 1.08081606833763e-14, 0.0, 0.0, 4.26614580154031, 0.0, 1.59211151818687e-14, 0.132992762321612, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19301771263474e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560574, 1.11125127955291, 1.1908743922773, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.43217810167785e-14, 0.0, 0.0, 1.69426718983743e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.08400918081233e-14, 1.2314217778122e-14, 0.0, 0.0, 0.0},
      {3.85449644663768, 0.0, 5.98171817966646, -2.48806675764058, 1.90029237516523, 2.42928092684253e-14, 0.0, -0.339934634239513, -2.35922146606162, 0.981306762925309, 0.0, 9.95226703056236, -0.43094580368566, 0.0, -1.09020872085301e-14, 0.0, -6.45364422706456, 0.0770899289327356, 0.535021138879208, -0.222539456105664, 0.0, 1.25806741269997e-14, 0.0, 21.5273297444417, -1.00024047132727, 0.043311682903134, 0.0, 0.0, 3.07692856596062e-14, 0.0, 2.75066129614999, 0.64861564944292, -0.00774782937530969, -0.0537716476536622, 0.022366056877348, 0.0, -1.81421059392863e-14, 0.0, -1.50126980556235e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04387256037139e-14, 0.0, 0.0, 0.0, 0.0, 1.11297525744228e-14, 0.0, -1.19267292859759e-14, 0.0, 0.0, 0.0},
      {3.51865775274498, 0.0, 1.92724822331888, 2.72554057547704, 1.73472166622178, 3.03480499348859e-14, 0.0, -2.01705689521109, 3.42052627529743, -4.03112887414927, 0.0, 4.08831086321547, 5.37643791520788, 0.0, 0.0, 0.0, 0.365669640132029, -6.25148194071022, -2.03981013744549, 1.80803322065282, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 1.4869174798376e-14, 0.0, 3.35300530003437e-14, 0.0, 0.0, 1.98456380761454, 1.4711357857929, 0.363818136386338, -0.294009452979938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42008984758617e-14, 0.0, 0.0, 0.0, -1.81106534534235e-14, -1.11644518738555e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66529227640905e-14, 0.0, 0.0},
      {3.14718316987777, 0.0, -0.861891607371338, 6.09449400220046, 1.55158222708544, 1.55866910664479e-14, 3.57119071785627e-14, 0.0693888666488674, -3.0594117081557, 1.802775637732, 0.0, 1.18900460355751e-14, 9.0312215653788, -1.94931540293056e-14, -2.60083475239832e-14, -1.5561245005079e-14, 0.0, 0.403888506801003, -3.67461994579465, -5.49650682721194, 0.0, 0.0, 0.0, 0.0, -1.66659503055397e-14, 21.7841260588565, 0.0, 1.05933934516737e-14, 1.23500749154168e-14, 1.34515365696787e-14, 0.0, 0.0, 0.974215733960542, 1.78974858569576, 1.80791908794929, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0494061411345e-14, 0.0, 1.3508514750524e-14, 0.0, -1.4321054699507e-14, -1.27312942544494e-14, -1.06704051106469e-14, 0.0, 0.0, 0.0, -1.21622256639311e-14, 0.0, 0.0, -1.4210854715202e-14},
      {1.16385412951124e-14, 1.54523626091312, 1.74790737439423e-14, 0.0, 0.0, 6.7177802456296, -0.609449400220053, 0.0, 1.63697180810551e-14, 0.0, 0.0, 0.0, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827959, 1.23001618729603e-14, 0.0, 1.90680804479371e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864228, -0.013890640994407, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.08091827158929e-14, 0.0, 0.0, 0.0, 0.0, 1.22032198955284e-14, 0.0, 0.0, 0.0},
      {5.29577580634154e-14, 1.41060126129511, 0.0, 2.28860345505758e-14, 0.0, 3.63405410063598, 2.50356888118883, 1.20056198225602e-14, 0.0, 0.0, 0.0, 0.0, 1.19705162678985e-14, 3.56857028479909, 11.263055412878, -1.49298814645066, 0.0, 0.0, -1.33226762955019e-14, 2.13441232972003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124152, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67577136606882e-14, 0.0, 0.0, 1.70127694606852e-14, 0.0, 0.0, 0.0, 0.0, 1.22094549594888e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.09263923666412e-14},
      {-5.47392883685355e-14, 1.26168012376111, -1.46549439250521e-14, 0.0, 0.0, 1.42204860051344, 4.4785201637531, 0.0, 0.0, 0.0, 1.13386714596404e-14, 0.0, 0.0, 1.0561106878038e-14, 8.27725128825318, 5.37909280973863, 0.0, 0.0, -1.4210854715202e-14, -3.26625440232753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.9654813226339, -2.61992366301005, 0.0, 0.0, 0.0, 0.0, -2.34427068588597e-14, 0.0, 0.0, 0.0, 0.0, -1.17657244882003e-14, 0.0, 0.0, -1.26821313670928e-14, 0.0, 0.0, 0.0, -1.22588883007438e-14, 0.0, 0.0, -2.30926389122032e-14, 0.0, -1.09431313650455e-14, -1.3756669688576e-14, 1.83280317299074e-14, 0.0, -1.71020098010008e-14},
      {2.99252416216265e-14, 1.09264703862704, 1.59872115546023e-14, 1.87579382203798e-14, 0.0, -1.06100948104966e-14, 5.38682254607085, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46786397162884e-14, 0.0, 0.0, 13.4377817958113, 1.61562731835987e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0223562098124e-14, 0.0, 0.0, 0.0, 0.0, -1.03609397032104e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752223, 0.0, 7.37120708415868, -1.62882203585591, 0.0, 1.46146638131678e-14, 0.0, 12.0171306297065, -2.90724328050878, 0.642416074439626, 0.0, -1.41650253574207e-14, 0.0, 0.0, 0.0, 0.0, 13.9858820944259, -2.72522906732404, 0.659300804656282, -0.145686271816933, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24196920770985e-14, 0.0, 10.8043830863937, -1.40563403846548, 0.273895826790433, -0.0662622240310456, 0.0146420212340156, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26791391102222e-14, 0.0, 0.0, 0.0},
      {2.30350221379957, 0.0, 4.41588043316395, 1.78428514239951, 0.0, 0.0, 0.0, 4.26614580154034, 7.83741028656786, -2.63899331455874, 0.0, 0.0, 1.34443561629548e-14, -1.37467072899596e-14, 0.0, 1.19086222563515e-14, 2.23427840700306, 13.2221027072096, -4.67379217909109, 1.18363558466229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.82290665255434e-14, 0.0, 1.70537687486985e-14, 12.1258851598178, -3.11150358274814, 0.833612074594092, -0.192474367616152, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.35721538900487e-14, 0.0, 0.0, 0.0, -2.63628815566622e-14, 2.1087783356318e-14, -1.8246160586734e-14, 1.78716172435152e-14, 0.0},
      {2.06031501455084, 0.0, 2.25696201807215, 3.98978286964829, 0.0, 1.39680156434655e-14, 0.0, 0.953939201416941, 8.01142041980429, 1.18019368870419, 0.0, 0.0, 0.0, 0.0, -2.40562940285141e-14, -2.69425057681437e-14, -2.30316410777416e-14, 5.55254896421456, 9.6224137438848, -3.59830837050571, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19383851062166e-14, 0.0, 0.0, 3.42746616939185e-14, 0.0, -1.06136959221059e-14, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.14976200658074e-14, 0.0, 0.0, -1.36039442037672e-14, 1.06476565092571e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 0.0, 0.814411017927954, 5.06770487035908, 0.0, 0.0, 2.16088195115045e-14, 0.0, 4.01510046524762, 6.58672305740271, 0.0, 1.33582894449063e-14, 0.0, -1.83180284470628e-14, 1.39370104874447e-14, 0.0, 2.22905902979741e-14, -3.16802833418626e-14, 10.0159311874144, 2.83293314500739, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.38395931824674e-14, -1.2405259764819e-14, 0.0, 14.0929454377398, -4.55552790213388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.45686271816937, -1.94128864338579e-14, 1.85103508870954e-14, 5.0781645406624, 0.0, 0.0, 0.0, 0.0, -2.39227846819003e-14, 10.0142755247554, 0.0, 0.0, 2.07229054481962e-14, 0.0, -4.329222701254e-14, -1.48420279375759e-14, 0.0, 3.16791989562027e-14, 0.0, 13.8784813613723, 0.0, 1.15908071798877e-14, 1.01997671695376e-14, 0.0, -1.1290335532565e-14, 0.0, 0.0, 1.75166794613893e-14, 0.0, 1.47536073401433e-14, 0.0, 0.0, 0.0, 0.0, 12.5535587278944, 0.0, 0.0, 1.12696365862788e-14, 0.0, -2.32538552005964e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82081327096343e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505169, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67188948864322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32815854628304e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747789, 7.24568837309472, 4.18330013267038, -0.591607978309974, 0.0, 0.0, 2.26176274380129e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03250585805554e-14, -1.31145122910058e-14, 1.44471272378285e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.03986948355621e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.10583422477808e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385683, 0.0, 0.0, 0.0, 0.0, 1.69439256067849e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.64575131106464, -1.09623763055384e-14, 9.66091783079296, 0.683130051063945, 0.0, -2.12700318229128e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11468586039765e-14, -1.04123608395564e-14, -1.1415987507357e-14, 0.0, 0.0, 1.46295421614672e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21636815751291e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07136521876328e-14},
      {3.24037034920392, 0.0, 0.0, 7.52994023880668, 0.0, -1.54171906569282e-14, 0.0, 0.0, 2.88102874890228e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.46010659930339e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01994612478062e-14, 0.0, 0.0, -1.72249746857342e-14, 0.0, 0.0, 0.0, 0.0, 1.29935052008754e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474621, -0.999999999999996, 9.16515138991168, 7.09929573971954, -0.414039335605409, -2.04939015319193, -0.478091443733749, 0.169030850945711, 0.0, 0.0, -1.36403152584891e-14, 0.0, 0.0, 0.0, 2.062713406596e-14, 2.47113051218353e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355964, -0.408248290463869, 3.17542648054293, 1.04942155502777e-14, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671767, 0.0, 0.0, -1.20580987145493e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000003, -5.69209978830308, 4.38178046004133, -0.774596669241496, 0.0, 10.998181667894, 0.962140470884726, 4.76235235991627, -0.740656079818042, 0.130930734141604, 0.0, 0.0, 0.0, -1.25005096694326e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.97064586870965e-14, 0.0, 0.0, 0.0, 0.0, -1.04177737563054e-14, 0.0, 0.0, 0.0, -1.73121731249159e-14, 0.0, 1.0640279773481e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01122181673985e-14, 0.0, 0.0, 0.0, 1.02717231839738e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681195, 2.73861278752582, 1.58113883008418, 2.45967477524975, 1.08323640616754e-14, 0.0, 9.25820099772553, 0.0, 5.3452248382485, -1.20948631362954, 0.0, 0.0, -1.30909999181878e-14, 0.0, 0.0, -2.07523142856229e-14, 0.0, 0.0, -1.83958955876703e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07078440897483e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0466576534315e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, 0.0, -1.0170474118527e-14, 3.83405790253616, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872628, 0.0, 0.0, 0.0, 0.0, 0.0, 1.02691764383034e-14, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.07685288906444e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 0.0, -3.16227766016839, -2.2360679774998, 0.0, -1.02482590321826e-14, 0.0, 13.7477270848675, 0.534522483824839, 0.37796447300923, -1.18033225968848e-14, -2.13006114292071e-14, 1.44091691895902e-14, 0.0, -1.02340641264601e-14, 0.0, 0.0, -1.05467039066338e-14, 2.90165469153009e-14, -2.68673971959288e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11262494741408e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.39460374316227e-14, 0.0, -1.75383384845777e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.60835200781568e-14, 0.0, 0.0, 0.0},
      {-1.80000000000001, 0.0, 3.65148371670109, -2.84018778721882, 0.0, 0.0, 0.0, 1.23940361622953e-14, 12.3442679969673, 1.39659449751036, 0.0, 0.0, 0.0, 0.0, -1.61223992078066e-14, -1.4992905752609e-14, 0.0, -1.19339350229854e-14, -3.15190779176534e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59624575451928e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4469063457423e-14, 0.0, 0.0},
      {2.20454076850485, 0.0, -1.56185210562598e-14, 6.64078308635358, 0.0, 0.0, 0.0, 0.0, 1.25531713073239e-14, 8.97997772825744, 0.0, 0.0, 0.0, 0.0, 1.41883969527382e-14, 1.42511083398306e-14, 0.0, 0.0, -1.2632727205578e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01559019437869e-14, 0.0, 0.0, 0.0, 1.93018308907762e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239555, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229246, -2.78313372537744, -0.524404424085068, 0.37080992435479, 11.1242977306435, 9.40174755792012, -0.301561193523227, -2.42752078113804, -0.794200206656974, 0.114434427054264, 0.840917865872089, 0.350641890608842, 0.0660687472643376, -0.0467176592151071, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00474274489137e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06496398337265e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047606, 2.74642624930237, -0.792824967172099, 2.24244764232556, 2.1408720964442, -0.33166247903553, 6.49923072370878, 0.0957427107756249, -1.87616630392937, -2.15580611373101, 0.0, 0.0, 9.44035859331467, 0.0, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932534, 0.590936840285266, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11834562701699e-14, 0.0, 0.0, -1.44685497566301e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.91485421551271, 1.29756443747036, 3.74574588632196, -1.05945692672795, -8.09173593712687, 5.01426536422406, -0.383824736789376, 1.80936716113937, -1.1080065755349, 0.313391585264, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648288, -0.631738055305803, 0.0483573714561709, -2.84282124887606, -0.227958835180076, 0.139595707137598, -0.0394836284566188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 2.53311402559511, 1.96214168703486, 2.74642624930239, -0.566421515598886, 3.17129986868837, -1.82198870938949, 0.0, 0.0, 11.1699829238239, 0.0, 8.65223156827828, -0.8650429583136, 0.0, -2.49768411251819, -0.998865569685896, 0.441440386136657, 0.0, 0.0, 0.0, 0.0, 0.0, 1.54576772432364e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02109627461819e-14, 0.0, 0.0, -1.71420357644706e-14, -1.44857733991283e-14, 1.540182245046e-14, -1.26003377560893e-14, 0.0, 0.0, -1.26725690998595e-14, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139072, -0.193429485824669, 3.55615782880926, 0.0, 0.0, 4.45964044034303, 0.0, -0.858258202906975, 2.67027285925185, 0.0, 0.0, 0.0, 0.0, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683795, -2.2937915319045, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00000619766181e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20046826012523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741916, 7.03562363973514, 4.06201920231799, -0.820651806648287, 0.0, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.242752078113807, 0.0, 0.0, 0.0, 14.8323969741913, 0.856348838577663, 0.262202212042538, 5.13809303146607, -0.741619848709576, 0.151382517704907, -0.0305838870920611, 0.0, -1.09090068087904e-14, 0.0, 0.0, 0.0, 0.0, -1.06687389147534e-14, 0.0, 0.0, 0.0, 1.04099252608575e-14, 0.0, 1.16889315344106e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07658419655195e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, -2.69693703815181, 2.07610309995256, 1.46802658041886, -1.29898573756985e-14, 3.03973683071413, -6.38211787508102, 1.31624465810883, 4.91295663090391, -1.41130638568868, 0.0, 0.0, 1.04236157429541e-14, 0.0, 13.4039795085887, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659736, 0.341938252770112, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02250450744812e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.3823750891058e-14, 0.0, 0.0, -1.09194416692302e-14, 1.48519464798164e-14, 0.0, 0.0, -1.26317215836086e-14, -1.30320973445429e-14, 0.0, 0.0, 0.0},
      {1.14891252930759, 1.29756443747034, 0.74914917726438, 2.75458800949268, 0.0, 0.0, 5.75737105184055, 0.0, 3.32401972660471, 2.06838446274242, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8804085776374, 0.0, -1.78283030874237e-14, 6.28180682119208, -1.77676328054755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43699990430023e-14, 0.0, 0.0, 0.0, 0.0, -1.52208185928549e-14, 0.0},
      {0.938083151964693, 0.0, 0.0, 3.11415464992884, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.4253953543107, 0.0, 9.38083151964688, 1.89521416591739, 0.0, 3.92670483430763e-14, 0.0, -3.39852909359328, -2.77488738510232, -0.560611910581397, 0.0, 0.0, 0.0, 1.4776781754089e-14, 0.0, 0.0, 17.7988763690296, 0.428174419288849, 0.349602949390068, 0.0706304617818849, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.14945855437217e-14, 1.92030515491915e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102535, 0.0, -1.49829835452878, 4.2378277069118, 0.0, 0.0, 0.0, 3.79967103839267, -3.54562104171171, -4.07409060843205, 0.0, 0.0, 2.55710987738197e-14, 0.0, -1.43687825188852e-14, 1.11266834553736e-14, 0.0, 16.7549743857359, 1.11676565710084, 0.98709071141529, 0.0, 0.0, 0.0, 0.0, -1.127306762275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3365284601573e-14, 0.0, 0.0, 0.0, 1.44790503330067e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3171354510566e-14, 0.0, 0.0, 0.0, -1.12168816246188e-14, 0.0, 0.0, 1.10138148261537e-14, 0.0},
      {-1.32664991614215, 0.0, 1.73008591662713, -3.18072425757422, 0.0, 2.00847335080497e-14, 0.0, 0.0, 7.67649473578736, -2.38836465270395, 0.0, 0.0, 1.48618125636802e-14, 0.0, -2.1585332440305e-14, -2.30549613990546e-14, 0.0, -1.8131228166119e-14, 14.5072114368499, 2.05162951662074, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71710712940356e-14, 0.0, 0.0, 3.25814406941166e-14, 0.0, 0.0, 0.0, 0.0, -1.24188980587068e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1158628913284e-14, 0.0, 0.0, 0.0, 1.02140518265514e-14},
      {1.6248076809272, 0.0, -1.17379983736491e-14, 5.39387407630362, 0.0, 0.0, 1.00014277056463e-14, 0.0, 0.0, 9.57317681262151, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15094222712566e-14, -1.30167446385747e-14, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07993551244188e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725402, 3.45525332069938, 0.997445717412056, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234729, 0.556348640264167, 2.33666428910959, 9.02784807228866, -0.732969916135151, -3.93352933905727, -1.19859611757307, 0.309047252182632, 1.64018416088946, 0.655588223176214, 0.0892142571199814, -0.126168012376116, 13.0996183150503, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163167, 0.810328731479525, 0.395334573318413, 0.120463442338385, -0.0310604175145443, -0.31189567416786, -0.164844710568313, -0.0658890955530754, -0.00896637020658186, 0.012680362351405, 0.0, 0.0, 0.0, 1.04258060106039e-14, 0.0, 0.0, -1.16733600039681e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091314, 2.9504842217604, -0.761811750275048, 5.59815020469135, 0.263899331455871, -1.93925611658549, -2.28543525082516, 2.13307290077016, -0.257539376818854, 9.14446135479766, 0.0664963811608009, -2.36108976914457, -3.33842298506509, -0.0230350221379948, 0.817905488277907, 1.1564636454577, 1.02505848514083, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776453, 0.595437196138652, 0.0, -0.125015768949705, -0.19247436761616, -0.206265495285707, -0.166687691932935, 0.0, 0.0, 0.0, 0.0, 1.77034931402367e-14, 0.0, 0.0, 1.41972107734095e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066014, 2.99085908983324, -1.24403337882029, 0.950146187582627, 5.74064456311311, -0.721110255092819, -0.169967317119738, -1.17961073303081, 0.490653381462651, -10.3037210107158, 4.97613351528116, -0.215472901842826, 5.7817446699566, -1.30185581821821, 0.163532434528624, -3.22682211353226, 0.0385449644663676, 0.26751056943962, -0.111269728052834, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663637, 0.0216558414515652, -3.57321383950879, -0.581087203147984, 0.130841432732466, -0.0164356280722688, 1.375330648075, 0.324307824721451, -0.00387391468765224, -0.0268858238268329, 0.0111830284386736, 0.0, -2.069139004663e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36008413681502e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079257, 0.963624111659438, 1.36277028773853, 0.867360833110876, 2.28430636201794, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707462, 2.41867732448956, 2.04415543160773, 2.68821895760395, -0.527798662911744, 7.07977053255619, -1.76652739792483, 0.182834820066019, -3.1257409703551, -1.01990506872276, 0.904016610326408, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.632607619454893, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468084, 0.0, 0.992281903807272, 0.735567892896452, 0.181909068193171, -0.147004726489971, 0.0, 0.0, 0.0, 0.0, 1.75102175374012e-14, 0.0, 0.0, 1.34369051637097e-14, 0.0, 1.72746956349542e-14, 0.0, 0.0, 0.0, 1.16610098942935e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.57359158493887, 1.49284005458434, -0.430945803685673, 3.04724700110023, 0.775791113542723, -0.120185042515449, 5.29905651979674, 0.0346944333244366, -1.52970585407784, 0.901387818865997, 0.0, 0.0, 4.5156107826894, -1.79618510159529e-14, -0.699555414372444, 6.36462844462228, 0.0, 0.201944253400494, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.0999354831996, 0.0, 0.0, 0.487107866980268, 0.894874292847878, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14},
      {1.62882203585591, 0.772618130456569, 3.86595114186569, -1.05140010313425, 9.1544290607646, 3.35889012281482, -0.304724700110031, 3.87851223317103, -1.52475169285615, 0.414677792940097, 0.0, -9.46260092820844, -2.07602937339135, 7.32969916135159, -0.761724681099356, 0.0691050664139777, 0.564240504518078, -0.879563899362198, 0.345781182036209, -0.0940400840863403, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584715, 9.05975063854701, -0.736662483163212, 0.0765562109321208, -0.00694532049720376, -3.48709964663733, -0.0567083043941667, 0.0883994979795876, -0.0347523163751443, 0.00945138406569324, 0.0, -2.40973042824816e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.79022103639014e-14, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647554, 2.03602754481989, 1.1517511068998, -2.93221479395415, 1.81702705031798, 1.25178444059441, 0.655663160391215, 3.61359041872286, -1.70346285967312, 0.0, 2.7642026565595, -9.08783875864325, 1.78428514239955, 5.631527706439, -0.746494073225338, -0.618094504365246, 2.03210252328838, -2.15494277062553, 0.76403348456261, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462396, 0.0, 9.68367982311022, -1.32524448062076, 0.133143804689782, 0.0, -3.3545250915713, -0.478206411017786, 0.384353057392901, -0.124241670058193, 0.0, 0.0, 1.26141099031277e-14, 0.0, 0.0, -1.04142311537298e-14, 0.0, 1.30453578655817e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880564, 0.728431359084661, 2.57539376818857, 0.917928424547683, 0.711024300256731, 2.23926008187656, -0.205255035576786, 2.58567482211399, 0.761811750275065, 0.0, 0.0, 5.34294273196309, 0.0, 4.13862564412657, 2.68954640486931, 0.0, -1.19471831485579, 3.10562068169858, -2.32269806558126, 0.0, 0.0, 1.04137318579115e-14, 0.0, 0.0, 12.8876627769294, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43119214085952e-14, 0.0, -1.29955648419624e-14, -1.09684519134626e-14, 0.0, 0.0, 0.0, -1.16096784799398e-14, -1.14869330647349e-14, 1.05799907917652e-14, 0.0, 0.0},
      {1.15175110689981, 0.546323519313529, -0.105140010313431, 3.2711894277325, 0.0, 0.0, 2.69341127303543, 0.0, -0.518347241175115, 4.25171145123352, 0.0, 0.0, 0.0, 0.0, 1.37212312898181e-14, 6.71889089790564, 0.0, 0.0, -1.29305115618719, 1.82865048192212, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264585, -2.94058061635795, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.19896685350443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840206, 3.68560354207936, -0.814411017927963, 0.0, 10.0216907612581, 2.67510569439611, 6.00856531485323, -1.45362164025439, 0.321208037219804, -1.54561210278816e-14, 0.0, 0.0, -7.57008074256671, -2.27270584034422, -0.606656948415849, 6.992941047213, -1.36261453366202, 0.329650402328146, -0.0728431359084714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084517, 0.228415531250647, 0.0609713174047712, 5.40219154319687, -0.70281701923275, 0.136947913395213, -0.0331311120155253, 0.00732101061700687, 0.0, 0.0, 0.0, -1.08379836100322e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.505711569012e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11424143311745e-14, 1.1468156352512e-14, 0.0, 0.0, 0.0},
      {1.1517511068998, 3.82426463519459, 2.20794021658198, 0.892142571199762, 1.29053191519313e-14, -2.46306042692149, 6.78739640804925, 2.13307290077017, 3.91870514328394, -1.31949665727938, 0.0, 0.0, 2.39783099316701e-14, 3.22490309931941, -7.63378455726036, -4.04762275910193, 1.11713920350153, 6.61105135360478, -2.33689608954553, 0.591817792331161, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499942, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297044, -0.0962371838080724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84612085315578e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.38069051168088e-14, -1.19991312128016e-14, 0.0, 0.0},
      {1.03015750727541, -1.4659398322703, 1.12848100903607, 1.99489143482413, 0.0, 1.10151410945722, -5.20357020296095, 0.476969600708466, 4.00571020990212, 0.590096844352097, 0.0, 0.0, 0.0, 1.68494124154739e-14, 6.41153127835568, -6.24994106094662, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.0440762145518, 0.0, 0.0, 6.69662601613678, -2.34333096254028, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.21941704251974e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.81331345921468e-14, 1.06621068653036e-14, 0.0, 0.0},
      {0.892142571199788, 0.705300630647561, 0.407205508963991, 2.53385243517955, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870135, -1.01114623239648e-14, 1.25375506556487e-14, 1.39962126651879e-14, 0.0, 0.0, 8.67405085085772, 1.68639974250234e-14, 0.0, 5.00796559370721, 1.41646657250371, 0.0, 0.0, 0.0, -1.01863921840645e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, -1.02508111934767e-14, 0.0, 7.04647271886991, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29274293852985e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.20086470944275e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.728431359084686, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, -1.19613923409502e-14, 5.00713776237769, 0.0, 0.0, 1.03614527240981e-14, 0.0, -2.164611350627e-14, 0.0, 0.0, 1.58395994781014e-14, 0.0, 6.93924068068617, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, -1.16269276002982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359275, -1.7113170615574e-14, -4.65474668125634, -3.29140294302193, 0.0, -1.1761894746004e-14, 0.0, 13.490737563232, 1.83585684909538, 1.29814682728311, 0.0, 0.0, 0.0, -1.87134080288653e-14, 0.0, -1.02151648729163e-14, -3.53270434653115, -3.05941170815568, -0.416333199893262, -0.294392028877587, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.465611581249e-14, 0.0, 21.8326971917504, 0.355050145839443, 0.30748244591433, 0.04184306096406, 0.0295875121532881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48387989870766e-14, 0.0, 0.0, 0.0, 0.0, -1.03538331415148e-14, 0.0, 0.0, -1.84166859624577e-14, 0.0},
      {-2.65985524643218, 0.0, 5.09901951359281, -2.06031501455085, 0.0, 3.32253482907945e-14, 0.0, -1.23153021346073, 9.04986187739902, 3.0472470011002, 0.0, 0.0, -1.95936513894522e-14, 0.0, 1.47247570213691e-14, 0.0, 3.86988371918332, -3.81689227863019, -5.39683034546923, -1.36674464685442, 0.0, 0.0, 1.25637822282896e-14, 0.0, 0.0, -1.10693425465008e-14, 0.0, -1.09132972116888e-14, 1.46675447063251e-14, 0.0, 0.0, 21.00264918355, 0.898213715542058, 0.962572311333251, 0.222250255910589, 0.0, 0.0, 0.0, 0.0, -1.0554173258815e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.66581518685003e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.41728882107196e-14, -1.00932416492049e-14, 0.0, 0.0},
      {2.97380857066588, 0.0, -0.81441101792795, 5.75875553449897, 0.0, 0.0, 0.0, 1.3768926368215, -2.89087233497831, 1.70346285967312, 0.0, 2.32509176961263e-14, 0.0, 0.0, -1.46522959991824e-14, -1.25534238592216e-14, 0.0, 8.01441409794465, -3.47218947830366, -5.19371076584692, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.48135503122828e-14, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252876, 1.7083229633002, 0.0, 0.0, 0.0, 0.0, -1.01360246623498e-14, 0.0, 0.0, -1.46120317582087e-14, 0.0, 0.0, 0.0, -1.15141416524375e-14, -1.29828016224512e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.03015750727544, -2.49712577805372e-14, 0.940400840863434, -2.92584077107539, 0.0, 0.0, 1.11242006769295e-14, -1.01183822699379e-14, 4.63623866886819, -3.80284633026897, 0.0, 2.51593477695333e-14, 0.0, -1.76429719944889e-14, -1.75344396051708e-14, -1.23675316462282e-14, 2.21199292279985e-14, -1.29852613729884e-14, 11.565401134477, -1.63559471386625, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42352724659797e-14, 0.0, 0.0, 0.0, 16.2731316843075, 2.63013526059784, 0.0, 0.0, 0.0, 0.0, -1.35422451249945e-14, 0.0, 0.0, 0.0, 1.22914688590703e-14, 1.10383788061295e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04022173966829e-14, 0.0, 0.0, 0.0},
      {1.26168012376113, -1.43388225778233e-14, 1.77601092747157e-14, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, -2.37244570631951e-14, 8.67261700493489, 0.0, 0.0, 1.83310017496234e-14, 0.0, -3.74050592048525e-14, -1.47574667599662e-14, 0.0, 2.73125639517699e-14, 1.08639591675699e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50668381437339e-14, 0.0, 1.29045354369716e-14, 0.0, 0.0, 0.0, 0.0, 10.8717007662564, 0.0, 0.0, 1.07335156299121e-14, 0.0, -1.97835851386513e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55478054673288e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.1622776601684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04819125054775e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835056, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494325, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.81236073388813e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24019196281157e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.95803989154981, 5.61248608016092, -1.08012344973464, -0.763762615825972, 0.0, 0.0, 1.2984368965554e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16694020690529e-14, 0.0, 1.01604504651519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747792, 1.44913767461894, 4.18330013267038, -0.591607978309973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21211990257508e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20099445186452e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338694, 7.09929573971953, 0.0, 4.34741302385683, 0.0, 0.0, 1.13120286894918e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, -1.13651438907064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231718, -0.341565025531998, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973462, 0.0, 7.09929573971954, 2.50998007960221, 0.0, -1.81931113492982e-14, 0.0, 0.0, 1.29340982368831e-14, 0.0, 0.0, 1.08136453583689e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.8556581303737e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912991, -1.08466105389134e-14, 0.0, 8.87411967464942, 0.0, -1.92488774177828e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.55747191971744e-14, 0.0, -1.11082577973414e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.29109484779505e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847577, -1.67332005306815, -1.18321595661991, 7.74596669241483, -1.2, -0.979795897113261, 0.346410161513771, 0.282842712474623, 0.200000000000003, 0.0, -1.14513537347049e-14, 0.0, 1.04049555239672e-14, -1.06781135142598e-14, 0.0, 0.0, 1.04750961573969e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06440475105247e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.32379000772447, 2.44948974278319, 2.82842712474619, -1, 1.30930734141596, 7.09929573971954, -0.414039335605406, -2.04939015319193, -0.478091443733752, 0.169030850945705, 0.0, 0.0, -1.03681357407313e-14, 0.0, 0.0, 0.0, 0.0, 1.74997090314722e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03023968932751e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974977, 2.12132034355963, -0.408248290463855, 3.17542648054294, 9.07114735222145, 0.0, 7.17137165600636, 0.0, -1.38013111868469, -1.56144011671766, 0.0, 0.0, -1.76128430047781e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12702757552127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.8, 0.632455532033673, 4.38178046004133, -0.774596669241501, 0.0, 3.14233761939828, -0.106904496764963, 4.76235235991627, -0.740656079818049, 0.130930734141599, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.43844057861131e-14, 0.0, 0.0, -1.65978342181461e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36075032726017e-14, 0.0, 1.13463922068443e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.558845726812, 0.547722557505162, 1.5811388300842, 2.45967477524976, 0.0, 9.07114735222144, 1.8516401995451, 0.0, 5.3452248382485, -1.20948631362953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.56503181802249e-14, -1.661953874827e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24615197438375e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613577, -6.26099033699942, -2.4825418043579e-14, 3.83405790253614, 0.0, 0.0, 10.5830052442584, 0.0, 2.08351914916202e-14, 5.18459255872627, 0.0, 0.0, -1.02243681968454e-14, 0.0, 1.75732163927929e-14, 1.4710068536804e-14, -1.89625726930492e-14, -1.39202390526635e-14, -2.44338367896933e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46716635798791e-14, 0.0, 0.0, 0.0, 1.16567073103445e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11100025269653e-14, -1.24344978758018e-14},
      {1.03923048454132, 0.0, 3.16227766016838, -0.447213595499971, 0.0, -1.15430323238016e-14, 0.0, 5.8918830363718, -0.534522483824852, 0.0755928946018485, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06379874836026e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.899999999999986, 0.0, 1.46059348668044, 1.42009389360937, 0.0, -2.06075074629368e-14, 0.0, 9.07114735222146, 4.93770719878694, -0.698297248755175, -1.06006922553717e-14, -1.57823220331274e-14, 0.0, 0.0, -1.11834345474354e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1646977159515e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52139975559217e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.734846922834963, 0.0, -6.26099033699942, 2.21359436211787, 0.0, 0.0, 0.0, 0.0, 10.5830052442584, 2.99332590941915, 0.0, -2.41530808380763e-14, 0.0, 1.22708289852378e-14, 0.0, 0.0, 0.0, 0.0, -2.61193100877958e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.20983509559115e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.1687807266701e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.71576766497732, 2.66443587314221e-14, -2.86936652466488e-14, -4.69574275274956, 0.0, 1.5233987604815e-14, 2.23297261202163e-14, 0.0, 0.0, 12.69960629311, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.7329081623293e-14, -2.48689957516035e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11145973108228e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 1.32745174758584e-14, 0.0, 0.0, 0.0, 1.30872303214021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14},
      {2.56904651573304, 5.80288457473996, -2.01017818278146, -1.42141062443801, 8.14215222512713, -2.1022946646802, -1.71651640581398, 0.728256234341417, 0.594618725379078, 0.420458932936048, 9.81070843517429, -1.18450885369836, -1.02581475831036, 0.305838870920603, 0.264864231681978, 0.21626073957838, -0.105945692672788, -0.0917516612761682, -0.0749149177264217, -0.052972846336388, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.26568606239553, 3.07059789431496, 1.77281052085584, -1.253566341056, 2.39356776939084, 6.30376871403132, -0.908295106229243, -2.78313372537744, -0.524404424085076, 0.370809924354784, 1.23603308118261, 9.40174755792011, -0.301561193523228, -2.42752078113804, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608844, 0.066068747264337, -0.0467176592151058, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.16499017783646e-14, 0.0, 1.05149212183459e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047608, 2.74642624930237, -0.792824967172098, 2.24244764232556, 2.14087209644419, -0.331662479035525, 6.49923072370878, 0.0957427107756281, -1.87616630392936, -2.155806113731, 11.0554159678513, 0.0, 9.44035859331467, -1.3031422550503e-14, -1.46249406456536, -2.04706526287665, 0.0, 0.422185670932533, 0.590936840285271, 0.522319308773342, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.40512108498651e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.00692910215529e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747034, 3.74574588632198, -1.05945692672796, 0.404586796856344, 5.01426536422405, -0.383824736789372, 1.80936716113936, -1.10800657553489, 0.313391585264007, 0.0, 2.82521847127455, -0.0509731451534321, 8.20651806648288, -0.631738055305799, 0.0483573714561724, -2.84282124887605, -0.227958835180071, 0.139595707137604, -0.0394836284566186, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02410615434263e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494799, 1.34011878852097, 1.89521416591738, 0.361873432227868, 1.96214168703486, 2.74642624930238, -0.566421515598882, 3.17129986868837, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251821, -0.998865569685885, 0.441440386136652, 0.0, 1.25094922078353e-14, 0.0, 0.0, 0.0, 0.0, 1.17641811167789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.5542870923281e-14, 0.0, 0.0, 0.0, -1.07569277121378e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 1.00508909139073, -0.193429485824688, 3.55615782880926, -8.46157280212812, 1.50756207226187e-14, 4.45964044034304, 0.0, -0.858258202906989, 2.67027285925186, 0.0, 0.0, 12.4373429638328, -1.98038392448788e-14, 0.0, 8.42792824422443, 0.0, 0.0, -1.62195554683797, -2.29379153190451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4764669953261e-14, 0.0, 0.0, 0.0, 0.0, -1.05314176681423e-14, 0.0, 0.0, 0.0, 0.0, -1.04283700926226e-14, 0.0, -1.17663587280494e-14, 0.0},
      {1.48323969741916, 0.335029697130244, 4.06201920231799, -0.820651806648294, 0.0, 1.69926454679664, -0.0991031208965162, 5.88642506110459, -1.20156148406979, 0.242752078113809, 0.0, 0.0, 0.0, 4.94413232473044, -0.214087209644422, 0.012485819621077, 5.13809303146606, -0.741619848709579, 0.151382517704895, -0.03058388709207, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04334156968119e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46666240087704e-14, 0.0, 0.0, 0.0},
      {1.32664991614218, 0.299659670905747, 2.07610309995256, 1.46802658041885, 0.0, 0.868496237346886, 0.709124208342323, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 11.0554159678513, 3.82970843102535, -0.223353131420173, 0.0, 5.80409338312194, -1.54743588659734, 0.341938252770105, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.04649844023076e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.25951288749407, 0.749149177264366, 2.75458800949269, 0.0, -8.46157280212811, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.05797799644199e-14, 0.0, 0.0, 12.4373429638328, 2.17608171552748, -2.05859474150797e-14, -2.42742488223711e-14, 6.28180682119206, -1.77676328054753, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.10786425434298e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16681049140544e-14, 0.0},
      {0.938083151964703, 7.62808987244125, 1.6321868014403e-14, 3.11415464992887, -1.07204497193666e-14, 0.0, -7.52139804633609, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 3.09654505822718e-14, 0.0, -1.19088867960207e-14, 14.2141062443803, 1.97456055389705e-14, 1.28734673838374e-14, 1.29546317995434e-14, 5.80288457473996, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.52957768188898e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.17822698518946e-14, 0.0, 0.0, 0.0, 0.0, -1.31528643214608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577692, 0.0, 2.68023757704197, -0.47380354147934, 0.0, 1.36855886629347e-14, 0.0, 5.09779364038994, -0.792824967172094, 0.140152977645347, 0.0, -1.26292377791434e-14, 0.0, 0.0, 0.0, 0.0, 7.91061171956872, -0.642261628933267, 0.0998865569685963, -0.0176576154454734, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205093, 1.39567290880551e-14, 1.49829835452879, 0.847565541382363, 0.0, 2.46042205023184e-14, 0.0, 1.62843044502545, 3.54562104171167, -0.814818121686408, 0.0, -1.58712657025661e-14, 1.14745183466215e-14, 1.27891939857454e-14, 0.0, 0.0, 11.0554159678513, 7.18070330817253, -1.11676565710082, 0.197418142283052, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.07471639882467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071065, 1.09648549793115e-14, 0.692034366650847, 1.59036212878711, 0.0, 2.00568804611028e-14, 0.0, -8.46157280212813, 3.07059789431494, 1.19418232635199, 1.03160102133141e-14, 0.0, 1.54928079801643e-14, 0.0, 0.0, 0.0, 0.0, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.26209984921976e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.42572253330643e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05960571077855e-14, 0.0, 0.0, 0.0, -1.05674262091421e-14, -1.3416584715789e-14, 0.0, 0.0, 0.0},
      {0.541602560309052, 0.0, 7.62808987244126, 1.79795802543454, 0.0, 1.41243372564957e-14, 1.08632362189182e-14, 0.0, -7.52139804633613, 3.19105893754051, 0.0, 1.45393175667359e-14, 0.0, -1.7494074335389e-14, 0.0, 0.0, 1.46514369285708e-14, -1.31143287856151e-14, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40026798037218e-14, 0.0, 1.12239490786269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04036565752576e-14, 0.0, 1.07716364475088e-14, 0.0, 0.0},
      {-5.36159180343549, -2.40874620958842e-14, 2.49838942213231e-14, 8.89943818451477, 0.0, 0.0, -1.63284004575804e-14, 0.0, -1.46094610136159e-14, -5.26497863243524, 0.0, 0.0, 0.0, -1.06941130250495e-14, -1.74798313988092e-14, -1.33892171591134e-14, 0.0, 1.71497580603867e-14, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.84427493731679e-14, 0.0, 0.0, 0.0, 0.0, 1.37667655053519e-14, 0.0, 0.0, 1.65498214377719e-14, 0.0, -1.18871137453869e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361252, 5.72988941019467, -2.20543546226845, -1.5594783708393, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750478, 0.615066564126896, 10.333118277332, -2.18326971917504, -1.8907670401189, 0.72478005108375, 0.627677936394715, 0.512496888990049, -0.278967971730972, -0.241593350361244, -0.197260144544825, -0.139483985865494, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.24532669073133, 0.219426862868126, 0.190029237516519, -0.0814411017927989, -0.0728431359084641, -0.0630840061880526, -0.0515078753637651, 0.0313466946954455, 0.0280373360835792, 0.0242810453028185, 0.0198253904711058, 0.0140186680417865, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412066, -1.4106012612951, 3.23209352764251, 5.28531208250978, -1.36277028773849, -2.89087233497829, -0.393397896234711, 0.556348640264183, 2.33666428910959, 9.02784807228866, -0.732969916135153, -3.93352933905727, -1.19859611757309, 0.309047252182627, 1.64018416088946, 0.65558822317622, 0.0892142571199899, -0.126168012376105, 1.1908743922773, 11.55277744383, -0.234843597212089, -2.44096975264613, -0.907332870306511, 0.0736662483163186, 0.810328731479531, 0.395334573318414, 0.120463442338387, -0.0310604175145487, -0.311895674167859, -0.164844710568315, -0.0658890955530732, -0.00896637020658606, 0.012680362351408, 0.0, 0.0, 0.0, 1.11259721423053e-14, 0.0, 0.0, -1.04560545935893e-14, 1.22737590889464e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482412, 3.15420030940281, -1.09264703862702, 1.54523626091314, 2.95048422176041, -0.761811750275047, 5.59815020469135, 0.263899331455875, -1.93925611658551, -2.28543525082518, 2.13307290077016, -0.257539376818854, 9.14446135479767, 0.0664963811607903, -2.36108976914457, -3.33842298506508, -0.023035022137994, 0.817905488277902, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280291, 0.555625639776452, 0.595437196138655, 0.0, -0.125015768949703, -0.192474367616162, -0.206265495285703, -0.166687691932941, 0.0, 0.0, 0.0, 0.0, 2.01332782911703e-14, 1.14629861867163e-14, 0.0, 1.06240528723087e-14, 0.0, 1.00868696648467e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331888, 1.82834820066014, 2.99085908983324, -1.24403337882028, 0.950146187582623, 5.74064456311311, -0.721110255092807, -0.169967317119746, -1.17961073303081, 0.490653381462658, 0.294392028877594, 4.97613351528118, -0.215472901842834, 5.78174466995659, -1.30185581821821, 0.163532434528629, -3.22682211353227, 0.0385449644663722, 0.267510569439605, -0.111269728052828, 0.0, 2.64638753839399, -0.0295875121532854, 10.7636648722209, -0.500120235663635, 0.0216558414515705, -3.57321383950878, -0.581087203147986, 0.130841432732468, -0.0164356280722715, 1.37533064807501, 0.324307824721457, -0.0038739146876535, -0.0268858238268318, 0.0111830284386748, 0.0, -1.0219695739434e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27676152301445e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637247, 1.66904592079256, 0.963624111659425, 1.36277028773852, 0.867360833110894, 2.28430636201793, 2.96226264871972, -1.00852844760553, 1.71026313764871, -2.01556443707464, 0.268741924943295, 2.04415543160771, 2.68821895760393, -0.527798662911739, 7.07977053255618, -1.76652739792484, 0.182834820066012, -3.12574097035511, -1.01990506872276, 0.904016610326409, 0.0, 13.0453753561106, 1.45851730441319, 0.0, 11.0940489487798, -0.632607619454899, 0.0, -2.86447112137561, -1.66605356688626, 0.315075748468087, 0.0, 0.992281903807267, 0.735567892896449, 0.18190906819317, -0.14700472648997, 0.0, 0.0, 0.0, 1.63127628833246e-14, 0.0, 0.0, 0.0, 1.04426771419413e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.5735915849389, 1.49284005458436, -0.430945803685661, 3.04724700110024, 0.775791113542718, -0.120185042515458, 5.29905651979673, 0.0346944333244409, -1.52970585407784, 0.901387818866002, -10.576283741361, 0.0, 4.51561078268942, -1.36757199011482e-14, -0.699555414372436, 6.36462844462229, 0.0, 0.201944253400492, -1.83730997289732, -2.74825341360597, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.0999354831996, 0.0, 0.0, 0.487107866980273, 0.894874292847874, 0.903959543974645, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.05690161928547e-14, 0.0, 1.69068701094327e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585592, 0.772618130456569, 3.86595114186569, -1.05140010313426, 0.160604018609913, 3.35889012281481, -0.304724700110027, 3.87851223317102, -1.52475169285616, 0.414677792940092, 0.0, 1.13551211138499, -0.0364215679542257, 7.3296991613516, -0.761724681099367, 0.0691050664139866, 0.564240504518063, -0.87956389936219, 0.345781182036203, -0.0940400840863472, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162678, 0.0036605053085024, 9.05975063854701, -0.73666248316321, 0.0765562109321079, -0.00694532049720516, -3.48709964663733, -0.0567083043941603, 0.0883994979795869, -0.0347523163751431, 0.00945138406569027, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21020486243213e-14, 0.0, 0.0, 0.0},
      {1.48690428533296, 0.705300630647562, 2.03602754481989, 1.15175110689981, 0.146610739697706, 1.817027050318, 1.25178444059441, 0.655663160391228, 3.61359041872286, -1.70346285967311, 0.0, 0.614267257013189, 0.454391937932185, 1.78428514239954, 5.63152770643899, -0.746494073225344, -0.618094504365248, 2.03210252328836, -2.15494277062554, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231194, 0.0, 9.68367982311022, -1.32524448062075, 0.133143804689784, 0.0, -3.3545250915713, -0.478206411017783, 0.384353057392906, -0.124241670058192, 0.0, -1.57908688885885e-14, 0.0, 0.0, 1.27210811114383e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321606, 0.630840061880565, 0.728431359084673, 2.57539376818858, 0.131132632078243, 0.711024300256735, 2.23926008187656, -0.205255035576797, 2.585674822114, 0.761811750275051, 1.13499156085505e-14, -10.576283741361, 0.763277533137602, 0.0, 4.1386256441266, 2.68954640486932, 0.0, -1.19471831485578, 3.10562068169859, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403078, -1.51261363209512, 0.763985254692598, 0.0, 0.0, 0.0, 1.57306717015361e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.16800101742086e-14, -1.15929160745736e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17556078118293e-14, 0.0, -1.24344978758018e-14},
      {1.15175110689982, 0.546323519313525, -0.105140010313421, 3.27118942773251, 9.65295620481434, 0.0, 2.69341127303543, 0.0, -0.518347241175133, 4.25171145123353, 0.0, 0.0, -9.91526600752597, 0.0, 0.0, 6.71889089790563, 0.0, 0.0, -1.29305115618717, 1.82865048192212, 0.0, 0.0, 1.19767218591939e-14, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059453, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22562692534596e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376111, 0.199489143482411, 3.68560354207935, -0.814411017927952, 0.0, 1.0278657191034, -0.0786795792469467, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907107, 0.0178428514239878, 6.99294104721298, -1.36261453366202, 0.329650402328151, -0.0728431359084655, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433805, 0.0234272339744324, -0.00179327404131587, 5.40219154319688, -0.702817019232746, 0.136947913395217, -0.0331311120155259, 0.00732101061700716, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27289844722241e-14, 0.0, 0.0, 0.0},
      {1.15175110689977, 0.182107839771169, 2.20794021658199, 0.892142571199755, 0.0, 0.615765106730374, 0.323209352764243, 2.13307290077017, 3.91870514328393, -1.31949665727938, 0.0, 0.0, 2.15210053409121e-14, 1.07496769977313, 1.90844613931508, -0.192743940909623, 1.11713920350154, 6.61105135360478, -2.33689608954553, 0.591817792331155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771038, 0.0343775825476229, 0.0, 6.06294257990891, -1.55575179137406, 0.416806037297043, -0.0962371838080742, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.36985186274036e-14, 0.0, 0.0, 0.0, -1.22926953652986e-14, 1.52236108034078e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, 0.162882203585602, 1.12848100903608, 1.99489143482413, 0.0, 0.314718316987784, 0.57817446699566, 0.476969600708474, 4.00571020990214, 0.590096844352085, 0.0, -1.0236880400916e-14, 1.84780862259548e-14, -10.576283741361, 1.83186607953019, 0.694437895660715, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525286, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780438, -0.338230690505756, -1.03537708592076e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.59178043363451, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199769, 0.14106012612951, 0.407205508963997, 2.53385243517952, 0.0, 9.65295620481432, 0.695435800330249, 0.0, 2.00755023262381, 3.29336152870135, 0.0, 1.53727126747605e-14, 0.0, 0.0, -9.91526600752596, 1.73481017017154, 3.09945749763524e-14, 0.0, 5.00796559370721, 1.41646657250367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 2.44096975264614, 1.1995682998667e-14, 0.0, 0.0, 7.04647271886991, -2.27776395106695, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.12989981303212e-14, 0.0, 0.0, 1.12710361396894e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {0.728431359084711, -7.94708263760856, 0.0, 2.5390822703312, 0.0, -1.51817455889641e-14, 10.2207771580387, -1.10417883136244e-14, -1.17393477360251e-14, 5.00713776237769, -1.6788670785722e-14, 1.16692395599207e-14, 0.0, -1.16983244350652e-14, -1.72078067004028e-14, -8.49879943502228, -1.30255896171053e-14, 2.38336582706624e-14, 0.0, 6.93924068068619, 0.0, 1.11347706836615e-14, 1.22585911744394e-14, 0.0, 0.0, 0.0, 0.0, 1.05392734863279e-14, 0.0, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394722, 0.0, 0.0, 0.0, -1.33619928008991e-14, -1.31653269318346e-14, -1.19219327188398e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.92380368648608e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.35447209004269e-14},
      {0.728431359084666, 0.0, 2.32737334062816, -0.470200420431691, 0.0, 1.17290559361541e-14, 0.0, 4.49691252107735, -0.917928424547676, 0.185449546754729, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.06540869306229, -1.01980390271856, 0.208166599946612, -0.0420560041253553, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59336650556232e-14, 0.0, 9.92395326897747, -0.710100291678875, 0.10249414863811, -0.02092153048203, 0.00422678745047192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811607997, -1.55732446924182e-14, 1.45686271816938, 0.515078753637719, 0.0, 1.75302325212418e-14, 0.0, 1.84729532019112, 2.58567482211399, -0.76181175027505, 0.0, 0.0, 0.0, -1.5771132441429e-14, 0.0, 0.0, 1.71994831963704, 5.72533841794523, -1.54195152727694, 0.341686161713627, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01415282846698e-14, 0.0, 0.0, 1.55537810005618e-14, 0.0, 13.0453753561106, 9.33451074824443, -1.3473205733131, 0.275020660380927, -0.0555625639776336, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15354195095738e-14, 0.0, 0.0, 2.18391286594405e-14, -1.6632527352539e-14, 0.0, 0.0, 0.0, 1.24344978758018e-14},
      {0.594761714133158, -1.70231042429006e-14, 0.814411017927961, 1.15175110689979, 0.0, 1.05122889803527e-14, 0.0, 0.590096844352062, 2.89087233497828, 0.340692571934632, 0.0, 1.96741032378448e-14, 0.0, -2.67239828517439e-14, 0.0, 0.0, -10.576283741361, 3.43474889911912, 3.47218947830366, -1.03874215316938, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4589662495124e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088318, -1.69115345252879, 0.341664592660044, 0.0, 0.0, 0.0, 0.0, -1.58107519128874e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.23983213164458e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637725, -2.12151679590624e-14, 0.376160336345367, 1.46292038553769, 0.0, 0.0, 0.0, 9.65295620481432, 1.85449546754729, 1.9014231651345, 0.0, 1.08417646420595e-14, -1.39425806410228e-14, -1.31312799465943e-14, -1.00034026957014e-14, -2.28947321498499e-14, 0.0, -9.91526600752594, 4.62616045379079, 0.817797356933117, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.18282399217016e-14, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372301, -1.31506763029893, 0.0, 0.0, 0.0, 0.0, -1.34440637760707e-14, 0.0, 0.0, -1.40321595102753e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.27033820993069e-14, 0.0, 0.0, -1.15463194561016e-14},
      {0.420560041253716, -2.13956334540513e-14, -7.94708263760856, 1.46593983227031, -1.44707708938326e-14, -1.61160863250687e-14, 1.32737570834153e-14, 0.0, 10.2207771580387, 2.89087233497829, 0.0, 1.54968957320001e-14, 0.0, 0.0, -1.03694448876387e-14, 0.0, -1.08416908996913e-14, 1.91284286439132e-14, -8.49879943502226, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, -1.04442202905116e-14, 0.0, 0.0, 0.0, -2.0768625719216e-14, 1.30100012769702e-14, 0.0, 0.0, -1.12942083813264e-14, 17.9373911146521, 3.62390025541881, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28244235280452e-14, 0.0, -1.2984409267287e-14, 0.0, -1.25455201782643e-14},
      {6.54237885546498, -1.1055459664935e-14, 2.348668208555e-14, -6.91050664139868, -1.48168770411177e-14, 0.0, 1.13942117771099e-14, 0.0, -2.11145049556143e-14, 12.2649325896465, 0.0, 0.0, 1.61580493801509e-14, 0.0, -4.74898554509598e-14, 0.0, 1.02925652887574e-14, 2.65830705666492e-14, 2.91812020982522e-14, -5.66586629001482, 0.0, 1.38739425433308e-14, 0.0, 0.0, 0.0, 0.0, 1.03603726671064e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.4998755596024, 0.0, 0.0, 0.0, -1.0855329829016e-14, -2.49407944254467e-14, 0.0, 0.0, 0.0, 1.14954920973716e-14, 0.0, 0.0, -1.74344999908208e-14, 0.0, -1.15224986201279e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.71039528115171e-14, 1.59872115546023e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 3;
    }// end loop over 'r'
    
    // Helper variable to hold values of a single dof.
    double *dof_values = new double[num_derivatives];
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      dof_values[r] = 0.0;
    }// end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 56; r++)
    {
      evaluate_basis_derivatives(r, n, dof_values, coordinates, c);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      }// end loop over 's'
    }// end loop over 'r'
    
    // Delete pointer.
    delete [] dof_values;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[3];
    const double * const * x = c.coordinates;
    switch (i)
    {
    case 0:
      {
        y[0] = x[0][0];
      y[1] = x[0][1];
      y[2] = x[0][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = x[1][0];
      y[1] = x[1][1];
      y[2] = x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = x[2][0];
      y[1] = x[2][1];
      y[2] = x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 3:
      {
        y[0] = x[3][0];
      y[1] = x[3][1];
      y[2] = x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 4:
      {
        y[0] = 0.8*x[2][0] + 0.2*x[3][0];
      y[1] = 0.8*x[2][1] + 0.2*x[3][1];
      y[2] = 0.8*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 5:
      {
        y[0] = 0.6*x[2][0] + 0.4*x[3][0];
      y[1] = 0.6*x[2][1] + 0.4*x[3][1];
      y[2] = 0.6*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 6:
      {
        y[0] = 0.4*x[2][0] + 0.6*x[3][0];
      y[1] = 0.4*x[2][1] + 0.6*x[3][1];
      y[2] = 0.4*x[2][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 7:
      {
        y[0] = 0.2*x[2][0] + 0.8*x[3][0];
      y[1] = 0.2*x[2][1] + 0.8*x[3][1];
      y[2] = 0.2*x[2][2] + 0.8*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 8:
      {
        y[0] = 0.8*x[1][0] + 0.2*x[3][0];
      y[1] = 0.8*x[1][1] + 0.2*x[3][1];
      y[2] = 0.8*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 9:
      {
        y[0] = 0.6*x[1][0] + 0.4*x[3][0];
      y[1] = 0.6*x[1][1] + 0.4*x[3][1];
      y[2] = 0.6*x[1][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 10:
      {
        y[0] = 0.4*x[1][0] + 0.6*x[3][0];
      y[1] = 0.4*x[1][1] + 0.6*x[3][1];
      y[2] = 0.4*x[1][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 11:
      {
        y[0] = 0.2*x[1][0] + 0.8*x[3][0];
      y[1] = 0.2*x[1][1] + 0.8*x[3][1];
      y[2] = 0.2*x[1][2] + 0.8*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 12:
      {
        y[0] = 0.8*x[1][0] + 0.2*x[2][0];
      y[1] = 0.8*x[1][1] + 0.2*x[2][1];
      y[2] = 0.8*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 13:
      {
        y[0] = 0.6*x[1][0] + 0.4*x[2][0];
      y[1] = 0.6*x[1][1] + 0.4*x[2][1];
      y[2] = 0.6*x[1][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 14:
      {
        y[0] = 0.4*x[1][0] + 0.6*x[2][0];
      y[1] = 0.4*x[1][1] + 0.6*x[2][1];
      y[2] = 0.4*x[1][2] + 0.6*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 15:
      {
        y[0] = 0.2*x[1][0] + 0.8*x[2][0];
      y[1] = 0.2*x[1][1] + 0.8*x[2][1];
      y[2] = 0.2*x[1][2] + 0.8*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 16:
      {
        y[0] = 0.8*x[0][0] + 0.2*x[3][0];
      y[1] = 0.8*x[0][1] + 0.2*x[3][1];
      y[2] = 0.8*x[0][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 17:
      {
        y[0] = 0.6*x[0][0] + 0.4*x[3][0];
      y[1] = 0.6*x[0][1] + 0.4*x[3][1];
      y[2] = 0.6*x[0][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 18:
      {
        y[0] = 0.4*x[0][0] + 0.6*x[3][0];
      y[1] = 0.4*x[0][1] + 0.6*x[3][1];
      y[2] = 0.4*x[0][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 19:
      {
        y[0] = 0.2*x[0][0] + 0.8*x[3][0];
      y[1] = 0.2*x[0][1] + 0.8*x[3][1];
      y[2] = 0.2*x[0][2] + 0.8*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 20:
      {
        y[0] = 0.8*x[0][0] + 0.2*x[2][0];
      y[1] = 0.8*x[0][1] + 0.2*x[2][1];
      y[2] = 0.8*x[0][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 21:
      {
        y[0] = 0.6*x[0][0] + 0.4*x[2][0];
      y[1] = 0.6*x[0][1] + 0.4*x[2][1];
      y[2] = 0.6*x[0][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 22:
      {
        y[0] = 0.4*x[0][0] + 0.6*x[2][0];
      y[1] = 0.4*x[0][1] + 0.6*x[2][1];
      y[2] = 0.4*x[0][2] + 0.6*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 23:
      {
        y[0] = 0.2*x[0][0] + 0.8*x[2][0];
      y[1] = 0.2*x[0][1] + 0.8*x[2][1];
      y[2] = 0.2*x[0][2] + 0.8*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 24:
      {
        y[0] = 0.8*x[0][0] + 0.2*x[1][0];
      y[1] = 0.8*x[0][1] + 0.2*x[1][1];
      y[2] = 0.8*x[0][2] + 0.2*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 25:
      {
        y[0] = 0.6*x[0][0] + 0.4*x[1][0];
      y[1] = 0.6*x[0][1] + 0.4*x[1][1];
      y[2] = 0.6*x[0][2] + 0.4*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 26:
      {
        y[0] = 0.4*x[0][0] + 0.6*x[1][0];
      y[1] = 0.4*x[0][1] + 0.6*x[1][1];
      y[2] = 0.4*x[0][2] + 0.6*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 27:
      {
        y[0] = 0.2*x[0][0] + 0.8*x[1][0];
      y[1] = 0.2*x[0][1] + 0.8*x[1][1];
      y[2] = 0.2*x[0][2] + 0.8*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 28:
      {
        y[0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 29:
      {
        y[0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
      y[1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
      y[2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 30:
      {
        y[0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 31:
      {
        y[0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
      y[1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
      y[2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 32:
      {
        y[0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
      y[1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
      y[2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 33:
      {
        y[0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
      y[1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
      y[2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 34:
      {
        y[0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 35:
      {
        y[0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
      y[1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
      y[2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 36:
      {
        y[0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 37:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
      y[1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
      y[2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 38:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
      y[1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
      y[2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 39:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 40:
      {
        y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
      y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
      y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 41:
      {
        y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
      y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
      y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 42:
      {
        y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 43:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
      y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
      y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 44:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
      y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
      y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 45:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 46:
      {
        y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
      y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
      y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 47:
      {
        y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
      y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
      y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 48:
      {
        y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
      y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
      y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 49:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
      y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
      y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 50:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
      y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
      y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 51:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 52:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 53:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 54:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 55:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[3];
    const double * const * x = c.coordinates;
    y[0] = x[0][0];
    y[1] = x[0][1];
    y[2] = x[0][2];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = x[1][0];
    y[1] = x[1][1];
    y[2] = x[1][2];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = x[2][0];
    y[1] = x[2][1];
    y[2] = x[2][2];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = x[3][0];
    y[1] = x[3][1];
    y[2] = x[3][2];
    f.evaluate(vals, y, c);
    values[3] = vals[0];
    y[0] = 0.8*x[2][0] + 0.2*x[3][0];
    y[1] = 0.8*x[2][1] + 0.2*x[3][1];
    y[2] = 0.8*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[4] = vals[0];
    y[0] = 0.6*x[2][0] + 0.4*x[3][0];
    y[1] = 0.6*x[2][1] + 0.4*x[3][1];
    y[2] = 0.6*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[5] = vals[0];
    y[0] = 0.4*x[2][0] + 0.6*x[3][0];
    y[1] = 0.4*x[2][1] + 0.6*x[3][1];
    y[2] = 0.4*x[2][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[6] = vals[0];
    y[0] = 0.2*x[2][0] + 0.8*x[3][0];
    y[1] = 0.2*x[2][1] + 0.8*x[3][1];
    y[2] = 0.2*x[2][2] + 0.8*x[3][2];
    f.evaluate(vals, y, c);
    values[7] = vals[0];
    y[0] = 0.8*x[1][0] + 0.2*x[3][0];
    y[1] = 0.8*x[1][1] + 0.2*x[3][1];
    y[2] = 0.8*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[8] = vals[0];
    y[0] = 0.6*x[1][0] + 0.4*x[3][0];
    y[1] = 0.6*x[1][1] + 0.4*x[3][1];
    y[2] = 0.6*x[1][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[9] = vals[0];
    y[0] = 0.4*x[1][0] + 0.6*x[3][0];
    y[1] = 0.4*x[1][1] + 0.6*x[3][1];
    y[2] = 0.4*x[1][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[10] = vals[0];
    y[0] = 0.2*x[1][0] + 0.8*x[3][0];
    y[1] = 0.2*x[1][1] + 0.8*x[3][1];
    y[2] = 0.2*x[1][2] + 0.8*x[3][2];
    f.evaluate(vals, y, c);
    values[11] = vals[0];
    y[0] = 0.8*x[1][0] + 0.2*x[2][0];
    y[1] = 0.8*x[1][1] + 0.2*x[2][1];
    y[2] = 0.8*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[12] = vals[0];
    y[0] = 0.6*x[1][0] + 0.4*x[2][0];
    y[1] = 0.6*x[1][1] + 0.4*x[2][1];
    y[2] = 0.6*x[1][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[13] = vals[0];
    y[0] = 0.4*x[1][0] + 0.6*x[2][0];
    y[1] = 0.4*x[1][1] + 0.6*x[2][1];
    y[2] = 0.4*x[1][2] + 0.6*x[2][2];
    f.evaluate(vals, y, c);
    values[14] = vals[0];
    y[0] = 0.2*x[1][0] + 0.8*x[2][0];
    y[1] = 0.2*x[1][1] + 0.8*x[2][1];
    y[2] = 0.2*x[1][2] + 0.8*x[2][2];
    f.evaluate(vals, y, c);
    values[15] = vals[0];
    y[0] = 0.8*x[0][0] + 0.2*x[3][0];
    y[1] = 0.8*x[0][1] + 0.2*x[3][1];
    y[2] = 0.8*x[0][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[16] = vals[0];
    y[0] = 0.6*x[0][0] + 0.4*x[3][0];
    y[1] = 0.6*x[0][1] + 0.4*x[3][1];
    y[2] = 0.6*x[0][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[17] = vals[0];
    y[0] = 0.4*x[0][0] + 0.6*x[3][0];
    y[1] = 0.4*x[0][1] + 0.6*x[3][1];
    y[2] = 0.4*x[0][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[18] = vals[0];
    y[0] = 0.2*x[0][0] + 0.8*x[3][0];
    y[1] = 0.2*x[0][1] + 0.8*x[3][1];
    y[2] = 0.2*x[0][2] + 0.8*x[3][2];
    f.evaluate(vals, y, c);
    values[19] = vals[0];
    y[0] = 0.8*x[0][0] + 0.2*x[2][0];
    y[1] = 0.8*x[0][1] + 0.2*x[2][1];
    y[2] = 0.8*x[0][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[20] = vals[0];
    y[0] = 0.6*x[0][0] + 0.4*x[2][0];
    y[1] = 0.6*x[0][1] + 0.4*x[2][1];
    y[2] = 0.6*x[0][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[21] = vals[0];
    y[0] = 0.4*x[0][0] + 0.6*x[2][0];
    y[1] = 0.4*x[0][1] + 0.6*x[2][1];
    y[2] = 0.4*x[0][2] + 0.6*x[2][2];
    f.evaluate(vals, y, c);
    values[22] = vals[0];
    y[0] = 0.2*x[0][0] + 0.8*x[2][0];
    y[1] = 0.2*x[0][1] + 0.8*x[2][1];
    y[2] = 0.2*x[0][2] + 0.8*x[2][2];
    f.evaluate(vals, y, c);
    values[23] = vals[0];
    y[0] = 0.8*x[0][0] + 0.2*x[1][0];
    y[1] = 0.8*x[0][1] + 0.2*x[1][1];
    y[2] = 0.8*x[0][2] + 0.2*x[1][2];
    f.evaluate(vals, y, c);
    values[24] = vals[0];
    y[0] = 0.6*x[0][0] + 0.4*x[1][0];
    y[1] = 0.6*x[0][1] + 0.4*x[1][1];
    y[2] = 0.6*x[0][2] + 0.4*x[1][2];
    f.evaluate(vals, y, c);
    values[25] = vals[0];
    y[0] = 0.4*x[0][0] + 0.6*x[1][0];
    y[1] = 0.4*x[0][1] + 0.6*x[1][1];
    y[2] = 0.4*x[0][2] + 0.6*x[1][2];
    f.evaluate(vals, y, c);
    values[26] = vals[0];
    y[0] = 0.2*x[0][0] + 0.8*x[1][0];
    y[1] = 0.2*x[0][1] + 0.8*x[1][1];
    y[2] = 0.2*x[0][2] + 0.8*x[1][2];
    f.evaluate(vals, y, c);
    values[27] = vals[0];
    y[0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[28] = vals[0];
    y[0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    y[1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    y[2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[29] = vals[0];
    y[0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[30] = vals[0];
    y[0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    y[1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    y[2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[31] = vals[0];
    y[0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    y[1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    y[2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[32] = vals[0];
    y[0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    y[1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    y[2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[33] = vals[0];
    y[0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[34] = vals[0];
    y[0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    y[1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    y[2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[35] = vals[0];
    y[0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[36] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    y[1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    y[2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[37] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    y[1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    y[2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[38] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[39] = vals[0];
    y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[40] = vals[0];
    y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[41] = vals[0];
    y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[42] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[43] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[44] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[45] = vals[0];
    y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[46] = vals[0];
    y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[47] = vals[0];
    y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[48] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[49] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[50] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    f.evaluate(vals, y, c);
    values[51] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[52] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[53] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[54] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[55] = vals[0];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Map coordinate xhat from reference cell to coordinate x in cell
  virtual void map_from_reference_cell(double* x,
                                       const double* xhat,
                                       const ufc::cell& c) const
  {
    throw std::runtime_error("map_from_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Map from coordinate x in cell to coordinate xhat in reference cell
  virtual void map_to_reference_cell(double* xhat,
                                     const double* x,
                                     const ufc::cell& c) const
  {
    throw std::runtime_error("map_to_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 0;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::finite_element* create() const
  {
    return new poisson3d_5_finite_element_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson3d_5_dofmap_0: public ufc::dofmap
{
private:

  unsigned int _global_dimension;
public:

  /// Constructor
  poisson3d_5_dofmap_0() : ufc::dofmap()
  {
    _global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson3d_5_dofmap_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dofmap
  virtual const char* signature() const
  {
    return "FFC dofmap for FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return true;
        break;
      }
    case 2:
      {
        return true;
        break;
      }
    case 3:
      {
        return true;
        break;
      }
    }
    
    return false;
  }

  /// Initialize dofmap for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    _global_dimension = m.num_entities[0] + 4.0*m.num_entities[1] + 6.0*m.num_entities[2] + 4.0*m.num_entities[3];
    return false;
  }

  /// Initialize dofmap for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dofmap for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the topological dimension of the associated cell shape
  virtual unsigned int topological_dimension() const
  {
    return 3;
  }

  /// Return the geometric dimension of the associated cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return _global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 56;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 4;
        break;
      }
    case 2:
      {
        return 6;
        break;
      }
    case 3:
      {
        return 4;
        break;
      }
    }
    
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    unsigned int offset = 0;
    dofs[0] = offset + c.entity_indices[0][0];
    dofs[1] = offset + c.entity_indices[0][1];
    dofs[2] = offset + c.entity_indices[0][2];
    dofs[3] = offset + c.entity_indices[0][3];
    offset += m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset += 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset += 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
    offset += 4*m.num_entities[3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
        break;
      }
    case 3:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
        break;
      }
    }
    
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    if (d > 3)
    {
    throw std::runtime_error("d is larger than dimension (3)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 3)
      {
      throw std::runtime_error("i is larger than number of entities (3)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      case 3:
        {
          dofs[0] = 3;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 5)
      {
      throw std::runtime_error("i is larger than number of entities (5)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 4;
        dofs[1] = 5;
        dofs[2] = 6;
        dofs[3] = 7;
          break;
        }
      case 1:
        {
          dofs[0] = 8;
        dofs[1] = 9;
        dofs[2] = 10;
        dofs[3] = 11;
          break;
        }
      case 2:
        {
          dofs[0] = 12;
        dofs[1] = 13;
        dofs[2] = 14;
        dofs[3] = 15;
          break;
        }
      case 3:
        {
          dofs[0] = 16;
        dofs[1] = 17;
        dofs[2] = 18;
        dofs[3] = 19;
          break;
        }
      case 4:
        {
          dofs[0] = 20;
        dofs[1] = 21;
        dofs[2] = 22;
        dofs[3] = 23;
          break;
        }
      case 5:
        {
          dofs[0] = 24;
        dofs[1] = 25;
        dofs[2] = 26;
        dofs[3] = 27;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 3)
      {
      throw std::runtime_error("i is larger than number of entities (3)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 28;
        dofs[1] = 29;
        dofs[2] = 30;
        dofs[3] = 31;
        dofs[4] = 32;
        dofs[5] = 33;
          break;
        }
      case 1:
        {
          dofs[0] = 34;
        dofs[1] = 35;
        dofs[2] = 36;
        dofs[3] = 37;
        dofs[4] = 38;
        dofs[5] = 39;
          break;
        }
      case 2:
        {
          dofs[0] = 40;
        dofs[1] = 41;
        dofs[2] = 42;
        dofs[3] = 43;
        dofs[4] = 44;
        dofs[5] = 45;
          break;
        }
      case 3:
        {
          dofs[0] = 46;
        dofs[1] = 47;
        dofs[2] = 48;
        dofs[3] = 49;
        dofs[4] = 50;
        dofs[5] = 51;
          break;
        }
      }
      
        break;
      }
    case 3:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 52;
      dofs[1] = 53;
      dofs[2] = 54;
      dofs[3] = 55;
        break;
      }
    }
    
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dofmaps (for a mixed element)
  virtual unsigned int num_sub_dofmaps() const
  {
    return 0;
  }

  /// Create a new dofmap for sub dofmap i (for a mixed element)
  virtual ufc::dofmap* create_sub_dofmap(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::dofmap* create() const
  {
    return new poisson3d_5_dofmap_0();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_cell_integral_0_0: public ufc::cell_integral
{
public:

  /// Constructor
  poisson3d_5_cell_integral_0_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_cell_integral_0_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      32
    // Number of operations (multiply-add pairs) for geometry tensor:    27
    // Number of operations (multiply-add pairs) for tensor contraction: 18292
    // Total number of operations (multiply-add pairs):                  18351
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    const double K_00 = d_00 / detJ;
    const double K_01 = d_10 / detJ;
    const double K_02 = d_20 / detJ;
    const double K_10 = d_01 / detJ;
    const double K_11 = d_11 / detJ;
    const double K_12 = d_21 / detJ;
    const double K_20 = d_02 / detJ;
    const double K_21 = d_12 / detJ;
    const double K_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0_0 = det*(K_00*K_00 + K_01*K_01 + K_02*K_02);
    const double G0_0_1 = det*(K_00*K_10 + K_01*K_11 + K_02*K_12);
    const double G0_0_2 = det*(K_00*K_20 + K_01*K_21 + K_02*K_22);
    const double G0_1_0 = det*(K_10*K_00 + K_11*K_01 + K_12*K_02);
    const double G0_1_1 = det*(K_10*K_10 + K_11*K_11 + K_12*K_12);
    const double G0_1_2 = det*(K_10*K_20 + K_11*K_21 + K_12*K_22);
    const double G0_2_0 = det*(K_20*K_00 + K_21*K_01 + K_22*K_02);
    const double G0_2_1 = det*(K_20*K_10 + K_21*K_11 + K_22*K_12);
    const double G0_2_2 = det*(K_20*K_20 + K_21*K_21 + K_22*K_22);
    
    // Compute element tensor
    A[0] = 0.0262652567340065*G0_0_0 + 0.0262652567340065*G0_0_1 + 0.0262652567340065*G0_0_2 + 0.0262652567340065*G0_1_0 + 0.0262652567340065*G0_1_1 + 0.0262652567340065*G0_1_2 + 0.0262652567340065*G0_2_0 + 0.0262652567340065*G0_2_1 + 0.0262652567340065*G0_2_2;
    A[1] = -0.00277890512265507*G0_0_0 - 0.00277890512265507*G0_1_0 - 0.00277890512265507*G0_2_0;
    A[2] = -0.0027789051226551*G0_0_1 - 0.00277890512265513*G0_1_1 - 0.00277890512265511*G0_2_1;
    A[3] = -0.00277890512265511*G0_0_2 - 0.00277890512265512*G0_1_2 - 0.00277890512265511*G0_2_2;
    A[4] = -0.00480034930989786*G0_0_1 - 0.00480034930989792*G0_0_2 - 0.00480034930989785*G0_1_1 - 0.00480034930989796*G0_1_2 - 0.00480034930989788*G0_2_1 - 0.00480034930989796*G0_2_2;
    A[5] = -0.00336507226698194*G0_0_1 - 0.0033650722669818*G0_0_2 - 0.00336507226698198*G0_1_1 - 0.00336507226698183*G0_1_2 - 0.00336507226698196*G0_2_1 - 0.00336507226698182*G0_2_2;
    A[6] = -0.00336507226698194*G0_0_1 - 0.00336507226698208*G0_0_2 - 0.00336507226698193*G0_1_1 - 0.00336507226698208*G0_1_2 - 0.00336507226698193*G0_2_1 - 0.00336507226698206*G0_2_2;
    A[7] = -0.00480034930989791*G0_0_1 - 0.00480034930989783*G0_0_2 - 0.00480034930989794*G0_1_1 - 0.00480034930989784*G0_1_2 - 0.00480034930989793*G0_2_1 - 0.00480034930989785*G0_2_2;
    A[8] = -0.00480034930989781*G0_0_0 - 0.00480034930989777*G0_0_2 - 0.00480034930989778*G0_1_0 - 0.00480034930989774*G0_1_2 - 0.00480034930989779*G0_2_0 - 0.00480034930989775*G0_2_2;
    A[9] = -0.00336507226698201*G0_0_0 - 0.00336507226698211*G0_0_2 - 0.00336507226698206*G0_1_0 - 0.00336507226698215*G0_1_2 - 0.00336507226698203*G0_2_0 - 0.00336507226698211*G0_2_2;
    A[10] = -0.00336507226698197*G0_0_0 - 0.00336507226698183*G0_0_2 - 0.00336507226698195*G0_1_0 - 0.00336507226698183*G0_1_2 - 0.00336507226698195*G0_2_0 - 0.00336507226698183*G0_2_2;
    A[11] = -0.00480034930989792*G0_0_0 - 0.00480034930989801*G0_0_2 - 0.00480034930989795*G0_1_0 - 0.00480034930989802*G0_1_2 - 0.00480034930989795*G0_2_0 - 0.00480034930989802*G0_2_2;
    A[12] = -0.00480034930989774*G0_0_0 - 0.00480034930989779*G0_0_1 - 0.00480034930989772*G0_1_0 - 0.00480034930989776*G0_1_1 - 0.00480034930989772*G0_2_0 - 0.00480034930989778*G0_2_1;
    A[13] = -0.00336507226698197*G0_0_0 - 0.00336507226698189*G0_0_1 - 0.00336507226698203*G0_1_0 - 0.00336507226698192*G0_1_1 - 0.00336507226698203*G0_2_0 - 0.00336507226698192*G0_2_1;
    A[14] = -0.0033650722669819*G0_0_0 - 0.00336507226698193*G0_0_1 - 0.00336507226698186*G0_1_0 - 0.00336507226698193*G0_1_1 - 0.00336507226698184*G0_2_0 - 0.00336507226698189*G0_2_1;
    A[15] = -0.00480034930989794*G0_0_0 - 0.00480034930989792*G0_0_1 - 0.00480034930989797*G0_1_0 - 0.00480034930989792*G0_1_1 - 0.00480034930989797*G0_2_0 - 0.00480034930989795*G0_2_1;
    A[16] = -0.0178705039816151*G0_0_0 - 0.017870503981615*G0_0_1 - 0.0551276864210888*G0_0_2 - 0.017870503981615*G0_1_0 - 0.017870503981615*G0_1_1 - 0.0551276864210887*G0_1_2 - 0.0178705039816151*G0_2_0 - 0.017870503981615*G0_2_1 - 0.0551276864210887*G0_2_2;
    A[17] = 0.0214691349673989*G0_0_0 + 0.0214691349673989*G0_0_1 + 0.0551276864210889*G0_0_2 + 0.0214691349673989*G0_1_0 + 0.0214691349673989*G0_1_1 + 0.0551276864210889*G0_1_2 + 0.0214691349673989*G0_2_0 + 0.0214691349673989*G0_2_1 + 0.0551276864210889*G0_2_2;
    A[18] = -0.0155544887532734*G0_0_0 - 0.0155544887532734*G0_0_1 - 0.038817720024317*G0_0_2 - 0.0155544887532735*G0_1_0 - 0.0155544887532734*G0_1_1 - 0.038817720024317*G0_1_2 - 0.0155544887532734*G0_2_0 - 0.0155544887532734*G0_2_1 - 0.038817720024317*G0_2_2;
    A[19] = 0.00480034930989789*G0_0_0 + 0.0048003493098979*G0_0_1 + 0.0153313684129656*G0_0_2 + 0.00480034930989792*G0_1_0 + 0.00480034930989793*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.00480034930989792*G0_2_0 + 0.00480034930989793*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[20] = -0.0178705039816149*G0_0_0 - 0.0551276864210887*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0551276864210887*G0_1_1 - 0.0178705039816149*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0551276864210887*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[21] = 0.0214691349673986*G0_0_0 + 0.0551276864210886*G0_0_1 + 0.0214691349673986*G0_0_2 + 0.0214691349673986*G0_1_0 + 0.0551276864210886*G0_1_1 + 0.0214691349673985*G0_1_2 + 0.0214691349673985*G0_2_0 + 0.0551276864210886*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[22] = -0.0155544887532734*G0_0_0 - 0.0388177200243168*G0_0_1 - 0.0155544887532734*G0_0_2 - 0.0155544887532734*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0155544887532734*G0_1_2 - 0.0155544887532734*G0_2_0 - 0.0388177200243168*G0_2_1 - 0.0155544887532734*G0_2_2;
    A[23] = 0.00480034930989792*G0_0_0 + 0.0153313684129655*G0_0_1 + 0.00480034930989791*G0_0_2 + 0.00480034930989795*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.00480034930989795*G0_1_2 + 0.00480034930989796*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.00480034930989795*G0_2_2;
    A[24] = -0.0551276864210885*G0_0_0 - 0.0178705039816148*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0551276864210885*G0_1_0 - 0.0178705039816148*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0551276864210886*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[25] = 0.0551276864210884*G0_0_0 + 0.0214691349673985*G0_0_1 + 0.0214691349673985*G0_0_2 + 0.0551276864210884*G0_1_0 + 0.0214691349673984*G0_1_1 + 0.0214691349673984*G0_1_2 + 0.0551276864210884*G0_2_0 + 0.0214691349673985*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[26] = -0.0388177200243167*G0_0_0 - 0.0155544887532732*G0_0_1 - 0.0155544887532732*G0_0_2 - 0.0388177200243167*G0_1_0 - 0.0155544887532732*G0_1_1 - 0.0155544887532732*G0_1_2 - 0.0388177200243167*G0_2_0 - 0.0155544887532732*G0_2_1 - 0.0155544887532732*G0_2_2;
    A[27] = 0.0153313684129655*G0_0_0 + 0.00480034930989776*G0_0_1 + 0.00480034930989777*G0_0_2 + 0.0153313684129654*G0_1_0 + 0.00480034930989774*G0_1_1 + 0.00480034930989774*G0_1_2 + 0.0153313684129654*G0_2_0 + 0.00480034930989775*G0_2_1 + 0.00480034930989775*G0_2_2;
    A[28] = -0.0137421252872641*G0_0_0 - 0.0137421252872639*G0_0_1 - 0.0137421252872637*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.013742125287264*G0_1_1 - 0.0137421252872638*G0_1_2 - 0.0137421252872641*G0_2_0 - 0.0137421252872639*G0_2_1 - 0.0137421252872638*G0_2_2;
    A[29] = -0.00946499969937451*G0_0_0 - 0.00946499969937474*G0_0_1 - 0.00946499969937458*G0_0_2 - 0.00946499969937443*G0_1_0 - 0.00946499969937466*G0_1_1 - 0.00946499969937456*G0_1_2 - 0.00946499969937444*G0_2_0 - 0.00946499969937473*G0_2_1 - 0.00946499969937454*G0_2_2;
    A[30] = -0.0137421252872638*G0_0_0 - 0.0137421252872637*G0_0_1 - 0.0137421252872639*G0_0_2 - 0.013742125287264*G0_1_0 - 0.0137421252872638*G0_1_1 - 0.013742125287264*G0_1_2 - 0.013742125287264*G0_2_0 - 0.0137421252872637*G0_2_1 - 0.0137421252872639*G0_2_2;
    A[31] = -0.00946499969937482*G0_0_0 - 0.00946499969937481*G0_0_1 - 0.00946499969937499*G0_0_2 - 0.00946499969937479*G0_1_0 - 0.00946499969937485*G0_1_1 - 0.00946499969937498*G0_1_2 - 0.00946499969937479*G0_2_0 - 0.00946499969937479*G0_2_1 - 0.00946499969937497*G0_2_2;
    A[32] = -0.00946499969937458*G0_0_0 - 0.00946499969937467*G0_0_1 - 0.00946499969937445*G0_0_2 - 0.00946499969937462*G0_1_0 - 0.00946499969937464*G0_1_1 - 0.00946499969937448*G0_1_2 - 0.00946499969937461*G0_2_0 - 0.00946499969937468*G0_2_1 - 0.00946499969937452*G0_2_2;
    A[33] = -0.013742125287264*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.013742125287264*G0_0_2 - 0.013742125287264*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.013742125287264*G0_1_2 - 0.013742125287264*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.013742125287264*G0_2_2;
    A[34] = 0.0479930547204848*G0_0_0 + 0.0335985307773499*G0_0_1 + 0.0335985307773499*G0_0_2 + 0.0479930547204848*G0_1_0 + 0.0335985307773498*G0_1_1 + 0.0335985307773498*G0_1_2 + 0.0479930547204848*G0_2_0 + 0.0335985307773499*G0_2_1 + 0.0335985307773499*G0_2_2;
    A[35] = -0.0345719095719093*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0298472385060923*G0_0_2 - 0.0345719095719093*G0_1_0 - 0.0298472385060922*G0_1_1 - 0.0298472385060923*G0_1_2 - 0.0345719095719093*G0_2_0 - 0.0298472385060924*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[36] = 0.0137421252872639*G0_0_0 + 0.0189195610202554*G0_0_1 + 0.0189195610202552*G0_0_2 + 0.013742125287264*G0_1_0 + 0.0189195610202553*G0_1_1 + 0.0189195610202552*G0_1_2 + 0.013742125287264*G0_2_0 + 0.0189195610202554*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[37] = -0.0345719095719094*G0_0_0 - 0.0298472385060925*G0_0_1 - 0.0298472385060925*G0_0_2 - 0.0345719095719094*G0_1_0 - 0.0298472385060926*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0345719095719094*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[38] = 0.00946499969937457*G0_0_0 + 0.0117431758056757*G0_0_1 + 0.0117431758056758*G0_0_2 + 0.00946499969937461*G0_1_0 + 0.0117431758056758*G0_1_1 + 0.0117431758056758*G0_1_2 + 0.00946499969937459*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[39] = 0.013742125287264*G0_0_0 + 0.0189195610202554*G0_0_1 + 0.0189195610202554*G0_0_2 + 0.013742125287264*G0_1_0 + 0.0189195610202554*G0_1_1 + 0.0189195610202554*G0_1_2 + 0.013742125287264*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[40] = 0.03359853077735*G0_0_0 + 0.0479930547204848*G0_0_1 + 0.0335985307773499*G0_0_2 + 0.03359853077735*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.03359853077735*G0_1_2 + 0.03359853077735*G0_2_0 + 0.0479930547204849*G0_2_1 + 0.03359853077735*G0_2_2;
    A[41] = -0.0298472385060921*G0_0_0 - 0.034571909571909*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0298472385060921*G0_1_0 - 0.034571909571909*G0_1_1 - 0.0298472385060921*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0345719095719091*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[42] = 0.018919561020255*G0_0_0 + 0.0137421252872639*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.018919561020255*G0_1_0 + 0.013742125287264*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.018919561020255*G0_2_0 + 0.0137421252872639*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[43] = -0.0298472385060929*G0_0_0 - 0.0345719095719097*G0_0_1 - 0.0298472385060927*G0_0_2 - 0.0298472385060929*G0_1_0 - 0.0345719095719097*G0_1_1 - 0.0298472385060928*G0_1_2 - 0.0298472385060928*G0_2_0 - 0.0345719095719097*G0_2_1 - 0.0298472385060927*G0_2_2;
    A[44] = 0.011743175805676*G0_0_0 + 0.00946499969937476*G0_0_1 + 0.0117431758056756*G0_0_2 + 0.011743175805676*G0_1_0 + 0.0094649996993748*G0_1_1 + 0.0117431758056756*G0_1_2 + 0.011743175805676*G0_2_0 + 0.00946499969937475*G0_2_1 + 0.0117431758056756*G0_2_2;
    A[45] = 0.0189195610202555*G0_0_0 + 0.0137421252872641*G0_0_1 + 0.0189195610202556*G0_0_2 + 0.0189195610202555*G0_1_0 + 0.0137421252872641*G0_1_1 + 0.0189195610202557*G0_1_2 + 0.0189195610202555*G0_2_0 + 0.0137421252872641*G0_2_1 + 0.0189195610202556*G0_2_2;
    A[46] = 0.0335985307773498*G0_0_0 + 0.0335985307773497*G0_0_1 + 0.0479930547204847*G0_0_2 + 0.0335985307773498*G0_1_0 + 0.0335985307773497*G0_1_1 + 0.0479930547204847*G0_1_2 + 0.0335985307773499*G0_2_0 + 0.0335985307773498*G0_2_1 + 0.0479930547204847*G0_2_2;
    A[47] = -0.0298472385060921*G0_0_0 - 0.0298472385060919*G0_0_1 - 0.0345719095719088*G0_0_2 - 0.0298472385060921*G0_1_0 - 0.0298472385060919*G0_1_1 - 0.0345719095719088*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060919*G0_2_1 - 0.0345719095719088*G0_2_2;
    A[48] = 0.0189195610202549*G0_0_0 + 0.0189195610202551*G0_0_1 + 0.0137421252872638*G0_0_2 + 0.0189195610202549*G0_1_0 + 0.0189195610202551*G0_1_1 + 0.0137421252872638*G0_1_2 + 0.0189195610202549*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.0137421252872638*G0_2_2;
    A[49] = -0.0298472385060923*G0_0_0 - 0.0298472385060925*G0_0_1 - 0.0345719095719093*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0298472385060925*G0_1_1 - 0.0345719095719094*G0_1_2 - 0.0298472385060923*G0_2_0 - 0.0298472385060926*G0_2_1 - 0.0345719095719094*G0_2_2;
    A[50] = 0.0117431758056757*G0_0_0 + 0.0117431758056754*G0_0_1 + 0.00946499969937461*G0_0_2 + 0.0117431758056757*G0_1_0 + 0.0117431758056754*G0_1_1 + 0.00946499969937459*G0_1_2 + 0.0117431758056757*G0_2_0 + 0.0117431758056754*G0_2_1 + 0.00946499969937458*G0_2_2;
    A[51] = 0.0189195610202553*G0_0_0 + 0.0189195610202555*G0_0_1 + 0.0137421252872639*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202555*G0_1_1 + 0.013742125287264*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202556*G0_2_1 + 0.013742125287264*G0_2_2;
    A[52] = -0.0782878387045046*G0_0_0 - 0.0782878387045047*G0_0_1 - 0.0782878387045047*G0_0_2 - 0.0782878387045046*G0_1_0 - 0.0782878387045049*G0_1_1 - 0.0782878387045048*G0_1_2 - 0.0782878387045046*G0_2_0 - 0.0782878387045048*G0_2_1 - 0.0782878387045048*G0_2_2;
    A[53] = 0.0440369092712839*G0_0_0 + 0.0440369092712838*G0_0_1 + 0.0440369092712837*G0_0_2 + 0.044036909271284*G0_1_0 + 0.0440369092712837*G0_1_1 + 0.0440369092712837*G0_1_2 + 0.044036909271284*G0_2_0 + 0.0440369092712838*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[54] = 0.0440369092712838*G0_0_0 + 0.0440369092712835*G0_0_1 + 0.0440369092712838*G0_0_2 + 0.0440369092712838*G0_1_0 + 0.0440369092712838*G0_1_1 + 0.0440369092712838*G0_1_2 + 0.0440369092712838*G0_2_0 + 0.0440369092712836*G0_2_1 + 0.0440369092712839*G0_2_2;
    A[55] = 0.0440369092712843*G0_0_0 + 0.0440369092712844*G0_0_1 + 0.0440369092712841*G0_0_2 + 0.0440369092712842*G0_1_0 + 0.0440369092712843*G0_1_1 + 0.0440369092712841*G0_1_2 + 0.0440369092712842*G0_2_0 + 0.0440369092712843*G0_2_1 + 0.0440369092712841*G0_2_2;
    A[56] = -0.00277890512265507*G0_0_0 - 0.00277890512265507*G0_0_1 - 0.00277890512265507*G0_0_2;
    A[57] = 0.0262652567340064*G0_0_0;
    A[58] = 0.00277890512265508*G0_0_1;
    A[59] = 0.00277890512265508*G0_0_2;
    A[60] = 0.00480034930989791*G0_0_1 + 0.00480034930989787*G0_0_2;
    A[61] = 0.00336507226698195*G0_0_1 + 0.003365072266982*G0_0_2;
    A[62] = 0.0033650722669819*G0_0_1 + 0.00336507226698188*G0_0_2;
    A[63] = 0.00480034930989788*G0_0_1 + 0.00480034930989789*G0_0_2;
    A[64] = -0.017870503981615*G0_0_0 + 0.0372571824394735*G0_0_2;
    A[65] = 0.0214691349673987*G0_0_0 - 0.0336585514536898*G0_0_2;
    A[66] = -0.0155544887532734*G0_0_0 + 0.0232632312710434*G0_0_2;
    A[67] = 0.00480034930989789*G0_0_0 - 0.0105310191030676*G0_0_2;
    A[68] = -0.0178705039816151*G0_0_0 + 0.0372571824394735*G0_0_1;
    A[69] = 0.0214691349673987*G0_0_0 - 0.0336585514536899*G0_0_1;
    A[70] = -0.0155544887532733*G0_0_0 + 0.0232632312710435*G0_0_1;
    A[71] = 0.00480034930989789*G0_0_0 - 0.0105310191030676*G0_0_1;
    A[72] = -0.00480034930989803*G0_0_0 - 0.00480034930989803*G0_0_1;
    A[73] = -0.00336507226698176*G0_0_0 - 0.00336507226698175*G0_0_1;
    A[74] = -0.00336507226698214*G0_0_0 - 0.00336507226698212*G0_0_1;
    A[75] = -0.00480034930989786*G0_0_0 - 0.00480034930989788*G0_0_1;
    A[76] = -0.00480034930989791*G0_0_0 - 0.00480034930989794*G0_0_2;
    A[77] = -0.00336507226698196*G0_0_0 - 0.00336507226698194*G0_0_2;
    A[78] = -0.00336507226698193*G0_0_0 - 0.00336507226698194*G0_0_2;
    A[79] = -0.00480034930989787*G0_0_0 - 0.00480034930989788*G0_0_2;
    A[80] = 0.0153313684129654*G0_0_0 + 0.0105310191030674*G0_0_1 + 0.0105310191030674*G0_0_2;
    A[81] = -0.0388177200243167*G0_0_0 - 0.0232632312710432*G0_0_1 - 0.0232632312710432*G0_0_2;
    A[82] = 0.0551276864210885*G0_0_0 + 0.0336585514536897*G0_0_1 + 0.0336585514536897*G0_0_2;
    A[83] = -0.0551276864210886*G0_0_0 - 0.0372571824394735*G0_0_1 - 0.0372571824394735*G0_0_2;
    A[84] = 0.0479930547204847*G0_0_0 + 0.0143945239431348*G0_0_1 + 0.0143945239431348*G0_0_2;
    A[85] = -0.0345719095719092*G0_0_0 - 0.00472467106581693*G0_0_1 - 0.00472467106581678*G0_0_2;
    A[86] = 0.0137421252872641*G0_0_0 - 0.0051774357329912*G0_0_1 - 0.0051774357329912*G0_0_2;
    A[87] = -0.0345719095719093*G0_0_0 - 0.00472467106581691*G0_0_1 - 0.0047246710658169*G0_0_2;
    A[88] = 0.0094649996993746*G0_0_0 - 0.00227817610630111*G0_0_1 - 0.00227817610630112*G0_0_2;
    A[89] = 0.013742125287264*G0_0_0 - 0.00517743573299123*G0_0_1 - 0.00517743573299125*G0_0_2;
    A[90] = -0.0137421252872639*G0_0_0;
    A[91] = -0.00946499969937464*G0_0_0;
    A[92] = -0.0137421252872641*G0_0_0;
    A[93] = -0.00946499969937492*G0_0_0;
    A[94] = -0.0094649996993745*G0_0_0;
    A[95] = -0.013742125287264*G0_0_0;
    A[96] = 0.0189195610202555*G0_0_0 + 0.0051774357329915*G0_0_1;
    A[97] = -0.0298472385060926*G0_0_0 + 0.00472467106581666*G0_0_1;
    A[98] = 0.0335985307773501*G0_0_0 - 0.0143945239431346*G0_0_1;
    A[99] = 0.0117431758056756*G0_0_0 + 0.00227817610630093*G0_0_1;
    A[100] = -0.0298472385060923*G0_0_0 + 0.00472467106581688*G0_0_1;
    A[101] = 0.0189195610202554*G0_0_0 + 0.00517743573299124*G0_0_1;
    A[102] = 0.0189195610202553*G0_0_0 + 0.00517743573299134*G0_0_2;
    A[103] = -0.0298472385060925*G0_0_0 + 0.00472467106581673*G0_0_2;
    A[104] = 0.0335985307773501*G0_0_0 - 0.0143945239431346*G0_0_2;
    A[105] = 0.0117431758056758*G0_0_0 + 0.00227817610630125*G0_0_2;
    A[106] = -0.0298472385060925*G0_0_0 + 0.0047246710658167*G0_0_2;
    A[107] = 0.0189195610202553*G0_0_0 + 0.00517743573299123*G0_0_2;
    A[108] = 0.0440369092712837*G0_0_0;
    A[109] = -0.0782878387045046*G0_0_0;
    A[110] = 0.0440369092712839*G0_0_0;
    A[111] = 0.0440369092712842*G0_0_0;
    A[112] = -0.0027789051226551*G0_1_0 - 0.00277890512265513*G0_1_1 - 0.00277890512265511*G0_1_2;
    A[113] = 0.00277890512265508*G0_1_0;
    A[114] = 0.0262652567340066*G0_1_1;
    A[115] = 0.00277890512265511*G0_1_2;
    A[116] = -0.0178705039816151*G0_1_1 + 0.0372571824394737*G0_1_2;
    A[117] = 0.0214691349673989*G0_1_1 - 0.03365855145369*G0_1_2;
    A[118] = -0.0155544887532735*G0_1_1 + 0.0232632312710435*G0_1_2;
    A[119] = 0.00480034930989788*G0_1_1 - 0.0105310191030677*G0_1_2;
    A[120] = 0.00480034930989784*G0_1_0 + 0.00480034930989785*G0_1_2;
    A[121] = 0.00336507226698202*G0_1_0 + 0.00336507226698196*G0_1_2;
    A[122] = 0.00336507226698193*G0_1_0 + 0.00336507226698196*G0_1_2;
    A[123] = 0.00480034930989789*G0_1_0 + 0.00480034930989786*G0_1_2;
    A[124] = -0.0105310191030676*G0_1_0 + 0.00480034930989784*G0_1_1;
    A[125] = 0.0232632312710435*G0_1_0 - 0.0155544887532733*G0_1_1;
    A[126] = -0.03365855145369*G0_1_0 + 0.0214691349673987*G0_1_1;
    A[127] = 0.0372571824394738*G0_1_0 - 0.0178705039816151*G0_1_1;
    A[128] = -0.00480034930989784*G0_1_0 - 0.00480034930989787*G0_1_1;
    A[129] = -0.00336507226698198*G0_1_0 - 0.00336507226698195*G0_1_1;
    A[130] = -0.00336507226698197*G0_1_0 - 0.00336507226698199*G0_1_1;
    A[131] = -0.00480034930989789*G0_1_0 - 0.00480034930989789*G0_1_1;
    A[132] = 0.0105310191030676*G0_1_0 + 0.0153313684129656*G0_1_1 + 0.0105310191030677*G0_1_2;
    A[133] = -0.0232632312710436*G0_1_0 - 0.0388177200243171*G0_1_1 - 0.0232632312710436*G0_1_2;
    A[134] = 0.03365855145369*G0_1_0 + 0.0551276864210889*G0_1_1 + 0.03365855145369*G0_1_2;
    A[135] = -0.0372571824394737*G0_1_0 - 0.0551276864210888*G0_1_1 - 0.0372571824394737*G0_1_2;
    A[136] = -0.00480034930989793*G0_1_1 - 0.00480034930989792*G0_1_2;
    A[137] = -0.00336507226698194*G0_1_1 - 0.00336507226698193*G0_1_2;
    A[138] = -0.00336507226698199*G0_1_1 - 0.00336507226698198*G0_1_2;
    A[139] = -0.00480034930989787*G0_1_1 - 0.00480034930989785*G0_1_2;
    A[140] = -0.00517743573299141*G0_1_0 + 0.013742125287264*G0_1_1 - 0.00517743573299138*G0_1_2;
    A[141] = -0.00472467106581692*G0_1_0 - 0.0345719095719095*G0_1_1 - 0.00472467106581676*G0_1_2;
    A[142] = 0.014394523943135*G0_1_0 + 0.0479930547204851*G0_1_1 + 0.0143945239431349*G0_1_2;
    A[143] = -0.00227817610630133*G0_1_0 + 0.00946499969937454*G0_1_1 - 0.00227817610630133*G0_1_2;
    A[144] = -0.00472467106581693*G0_1_0 - 0.0345719095719097*G0_1_1 - 0.00472467106581698*G0_1_2;
    A[145] = -0.00517743573299142*G0_1_0 + 0.013742125287264*G0_1_1 - 0.00517743573299146*G0_1_2;
    A[146] = 0.00517743573299131*G0_1_0 + 0.0189195610202553*G0_1_1;
    A[147] = 0.00472467106581688*G0_1_0 - 0.0298472385060924*G0_1_1;
    A[148] = -0.014394523943135*G0_1_0 + 0.0335985307773501*G0_1_1;
    A[149] = 0.00227817610630123*G0_1_0 + 0.0117431758056758*G0_1_1;
    A[150] = 0.00472467106581691*G0_1_0 - 0.0298472385060927*G0_1_1;
    A[151] = 0.00517743573299144*G0_1_0 + 0.0189195610202555*G0_1_1;
    A[152] = -0.0137421252872641*G0_1_1;
    A[153] = -0.00946499969937482*G0_1_1;
    A[154] = -0.013742125287264*G0_1_1;
    A[155] = -0.00946499969937459*G0_1_1;
    A[156] = -0.00946499969937459*G0_1_1;
    A[157] = -0.0137421252872639*G0_1_1;
    A[158] = 0.0189195610202553*G0_1_1 + 0.00517743573299124*G0_1_2;
    A[159] = 0.0117431758056758*G0_1_1 + 0.00227817610630102*G0_1_2;
    A[160] = 0.0189195610202553*G0_1_1 + 0.00517743573299135*G0_1_2;
    A[161] = -0.0298472385060924*G0_1_1 + 0.00472467106581685*G0_1_2;
    A[162] = -0.0298472385060926*G0_1_1 + 0.00472467106581684*G0_1_2;
    A[163] = 0.0335985307773501*G0_1_1 - 0.0143945239431348*G0_1_2;
    A[164] = 0.0440369092712842*G0_1_1;
    A[165] = 0.0440369092712844*G0_1_1;
    A[166] = -0.0782878387045052*G0_1_1;
    A[167] = 0.0440369092712841*G0_1_1;
    A[168] = -0.00277890512265511*G0_2_0 - 0.00277890512265512*G0_2_1 - 0.00277890512265511*G0_2_2;
    A[169] = 0.00277890512265508*G0_2_0;
    A[170] = 0.00277890512265511*G0_2_1;
    A[171] = 0.0262652567340067*G0_2_2;
    A[172] = -0.0105310191030677*G0_2_1 + 0.00480034930989791*G0_2_2;
    A[173] = 0.0232632312710438*G0_2_1 - 0.0155544887532734*G0_2_2;
    A[174] = -0.0336585514536902*G0_2_1 + 0.0214691349673989*G0_2_2;
    A[175] = 0.037257182439474*G0_2_1 - 0.0178705039816152*G0_2_2;
    A[176] = -0.0105310191030678*G0_2_0 + 0.00480034930989772*G0_2_2;
    A[177] = 0.0232632312710439*G0_2_0 - 0.0155544887532732*G0_2_2;
    A[178] = -0.0336585514536904*G0_2_0 + 0.0214691349673987*G0_2_2;
    A[179] = 0.0372571824394741*G0_2_0 - 0.017870503981615*G0_2_2;
    A[180] = 0.00480034930989775*G0_2_0 + 0.00480034930989773*G0_2_1;
    A[181] = 0.00336507226698206*G0_2_0 + 0.00336507226698203*G0_2_1;
    A[182] = 0.00336507226698196*G0_2_0 + 0.00336507226698197*G0_2_1;
    A[183] = 0.00480034930989792*G0_2_0 + 0.0048003493098979*G0_2_1;
    A[184] = 0.0105310191030678*G0_2_0 + 0.0105310191030678*G0_2_1 + 0.0153313684129656*G0_2_2;
    A[185] = -0.0232632312710438*G0_2_0 - 0.0232632312710439*G0_2_1 - 0.0388177200243172*G0_2_2;
    A[186] = 0.0336585514536903*G0_2_0 + 0.0336585514536903*G0_2_1 + 0.0551276864210891*G0_2_2;
    A[187] = -0.0372571824394741*G0_2_0 - 0.0372571824394741*G0_2_1 - 0.0551276864210891*G0_2_2;
    A[188] = -0.00480034930989781*G0_2_0 - 0.00480034930989782*G0_2_2;
    A[189] = -0.00336507226698201*G0_2_0 - 0.00336507226698201*G0_2_2;
    A[190] = -0.00336507226698192*G0_2_0 - 0.00336507226698193*G0_2_2;
    A[191] = -0.00480034930989792*G0_2_0 - 0.00480034930989792*G0_2_2;
    A[192] = -0.00480034930989785*G0_2_1 - 0.00480034930989785*G0_2_2;
    A[193] = -0.00336507226698186*G0_2_1 - 0.00336507226698186*G0_2_2;
    A[194] = -0.00336507226698204*G0_2_1 - 0.00336507226698201*G0_2_2;
    A[195] = -0.00480034930989772*G0_2_1 - 0.00480034930989774*G0_2_2;
    A[196] = -0.00517743573299123*G0_2_0 - 0.00517743573299137*G0_2_1 + 0.0137421252872638*G0_2_2;
    A[197] = -0.00227817610630131*G0_2_0 - 0.00227817610630131*G0_2_1 + 0.00946499969937474*G0_2_2;
    A[198] = -0.0051774357329913*G0_2_0 - 0.00517743573299127*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[199] = -0.00472467106581711*G0_2_0 - 0.00472467106581675*G0_2_1 - 0.0345719095719094*G0_2_2;
    A[200] = -0.00472467106581676*G0_2_0 - 0.00472467106581703*G0_2_1 - 0.0345719095719096*G0_2_2;
    A[201] = 0.0143945239431349*G0_2_0 + 0.0143945239431349*G0_2_1 + 0.0479930547204849*G0_2_2;
    A[202] = 0.00517743573299118*G0_2_0 + 0.0189195610202551*G0_2_2;
    A[203] = 0.00227817610630121*G0_2_0 + 0.0117431758056759*G0_2_2;
    A[204] = 0.0051774357329913*G0_2_0 + 0.0189195610202553*G0_2_2;
    A[205] = 0.00472467106581674*G0_2_0 - 0.0298472385060924*G0_2_2;
    A[206] = 0.00472467106581677*G0_2_0 - 0.0298472385060927*G0_2_2;
    A[207] = -0.0143945239431349*G0_2_0 + 0.0335985307773502*G0_2_2;
    A[208] = 0.00517743573299124*G0_2_1 + 0.0189195610202551*G0_2_2;
    A[209] = 0.00227817610630128*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[210] = 0.00517743573299139*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[211] = 0.00472467106581671*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[212] = 0.00472467106581669*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[213] = -0.0143945239431349*G0_2_1 + 0.03359853077735*G0_2_2;
    A[214] = -0.013742125287264*G0_2_2;
    A[215] = -0.00946499969937467*G0_2_2;
    A[216] = -0.0137421252872638*G0_2_2;
    A[217] = -0.00946499969937474*G0_2_2;
    A[218] = -0.0094649996993747*G0_2_2;
    A[219] = -0.013742125287264*G0_2_2;
    A[220] = 0.0440369092712838*G0_2_2;
    A[221] = 0.044036909271284*G0_2_2;
    A[222] = 0.0440369092712842*G0_2_2;
    A[223] = -0.0782878387045046*G0_2_2;
    A[224] = -0.00480034930989786*G0_1_0 - 0.00480034930989785*G0_1_1 - 0.00480034930989789*G0_1_2 - 0.00480034930989792*G0_2_0 - 0.00480034930989796*G0_2_1 - 0.00480034930989796*G0_2_2;
    A[225] = 0.00480034930989791*G0_1_0 + 0.00480034930989787*G0_2_0;
    A[226] = -0.0178705039816151*G0_1_1 + 0.0372571824394737*G0_2_1;
    A[227] = -0.0105310191030677*G0_1_2 + 0.00480034930989791*G0_2_2;
    A[228] = 0.179096479009673*G0_1_1 + 0.069911039963123*G0_1_2 + 0.069911039963123*G0_2_1 + 0.139822079926246*G0_2_2;
    A[229] = -0.147063705006413*G0_1_1 + 0.026383001643418*G0_1_2 - 0.0387263842124951*G0_2_1 - 0.108637424175618*G0_2_2;
    A[230] = 0.100717304493346*G0_1_1 - 0.0246345732456839*G0_1_2 + 0.0189782768992839*G0_2_1 + 0.0788945694544649*G0_2_2;
    A[231] = -0.0328352243499812*G0_1_1 + 0.0209159009072202*G0_1_2 - 0.00537576492437601*G0_2_1 - 0.0328352243499812*G0_2_2;
    A[232] = -0.0107515298487518*G0_1_0 - 0.00537576492437598*G0_1_2 - 0.00537576492437593*G0_2_0 - 0.00537576492437605*G0_2_2;
    A[233] = -0.0139743792087545*G0_1_0 - 0.0116844599266474*G0_1_2 - 0.00228991928210669*G0_2_0 - 0.00228991928210653*G0_2_2;
    A[234] = -0.0111690649885091*G0_1_0 - 0.00887914570640251*G0_1_2 - 0.00228991928210687*G0_2_0 - 0.00228991928210696*G0_2_2;
    A[235] = -0.0328352243499814*G0_1_0 - 0.0274594594256052*G0_1_2 - 0.00537576492437604*G0_2_0 - 0.00537576492437603*G0_2_2;
    A[236] = -0.0274594594256049*G0_1_0 - 0.005375764924376*G0_1_1 - 0.0328352243499809*G0_2_0 - 0.0053757649243761*G0_2_1;
    A[237] = 0.0599162925551811*G0_1_0 + 0.0283728175438245*G0_1_1 + 0.0788945694544646*G0_2_0 + 0.018978276899284*G0_2_1;
    A[238] = -0.069911039963123*G0_1_0 - 0.0574176057031959*G0_1_1 - 0.108637424175618*G0_2_0 - 0.0387263842124952*G0_2_1;
    A[239] = 0.0699110399631231*G0_1_0 + 0.0895482395048365*G0_1_1 + 0.139822079926246*G0_2_0 + 0.069911039963123*G0_2_1;
    A[240] = 0.0107515298487519*G0_1_0 + 0.0107515298487519*G0_1_1 + 0.00537576492437591*G0_1_2 + 0.00537576492437613*G0_2_0 + 0.00537576492437609*G0_2_1;
    A[241] = 0.0139743792087541*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.00228991928210705*G0_1_2 + 0.00228991928210648*G0_2_0 + 0.00228991928210658*G0_2_1;
    A[242] = 0.0111690649885094*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.00228991928210652*G0_1_2 + 0.00228991928210701*G0_2_0 + 0.00228991928210698*G0_2_1;
    A[243] = 0.0328352243499813*G0_1_0 + 0.0328352243499813*G0_1_1 + 0.00537576492437614*G0_1_2 + 0.00537576492437599*G0_2_0 + 0.005375764924376*G0_2_1;
    A[244] = 0.027459459425605*G0_1_0 + 0.022083694501229*G0_1_1 + 0.0274594594256051*G0_1_2 + 0.0328352243499811*G0_2_0 + 0.0274594594256053*G0_2_1 + 0.0328352243499812*G0_2_2;
    A[245] = -0.059916292555181*G0_1_0 - 0.0315434750113566*G0_1_1 - 0.0599162925551811*G0_1_2 - 0.0788945694544647*G0_2_0 - 0.0599162925551812*G0_2_1 - 0.0788945694544648*G0_2_2;
    A[246] = 0.0699110399631229*G0_1_0 + 0.0124934342599271*G0_1_1 + 0.0699110399631229*G0_1_2 + 0.108637424175618*G0_2_0 + 0.0699110399631228*G0_2_1 + 0.108637424175618*G0_2_2;
    A[247] = -0.069911039963123*G0_1_0 + 0.0196371995417135*G0_1_1 - 0.069911039963123*G0_1_2 - 0.139822079926246*G0_2_0 - 0.0699110399631226*G0_2_1 - 0.139822079926246*G0_2_2;
    A[248] = 0.00537576492437603*G0_1_1 + 0.00537576492437608*G0_1_2 + 0.00537576492437606*G0_2_1 + 0.00537576492437612*G0_2_2;
    A[249] = 0.00228991928210685*G0_1_1 + 0.00228991928210677*G0_1_2 + 0.00228991928210686*G0_2_1 + 0.00228991928210683*G0_2_2;
    A[250] = 0.00228991928210694*G0_1_1 + 0.00228991928210692*G0_1_2 + 0.00228991928210697*G0_2_1 + 0.00228991928210698*G0_2_2;
    A[251] = 0.0053757649243759*G0_1_1 + 0.00537576492437595*G0_1_2 + 0.00537576492437593*G0_2_1 + 0.00537576492437602*G0_2_2;
    A[252] = 0.0425363923627809*G0_1_0 - 0.018789081289081*G0_1_1 + 0.0212681961813908*G0_1_2 + 0.0118736555368499*G0_2_0 - 0.00939454064454033*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[253] = -0.0845508658008654*G0_1_0 + 0.0439716694056968*G0_1_1 - 0.0422754329004326*G0_1_2 - 0.0429604514890975*G0_2_0 - 0.000685018588664922*G0_2_1 - 0.0429604514890967*G0_2_2;
    A[254] = 0.192588083213083*G0_1_0 - 0.0644569872000427*G0_1_1 + 0.0962940416065412*G0_1_2 + 0.124738623002512*G0_2_0 + 0.0284445813959704*G0_2_1 + 0.124738623002512*G0_2_2;
    A[255] = 0.0142875305635725*G0_1_0 - 0.0292600797158086*G0_1_1 + 0.00844856259352771*G0_1_2 + 0.00254435475789609*G0_2_0 - 0.00329461321214794*G0_2_1 + 0.00254435475789608*G0_2_2;
    A[256] = -0.0493213383838377*G0_1_0 + 0.127315597693202*G0_1_1 - 0.00704590548340559*G0_1_2 - 0.0429604514890968*G0_2_0 - 0.000685018588664324*G0_2_1 - 0.0429604514890977*G0_2_2;
    A[257] = 0.0477555816097483*G0_1_0 - 0.064130787872107*G0_1_1 + 0.0264873854283582*G0_1_2 + 0.0118736555368499*G0_2_0 - 0.00939454064454091*G0_2_1 + 0.0118736555368501*G0_2_2;
    A[258] = -0.042536392362781*G0_1_0 - 0.0613254736518619*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.01187365553685*G0_2_0 - 0.0212681961813906*G0_2_1;
    A[259] = 0.0845508658008655*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0422754329004327*G0_1_2 + 0.0429604514890976*G0_2_0 + 0.0422754329004329*G0_2_1;
    A[260] = -0.192588083213083*G0_1_0 - 0.257045070413125*G0_1_1 - 0.0962940416065411*G0_1_2 - 0.124738623002512*G0_2_0 - 0.0962940416065415*G0_2_1;
    A[261] = -0.0142875305635725*G0_1_0 - 0.0435476102793811*G0_1_1 - 0.00583896797004432*G0_1_2 - 0.00254435475789642*G0_2_0 - 0.00583896797004417*G0_2_1;
    A[262] = 0.0493213383838378*G0_1_0 + 0.17663693607704*G0_1_1 + 0.0422754329004324*G0_1_2 + 0.0429604514890968*G0_2_0 + 0.0422754329004326*G0_2_1;
    A[263] = -0.0477555816097486*G0_1_0 - 0.111886369481856*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.01187365553685*G0_2_0 - 0.0212681961813909*G0_2_1;
    A[264] = 0.0187890812890812*G0_1_1 + 0.00939454064454053*G0_1_2 + 0.00939454064454046*G0_2_1;
    A[265] = 0.00658922642429603*G0_1_1 + 0.00329461321214782*G0_1_2 + 0.00329461321214786*G0_2_1;
    A[266] = 0.0187890812890812*G0_1_1 + 0.00939454064454049*G0_1_2 + 0.00939454064454041*G0_2_1;
    A[267] = 0.0292600797158083*G0_1_1 + 0.00329461321214781*G0_1_2 + 0.00329461321214749*G0_2_1;
    A[268] = 0.0292600797158087*G0_1_1 + 0.00329461321214786*G0_1_2 + 0.00329461321214798*G0_2_1;
    A[269] = 0.0641307878721068*G0_1_1 + 0.00939454064454087*G0_1_2 + 0.00939454064454087*G0_2_1;
    A[270] = -0.0306627368259311*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813906*G0_2_1 - 0.0118736555368497*G0_2_2;
    A[271] = -0.0091335811821928*G0_1_1 - 0.00583896797004427*G0_1_2 - 0.00583896797004477*G0_2_1 - 0.00254435475789639*G0_2_2;
    A[272] = -0.0306627368259312*G0_1_1 - 0.0212681961813909*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.01187365553685*G0_2_2;
    A[273] = 0.0642612676032814*G0_1_1 + 0.0422754329004327*G0_1_2 + 0.0422754329004329*G0_2_1 + 0.042960451489097*G0_2_2;
    A[274] = 0.0642612676032816*G0_1_1 + 0.0422754329004329*G0_1_2 + 0.0422754329004328*G0_2_1 + 0.0429604514890971*G0_2_2;
    A[275] = -0.128522535206563*G0_1_1 - 0.0962940416065411*G0_1_2 - 0.0962940416065412*G0_2_1 - 0.124738623002511*G0_2_2;
    A[276] = -0.0505608958299934*G0_1_1 - 0.00260959462348447*G0_1_2 - 0.00260959462348357*G0_2_1;
    A[277] = -0.0505608958299934*G0_1_1 - 0.00260959462348356*G0_1_2 - 0.0026095946234831*G0_2_1;
    A[278] = 0.0962288017409547*G0_1_1 - 0.0352295274170274*G0_1_2 - 0.0352295274170272*G0_2_1;
    A[279] = -0.15657567740901*G0_1_1 - 0.0026095946234833*G0_1_2 - 0.00260959462348317*G0_2_1;
    A[280] = -0.00336507226698194*G0_1_0 - 0.00336507226698198*G0_1_1 - 0.00336507226698196*G0_1_2 - 0.0033650722669818*G0_2_0 - 0.00336507226698183*G0_2_1 - 0.00336507226698182*G0_2_2;
    A[281] = 0.00336507226698195*G0_1_0 + 0.003365072266982*G0_2_0;
    A[282] = 0.0214691349673989*G0_1_1 - 0.03365855145369*G0_2_1;
    A[283] = 0.0232632312710438*G0_1_2 - 0.0155544887532734*G0_2_2;
    A[284] = -0.147063705006413*G0_1_1 - 0.0387263842124951*G0_1_2 + 0.026383001643418*G0_2_1 - 0.108637424175618*G0_2_2;
    A[285] = 0.304226541205707*G0_1_1 + 0.0781312630270954*G0_1_2 + 0.0781312630270955*G0_2_1 + 0.281001149056702*G0_2_2;
    A[286] = -0.204187731314467*G0_1_1 + 0.0153052724667311*G0_1_2 - 0.0364951808094166*G0_2_1 - 0.204187731314466*G0_2_2;
    A[287] = 0.0788945694544649*G0_1_1 - 0.0246345732456847*G0_1_2 + 0.0189782768992833*G0_2_1 + 0.100717304493345*G0_2_2;
    A[288] = -0.0139743792087543*G0_1_0 - 0.00228991928210673*G0_1_2 - 0.0116844599266473*G0_2_0 - 0.00228991928210661*G0_2_2;
    A[289] = 0.0263047138047141*G0_1_0 + 0.0131523569023568*G0_1_2 + 0.0131523569023563*G0_2_0 + 0.0132175967679433*G0_2_2;
    A[290] = 0.0052322372200841*G0_1_0 + 0.00578025209101594*G0_1_2 - 0.000548014870930904*G0_2_0 - 0.000482775005343929*G0_2_2;
    A[291] = 0.078894569454465*G0_1_0 + 0.059916292555181*G0_1_2 + 0.0189782768992833*G0_2_0 + 0.0283728175438241*G0_2_2;
    A[292] = -0.0088791457064026*G0_1_0 - 0.00228991928210667*G0_1_1 - 0.0111690649885093*G0_2_0 - 0.00228991928210653*G0_2_1;
    A[293] = 0.0057802520910158*G0_1_0 - 0.000482775005344784*G0_1_1 + 0.00523223722008443*G0_2_0 - 0.000548014870931813*G0_2_1;
    A[294] = 0.00480165410720999*G0_1_0 + 0.0050626135695584*G0_1_1 + 0.0096033082144191*G0_2_0 + 0.00480165410720983*G0_2_1;
    A[295] = -0.038726384212495*G0_1_0 - 0.0574176057031958*G0_1_1 - 0.108637424175618*G0_2_0 - 0.0699110399631229*G0_2_1;
    A[296] = 0.0139743792087542*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.0116844599266475*G0_1_2 + 0.0116844599266469*G0_2_0 + 0.011684459926647*G0_2_1 + 0.00939454064454018*G0_2_2;
    A[297] = -0.0263047138047138*G0_1_0 - 0.0263047138047139*G0_1_1 - 0.0131523569023572*G0_1_2 - 0.0131523569023563*G0_2_0 - 0.0131523569023565*G0_2_1 + 6.52398655874058e-05*G0_2_2;
    A[298] = -0.00523223722008431*G0_1_0 - 0.00523223722008435*G0_1_1 + 0.000548014870932083*G0_1_2 + 0.000548014870931168*G0_2_0 + 0.000548014870931203*G0_2_1 + 6.52398655871043e-05*G0_2_2;
    A[299] = -0.078894569454465*G0_1_0 - 0.078894569454465*G0_1_1 - 0.0189782768992841*G0_1_2 - 0.0189782768992833*G0_2_0 - 0.0189782768992834*G0_2_1 + 0.00939454064454055*G0_2_2;
    A[300] = 0.00887914570640263*G0_1_0 + 0.00658922642429599*G0_1_1 + 0.0088791457064026*G0_1_2 + 0.0111690649885091*G0_2_0 + 0.00887914570640228*G0_2_1 + 0.011169064988509*G0_2_2;
    A[301] = -0.00578025209101582*G0_1_0 - 0.00626302709636063*G0_1_1 - 0.00578025209101584*G0_1_2 - 0.00523223722008425*G0_2_0 - 0.00578025209101562*G0_2_1 - 0.00523223722008418*G0_2_2;
    A[302] = -0.00480165410720958*G0_1_0 + 0.00026095946234856*G0_1_1 - 0.0048016541072095*G0_1_2 - 0.00960330821441926*G0_2_0 - 0.00480165410720973*G0_2_1 - 0.00960330821441911*G0_2_2;
    A[303] = 0.0387263842124951*G0_1_0 - 0.0186912214907008*G0_1_1 + 0.038726384212495*G0_1_2 + 0.108637424175618*G0_2_0 + 0.038726384212495*G0_2_1 + 0.108637424175618*G0_2_2;
    A[304] = 0.00228991928210676*G0_1_1 + 0.0022899192821067*G0_1_2 + 0.00228991928210662*G0_2_1 + 0.00228991928210655*G0_2_2;
    A[305] = 0.00178757231708619*G0_1_1 + 0.00178757231708626*G0_1_2 + 0.00178757231708606*G0_2_1 + 0.00178757231708605*G0_2_2;
    A[306] = 0.00178757231708605*G0_1_1 + 0.00178757231708608*G0_1_2 + 0.00178757231708599*G0_2_1 + 0.00178757231708594*G0_2_2;
    A[307] = 0.00228991928210683*G0_1_1 + 0.00228991928210677*G0_1_2 + 0.00228991928210672*G0_2_1 + 0.00228991928210663*G0_2_2;
    A[308] = 0.0403182369328201*G0_1_0 + 0.0148746893538559*G0_1_1 + 0.00123955744615454*G0_1_2 + 0.0540186087061082*G0_2_0 + 0.0149399292194427*G0_2_1 + 0.00117431758056792*G0_2_2;
    A[309] = -0.0808321934624001*G0_1_0 - 0.0436454700777606*G0_1_1 - 0.00649136662591457*G0_1_2 - 0.108232937008976*G0_2_0 - 0.0338921101724903*G0_2_1 - 0.00352295274170247*G0_2_2;
    A[310] = 0.0109602974186293*G0_1_0 + 0.132306447410613*G0_1_1 - 0.0623693115012558*G0_1_2 + 0.146659217839771*G0_2_0 + 0.0733296089198853*G0_2_1 - 0.124738623002512*G0_2_2;
    A[311] = -0.0377738821749241*G0_1_0 + 0.0189848008858424*G0_1_1 - 0.0188869410874617*G0_1_2 - 0.00939454064454015*G0_2_0 + 0.00949240044292109*G0_2_1 - 0.0155923278753137*G0_2_2;
    A[312] = 0.167731694424402*G0_1_0 - 0.166948816037358*G0_1_1 + 0.0838658472122008*G0_1_2 + 0.0777006799142209*G0_2_0 - 0.0061651672979797*G0_2_1 + 0.182410664181497*G0_2_2;
    A[313] = -0.082332710370905*G0_1_0 + 0.127087258163646*G0_1_1 - 0.0463855444324201*G0_1_2 + 0.00182671623643819*G0_2_0 + 0.0377738821749238*G0_2_1 - 0.0510175748891025*G0_2_2;
    A[314] = -0.0403182369328198*G0_1_0 - 0.0254435475789642*G0_1_1 - 0.0390786794866651*G0_1_2 - 0.0540186087061079*G0_2_0 - 0.0390786794866648*G0_2_1 - 0.0528442911255396*G0_2_2;
    A[315] = 0.080832193462401*G0_1_0 + 0.0371867233846402*G0_1_1 + 0.0743408268364854*G0_1_2 + 0.108232937008977*G0_2_0 + 0.0743408268364859*G0_2_1 + 0.104709984267274*G0_2_2;
    A[316] = -0.0109602974186295*G0_1_0 + 0.121346149991983*G0_1_1 - 0.0733296089198855*G0_1_2 - 0.146659217839771*G0_2_0 - 0.0733296089198858*G0_2_1 - 0.271397840842282*G0_2_2;
    A[317] = 0.037773882174923*G0_1_0 + 0.056758683060766*G0_1_1 + 0.0188869410874616*G0_1_2 + 0.00939454064453975*G0_2_0 + 0.0188869410874609*G0_2_1 - 0.00619778723077428*G0_2_2;
    A[318] = -0.167731694424402*G0_1_0 - 0.334680510461759*G0_1_1 - 0.083865847212201*G0_1_2 - 0.0777006799142207*G0_2_0 - 0.0838658472122*G0_2_1 + 0.104709984267276*G0_2_2;
    A[319] = 0.0823327103709051*G0_1_0 + 0.209419968534551*G0_1_1 + 0.0359471659384856*G0_1_2 - 0.00182671623643823*G0_2_0 + 0.0359471659384854*G0_2_1 - 0.0528442911255402*G0_2_2;
    A[320] = -0.0148746893538562*G0_1_1 - 0.0149399292194432*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0150051690850294*G0_2_2;
    A[321] = 0.00450155072550945*G0_1_1 + 0.00192457603481913*G0_1_2 + 0.00192457603481957*G0_2_1 - 0.000652398655870256*G0_2_2;
    A[322] = -0.0148746893538561*G0_1_1 - 0.014939929219443*G0_1_2 - 0.0149399292194427*G0_2_1 - 0.0150051690850292*G0_2_2;
    A[323] = -0.0189848008858424*G0_1_1 - 0.00949240044292074*G0_1_2 - 0.00949240044292084*G0_2_1 - 0.000652398655871592*G0_2_2;
    A[324] = -0.0189848008858425*G0_1_1 - 0.00949240044292137*G0_1_2 - 0.00949240044292117*G0_2_1 - 0.000652398655871956*G0_2_2;
    A[325] = -0.127087258163646*G0_1_1 - 0.0377738821749239*G0_1_2 - 0.0377738821749238*G0_2_1 - 0.0150051690850296*G0_2_2;
    A[326] = -0.00130479731174165*G0_1_1 - 0.0012395574461548*G0_1_2 - 0.0012395574461544*G0_2_1 - 0.00117431758056795*G0_2_2;
    A[327] = -0.0156575677409011*G0_1_1 - 0.0130805930502113*G0_1_2 - 0.013080593050211*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[328] = -0.00130479731174156*G0_1_1 - 0.00123955744615444*G0_1_2 - 0.0012395574461545*G0_2_1 - 0.00117431758056772*G0_2_2;
    A[329] = 0.00880738185425689*G0_1_1 + 0.00649136662591545*G0_1_2 + 0.00649136662591501*G0_2_1 + 0.00352295274170279*G0_2_2;
    A[330] = 0.00880738185425669*G0_1_1 + 0.0064913666259146*G0_1_2 + 0.00649136662591533*G0_2_1 + 0.0035229527417023*G0_2_2;
    A[331] = 0.0476251018785736*G0_1_1 + 0.0623693115012555*G0_1_2 + 0.0623693115012555*G0_2_1 + 0.124738623002511*G0_2_2;
    A[332] = 0.0391439193522526*G0_1_1 + 0.031967534137673*G0_1_2 + 0.0319675341376721*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[333] = 0.039143919352252*G0_1_1 + 0.031967534137673*G0_1_2 + 0.0319675341376709*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[334] = -0.15657567740901*G0_1_1 - 0.0903572138381159*G0_1_2 - 0.0903572138381149*G0_2_1 - 0.185933616923199*G0_2_2;
    A[335] = 0.1859336169232*G0_1_1 + 0.0156575677409015*G0_1_2 + 0.0156575677409007*G0_2_1 + 0.0260959462348343*G0_2_2;
    A[336] = -0.00336507226698194*G0_1_0 - 0.00336507226698193*G0_1_1 - 0.00336507226698193*G0_1_2 - 0.00336507226698208*G0_2_0 - 0.00336507226698208*G0_2_1 - 0.00336507226698206*G0_2_2;
    A[337] = 0.0033650722669819*G0_1_0 + 0.00336507226698188*G0_2_0;
    A[338] = -0.0155544887532735*G0_1_1 + 0.0232632312710435*G0_2_1;
    A[339] = -0.0336585514536902*G0_1_2 + 0.0214691349673989*G0_2_2;
    A[340] = 0.100717304493346*G0_1_1 + 0.0189782768992839*G0_1_2 - 0.0246345732456839*G0_2_1 + 0.0788945694544649*G0_2_2;
    A[341] = -0.204187731314467*G0_1_1 - 0.0364951808094166*G0_1_2 + 0.0153052724667311*G0_2_1 - 0.204187731314466*G0_2_2;
    A[342] = 0.281001149056703*G0_1_1 + 0.0781312630270952*G0_1_2 + 0.0781312630270952*G0_2_1 + 0.304226541205706*G0_2_2;
    A[343] = -0.108637424175618*G0_1_1 + 0.0263830016434185*G0_1_2 - 0.0387263842124944*G0_2_1 - 0.147063705006412*G0_2_2;
    A[344] = -0.0111690649885089*G0_1_0 - 0.00228991928210674*G0_1_2 - 0.00887914570640262*G0_2_0 - 0.00228991928210698*G0_2_2;
    A[345] = 0.00523223722008359*G0_1_0 - 0.000548014870931731*G0_1_2 + 0.00578025209101647*G0_2_0 - 0.00048277500534381*G0_2_2;
    A[346] = 0.00960330821441978*G0_1_0 + 0.00480165410720975*G0_1_2 + 0.00480165410720897*G0_2_0 + 0.00506261356955755*G0_2_2;
    A[347] = -0.108637424175618*G0_1_0 - 0.0699110399631229*G0_1_2 - 0.0387263842124945*G0_2_0 - 0.0574176057031954*G0_2_2;
    A[348] = -0.0116844599266469*G0_1_0 - 0.00228991928210676*G0_1_1 - 0.0139743792087539*G0_2_0 - 0.00228991928210702*G0_2_1;
    A[349] = 0.0131523569023566*G0_1_0 + 0.0132175967679442*G0_1_1 + 0.0263047138047133*G0_2_0 + 0.0131523569023571*G0_2_1;
    A[350] = -0.000548014870931627*G0_1_0 - 0.000482775005344918*G0_1_1 + 0.00523223722008468*G0_2_0 + 0.0057802520910155*G0_2_1;
    A[351] = 0.0189782768992839*G0_1_0 + 0.0283728175438248*G0_1_1 + 0.0788945694544648*G0_2_0 + 0.0599162925551816*G0_2_1;
    A[352] = 0.0111690649885091*G0_1_0 + 0.0111690649885091*G0_1_1 + 0.0088791457064024*G0_1_2 + 0.00887914570640288*G0_2_0 + 0.00887914570640285*G0_2_1 + 0.00658922642429613*G0_2_2;
    A[353] = -0.00523223722008375*G0_1_0 - 0.00523223722008369*G0_1_1 - 0.00578025209101555*G0_1_2 - 0.00578025209101628*G0_2_0 - 0.00578025209101622*G0_2_1 - 0.00626302709636069*G0_2_2;
    A[354] = -0.00960330821441976*G0_1_0 - 0.00960330821441963*G0_1_1 - 0.00480165410721015*G0_1_2 - 0.00480165410720934*G0_2_0 - 0.00480165410720928*G0_2_1 + 0.000260959462348355*G0_2_2;
    A[355] = 0.108637424175618*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0387263842124954*G0_1_2 + 0.0387263842124946*G0_2_0 + 0.0387263842124945*G0_2_1 - 0.0186912214907006*G0_2_2;
    A[356] = 0.0116844599266471*G0_1_0 + 0.00939454064454035*G0_1_1 + 0.0116844599266471*G0_1_2 + 0.0139743792087544*G0_2_0 + 0.0116844599266476*G0_2_1 + 0.0139743792087545*G0_2_2;
    A[357] = -0.0131523569023567*G0_1_0 + 6.52398655875253e-05*G0_1_1 - 0.0131523569023567*G0_1_2 - 0.0263047138047136*G0_2_0 - 0.013152356902357*G0_2_1 - 0.0263047138047137*G0_2_2;
    A[358] = 0.000548014870931835*G0_1_0 + 6.52398655866746e-05*G0_1_1 + 0.000548014870931798*G0_1_2 - 0.00523223722008388*G0_2_0 + 0.000548014870931404*G0_2_1 - 0.00523223722008401*G0_2_2;
    A[359] = -0.0189782768992839*G0_1_0 + 0.0093945406445409*G0_1_1 - 0.0189782768992839*G0_1_2 - 0.0788945694544648*G0_2_0 - 0.0189782768992835*G0_2_1 - 0.0788945694544648*G0_2_2;
    A[360] = 0.00228991928210659*G0_1_1 + 0.00228991928210664*G0_1_2 + 0.00228991928210687*G0_2_1 + 0.00228991928210696*G0_2_2;
    A[361] = 0.00178757231708612*G0_1_1 + 0.00178757231708607*G0_1_2 + 0.00178757231708635*G0_2_1 + 0.00178757231708636*G0_2_2;
    A[362] = 0.00178757231708628*G0_1_1 + 0.00178757231708624*G0_1_2 + 0.00178757231708635*G0_2_1 + 0.00178757231708636*G0_2_2;
    A[363] = 0.00228991928210663*G0_1_1 + 0.00228991928210669*G0_1_2 + 0.00228991928210689*G0_2_1 + 0.00228991928210697*G0_2_2;
    A[364] = 0.0540186087061076*G0_1_0 + 0.00117431758056759*G0_1_1 + 0.0149399292194435*G0_1_2 + 0.04031823693282*G0_2_0 + 0.00123955744615496*G0_2_1 + 0.0148746893538557*G0_2_2;
    A[365] = -0.0093945406445407*G0_1_0 - 0.0155923278753141*G0_1_1 + 0.00949240044292053*G0_1_2 - 0.0377738821749245*G0_2_0 - 0.0188869410874627*G0_2_1 + 0.0189848008858421*G0_2_2;
    A[366] = 0.00182671623643931*G0_1_0 - 0.0510175748891026*G0_1_1 + 0.0377738821749243*G0_1_2 - 0.082332710370903*G0_2_0 - 0.0463855444324186*G0_2_1 + 0.127087258163647*G0_2_2;
    A[367] = -0.108232937008977*G0_1_0 - 0.00352295274170266*G0_1_1 - 0.033892110172492*G0_1_2 - 0.0808321934624012*G0_2_0 - 0.0064913666259148*G0_2_1 - 0.0436454700777613*G0_2_2;
    A[368] = 0.0777006799142203*G0_1_0 + 0.182410664181497*G0_1_1 - 0.00616516729797992*G0_1_2 + 0.167731694424402*G0_2_0 + 0.0838658472122004*G0_2_1 - 0.166948816037357*G0_2_2;
    A[369] = 0.146659217839772*G0_1_0 - 0.124738623002511*G0_1_1 + 0.0733296089198868*G0_1_2 + 0.0109602974186305*G0_2_0 - 0.0623693115012557*G0_2_1 + 0.132306447410613*G0_2_2;
    A[370] = -0.0540186087061083*G0_1_0 - 0.0528442911255406*G0_1_1 - 0.0390786794866648*G0_1_2 - 0.0403182369328201*G0_2_0 - 0.0390786794866657*G0_2_1 - 0.025443547578965*G0_2_2;
    A[371] = 0.00939454064454054*G0_1_0 - 0.00619778723077307*G0_1_1 + 0.0188869410874612*G0_1_2 + 0.0377738821749244*G0_2_0 + 0.0188869410874625*G0_2_1 + 0.0567586830607662*G0_2_2;
    A[372] = -0.00182671623643939*G0_1_0 - 0.052844291125542*G0_1_1 + 0.0359471659384848*G0_1_2 + 0.082332710370903*G0_2_0 + 0.0359471659384841*G0_2_1 + 0.209419968534549*G0_2_2;
    A[373] = 0.108232937008977*G0_1_0 + 0.104709984267274*G0_1_1 + 0.0743408268364856*G0_1_2 + 0.0808321934624005*G0_2_0 + 0.0743408268364859*G0_2_1 + 0.0371867233846406*G0_2_2;
    A[374] = -0.0777006799142203*G0_1_0 + 0.104709984267277*G0_1_1 - 0.0838658472121997*G0_1_2 - 0.167731694424402*G0_2_0 - 0.0838658472122006*G0_2_1 - 0.334680510461758*G0_2_2;
    A[375] = -0.146659217839773*G0_1_0 - 0.271397840842284*G0_1_1 - 0.0733296089198863*G0_1_2 - 0.0109602974186306*G0_2_0 - 0.073329608919886*G0_2_1 + 0.121346149991982*G0_2_2;
    A[376] = -0.00117431758056745*G0_1_1 - 0.00123955744615409*G0_1_2 - 0.00123955744615476*G0_2_1 - 0.00130479731174235*G0_2_2;
    A[377] = -0.0105036183595213*G0_1_1 - 0.013080593050211*G0_1_2 - 0.013080593050211*G0_2_1 - 0.0156575677409013*G0_2_2;
    A[378] = -0.00117431758056766*G0_1_1 - 0.00123955744615456*G0_1_2 - 0.00123955744615516*G0_2_1 - 0.00130479731174267*G0_2_2;
    A[379] = 0.00352295274170231*G0_1_1 + 0.00649136662591435*G0_1_2 + 0.00649136662591446*G0_2_1 + 0.00880738185425734*G0_2_2;
    A[380] = 0.00352295274170306*G0_1_1 + 0.00649136662591512*G0_1_2 + 0.00649136662591508*G0_2_1 + 0.00880738185425746*G0_2_2;
    A[381] = 0.124738623002511*G0_1_1 + 0.062369311501256*G0_1_2 + 0.0623693115012556*G0_2_1 + 0.0476251018785734*G0_2_2;
    A[382] = -0.0150051690850301*G0_1_1 - 0.0149399292194434*G0_1_2 - 0.0149399292194433*G0_2_1 - 0.0148746893538557*G0_2_2;
    A[383] = -0.000652398655870701*G0_1_1 + 0.00192457603481927*G0_1_2 + 0.0019245760348187*G0_2_1 + 0.00450155072550905*G0_2_2;
    A[384] = -0.0150051690850302*G0_1_1 - 0.0149399292194435*G0_1_2 - 0.0149399292194432*G0_2_1 - 0.0148746893538559*G0_2_2;
    A[385] = -0.000652398655870402*G0_1_1 - 0.00949240044292083*G0_1_2 - 0.00949240044292027*G0_2_1 - 0.0189848008858418*G0_2_2;
    A[386] = -0.000652398655870635*G0_1_1 - 0.00949240044292064*G0_1_2 - 0.00949240044292077*G0_2_1 - 0.0189848008858419*G0_2_2;
    A[387] = -0.0150051690850308*G0_1_1 - 0.0377738821749241*G0_1_2 - 0.0377738821749248*G0_2_1 - 0.127087258163647*G0_2_2;
    A[388] = 0.0260959462348343*G0_1_1 + 0.0319675341376724*G0_1_2 + 0.0319675341376727*G0_2_1 + 0.0391439193522508*G0_2_2;
    A[389] = 0.0260959462348348*G0_1_1 + 0.0319675341376728*G0_1_2 + 0.0319675341376737*G0_2_1 + 0.0391439193522525*G0_2_2;
    A[390] = 0.0260959462348358*G0_1_1 + 0.0156575677409003*G0_1_2 + 0.0156575677409008*G0_2_1 + 0.185933616923198*G0_2_2;
    A[391] = -0.185933616923199*G0_1_1 - 0.0903572138381156*G0_1_2 - 0.0903572138381151*G0_2_1 - 0.156575677409008*G0_2_2;
    A[392] = -0.00480034930989791*G0_1_0 - 0.00480034930989794*G0_1_1 - 0.00480034930989794*G0_1_2 - 0.00480034930989783*G0_2_0 - 0.00480034930989784*G0_2_1 - 0.00480034930989785*G0_2_2;
    A[393] = 0.00480034930989788*G0_1_0 + 0.00480034930989789*G0_2_0;
    A[394] = 0.00480034930989788*G0_1_1 - 0.0105310191030677*G0_2_1;
    A[395] = 0.037257182439474*G0_1_2 - 0.0178705039816152*G0_2_2;
    A[396] = -0.0328352243499812*G0_1_1 - 0.00537576492437601*G0_1_2 + 0.0209159009072202*G0_2_1 - 0.0328352243499812*G0_2_2;
    A[397] = 0.0788945694544649*G0_1_1 + 0.0189782768992833*G0_1_2 - 0.0246345732456847*G0_2_1 + 0.100717304493345*G0_2_2;
    A[398] = -0.108637424175618*G0_1_1 - 0.0387263842124944*G0_1_2 + 0.0263830016434185*G0_2_1 - 0.147063705006412*G0_2_2;
    A[399] = 0.139822079926245*G0_1_1 + 0.0699110399631222*G0_1_2 + 0.0699110399631222*G0_2_1 + 0.179096479009672*G0_2_2;
    A[400] = -0.0328352243499811*G0_1_0 - 0.00537576492437596*G0_1_2 - 0.0274594594256047*G0_2_0 - 0.00537576492437572*G0_2_2;
    A[401] = 0.0788945694544651*G0_1_0 + 0.0189782768992838*G0_1_2 + 0.0599162925551803*G0_2_0 + 0.0283728175438238*G0_2_2;
    A[402] = -0.108637424175618*G0_1_0 - 0.038726384212495*G0_1_2 - 0.0699110399631222*G0_2_0 - 0.0574176057031954*G0_2_2;
    A[403] = 0.139822079926246*G0_1_0 + 0.0699110399631226*G0_1_2 + 0.0699110399631224*G0_2_0 + 0.089548239504836*G0_2_2;
    A[404] = -0.00537576492437606*G0_1_0 - 0.00537576492437599*G0_1_1 - 0.0107515298487518*G0_2_0 - 0.00537576492437574*G0_2_1;
    A[405] = -0.00228991928210666*G0_1_0 - 0.00228991928210684*G0_1_1 - 0.013974379208754*G0_2_0 - 0.0116844599266474*G0_2_1;
    A[406] = -0.0022899192821067*G0_1_0 - 0.00228991928210659*G0_1_1 - 0.0111690649885095*G0_2_0 - 0.00887914570640245*G0_2_1;
    A[407] = -0.00537576492437597*G0_1_0 - 0.00537576492437613*G0_1_1 - 0.0328352243499811*G0_2_0 - 0.0274594594256053*G0_2_1;
    A[408] = 0.0328352243499812*G0_1_0 + 0.0328352243499813*G0_1_1 + 0.0274594594256052*G0_1_2 + 0.0274594594256049*G0_2_0 + 0.027459459425605*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[409] = -0.0788945694544649*G0_1_0 - 0.0788945694544652*G0_1_1 - 0.0599162925551814*G0_1_2 - 0.0599162925551806*G0_2_0 - 0.0599162925551808*G0_2_1 - 0.0315434750113565*G0_2_2;
    A[410] = 0.108637424175618*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0699110399631235*G0_1_2 + 0.0699110399631223*G0_2_0 + 0.0699110399631223*G0_2_1 + 0.0124934342599271*G0_2_2;
    A[411] = -0.139822079926246*G0_1_0 - 0.139822079926246*G0_1_1 - 0.0699110399631233*G0_1_2 - 0.0699110399631223*G0_2_0 - 0.0699110399631223*G0_2_1 + 0.0196371995417135*G0_2_2;
    A[412] = 0.00537576492437612*G0_1_0 + 0.00537576492437612*G0_1_2 + 0.0107515298487519*G0_2_0 + 0.00537576492437591*G0_2_1 + 0.0107515298487519*G0_2_2;
    A[413] = 0.00228991928210674*G0_1_0 + 0.00228991928210665*G0_1_2 + 0.0139743792087541*G0_2_0 + 0.00228991928210691*G0_2_1 + 0.013974379208754*G0_2_2;
    A[414] = 0.00228991928210669*G0_1_0 + 0.0022899192821067*G0_1_2 + 0.0111690649885092*G0_2_0 + 0.00228991928210666*G0_2_1 + 0.0111690649885092*G0_2_2;
    A[415] = 0.00537576492437601*G0_1_0 + 0.00537576492437603*G0_1_2 + 0.0328352243499812*G0_2_0 + 0.0053757649243761*G0_2_1 + 0.0328352243499811*G0_2_2;
    A[416] = 0.00537576492437606*G0_1_1 + 0.00537576492437601*G0_1_2 + 0.00537576492437601*G0_2_1 + 0.00537576492437594*G0_2_2;
    A[417] = 0.00228991928210686*G0_1_1 + 0.00228991928210691*G0_1_2 + 0.00228991928210663*G0_2_1 + 0.00228991928210665*G0_2_2;
    A[418] = 0.00228991928210665*G0_1_1 + 0.00228991928210667*G0_1_2 + 0.00228991928210678*G0_2_1 + 0.00228991928210676*G0_2_2;
    A[419] = 0.00537576492437605*G0_1_1 + 0.00537576492437599*G0_1_2 + 0.0053757649243758*G0_2_1 + 0.00537576492437577*G0_2_2;
    A[420] = 0.0118736555368503*G0_1_0 + 0.0118736555368502*G0_1_1 - 0.00939454064454049*G0_1_2 + 0.042536392362781*G0_2_0 + 0.0212681961813908*G0_2_1 - 0.0187890812890805*G0_2_2;
    A[421] = 0.0025443547578967*G0_1_0 + 0.00254435475789674*G0_1_1 - 0.0032946132121474*G0_1_2 + 0.0142875305635731*G0_2_0 + 0.00844856259352872*G0_2_1 - 0.0292600797158084*G0_2_2;
    A[422] = 0.011873655536849*G0_1_0 + 0.0118736555368495*G0_1_1 - 0.00939454064454109*G0_1_2 + 0.047755581609747*G0_2_0 + 0.0264873854283569*G0_2_1 - 0.0641307878721076*G0_2_2;
    A[423] = -0.0429604514890973*G0_1_0 - 0.0429604514890969*G0_1_1 - 0.000685018588664183*G0_1_2 - 0.0845508658008648*G0_2_0 - 0.0422754329004329*G0_2_1 + 0.043971669405697*G0_2_2;
    A[424] = -0.0429604514890966*G0_1_0 - 0.0429604514890972*G0_1_1 - 0.000685018588664485*G0_1_2 - 0.049321338383838*G0_2_0 - 0.00704590548340517*G0_2_1 + 0.127315597693202*G0_2_2;
    A[425] = 0.12473862300251*G0_1_0 + 0.12473862300251*G0_1_1 + 0.0284445813959694*G0_1_2 + 0.192588083213082*G0_2_0 + 0.0962940416065408*G0_2_1 - 0.0644569872000427*G0_2_2;
    A[426] = -0.0118736555368506*G0_1_0 - 0.0212681961813909*G0_1_2 - 0.0425363923627815*G0_2_0 - 0.0212681961813907*G0_2_1 - 0.0613254736518619*G0_2_2;
    A[427] = -0.00254435475789622*G0_1_0 - 0.0058389679700438*G0_1_2 - 0.0142875305635724*G0_2_0 - 0.00583896797004379*G0_2_1 - 0.0435476102793808*G0_2_2;
    A[428] = -0.011873655536849*G0_1_0 - 0.0212681961813901*G0_1_2 - 0.0477555816097471*G0_2_0 - 0.0212681961813906*G0_2_1 - 0.111886369481854*G0_2_2;
    A[429] = 0.0429604514890963*G0_1_0 + 0.0422754329004326*G0_1_2 + 0.0845508658008645*G0_2_0 + 0.0422754329004323*G0_2_1 + 0.128522535206562*G0_2_2;
    A[430] = 0.0429604514890967*G0_1_0 + 0.0422754329004318*G0_1_2 + 0.0493213383838382*G0_2_0 + 0.042275432900433*G0_2_1 + 0.176636936077039*G0_2_2;
    A[431] = -0.12473862300251*G0_1_0 - 0.0962940416065402*G0_1_2 - 0.192588083213082*G0_2_0 - 0.0962940416065408*G0_2_1 - 0.257045070413124*G0_2_2;
    A[432] = -0.0118736555368504*G0_1_1 - 0.0212681961813909*G0_1_2 - 0.021268196181391*G0_2_1 - 0.030662736825931*G0_2_2;
    A[433] = -0.00254435475789568*G0_1_1 - 0.00583896797004432*G0_1_2 - 0.00583896797004383*G0_2_1 - 0.0091335811821921*G0_2_2;
    A[434] = -0.0118736555368502*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.0212681961813907*G0_2_1 - 0.0306627368259307*G0_2_2;
    A[435] = 0.0429604514890966*G0_1_1 + 0.0422754329004332*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[436] = 0.0429604514890968*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004328*G0_2_1 + 0.0642612676032807*G0_2_2;
    A[437] = -0.12473862300251*G0_1_1 - 0.0962940416065409*G0_1_2 - 0.0962940416065408*G0_2_1 - 0.128522535206562*G0_2_2;
    A[438] = 0.00939454064454017*G0_1_2 + 0.00939454064454042*G0_2_1 + 0.0187890812890804*G0_2_2;
    A[439] = 0.0032946132121482*G0_1_2 + 0.003294613212148*G0_2_1 + 0.00658922642429634*G0_2_2;
    A[440] = 0.00939454064454044*G0_1_2 + 0.00939454064454052*G0_2_1 + 0.0187890812890805*G0_2_2;
    A[441] = 0.00329461321214796*G0_1_2 + 0.00329461321214812*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[442] = 0.00329461321214755*G0_1_2 + 0.00329461321214783*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[443] = 0.00939454064454088*G0_1_2 + 0.00939454064454072*G0_2_1 + 0.0641307878721074*G0_2_2;
    A[444] = -0.0026095946234837*G0_1_2 - 0.00260959462348324*G0_2_1 - 0.0505608958299924*G0_2_2;
    A[445] = -0.00260959462348392*G0_1_2 - 0.00260959462348488*G0_2_1 - 0.0505608958299933*G0_2_2;
    A[446] = -0.00260959462348333*G0_1_2 - 0.00260959462348275*G0_2_1 - 0.15657567740901*G0_2_2;
    A[447] = -0.035229527417026*G0_1_2 - 0.035229527417027*G0_2_1 + 0.0962288017409544*G0_2_2;
    A[448] = -0.00480034930989781*G0_0_0 - 0.00480034930989778*G0_0_1 - 0.00480034930989779*G0_0_2 - 0.00480034930989777*G0_2_0 - 0.00480034930989774*G0_2_1 - 0.00480034930989775*G0_2_2;
    A[449] = -0.017870503981615*G0_0_0 + 0.0372571824394735*G0_2_0;
    A[450] = 0.00480034930989784*G0_0_1 + 0.00480034930989785*G0_2_1;
    A[451] = -0.0105310191030678*G0_0_2 + 0.00480034930989772*G0_2_2;
    A[452] = -0.0107515298487518*G0_0_1 - 0.00537576492437593*G0_0_2 - 0.00537576492437598*G0_2_1 - 0.00537576492437605*G0_2_2;
    A[453] = -0.0139743792087543*G0_0_1 - 0.0116844599266473*G0_0_2 - 0.00228991928210673*G0_2_1 - 0.00228991928210661*G0_2_2;
    A[454] = -0.0111690649885089*G0_0_1 - 0.00887914570640263*G0_0_2 - 0.00228991928210674*G0_2_1 - 0.00228991928210697*G0_2_2;
    A[455] = -0.0328352243499811*G0_0_1 - 0.0274594594256047*G0_0_2 - 0.00537576492437596*G0_2_1 - 0.00537576492437572*G0_2_2;
    A[456] = 0.179096479009672*G0_0_0 + 0.0699110399631227*G0_0_2 + 0.0699110399631227*G0_2_0 + 0.139822079926245*G0_2_2;
    A[457] = -0.147063705006412*G0_0_0 + 0.0263830016434181*G0_0_2 - 0.0387263842124952*G0_2_0 - 0.108637424175617*G0_2_2;
    A[458] = 0.100717304493345*G0_0_0 - 0.0246345732456844*G0_0_2 + 0.0189782768992838*G0_2_0 + 0.0788945694544639*G0_2_2;
    A[459] = -0.0328352243499813*G0_0_0 + 0.0209159009072204*G0_0_2 - 0.00537576492437602*G0_2_0 - 0.0328352243499805*G0_2_2;
    A[460] = 0.0895482395048358*G0_0_0 + 0.0699110399631227*G0_0_1 + 0.0699110399631224*G0_2_0 + 0.139822079926245*G0_2_1;
    A[461] = -0.0574176057031955*G0_0_0 - 0.0699110399631228*G0_0_1 - 0.0387263842124948*G0_2_0 - 0.108637424175617*G0_2_1;
    A[462] = 0.028372817543824*G0_0_0 + 0.059916292555181*G0_0_1 + 0.0189782768992836*G0_2_0 + 0.0788945694544646*G0_2_1;
    A[463] = -0.00537576492437595*G0_0_0 - 0.027459459425605*G0_0_1 - 0.00537576492437602*G0_2_0 - 0.032835224349981*G0_2_1;
    A[464] = 0.0107515298487519*G0_0_0 + 0.0107515298487518*G0_0_1 + 0.00537576492437581*G0_0_2 + 0.00537576492437558*G0_2_0 + 0.00537576492437554*G0_2_1;
    A[465] = 0.0139743792087542*G0_0_0 + 0.0139743792087544*G0_0_1 + 0.00228991928210715*G0_0_2 + 0.00228991928210729*G0_2_0 + 0.0022899192821074*G0_2_1;
    A[466] = 0.0111690649885092*G0_0_0 + 0.0111690649885091*G0_0_1 + 0.00228991928210631*G0_0_2 + 0.00228991928210626*G0_2_0 + 0.00228991928210622*G0_2_1;
    A[467] = 0.0328352243499812*G0_0_0 + 0.0328352243499812*G0_0_1 + 0.00537576492437638*G0_0_2 + 0.00537576492437601*G0_2_0 + 0.00537576492437601*G0_2_1;
    A[468] = 0.00537576492437596*G0_0_0 + 0.00537576492437596*G0_0_2 + 0.00537576492437583*G0_2_0 + 0.00537576492437578*G0_2_2;
    A[469] = 0.00228991928210665*G0_0_0 + 0.00228991928210672*G0_0_2 + 0.00228991928210683*G0_2_0 + 0.00228991928210691*G0_2_2;
    A[470] = 0.00228991928210664*G0_0_0 + 0.00228991928210664*G0_0_2 + 0.00228991928210663*G0_2_0 + 0.00228991928210663*G0_2_2;
    A[471] = 0.00537576492437595*G0_0_0 + 0.00537576492437596*G0_0_2 + 0.00537576492437603*G0_2_0 + 0.00537576492437603*G0_2_2;
    A[472] = 0.0220836945012288*G0_0_0 + 0.0274594594256049*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0274594594256047*G0_2_0 + 0.0328352243499805*G0_2_1 + 0.0328352243499804*G0_2_2;
    A[473] = -0.0315434750113564*G0_0_0 - 0.0599162925551807*G0_0_1 - 0.0599162925551806*G0_0_2 - 0.0599162925551803*G0_2_0 - 0.0788945694544637*G0_2_1 - 0.0788945694544636*G0_2_2;
    A[474] = 0.0124934342599269*G0_0_0 + 0.0699110399631222*G0_0_1 + 0.0699110399631223*G0_0_2 + 0.0699110399631218*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[475] = 0.0196371995417134*G0_0_0 - 0.0699110399631226*G0_0_1 - 0.0699110399631226*G0_0_2 - 0.0699110399631221*G0_2_0 - 0.139822079926245*G0_2_1 - 0.139822079926245*G0_2_2;
    A[476] = -0.0644569872000418*G0_0_0 + 0.192588083213082*G0_0_1 + 0.0962940416065409*G0_0_2 + 0.0284445813959699*G0_2_0 + 0.124738623002511*G0_2_1 + 0.12473862300251*G0_2_2;
    A[477] = 0.0439716694056967*G0_0_0 - 0.0845508658008649*G0_0_1 - 0.0422754329004329*G0_0_2 - 0.000685018588664527*G0_2_0 - 0.0429604514890973*G0_2_1 - 0.042960451489097*G0_2_2;
    A[478] = -0.0187890812890808*G0_0_0 + 0.0425363923627809*G0_0_1 + 0.0212681961813905*G0_0_2 - 0.00939454064454038*G0_2_0 + 0.0118736555368501*G0_2_1 + 0.0118736555368498*G0_2_2;
    A[479] = 0.127315597693201*G0_0_0 - 0.0493213383838383*G0_0_1 - 0.007045905483405*G0_0_2 - 0.000685018588664389*G0_2_0 - 0.0429604514890973*G0_2_1 - 0.0429604514890968*G0_2_2;
    A[480] = -0.0292600797158086*G0_0_0 + 0.0142875305635726*G0_0_1 + 0.00844856259352842*G0_0_2 - 0.00329461321214802*G0_2_0 + 0.00254435475789662*G0_2_1 + 0.00254435475789674*G0_2_2;
    A[481] = -0.0641307878721063*G0_0_0 + 0.0477555816097478*G0_0_1 + 0.0264873854283572*G0_0_2 - 0.00939454064454001*G0_2_0 + 0.0118736555368504*G0_2_1 + 0.0118736555368501*G0_2_2;
    A[482] = 0.0187890812890807*G0_0_0 + 0.00939454064454066*G0_0_2 + 0.00939454064454041*G0_2_0;
    A[483] = 0.00658922642429615*G0_0_0 + 0.00329461321214818*G0_0_2 + 0.00329461321214822*G0_2_0;
    A[484] = 0.0187890812890808*G0_0_0 + 0.00939454064454057*G0_0_2 + 0.00939454064454036*G0_2_0;
    A[485] = 0.0292600797158095*G0_0_0 + 0.00329461321214792*G0_0_2 + 0.00329461321214793*G0_2_0;
    A[486] = 0.0292600797158085*G0_0_0 + 0.00329461321214774*G0_0_2 + 0.00329461321214806*G0_2_0;
    A[487] = 0.0641307878721063*G0_0_0 + 0.00939454064454016*G0_0_2 + 0.00939454064453996*G0_2_0;
    A[488] = -0.0613254736518617*G0_0_0 - 0.0425363923627808*G0_0_1 - 0.0212681961813905*G0_0_2 - 0.0212681961813898*G0_2_0 - 0.0118736555368498*G0_2_1;
    A[489] = 0.128522535206561*G0_0_0 + 0.084550865800864*G0_0_1 + 0.0422754329004323*G0_0_2 + 0.042275432900432*G0_2_0 + 0.0429604514890962*G0_2_1;
    A[490] = -0.257045070413123*G0_0_0 - 0.192588083213081*G0_0_1 - 0.0962940416065406*G0_0_2 - 0.0962940416065404*G0_2_0 - 0.12473862300251*G0_2_1;
    A[491] = -0.0435476102793811*G0_0_0 - 0.0142875305635717*G0_0_1 - 0.00583896797004438*G0_0_2 - 0.00583896797004491*G0_2_0 - 0.00254435475789641*G0_2_1;
    A[492] = 0.176636936077039*G0_0_0 + 0.0493213383838384*G0_0_1 + 0.0422754329004332*G0_0_2 + 0.0422754329004327*G0_2_0 + 0.0429604514890972*G0_2_1;
    A[493] = -0.111886369481854*G0_0_0 - 0.047755581609748*G0_0_1 - 0.0212681961813907*G0_0_2 - 0.0212681961813901*G0_2_0 - 0.0118736555368505*G0_2_1;
    A[494] = -0.0306627368259309*G0_0_0 - 0.0212681961813907*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.01187365553685*G0_2_2;
    A[495] = 0.0642612676032805*G0_0_0 + 0.0422754329004317*G0_0_2 + 0.0422754329004324*G0_2_0 + 0.0429604514890963*G0_2_2;
    A[496] = -0.128522535206561*G0_0_0 - 0.0962940416065407*G0_0_2 - 0.0962940416065402*G0_2_0 - 0.12473862300251*G0_2_2;
    A[497] = -0.00913358118219213*G0_0_0 - 0.00583896797004434*G0_0_2 - 0.0058389679700442*G0_2_0 - 0.00254435475789614*G0_2_2;
    A[498] = 0.0642612676032809*G0_0_0 + 0.0422754329004327*G0_0_2 + 0.0422754329004322*G0_2_0 + 0.0429604514890967*G0_2_2;
    A[499] = -0.0306627368259308*G0_0_0 - 0.0212681961813905*G0_0_2 - 0.0212681961813902*G0_2_0 - 0.0118736555368497*G0_2_2;
    A[500] = -0.050560895829993*G0_0_0 - 0.00260959462348271*G0_0_2 - 0.00260959462348375*G0_2_0;
    A[501] = 0.0962288017409542*G0_0_0 - 0.035229527417027*G0_0_2 - 0.0352295274170265*G0_2_0;
    A[502] = -0.0505608958299927*G0_0_0 - 0.0026095946234839*G0_0_2 - 0.00260959462348371*G0_2_0;
    A[503] = -0.15657567740901*G0_0_0 - 0.00260959462348372*G0_0_2 - 0.00260959462348357*G0_2_0;
    A[504] = -0.00336507226698201*G0_0_0 - 0.00336507226698206*G0_0_1 - 0.00336507226698204*G0_0_2 - 0.00336507226698211*G0_2_0 - 0.00336507226698215*G0_2_1 - 0.00336507226698211*G0_2_2;
    A[505] = 0.0214691349673987*G0_0_0 - 0.0336585514536898*G0_2_0;
    A[506] = 0.00336507226698202*G0_0_1 + 0.00336507226698196*G0_2_1;
    A[507] = 0.0232632312710439*G0_0_2 - 0.0155544887532732*G0_2_2;
    A[508] = -0.0139743792087545*G0_0_1 - 0.00228991928210669*G0_0_2 - 0.0116844599266474*G0_2_1 - 0.00228991928210653*G0_2_2;
    A[509] = 0.0263047138047141*G0_0_1 + 0.0131523569023563*G0_0_2 + 0.0131523569023568*G0_2_1 + 0.0132175967679433*G0_2_2;
    A[510] = 0.00523223722008358*G0_0_1 + 0.00578025209101647*G0_0_2 - 0.000548014870931732*G0_2_1 - 0.000482775005343812*G0_2_2;
    A[511] = 0.0788945694544651*G0_0_1 + 0.0599162925551803*G0_0_2 + 0.0189782768992838*G0_2_1 + 0.0283728175438238*G0_2_2;
    A[512] = -0.147063705006412*G0_0_0 - 0.0387263842124952*G0_0_2 + 0.0263830016434181*G0_2_0 - 0.108637424175617*G0_2_2;
    A[513] = 0.304226541205706*G0_0_0 + 0.0781312630270959*G0_0_2 + 0.0781312630270959*G0_2_0 + 0.281001149056701*G0_2_2;
    A[514] = -0.204187731314466*G0_0_0 + 0.0153052724667307*G0_0_2 - 0.0364951808094168*G0_2_0 - 0.204187731314465*G0_2_2;
    A[515] = 0.0788945694544653*G0_0_0 - 0.0246345732456843*G0_0_2 + 0.0189782768992838*G0_2_0 + 0.100717304493345*G0_2_2;
    A[516] = -0.0574176057031954*G0_0_0 - 0.0387263842124951*G0_0_1 - 0.0699110399631223*G0_2_0 - 0.108637424175617*G0_2_1;
    A[517] = 0.00506261356955813*G0_0_0 + 0.0048016541072101*G0_0_1 + 0.00480165410720939*G0_2_0 + 0.00960330821441913*G0_2_1;
    A[518] = -0.000482775005344182*G0_0_0 + 0.0057802520910159*G0_0_1 - 0.000548014870931333*G0_2_0 + 0.00523223722008425*G0_2_1;
    A[519] = -0.00228991928210671*G0_0_0 - 0.00887914570640247*G0_0_1 - 0.00228991928210661*G0_2_0 - 0.011169064988509*G0_2_1;
    A[520] = 0.0139743792087542*G0_0_0 + 0.0139743792087543*G0_0_1 + 0.0116844599266477*G0_0_2 + 0.0116844599266479*G0_2_0 + 0.0116844599266479*G0_2_1 + 0.00939454064454121*G0_2_2;
    A[521] = -0.0263047138047139*G0_0_0 - 0.0263047138047142*G0_0_1 - 0.0131523569023576*G0_0_2 - 0.0131523569023576*G0_2_0 - 0.0131523569023577*G0_2_1 + 6.52398655859062e-05*G0_2_2;
    A[522] = -0.0052322372200841*G0_0_0 - 0.00523223722008398*G0_0_1 + 0.000548014870932427*G0_0_2 + 0.000548014870932239*G0_2_0 + 0.00054801487093224*G0_2_1 + 6.52398655884127e-05*G0_2_2;
    A[523] = -0.0788945694544651*G0_0_0 - 0.0788945694544652*G0_0_1 - 0.0189782768992844*G0_0_2 - 0.0189782768992837*G0_2_0 - 0.0189782768992837*G0_2_1 + 0.00939454064453983*G0_2_2;
    A[524] = 0.00228991928210677*G0_0_0 + 0.00228991928210679*G0_0_2 + 0.00228991928210699*G0_2_0 + 0.0022899192821071*G0_2_2;
    A[525] = 0.00178757231708632*G0_0_0 + 0.00178757231708621*G0_0_2 + 0.00178757231708602*G0_2_0 + 0.00178757231708592*G0_2_2;
    A[526] = 0.0017875723170862*G0_0_0 + 0.00178757231708617*G0_0_2 + 0.00178757231708636*G0_2_0 + 0.00178757231708633*G0_2_2;
    A[527] = 0.00228991928210675*G0_0_0 + 0.00228991928210672*G0_0_2 + 0.00228991928210663*G0_2_0 + 0.00228991928210662*G0_2_2;
    A[528] = 0.00658922642429587*G0_0_0 + 0.00887914570640254*G0_0_1 + 0.00887914570640248*G0_0_2 + 0.00887914570640286*G0_2_0 + 0.0111690649885097*G0_2_1 + 0.0111690649885097*G0_2_2;
    A[529] = -0.00626302709636014*G0_0_0 - 0.00578025209101598*G0_0_1 - 0.00578025209101588*G0_0_2 - 0.0057802520910162*G0_2_0 - 0.00523223722008508*G0_2_1 - 0.00523223722008489*G0_2_2;
    A[530] = 0.00026095946234813*G0_0_0 - 0.00480165410720958*G0_0_1 - 0.00480165410720961*G0_0_2 - 0.00480165410720925*G0_2_0 - 0.00960330821441847*G0_2_1 - 0.00960330821441865*G0_2_2;
    A[531] = -0.0186912214907004*G0_0_0 + 0.0387263842124952*G0_0_1 + 0.0387263842124951*G0_0_2 + 0.0387263842124946*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[532] = 0.132306447410613*G0_0_0 + 0.0109602974186307*G0_0_1 - 0.0623693115012549*G0_0_2 + 0.0733296089198872*G0_2_0 + 0.146659217839773*G0_2_1 - 0.12473862300251*G0_2_2;
    A[533] = -0.0436454700777609*G0_0_0 - 0.0808321934624008*G0_0_1 - 0.00649136662591446*G0_0_2 - 0.0338921101724915*G0_2_0 - 0.108232937008976*G0_2_1 - 0.00352295274170262*G0_2_2;
    A[534] = 0.0148746893538551*G0_0_0 + 0.0403182369328199*G0_0_1 + 0.00123955744615471*G0_0_2 + 0.0149399292194424*G0_2_0 + 0.0540186087061076*G0_2_1 + 0.00117431758056771*G0_2_2;
    A[535] = -0.166948816037357*G0_0_0 + 0.167731694424401*G0_0_1 + 0.0838658472122001*G0_0_2 - 0.00616516729798015*G0_2_0 + 0.077700679914221*G0_2_1 + 0.182410664181496*G0_2_2;
    A[536] = 0.0189848008858425*G0_0_0 - 0.0377738821749243*G0_0_1 - 0.0188869410874625*G0_0_2 + 0.00949240044292149*G0_2_0 - 0.00939454064454089*G0_2_1 - 0.0155923278753142*G0_2_2;
    A[537] = 0.127087258163645*G0_0_0 - 0.0823327103709043*G0_0_1 - 0.0463855444324192*G0_0_2 + 0.0377738821749225*G0_2_0 + 0.00182671623643729*G0_2_1 - 0.0510175748891033*G0_2_2;
    A[538] = -0.014874689353856*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194433*G0_2_0 - 0.0150051690850308*G0_2_2;
    A[539] = 0.0045015507255088*G0_0_0 + 0.00192457603481889*G0_0_2 + 0.0019245760348188*G0_2_0 - 0.000652398655870705*G0_2_2;
    A[540] = -0.0148746893538552*G0_0_0 - 0.0149399292194424*G0_0_2 - 0.0149399292194424*G0_2_0 - 0.0150051690850296*G0_2_2;
    A[541] = -0.0189848008858432*G0_0_0 - 0.00949240044292139*G0_0_2 - 0.00949240044292092*G0_2_0 - 0.000652398655869979*G0_2_2;
    A[542] = -0.0189848008858423*G0_0_0 - 0.0094924004429216*G0_0_2 - 0.00949240044292138*G0_2_0 - 0.000652398655871498*G0_2_2;
    A[543] = -0.127087258163645*G0_0_0 - 0.0377738821749228*G0_0_2 - 0.0377738821749225*G0_2_0 - 0.01500516908503*G0_2_2;
    A[544] = -0.0254435475789644*G0_0_0 - 0.0403182369328204*G0_0_1 - 0.0390786794866652*G0_0_2 - 0.039078679486666*G0_2_0 - 0.0540186087061084*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[545] = 0.0371867233846397*G0_0_0 + 0.0808321934624013*G0_0_1 + 0.0743408268364854*G0_0_2 + 0.0743408268364858*G0_2_0 + 0.108232937008978*G0_2_1 + 0.104709984267275*G0_2_2;
    A[546] = 0.121346149991983*G0_0_0 - 0.0109602974186309*G0_0_1 - 0.073329608919886*G0_0_2 - 0.0733296089198859*G0_2_0 - 0.146659217839773*G0_2_1 - 0.271397840842283*G0_2_2;
    A[547] = 0.0567586830607669*G0_0_0 + 0.0377738821749237*G0_0_1 + 0.0188869410874624*G0_0_2 + 0.018886941087463*G0_2_0 + 0.00939454064454107*G0_2_1 - 0.00619778723077244*G0_2_2;
    A[548] = -0.334680510461758*G0_0_0 - 0.167731694424402*G0_0_1 - 0.0838658472122007*G0_0_2 - 0.0838658472122011*G0_2_0 - 0.0777006799142209*G0_2_1 + 0.104709984267276*G0_2_2;
    A[549] = 0.20941996853455*G0_0_0 + 0.0823327103709042*G0_0_1 + 0.0359471659384847*G0_0_2 + 0.0359471659384844*G0_2_0 - 0.0018267162364375*G0_2_1 - 0.052844291125541*G0_2_2;
    A[550] = -0.00130479731174216*G0_0_0 - 0.00123955744615504*G0_0_2 - 0.001239557446155*G0_2_0 - 0.00117431758056783*G0_2_2;
    A[551] = 0.00880738185425737*G0_0_0 + 0.00649136662591579*G0_0_2 + 0.00649136662591535*G0_2_0 + 0.00352295274170323*G0_2_2;
    A[552] = 0.0476251018785734*G0_0_0 + 0.0623693115012547*G0_0_2 + 0.0623693115012549*G0_2_0 + 0.124738623002509*G0_2_2;
    A[553] = -0.0156575677409006*G0_0_0 - 0.0130805930502107*G0_0_2 - 0.0130805930502107*G0_2_0 - 0.0105036183595211*G0_2_2;
    A[554] = 0.00880738185425628*G0_0_0 + 0.00649136662591455*G0_0_2 + 0.00649136662591539*G0_2_0 + 0.00352295274170293*G0_2_2;
    A[555] = -0.00130479731174196*G0_0_0 - 0.00123955744615481*G0_0_2 - 0.00123955744615503*G0_2_0 - 0.00117431758056795*G0_2_2;
    A[556] = 0.0391439193522527*G0_0_0 + 0.0319675341376723*G0_0_2 + 0.031967534137673*G0_2_0 + 0.0260959462348343*G0_2_2;
    A[557] = -0.15657567740901*G0_0_0 - 0.0903572138381156*G0_0_2 - 0.090357213838117*G0_2_0 - 0.185933616923199*G0_2_2;
    A[558] = 0.0391439193522519*G0_0_0 + 0.0319675341376732*G0_0_2 + 0.0319675341376727*G0_2_0 + 0.0260959462348356*G0_2_2;
    A[559] = 0.1859336169232*G0_0_0 + 0.015657567740902*G0_0_2 + 0.0156575677409011*G0_2_0 + 0.0260959462348363*G0_2_2;
    A[560] = -0.00336507226698197*G0_0_0 - 0.00336507226698195*G0_0_1 - 0.00336507226698195*G0_0_2 - 0.00336507226698183*G0_2_0 - 0.00336507226698183*G0_2_1 - 0.00336507226698183*G0_2_2;
    A[561] = -0.0155544887532734*G0_0_0 + 0.0232632312710434*G0_2_0;
    A[562] = 0.00336507226698193*G0_0_1 + 0.00336507226698196*G0_2_1;
    A[563] = -0.0336585514536904*G0_0_2 + 0.0214691349673987*G0_2_2;
    A[564] = -0.0111690649885091*G0_0_1 - 0.00228991928210687*G0_0_2 - 0.00887914570640251*G0_2_1 - 0.00228991928210696*G0_2_2;
    A[565] = 0.00523223722008409*G0_0_1 - 0.000548014870930907*G0_0_2 + 0.00578025209101594*G0_2_1 - 0.000482775005343927*G0_2_2;
    A[566] = 0.00960330821441976*G0_0_1 + 0.00480165410720898*G0_0_2 + 0.00480165410720975*G0_2_1 + 0.00506261356955755*G0_2_2;
    A[567] = -0.108637424175618*G0_0_1 - 0.0699110399631222*G0_0_2 - 0.038726384212495*G0_2_1 - 0.0574176057031954*G0_2_2;
    A[568] = 0.100717304493345*G0_0_0 + 0.0189782768992838*G0_0_2 - 0.0246345732456844*G0_2_0 + 0.0788945694544639*G0_2_2;
    A[569] = -0.204187731314466*G0_0_0 - 0.0364951808094168*G0_0_2 + 0.0153052724667307*G0_2_0 - 0.204187731314465*G0_2_2;
    A[570] = 0.281001149056703*G0_0_0 + 0.078131263027096*G0_0_2 + 0.078131263027096*G0_2_0 + 0.304226541205706*G0_2_2;
    A[571] = -0.108637424175618*G0_0_0 + 0.0263830016434183*G0_0_2 - 0.038726384212495*G0_2_0 - 0.147063705006412*G0_2_2;
    A[572] = 0.0283728175438242*G0_0_0 + 0.0189782768992838*G0_0_1 + 0.0599162925551805*G0_2_0 + 0.078894569454464*G0_2_1;
    A[573] = -0.000482775005344494*G0_0_0 - 0.000548014870931662*G0_0_1 + 0.00578025209101624*G0_2_0 + 0.00523223722008472*G0_2_1;
    A[574] = 0.0132175967679437*G0_0_0 + 0.0131523569023567*G0_0_1 + 0.0131523569023566*G0_2_0 + 0.0263047138047136*G0_2_1;
    A[575] = -0.00228991928210686*G0_0_0 - 0.0116844599266474*G0_0_1 - 0.00228991928210687*G0_2_0 - 0.0139743792087543*G0_2_1;
    A[576] = 0.0111690649885093*G0_0_0 + 0.0111690649885092*G0_0_1 + 0.00887914570640247*G0_0_2 + 0.00887914570640199*G0_2_0 + 0.00887914570640198*G0_2_1 + 0.00658922642429536*G0_2_2;
    A[577] = -0.00523223722008431*G0_0_0 - 0.00523223722008405*G0_0_1 - 0.00578025209101565*G0_0_2 - 0.00578025209101524*G0_2_0 - 0.00578025209101509*G0_2_1 - 0.00626302709635969*G0_2_2;
    A[578] = -0.00960330821441943*G0_0_0 - 0.00960330821441959*G0_0_1 - 0.00480165410721021*G0_0_2 - 0.00480165410721007*G0_2_0 - 0.00480165410721013*G0_2_1 + 0.000260959462347501*G0_2_2;
    A[579] = 0.108637424175618*G0_0_0 + 0.108637424175618*G0_0_1 + 0.0387263842124957*G0_0_2 + 0.038726384212495*G0_2_0 + 0.038726384212495*G0_2_1 - 0.0186912214907001*G0_2_2;
    A[580] = 0.00228991928210653*G0_0_0 + 0.00228991928210652*G0_0_2 + 0.0022899192821063*G0_2_0 + 0.0022899192821062*G0_2_2;
    A[581] = 0.00178757231708649*G0_0_0 + 0.00178757231708659*G0_0_2 + 0.00178757231708656*G0_2_0 + 0.00178757231708666*G0_2_2;
    A[582] = 0.00178757231708601*G0_0_0 + 0.00178757231708604*G0_0_2 + 0.00178757231708596*G0_2_0 + 0.001787572317086*G0_2_2;
    A[583] = 0.00228991928210685*G0_0_0 + 0.00228991928210686*G0_0_2 + 0.00228991928210687*G0_2_0 + 0.00228991928210685*G0_2_2;
    A[584] = 0.00939454064454063*G0_0_0 + 0.0116844599266473*G0_0_1 + 0.0116844599266474*G0_0_2 + 0.011684459926647*G0_2_0 + 0.0139743792087536*G0_2_1 + 0.0139743792087536*G0_2_2;
    A[585] = 6.52398655867033e-05*G0_0_0 - 0.0131523569023569*G0_0_1 - 0.013152356902357*G0_0_2 - 0.0131523569023566*G0_2_0 - 0.0263047138047131*G0_2_1 - 0.0263047138047133*G0_2_2;
    A[586] = 6.52398655875006e-05*G0_0_0 + 0.000548014870931564*G0_0_1 + 0.000548014870931576*G0_0_2 + 0.000548014870931279*G0_2_0 - 0.00523223722008506*G0_2_1 - 0.00523223722008491*G0_2_2;
    A[587] = 0.00939454064454037*G0_0_0 - 0.0189782768992838*G0_0_1 - 0.0189782768992837*G0_0_2 - 0.0189782768992834*G0_2_0 - 0.078894569454464*G0_2_1 - 0.078894569454464*G0_2_2;
    A[588] = -0.0510175748891028*G0_0_0 + 0.00182671623643803*G0_0_1 + 0.0377738821749231*G0_0_2 - 0.0463855444324205*G0_2_0 - 0.0823327103709051*G0_2_1 + 0.127087258163645*G0_2_2;
    A[589] = -0.0155923278753142*G0_0_0 - 0.00939454064454092*G0_0_1 + 0.00949240044292071*G0_0_2 - 0.0188869410874623*G0_2_0 - 0.037773882174925*G0_2_1 + 0.0189848008858423*G0_2_2;
    A[590] = 0.00117431758056857*G0_0_0 + 0.0540186087061089*G0_0_1 + 0.0149399292194432*G0_0_2 + 0.00123955744615539*G0_2_0 + 0.0403182369328207*G0_2_1 + 0.0148746893538561*G0_2_2;
    A[591] = 0.182410664181497*G0_0_0 + 0.0777006799142212*G0_0_1 - 0.00616516729797981*G0_0_2 + 0.0838658472122006*G0_2_0 + 0.167731694424401*G0_2_1 - 0.166948816037357*G0_2_2;
    A[592] = -0.00352295274170297*G0_0_0 - 0.108232937008978*G0_0_1 - 0.0338921101724914*G0_0_2 - 0.00649136662591537*G0_2_0 - 0.0808321934624011*G0_2_1 - 0.0436454700777616*G0_2_2;
    A[593] = -0.12473862300251*G0_0_0 + 0.146659217839773*G0_0_1 + 0.0733296089198868*G0_0_2 - 0.0623693115012544*G0_2_0 + 0.0109602974186318*G0_2_1 + 0.132306447410615*G0_2_2;
    A[594] = -0.00117431758056711*G0_0_0 - 0.0012395574461542*G0_0_2 - 0.00123955744615417*G0_2_0 - 0.00130479731174049*G0_2_2;
    A[595] = -0.0105036183595208*G0_0_0 - 0.0130805930502114*G0_0_2 - 0.0130805930502108*G0_2_0 - 0.0156575677409015*G0_2_2;
    A[596] = -0.00117431758056849*G0_0_0 - 0.00123955744615499*G0_0_2 - 0.00123955744615534*G0_2_0 - 0.00130479731174186*G0_2_2;
    A[597] = 0.0035229527417036*G0_0_0 + 0.00649136662591518*G0_0_2 + 0.00649136662591534*G0_2_0 + 0.00880738185425566*G0_2_2;
    A[598] = 0.00352295274170271*G0_0_0 + 0.00649136662591559*G0_0_2 + 0.00649136662591519*G0_2_0 + 0.00880738185425708*G0_2_2;
    A[599] = 0.12473862300251*G0_0_0 + 0.062369311501255*G0_0_2 + 0.0623693115012546*G0_2_0 + 0.047625101878574*G0_2_2;
    A[600] = -0.0528442911255406*G0_0_0 - 0.0540186087061082*G0_0_1 - 0.0390786794866654*G0_0_2 - 0.0390786794866646*G0_2_0 - 0.0403182369328199*G0_2_1 - 0.0254435475789635*G0_2_2;
    A[601] = -0.00619778723077332*G0_0_0 + 0.0093945406445398*G0_0_1 + 0.018886941087462*G0_0_2 + 0.0188869410874617*G0_2_0 + 0.0377738821749229*G0_2_1 + 0.056758683060766*G0_2_2;
    A[602] = -0.0528442911255406*G0_0_0 - 0.00182671623643807*G0_0_1 + 0.0359471659384852*G0_0_2 + 0.0359471659384854*G0_2_0 + 0.0823327103709055*G0_2_1 + 0.20941996853455*G0_2_2;
    A[603] = 0.104709984267275*G0_0_0 + 0.108232937008978*G0_0_1 + 0.074340826836486*G0_0_2 + 0.074340826836485*G0_2_0 + 0.0808321934624013*G0_2_1 + 0.0371867233846392*G0_2_2;
    A[604] = 0.104709984267275*G0_0_0 - 0.0777006799142209*G0_0_1 - 0.0838658472122008*G0_0_2 - 0.0838658472122006*G0_2_0 - 0.167731694424401*G0_2_1 - 0.334680510461758*G0_2_2;
    A[605] = -0.271397840842284*G0_0_0 - 0.146659217839773*G0_0_1 - 0.0733296089198864*G0_0_2 - 0.0733296089198858*G0_2_0 - 0.0109602974186317*G0_2_1 + 0.121346149991983*G0_2_2;
    A[606] = -0.0150051690850296*G0_0_0 - 0.0149399292194427*G0_0_2 - 0.0149399292194425*G0_2_0 - 0.0148746893538559*G0_2_2;
    A[607] = -0.000652398655871397*G0_0_0 - 0.00949240044292201*G0_0_2 - 0.00949240044292144*G0_2_0 - 0.0189848008858434*G0_2_2;
    A[608] = -0.0150051690850297*G0_0_0 - 0.0377738821749232*G0_0_2 - 0.0377738821749228*G0_2_0 - 0.127087258163644*G0_2_2;
    A[609] = -0.000652398655871467*G0_0_0 + 0.00192457603481913*G0_0_2 + 0.00192457603481872*G0_2_0 + 0.00450155072550923*G0_2_2;
    A[610] = -0.000652398655870626*G0_0_0 - 0.00949240044292072*G0_0_2 - 0.0094924004429215*G0_2_0 - 0.0189848008858425*G0_2_2;
    A[611] = -0.0150051690850298*G0_0_0 - 0.0149399292194432*G0_0_2 - 0.0149399292194426*G0_2_0 - 0.0148746893538559*G0_2_2;
    A[612] = 0.0260959462348346*G0_0_0 + 0.0319675341376738*G0_0_2 + 0.0319675341376725*G0_2_0 + 0.0391439193522539*G0_2_2;
    A[613] = 0.0260959462348356*G0_0_0 + 0.0156575677409018*G0_0_2 + 0.0156575677409022*G0_2_0 + 0.1859336169232*G0_2_2;
    A[614] = 0.0260959462348352*G0_0_0 + 0.0319675341376723*G0_0_2 + 0.0319675341376731*G0_2_0 + 0.039143919352252*G0_2_2;
    A[615] = -0.1859336169232*G0_0_0 - 0.090357213838118*G0_0_2 - 0.090357213838116*G0_2_0 - 0.156575677409012*G0_2_2;
    A[616] = -0.00480034930989792*G0_0_0 - 0.00480034930989795*G0_0_1 - 0.00480034930989795*G0_0_2 - 0.00480034930989801*G0_2_0 - 0.00480034930989802*G0_2_1 - 0.00480034930989801*G0_2_2;
    A[617] = 0.00480034930989788*G0_0_0 - 0.0105310191030676*G0_2_0;
    A[618] = 0.00480034930989788*G0_0_1 + 0.00480034930989786*G0_2_1;
    A[619] = 0.0372571824394741*G0_0_2 - 0.017870503981615*G0_2_2;
    A[620] = -0.0328352243499814*G0_0_1 - 0.00537576492437604*G0_0_2 - 0.0274594594256052*G0_2_1 - 0.00537576492437603*G0_2_2;
    A[621] = 0.078894569454465*G0_0_1 + 0.0189782768992833*G0_0_2 + 0.059916292555181*G0_2_1 + 0.0283728175438241*G0_2_2;
    A[622] = -0.108637424175618*G0_0_1 - 0.0387263842124945*G0_0_2 - 0.0699110399631229*G0_2_1 - 0.0574176057031954*G0_2_2;
    A[623] = 0.139822079926246*G0_0_1 + 0.0699110399631224*G0_0_2 + 0.0699110399631226*G0_2_1 + 0.089548239504836*G0_2_2;
    A[624] = -0.0328352243499813*G0_0_0 - 0.00537576492437602*G0_0_2 + 0.0209159009072204*G0_2_0 - 0.0328352243499805*G0_2_2;
    A[625] = 0.0788945694544653*G0_0_0 + 0.0189782768992838*G0_0_2 - 0.0246345732456843*G0_2_0 + 0.100717304493345*G0_2_2;
    A[626] = -0.108637424175618*G0_0_0 - 0.038726384212495*G0_0_2 + 0.0263830016434183*G0_2_0 - 0.147063705006412*G0_2_2;
    A[627] = 0.139822079926246*G0_0_0 + 0.0699110399631227*G0_0_2 + 0.0699110399631227*G0_2_0 + 0.179096479009672*G0_2_2;
    A[628] = -0.00537576492437611*G0_0_0 - 0.00537576492437605*G0_0_1 - 0.0274594594256048*G0_2_0 - 0.0328352243499806*G0_2_1;
    A[629] = -0.00228991928210666*G0_0_0 - 0.00228991928210679*G0_0_1 - 0.00887914570640271*G0_2_0 - 0.0111690649885096*G0_2_1;
    A[630] = -0.00228991928210669*G0_0_0 - 0.00228991928210662*G0_0_1 - 0.0116844599266472*G0_2_0 - 0.013974379208754*G0_2_1;
    A[631] = -0.00537576492437601*G0_0_0 - 0.00537576492437615*G0_0_1 - 0.00537576492437606*G0_2_0 - 0.0107515298487521*G0_2_1;
    A[632] = 0.0328352243499813*G0_0_0 + 0.0328352243499814*G0_0_1 + 0.0274594594256053*G0_0_2 + 0.0274594594256054*G0_2_0 + 0.0274594594256055*G0_2_1 + 0.0220836945012295*G0_2_2;
    A[633] = -0.0788945694544652*G0_0_0 - 0.0788945694544654*G0_0_1 - 0.0599162925551816*G0_0_2 - 0.0599162925551816*G0_2_0 - 0.0599162925551817*G0_2_1 - 0.0315434750113575*G0_2_2;
    A[634] = 0.108637424175618*G0_0_0 + 0.108637424175618*G0_0_1 + 0.0699110399631237*G0_0_2 + 0.0699110399631231*G0_2_0 + 0.0699110399631231*G0_2_1 + 0.012493434259928*G0_2_2;
    A[635] = -0.139822079926246*G0_0_0 - 0.139822079926246*G0_0_1 - 0.0699110399631235*G0_0_2 - 0.0699110399631227*G0_2_0 - 0.0699110399631227*G0_2_1 + 0.019637199541713*G0_2_2;
    A[636] = 0.00537576492437614*G0_0_0 + 0.00537576492437614*G0_0_2 + 0.00537576492437606*G0_2_0 + 0.00537576492437611*G0_2_2;
    A[637] = 0.00228991928210676*G0_0_0 + 0.00228991928210667*G0_0_2 + 0.00228991928210683*G0_2_0 + 0.00228991928210673*G0_2_2;
    A[638] = 0.00228991928210668*G0_0_0 + 0.00228991928210669*G0_0_2 + 0.00228991928210666*G0_2_0 + 0.00228991928210665*G0_2_2;
    A[639] = 0.00537576492437604*G0_0_0 + 0.00537576492437606*G0_0_2 + 0.00537576492437609*G0_2_0 + 0.00537576492437612*G0_2_2;
    A[640] = 0.00537576492437608*G0_0_1 + 0.00537576492437602*G0_0_2 + 0.00537576492437622*G0_2_0 + 0.0107515298487522*G0_2_1 + 0.0107515298487522*G0_2_2;
    A[641] = 0.00228991928210692*G0_0_1 + 0.00228991928210697*G0_0_2 + 0.00228991928210667*G0_2_0 + 0.0139743792087538*G0_2_1 + 0.0139743792087539*G0_2_2;
    A[642] = 0.00228991928210661*G0_0_1 + 0.00228991928210663*G0_0_2 + 0.00228991928210685*G0_2_0 + 0.0111690649885097*G0_2_1 + 0.0111690649885096*G0_2_2;
    A[643] = 0.00537576492437611*G0_0_1 + 0.00537576492437605*G0_0_2 + 0.00537576492437583*G0_2_0 + 0.0328352243499807*G0_2_1 + 0.0328352243499807*G0_2_2;
    A[644] = 0.0118736555368503*G0_0_0 + 0.01187365553685*G0_0_1 - 0.00939454064454059*G0_0_2 + 0.0264873854283582*G0_2_0 + 0.0477555816097485*G0_2_1 - 0.0641307878721062*G0_2_2;
    A[645] = 0.00254435475789672*G0_0_0 + 0.00254435475789678*G0_0_1 - 0.00329461321214734*G0_0_2 + 0.00844856259352856*G0_2_0 + 0.0142875305635733*G0_2_1 - 0.0292600797158082*G0_2_2;
    A[646] = 0.0118736555368489*G0_0_0 + 0.0118736555368494*G0_0_1 - 0.00939454064454121*G0_0_2 + 0.0212681961813897*G0_2_0 + 0.0425363923627806*G0_2_1 - 0.0187890812890817*G0_2_2;
    A[647] = -0.0429604514890975*G0_0_0 - 0.0429604514890969*G0_0_1 - 0.000685018588664238*G0_0_2 - 0.00704590548340547*G0_2_0 - 0.049321338383838*G0_2_1 + 0.127315597693202*G0_2_2;
    A[648] = -0.0429604514890968*G0_0_0 - 0.0429604514890975*G0_0_1 - 0.000685018588664616*G0_0_2 - 0.0422754329004325*G0_2_0 - 0.0845508658008656*G0_2_1 + 0.0439716694056973*G0_2_2;
    A[649] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0284445813959694*G0_0_2 + 0.0962940416065404*G0_2_0 + 0.192588083213081*G0_2_1 - 0.0644569872000433*G0_2_2;
    A[650] = -0.0118736555368506*G0_0_0 - 0.021268196181391*G0_0_2 - 0.0212681961813912*G0_2_0 - 0.0306627368259315*G0_2_2;
    A[651] = -0.00254435475789629*G0_0_0 - 0.00583896797004384*G0_0_2 - 0.00583896797004422*G0_2_0 - 0.00913358118219194*G0_2_2;
    A[652] = -0.0118736555368489*G0_0_0 - 0.0212681961813901*G0_0_2 - 0.0212681961813898*G0_2_0 - 0.0306627368259308*G0_2_2;
    A[653] = 0.0429604514890965*G0_0_0 + 0.0422754329004328*G0_0_2 + 0.0422754329004326*G0_2_0 + 0.0642612676032815*G0_2_2;
    A[654] = 0.0429604514890969*G0_0_0 + 0.0422754329004319*G0_0_2 + 0.0422754329004326*G0_2_0 + 0.0642612676032807*G0_2_2;
    A[655] = -0.124738623002511*G0_0_0 - 0.0962940416065405*G0_0_2 - 0.0962940416065403*G0_2_0 - 0.128522535206562*G0_2_2;
    A[656] = -0.0118736555368504*G0_0_1 - 0.0212681961813909*G0_0_2 - 0.0212681961813913*G0_2_0 - 0.0425363923627817*G0_2_1 - 0.0613254736518626*G0_2_2;
    A[657] = -0.00254435475789572*G0_0_1 - 0.00583896797004435*G0_0_2 - 0.00583896797004399*G0_2_0 - 0.0142875305635717*G0_2_1 - 0.0435476102793808*G0_2_2;
    A[658] = -0.0118736555368501*G0_0_1 - 0.0212681961813904*G0_0_2 - 0.0212681961813907*G0_2_0 - 0.0477555816097487*G0_2_1 - 0.111886369481855*G0_2_2;
    A[659] = 0.0429604514890968*G0_0_1 + 0.0422754329004335*G0_0_2 + 0.0422754329004337*G0_2_0 + 0.0845508658008655*G0_2_1 + 0.128522535206563*G0_2_2;
    A[660] = 0.0429604514890968*G0_0_1 + 0.0422754329004323*G0_0_2 + 0.0422754329004323*G0_2_0 + 0.049321338383838*G0_2_1 + 0.176636936077039*G0_2_2;
    A[661] = -0.124738623002511*G0_0_1 - 0.0962940416065412*G0_0_2 - 0.0962940416065416*G0_2_0 - 0.192588083213081*G0_2_1 - 0.257045070413125*G0_2_2;
    A[662] = 0.00939454064454023*G0_0_2 + 0.00939454064454033*G0_2_0 + 0.0187890812890806*G0_2_2;
    A[663] = 0.00329461321214819*G0_0_2 + 0.00329461321214819*G0_2_0 + 0.0292600797158091*G0_2_2;
    A[664] = 0.00939454064454054*G0_0_2 + 0.00939454064454019*G0_2_0 + 0.0641307878721059*G0_2_2;
    A[665] = 0.00329461321214795*G0_0_2 + 0.00329461321214794*G0_2_0 + 0.00658922642429593*G0_2_2;
    A[666] = 0.00329461321214749*G0_0_2 + 0.00329461321214798*G0_2_0 + 0.0292600797158084*G0_2_2;
    A[667] = 0.009394540644541*G0_0_2 + 0.00939454064454058*G0_2_0 + 0.0187890812890814*G0_2_2;
    A[668] = -0.00260959462348357*G0_0_2 - 0.00260959462348279*G0_2_0 - 0.0505608958299926*G0_2_2;
    A[669] = -0.00260959462348384*G0_0_2 - 0.00260959462348428*G0_2_0 - 0.15657567740901*G0_2_2;
    A[670] = -0.00260959462348319*G0_0_2 - 0.00260959462348432*G0_2_0 - 0.0505608958299927*G0_2_2;
    A[671] = -0.0352295274170262*G0_0_2 - 0.0352295274170272*G0_2_0 + 0.0962288017409546*G0_2_2;
    A[672] = -0.00480034930989774*G0_0_0 - 0.00480034930989772*G0_0_1 - 0.00480034930989772*G0_0_2 - 0.00480034930989779*G0_1_0 - 0.00480034930989776*G0_1_1 - 0.00480034930989778*G0_1_2;
    A[673] = -0.0178705039816151*G0_0_0 + 0.0372571824394735*G0_1_0;
    A[674] = -0.0105310191030676*G0_0_1 + 0.00480034930989784*G0_1_1;
    A[675] = 0.00480034930989775*G0_0_2 + 0.00480034930989773*G0_1_2;
    A[676] = -0.0274594594256049*G0_0_1 - 0.0328352243499809*G0_0_2 - 0.005375764924376*G0_1_1 - 0.0053757649243761*G0_1_2;
    A[677] = -0.00887914570640261*G0_0_1 - 0.0111690649885093*G0_0_2 - 0.00228991928210667*G0_1_1 - 0.00228991928210653*G0_1_2;
    A[678] = -0.0116844599266469*G0_0_1 - 0.0139743792087539*G0_0_2 - 0.00228991928210676*G0_1_1 - 0.00228991928210702*G0_1_2;
    A[679] = -0.00537576492437606*G0_0_1 - 0.0107515298487518*G0_0_2 - 0.00537576492437599*G0_1_1 - 0.00537576492437574*G0_1_2;
    A[680] = 0.0895482395048358*G0_0_0 + 0.0699110399631224*G0_0_2 + 0.0699110399631227*G0_1_0 + 0.139822079926245*G0_1_2;
    A[681] = -0.0574176057031954*G0_0_0 - 0.0699110399631223*G0_0_2 - 0.0387263842124951*G0_1_0 - 0.108637424175617*G0_1_2;
    A[682] = 0.0283728175438242*G0_0_0 + 0.0599162925551805*G0_0_2 + 0.0189782768992838*G0_1_0 + 0.078894569454464*G0_1_2;
    A[683] = -0.00537576492437611*G0_0_0 - 0.0274594594256048*G0_0_2 - 0.00537576492437605*G0_1_0 - 0.0328352243499806*G0_1_2;
    A[684] = 0.179096479009671*G0_0_0 + 0.0699110399631224*G0_0_1 + 0.0699110399631224*G0_1_0 + 0.139822079926245*G0_1_1;
    A[685] = -0.147063705006412*G0_0_0 + 0.026383001643418*G0_0_1 - 0.0387263842124949*G0_1_0 - 0.108637424175617*G0_1_1;
    A[686] = 0.100717304493345*G0_0_0 - 0.024634573245684*G0_0_1 + 0.0189782768992835*G0_1_0 + 0.0788945694544646*G0_1_1;
    A[687] = -0.032835224349981*G0_0_0 + 0.02091590090722*G0_0_1 - 0.00537576492437607*G0_1_0 - 0.0328352243499811*G0_1_1;
    A[688] = 0.00537576492437583*G0_0_0 + 0.00537576492437577*G0_0_1 + 0.0053757649243755*G0_1_0 + 0.00537576492437545*G0_1_1;
    A[689] = 0.0022899192821071*G0_0_0 + 0.0022899192821072*G0_0_1 + 0.00228991928210742*G0_1_0 + 0.00228991928210752*G0_1_1;
    A[690] = 0.00228991928210635*G0_0_0 + 0.00228991928210628*G0_0_1 + 0.00228991928210614*G0_1_0 + 0.00228991928210611*G0_1_1;
    A[691] = 0.00537576492437608*G0_0_0 + 0.00537576492437611*G0_0_1 + 0.00537576492437604*G0_1_0 + 0.00537576492437605*G0_1_1;
    A[692] = 0.0107515298487518*G0_0_0 + 0.00537576492437562*G0_0_1 + 0.0107515298487517*G0_0_2 + 0.00537576492437571*G0_1_0 + 0.00537576492437566*G0_1_2;
    A[693] = 0.013974379208754*G0_0_0 + 0.00228991928210716*G0_0_1 + 0.0139743792087541*G0_0_2 + 0.00228991928210693*G0_1_0 + 0.00228991928210701*G0_1_2;
    A[694] = 0.0111690649885091*G0_0_0 + 0.00228991928210639*G0_0_1 + 0.0111690649885092*G0_0_2 + 0.00228991928210654*G0_1_0 + 0.00228991928210654*G0_1_2;
    A[695] = 0.0328352243499809*G0_0_0 + 0.00537576492437615*G0_0_1 + 0.0328352243499809*G0_0_2 + 0.00537576492437609*G0_1_0 + 0.00537576492437608*G0_1_2;
    A[696] = 0.0220836945012286*G0_0_0 + 0.0274594594256047*G0_0_1 + 0.0274594594256047*G0_0_2 + 0.0274594594256048*G0_1_0 + 0.0328352243499804*G0_1_1 + 0.0328352243499804*G0_1_2;
    A[697] = -0.0315434750113561*G0_0_0 - 0.0599162925551804*G0_0_1 - 0.0599162925551805*G0_0_2 - 0.0599162925551804*G0_1_0 - 0.0788945694544638*G0_1_1 - 0.0788945694544637*G0_1_2;
    A[698] = 0.0124934342599266*G0_0_0 + 0.0699110399631221*G0_0_1 + 0.0699110399631222*G0_0_2 + 0.069911039963122*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[699] = 0.0196371995417135*G0_0_0 - 0.0699110399631223*G0_0_1 - 0.0699110399631224*G0_0_2 - 0.0699110399631222*G0_1_0 - 0.139822079926245*G0_1_1 - 0.139822079926245*G0_1_2;
    A[700] = -0.0644569872000416*G0_0_0 + 0.0962940416065411*G0_0_1 + 0.192588083213081*G0_0_2 + 0.0284445813959699*G0_1_0 + 0.124738623002511*G0_1_1 + 0.12473862300251*G0_1_2;
    A[701] = 0.1273155976932*G0_0_0 - 0.00704590548340569*G0_0_1 - 0.049321338383838*G0_0_2 - 0.000685018588664558*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0429604514890971*G0_1_2;
    A[702] = -0.0641307878721065*G0_0_0 + 0.0264873854283577*G0_0_1 + 0.0477555816097477*G0_0_2 - 0.00939454064454038*G0_1_0 + 0.0118736555368501*G0_1_1 + 0.0118736555368497*G0_1_2;
    A[703] = 0.0439716694056961*G0_0_0 - 0.0422754329004329*G0_0_1 - 0.0845508658008649*G0_0_2 - 0.000685018588664447*G0_1_0 - 0.0429604514890972*G0_1_1 - 0.0429604514890968*G0_1_2;
    A[704] = -0.0292600797158084*G0_0_0 + 0.00844856259352805*G0_0_1 + 0.0142875305635728*G0_0_2 - 0.00329461321214796*G0_1_0 + 0.00254435475789659*G0_1_1 + 0.00254435475789686*G0_1_2;
    A[705] = -0.0187890812890804*G0_0_0 + 0.0212681961813906*G0_0_1 + 0.0425363923627808*G0_0_2 - 0.0093945406445401*G0_1_0 + 0.0118736555368503*G0_1_1 + 0.01187365553685*G0_1_2;
    A[706] = 0.0187890812890806*G0_0_0 + 0.00939454064454059*G0_0_1 + 0.00939454064454062*G0_1_0;
    A[707] = 0.0292600797158091*G0_0_0 + 0.00329461321214802*G0_0_1 + 0.00329461321214821*G0_1_0;
    A[708] = 0.0641307878721066*G0_0_0 + 0.00939454064454048*G0_0_1 + 0.00939454064454036*G0_1_0;
    A[709] = 0.00658922642429614*G0_0_0 + 0.00329461321214775*G0_0_1 + 0.00329461321214794*G0_1_0;
    A[710] = 0.0292600797158083*G0_0_0 + 0.0032946132121477*G0_0_1 + 0.00329461321214801*G0_1_0;
    A[711] = 0.0187890812890804*G0_0_0 + 0.00939454064454056*G0_0_1 + 0.00939454064454005*G0_1_0;
    A[712] = -0.0306627368259308*G0_0_0 - 0.0212681961813906*G0_0_1 - 0.0212681961813898*G0_1_0 - 0.0118736555368497*G0_1_1;
    A[713] = 0.0642612676032806*G0_0_0 + 0.0422754329004318*G0_0_1 + 0.0422754329004321*G0_1_0 + 0.0429604514890962*G0_1_1;
    A[714] = -0.128522535206561*G0_0_0 - 0.0962940416065407*G0_0_1 - 0.0962940416065405*G0_1_0 - 0.12473862300251*G0_1_1;
    A[715] = -0.00913358118219256*G0_0_0 - 0.00583896797004408*G0_0_1 - 0.00583896797004504*G0_1_0 - 0.00254435475789649*G0_1_1;
    A[716] = 0.0642612676032806*G0_0_0 + 0.0422754329004329*G0_0_1 + 0.0422754329004327*G0_1_0 + 0.0429604514890972*G0_1_1;
    A[717] = -0.0306627368259305*G0_0_0 - 0.0212681961813907*G0_0_1 - 0.0212681961813899*G0_1_0 - 0.0118736555368504*G0_1_1;
    A[718] = -0.0613254736518618*G0_0_0 - 0.0212681961813906*G0_0_1 - 0.0425363923627813*G0_0_2 - 0.0212681961813903*G0_1_0 - 0.01187365553685*G0_1_2;
    A[719] = 0.128522535206561*G0_0_0 + 0.0422754329004322*G0_0_1 + 0.0845508658008642*G0_0_2 + 0.0422754329004325*G0_1_0 + 0.0429604514890961*G0_1_2;
    A[720] = -0.257045070413123*G0_0_0 - 0.0962940416065403*G0_0_1 - 0.192588083213081*G0_0_2 - 0.0962940416065403*G0_1_0 - 0.12473862300251*G0_1_2;
    A[721] = -0.0435476102793805*G0_0_0 - 0.00583896797004396*G0_0_1 - 0.0142875305635721*G0_0_2 - 0.0058389679700444*G0_1_0 - 0.00254435475789634*G0_1_2;
    A[722] = 0.176636936077038*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0493213383838378*G0_0_2 + 0.0422754329004323*G0_1_0 + 0.0429604514890968*G0_1_2;
    A[723] = -0.111886369481854*G0_0_0 - 0.0212681961813902*G0_0_1 - 0.0477555816097477*G0_0_2 - 0.0212681961813901*G0_1_0 - 0.0118736555368496*G0_1_2;
    A[724] = -0.0505608958299927*G0_0_0 - 0.00260959462348302*G0_0_1 - 0.00260959462348422*G0_1_0;
    A[725] = 0.0962288017409537*G0_0_0 - 0.0352295274170267*G0_0_1 - 0.0352295274170262*G0_1_0;
    A[726] = -0.15657567740901*G0_0_0 - 0.00260959462348361*G0_0_1 - 0.00260959462348366*G0_1_0;
    A[727] = -0.0505608958299923*G0_0_0 - 0.0026095946234841*G0_0_1 - 0.0026095946234835*G0_1_0;
    A[728] = -0.00336507226698198*G0_0_0 - 0.00336507226698203*G0_0_1 - 0.00336507226698203*G0_0_2 - 0.00336507226698189*G0_1_0 - 0.00336507226698192*G0_1_1 - 0.00336507226698192*G0_1_2;
    A[729] = 0.0214691349673987*G0_0_0 - 0.0336585514536899*G0_1_0;
    A[730] = 0.0232632312710435*G0_0_1 - 0.0155544887532733*G0_1_1;
    A[731] = 0.00336507226698206*G0_0_2 + 0.00336507226698203*G0_1_2;
    A[732] = 0.0599162925551811*G0_0_1 + 0.0788945694544646*G0_0_2 + 0.0283728175438245*G0_1_1 + 0.018978276899284*G0_1_2;
    A[733] = 0.0057802520910158*G0_0_1 + 0.00523223722008443*G0_0_2 - 0.000482775005344785*G0_1_1 - 0.000548014870931811*G0_1_2;
    A[734] = 0.0131523569023566*G0_0_1 + 0.0263047138047133*G0_0_2 + 0.0132175967679442*G0_1_1 + 0.0131523569023571*G0_1_2;
    A[735] = -0.00228991928210666*G0_0_1 - 0.013974379208754*G0_0_2 - 0.00228991928210684*G0_1_1 - 0.0116844599266474*G0_1_2;
    A[736] = -0.0574176057031955*G0_0_0 - 0.0387263842124948*G0_0_2 - 0.0699110399631228*G0_1_0 - 0.108637424175617*G0_1_2;
    A[737] = 0.00506261356955813*G0_0_0 + 0.00480165410720938*G0_0_2 + 0.0048016541072101*G0_1_0 + 0.00960330821441911*G0_1_2;
    A[738] = -0.000482775005344493*G0_0_0 + 0.00578025209101625*G0_0_2 - 0.000548014870931661*G0_1_0 + 0.00523223722008472*G0_1_2;
    A[739] = -0.00228991928210666*G0_0_0 - 0.00887914570640271*G0_0_2 - 0.00228991928210679*G0_1_0 - 0.0111690649885096*G0_1_2;
    A[740] = -0.147063705006412*G0_0_0 - 0.0387263842124949*G0_0_1 + 0.026383001643418*G0_1_0 - 0.108637424175617*G0_1_1;
    A[741] = 0.304226541205704*G0_0_0 + 0.0781312630270954*G0_0_1 + 0.0781312630270954*G0_1_0 + 0.281001149056701*G0_1_1;
    A[742] = -0.204187731314465*G0_0_0 + 0.0153052724667307*G0_0_1 - 0.0364951808094162*G0_1_0 - 0.204187731314465*G0_1_1;
    A[743] = 0.0788945694544646*G0_0_0 - 0.0246345732456837*G0_0_1 + 0.018978276899284*G0_1_0 + 0.100717304493346*G0_1_1;
    A[744] = 0.00228991928210688*G0_0_0 + 0.00228991928210691*G0_0_1 + 0.00228991928210724*G0_1_0 + 0.00228991928210728*G0_1_1;
    A[745] = 0.00178757231708583*G0_0_0 + 0.00178757231708582*G0_0_1 + 0.0017875723170856*G0_1_0 + 0.00178757231708554*G0_1_1;
    A[746] = 0.00178757231708662*G0_0_0 + 0.00178757231708666*G0_0_1 + 0.00178757231708679*G0_1_0 + 0.00178757231708682*G0_1_1;
    A[747] = 0.00228991928210667*G0_0_0 + 0.00228991928210665*G0_0_1 + 0.00228991928210682*G0_1_0 + 0.00228991928210682*G0_1_1;
    A[748] = 0.0139743792087542*G0_0_0 + 0.0116844599266476*G0_0_1 + 0.0139743792087542*G0_0_2 + 0.0116844599266475*G0_1_0 + 0.00939454064454036*G0_1_1 + 0.0116844599266475*G0_1_2;
    A[749] = -0.0263047138047136*G0_0_0 - 0.0131523569023571*G0_0_1 - 0.0263047138047136*G0_0_2 - 0.0131523569023572*G0_1_0 + 6.52398655871721e-05*G0_1_1 - 0.0131523569023572*G0_1_2;
    A[750] = -0.00523223722008386*G0_0_0 + 0.000548014870931987*G0_0_1 - 0.00523223722008386*G0_0_2 + 0.000548014870932234*G0_1_0 + 6.52398655872046e-05*G0_1_1 + 0.000548014870932244*G0_1_2;
    A[751] = -0.0788945694544646*G0_0_0 - 0.0189782768992838*G0_0_1 - 0.0788945694544646*G0_0_2 - 0.018978276899284*G0_1_0 + 0.00939454064454048*G0_1_1 - 0.018978276899284*G0_1_2;
    A[752] = 0.00658922642429595*G0_0_0 + 0.0088791457064026*G0_0_1 + 0.00887914570640266*G0_0_2 + 0.00887914570640245*G0_1_0 + 0.0111690649885097*G0_1_1 + 0.0111690649885098*G0_1_2;
    A[753] = -0.00626302709636052*G0_0_0 - 0.00578025209101592*G0_0_1 - 0.005780252091016*G0_0_2 - 0.00578025209101584*G0_1_0 - 0.00523223722008457*G0_1_1 - 0.00523223722008473*G0_1_2;
    A[754] = 0.000260959462348485*G0_0_0 - 0.00480165410720944*G0_0_1 - 0.00480165410720944*G0_0_2 - 0.00480165410720955*G0_1_0 - 0.00960330821441885*G0_1_1 - 0.00960330821441882*G0_1_2;
    A[755] = -0.0186912214907005*G0_0_0 + 0.0387263842124948*G0_0_1 + 0.0387263842124948*G0_0_2 + 0.0387263842124948*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[756] = 0.132306447410612*G0_0_0 - 0.0623693115012559*G0_0_1 + 0.01096029741863*G0_0_2 + 0.0733296089198859*G0_1_0 - 0.124738623002511*G0_1_1 + 0.146659217839771*G0_1_2;
    A[757] = -0.166948816037355*G0_0_0 + 0.0838658472122003*G0_0_1 + 0.167731694424401*G0_0_2 - 0.00616516729797926*G0_1_0 + 0.182410664181496*G0_1_1 + 0.0777006799142213*G0_1_2;
    A[758] = 0.127087258163646*G0_0_0 - 0.0463855444324183*G0_0_1 - 0.0823327103709028*G0_0_2 + 0.0377738821749238*G0_1_0 - 0.0510175748891015*G0_1_1 + 0.00182671623643949*G0_1_2;
    A[759] = -0.0436454700777611*G0_0_0 - 0.00649136662591466*G0_0_1 - 0.0808321934624008*G0_0_2 - 0.0338921101724922*G0_1_0 - 0.00352295274170201*G0_1_1 - 0.108232937008978*G0_1_2;
    A[760] = 0.0189848008858421*G0_0_0 - 0.0188869410874622*G0_0_1 - 0.0377738821749243*G0_0_2 + 0.00949240044292132*G0_1_0 - 0.0155923278753142*G0_1_1 - 0.00939454064454119*G0_1_2;
    A[761] = 0.0148746893538553*G0_0_0 + 0.00123955744615432*G0_0_1 + 0.0403182369328196*G0_0_2 + 0.0149399292194426*G0_1_0 + 0.00117431758056698*G0_1_1 + 0.0540186087061077*G0_1_2;
    A[762] = -0.0148746893538554*G0_0_0 - 0.014939929219443*G0_0_1 - 0.0149399292194432*G0_1_0 - 0.015005169085031*G0_1_1;
    A[763] = -0.0189848008858428*G0_0_0 - 0.00949240044292111*G0_0_1 - 0.00949240044292121*G0_1_0 - 0.000652398655870054*G0_1_1;
    A[764] = -0.127087258163647*G0_0_0 - 0.0377738821749237*G0_0_1 - 0.0377738821749239*G0_1_0 - 0.0150051690850307*G0_1_1;
    A[765] = 0.0045015507255088*G0_0_0 + 0.0019245760348192*G0_0_1 + 0.00192457603481894*G0_1_0 - 0.000652398655870326*G0_1_1;
    A[766] = -0.0189848008858419*G0_0_0 - 0.00949240044292082*G0_0_1 - 0.00949240044292123*G0_1_0 - 0.000652398655870543*G0_1_1;
    A[767] = -0.0148746893538555*G0_0_0 - 0.0149399292194432*G0_0_1 - 0.0149399292194427*G0_1_0 - 0.015005169085031*G0_1_1;
    A[768] = -0.00130479731174175*G0_0_0 - 0.00123955744615452*G0_0_1 - 0.00123955744615517*G0_1_0 - 0.00117431758056766*G0_1_1;
    A[769] = 0.00880738185425691*G0_0_0 + 0.00649136662591558*G0_0_1 + 0.00649136662591535*G0_1_0 + 0.00352295274170345*G0_1_1;
    A[770] = 0.0476251018785734*G0_0_0 + 0.0623693115012558*G0_0_1 + 0.0623693115012553*G0_1_0 + 0.124738623002511*G0_1_1;
    A[771] = -0.0156575677409005*G0_0_0 - 0.013080593050211*G0_0_1 - 0.0130805930502101*G0_1_0 - 0.0105036183595208*G0_1_1;
    A[772] = 0.00880738185425676*G0_0_0 + 0.00649136662591465*G0_0_1 + 0.00649136662591455*G0_1_0 + 0.00352295274170205*G0_1_1;
    A[773] = -0.00130479731174214*G0_0_0 - 0.00123955744615428*G0_0_1 - 0.00123955744615517*G0_1_0 - 0.00117431758056688*G0_1_1;
    A[774] = -0.0254435475789637*G0_0_0 - 0.0390786794866649*G0_0_1 - 0.0403182369328194*G0_0_2 - 0.0390786794866649*G0_1_0 - 0.0528442911255406*G0_1_1 - 0.054018608706108*G0_1_2;
    A[775] = 0.0371867233846394*G0_0_0 + 0.0743408268364855*G0_0_1 + 0.0808321934624009*G0_0_2 + 0.0743408268364851*G0_1_0 + 0.104709984267274*G0_1_1 + 0.108232937008978*G0_1_2;
    A[776] = 0.121346149991982*G0_0_0 - 0.0733296089198856*G0_0_1 - 0.0109602974186302*G0_0_2 - 0.0733296089198857*G0_1_0 - 0.271397840842282*G0_1_1 - 0.146659217839772*G0_1_2;
    A[777] = 0.0567586830607653*G0_0_0 + 0.0188869410874616*G0_0_1 + 0.0377738821749237*G0_0_2 + 0.0188869410874617*G0_1_0 - 0.00619778723077341*G0_1_1 + 0.00939454064454084*G0_1_2;
    A[778] = -0.334680510461756*G0_0_0 - 0.0838658472122004*G0_0_1 - 0.167731694424401*G0_0_2 - 0.0838658472121998*G0_1_0 + 0.104709984267275*G0_1_1 - 0.077700679914221*G0_1_2;
    A[779] = 0.209419968534549*G0_0_0 + 0.0359471659384844*G0_0_1 + 0.0823327103709029*G0_0_2 + 0.035947165938484*G0_1_0 - 0.0528442911255412*G0_1_1 - 0.00182671623643949*G0_1_2;
    A[780] = 0.0391439193522516*G0_0_0 + 0.0319675341376715*G0_0_1 + 0.0319675341376733*G0_1_0 + 0.0260959462348341*G0_1_1;
    A[781] = -0.156575677409008*G0_0_0 - 0.0903572138381156*G0_0_1 - 0.0903572138381161*G0_1_0 - 0.185933616923198*G0_1_1;
    A[782] = 0.185933616923199*G0_0_0 + 0.0156575677409008*G0_0_1 + 0.0156575677409004*G0_1_0 + 0.0260959462348347*G0_1_1;
    A[783] = 0.0391439193522523*G0_0_0 + 0.0319675341376733*G0_0_1 + 0.0319675341376732*G0_1_0 + 0.0260959462348352*G0_1_1;
    A[784] = -0.0033650722669819*G0_0_0 - 0.00336507226698186*G0_0_1 - 0.00336507226698184*G0_0_2 - 0.00336507226698192*G0_1_0 - 0.00336507226698192*G0_1_1 - 0.00336507226698189*G0_1_2;
    A[785] = -0.0155544887532733*G0_0_0 + 0.0232632312710435*G0_1_0;
    A[786] = -0.03365855145369*G0_0_1 + 0.0214691349673987*G0_1_1;
    A[787] = 0.00336507226698196*G0_0_2 + 0.00336507226698197*G0_1_2;
    A[788] = -0.069911039963123*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0574176057031959*G0_1_1 - 0.0387263842124952*G0_1_2;
    A[789] = 0.00480165410720998*G0_0_1 + 0.00960330821441909*G0_0_2 + 0.0050626135695584*G0_1_1 + 0.00480165410720983*G0_1_2;
    A[790] = -0.000548014870931627*G0_0_1 + 0.00523223722008468*G0_0_2 - 0.000482775005344919*G0_1_1 + 0.0057802520910155*G0_1_2;
    A[791] = -0.0022899192821067*G0_0_1 - 0.0111690649885095*G0_0_2 - 0.00228991928210659*G0_1_1 - 0.00887914570640245*G0_1_2;
    A[792] = 0.028372817543824*G0_0_0 + 0.0189782768992836*G0_0_2 + 0.059916292555181*G0_1_0 + 0.0788945694544646*G0_1_2;
    A[793] = -0.000482775005344182*G0_0_0 - 0.000548014870931331*G0_0_2 + 0.00578025209101589*G0_1_0 + 0.00523223722008424*G0_1_2;
    A[794] = 0.0132175967679437*G0_0_0 + 0.0131523569023566*G0_0_2 + 0.0131523569023567*G0_1_0 + 0.0263047138047136*G0_1_2;
    A[795] = -0.00228991928210669*G0_0_0 - 0.0116844599266472*G0_0_2 - 0.00228991928210662*G0_1_0 - 0.013974379208754*G0_1_2;
    A[796] = 0.100717304493345*G0_0_0 + 0.0189782768992835*G0_0_1 - 0.0246345732456839*G0_1_0 + 0.0788945694544646*G0_1_1;
    A[797] = -0.204187731314465*G0_0_0 - 0.0364951808094162*G0_0_1 + 0.0153052724667307*G0_1_0 - 0.204187731314465*G0_1_1;
    A[798] = 0.281001149056701*G0_0_0 + 0.0781312630270953*G0_0_1 + 0.0781312630270953*G0_1_0 + 0.304226541205705*G0_1_1;
    A[799] = -0.108637424175618*G0_0_0 + 0.0263830016434178*G0_0_1 - 0.0387263842124953*G0_1_0 - 0.147063705006413*G0_1_1;
    A[800] = 0.00228991928210667*G0_0_0 + 0.00228991928210671*G0_0_1 + 0.00228991928210646*G0_1_0 + 0.00228991928210645*G0_1_1;
    A[801] = 0.00178757231708632*G0_0_0 + 0.00178757231708624*G0_0_1 + 0.00178757231708646*G0_1_0 + 0.00178757231708646*G0_1_1;
    A[802] = 0.00178757231708593*G0_0_0 + 0.00178757231708596*G0_0_1 + 0.00178757231708587*G0_1_0 + 0.00178757231708586*G0_1_1;
    A[803] = 0.00228991928210672*G0_0_0 + 0.00228991928210671*G0_0_1 + 0.00228991928210662*G0_1_0 + 0.0022899192821066*G0_1_1;
    A[804] = 0.011169064988509*G0_0_0 + 0.00887914570640209*G0_0_1 + 0.0111690649885089*G0_0_2 + 0.00887914570640211*G0_1_0 + 0.00658922642429564*G0_1_1 + 0.00887914570640209*G0_1_2;
    A[805] = -0.00523223722008387*G0_0_0 - 0.00578025209101512*G0_0_1 - 0.00523223722008388*G0_0_2 - 0.00578025209101512*G0_1_0 - 0.00626302709635988*G0_1_1 - 0.00578025209101504*G0_1_2;
    A[806] = -0.00960330821441978*G0_0_0 - 0.00480165410721023*G0_0_1 - 0.00960330821441981*G0_0_2 - 0.00480165410721029*G0_1_0 + 0.000260959462347993*G0_1_1 - 0.0048016541072103*G0_1_2;
    A[807] = 0.108637424175618*G0_0_0 + 0.0387263842124953*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0387263842124952*G0_1_0 - 0.0186912214907004*G0_1_1 + 0.0387263842124953*G0_1_2;
    A[808] = 0.00939454064454027*G0_0_0 + 0.0116844599266472*G0_0_1 + 0.0116844599266471*G0_0_2 + 0.0116844599266472*G0_1_0 + 0.0139743792087537*G0_1_1 + 0.0139743792087536*G0_1_2;
    A[809] = 6.52398655874214e-05*G0_0_0 - 0.0131523569023567*G0_0_1 - 0.0131523569023566*G0_0_2 - 0.0131523569023566*G0_1_0 - 0.0263047138047135*G0_1_1 - 0.0263047138047133*G0_1_2;
    A[810] = 6.52398655868415e-05*G0_0_0 + 0.000548014870931378*G0_0_1 + 0.00054801487093136*G0_0_2 + 0.000548014870931297*G0_1_0 - 0.00523223722008468*G0_1_1 - 0.00523223722008471*G0_1_2;
    A[811] = 0.00939454064454061*G0_0_0 - 0.0189782768992834*G0_0_1 - 0.0189782768992835*G0_0_2 - 0.0189782768992835*G0_1_0 - 0.0788945694544644*G0_1_1 - 0.0788945694544645*G0_1_2;
    A[812] = -0.0510175748891008*G0_0_0 + 0.0377738821749238*G0_0_1 + 0.00182671623643899*G0_0_2 - 0.046385544432419*G0_1_0 + 0.127087258163646*G0_1_1 - 0.0823327103709038*G0_1_2;
    A[813] = 0.182410664181495*G0_0_0 - 0.00616516729797901*G0_0_1 + 0.0777006799142208*G0_0_2 + 0.0838658472122003*G0_1_0 - 0.166948816037356*G0_1_1 + 0.167731694424401*G0_1_2;
    A[814] = -0.124738623002512*G0_0_0 + 0.0733296089198849*G0_0_1 + 0.146659217839771*G0_0_2 - 0.0623693115012559*G0_1_0 + 0.132306447410613*G0_1_1 + 0.0109602974186299*G0_1_2;
    A[815] = -0.0155923278753143*G0_0_0 + 0.00949240044292106*G0_0_1 - 0.00939454064454027*G0_0_2 - 0.018886941087462*G0_1_0 + 0.0189848008858419*G0_1_1 - 0.0377738821749237*G0_1_2;
    A[816] = -0.00352295274170224*G0_0_0 - 0.0338921101724919*G0_0_1 - 0.108232937008977*G0_0_2 - 0.00649136662591532*G0_1_0 - 0.0436454700777616*G0_1_1 - 0.0808321934624004*G0_1_2;
    A[817] = 0.0011743175805675*G0_0_0 + 0.0149399292194429*G0_0_1 + 0.0540186087061079*G0_0_2 + 0.00123955744615446*G0_1_0 + 0.0148746893538559*G0_1_1 + 0.0403182369328195*G0_1_2;
    A[818] = -0.00117431758056818*G0_0_0 - 0.00123955744615477*G0_0_1 - 0.00123955744615448*G0_1_0 - 0.00130479731174111*G0_1_1;
    A[819] = 0.00352295274170251*G0_0_0 + 0.00649136662591521*G0_0_1 + 0.00649136662591454*G0_1_0 + 0.00880738185425594*G0_1_1;
    A[820] = 0.124738623002512*G0_0_0 + 0.0623693115012557*G0_0_1 + 0.0623693115012559*G0_1_0 + 0.0476251018785746*G0_1_1;
    A[821] = -0.0105036183595211*G0_0_0 - 0.0130805930502109*G0_0_1 - 0.0130805930502108*G0_1_0 - 0.0156575677409011*G0_1_1;
    A[822] = 0.00352295274170212*G0_0_0 + 0.00649136662591456*G0_0_1 + 0.00649136662591523*G0_1_0 + 0.00880738185425626*G0_1_1;
    A[823] = -0.00117431758056745*G0_0_0 - 0.00123955744615434*G0_0_1 - 0.00123955744615441*G0_1_0 - 0.00130479731174106*G0_1_1;
    A[824] = -0.0150051690850303*G0_0_0 - 0.0149399292194434*G0_0_1 - 0.0149399292194429*G0_1_0 - 0.0148746893538562*G0_1_1;
    A[825] = -0.000652398655870861*G0_0_0 - 0.00949240044292136*G0_0_1 - 0.00949240044292123*G0_1_0 - 0.0189848008858431*G0_1_1;
    A[826] = -0.0150051690850295*G0_0_0 - 0.0377738821749239*G0_0_1 - 0.0377738821749233*G0_1_0 - 0.127087258163646*G0_1_1;
    A[827] = -0.000652398655870701*G0_0_0 + 0.00192457603481928*G0_0_1 + 0.00192457603481901*G0_1_0 + 0.00450155072550929*G0_1_1;
    A[828] = -0.000652398655871546*G0_0_0 - 0.00949240044292107*G0_0_1 - 0.00949240044292143*G0_1_0 - 0.018984800885842*G0_1_1;
    A[829] = -0.0150051690850297*G0_0_0 - 0.0149399292194429*G0_0_1 - 0.0149399292194426*G0_1_0 - 0.014874689353856*G0_1_1;
    A[830] = -0.0528442911255405*G0_0_0 - 0.0390786794866651*G0_0_1 - 0.0540186087061085*G0_0_2 - 0.0390786794866652*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.04031823693282*G0_1_2;
    A[831] = -0.00619778723077336*G0_0_0 + 0.0188869410874617*G0_0_1 + 0.00939454064454042*G0_0_2 + 0.0188869410874622*G0_1_0 + 0.0567586830607662*G0_1_1 + 0.0377738821749231*G0_1_2;
    A[832] = -0.05284429112554*G0_0_0 + 0.0359471659384846*G0_0_1 - 0.00182671623643884*G0_0_2 + 0.0359471659384848*G0_1_0 + 0.20941996853455*G0_1_1 + 0.082332710370904*G0_1_2;
    A[833] = 0.104709984267275*G0_0_0 + 0.0743408268364855*G0_0_1 + 0.108232937008977*G0_0_2 + 0.0743408268364853*G0_1_0 + 0.0371867233846393*G0_1_1 + 0.0808321934624003*G0_1_2;
    A[834] = 0.104709984267274*G0_0_0 - 0.0838658472122002*G0_0_1 - 0.0777006799142211*G0_0_2 - 0.0838658472122007*G0_1_0 - 0.334680510461758*G0_1_1 - 0.167731694424401*G0_1_2;
    A[835] = -0.271397840842282*G0_0_0 - 0.0733296089198856*G0_0_1 - 0.146659217839771*G0_0_2 - 0.0733296089198851*G0_1_0 + 0.121346149991983*G0_1_1 - 0.0109602974186299*G0_1_2;
    A[836] = 0.0260959462348359*G0_0_0 + 0.0319675341376735*G0_0_1 + 0.0319675341376724*G0_1_0 + 0.039143919352253*G0_1_1;
    A[837] = 0.0260959462348332*G0_0_0 + 0.0156575677409005*G0_0_1 + 0.0156575677409012*G0_1_0 + 0.185933616923199*G0_1_1;
    A[838] = -0.185933616923198*G0_0_0 - 0.0903572138381148*G0_0_1 - 0.0903572138381149*G0_1_0 - 0.156575677409009*G0_1_1;
    A[839] = 0.0260959462348354*G0_0_0 + 0.0319675341376726*G0_0_1 + 0.0319675341376728*G0_1_0 + 0.0391439193522522*G0_1_1;
    A[840] = -0.00480034930989794*G0_0_0 - 0.00480034930989797*G0_0_1 - 0.00480034930989797*G0_0_2 - 0.00480034930989792*G0_1_0 - 0.00480034930989792*G0_1_1 - 0.00480034930989795*G0_1_2;
    A[841] = 0.00480034930989789*G0_0_0 - 0.0105310191030676*G0_1_0;
    A[842] = 0.0372571824394738*G0_0_1 - 0.0178705039816151*G0_1_1;
    A[843] = 0.00480034930989792*G0_0_2 + 0.0048003493098979*G0_1_2;
    A[844] = 0.0699110399631231*G0_0_1 + 0.139822079926246*G0_0_2 + 0.0895482395048365*G0_1_1 + 0.069911039963123*G0_1_2;
    A[845] = -0.038726384212495*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0574176057031958*G0_1_1 - 0.0699110399631229*G0_1_2;
    A[846] = 0.0189782768992839*G0_0_1 + 0.0788945694544648*G0_0_2 + 0.0283728175438248*G0_1_1 + 0.0599162925551816*G0_1_2;
    A[847] = -0.00537576492437597*G0_0_1 - 0.0328352243499811*G0_0_2 - 0.00537576492437613*G0_1_1 - 0.0274594594256053*G0_1_2;
    A[848] = -0.00537576492437595*G0_0_0 - 0.00537576492437602*G0_0_2 - 0.027459459425605*G0_1_0 - 0.032835224349981*G0_1_2;
    A[849] = -0.00228991928210671*G0_0_0 - 0.00228991928210661*G0_0_2 - 0.00887914570640247*G0_1_0 - 0.011169064988509*G0_1_2;
    A[850] = -0.00228991928210686*G0_0_0 - 0.00228991928210687*G0_0_2 - 0.0116844599266474*G0_1_0 - 0.0139743792087543*G0_1_2;
    A[851] = -0.00537576492437601*G0_0_0 - 0.00537576492437606*G0_0_2 - 0.00537576492437615*G0_1_0 - 0.0107515298487521*G0_1_2;
    A[852] = -0.032835224349981*G0_0_0 - 0.00537576492437607*G0_0_1 + 0.02091590090722*G0_1_0 - 0.0328352243499811*G0_1_1;
    A[853] = 0.0788945694544646*G0_0_0 + 0.018978276899284*G0_0_1 - 0.0246345732456837*G0_1_0 + 0.100717304493346*G0_1_1;
    A[854] = -0.108637424175618*G0_0_0 - 0.0387263842124953*G0_0_1 + 0.0263830016434178*G0_1_0 - 0.147063705006413*G0_1_1;
    A[855] = 0.139822079926246*G0_0_0 + 0.0699110399631231*G0_0_1 + 0.0699110399631231*G0_1_0 + 0.179096479009673*G0_1_1;
    A[856] = 0.00537576492437611*G0_0_0 + 0.00537576492437607*G0_0_1 + 0.00537576492437616*G0_1_0 + 0.00537576492437617*G0_1_1;
    A[857] = 0.00228991928210653*G0_0_0 + 0.00228991928210663*G0_0_1 + 0.00228991928210648*G0_1_0 + 0.00228991928210652*G0_1_1;
    A[858] = 0.00228991928210703*G0_0_0 + 0.00228991928210699*G0_0_1 + 0.00228991928210699*G0_1_0 + 0.00228991928210701*G0_1_1;
    A[859] = 0.00537576492437596*G0_0_0 + 0.00537576492437596*G0_0_1 + 0.00537576492437613*G0_1_0 + 0.00537576492437613*G0_1_1;
    A[860] = 0.0328352243499811*G0_0_0 + 0.0274594594256053*G0_0_1 + 0.0328352243499812*G0_0_2 + 0.0274594594256052*G0_1_0 + 0.0220836945012291*G0_1_1 + 0.0274594594256053*G0_1_2;
    A[861] = -0.0788945694544648*G0_0_0 - 0.0599162925551813*G0_0_1 - 0.0788945694544649*G0_0_2 - 0.0599162925551812*G0_1_0 - 0.0315434750113568*G0_1_1 - 0.0599162925551814*G0_1_2;
    A[862] = 0.108637424175618*G0_0_0 + 0.0699110399631229*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0699110399631231*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.0699110399631231*G0_1_2;
    A[863] = -0.139822079926246*G0_0_0 - 0.0699110399631228*G0_0_1 - 0.139822079926246*G0_0_2 - 0.069911039963123*G0_1_0 + 0.0196371995417134*G0_1_1 - 0.069911039963123*G0_1_2;
    A[864] = 0.00537576492437599*G0_0_1 + 0.00537576492437605*G0_0_2 + 0.00537576492437609*G0_1_0 + 0.0107515298487521*G0_1_1 + 0.0107515298487522*G0_1_2;
    A[865] = 0.00228991928210686*G0_0_1 + 0.00228991928210684*G0_0_2 + 0.00228991928210667*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.0139743792087541*G0_1_2;
    A[866] = 0.00228991928210694*G0_0_1 + 0.00228991928210696*G0_0_2 + 0.00228991928210685*G0_1_0 + 0.0111690649885096*G0_1_1 + 0.0111690649885096*G0_1_2;
    A[867] = 0.00537576492437591*G0_0_1 + 0.00537576492437599*G0_0_2 + 0.00537576492437598*G0_1_0 + 0.032835224349981*G0_1_1 + 0.032835224349981*G0_1_2;
    A[868] = 0.0118736555368499*G0_0_0 - 0.00939454064454035*G0_0_1 + 0.01187365553685*G0_0_2 + 0.0264873854283582*G0_1_0 - 0.0641307878721065*G0_1_1 + 0.047755581609748*G0_1_2;
    A[869] = -0.0429604514890978*G0_0_0 - 0.000685018588665085*G0_0_1 - 0.0429604514890969*G0_0_2 - 0.00704590548340632*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0493213383838377*G0_1_2;
    A[870] = 0.124738623002512*G0_0_0 + 0.0284445813959705*G0_0_1 + 0.124738623002512*G0_0_2 + 0.0962940416065416*G0_1_0 - 0.0644569872000426*G0_1_1 + 0.192588083213082*G0_1_2;
    A[871] = 0.00254435475789601*G0_0_0 - 0.003294613212148*G0_0_1 + 0.00254435475789586*G0_0_2 + 0.00844856259352758*G0_1_0 - 0.0292600797158084*G0_1_1 + 0.0142875305635722*G0_1_2;
    A[872] = -0.0429604514890968*G0_0_0 - 0.000685018588664362*G0_0_1 - 0.0429604514890976*G0_0_2 - 0.042275432900432*G0_1_0 + 0.0439716694056973*G0_1_1 - 0.0845508658008658*G0_1_2;
    A[873] = 0.01187365553685*G0_0_0 - 0.00939454064454081*G0_0_1 + 0.0118736555368501*G0_0_2 + 0.0212681961813905*G0_1_0 - 0.0187890812890816*G0_1_1 + 0.0425363923627813*G0_1_2;
    A[874] = -0.01187365553685*G0_0_0 - 0.0212681961813906*G0_0_1 - 0.0212681961813911*G0_1_0 - 0.0306627368259316*G0_1_1;
    A[875] = 0.0429604514890978*G0_0_0 + 0.042275432900433*G0_0_1 + 0.0422754329004337*G0_1_0 + 0.0642612676032819*G0_1_1;
    A[876] = -0.124738623002512*G0_0_0 - 0.0962940416065417*G0_0_1 - 0.0962940416065416*G0_1_0 - 0.128522535206563*G0_1_1;
    A[877] = -0.00254435475789639*G0_0_0 - 0.00583896797004428*G0_0_1 - 0.00583896797004456*G0_1_0 - 0.00913358118219195*G0_1_1;
    A[878] = 0.0429604514890968*G0_0_0 + 0.0422754329004326*G0_0_1 + 0.042275432900432*G0_1_0 + 0.0642612676032811*G0_1_1;
    A[879] = -0.01187365553685*G0_0_0 - 0.0212681961813909*G0_0_1 - 0.0212681961813907*G0_1_0 - 0.0306627368259318*G0_1_1;
    A[880] = 0.00939454064454057*G0_0_1 + 0.00939454064454058*G0_1_0 + 0.018789081289081*G0_1_1;
    A[881] = 0.00329461321214777*G0_0_1 + 0.00329461321214823*G0_1_0 + 0.0292600797158089*G0_1_1;
    A[882] = 0.00939454064454043*G0_0_1 + 0.0093945406445401*G0_1_0 + 0.0641307878721064*G0_1_1;
    A[883] = 0.00329461321214755*G0_0_1 + 0.00329461321214777*G0_1_0 + 0.00658922642429527*G0_1_1;
    A[884] = 0.00329461321214804*G0_0_1 + 0.00329461321214822*G0_1_0 + 0.0292600797158085*G0_1_1;
    A[885] = 0.00939454064454077*G0_0_1 + 0.00939454064454042*G0_1_0 + 0.0187890812890815*G0_1_1;
    A[886] = -0.0212681961813905*G0_0_1 - 0.0118736555368498*G0_0_2 - 0.0212681961813902*G0_1_0 - 0.0613254736518622*G0_1_1 - 0.0425363923627809*G0_1_2;
    A[887] = -0.00583896797004485*G0_0_1 - 0.00254435475789657*G0_0_2 - 0.00583896797004473*G0_1_0 - 0.0435476102793816*G0_1_1 - 0.0142875305635722*G0_1_2;
    A[888] = -0.0212681961813905*G0_0_1 - 0.0118736555368501*G0_0_2 - 0.0212681961813903*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0477555816097483*G0_1_2;
    A[889] = 0.0422754329004331*G0_0_1 + 0.0429604514890973*G0_0_2 + 0.0422754329004325*G0_1_0 + 0.128522535206563*G0_1_1 + 0.0845508658008655*G0_1_2;
    A[890] = 0.042275432900433*G0_0_1 + 0.0429604514890974*G0_0_2 + 0.0422754329004325*G0_1_0 + 0.17663693607704*G0_1_1 + 0.0493213383838381*G0_1_2;
    A[891] = -0.0962940416065414*G0_0_1 - 0.124738623002511*G0_0_2 - 0.096294041606541*G0_1_0 - 0.257045070413125*G0_1_1 - 0.192588083213082*G0_1_2;
    A[892] = -0.00260959462348361*G0_0_1 - 0.00260959462348349*G0_1_0 - 0.0505608958299929*G0_1_1;
    A[893] = -0.00260959462348285*G0_0_1 - 0.00260959462348365*G0_1_0 - 0.15657567740901*G0_1_1;
    A[894] = -0.0352295274170274*G0_0_1 - 0.0352295274170273*G0_1_0 + 0.0962288017409544*G0_1_1;
    A[895] = -0.00260959462348318*G0_0_1 - 0.00260959462348305*G0_1_0 - 0.0505608958299923*G0_1_1;
    A[896] = -0.0178705039816151*G0_0_0 - 0.017870503981615*G0_0_1 - 0.0178705039816151*G0_0_2 - 0.017870503981615*G0_1_0 - 0.017870503981615*G0_1_1 - 0.017870503981615*G0_1_2 - 0.0551276864210888*G0_2_0 - 0.0551276864210887*G0_2_1 - 0.0551276864210887*G0_2_2;
    A[897] = -0.00480034930989803*G0_0_0 - 0.00480034930989802*G0_1_0;
    A[898] = -0.00480034930989784*G0_0_1 - 0.00480034930989787*G0_1_1;
    A[899] = 0.0105310191030678*G0_0_2 + 0.0105310191030678*G0_1_2 + 0.0153313684129656*G0_2_2;
    A[900] = 0.0107515298487519*G0_0_1 + 0.00537576492437613*G0_0_2 + 0.0107515298487519*G0_1_1 + 0.00537576492437609*G0_1_2 + 0.00537576492437591*G0_2_1;
    A[901] = 0.0139743792087542*G0_0_1 + 0.0116844599266469*G0_0_2 + 0.0139743792087542*G0_1_1 + 0.011684459926647*G0_1_2 + 0.0116844599266475*G0_2_1 + 0.00939454064454018*G0_2_2;
    A[902] = 0.0111690649885091*G0_0_1 + 0.00887914570640288*G0_0_2 + 0.0111690649885091*G0_1_1 + 0.00887914570640285*G0_1_2 + 0.0088791457064024*G0_2_1 + 0.00658922642429613*G0_2_2;
    A[903] = 0.0328352243499812*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0328352243499813*G0_1_1 + 0.0274594594256049*G0_1_2 + 0.0274594594256052*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[904] = 0.0107515298487519*G0_0_0 + 0.00537576492437558*G0_0_2 + 0.0107515298487518*G0_1_0 + 0.00537576492437554*G0_1_2 + 0.00537576492437582*G0_2_0;
    A[905] = 0.0139743792087542*G0_0_0 + 0.0116844599266479*G0_0_2 + 0.0139743792087543*G0_1_0 + 0.0116844599266479*G0_1_2 + 0.0116844599266477*G0_2_0 + 0.0093945406445412*G0_2_2;
    A[906] = 0.0111690649885093*G0_0_0 + 0.00887914570640199*G0_0_2 + 0.0111690649885092*G0_1_0 + 0.00887914570640199*G0_1_2 + 0.00887914570640247*G0_2_0 + 0.00658922642429536*G0_2_2;
    A[907] = 0.0328352243499813*G0_0_0 + 0.0274594594256054*G0_0_2 + 0.0328352243499814*G0_1_0 + 0.0274594594256055*G0_1_2 + 0.0274594594256053*G0_2_0 + 0.0220836945012295*G0_2_2;
    A[908] = 0.00537576492437583*G0_0_0 + 0.0053757649243755*G0_0_1 + 0.00537576492437577*G0_1_0 + 0.00537576492437545*G0_1_1;
    A[909] = 0.00228991928210688*G0_0_0 + 0.00228991928210723*G0_0_1 + 0.00228991928210692*G0_1_0 + 0.00228991928210728*G0_1_1;
    A[910] = 0.00228991928210667*G0_0_0 + 0.00228991928210646*G0_0_1 + 0.00228991928210671*G0_1_0 + 0.00228991928210645*G0_1_1;
    A[911] = 0.00537576492437611*G0_0_0 + 0.00537576492437616*G0_0_1 + 0.00537576492437606*G0_1_0 + 0.00537576492437617*G0_1_1;
    A[912] = 0.179096479009672*G0_0_0 + 0.179096479009672*G0_0_1 + 0.10918543904655*G0_0_2 + 0.179096479009672*G0_1_0 + 0.179096479009672*G0_1_1 + 0.10918543904655*G0_1_2 + 0.10918543904655*G0_2_0 + 0.10918543904655*G0_2_1 + 0.179096479009672*G0_2_2;
    A[913] = -0.147063705006412*G0_0_0 - 0.147063705006413*G0_0_1 - 0.173446706649831*G0_0_2 - 0.147063705006413*G0_1_0 - 0.147063705006413*G0_1_1 - 0.173446706649831*G0_1_2 - 0.108337320793918*G0_2_0 - 0.108337320793918*G0_2_1 - 0.243357746612954*G0_2_2;
    A[914] = 0.100717304493345*G0_0_0 + 0.100717304493345*G0_0_1 + 0.12535187773903*G0_0_2 + 0.100717304493346*G0_1_0 + 0.100717304493346*G0_1_1 + 0.12535187773903*G0_1_2 + 0.081739027594062*G0_2_0 + 0.081739027594062*G0_2_1 + 0.185268170294211*G0_2_2;
    A[915] = -0.0328352243499812*G0_0_0 - 0.0328352243499812*G0_0_1 - 0.0537511252572015*G0_0_2 - 0.0328352243499813*G0_1_0 - 0.0328352243499813*G0_1_1 - 0.0537511252572015*G0_1_2 - 0.0274594594256052*G0_2_0 - 0.0274594594256053*G0_2_1 - 0.0812105846828066*G0_2_2;
    A[916] = 0.0895482395048357*G0_0_0 + 0.0196371995417134*G0_0_1 + 0.0895482395048358*G0_0_2 + 0.0895482395048358*G0_1_0 + 0.0196371995417133*G0_1_1 + 0.0895482395048359*G0_1_2 + 0.0196371995417134*G0_2_0 + 0.0895482395048359*G0_2_1 + 0.0196371995417134*G0_2_2;
    A[917] = -0.0574176057031952*G0_0_0 + 0.0124934342599271*G0_0_1 - 0.0574176057031953*G0_0_2 - 0.0574176057031952*G0_1_0 + 0.0124934342599272*G0_1_1 - 0.0574176057031953*G0_1_2 - 0.0186912214907006*G0_2_0 - 0.0574176057031955*G0_2_1 - 0.0186912214907005*G0_2_2;
    A[918] = 0.0283728175438243*G0_0_0 - 0.0315434750113564*G0_0_1 + 0.0283728175438243*G0_0_2 + 0.0283728175438243*G0_1_0 - 0.0315434750113566*G0_1_1 + 0.0283728175438243*G0_1_2 + 0.00939454064454077*G0_2_0 + 0.0283728175438243*G0_2_1 + 0.00939454064454075*G0_2_2;
    A[919] = -0.00537576492437607*G0_0_0 + 0.0220836945012289*G0_0_1 - 0.00537576492437608*G0_0_2 - 0.00537576492437603*G0_1_0 + 0.022083694501229*G0_1_1 - 0.00537576492437603*G0_1_2 - 0.00537576492437606*G0_2_1;
    A[920] = 0.0196371995417134*G0_0_0 + 0.0895482395048358*G0_0_1 + 0.0895482395048358*G0_0_2 + 0.0196371995417134*G0_1_0 + 0.0895482395048359*G0_1_1 + 0.0895482395048359*G0_1_2 + 0.0895482395048358*G0_2_0 + 0.0196371995417133*G0_2_1 + 0.0196371995417133*G0_2_2;
    A[921] = 0.0124934342599272*G0_0_0 - 0.0574176057031953*G0_0_1 - 0.0574176057031952*G0_0_2 + 0.0124934342599272*G0_1_0 - 0.0574176057031952*G0_1_1 - 0.0574176057031952*G0_1_2 - 0.0574176057031952*G0_2_0 - 0.0186912214907005*G0_2_1 - 0.0186912214907005*G0_2_2;
    A[922] = -0.0315434750113567*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.0283728175438241*G0_0_2 - 0.0315434750113568*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.028372817543824*G0_1_2 + 0.028372817543824*G0_2_0 + 0.00939454064454067*G0_2_1 + 0.00939454064454067*G0_2_2;
    A[923] = 0.0220836945012291*G0_0_0 - 0.0053757649243755*G0_0_1 - 0.00537576492437552*G0_0_2 + 0.0220836945012291*G0_1_0 - 0.00537576492437545*G0_1_1 - 0.00537576492437547*G0_1_2 - 0.005375764924376*G0_2_0;
    A[924] = 0.0187890812890812*G0_0_0 + 0.018789081289081*G0_0_1 + 0.00939454064454021*G0_0_2 + 0.0187890812890813*G0_1_0 + 0.0187890812890809*G0_1_1 + 0.00939454064454018*G0_1_2 + 0.00939454064454104*G0_2_0 + 0.00939454064454052*G0_2_1;
    A[925] = 0.00658922642429591*G0_0_0 + 0.0065892264242963*G0_0_1 + 0.00329461321214844*G0_0_2 + 0.00658922642429591*G0_1_0 + 0.00658922642429639*G0_1_1 + 0.00329461321214854*G0_1_2 + 0.00329461321214764*G0_2_0 + 0.00329461321214829*G0_2_1;
    A[926] = 0.0187890812890807*G0_0_0 + 0.0187890812890806*G0_0_1 + 0.0093945406445403*G0_0_2 + 0.0187890812890808*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454031*G0_1_2 + 0.00939454064454029*G0_2_0 + 0.00939454064454009*G0_2_1;
    A[927] = 0.0292600797158088*G0_0_0 + 0.0292600797158089*G0_0_1 + 0.025965466503661*G0_0_2 + 0.0292600797158087*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.025965466503661*G0_1_2 + 0.025965466503661*G0_2_0 + 0.0259654665036613*G0_2_1 + 0.0226708532915138*G0_2_2;
    A[928] = 0.0292600797158083*G0_0_0 + 0.0292600797158081*G0_0_1 + 0.02596546650366*G0_0_2 + 0.0292600797158083*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.02596546650366*G0_1_2 + 0.0259654665036606*G0_2_0 + 0.0259654665036605*G0_2_1 + 0.0226708532915122*G0_2_2;
    A[929] = 0.0641307878721067*G0_0_0 + 0.064130787872107*G0_0_1 + 0.0547362472275663*G0_0_2 + 0.0641307878721067*G0_1_0 + 0.064130787872107*G0_1_1 + 0.0547362472275662*G0_1_2 + 0.0547362472275662*G0_2_0 + 0.0547362472275664*G0_2_1 + 0.0453417065830256*G0_2_2;
    A[930] = -0.0644569872000421*G0_0_0 - 0.257045070413124*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0644569872000421*G0_1_0 - 0.257045070413124*G0_1_1 - 0.160751028806583*G0_1_2 - 0.0929015685960121*G0_2_0 - 0.160751028806583*G0_2_1 - 0.064456987200042*G0_2_2;
    A[931] = 0.0439716694056967*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061289*G0_0_2 + 0.0439716694056966*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0862471023061289*G0_1_2 + 0.0446566879943612*G0_2_0 + 0.0862471023061292*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[932] = -0.0187890812890808*G0_0_0 - 0.061325473651862*G0_0_1 - 0.0400572774704713*G0_0_2 - 0.0187890812890809*G0_1_0 - 0.061325473651862*G0_1_1 - 0.0400572774704714*G0_1_2 - 0.00939454064454036*G0_2_0 - 0.0400572774704716*G0_2_1 - 0.018789081289081*G0_2_2;
    A[933] = 0.1273155976932*G0_0_0 + 0.176636936077039*G0_0_1 + 0.134361503176606*G0_0_2 + 0.1273155976932*G0_1_0 + 0.176636936077039*G0_1_1 + 0.134361503176606*G0_1_2 + 0.128000616281865*G0_2_0 + 0.134361503176607*G0_2_1 + 0.0920860702761735*G0_2_2;
    A[934] = -0.0292600797158083*G0_0_0 - 0.0435476102793807*G0_0_1 - 0.0377086423093364*G0_0_2 - 0.0292600797158083*G0_1_0 - 0.0435476102793808*G0_1_1 - 0.0377086423093364*G0_1_2 - 0.0259654665036606*G0_2_0 - 0.0377086423093366*G0_2_1 - 0.0318696743392923*G0_2_2;
    A[935] = -0.0641307878721067*G0_0_0 - 0.111886369481854*G0_0_1 - 0.090618173300464*G0_0_2 - 0.0641307878721067*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0906181733004642*G0_1_2 - 0.0547362472275662*G0_2_0 - 0.0906181733004642*G0_2_1 - 0.0693499771190738*G0_2_2;
    A[936] = -0.257045070413124*G0_0_0 - 0.0644569872000421*G0_0_1 - 0.160751028806583*G0_0_2 - 0.257045070413124*G0_1_0 - 0.0644569872000421*G0_1_1 - 0.160751028806583*G0_1_2 - 0.160751028806583*G0_2_0 - 0.0929015685960123*G0_2_1 - 0.0644569872000422*G0_2_2;
    A[937] = 0.128522535206562*G0_0_0 + 0.0439716694056971*G0_0_1 + 0.0862471023061291*G0_0_2 + 0.128522535206562*G0_1_0 + 0.0439716694056971*G0_1_1 + 0.0862471023061291*G0_1_2 + 0.0862471023061292*G0_2_0 + 0.0446566879943612*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[938] = -0.0613254736518618*G0_0_0 - 0.0187890812890812*G0_0_1 - 0.0400572774704722*G0_0_2 - 0.0613254736518617*G0_1_0 - 0.0187890812890811*G0_1_1 - 0.0400572774704721*G0_1_2 - 0.0400572774704712*G0_2_0 - 0.00939454064454062*G0_2_1 - 0.0187890812890818*G0_2_2;
    A[939] = 0.176636936077039*G0_0_0 + 0.127315597693201*G0_0_1 + 0.134361503176607*G0_0_2 + 0.176636936077039*G0_1_0 + 0.127315597693201*G0_1_1 + 0.134361503176607*G0_1_2 + 0.134361503176607*G0_2_0 + 0.128000616281866*G0_2_1 + 0.092086070276174*G0_2_2;
    A[940] = -0.0435476102793812*G0_0_0 - 0.0292600797158087*G0_0_1 - 0.0377086423093361*G0_0_2 - 0.0435476102793812*G0_1_0 - 0.0292600797158088*G0_1_1 - 0.0377086423093361*G0_1_2 - 0.0377086423093373*G0_2_0 - 0.0259654665036612*G0_2_1 - 0.0318696743392917*G0_2_2;
    A[941] = -0.111886369481855*G0_0_0 - 0.0641307878721072*G0_0_1 - 0.0906181733004648*G0_0_2 - 0.111886369481855*G0_1_0 - 0.0641307878721071*G0_1_1 - 0.0906181733004649*G0_1_2 - 0.0906181733004646*G0_2_0 - 0.0547362472275665*G0_2_1 - 0.0693499771190745*G0_2_2;
    A[942] = -0.128522535206561*G0_0_0 - 0.128522535206561*G0_0_1 - 0.0322284936000209*G0_0_2 - 0.128522535206562*G0_1_0 - 0.128522535206562*G0_1_1 - 0.0322284936000208*G0_1_2 - 0.0322284936000213*G0_2_0 - 0.032228493600021*G0_2_1 - 0.0606730749959912*G0_2_2;
    A[943] = 0.0642612676032806*G0_0_0 + 0.0642612676032806*G0_0_1 + 0.0219858347028484*G0_0_2 + 0.0642612676032806*G0_1_0 + 0.0642612676032806*G0_1_1 + 0.0219858347028484*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0219858347028482*G0_2_1 + 0.0226708532915127*G0_2_2;
    A[944] = -0.0306627368259308*G0_0_0 - 0.0306627368259313*G0_0_1 - 0.0093945406445403*G0_0_2 - 0.0306627368259307*G0_1_0 - 0.0306627368259313*G0_1_1 - 0.00939454064454027*G0_1_2 - 0.00939454064454023*G0_2_0 - 0.0093945406445408*G0_2_1;
    A[945] = 0.0642612676032806*G0_0_0 + 0.0642612676032805*G0_0_1 + 0.0219858347028484*G0_0_2 + 0.0642612676032806*G0_1_0 + 0.0642612676032806*G0_1_1 + 0.0219858347028483*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0219858347028485*G0_2_1 + 0.022670853291513*G0_2_2;
    A[946] = -0.00913358118219219*G0_0_0 - 0.00913358118219194*G0_0_1 - 0.00329461321214821*G0_0_2 - 0.00913358118219224*G0_1_0 - 0.00913358118219194*G0_1_1 - 0.00329461321214831*G0_1_2 - 0.00329461321214825*G0_2_0 - 0.00329461321214764*G0_2_1;
    A[947] = -0.030662736825931*G0_0_0 - 0.0306627368259311*G0_0_1 - 0.00939454064454048*G0_0_2 - 0.0306627368259311*G0_1_0 - 0.0306627368259311*G0_1_1 - 0.00939454064454049*G0_1_2 - 0.00939454064454062*G0_2_0 - 0.00939454064454095*G0_2_1;
    A[948] = 0.0962288017409539*G0_0_0 + 0.0962288017409537*G0_0_1 + 0.13145832915798*G0_0_2 + 0.0962288017409539*G0_1_0 + 0.0962288017409537*G0_1_1 + 0.13145832915798*G0_1_2 + 0.131458329157981*G0_2_0 + 0.131458329157981*G0_2_1 + 0.166687856575008*G0_2_2;
    A[949] = -0.0505608958299931*G0_0_0 - 0.0505608958299934*G0_0_1 - 0.0479513012065094*G0_0_2 - 0.0505608958299931*G0_1_0 - 0.0505608958299934*G0_1_1 - 0.0479513012065094*G0_1_2 - 0.04795130120651*G0_2_0 - 0.0479513012065096*G0_2_1 - 0.0453417065830263*G0_2_2;
    A[950] = -0.0505608958299925*G0_0_0 - 0.0505608958299921*G0_0_1 - 0.0479513012065083*G0_0_2 - 0.0505608958299925*G0_1_0 - 0.0505608958299921*G0_1_1 - 0.0479513012065083*G0_1_2 - 0.0479513012065088*G0_2_0 - 0.0479513012065087*G0_2_1 - 0.0453417065830252*G0_2_2;
    A[951] = -0.156575677409009*G0_0_0 - 0.156575677409009*G0_0_1 - 0.153966082785526*G0_0_2 - 0.156575677409009*G0_1_0 - 0.156575677409009*G0_1_1 - 0.153966082785526*G0_1_2 - 0.153966082785527*G0_2_0 - 0.153966082785526*G0_2_1 - 0.151356488162043*G0_2_2;
    A[952] = 0.0214691349673989*G0_0_0 + 0.0214691349673989*G0_0_1 + 0.0214691349673989*G0_0_2 + 0.0214691349673989*G0_1_0 + 0.0214691349673989*G0_1_1 + 0.0214691349673989*G0_1_2 + 0.0551276864210889*G0_2_0 + 0.0551276864210889*G0_2_1 + 0.0551276864210889*G0_2_2;
    A[953] = -0.00336507226698176*G0_0_0 - 0.00336507226698175*G0_1_0;
    A[954] = -0.00336507226698198*G0_0_1 - 0.00336507226698195*G0_1_1;
    A[955] = -0.0232632312710438*G0_0_2 - 0.0232632312710439*G0_1_2 - 0.0388177200243172*G0_2_2;
    A[956] = 0.0139743792087541*G0_0_1 + 0.00228991928210648*G0_0_2 + 0.0139743792087542*G0_1_1 + 0.00228991928210658*G0_1_2 + 0.00228991928210705*G0_2_1;
    A[957] = -0.0263047138047138*G0_0_1 - 0.0131523569023563*G0_0_2 - 0.0263047138047139*G0_1_1 - 0.0131523569023565*G0_1_2 - 0.0131523569023572*G0_2_1 + 6.52398655874015e-05*G0_2_2;
    A[958] = -0.00523223722008374*G0_0_1 - 0.00578025209101628*G0_0_2 - 0.0052322372200837*G0_1_1 - 0.00578025209101622*G0_1_2 - 0.00578025209101555*G0_2_1 - 0.00626302709636069*G0_2_2;
    A[959] = -0.0788945694544649*G0_0_1 - 0.0599162925551806*G0_0_2 - 0.0788945694544652*G0_1_1 - 0.0599162925551808*G0_1_2 - 0.0599162925551814*G0_2_1 - 0.0315434750113565*G0_2_2;
    A[960] = 0.0139743792087542*G0_0_0 + 0.00228991928210729*G0_0_2 + 0.0139743792087544*G0_1_0 + 0.0022899192821074*G0_1_2 + 0.00228991928210715*G0_2_0;
    A[961] = -0.0263047138047139*G0_0_0 - 0.0131523569023576*G0_0_2 - 0.0263047138047142*G0_1_0 - 0.0131523569023577*G0_1_2 - 0.0131523569023576*G0_2_0 + 6.52398655859048e-05*G0_2_2;
    A[962] = -0.00523223722008431*G0_0_0 - 0.00578025209101523*G0_0_2 - 0.00523223722008404*G0_1_0 - 0.00578025209101509*G0_1_2 - 0.00578025209101565*G0_2_0 - 0.00626302709635969*G0_2_2;
    A[963] = -0.0788945694544652*G0_0_0 - 0.0599162925551816*G0_0_2 - 0.0788945694544654*G0_1_0 - 0.0599162925551817*G0_1_2 - 0.0599162925551816*G0_2_0 - 0.0315434750113575*G0_2_2;
    A[964] = 0.0022899192821071*G0_0_0 + 0.00228991928210742*G0_0_1 + 0.0022899192821072*G0_1_0 + 0.00228991928210752*G0_1_1;
    A[965] = 0.00178757231708583*G0_0_0 + 0.0017875723170856*G0_0_1 + 0.00178757231708582*G0_1_0 + 0.00178757231708554*G0_1_1;
    A[966] = 0.00178757231708632*G0_0_0 + 0.00178757231708647*G0_0_1 + 0.00178757231708624*G0_1_0 + 0.00178757231708646*G0_1_1;
    A[967] = 0.00228991928210652*G0_0_0 + 0.00228991928210648*G0_0_1 + 0.00228991928210663*G0_1_0 + 0.00228991928210652*G0_1_1;
    A[968] = -0.147063705006412*G0_0_0 - 0.147063705006413*G0_0_1 - 0.108337320793918*G0_0_2 - 0.147063705006413*G0_1_0 - 0.147063705006413*G0_1_1 - 0.108337320793918*G0_1_2 - 0.173446706649831*G0_2_0 - 0.173446706649831*G0_2_1 - 0.243357746612954*G0_2_2;
    A[969] = 0.304226541205706*G0_0_0 + 0.304226541205706*G0_0_1 + 0.226095278178611*G0_0_2 + 0.304226541205706*G0_1_0 + 0.304226541205706*G0_1_1 + 0.226095278178611*G0_1_2 + 0.226095278178611*G0_2_0 + 0.226095278178611*G0_2_1 + 0.428965164208218*G0_2_2;
    A[970] = -0.204187731314466*G0_0_0 - 0.204187731314466*G0_0_1 - 0.219493003781197*G0_0_2 - 0.204187731314466*G0_1_0 - 0.204187731314466*G0_1_1 - 0.219493003781198*G0_1_2 - 0.16769255050505*G0_2_0 - 0.16769255050505*G0_2_1 - 0.387185554286247*G0_2_2;
    A[971] = 0.0788945694544649*G0_0_0 + 0.078894569454465*G0_0_1 + 0.103529142700149*G0_0_2 + 0.0788945694544652*G0_1_0 + 0.0788945694544652*G0_1_1 + 0.103529142700149*G0_1_2 + 0.0599162925551815*G0_2_0 + 0.0599162925551815*G0_2_1 + 0.185268170294211*G0_2_2;
    A[972] = -0.0574176057031957*G0_0_0 - 0.0186912214907008*G0_0_1 - 0.0574176057031956*G0_0_2 - 0.0574176057031957*G0_1_0 - 0.0186912214907008*G0_1_1 - 0.0574176057031957*G0_1_2 + 0.012493434259927*G0_2_0 - 0.0574176057031957*G0_2_1 + 0.0124934342599271*G0_2_2;
    A[973] = 0.00506261356955808*G0_0_0 + 0.000260959462348513*G0_0_1 + 0.00506261356955816*G0_0_2 + 0.00506261356955805*G0_1_0 + 0.00026095946234846*G0_1_1 + 0.00506261356955814*G0_1_2 + 0.000260959462348703*G0_2_0 + 0.00506261356955813*G0_2_1 + 0.000260959462348668*G0_2_2;
    A[974] = -0.000482775005344587*G0_0_0 - 0.00626302709636029*G0_0_1 - 0.00048277500534457*G0_0_2 - 0.000482775005344504*G0_1_0 - 0.00626302709636023*G0_1_1 - 0.000482775005344487*G0_1_2 + 6.52398655869326e-05*G0_2_0 - 0.000482775005344601*G0_2_1 + 6.52398655869372e-05*G0_2_2;
    A[975] = -0.00228991928210658*G0_0_0 + 0.00658922642429562*G0_0_1 - 0.00228991928210659*G0_0_2 - 0.00228991928210668*G0_1_0 + 0.00658922642429557*G0_1_1 - 0.00228991928210669*G0_1_2 - 0.00228991928210667*G0_2_1;
    A[976] = -0.0186912214907008*G0_0_0 - 0.0574176057031957*G0_0_1 - 0.0574176057031955*G0_0_2 - 0.0186912214907007*G0_1_0 - 0.0574176057031957*G0_1_1 - 0.0574176057031956*G0_1_2 - 0.0574176057031955*G0_2_0 + 0.012493434259927*G0_2_1 + 0.0124934342599271*G0_2_2;
    A[977] = 0.000260959462348328*G0_0_0 + 0.00506261356955787*G0_0_1 + 0.00506261356955781*G0_0_2 + 0.000260959462348259*G0_1_0 + 0.00506261356955775*G0_1_1 + 0.00506261356955769*G0_1_2 + 0.00506261356955769*G0_2_0 + 0.000260959462348353*G0_2_1 + 0.000260959462348316*G0_2_2;
    A[978] = -0.00626302709636022*G0_0_0 - 0.000482775005344233*G0_0_1 - 0.000482775005344212*G0_0_2 - 0.00626302709636013*G0_1_0 - 0.000482775005344159*G0_1_1 - 0.000482775005344137*G0_1_2 - 0.000482775005344207*G0_2_0 + 6.52398655871405e-05*G0_2_1 + 6.52398655871196e-05*G0_2_2;
    A[979] = 0.00658922642429564*G0_0_0 - 0.00228991928210741*G0_0_1 - 0.0022899192821074*G0_0_2 + 0.0065892264242956*G0_1_0 - 0.00228991928210752*G0_1_1 - 0.0022899192821075*G0_1_2 - 0.00228991928210685*G0_2_0;
    A[980] = -0.0148746893538559*G0_0_0 - 0.0148746893538559*G0_0_1 + 6.52398655875225e-05*G0_0_2 - 0.014874689353856*G0_1_0 - 0.0148746893538558*G0_1_1 + 6.52398655876708e-05*G0_1_2 + 6.52398655862736e-05*G0_2_0 + 6.52398655870496e-05*G0_2_1;
    A[981] = 0.00450155072550884*G0_0_0 + 0.00450155072550875*G0_0_1 + 0.00257697469068927*G0_0_2 + 0.00450155072550887*G0_1_0 + 0.00450155072550874*G0_1_1 + 0.00257697469068917*G0_1_2 + 0.00257697469069019*G0_2_0 + 0.00257697469068974*G0_2_1;
    A[982] = -0.0148746893538556*G0_0_0 - 0.0148746893538557*G0_0_1 + 6.52398655870903e-05*G0_0_2 - 0.0148746893538556*G0_1_0 - 0.0148746893538558*G0_1_1 + 6.52398655871485e-05*G0_1_2 + 6.52398655875e-05*G0_2_0 + 6.52398655872593e-05*G0_2_1;
    A[983] = -0.0189848008858424*G0_0_0 - 0.0189848008858427*G0_0_1 - 0.00949240044292144*G0_0_2 - 0.0189848008858421*G0_1_0 - 0.0189848008858427*G0_1_1 - 0.00949240044292146*G0_1_2 - 0.00949240044292097*G0_2_0 - 0.00949240044292196*G0_2_1 - 0.000652398655871744*G0_2_2;
    A[984] = -0.0189848008858416*G0_0_0 - 0.018984800885841*G0_0_1 - 0.00949240044292008*G0_0_2 - 0.0189848008858417*G0_1_0 - 0.018984800885841*G0_1_1 - 0.00949240044292013*G0_1_2 - 0.00949240044292107*G0_2_0 - 0.00949240044292039*G0_2_1 - 0.000652398655870078*G0_2_2;
    A[985] = -0.127087258163646*G0_0_0 - 0.127087258163647*G0_0_1 - 0.0893133759887222*G0_0_2 - 0.127087258163646*G0_1_0 - 0.127087258163647*G0_1_1 - 0.0893133759887223*G0_1_2 - 0.0893133759887227*G0_2_0 - 0.089313375988723*G0_2_1 - 0.0665446628988288*G0_2_2;
    A[986] = 0.132306447410614*G0_0_0 + 0.121346149991983*G0_0_1 + 0.194675758911869*G0_0_2 + 0.132306447410614*G0_1_0 + 0.121346149991983*G0_1_1 + 0.194675758911869*G0_1_2 + 0.0589768384907269*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052866*G0_2_2;
    A[987] = -0.0436454700777619*G0_0_0 + 0.0371867233846383*G0_0_1 - 0.0371541034518465*G0_0_2 - 0.0436454700777619*G0_1_0 + 0.0371867233846383*G0_1_1 - 0.0371541034518465*G0_1_2 - 0.00975335990526929*G0_2_0 - 0.0371541034518464*G0_2_1 - 0.00678494602105742*G0_2_2;
    A[988] = 0.0148746893538557*G0_0_0 - 0.0254435475789631*G0_0_1 + 0.013635131907701*G0_0_2 + 0.0148746893538557*G0_1_0 - 0.0254435475789632*G0_1_1 + 0.0136351319077011*G0_1_2 - 6.5239865587419e-05*G0_2_0 + 0.0136351319077012*G0_2_1 - 0.000130479731174346*G0_2_2;
    A[989] = -0.166948816037355*G0_0_0 - 0.334680510461758*G0_0_1 - 0.250814663249557*G0_0_2 - 0.166948816037355*G0_1_0 - 0.334680510461758*G0_1_1 - 0.250814663249557*G0_1_2 - 0.160783648739377*G0_2_0 - 0.250814663249558*G0_2_1 - 0.062238831770081*G0_2_2;
    A[990] = 0.0189848008858415*G0_0_0 + 0.0567586830607655*G0_0_1 + 0.0378717419733045*G0_0_2 + 0.0189848008858417*G0_1_0 + 0.0567586830607655*G0_1_1 + 0.0378717419733045*G0_1_2 + 0.00949240044292105*G0_2_0 + 0.0378717419733044*G0_2_1 + 0.0127870136550695*G0_2_2;
    A[991] = 0.127087258163646*G0_0_0 + 0.20941996853455*G0_0_1 + 0.173472802596064*G0_0_2 + 0.127087258163646*G0_1_0 + 0.20941996853455*G0_1_1 + 0.173472802596065*G0_1_2 + 0.0893133759887228*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320392*G0_2_2;
    A[992] = 0.121346149991983*G0_0_0 + 0.132306447410614*G0_0_1 + 0.194675758911869*G0_0_2 + 0.121346149991983*G0_1_0 + 0.132306447410614*G0_1_1 + 0.194675758911869*G0_1_2 + 0.194675758911869*G0_2_0 + 0.0589768384907271*G0_2_1 - 0.00339247301052828*G0_2_2;
    A[993] = 0.0371867233846393*G0_0_0 - 0.0436454700777619*G0_0_1 - 0.0371541034518462*G0_0_2 + 0.0371867233846393*G0_1_0 - 0.0436454700777617*G0_1_1 - 0.0371541034518461*G0_1_2 - 0.0371541034518462*G0_2_0 - 0.0097533599052697*G0_2_1 - 0.00678494602105703*G0_2_2;
    A[994] = -0.025443547578964*G0_0_0 + 0.0148746893538563*G0_0_1 + 0.013635131907702*G0_0_2 - 0.0254435475789641*G0_1_0 + 0.0148746893538562*G0_1_1 + 0.0136351319077021*G0_1_2 + 0.0136351319077007*G0_2_0 - 6.52398655869372e-05*G0_2_1 - 0.000130479731173282*G0_2_2;
    A[995] = -0.334680510461759*G0_0_0 - 0.166948816037356*G0_0_1 - 0.250814663249558*G0_0_2 - 0.334680510461759*G0_1_0 - 0.166948816037356*G0_1_1 - 0.250814663249558*G0_1_2 - 0.250814663249558*G0_2_0 - 0.160783648739378*G0_2_1 - 0.0622388317700819*G0_2_2;
    A[996] = 0.0567586830607667*G0_0_0 + 0.0189848008858425*G0_0_1 + 0.0378717419733039*G0_0_2 + 0.0567586830607667*G0_1_0 + 0.0189848008858425*G0_1_1 + 0.0378717419733037*G0_1_2 + 0.0378717419733055*G0_2_0 + 0.00949240044292195*G0_2_1 + 0.0127870136550687*G0_2_2;
    A[997] = 0.20941996853455*G0_0_0 + 0.127087258163647*G0_0_1 + 0.173472802596066*G0_0_2 + 0.20941996853455*G0_1_0 + 0.127087258163647*G0_1_1 + 0.173472802596066*G0_1_2 + 0.173472802596066*G0_2_0 + 0.0893133759887231*G0_2_1 + 0.0846813455320404*G0_2_2;
    A[998] = 0.0476251018785742*G0_0_0 + 0.0476251018785742*G0_0_1 - 0.0147442096226811*G0_0_2 + 0.0476251018785742*G0_1_0 + 0.0476251018785743*G0_1_1 - 0.0147442096226811*G0_1_2 - 0.0147442096226813*G0_2_0 - 0.0147442096226814*G0_2_1 + 0.0476251018785741*G0_2_2;
    A[999] = 0.00880738185425655*G0_0_0 + 0.00880738185425695*G0_0_1 + 0.0023160152283416*G0_0_2 + 0.00880738185425658*G0_1_0 + 0.00880738185425701*G0_1_1 + 0.00231601522834163*G0_1_2 + 0.00231601522834145*G0_2_0 + 0.0023160152283416*G0_2_1 - 0.00065239865587086*G0_2_2;
    A[1000] = -0.00130479731174213*G0_0_0 - 0.00130479731174141*G0_0_1 - 6.52398655874004e-05*G0_0_2 - 0.00130479731174223*G0_1_0 - 0.00130479731174143*G0_1_1 - 6.52398655875498e-05*G0_1_2 - 6.52398655876753e-05*G0_2_0 - 6.52398655867691e-05*G0_2_1;
    A[1001] = 0.00880738185425629*G0_0_0 + 0.00880738185425587*G0_0_1 + 0.00231601522834103*G0_0_2 + 0.00880738185425627*G0_1_0 + 0.00880738185425586*G0_1_1 + 0.00231601522834107*G0_1_2 + 0.00231601522834096*G0_2_0 + 0.00231601522834126*G0_2_1 - 0.000652398655870861*G0_2_2;
    A[1002] = -0.0156575677409003*G0_0_0 - 0.0156575677409012*G0_0_1 - 0.00257697469068936*G0_0_2 - 0.0156575677409002*G0_1_0 - 0.0156575677409011*G0_1_1 - 0.00257697469068926*G0_1_2 - 0.00257697469068916*G0_2_0 - 0.00257697469068983*G0_2_1;
    A[1003] = -0.00130479731174171*G0_0_0 - 0.00130479731174085*G0_0_1 - 6.52398655869128e-05*G0_0_2 - 0.00130479731174172*G0_1_0 - 0.00130479731174095*G0_1_1 - 6.52398655869714e-05*G0_1_2 - 6.52398655870575e-05*G0_2_0 - 6.52398655869072e-05*G0_2_1;
    A[1004] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.0662184635708937*G0_0_2 - 0.15657567740901*G0_1_0 - 0.15657567740901*G0_1_1 - 0.0662184635708935*G0_1_2 - 0.0662184635708941*G0_2_0 - 0.0662184635708939*G0_2_1 - 0.161794866655977*G0_2_2;
    A[1005] = 0.0391439193522529*G0_0_0 + 0.0391439193522528*G0_0_1 + 0.00717638521457974*G0_0_2 + 0.0391439193522528*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.00717638521457974*G0_1_2 + 0.00717638521458099*G0_2_0 + 0.00717638521457996*G0_2_1 + 0.00130479731174257*G0_2_2;
    A[1006] = 0.0391439193522531*G0_0_0 + 0.0391439193522519*G0_0_1 + 0.00717638521457888*G0_0_2 + 0.039143919352253*G0_1_0 + 0.039143919352252*G0_1_1 + 0.00717638521457888*G0_1_2 + 0.00717638521457903*G0_2_0 + 0.00717638521457937*G0_2_1 + 0.00130479731174105*G0_2_2;
    A[1007] = 0.185933616923197*G0_0_0 + 0.185933616923197*G0_0_1 + 0.170276049182297*G0_0_2 + 0.185933616923197*G0_1_0 + 0.185933616923197*G0_1_1 + 0.170276049182297*G0_1_2 + 0.170276049182298*G0_2_0 + 0.170276049182298*G0_2_1 + 0.180714427676232*G0_2_2;
    A[1008] = -0.0155544887532734*G0_0_0 - 0.0155544887532735*G0_0_1 - 0.0155544887532734*G0_0_2 - 0.0155544887532734*G0_1_0 - 0.0155544887532734*G0_1_1 - 0.0155544887532734*G0_1_2 - 0.038817720024317*G0_2_0 - 0.038817720024317*G0_2_1 - 0.038817720024317*G0_2_2;
    A[1009] = -0.00336507226698214*G0_0_0 - 0.00336507226698212*G0_1_0;
    A[1010] = -0.00336507226698197*G0_0_1 - 0.00336507226698199*G0_1_1;
    A[1011] = 0.0336585514536903*G0_0_2 + 0.0336585514536903*G0_1_2 + 0.0551276864210891*G0_2_2;
    A[1012] = 0.0111690649885094*G0_0_1 + 0.00228991928210701*G0_0_2 + 0.0111690649885095*G0_1_1 + 0.00228991928210698*G0_1_2 + 0.00228991928210652*G0_2_1;
    A[1013] = -0.00523223722008432*G0_0_1 + 0.000548014870931168*G0_0_2 - 0.00523223722008434*G0_1_1 + 0.000548014870931197*G0_1_2 + 0.000548014870932083*G0_2_1 + 6.52398655871041e-05*G0_2_2;
    A[1014] = -0.00960330821441976*G0_0_1 - 0.00480165410720934*G0_0_2 - 0.00960330821441963*G0_1_1 - 0.00480165410720927*G0_1_2 - 0.00480165410721016*G0_2_1 + 0.000260959462348362*G0_2_2;
    A[1015] = 0.108637424175618*G0_0_1 + 0.0699110399631223*G0_0_2 + 0.108637424175618*G0_1_1 + 0.0699110399631223*G0_1_2 + 0.0699110399631234*G0_2_1 + 0.0124934342599271*G0_2_2;
    A[1016] = 0.0111690649885092*G0_0_0 + 0.00228991928210626*G0_0_2 + 0.0111690649885091*G0_1_0 + 0.00228991928210622*G0_1_2 + 0.00228991928210631*G0_2_0;
    A[1017] = -0.0052322372200841*G0_0_0 + 0.00054801487093224*G0_0_2 - 0.00523223722008398*G0_1_0 + 0.000548014870932242*G0_1_2 + 0.000548014870932427*G0_2_0 + 6.52398655884164e-05*G0_2_2;
    A[1018] = -0.00960330821441944*G0_0_0 - 0.00480165410721007*G0_0_2 - 0.00960330821441959*G0_1_0 - 0.00480165410721013*G0_1_2 - 0.00480165410721022*G0_2_0 + 0.000260959462347501*G0_2_2;
    A[1019] = 0.108637424175618*G0_0_0 + 0.0699110399631231*G0_0_2 + 0.108637424175618*G0_1_0 + 0.0699110399631231*G0_1_2 + 0.0699110399631236*G0_2_0 + 0.012493434259928*G0_2_2;
    A[1020] = 0.00228991928210635*G0_0_0 + 0.00228991928210614*G0_0_1 + 0.00228991928210628*G0_1_0 + 0.00228991928210611*G0_1_1;
    A[1021] = 0.00178757231708662*G0_0_0 + 0.00178757231708679*G0_0_1 + 0.00178757231708666*G0_1_0 + 0.00178757231708682*G0_1_1;
    A[1022] = 0.00178757231708593*G0_0_0 + 0.00178757231708587*G0_0_1 + 0.00178757231708596*G0_1_0 + 0.00178757231708586*G0_1_1;
    A[1023] = 0.00228991928210703*G0_0_0 + 0.00228991928210699*G0_0_1 + 0.00228991928210699*G0_1_0 + 0.00228991928210701*G0_1_1;
    A[1024] = 0.100717304493345*G0_0_0 + 0.100717304493346*G0_0_1 + 0.081739027594062*G0_0_2 + 0.100717304493345*G0_1_0 + 0.100717304493346*G0_1_1 + 0.081739027594062*G0_1_2 + 0.12535187773903*G0_2_0 + 0.12535187773903*G0_2_1 + 0.185268170294211*G0_2_2;
    A[1025] = -0.204187731314466*G0_0_0 - 0.204187731314466*G0_0_1 - 0.16769255050505*G0_0_2 - 0.204187731314466*G0_1_0 - 0.204187731314466*G0_1_1 - 0.16769255050505*G0_1_2 - 0.219493003781197*G0_2_0 - 0.219493003781198*G0_2_1 - 0.387185554286247*G0_2_2;
    A[1026] = 0.281001149056703*G0_0_0 + 0.281001149056703*G0_0_1 + 0.202869886029608*G0_0_2 + 0.281001149056703*G0_1_0 + 0.281001149056703*G0_1_1 + 0.202869886029608*G0_1_2 + 0.202869886029608*G0_2_0 + 0.202869886029608*G0_2_1 + 0.428965164208218*G0_2_2;
    A[1027] = -0.108637424175618*G0_0_0 - 0.108637424175618*G0_0_1 - 0.135020425819037*G0_0_2 - 0.108637424175618*G0_1_0 - 0.108637424175618*G0_1_1 - 0.135020425819037*G0_1_2 - 0.0699110399631235*G0_2_0 - 0.0699110399631235*G0_2_1 - 0.243357746612954*G0_2_2;
    A[1028] = 0.0283728175438242*G0_0_0 + 0.0093945406445407*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438243*G0_1_0 + 0.00939454064454071*G0_1_1 + 0.0283728175438243*G0_1_2 - 0.0315434750113566*G0_2_0 + 0.0283728175438243*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[1029] = -0.000482775005344493*G0_0_0 + 6.52398655869463e-05*G0_0_1 - 0.000482775005344586*G0_0_2 - 0.000482775005344495*G0_1_0 + 6.52398655869541e-05*G0_1_1 - 0.000482775005344587*G0_1_2 - 0.00626302709636059*G0_2_0 - 0.000482775005344526*G0_2_1 - 0.00626302709636054*G0_2_2;
    A[1030] = 0.0132175967679439*G0_0_0 + 6.52398655871242e-05*G0_0_1 + 0.0132175967679439*G0_0_2 + 0.0132175967679439*G0_1_0 + 6.52398655871043e-05*G0_1_1 + 0.0132175967679439*G0_1_2 + 6.52398655870984e-05*G0_2_0 + 0.013217596767944*G0_2_1 + 6.52398655871056e-05*G0_2_2;
    A[1031] = -0.00228991928210696*G0_0_0 + 0.00939454064454067*G0_0_1 - 0.00228991928210696*G0_0_2 - 0.00228991928210693*G0_1_0 + 0.00939454064454069*G0_1_1 - 0.00228991928210692*G0_1_2 - 0.00228991928210677*G0_2_1;
    A[1032] = 0.00939454064454062*G0_0_0 + 0.0283728175438242*G0_0_1 + 0.0283728175438241*G0_0_2 + 0.00939454064454061*G0_1_0 + 0.0283728175438242*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.0283728175438242*G0_2_0 - 0.0315434750113567*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[1033] = 6.52398655872858e-05*G0_0_0 - 0.000482775005344203*G0_0_1 - 0.000482775005344157*G0_0_2 + 6.5239865587305e-05*G0_1_0 - 0.000482775005344153*G0_1_1 - 0.000482775005344106*G0_1_2 - 0.000482775005344161*G0_2_0 - 0.00626302709636012*G0_2_1 - 0.00626302709636008*G0_2_2;
    A[1034] = 6.52398655868603e-05*G0_0_0 + 0.0132175967679437*G0_0_1 + 0.0132175967679437*G0_0_2 + 6.52398655868293e-05*G0_1_0 + 0.0132175967679436*G0_1_1 + 0.0132175967679436*G0_1_2 + 0.0132175967679436*G0_2_0 + 6.52398655868658e-05*G0_2_1 + 6.52398655868945e-05*G0_2_2;
    A[1035] = 0.00939454064454079*G0_0_0 - 0.00228991928210617*G0_0_1 - 0.00228991928210619*G0_0_2 + 0.00939454064454078*G0_1_0 - 0.00228991928210614*G0_1_1 - 0.00228991928210614*G0_1_2 - 0.00228991928210655*G0_2_0;
    A[1036] = -0.00117431758056754*G0_0_0 - 0.00117431758056761*G0_0_1 + 6.5239865586709e-05*G0_0_2 - 0.00117431758056743*G0_1_0 - 0.00117431758056769*G0_1_1 + 6.52398655866576e-05*G0_1_2 + 6.52398655877488e-05*G0_2_0 + 6.52398655870589e-05*G0_2_1;
    A[1037] = -0.0105036183595215*G0_0_0 - 0.0105036183595213*G0_0_1 + 0.00257697469069022*G0_0_2 - 0.0105036183595216*G0_1_0 - 0.0105036183595212*G0_1_1 + 0.0025769746906903*G0_1_2 + 0.00257697469068942*G0_2_0 + 0.00257697469068966*G0_2_1;
    A[1038] = -0.00117431758056765*G0_0_0 - 0.00117431758056771*G0_0_1 + 6.52398655870417e-05*G0_0_2 - 0.00117431758056766*G0_1_0 - 0.00117431758056778*G0_1_1 + 6.52398655870208e-05*G0_1_2 + 6.5239865586993e-05*G0_2_0 + 6.5239865587249e-05*G0_2_1;
    A[1039] = 0.00352295274170268*G0_0_0 + 0.00352295274170318*G0_0_1 - 0.00296841388421224*G0_0_2 + 0.00352295274170245*G0_1_0 + 0.00352295274170309*G0_1_1 - 0.00296841388421231*G0_1_2 - 0.00296841388421319*G0_2_0 - 0.00296841388421199*G0_2_1 - 0.000652398655870299*G0_2_2;
    A[1040] = 0.00352295274170223*G0_0_0 + 0.00352295274170154*G0_0_1 - 0.00296841388421338*G0_0_2 + 0.0035229527417024*G0_1_0 + 0.00352295274170172*G0_1_1 - 0.00296841388421334*G0_1_2 - 0.00296841388421239*G0_2_0 - 0.00296841388421329*G0_2_1 - 0.000652398655871258*G0_2_2;
    A[1041] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012551*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0623693115012551*G0_1_2 + 0.0623693115012556*G0_2_0 + 0.0623693115012557*G0_2_1 + 0.0476251018785731*G0_2_2;
    A[1042] = -0.0510175748891024*G0_0_0 - 0.0528442911255408*G0_0_1 - 0.0887914570640256*G0_0_2 - 0.0510175748891024*G0_1_0 - 0.0528442911255409*G0_1_1 - 0.0887914570640256*G0_1_2 - 0.00463203045668278*G0_2_0 - 0.0887914570640261*G0_2_1 + 0.0846813455320394*G0_2_2;
    A[1043] = -0.0155923278753139*G0_0_0 - 0.00619778723077272*G0_0_1 - 0.025084728318235*G0_0_2 - 0.0155923278753139*G0_1_0 - 0.00619778723077276*G0_1_1 - 0.025084728318235*G0_1_2 + 0.0032946132121473*G0_2_0 - 0.0250847283182352*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[1044] = 0.00117431758056755*G0_0_0 - 0.0528442911255414*G0_0_1 - 0.0137656116388752*G0_0_2 + 0.00117431758056755*G0_1_0 - 0.0528442911255414*G0_1_1 - 0.0137656116388752*G0_1_2 - 6.52398655870693e-05*G0_2_0 - 0.0137656116388754*G0_2_1 - 0.000130479731174149*G0_2_2;
    A[1045] = 0.182410664181495*G0_0_0 + 0.104709984267274*G0_0_1 + 0.188575831479475*G0_0_2 + 0.182410664181495*G0_1_0 + 0.104709984267274*G0_1_1 + 0.188575831479475*G0_1_2 + 0.0985448169692953*G0_2_0 + 0.188575831479476*G0_2_1 - 0.0622388317700816*G0_2_2;
    A[1046] = -0.0035229527417021*G0_0_0 + 0.104709984267276*G0_0_1 + 0.0303691574307892*G0_0_2 - 0.00352295274170228*G0_1_0 + 0.104709984267276*G0_1_1 + 0.030369157430789*G0_1_2 + 0.00296841388421246*G0_2_0 + 0.0303691574307893*G0_2_1 - 0.00678494602105711*G0_2_2;
    A[1047] = -0.124738623002511*G0_0_0 - 0.271397840842283*G0_0_1 - 0.198068231922396*G0_0_2 - 0.124738623002511*G0_1_0 - 0.271397840842283*G0_1_1 - 0.198068231922396*G0_1_2 - 0.0623693115012558*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052848*G0_2_2;
    A[1048] = -0.0528442911255406*G0_0_0 - 0.051017574889102*G0_0_1 - 0.0887914570640257*G0_0_2 - 0.0528442911255406*G0_1_0 - 0.051017574889102*G0_1_1 - 0.0887914570640257*G0_1_2 - 0.0887914570640261*G0_2_0 - 0.00463203045668275*G0_2_1 + 0.0846813455320392*G0_2_2;
    A[1049] = -0.00619778723077307*G0_0_0 - 0.0155923278753132*G0_0_1 - 0.0250847283182351*G0_0_2 - 0.00619778723077313*G0_1_0 - 0.0155923278753133*G0_1_1 - 0.0250847283182352*G0_1_2 - 0.0250847283182352*G0_2_0 + 0.00329461321214799*G0_2_1 + 0.0127870136550688*G0_2_2;
    A[1050] = -0.052844291125541*G0_0_0 + 0.0011743175805674*G0_0_1 - 0.0137656116388761*G0_0_2 - 0.0528442911255408*G0_1_0 + 0.00117431758056748*G0_1_1 - 0.0137656116388761*G0_1_2 - 0.0137656116388751*G0_2_0 - 6.5239865587157e-05*G0_2_1 - 0.000130479731175022*G0_2_2;
    A[1051] = 0.104709984267275*G0_0_0 + 0.182410664181496*G0_0_1 + 0.188575831479476*G0_0_2 + 0.104709984267275*G0_1_0 + 0.182410664181495*G0_1_1 + 0.188575831479476*G0_1_2 + 0.188575831479476*G0_2_0 + 0.0985448169692957*G0_2_1 - 0.0622388317700811*G0_2_2;
    A[1052] = 0.104709984267275*G0_0_0 - 0.00352295274170296*G0_0_1 + 0.0303691574307892*G0_0_2 + 0.104709984267275*G0_1_0 - 0.00352295274170288*G0_1_1 + 0.0303691574307893*G0_1_2 + 0.0303691574307885*G0_2_0 + 0.00296841388421192*G0_2_1 - 0.00678494602105646*G0_2_2;
    A[1053] = -0.271397840842284*G0_0_0 - 0.124738623002511*G0_0_1 - 0.198068231922397*G0_0_2 - 0.271397840842283*G0_1_0 - 0.124738623002511*G0_1_1 - 0.198068231922397*G0_1_2 - 0.198068231922398*G0_2_0 - 0.0623693115012559*G0_2_1 - 0.00339247301052944*G0_2_2;
    A[1054] = -0.01500516908503*G0_0_0 - 0.01500516908503*G0_0_1 + 0.0227687130898934*G0_0_2 - 0.01500516908503*G0_1_0 - 0.01500516908503*G0_1_1 + 0.0227687130898935*G0_1_2 + 0.0227687130898934*G0_2_0 + 0.0227687130898933*G0_2_1 - 0.0665446628988287*G0_2_2;
    A[1055] = -0.000652398655870754*G0_0_0 - 0.00065239865587112*G0_0_1 + 0.00884000178705075*G0_0_2 - 0.000652398655870788*G0_1_0 - 0.00065239865587114*G0_1_1 + 0.00884000178705074*G0_1_2 + 0.00884000178705025*G0_2_0 + 0.00884000178705016*G0_2_1 - 0.000652398655870948*G0_2_2;
    A[1056] = -0.0150051690850299*G0_0_0 - 0.0150051690850304*G0_0_1 - 6.52398655867255e-05*G0_0_2 - 0.0150051690850299*G0_1_0 - 0.0150051690850304*G0_1_1 - 6.52398655866774e-05*G0_1_2 - 6.52398655865079e-05*G0_2_0 - 6.52398655873627e-05*G0_2_1;
    A[1057] = -0.000652398655870689*G0_0_0 - 0.000652398655870641*G0_0_1 + 0.00884000178705032*G0_0_2 - 0.000652398655870674*G0_1_0 - 0.000652398655870639*G0_1_1 + 0.00884000178705032*G0_1_2 + 0.00884000178705056*G0_2_0 + 0.00884000178705032*G0_2_1 - 0.000652398655871491*G0_2_2;
    A[1058] = -0.000652398655871333*G0_0_0 - 0.000652398655870727*G0_0_1 - 0.00257697469069022*G0_0_2 - 0.000652398655871393*G0_1_0 - 0.000652398655870759*G0_1_1 - 0.0025769746906903*G0_1_2 - 0.00257697469069041*G0_2_0 - 0.00257697469068992*G0_2_1;
    A[1059] = -0.0150051690850299*G0_0_0 - 0.0150051690850304*G0_0_1 - 6.52398655872215e-05*G0_0_2 - 0.0150051690850299*G0_1_0 - 0.0150051690850304*G0_1_1 - 6.52398655872013e-05*G0_1_2 - 6.52398655868794e-05*G0_2_0 - 6.52398655869805e-05*G0_2_1;
    A[1060] = 0.0260959462348353*G0_0_0 + 0.026095946234834*G0_0_1 + 0.0104383784939331*G0_0_2 + 0.0260959462348353*G0_1_0 + 0.026095946234834*G0_1_1 + 0.010438378493933*G0_1_2 + 0.0104383784939335*G0_2_0 + 0.0104383784939329*G0_2_1 + 0.180714427676231*G0_2_2;
    A[1061] = 0.0260959462348346*G0_0_0 + 0.0260959462348346*G0_0_1 - 0.00587158790283847*G0_0_2 + 0.0260959462348345*G0_1_0 + 0.0260959462348346*G0_1_1 - 0.00587158790283838*G0_1_2 - 0.00587158790283843*G0_2_0 - 0.00587158790283754*G0_2_1 + 0.00130479731174122*G0_2_2;
    A[1062] = 0.0260959462348355*G0_0_0 + 0.0260959462348357*G0_0_1 - 0.00587158790283676*G0_0_2 + 0.0260959462348355*G0_1_0 + 0.0260959462348357*G0_1_1 - 0.00587158790283679*G0_1_2 - 0.00587158790283672*G0_2_0 - 0.00587158790283739*G0_2_1 + 0.00130479731174245*G0_2_2;
    A[1063] = -0.185933616923197*G0_0_0 - 0.185933616923197*G0_0_1 - 0.0955764030850818*G0_0_2 - 0.185933616923197*G0_1_0 - 0.185933616923197*G0_1_1 - 0.0955764030850818*G0_1_2 - 0.0955764030850821*G0_2_0 - 0.0955764030850822*G0_2_1 - 0.161794866655975*G0_2_2;
    A[1064] = 0.00480034930989789*G0_0_0 + 0.00480034930989792*G0_0_1 + 0.00480034930989792*G0_0_2 + 0.0048003493098979*G0_1_0 + 0.00480034930989793*G0_1_1 + 0.00480034930989793*G0_1_2 + 0.0153313684129656*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[1065] = -0.00480034930989786*G0_0_0 - 0.00480034930989788*G0_1_0;
    A[1066] = -0.00480034930989789*G0_0_1 - 0.00480034930989789*G0_1_1;
    A[1067] = -0.0372571824394741*G0_0_2 - 0.0372571824394741*G0_1_2 - 0.0551276864210891*G0_2_2;
    A[1068] = 0.0328352243499813*G0_0_1 + 0.00537576492437599*G0_0_2 + 0.0328352243499813*G0_1_1 + 0.005375764924376*G0_1_2 + 0.00537576492437614*G0_2_1;
    A[1069] = -0.078894569454465*G0_0_1 - 0.0189782768992833*G0_0_2 - 0.078894569454465*G0_1_1 - 0.0189782768992834*G0_1_2 - 0.0189782768992841*G0_2_1 + 0.00939454064454055*G0_2_2;
    A[1070] = 0.108637424175618*G0_0_1 + 0.0387263842124946*G0_0_2 + 0.108637424175618*G0_1_1 + 0.0387263842124945*G0_1_2 + 0.0387263842124954*G0_2_1 - 0.0186912214907006*G0_2_2;
    A[1071] = -0.139822079926246*G0_0_1 - 0.0699110399631223*G0_0_2 - 0.139822079926246*G0_1_1 - 0.0699110399631223*G0_1_2 - 0.0699110399631233*G0_2_1 + 0.0196371995417135*G0_2_2;
    A[1072] = 0.0328352243499812*G0_0_0 + 0.00537576492437601*G0_0_2 + 0.0328352243499812*G0_1_0 + 0.00537576492437601*G0_1_2 + 0.00537576492437638*G0_2_0;
    A[1073] = -0.0788945694544651*G0_0_0 - 0.0189782768992837*G0_0_2 - 0.0788945694544652*G0_1_0 - 0.0189782768992837*G0_1_2 - 0.0189782768992844*G0_2_0 + 0.00939454064453983*G0_2_2;
    A[1074] = 0.108637424175618*G0_0_0 + 0.038726384212495*G0_0_2 + 0.108637424175618*G0_1_0 + 0.038726384212495*G0_1_2 + 0.0387263842124957*G0_2_0 - 0.0186912214907001*G0_2_2;
    A[1075] = -0.139822079926246*G0_0_0 - 0.0699110399631227*G0_0_2 - 0.139822079926246*G0_1_0 - 0.0699110399631227*G0_1_2 - 0.0699110399631235*G0_2_0 + 0.019637199541713*G0_2_2;
    A[1076] = 0.00537576492437608*G0_0_0 + 0.00537576492437604*G0_0_1 + 0.00537576492437611*G0_1_0 + 0.00537576492437605*G0_1_1;
    A[1077] = 0.00228991928210667*G0_0_0 + 0.00228991928210682*G0_0_1 + 0.00228991928210665*G0_1_0 + 0.00228991928210682*G0_1_1;
    A[1078] = 0.00228991928210672*G0_0_0 + 0.00228991928210662*G0_0_1 + 0.00228991928210671*G0_1_0 + 0.0022899192821066*G0_1_1;
    A[1079] = 0.00537576492437596*G0_0_0 + 0.00537576492437613*G0_0_1 + 0.00537576492437596*G0_1_0 + 0.00537576492437613*G0_1_1;
    A[1080] = -0.0328352243499812*G0_0_0 - 0.0328352243499813*G0_0_1 - 0.0274594594256052*G0_0_2 - 0.0328352243499812*G0_1_0 - 0.0328352243499813*G0_1_1 - 0.0274594594256053*G0_1_2 - 0.0537511252572015*G0_2_0 - 0.0537511252572015*G0_2_1 - 0.0812105846828066*G0_2_2;
    A[1081] = 0.0788945694544649*G0_0_0 + 0.0788945694544652*G0_0_1 + 0.0599162925551815*G0_0_2 + 0.078894569454465*G0_1_0 + 0.0788945694544652*G0_1_1 + 0.0599162925551815*G0_1_2 + 0.103529142700149*G0_2_0 + 0.103529142700149*G0_2_1 + 0.185268170294211*G0_2_2;
    A[1082] = -0.108637424175618*G0_0_0 - 0.108637424175618*G0_0_1 - 0.0699110399631235*G0_0_2 - 0.108637424175618*G0_1_0 - 0.108637424175618*G0_1_1 - 0.0699110399631235*G0_1_2 - 0.135020425819037*G0_2_0 - 0.135020425819037*G0_2_1 - 0.243357746612954*G0_2_2;
    A[1083] = 0.139822079926246*G0_0_0 + 0.139822079926246*G0_0_1 + 0.0699110399631234*G0_0_2 + 0.139822079926246*G0_1_0 + 0.139822079926246*G0_1_1 + 0.0699110399631234*G0_1_2 + 0.0699110399631234*G0_2_0 + 0.0699110399631234*G0_2_1 + 0.179096479009673*G0_2_2;
    A[1084] = -0.00537576492437612*G0_0_0 - 0.00537576492437612*G0_0_2 - 0.00537576492437613*G0_1_0 - 0.00537576492437613*G0_1_2 + 0.0220836945012291*G0_2_0 - 0.00537576492437597*G0_2_1 + 0.0220836945012291*G0_2_2;
    A[1085] = -0.00228991928210672*G0_0_0 - 0.00228991928210663*G0_0_2 - 0.0022899192821067*G0_1_0 - 0.00228991928210661*G0_1_2 + 0.00658922642429585*G0_2_0 - 0.00228991928210674*G0_2_1 + 0.00658922642429582*G0_2_2;
    A[1086] = -0.00228991928210673*G0_0_0 - 0.00228991928210673*G0_0_2 - 0.00228991928210674*G0_1_0 - 0.00228991928210674*G0_1_2 + 0.00939454064454059*G0_2_0 - 0.00228991928210678*G0_2_1 + 0.00939454064454059*G0_2_2;
    A[1087] = -0.005375764924376*G0_0_0 - 0.00537576492437602*G0_0_2 - 0.005375764924376*G0_1_0 - 0.00537576492437602*G0_1_2 - 0.00537576492437601*G0_2_1;
    A[1088] = -0.00537576492437602*G0_0_1 - 0.00537576492437596*G0_0_2 - 0.00537576492437603*G0_1_1 - 0.00537576492437597*G0_1_2 - 0.0053757649243759*G0_2_0 + 0.0220836945012291*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1089] = -0.00228991928210695*G0_0_1 - 0.002289919282107*G0_0_2 - 0.00228991928210695*G0_1_1 - 0.002289919282107*G0_1_2 - 0.00228991928210699*G0_2_0 + 0.00658922642429554*G0_2_1 + 0.00658922642429552*G0_2_2;
    A[1090] = -0.00228991928210661*G0_0_1 - 0.00228991928210662*G0_0_2 - 0.00228991928210659*G0_1_1 - 0.0022899192821066*G0_1_2 - 0.00228991928210655*G0_2_0 + 0.0093945406445408*G0_2_1 + 0.00939454064454076*G0_2_2;
    A[1091] = -0.00537576492437609*G0_0_1 - 0.00537576492437604*G0_0_2 - 0.0053757649243761*G0_1_1 - 0.00537576492437604*G0_1_2 - 0.00537576492437617*G0_2_0;
    A[1092] = -0.01187365553685*G0_0_0 - 0.0118736555368499*G0_0_1 + 0.00939454064454066*G0_0_2 - 0.0118736555368502*G0_1_0 - 0.01187365553685*G0_1_1 + 0.00939454064454062*G0_1_2 + 0.00939454064454044*G0_2_0 + 0.00939454064454079*G0_2_1;
    A[1093] = -0.00254435475789674*G0_0_0 - 0.0025443547578967*G0_0_1 + 0.0032946132121474*G0_0_2 - 0.0025443547578967*G0_1_0 - 0.00254435475789668*G0_1_1 + 0.0032946132121474*G0_1_2 + 0.00329461321214841*G0_2_0 + 0.00329461321214846*G0_2_1;
    A[1094] = -0.011873655536849*G0_0_0 - 0.0118736555368495*G0_0_1 + 0.00939454064454112*G0_0_2 - 0.0118736555368491*G0_1_0 - 0.0118736555368496*G0_1_1 + 0.00939454064454108*G0_1_2 + 0.00939454064454058*G0_2_0 + 0.00939454064454039*G0_2_1;
    A[1095] = 0.0429604514890974*G0_0_0 + 0.042960451489097*G0_0_1 + 0.000685018588664392*G0_0_2 + 0.0429604514890976*G0_1_0 + 0.0429604514890971*G0_1_1 + 0.000685018588664357*G0_1_2 + 0.000685018588665032*G0_2_0 + 0.000685018588664182*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[1096] = 0.0429604514890969*G0_0_0 + 0.0429604514890976*G0_0_1 + 0.000685018588664637*G0_0_2 + 0.0429604514890968*G0_1_0 + 0.0429604514890975*G0_1_1 + 0.000685018588664562*G0_1_2 + 0.000685018588664249*G0_2_0 + 0.000685018588664847*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[1097] = -0.12473862300251*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0284445813959693*G0_0_2 - 0.12473862300251*G0_1_0 - 0.124738623002511*G0_1_1 - 0.0284445813959693*G0_1_2 - 0.0284445813959702*G0_2_0 - 0.0284445813959701*G0_2_1 - 0.0606730749959909*G0_2_2;
    A[1098] = 0.0118736555368505*G0_0_0 + 0.0212681961813908*G0_0_2 + 0.0118736555368505*G0_1_0 + 0.0212681961813909*G0_1_2 - 0.014613729891508*G0_2_0 + 0.0212681961813904*G0_2_1 - 0.0693499771190741*G0_2_2;
    A[1099] = 0.00254435475789621*G0_0_0 + 0.00583896797004378*G0_0_2 + 0.00254435475789615*G0_1_0 + 0.00583896797004375*G0_1_2 - 0.00590420783563106*G0_2_0 + 0.0058389679700445*G0_2_1 - 0.0318696743392921*G0_2_2;
    A[1100] = 0.011873655536849*G0_0_0 + 0.0212681961813901*G0_0_2 + 0.0118736555368491*G0_1_0 + 0.0212681961813902*G0_1_2 - 0.00939454064454054*G0_2_0 + 0.0212681961813905*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[1101] = -0.0429604514890964*G0_0_0 - 0.0422754329004326*G0_0_2 - 0.0429604514890963*G0_1_0 - 0.0422754329004326*G0_1_2 - 0.0359145460056913*G0_2_0 - 0.0422754329004324*G0_2_1 + 0.0920860702761741*G0_2_2;
    A[1102] = -0.042960451489097*G0_0_0 - 0.0422754329004319*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.0422754329004319*G0_1_2 - 0.000685018588664295*G0_2_0 - 0.0422754329004328*G0_2_1 + 0.043971669405697*G0_2_2;
    A[1103] = 0.12473862300251*G0_0_0 + 0.0962940416065403*G0_0_2 + 0.124738623002511*G0_1_0 + 0.0962940416065403*G0_1_2 + 0.0284445813959703*G0_2_0 + 0.0962940416065412*G0_2_1 - 0.0644569872000425*G0_2_2;
    A[1104] = 0.0118736555368502*G0_0_1 + 0.0212681961813907*G0_0_2 + 0.0118736555368502*G0_1_1 + 0.0212681961813908*G0_1_2 + 0.0212681961813903*G0_2_0 - 0.0146137298915081*G0_2_1 - 0.069349977119074*G0_2_2;
    A[1105] = 0.00254435475789586*G0_0_1 + 0.00583896797004444*G0_0_2 + 0.00254435475789589*G0_1_1 + 0.00583896797004445*G0_1_2 + 0.00583896797004454*G0_2_0 - 0.00590420783563157*G0_2_1 - 0.031869674339292*G0_2_2;
    A[1106] = 0.0118736555368499*G0_0_1 + 0.0212681961813903*G0_0_2 + 0.01187365553685*G0_1_1 + 0.0212681961813904*G0_1_2 + 0.0212681961813903*G0_2_0 - 0.00939454064454075*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[1107] = -0.0429604514890965*G0_0_1 - 0.0422754329004332*G0_0_2 - 0.0429604514890965*G0_1_1 - 0.0422754329004332*G0_1_2 - 0.0422754329004325*G0_2_0 - 0.0359145460056914*G0_2_1 + 0.0920860702761741*G0_2_2;
    A[1108] = -0.0429604514890969*G0_0_1 - 0.0422754329004323*G0_0_2 - 0.0429604514890969*G0_1_1 - 0.0422754329004324*G0_1_2 - 0.0422754329004324*G0_2_0 - 0.000685018588664081*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[1109] = 0.124738623002511*G0_0_1 + 0.096294041606541*G0_0_2 + 0.12473862300251*G0_1_1 + 0.096294041606541*G0_1_2 + 0.096294041606541*G0_2_0 + 0.0284445813959702*G0_2_1 - 0.0644569872000422*G0_2_2;
    A[1110] = -0.00939454064454031*G0_0_2 - 0.00939454064454028*G0_1_2 - 0.00939454064454077*G0_2_0 - 0.0093945406445407*G0_2_1 + 0.0453417065830253*G0_2_2;
    A[1111] = -0.00329461321214808*G0_0_2 - 0.00329461321214808*G0_1_2 - 0.00329461321214786*G0_2_0 - 0.00329461321214772*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1112] = -0.00939454064454061*G0_0_2 - 0.00939454064454057*G0_1_2 - 0.00939454064454084*G0_2_0 - 0.00939454064454052*G0_2_1;
    A[1113] = -0.00329461321214795*G0_0_2 - 0.003294613212148*G0_1_2 - 0.00329461321214781*G0_2_0 - 0.00329461321214784*G0_2_1 + 0.0226708532915134*G0_2_2;
    A[1114] = -0.00329461321214754*G0_0_2 - 0.00329461321214754*G0_1_2 - 0.00329461321214789*G0_2_0 - 0.00329461321214793*G0_2_1;
    A[1115] = -0.00939454064454091*G0_0_2 - 0.00939454064454087*G0_1_2 - 0.00939454064454068*G0_2_0 - 0.00939454064454066*G0_2_1;
    A[1116] = 0.00260959462348375*G0_0_2 + 0.00260959462348379*G0_1_2 + 0.00260959462348415*G0_2_0 + 0.00260959462348455*G0_2_1 - 0.151356488162042*G0_2_2;
    A[1117] = 0.00260959462348359*G0_0_2 + 0.00260959462348362*G0_1_2 + 0.00260959462348342*G0_2_0 + 0.00260959462348303*G0_2_1 - 0.0453417065830257*G0_2_2;
    A[1118] = 0.00260959462348317*G0_0_2 + 0.0026095946234833*G0_1_2 + 0.0026095946234827*G0_2_0 + 0.00260959462348329*G0_2_1 - 0.0453417065830261*G0_2_2;
    A[1119] = 0.035229527417026*G0_0_2 + 0.0352295274170259*G0_1_2 + 0.0352295274170263*G0_2_0 + 0.0352295274170263*G0_2_1 + 0.166687856575007*G0_2_2;
    A[1120] = -0.0178705039816149*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0551276864210887*G0_1_0 - 0.0551276864210887*G0_1_1 - 0.0551276864210887*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0178705039816149*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[1121] = -0.00480034930989791*G0_0_0 - 0.00480034930989794*G0_2_0;
    A[1122] = 0.0105310191030676*G0_0_1 + 0.0153313684129656*G0_1_1 + 0.0105310191030677*G0_2_1;
    A[1123] = -0.00480034930989781*G0_0_2 - 0.00480034930989782*G0_2_2;
    A[1124] = 0.027459459425605*G0_0_1 + 0.0328352243499811*G0_0_2 + 0.022083694501229*G0_1_1 + 0.0274594594256053*G0_1_2 + 0.0274594594256051*G0_2_1 + 0.0328352243499812*G0_2_2;
    A[1125] = 0.00887914570640263*G0_0_1 + 0.0111690649885091*G0_0_2 + 0.006589226424296*G0_1_1 + 0.00887914570640228*G0_1_2 + 0.0088791457064026*G0_2_1 + 0.011169064988509*G0_2_2;
    A[1126] = 0.0116844599266471*G0_0_1 + 0.0139743792087544*G0_0_2 + 0.00939454064454035*G0_1_1 + 0.0116844599266477*G0_1_2 + 0.0116844599266471*G0_2_1 + 0.0139743792087545*G0_2_2;
    A[1127] = 0.00537576492437612*G0_0_1 + 0.0107515298487519*G0_0_2 + 0.00537576492437591*G0_1_2 + 0.00537576492437612*G0_2_1 + 0.0107515298487519*G0_2_2;
    A[1128] = 0.00537576492437596*G0_0_0 + 0.00537576492437582*G0_0_2 + 0.00537576492437596*G0_2_0 + 0.00537576492437578*G0_2_2;
    A[1129] = 0.00228991928210677*G0_0_0 + 0.00228991928210699*G0_0_2 + 0.00228991928210679*G0_2_0 + 0.0022899192821071*G0_2_2;
    A[1130] = 0.00228991928210653*G0_0_0 + 0.0022899192821063*G0_0_2 + 0.00228991928210652*G0_2_0 + 0.0022899192821062*G0_2_2;
    A[1131] = 0.00537576492437614*G0_0_0 + 0.00537576492437606*G0_0_2 + 0.00537576492437614*G0_2_0 + 0.00537576492437611*G0_2_2;
    A[1132] = 0.0107515298487518*G0_0_0 + 0.00537576492437571*G0_0_1 + 0.00537576492437562*G0_1_0 + 0.0107515298487517*G0_2_0 + 0.00537576492437566*G0_2_1;
    A[1133] = 0.0139743792087542*G0_0_0 + 0.0116844599266475*G0_0_1 + 0.0116844599266476*G0_1_0 + 0.00939454064454036*G0_1_1 + 0.0139743792087542*G0_2_0 + 0.0116844599266475*G0_2_1;
    A[1134] = 0.0111690649885089*G0_0_0 + 0.00887914570640211*G0_0_1 + 0.00887914570640209*G0_1_0 + 0.00658922642429564*G0_1_1 + 0.0111690649885089*G0_2_0 + 0.00887914570640209*G0_2_1;
    A[1135] = 0.0328352243499811*G0_0_0 + 0.0274594594256052*G0_0_1 + 0.0274594594256053*G0_1_0 + 0.0220836945012291*G0_1_1 + 0.0328352243499812*G0_2_0 + 0.0274594594256053*G0_2_1;
    A[1136] = 0.0895482395048357*G0_0_0 + 0.0895482395048358*G0_0_1 + 0.0196371995417134*G0_0_2 + 0.0196371995417134*G0_1_0 + 0.0196371995417133*G0_1_1 + 0.0895482395048359*G0_1_2 + 0.0895482395048358*G0_2_0 + 0.0895482395048358*G0_2_1 + 0.0196371995417134*G0_2_2;
    A[1137] = -0.0574176057031957*G0_0_0 - 0.0574176057031957*G0_0_1 + 0.012493434259927*G0_0_2 - 0.0186912214907008*G0_1_0 - 0.0186912214907008*G0_1_1 - 0.0574176057031957*G0_1_2 - 0.0574176057031956*G0_2_0 - 0.0574176057031956*G0_2_1 + 0.0124934342599271*G0_2_2;
    A[1138] = 0.0283728175438242*G0_0_0 + 0.0283728175438243*G0_0_1 - 0.0315434750113566*G0_0_2 + 0.0093945406445407*G0_1_0 + 0.00939454064454071*G0_1_1 + 0.0283728175438243*G0_1_2 + 0.0283728175438242*G0_2_0 + 0.0283728175438243*G0_2_1 - 0.0315434750113567*G0_2_2;
    A[1139] = -0.00537576492437612*G0_0_0 - 0.00537576492437613*G0_0_1 + 0.0220836945012291*G0_0_2 - 0.00537576492437598*G0_1_2 - 0.00537576492437612*G0_2_0 - 0.00537576492437613*G0_2_1 + 0.0220836945012291*G0_2_2;
    A[1140] = 0.179096479009671*G0_0_0 + 0.109185439046549*G0_0_1 + 0.179096479009672*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009672*G0_1_1 + 0.109185439046549*G0_1_2 + 0.179096479009672*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009672*G0_2_2;
    A[1141] = -0.147063705006411*G0_0_0 - 0.17344670664983*G0_0_1 - 0.147063705006411*G0_0_2 - 0.108337320793917*G0_1_0 - 0.243357746612953*G0_1_1 - 0.108337320793917*G0_1_2 - 0.147063705006411*G0_2_0 - 0.17344670664983*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1142] = 0.100717304493345*G0_0_0 + 0.125351877739029*G0_0_1 + 0.100717304493345*G0_0_2 + 0.0817390275940618*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0817390275940618*G0_1_2 + 0.100717304493345*G0_2_0 + 0.125351877739029*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1143] = -0.032835224349981*G0_0_0 - 0.0537511252572011*G0_0_1 - 0.032835224349981*G0_0_2 - 0.0274594594256053*G0_1_0 - 0.0812105846828062*G0_1_1 - 0.0274594594256052*G0_1_2 - 0.0328352243499812*G0_2_0 - 0.0537511252572012*G0_2_1 - 0.0328352243499811*G0_2_2;
    A[1144] = 0.0196371995417133*G0_0_0 + 0.0895482395048357*G0_0_1 + 0.0895482395048357*G0_0_2 + 0.0895482395048358*G0_1_0 + 0.0196371995417131*G0_1_1 + 0.0196371995417132*G0_1_2 + 0.0196371995417133*G0_2_0 + 0.0895482395048358*G0_2_1 + 0.0895482395048358*G0_2_2;
    A[1145] = 0.0124934342599271*G0_0_0 - 0.0574176057031951*G0_0_1 - 0.0574176057031951*G0_0_2 - 0.0574176057031952*G0_1_0 - 0.0186912214907002*G0_1_1 - 0.0186912214907002*G0_1_2 + 0.0124934342599271*G0_2_0 - 0.0574176057031951*G0_2_1 - 0.0574176057031951*G0_2_2;
    A[1146] = -0.0315434750113566*G0_0_0 + 0.0283728175438242*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.00939454064454062*G0_1_1 + 0.00939454064454063*G0_1_2 - 0.0315434750113566*G0_2_0 + 0.0283728175438243*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1147] = 0.022083694501229*G0_0_0 - 0.00537576492437573*G0_0_1 - 0.00537576492437571*G0_0_2 - 0.00537576492437607*G0_1_0 + 0.022083694501229*G0_2_0 - 0.00537576492437568*G0_2_1 - 0.00537576492437566*G0_2_2;
    A[1148] = 0.0187890812890812*G0_0_0 + 0.00939454064454039*G0_0_1 + 0.0187890812890806*G0_0_2 + 0.00939454064454088*G0_1_0 + 0.00939454064453989*G0_1_2 + 0.0187890812890812*G0_2_0 + 0.00939454064454039*G0_2_1 + 0.0187890812890806*G0_2_2;
    A[1149] = 0.0292600797158081*G0_0_0 + 0.0259654665036612*G0_0_1 + 0.0292600797158088*G0_0_2 + 0.0259654665036599*G0_1_0 + 0.022670853291513*G0_1_1 + 0.0259654665036605*G0_1_2 + 0.0292600797158081*G0_2_0 + 0.0259654665036611*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[1150] = 0.0641307878721064*G0_0_0 + 0.054736247227565*G0_0_1 + 0.064130787872106*G0_0_2 + 0.054736247227566*G0_1_0 + 0.0453417065830246*G0_1_1 + 0.0547362472275657*G0_1_2 + 0.0641307878721066*G0_2_0 + 0.0547362472275651*G0_2_1 + 0.0641307878721062*G0_2_2;
    A[1151] = 0.00658922642429612*G0_0_0 + 0.00329461321214801*G0_0_1 + 0.00658922642429614*G0_0_2 + 0.00329461321214827*G0_1_0 + 0.00329461321214881*G0_1_2 + 0.00658922642429607*G0_2_0 + 0.003294613212148*G0_2_1 + 0.00658922642429619*G0_2_2;
    A[1152] = 0.0292600797158084*G0_0_0 + 0.0259654665036606*G0_0_1 + 0.029260079715808*G0_0_2 + 0.0259654665036607*G0_1_0 + 0.0226708532915129*G0_1_1 + 0.0259654665036603*G0_1_2 + 0.0292600797158084*G0_2_0 + 0.0259654665036606*G0_2_1 + 0.0292600797158079*G0_2_2;
    A[1153] = 0.0187890812890807*G0_0_0 + 0.00939454064454029*G0_0_1 + 0.018789081289081*G0_0_2 + 0.00939454064454022*G0_1_0 + 0.00939454064454031*G0_1_2 + 0.0187890812890808*G0_2_0 + 0.00939454064454031*G0_2_1 + 0.018789081289081*G0_2_2;
    A[1154] = -0.0644569872000419*G0_0_0 - 0.160751028806582*G0_0_1 - 0.257045070413124*G0_0_2 - 0.092901568596012*G0_1_0 - 0.0644569872000419*G0_1_1 - 0.160751028806583*G0_1_2 - 0.064456987200042*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413124*G0_2_2;
    A[1155] = 0.127315597693201*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077038*G0_0_2 + 0.128000616281866*G0_1_0 + 0.0920860702761736*G0_1_1 + 0.134361503176606*G0_1_2 + 0.127315597693201*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1156] = -0.0641307878721064*G0_0_0 - 0.0906181733004643*G0_0_1 - 0.111886369481854*G0_0_2 - 0.054736247227566*G0_1_0 - 0.069349977119074*G0_1_1 - 0.0906181733004643*G0_1_2 - 0.0641307878721066*G0_2_0 - 0.0906181733004646*G0_2_1 - 0.111886369481855*G0_2_2;
    A[1157] = 0.043971669405696*G0_0_0 + 0.0862471023061292*G0_0_1 + 0.128522535206562*G0_0_2 + 0.0446566879943613*G0_1_0 + 0.043971669405697*G0_1_1 + 0.0862471023061296*G0_1_2 + 0.043971669405696*G0_2_0 + 0.0862471023061291*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1158] = -0.0292600797158084*G0_0_0 - 0.0377086423093364*G0_0_1 - 0.0435476102793808*G0_0_2 - 0.0259654665036607*G0_1_0 - 0.0318696743392924*G0_1_1 - 0.0377086423093367*G0_1_2 - 0.0292600797158084*G0_2_0 - 0.0377086423093364*G0_2_1 - 0.0435476102793809*G0_2_2;
    A[1159] = -0.0187890812890808*G0_0_0 - 0.0400572774704712*G0_0_1 - 0.061325473651862*G0_0_2 - 0.00939454064454033*G0_1_0 - 0.018789081289081*G0_1_1 - 0.0400572774704718*G0_1_2 - 0.0187890812890809*G0_2_0 - 0.0400572774704712*G0_2_1 - 0.061325473651862*G0_2_2;
    A[1160] = -0.128522535206561*G0_0_0 - 0.0322284936000209*G0_0_1 - 0.128522535206562*G0_0_2 - 0.032228493600021*G0_1_0 - 0.060673074995991*G0_1_1 - 0.0322284936000211*G0_1_2 - 0.128522535206561*G0_2_0 - 0.0322284936000208*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1161] = 0.0642612676032807*G0_0_0 + 0.0219858347028488*G0_0_1 + 0.0642612676032806*G0_0_2 + 0.021985834702848*G0_1_0 + 0.0226708532915128*G0_1_1 + 0.0219858347028481*G0_1_2 + 0.0642612676032807*G0_2_0 + 0.0219858347028488*G0_2_1 + 0.0642612676032806*G0_2_2;
    A[1162] = -0.030662736825931*G0_0_0 - 0.00939454064454058*G0_0_1 - 0.0306627368259314*G0_0_2 - 0.0093945406445402*G0_1_0 - 0.00939454064454098*G0_1_2 - 0.030662736825931*G0_2_0 - 0.00939454064454059*G0_2_1 - 0.0306627368259316*G0_2_2;
    A[1163] = 0.0642612676032809*G0_0_0 + 0.0219858347028479*G0_0_1 + 0.0642612676032812*G0_0_2 + 0.0219858347028489*G0_1_0 + 0.0226708532915132*G0_1_1 + 0.0219858347028487*G0_1_2 + 0.0642612676032808*G0_2_0 + 0.0219858347028479*G0_2_1 + 0.0642612676032812*G0_2_2;
    A[1164] = -0.00913358118219224*G0_0_0 - 0.00329461321214778*G0_0_1 - 0.00913358118219192*G0_0_2 - 0.00329461321214843*G0_1_0 - 0.00329461321214753*G0_1_2 - 0.00913358118219221*G0_2_0 - 0.00329461321214778*G0_2_1 - 0.00913358118219182*G0_2_2;
    A[1165] = -0.0306627368259308*G0_0_0 - 0.00939454064454035*G0_0_1 - 0.0306627368259312*G0_0_2 - 0.00939454064454045*G0_1_0 - 0.0093945406445409*G0_1_2 - 0.0306627368259308*G0_2_0 - 0.00939454064454038*G0_2_1 - 0.0306627368259312*G0_2_2;
    A[1166] = -0.257045070413123*G0_0_0 - 0.160751028806582*G0_0_1 - 0.0644569872000413*G0_0_2 - 0.160751028806583*G0_1_0 - 0.0644569872000416*G0_1_1 - 0.0929015685960116*G0_1_2 - 0.257045070413123*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000414*G0_2_2;
    A[1167] = 0.128522535206561*G0_0_0 + 0.0862471023061283*G0_0_1 + 0.0439716694056963*G0_0_2 + 0.0862471023061288*G0_1_0 + 0.0439716694056957*G0_1_1 + 0.0446566879943604*G0_1_2 + 0.128522535206561*G0_2_0 + 0.0862471023061283*G0_2_1 + 0.0439716694056963*G0_2_2;
    A[1168] = -0.0613254736518615*G0_0_0 - 0.0400572774704717*G0_0_1 - 0.0187890812890808*G0_0_2 - 0.0400572774704709*G0_1_0 - 0.018789081289081*G0_1_1 - 0.00939454064454002*G0_1_2 - 0.0613254736518616*G0_2_0 - 0.0400572774704717*G0_2_1 - 0.0187890812890807*G0_2_2;
    A[1169] = 0.176636936077038*G0_0_0 + 0.134361503176606*G0_0_1 + 0.127315597693201*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761738*G0_1_1 + 0.128000616281865*G0_1_2 + 0.176636936077038*G0_2_0 + 0.134361503176606*G0_2_1 + 0.127315597693201*G0_2_2;
    A[1170] = -0.0435476102793805*G0_0_0 - 0.0377086423093354*G0_0_1 - 0.0292600797158084*G0_0_2 - 0.0377086423093364*G0_1_0 - 0.031869674339291*G0_1_1 - 0.0259654665036604*G0_1_2 - 0.0435476102793804*G0_2_0 - 0.0377086423093353*G0_2_1 - 0.0292600797158084*G0_2_2;
    A[1171] = -0.111886369481854*G0_0_0 - 0.0906181733004645*G0_0_1 - 0.0641307878721061*G0_0_2 - 0.090618173300464*G0_1_0 - 0.0693499771190744*G0_1_1 - 0.0547362472275657*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004648*G0_2_1 - 0.0641307878721063*G0_2_2;
    A[1172] = 0.0962288017409537*G0_0_0 + 0.131458329157981*G0_0_1 + 0.0962288017409526*G0_0_2 + 0.131458329157981*G0_1_0 + 0.166687856575008*G0_1_1 + 0.131458329157981*G0_1_2 + 0.0962288017409537*G0_2_0 + 0.131458329157981*G0_2_1 + 0.0962288017409527*G0_2_2;
    A[1173] = -0.0505608958299931*G0_0_0 - 0.0479513012065098*G0_0_1 - 0.0505608958299926*G0_0_2 - 0.0479513012065098*G0_1_0 - 0.0453417065830257*G0_1_1 - 0.0479513012065092*G0_1_2 - 0.050560895829993*G0_2_0 - 0.0479513012065098*G0_2_1 - 0.0505608958299926*G0_2_2;
    A[1174] = -0.156575677409009*G0_0_0 - 0.153966082785525*G0_0_1 - 0.156575677409009*G0_0_2 - 0.153966082785526*G0_1_0 - 0.151356488162042*G0_1_1 - 0.153966082785526*G0_1_2 - 0.156575677409009*G0_2_0 - 0.153966082785525*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1175] = -0.0505608958299922*G0_0_0 - 0.0479513012065086*G0_0_1 - 0.0505608958299925*G0_0_2 - 0.0479513012065095*G0_1_0 - 0.0453417065830262*G0_1_1 - 0.0479513012065094*G0_1_2 - 0.0505608958299922*G0_2_0 - 0.0479513012065084*G0_2_1 - 0.0505608958299924*G0_2_2;
    A[1176] = 0.0214691349673986*G0_0_0 + 0.0214691349673986*G0_0_1 + 0.0214691349673985*G0_0_2 + 0.0551276864210886*G0_1_0 + 0.0551276864210886*G0_1_1 + 0.0551276864210886*G0_1_2 + 0.0214691349673986*G0_2_0 + 0.0214691349673985*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[1177] = -0.00336507226698196*G0_0_0 - 0.00336507226698194*G0_2_0;
    A[1178] = -0.0232632312710436*G0_0_1 - 0.0388177200243171*G0_1_1 - 0.0232632312710436*G0_2_1;
    A[1179] = -0.00336507226698201*G0_0_2 - 0.00336507226698201*G0_2_2;
    A[1180] = -0.059916292555181*G0_0_1 - 0.0788945694544647*G0_0_2 - 0.0315434750113566*G0_1_1 - 0.0599162925551812*G0_1_2 - 0.0599162925551811*G0_2_1 - 0.0788945694544648*G0_2_2;
    A[1181] = -0.00578025209101582*G0_0_1 - 0.00523223722008425*G0_0_2 - 0.00626302709636063*G0_1_1 - 0.00578025209101562*G0_1_2 - 0.00578025209101584*G0_2_1 - 0.00523223722008417*G0_2_2;
    A[1182] = -0.0131523569023567*G0_0_1 - 0.0263047138047136*G0_0_2 + 6.52398655875253e-05*G0_1_1 - 0.013152356902357*G0_1_2 - 0.0131523569023567*G0_2_1 - 0.0263047138047137*G0_2_2;
    A[1183] = 0.00228991928210674*G0_0_1 + 0.0139743792087541*G0_0_2 + 0.00228991928210691*G0_1_2 + 0.00228991928210665*G0_2_1 + 0.013974379208754*G0_2_2;
    A[1184] = 0.00228991928210665*G0_0_0 + 0.00228991928210683*G0_0_2 + 0.00228991928210671*G0_2_0 + 0.00228991928210691*G0_2_2;
    A[1185] = 0.00178757231708632*G0_0_0 + 0.00178757231708602*G0_0_2 + 0.00178757231708621*G0_2_0 + 0.00178757231708592*G0_2_2;
    A[1186] = 0.00178757231708649*G0_0_0 + 0.00178757231708656*G0_0_2 + 0.00178757231708659*G0_2_0 + 0.00178757231708666*G0_2_2;
    A[1187] = 0.00228991928210676*G0_0_0 + 0.00228991928210683*G0_0_2 + 0.00228991928210667*G0_2_0 + 0.00228991928210673*G0_2_2;
    A[1188] = 0.013974379208754*G0_0_0 + 0.00228991928210693*G0_0_1 + 0.00228991928210716*G0_1_0 + 0.0139743792087541*G0_2_0 + 0.00228991928210701*G0_2_1;
    A[1189] = -0.0263047138047136*G0_0_0 - 0.0131523569023572*G0_0_1 - 0.0131523569023571*G0_1_0 + 6.52398655871741e-05*G0_1_1 - 0.0263047138047136*G0_2_0 - 0.0131523569023572*G0_2_1;
    A[1190] = -0.00523223722008387*G0_0_0 - 0.00578025209101513*G0_0_1 - 0.00578025209101512*G0_1_0 - 0.00626302709635988*G0_1_1 - 0.00523223722008388*G0_2_0 - 0.00578025209101503*G0_2_1;
    A[1191] = -0.0788945694544648*G0_0_0 - 0.0599162925551812*G0_0_1 - 0.0599162925551813*G0_1_0 - 0.0315434750113568*G0_1_1 - 0.0788945694544649*G0_2_0 - 0.0599162925551814*G0_2_1;
    A[1192] = -0.0574176057031952*G0_0_0 - 0.0574176057031952*G0_0_1 - 0.0186912214907006*G0_0_2 + 0.0124934342599271*G0_1_0 + 0.0124934342599272*G0_1_1 - 0.0574176057031955*G0_1_2 - 0.0574176057031953*G0_2_0 - 0.0574176057031953*G0_2_1 - 0.0186912214907005*G0_2_2;
    A[1193] = 0.00506261356955807*G0_0_0 + 0.00506261356955805*G0_0_1 + 0.000260959462348702*G0_0_2 + 0.000260959462348514*G0_1_0 + 0.000260959462348462*G0_1_1 + 0.00506261356955813*G0_1_2 + 0.00506261356955816*G0_2_0 + 0.00506261356955814*G0_2_1 + 0.00026095946234867*G0_2_2;
    A[1194] = -0.000482775005344492*G0_0_0 - 0.000482775005344495*G0_0_1 - 0.00626302709636059*G0_0_2 + 6.52398655869482e-05*G0_1_0 + 6.52398655869541e-05*G0_1_1 - 0.000482775005344536*G0_1_2 - 0.000482775005344588*G0_2_0 - 0.000482775005344588*G0_2_1 - 0.00626302709636054*G0_2_2;
    A[1195] = -0.00228991928210672*G0_0_0 - 0.0022899192821067*G0_0_1 + 0.00658922642429585*G0_0_2 - 0.00228991928210674*G0_1_2 - 0.00228991928210663*G0_2_0 - 0.00228991928210661*G0_2_1 + 0.00658922642429581*G0_2_2;
    A[1196] = -0.147063705006411*G0_0_0 - 0.108337320793917*G0_0_1 - 0.147063705006411*G0_0_2 - 0.17344670664983*G0_1_0 - 0.243357746612953*G0_1_1 - 0.17344670664983*G0_1_2 - 0.147063705006411*G0_2_0 - 0.108337320793917*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1197] = 0.304226541205704*G0_0_0 + 0.226095278178609*G0_0_1 + 0.304226541205704*G0_0_2 + 0.226095278178609*G0_1_0 + 0.428965164208215*G0_1_1 + 0.226095278178609*G0_1_2 + 0.304226541205704*G0_2_0 + 0.226095278178609*G0_2_1 + 0.304226541205704*G0_2_2;
    A[1198] = -0.204187731314465*G0_0_0 - 0.219493003781196*G0_0_1 - 0.204187731314465*G0_0_2 - 0.167692550505049*G0_1_0 - 0.387185554286245*G0_1_1 - 0.167692550505049*G0_1_2 - 0.204187731314465*G0_2_0 - 0.219493003781196*G0_2_1 - 0.204187731314466*G0_2_2;
    A[1199] = 0.0788945694544646*G0_0_0 + 0.103529142700149*G0_0_1 + 0.0788945694544646*G0_0_2 + 0.0599162925551812*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0599162925551811*G0_1_2 + 0.0788945694544648*G0_2_0 + 0.103529142700149*G0_2_1 + 0.0788945694544647*G0_2_2;
    A[1200] = -0.0186912214907005*G0_0_0 - 0.0574176057031951*G0_0_1 - 0.0574176057031952*G0_0_2 - 0.0574176057031954*G0_1_0 + 0.0124934342599273*G0_1_1 + 0.0124934342599272*G0_1_2 - 0.0186912214907004*G0_2_0 - 0.0574176057031952*G0_2_1 - 0.0574176057031952*G0_2_2;
    A[1201] = 0.00026095946234844*G0_0_0 + 0.00506261356955793*G0_0_1 + 0.00506261356955787*G0_0_2 + 0.00506261356955793*G0_1_0 + 0.000260959462348072*G0_1_1 + 0.000260959462348119*G0_1_2 + 0.000260959462348375*G0_2_0 + 0.0050626135695579*G0_2_1 + 0.00506261356955784*G0_2_2;
    A[1202] = -0.00626302709636046*G0_0_0 - 0.000482775005344786*G0_0_1 - 0.000482775005344715*G0_0_2 - 0.000482775005344447*G0_1_0 + 6.52398655868995e-05*G0_1_1 + 6.52398655868993e-05*G0_1_2 - 0.00626302709636042*G0_2_0 - 0.000482775005344844*G0_2_1 - 0.000482775005344775*G0_2_2;
    A[1203] = 0.00658922642429582*G0_0_0 - 0.00228991928210679*G0_0_1 - 0.00228991928210683*G0_0_2 - 0.00228991928210666*G0_1_0 + 0.00658922642429578*G0_2_0 - 0.00228991928210687*G0_2_1 - 0.00228991928210691*G0_2_2;
    A[1204] = -0.014874689353856*G0_0_0 + 6.52398655871993e-05*G0_0_1 - 0.0148746893538559*G0_0_2 + 6.52398655869212e-05*G0_1_0 + 6.52398655879272e-05*G0_1_2 - 0.0148746893538558*G0_2_0 + 6.52398655873381e-05*G0_2_1 - 0.0148746893538556*G0_2_2;
    A[1205] = -0.0189848008858421*G0_0_0 - 0.00949240044292217*G0_0_1 - 0.0189848008858425*G0_0_2 - 0.00949240044292025*G0_1_0 - 0.000652398655871332*G0_1_1 - 0.00949240044292098*G0_1_2 - 0.0189848008858421*G0_2_0 - 0.00949240044292212*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[1206] = -0.127087258163646*G0_0_0 - 0.0893133759887216*G0_0_1 - 0.127087258163645*G0_0_2 - 0.0893133759887223*G0_1_0 - 0.0665446628988276*G0_1_1 - 0.0893133759887215*G0_1_2 - 0.127087258163646*G0_2_0 - 0.0893133759887216*G0_2_1 - 0.127087258163646*G0_2_2;
    A[1207] = 0.00450155072550883*G0_0_0 + 0.00257697469069014*G0_0_1 + 0.00450155072550868*G0_0_2 + 0.00257697469068948*G0_1_0 + 0.00257697469068903*G0_1_2 + 0.00450155072550878*G0_2_0 + 0.00257697469069006*G0_2_1 + 0.00450155072550857*G0_2_2;
    A[1208] = -0.0189848008858423*G0_0_0 - 0.00949240044292138*G0_0_1 - 0.0189848008858424*G0_0_2 - 0.00949240044292112*G0_1_0 - 0.000652398655871017*G0_1_1 - 0.00949240044292098*G0_1_2 - 0.0189848008858423*G0_2_0 - 0.0094924004429215*G0_2_1 - 0.0189848008858425*G0_2_2;
    A[1209] = -0.0148746893538552*G0_0_0 + 6.52398655878111e-05*G0_0_1 - 0.0148746893538551*G0_0_2 + 6.52398655879915e-05*G0_1_0 + 6.52398655878515e-05*G0_1_2 - 0.0148746893538554*G0_2_0 + 6.5239865587758e-05*G0_2_1 - 0.0148746893538551*G0_2_2;
    A[1210] = 0.132306447410612*G0_0_0 + 0.194675758911867*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0589768384907262*G0_1_0 - 0.00339247301052879*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410612*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1211] = -0.166948816037356*G0_0_0 - 0.250814663249556*G0_0_1 - 0.334680510461756*G0_0_2 - 0.160783648739377*G0_1_0 - 0.062238831770081*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037356*G0_2_0 - 0.250814663249556*G0_2_1 - 0.334680510461756*G0_2_2;
    A[1212] = 0.127087258163646*G0_0_0 + 0.173472802596065*G0_0_1 + 0.20941996853455*G0_0_2 + 0.0893133759887222*G0_1_0 + 0.0846813455320394*G0_1_1 + 0.173472802596065*G0_1_2 + 0.127087258163646*G0_2_0 + 0.173472802596065*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1213] = -0.0436454700777606*G0_0_0 - 0.0371541034518459*G0_0_1 + 0.0371867233846396*G0_0_2 - 0.00975335990526953*G0_1_0 - 0.00678494602105709*G0_1_1 - 0.0371541034518463*G0_1_2 - 0.0436454700777605*G0_2_0 - 0.0371541034518459*G0_2_1 + 0.0371867233846395*G0_2_2;
    A[1214] = 0.0189848008858423*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.0567586830607654*G0_0_2 + 0.00949240044292114*G0_1_0 + 0.0127870136550695*G0_1_1 + 0.0378717419733042*G0_1_2 + 0.0189848008858423*G0_2_0 + 0.037871741973304*G0_2_1 + 0.0567586830607654*G0_2_2;
    A[1215] = 0.0148746893538553*G0_0_0 + 0.013635131907701*G0_0_1 - 0.0254435475789637*G0_0_2 - 6.52398655878206e-05*G0_1_0 - 0.000130479731174506*G0_1_1 + 0.0136351319077014*G0_1_2 + 0.0148746893538555*G0_2_0 + 0.0136351319077011*G0_2_1 - 0.0254435475789636*G0_2_2;
    A[1216] = 0.0476251018785732*G0_0_0 - 0.014744209622682*G0_0_1 + 0.0476251018785732*G0_0_2 - 0.0147442096226819*G0_1_0 + 0.0476251018785732*G0_1_1 - 0.0147442096226818*G0_1_2 + 0.0476251018785733*G0_2_0 - 0.0147442096226821*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[1217] = 0.00880738185425651*G0_0_0 + 0.00231601522834095*G0_0_1 + 0.0088073818542568*G0_0_2 + 0.00231601522834188*G0_1_0 - 0.0006523986558711*G0_1_1 + 0.00231601522834173*G0_1_2 + 0.00880738185425647*G0_2_0 + 0.00231601522834089*G0_2_1 + 0.00880738185425683*G0_2_2;
    A[1218] = -0.00130479731174122*G0_0_0 - 6.52398655869307e-05*G0_0_1 - 0.00130479731174128*G0_0_2 - 6.52398655872735e-05*G0_1_0 - 6.52398655866655e-05*G0_1_2 - 0.00130479731174117*G0_2_0 - 6.52398655870697e-05*G0_2_1 - 0.00130479731174113*G0_2_2;
    A[1219] = 0.00880738185425683*G0_0_0 + 0.00231601522834213*G0_0_1 + 0.00880738185425695*G0_0_2 + 0.0023160152283414*G0_1_0 - 0.000652398655871117*G0_1_1 + 0.00231601522834167*G0_1_2 + 0.0088073818542569*G0_2_0 + 0.00231601522834224*G0_2_1 + 0.0088073818542569*G0_2_2;
    A[1220] = -0.0156575677409011*G0_0_0 - 0.00257697469069027*G0_0_1 - 0.0156575677409013*G0_0_2 - 0.00257697469068968*G0_1_0 - 0.00257697469069028*G0_1_2 - 0.0156575677409011*G0_2_0 - 0.00257697469069018*G0_2_1 - 0.0156575677409015*G0_2_2;
    A[1221] = -0.00130479731174206*G0_0_0 - 6.52398655877535e-05*G0_0_1 - 0.00130479731174184*G0_0_2 - 6.52398655873528e-05*G0_1_0 - 6.523986558706e-05*G0_1_2 - 0.00130479731174208*G0_2_0 - 6.52398655877005e-05*G0_2_1 - 0.00130479731174173*G0_2_2;
    A[1222] = 0.121346149991981*G0_0_0 + 0.194675758911867*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052918*G0_1_1 + 0.0589768384907258*G0_1_2 + 0.121346149991981*G0_2_0 + 0.194675758911867*G0_2_1 + 0.132306447410612*G0_2_2;
    A[1223] = 0.0371867233846399*G0_0_0 - 0.0371541034518453*G0_0_1 - 0.0436454700777611*G0_0_2 - 0.0371541034518456*G0_1_0 - 0.0067849460210558*G0_1_1 - 0.00975335990526872*G0_1_2 + 0.0371867233846399*G0_2_0 - 0.0371541034518453*G0_2_1 - 0.0436454700777612*G0_2_2;
    A[1224] = -0.0254435475789639*G0_0_0 + 0.0136351319077015*G0_0_1 + 0.0148746893538559*G0_0_2 + 0.0136351319077006*G0_1_0 - 0.0001304797311741*G0_1_1 - 6.52398655877831e-05*G0_1_2 - 0.0254435475789637*G0_2_0 + 0.0136351319077016*G0_2_1 + 0.0148746893538557*G0_2_2;
    A[1225] = -0.334680510461755*G0_0_0 - 0.250814663249556*G0_0_1 - 0.166948816037356*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700814*G0_1_1 - 0.160783648739377*G0_1_2 - 0.334680510461756*G0_2_0 - 0.250814663249556*G0_2_1 - 0.166948816037356*G0_2_2;
    A[1226] = 0.0567586830607651*G0_0_0 + 0.0378717419733028*G0_0_1 + 0.0189848008858419*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.0127870136550674*G0_1_1 + 0.00949240044292071*G0_1_2 + 0.056758683060765*G0_2_0 + 0.0378717419733027*G0_2_1 + 0.0189848008858418*G0_2_2;
    A[1227] = 0.209419968534549*G0_0_0 + 0.173472802596065*G0_0_1 + 0.127087258163645*G0_0_2 + 0.173472802596064*G0_1_0 + 0.0846813455320401*G0_1_1 + 0.0893133759887214*G0_1_2 + 0.20941996853455*G0_2_0 + 0.173472802596065*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1228] = -0.156575677409008*G0_0_0 - 0.0662184635708928*G0_0_1 - 0.156575677409008*G0_0_2 - 0.0662184635708935*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0662184635708934*G0_1_2 - 0.156575677409008*G0_2_0 - 0.0662184635708928*G0_2_1 - 0.156575677409007*G0_2_2;
    A[1229] = 0.0391439193522524*G0_0_0 + 0.00717638521458099*G0_0_1 + 0.0391439193522528*G0_0_2 + 0.00717638521458054*G0_1_0 + 0.00130479731174216*G0_1_1 + 0.0071763852145798*G0_1_2 + 0.0391439193522522*G0_2_0 + 0.00717638521458099*G0_2_1 + 0.039143919352253*G0_2_2;
    A[1230] = 0.185933616923198*G0_0_0 + 0.170276049182297*G0_0_1 + 0.185933616923198*G0_0_2 + 0.170276049182297*G0_1_0 + 0.180714427676231*G0_1_1 + 0.170276049182298*G0_1_2 + 0.185933616923198*G0_2_0 + 0.170276049182297*G0_2_1 + 0.185933616923199*G0_2_2;
    A[1231] = 0.0391439193522518*G0_0_0 + 0.00717638521457915*G0_0_1 + 0.039143919352251*G0_0_2 + 0.00717638521458003*G0_1_0 + 0.0013047973117422*G0_1_1 + 0.00717638521457957*G0_1_2 + 0.0391439193522518*G0_2_0 + 0.00717638521457916*G0_2_1 + 0.0391439193522509*G0_2_2;
    A[1232] = -0.0155544887532734*G0_0_0 - 0.0155544887532734*G0_0_1 - 0.0155544887532734*G0_0_2 - 0.0388177200243168*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0388177200243168*G0_1_2 - 0.0155544887532734*G0_2_0 - 0.0155544887532734*G0_2_1 - 0.0155544887532734*G0_2_2;
    A[1233] = -0.00336507226698193*G0_0_0 - 0.00336507226698194*G0_2_0;
    A[1234] = 0.03365855145369*G0_0_1 + 0.0551276864210889*G0_1_1 + 0.03365855145369*G0_2_1;
    A[1235] = -0.00336507226698192*G0_0_2 - 0.00336507226698193*G0_2_2;
    A[1236] = 0.0699110399631229*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0124934342599271*G0_1_1 + 0.0699110399631228*G0_1_2 + 0.0699110399631229*G0_2_1 + 0.108637424175618*G0_2_2;
    A[1237] = -0.00480165410720958*G0_0_1 - 0.00960330821441925*G0_0_2 + 0.00026095946234856*G0_1_1 - 0.00480165410720974*G0_1_2 - 0.0048016541072095*G0_2_1 - 0.00960330821441911*G0_2_2;
    A[1238] = 0.000548014870931835*G0_0_1 - 0.00523223722008389*G0_0_2 + 6.52398655866754e-05*G0_1_1 + 0.000548014870931409*G0_1_2 + 0.000548014870931798*G0_2_1 - 0.00523223722008403*G0_2_2;
    A[1239] = 0.00228991928210669*G0_0_1 + 0.0111690649885092*G0_0_2 + 0.00228991928210666*G0_1_2 + 0.0022899192821067*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[1240] = 0.00228991928210664*G0_0_0 + 0.00228991928210663*G0_0_2 + 0.00228991928210665*G0_2_0 + 0.00228991928210663*G0_2_2;
    A[1241] = 0.0017875723170862*G0_0_0 + 0.00178757231708636*G0_0_2 + 0.00178757231708617*G0_2_0 + 0.00178757231708633*G0_2_2;
    A[1242] = 0.00178757231708601*G0_0_0 + 0.00178757231708596*G0_0_2 + 0.00178757231708604*G0_2_0 + 0.001787572317086*G0_2_2;
    A[1243] = 0.00228991928210668*G0_0_0 + 0.00228991928210666*G0_0_2 + 0.00228991928210669*G0_2_0 + 0.00228991928210665*G0_2_2;
    A[1244] = 0.0111690649885091*G0_0_0 + 0.00228991928210654*G0_0_1 + 0.00228991928210639*G0_1_0 + 0.0111690649885092*G0_2_0 + 0.00228991928210654*G0_2_1;
    A[1245] = -0.00523223722008386*G0_0_0 + 0.000548014870932236*G0_0_1 + 0.000548014870931985*G0_1_0 + 6.5239865587205e-05*G0_1_1 - 0.00523223722008385*G0_2_0 + 0.000548014870932243*G0_2_1;
    A[1246] = -0.00960330821441978*G0_0_0 - 0.00480165410721029*G0_0_1 - 0.00480165410721023*G0_1_0 + 0.000260959462347989*G0_1_1 - 0.00960330821441981*G0_2_0 - 0.0048016541072103*G0_2_1;
    A[1247] = 0.108637424175618*G0_0_0 + 0.0699110399631231*G0_0_1 + 0.0699110399631229*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.108637424175618*G0_2_0 + 0.0699110399631231*G0_2_1;
    A[1248] = 0.0283728175438243*G0_0_0 + 0.0283728175438243*G0_0_1 + 0.00939454064454077*G0_0_2 - 0.0315434750113565*G0_1_0 - 0.0315434750113566*G0_1_1 + 0.0283728175438243*G0_1_2 + 0.0283728175438243*G0_2_0 + 0.0283728175438243*G0_2_1 + 0.00939454064454075*G0_2_2;
    A[1249] = -0.000482775005344586*G0_0_0 - 0.000482775005344504*G0_0_1 + 6.52398655869334e-05*G0_0_2 - 0.00626302709636029*G0_1_0 - 0.00626302709636023*G0_1_1 - 0.000482775005344599*G0_1_2 - 0.00048277500534457*G0_2_0 - 0.000482775005344487*G0_2_1 + 6.52398655869371e-05*G0_2_2;
    A[1250] = 0.0132175967679439*G0_0_0 + 0.0132175967679439*G0_0_1 + 6.52398655871012e-05*G0_0_2 + 6.52398655871229e-05*G0_1_0 + 6.52398655871026e-05*G0_1_1 + 0.013217596767944*G0_1_2 + 0.0132175967679439*G0_2_0 + 0.0132175967679439*G0_2_1 + 6.52398655871059e-05*G0_2_2;
    A[1251] = -0.00228991928210673*G0_0_0 - 0.00228991928210674*G0_0_1 + 0.00939454064454059*G0_0_2 - 0.00228991928210678*G0_1_2 - 0.00228991928210673*G0_2_0 - 0.00228991928210674*G0_2_1 + 0.00939454064454059*G0_2_2;
    A[1252] = 0.100717304493345*G0_0_0 + 0.0817390275940618*G0_0_1 + 0.100717304493345*G0_0_2 + 0.125351877739029*G0_1_0 + 0.18526817029421*G0_1_1 + 0.125351877739029*G0_1_2 + 0.100717304493345*G0_2_0 + 0.0817390275940618*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1253] = -0.204187731314465*G0_0_0 - 0.167692550505049*G0_0_1 - 0.204187731314465*G0_0_2 - 0.219493003781196*G0_1_0 - 0.387185554286245*G0_1_1 - 0.219493003781196*G0_1_2 - 0.204187731314465*G0_2_0 - 0.167692550505049*G0_2_1 - 0.204187731314466*G0_2_2;
    A[1254] = 0.281001149056702*G0_0_0 + 0.202869886029606*G0_0_1 + 0.281001149056702*G0_0_2 + 0.202869886029606*G0_1_0 + 0.428965164208215*G0_1_1 + 0.202869886029606*G0_1_2 + 0.281001149056702*G0_2_0 + 0.202869886029606*G0_2_1 + 0.281001149056702*G0_2_2;
    A[1255] = -0.108637424175618*G0_0_0 - 0.135020425819036*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0699110399631228*G0_1_0 - 0.243357746612953*G0_1_1 - 0.0699110399631227*G0_1_2 - 0.108637424175618*G0_2_0 - 0.135020425819036*G0_2_1 - 0.108637424175618*G0_2_2;
    A[1256] = 0.00939454064454069*G0_0_0 + 0.0283728175438242*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438242*G0_1_0 - 0.0315434750113567*G0_1_1 - 0.0315434750113567*G0_1_2 + 0.00939454064454064*G0_2_0 + 0.0283728175438242*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1257] = 6.52398655871219e-05*G0_0_0 - 0.000482775005344264*G0_0_1 - 0.000482775005344249*G0_0_2 - 0.000482775005344263*G0_1_0 - 0.00626302709636009*G0_1_1 - 0.00626302709636015*G0_1_2 + 6.52398655871889e-05*G0_2_0 - 0.000482775005344256*G0_2_1 - 0.000482775005344246*G0_2_2;
    A[1258] = 6.52398655870937e-05*G0_0_0 + 0.0132175967679441*G0_0_1 + 0.013217596767944*G0_0_2 + 0.0132175967679436*G0_1_0 + 6.52398655870671e-05*G0_1_1 + 6.52398655870633e-05*G0_1_2 + 6.52398655870265e-05*G0_2_0 + 0.0132175967679441*G0_2_1 + 0.0132175967679441*G0_2_2;
    A[1259] = 0.00939454064454044*G0_0_0 - 0.00228991928210671*G0_0_1 - 0.00228991928210665*G0_0_2 - 0.00228991928210669*G0_1_0 + 0.00939454064454048*G0_2_0 - 0.00228991928210671*G0_2_1 - 0.00228991928210665*G0_2_2;
    A[1260] = -0.00117431758056653*G0_0_0 + 6.52398655874424e-05*G0_0_1 - 0.00117431758056702*G0_0_2 + 6.5239865587038e-05*G0_1_0 + 6.52398655863479e-05*G0_1_2 - 0.00117431758056658*G0_2_0 + 6.52398655874026e-05*G0_2_1 - 0.00117431758056703*G0_2_2;
    A[1261] = 0.00352295274170181*G0_0_0 - 0.00296841388421189*G0_0_1 + 0.00352295274170319*G0_0_2 - 0.00296841388421306*G0_1_0 - 0.000652398655870354*G0_1_1 - 0.00296841388421228*G0_1_2 + 0.00352295274170175*G0_2_0 - 0.00296841388421193*G0_2_1 + 0.00352295274170319*G0_2_2;
    A[1262] = 0.124738623002512*G0_0_0 + 0.0623693115012547*G0_0_1 + 0.124738623002511*G0_0_2 + 0.0623693115012554*G0_1_0 + 0.0476251018785727*G0_1_1 + 0.0623693115012546*G0_1_2 + 0.124738623002512*G0_2_0 + 0.0623693115012547*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1263] = -0.0105036183595214*G0_0_0 + 0.00257697469068966*G0_0_1 - 0.0105036183595213*G0_0_2 + 0.00257697469069032*G0_1_0 + 0.00257697469069048*G0_1_2 - 0.0105036183595213*G0_2_0 + 0.00257697469068971*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[1264] = 0.00352295274170312*G0_0_0 - 0.00296841388421201*G0_0_1 + 0.00352295274170253*G0_0_2 - 0.00296841388421274*G0_1_0 - 0.000652398655870962*G0_1_1 - 0.00296841388421254*G0_1_2 + 0.00352295274170323*G0_2_0 - 0.00296841388421196*G0_2_1 + 0.00352295274170269*G0_2_2;
    A[1265] = -0.00117431758056744*G0_0_0 + 6.52398655867422e-05*G0_0_1 - 0.00117431758056732*G0_0_2 + 6.5239865586231e-05*G0_1_0 + 6.52398655863273e-05*G0_1_2 - 0.00117431758056746*G0_2_0 + 6.52398655867902e-05*G0_2_1 - 0.0011743175805673*G0_2_2;
    A[1266] = -0.0510175748891024*G0_0_0 - 0.0887914570640259*G0_0_1 - 0.0528442911255408*G0_0_2 - 0.00463203045668275*G0_1_0 + 0.0846813455320392*G0_1_1 - 0.0887914570640254*G0_1_2 - 0.0510175748891024*G0_2_0 - 0.0887914570640259*G0_2_1 - 0.0528442911255408*G0_2_2;
    A[1267] = 0.182410664181496*G0_0_0 + 0.188575831479476*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0985448169692953*G0_1_0 - 0.0622388317700809*G0_1_1 + 0.188575831479475*G0_1_2 + 0.182410664181496*G0_2_0 + 0.188575831479476*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1268] = -0.124738623002512*G0_0_0 - 0.198068231922397*G0_0_1 - 0.271397840842283*G0_0_2 - 0.0623693115012553*G0_1_0 - 0.00339247301052866*G0_1_1 - 0.198068231922397*G0_1_2 - 0.124738623002512*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842284*G0_2_2;
    A[1269] = -0.0155923278753139*G0_0_0 - 0.025084728318235*G0_0_1 - 0.00619778723077306*G0_0_2 + 0.00329461321214813*G0_1_0 + 0.0127870136550689*G0_1_1 - 0.0250847283182351*G0_1_2 - 0.0155923278753139*G0_2_0 - 0.025084728318235*G0_2_1 - 0.00619778723077313*G0_2_2;
    A[1270] = -0.00352295274170318*G0_0_0 + 0.0303691574307889*G0_0_1 + 0.104709984267274*G0_0_2 + 0.00296841388421273*G0_1_0 - 0.00678494602105733*G0_1_1 + 0.0303691574307893*G0_1_2 - 0.00352295274170328*G0_2_0 + 0.0303691574307888*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1271] = 0.0011743175805673*G0_0_0 - 0.0137656116388757*G0_0_1 - 0.0528442911255407*G0_0_2 - 6.52398655863683e-05*G0_1_0 - 0.000130479731173825*G0_1_1 - 0.0137656116388756*G0_1_2 + 0.00117431758056732*G0_2_0 - 0.0137656116388757*G0_2_1 - 0.0528442911255407*G0_2_2;
    A[1272] = -0.0150051690850298*G0_0_0 + 0.0227687130898936*G0_0_1 - 0.01500516908503*G0_0_2 + 0.0227687130898934*G0_1_0 - 0.0665446628988286*G0_1_1 + 0.0227687130898936*G0_1_2 - 0.0150051690850298*G0_2_0 + 0.0227687130898936*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1273] = -0.000652398655870912*G0_0_0 + 0.00884000178705046*G0_0_1 - 0.00065239865587136*G0_0_2 + 0.00884000178705024*G0_1_0 - 0.000652398655870833*G0_1_1 + 0.0088400017870502*G0_1_2 - 0.000652398655870866*G0_2_0 + 0.00884000178705038*G0_2_1 - 0.000652398655871415*G0_2_2;
    A[1274] = -0.0150051690850302*G0_0_0 - 6.52398655872888e-05*G0_0_1 - 0.0150051690850304*G0_0_2 - 6.52398655869019e-05*G0_1_0 - 6.52398655871304e-05*G0_1_2 - 0.0150051690850302*G0_2_0 - 6.52398655872486e-05*G0_2_1 - 0.0150051690850304*G0_2_2;
    A[1275] = -0.000652398655871094*G0_0_0 + 0.0088400017870501*G0_0_1 - 0.000652398655870762*G0_0_2 + 0.00884000178705037*G0_1_0 - 0.000652398655870407*G0_1_1 + 0.00884000178705012*G0_1_2 - 0.000652398655871162*G0_2_0 + 0.00884000178705006*G0_2_1 - 0.00065239865587082*G0_2_2;
    A[1276] = -0.000652398655870707*G0_0_0 - 0.0025769746906896*G0_0_1 - 0.000652398655870287*G0_0_2 - 0.00257697469069009*G0_1_0 - 0.00257697469068978*G0_1_2 - 0.000652398655870634*G0_2_0 - 0.00257697469068965*G0_2_1 - 0.000652398655870286*G0_2_2;
    A[1277] = -0.0150051690850299*G0_0_0 - 6.52398655867693e-05*G0_0_1 - 0.0150051690850301*G0_0_2 - 6.52398655869649e-05*G0_1_0 - 6.52398655870825e-05*G0_1_2 - 0.0150051690850299*G0_2_0 - 6.52398655868179e-05*G0_2_1 - 0.01500516908503*G0_2_2;
    A[1278] = -0.0528442911255404*G0_0_0 - 0.0887914570640257*G0_0_1 - 0.0510175748891023*G0_0_2 - 0.0887914570640255*G0_1_0 + 0.0846813455320394*G0_1_1 - 0.00463203045668288*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0887914570640258*G0_2_1 - 0.0510175748891024*G0_2_2;
    A[1279] = -0.0061977872307736*G0_0_0 - 0.0250847283182359*G0_0_1 - 0.0155923278753141*G0_0_2 - 0.0250847283182353*G0_1_0 + 0.0127870136550684*G0_1_1 + 0.00329461321214744*G0_1_2 - 0.0061977872307735*G0_2_0 - 0.0250847283182359*G0_2_1 - 0.0155923278753142*G0_2_2;
    A[1280] = -0.0528442911255402*G0_0_0 - 0.0137656116388758*G0_0_1 + 0.00117431758056707*G0_0_2 - 0.0137656116388748*G0_1_0 - 0.000130479731174212*G0_1_1 - 6.5239865586424e-05*G0_1_2 - 0.0528442911255403*G0_2_0 - 0.0137656116388758*G0_2_1 + 0.00117431758056708*G0_2_2;
    A[1281] = 0.104709984267275*G0_0_0 + 0.188575831479476*G0_0_1 + 0.182410664181496*G0_0_2 + 0.188575831479475*G0_1_0 - 0.0622388317700805*G0_1_1 + 0.0985448169692955*G0_1_2 + 0.104709984267275*G0_2_0 + 0.188575831479476*G0_2_1 + 0.182410664181496*G0_2_2;
    A[1282] = 0.104709984267275*G0_0_0 + 0.0303691574307901*G0_0_1 - 0.00352295274170271*G0_0_2 + 0.0303691574307887*G0_1_0 - 0.0067849460210561*G0_1_1 + 0.00296841388421254*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0303691574307901*G0_2_1 - 0.00352295274170271*G0_2_2;
    A[1283] = -0.271397840842282*G0_0_0 - 0.198068231922397*G0_0_1 - 0.12473862300251*G0_0_2 - 0.198068231922396*G0_1_0 - 0.00339247301052918*G0_1_1 - 0.0623693115012544*G0_1_2 - 0.271397840842283*G0_2_0 - 0.198068231922398*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1284] = 0.0260959462348349*G0_0_0 + 0.0104383784939338*G0_0_1 + 0.0260959462348348*G0_0_2 + 0.0104383784939341*G0_1_0 + 0.180714427676232*G0_1_1 + 0.0104383784939345*G0_1_2 + 0.0260959462348348*G0_2_0 + 0.0104383784939338*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[1285] = 0.0260959462348339*G0_0_0 - 0.00587158790283884*G0_0_1 + 0.0260959462348349*G0_0_2 - 0.00587158790283852*G0_1_0 + 0.00130479731174155*G0_1_1 - 0.00587158790283811*G0_1_2 + 0.0260959462348341*G0_2_0 - 0.00587158790283871*G0_2_1 + 0.026095946234835*G0_2_2;
    A[1286] = -0.185933616923198*G0_0_0 - 0.0955764030850821*G0_0_1 - 0.185933616923199*G0_0_2 - 0.0955764030850824*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0955764030850833*G0_1_2 - 0.185933616923198*G0_2_0 - 0.0955764030850821*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1287] = 0.0260959462348353*G0_0_0 - 0.005871587902838*G0_0_1 + 0.0260959462348346*G0_0_2 - 0.00587158790283839*G0_1_0 + 0.00130479731174127*G0_1_1 - 0.00587158790283773*G0_1_2 + 0.0260959462348352*G0_2_0 - 0.00587158790283801*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[1288] = 0.00480034930989792*G0_0_0 + 0.00480034930989795*G0_0_1 + 0.00480034930989796*G0_0_2 + 0.0153313684129655*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.00480034930989791*G0_2_0 + 0.00480034930989795*G0_2_1 + 0.00480034930989795*G0_2_2;
    A[1289] = -0.00480034930989787*G0_0_0 - 0.00480034930989787*G0_2_0;
    A[1290] = -0.0372571824394737*G0_0_1 - 0.0551276864210888*G0_1_1 - 0.0372571824394737*G0_2_1;
    A[1291] = -0.00480034930989792*G0_0_2 - 0.00480034930989792*G0_2_2;
    A[1292] = -0.069911039963123*G0_0_1 - 0.139822079926246*G0_0_2 + 0.0196371995417135*G0_1_1 - 0.0699110399631226*G0_1_2 - 0.069911039963123*G0_2_1 - 0.139822079926246*G0_2_2;
    A[1293] = 0.0387263842124951*G0_0_1 + 0.108637424175618*G0_0_2 - 0.0186912214907008*G0_1_1 + 0.038726384212495*G0_1_2 + 0.038726384212495*G0_2_1 + 0.108637424175618*G0_2_2;
    A[1294] = -0.0189782768992839*G0_0_1 - 0.0788945694544648*G0_0_2 + 0.0093945406445409*G0_1_1 - 0.0189782768992835*G0_1_2 - 0.018978276899284*G0_2_1 - 0.0788945694544648*G0_2_2;
    A[1295] = 0.00537576492437601*G0_0_1 + 0.0328352243499812*G0_0_2 + 0.0053757649243761*G0_1_2 + 0.00537576492437603*G0_2_1 + 0.0328352243499811*G0_2_2;
    A[1296] = 0.00537576492437595*G0_0_0 + 0.00537576492437603*G0_0_2 + 0.00537576492437596*G0_2_0 + 0.00537576492437603*G0_2_2;
    A[1297] = 0.00228991928210675*G0_0_0 + 0.00228991928210663*G0_0_2 + 0.00228991928210671*G0_2_0 + 0.00228991928210662*G0_2_2;
    A[1298] = 0.00228991928210685*G0_0_0 + 0.00228991928210687*G0_0_2 + 0.00228991928210686*G0_2_0 + 0.00228991928210685*G0_2_2;
    A[1299] = 0.00537576492437604*G0_0_0 + 0.00537576492437609*G0_0_2 + 0.00537576492437606*G0_2_0 + 0.00537576492437612*G0_2_2;
    A[1300] = 0.0328352243499809*G0_0_0 + 0.00537576492437608*G0_0_1 + 0.00537576492437616*G0_1_0 + 0.0328352243499809*G0_2_0 + 0.00537576492437608*G0_2_1;
    A[1301] = -0.0788945694544646*G0_0_0 - 0.018978276899284*G0_0_1 - 0.0189782768992838*G0_1_0 + 0.00939454064454048*G0_1_1 - 0.0788945694544646*G0_2_0 - 0.018978276899284*G0_2_1;
    A[1302] = 0.108637424175618*G0_0_0 + 0.0387263842124952*G0_0_1 + 0.0387263842124953*G0_1_0 - 0.0186912214907004*G0_1_1 + 0.108637424175618*G0_2_0 + 0.0387263842124953*G0_2_1;
    A[1303] = -0.139822079926246*G0_0_0 - 0.069911039963123*G0_0_1 - 0.0699110399631228*G0_1_0 + 0.0196371995417134*G0_1_1 - 0.139822079926246*G0_2_0 - 0.069911039963123*G0_2_1;
    A[1304] = -0.00537576492437607*G0_0_0 - 0.00537576492437603*G0_0_1 + 0.0220836945012289*G0_1_0 + 0.022083694501229*G0_1_1 - 0.00537576492437606*G0_1_2 - 0.00537576492437608*G0_2_0 - 0.00537576492437603*G0_2_1;
    A[1305] = -0.00228991928210658*G0_0_0 - 0.00228991928210668*G0_0_1 + 0.00658922642429562*G0_1_0 + 0.00658922642429557*G0_1_1 - 0.00228991928210667*G0_1_2 - 0.00228991928210659*G0_2_0 - 0.00228991928210669*G0_2_1;
    A[1306] = -0.00228991928210696*G0_0_0 - 0.00228991928210693*G0_0_1 + 0.00939454064454067*G0_1_0 + 0.00939454064454069*G0_1_1 - 0.00228991928210677*G0_1_2 - 0.00228991928210696*G0_2_0 - 0.00228991928210692*G0_2_1;
    A[1307] = -0.005375764924376*G0_0_0 - 0.005375764924376*G0_0_1 - 0.00537576492437601*G0_1_2 - 0.00537576492437602*G0_2_0 - 0.00537576492437602*G0_2_1;
    A[1308] = -0.032835224349981*G0_0_0 - 0.0274594594256053*G0_0_1 - 0.0328352243499812*G0_0_2 - 0.0537511252572011*G0_1_0 - 0.0812105846828063*G0_1_1 - 0.0537511252572012*G0_1_2 - 0.032835224349981*G0_2_0 - 0.0274594594256052*G0_2_1 - 0.0328352243499811*G0_2_2;
    A[1309] = 0.0788945694544646*G0_0_0 + 0.0599162925551812*G0_0_1 + 0.0788945694544648*G0_0_2 + 0.103529142700149*G0_1_0 + 0.18526817029421*G0_1_1 + 0.103529142700149*G0_1_2 + 0.0788945694544646*G0_2_0 + 0.0599162925551811*G0_2_1 + 0.0788945694544647*G0_2_2;
    A[1310] = -0.108637424175618*G0_0_0 - 0.0699110399631228*G0_0_1 - 0.108637424175618*G0_0_2 - 0.135020425819036*G0_1_0 - 0.243357746612953*G0_1_1 - 0.135020425819036*G0_1_2 - 0.108637424175618*G0_2_0 - 0.0699110399631227*G0_2_1 - 0.108637424175618*G0_2_2;
    A[1311] = 0.139822079926246*G0_0_0 + 0.0699110399631227*G0_0_1 + 0.139822079926246*G0_0_2 + 0.0699110399631227*G0_1_0 + 0.179096479009672*G0_1_1 + 0.0699110399631226*G0_1_2 + 0.139822079926246*G0_2_0 + 0.0699110399631226*G0_2_1 + 0.139822079926246*G0_2_2;
    A[1312] = -0.00537576492437598*G0_0_1 - 0.00537576492437604*G0_0_2 - 0.00537576492437599*G0_1_0 + 0.0220836945012291*G0_1_1 + 0.022083694501229*G0_1_2 - 0.00537576492437597*G0_2_1 - 0.00537576492437603*G0_2_2;
    A[1313] = -0.00228991928210687*G0_0_1 - 0.00228991928210684*G0_0_2 - 0.00228991928210688*G0_1_0 + 0.00658922642429564*G0_1_1 + 0.00658922642429568*G0_1_2 - 0.00228991928210686*G0_2_1 - 0.00228991928210683*G0_2_2;
    A[1314] = -0.00228991928210691*G0_0_1 - 0.00228991928210692*G0_0_2 - 0.00228991928210658*G0_1_0 + 0.00939454064454063*G0_1_1 + 0.00939454064454062*G0_1_2 - 0.0022899192821069*G0_2_1 - 0.00228991928210692*G0_2_2;
    A[1315] = -0.00537576492437592*G0_0_1 - 0.00537576492437601*G0_0_2 - 0.00537576492437605*G0_1_0 - 0.00537576492437592*G0_2_1 - 0.005375764924376*G0_2_2;
    A[1316] = -0.0118736555368498*G0_0_0 + 0.00939454064454042*G0_0_1 - 0.0118736555368499*G0_0_2 + 0.0093945406445408*G0_1_0 + 0.009394540644541*G0_1_2 - 0.0118736555368499*G0_2_0 + 0.00939454064454035*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1317] = 0.0429604514890975*G0_0_0 + 0.000685018588664926*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.000685018588664798*G0_1_0 + 0.0226708532915129*G0_1_1 + 0.000685018588664297*G0_1_2 + 0.0429604514890976*G0_2_0 + 0.000685018588665031*G0_2_1 + 0.0429604514890969*G0_2_2;
    A[1318] = -0.124738623002512*G0_0_0 - 0.0284445813959704*G0_0_1 - 0.124738623002511*G0_0_2 - 0.0284445813959702*G0_1_0 - 0.060673074995991*G0_1_1 - 0.0284445813959697*G0_1_2 - 0.124738623002512*G0_2_0 - 0.0284445813959705*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1319] = -0.00254435475789606*G0_0_0 + 0.00329461321214803*G0_0_1 - 0.0025443547578959*G0_0_2 + 0.00329461321214805*G0_1_0 + 0.00329461321214804*G0_1_2 - 0.002544354757896*G0_2_0 + 0.00329461321214802*G0_2_1 - 0.00254435475789583*G0_2_2;
    A[1320] = 0.0429604514890969*G0_0_0 + 0.000685018588664383*G0_0_1 + 0.0429604514890976*G0_0_2 + 0.000685018588664695*G0_1_0 + 0.0226708532915133*G0_1_1 + 0.000685018588664581*G0_1_2 + 0.0429604514890967*G0_2_0 + 0.000685018588664231*G0_2_1 + 0.0429604514890974*G0_2_2;
    A[1321] = -0.0118736555368498*G0_0_0 + 0.00939454064454089*G0_0_1 - 0.01187365553685*G0_0_2 + 0.00939454064454114*G0_1_0 + 0.00939454064454116*G0_1_2 - 0.0118736555368499*G0_2_0 + 0.00939454064454091*G0_2_1 - 0.0118736555368501*G0_2_2;
    A[1322] = 0.0118736555368499*G0_0_0 + 0.0212681961813905*G0_0_1 - 0.0146137298915078*G0_1_0 - 0.0693499771190735*G0_1_1 + 0.0212681961813902*G0_1_2 + 0.0118736555368499*G0_2_0 + 0.0212681961813905*G0_2_1;
    A[1323] = -0.0429604514890975*G0_0_0 - 0.0422754329004329*G0_0_1 - 0.0359145460056914*G0_1_0 + 0.0920860702761733*G0_1_1 - 0.0422754329004324*G0_1_2 - 0.0429604514890976*G0_2_0 - 0.0422754329004328*G0_2_1;
    A[1324] = 0.124738623002512*G0_0_0 + 0.0962940416065414*G0_0_1 + 0.0284445813959701*G0_1_0 - 0.0644569872000422*G0_1_1 + 0.0962940416065409*G0_1_2 + 0.124738623002512*G0_2_0 + 0.0962940416065414*G0_2_1;
    A[1325] = 0.0025443547578965*G0_0_0 + 0.00583896797004434*G0_0_1 - 0.00590420783563159*G0_1_0 - 0.031869674339292*G0_1_1 + 0.00583896797004443*G0_1_2 + 0.00254435475789644*G0_2_0 + 0.00583896797004433*G0_2_1;
    A[1326] = -0.0429604514890969*G0_0_0 - 0.0422754329004327*G0_0_1 - 0.000685018588664666*G0_1_0 + 0.0439716694056971*G0_1_1 - 0.0422754329004327*G0_1_2 - 0.0429604514890967*G0_2_0 - 0.0422754329004326*G0_2_1;
    A[1327] = 0.0118736555368499*G0_0_0 + 0.0212681961813908*G0_0_1 - 0.0093945406445411*G0_1_0 - 0.0187890812890816*G0_1_1 + 0.0212681961813904*G0_1_2 + 0.01187365553685*G0_2_0 + 0.0212681961813909*G0_2_1;
    A[1328] = -0.00939454064454056*G0_0_1 - 0.00939454064454039*G0_1_0 + 0.0453417065830254*G0_1_1 - 0.00939454064454075*G0_1_2 - 0.00939454064454052*G0_2_1;
    A[1329] = -0.00329461321214779*G0_0_1 - 0.00329461321214795*G0_1_0 + 0.022670853291513*G0_1_1 - 0.00329461321214772*G0_1_2 - 0.00329461321214773*G0_2_1;
    A[1330] = -0.00939454064454051*G0_0_1 - 0.00939454064454069*G0_1_0 - 0.00939454064454063*G0_1_2 - 0.00939454064454043*G0_2_1;
    A[1331] = -0.0032946132121475*G0_0_1 - 0.00329461321214782*G0_1_0 + 0.0226708532915126*G0_1_1 - 0.00329461321214781*G0_1_2 - 0.00329461321214752*G0_2_1;
    A[1332] = -0.00329461321214807*G0_0_1 - 0.00329461321214779*G0_1_0 - 0.00329461321214808*G0_1_2 - 0.00329461321214806*G0_2_1;
    A[1333] = -0.00939454064454085*G0_0_1 - 0.00939454064454078*G0_1_0 - 0.00939454064454055*G0_1_2 - 0.00939454064454087*G0_2_1;
    A[1334] = 0.0212681961813905*G0_0_1 + 0.0118736555368498*G0_0_2 + 0.0212681961813905*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.0146137298915075*G0_1_2 + 0.0212681961813906*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[1335] = 0.00583896797004478*G0_0_1 + 0.00254435475789653*G0_0_2 + 0.00583896797004429*G0_1_0 - 0.0318696743392919*G0_1_1 - 0.00590420783563113*G0_1_2 + 0.0058389679700448*G0_2_1 + 0.00254435475789658*G0_2_2;
    A[1336] = 0.0212681961813905*G0_0_1 + 0.0118736555368501*G0_0_2 + 0.0212681961813901*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.00939454064454095*G0_1_2 + 0.0212681961813905*G0_2_1 + 0.0118736555368502*G0_2_2;
    A[1337] = -0.042275432900433*G0_0_1 - 0.0429604514890972*G0_0_2 - 0.0422754329004325*G0_1_0 + 0.0920860702761732*G0_1_1 - 0.0359145460056916*G0_1_2 - 0.0422754329004331*G0_2_1 - 0.0429604514890973*G0_2_2;
    A[1338] = -0.0422754329004328*G0_0_1 - 0.0429604514890972*G0_0_2 - 0.0422754329004322*G0_1_0 + 0.0439716694056966*G0_1_1 - 0.000685018588664468*G0_1_2 - 0.0422754329004329*G0_2_1 - 0.0429604514890973*G0_2_2;
    A[1339] = 0.0962940416065412*G0_0_1 + 0.124738623002511*G0_0_2 + 0.0962940416065405*G0_1_0 - 0.064456987200042*G0_1_1 + 0.0284445813959695*G0_1_2 + 0.0962940416065413*G0_2_1 + 0.124738623002511*G0_2_2;
    A[1340] = 0.00260959462348361*G0_0_1 + 0.00260959462348357*G0_1_0 - 0.151356488162042*G0_1_1 + 0.00260959462348299*G0_1_2 + 0.00260959462348355*G0_2_1;
    A[1341] = 0.00260959462348302*G0_0_1 + 0.0026095946234835*G0_1_0 - 0.0453417065830263*G0_1_1 + 0.00260959462348321*G0_1_2 + 0.00260959462348287*G0_2_1;
    A[1342] = 0.0352295274170273*G0_0_1 + 0.0352295274170267*G0_1_0 + 0.166687856575008*G0_1_1 + 0.0352295274170272*G0_1_2 + 0.0352295274170274*G0_2_1;
    A[1343] = 0.00260959462348311*G0_0_1 + 0.00260959462348347*G0_1_0 - 0.0453417065830257*G0_1_1 + 0.0026095946234832*G0_1_2 + 0.00260959462348329*G0_2_1;
    A[1344] = -0.0551276864210885*G0_0_0 - 0.0551276864210885*G0_0_1 - 0.0551276864210886*G0_0_2 - 0.0178705039816148*G0_1_0 - 0.0178705039816148*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[1345] = 0.0153313684129654*G0_0_0 + 0.0105310191030674*G0_1_0 + 0.0105310191030674*G0_2_0;
    A[1346] = -0.00480034930989793*G0_1_1 - 0.00480034930989792*G0_2_1;
    A[1347] = -0.00480034930989785*G0_1_2 - 0.00480034930989785*G0_2_2;
    A[1348] = 0.00537576492437603*G0_1_1 + 0.00537576492437606*G0_1_2 + 0.00537576492437608*G0_2_1 + 0.00537576492437612*G0_2_2;
    A[1349] = 0.00228991928210676*G0_1_1 + 0.00228991928210662*G0_1_2 + 0.0022899192821067*G0_2_1 + 0.00228991928210655*G0_2_2;
    A[1350] = 0.00228991928210659*G0_1_1 + 0.00228991928210687*G0_1_2 + 0.00228991928210664*G0_2_1 + 0.00228991928210696*G0_2_2;
    A[1351] = 0.00537576492437606*G0_1_1 + 0.00537576492437601*G0_1_2 + 0.00537576492437601*G0_2_1 + 0.00537576492437594*G0_2_2;
    A[1352] = 0.0220836945012288*G0_0_0 + 0.0274594594256047*G0_0_2 + 0.0274594594256049*G0_1_0 + 0.0328352243499805*G0_1_2 + 0.0274594594256049*G0_2_0 + 0.0328352243499804*G0_2_2;
    A[1353] = 0.00658922642429587*G0_0_0 + 0.00887914570640286*G0_0_2 + 0.00887914570640254*G0_1_0 + 0.0111690649885097*G0_1_2 + 0.00887914570640248*G0_2_0 + 0.0111690649885097*G0_2_2;
    A[1354] = 0.00939454064454063*G0_0_0 + 0.011684459926647*G0_0_2 + 0.0116844599266473*G0_1_0 + 0.0139743792087536*G0_1_2 + 0.0116844599266474*G0_2_0 + 0.0139743792087536*G0_2_2;
    A[1355] = 0.00537576492437623*G0_0_2 + 0.00537576492437608*G0_1_0 + 0.0107515298487522*G0_1_2 + 0.00537576492437602*G0_2_0 + 0.0107515298487522*G0_2_2;
    A[1356] = 0.0220836945012286*G0_0_0 + 0.0274594594256048*G0_0_1 + 0.0274594594256047*G0_1_0 + 0.0328352243499804*G0_1_1 + 0.0274594594256047*G0_2_0 + 0.0328352243499804*G0_2_1;
    A[1357] = 0.00658922642429595*G0_0_0 + 0.00887914570640245*G0_0_1 + 0.00887914570640259*G0_1_0 + 0.0111690649885097*G0_1_1 + 0.00887914570640266*G0_2_0 + 0.0111690649885098*G0_2_1;
    A[1358] = 0.00939454064454027*G0_0_0 + 0.0116844599266472*G0_0_1 + 0.0116844599266472*G0_1_0 + 0.0139743792087537*G0_1_1 + 0.0116844599266471*G0_2_0 + 0.0139743792087536*G0_2_1;
    A[1359] = 0.00537576492437609*G0_0_1 + 0.00537576492437599*G0_1_0 + 0.0107515298487521*G0_1_1 + 0.00537576492437605*G0_2_0 + 0.0107515298487522*G0_2_1;
    A[1360] = 0.0196371995417134*G0_0_0 + 0.0196371995417134*G0_0_1 + 0.0895482395048358*G0_0_2 + 0.0895482395048358*G0_1_0 + 0.0895482395048359*G0_1_1 + 0.0196371995417133*G0_1_2 + 0.0895482395048358*G0_2_0 + 0.0895482395048359*G0_2_1 + 0.0196371995417133*G0_2_2;
    A[1361] = -0.0186912214907008*G0_0_0 - 0.0186912214907007*G0_0_1 - 0.0574176057031955*G0_0_2 - 0.0574176057031957*G0_1_0 - 0.0574176057031957*G0_1_1 + 0.012493434259927*G0_1_2 - 0.0574176057031955*G0_2_0 - 0.0574176057031956*G0_2_1 + 0.0124934342599271*G0_2_2;
    A[1362] = 0.00939454064454062*G0_0_0 + 0.00939454064454061*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438242*G0_1_0 + 0.0283728175438242*G0_1_1 - 0.0315434750113567*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.0283728175438241*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[1363] = -0.0053757649243759*G0_0_2 - 0.00537576492437602*G0_1_0 - 0.00537576492437603*G0_1_1 + 0.0220836945012291*G0_1_2 - 0.00537576492437596*G0_2_0 - 0.00537576492437597*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1364] = 0.0196371995417133*G0_0_0 + 0.0895482395048358*G0_0_1 + 0.0196371995417133*G0_0_2 + 0.0895482395048357*G0_1_0 + 0.0196371995417131*G0_1_1 + 0.0895482395048358*G0_1_2 + 0.0895482395048357*G0_2_0 + 0.0196371995417132*G0_2_1 + 0.0895482395048358*G0_2_2;
    A[1365] = -0.0186912214907005*G0_0_0 - 0.0574176057031954*G0_0_1 - 0.0186912214907004*G0_0_2 - 0.0574176057031951*G0_1_0 + 0.0124934342599273*G0_1_1 - 0.0574176057031952*G0_1_2 - 0.0574176057031952*G0_2_0 + 0.0124934342599272*G0_2_1 - 0.0574176057031952*G0_2_2;
    A[1366] = 0.00939454064454069*G0_0_0 + 0.0283728175438242*G0_0_1 + 0.00939454064454064*G0_0_2 + 0.0283728175438242*G0_1_0 - 0.0315434750113567*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.0283728175438242*G0_2_0 - 0.0315434750113567*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1367] = -0.00537576492437599*G0_0_1 - 0.00537576492437598*G0_1_0 + 0.0220836945012291*G0_1_1 - 0.00537576492437597*G0_1_2 - 0.00537576492437604*G0_2_0 + 0.022083694501229*G0_2_1 - 0.00537576492437603*G0_2_2;
    A[1368] = 0.179096479009671*G0_0_0 + 0.109185439046549*G0_0_1 + 0.109185439046549*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009671*G0_1_1 + 0.179096479009671*G0_1_2 + 0.109185439046549*G0_2_0 + 0.179096479009671*G0_2_1 + 0.179096479009671*G0_2_2;
    A[1369] = -0.243357746612952*G0_0_0 - 0.108337320793916*G0_0_1 - 0.108337320793916*G0_0_2 - 0.17344670664983*G0_1_0 - 0.147063705006411*G0_1_1 - 0.147063705006411*G0_1_2 - 0.17344670664983*G0_2_0 - 0.147063705006411*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1370] = 0.18526817029421*G0_0_0 + 0.081739027594061*G0_0_1 + 0.081739027594061*G0_0_2 + 0.125351877739029*G0_1_0 + 0.100717304493344*G0_1_1 + 0.100717304493344*G0_1_2 + 0.125351877739029*G0_2_0 + 0.100717304493344*G0_2_1 + 0.100717304493344*G0_2_2;
    A[1371] = -0.081210584682806*G0_0_0 - 0.0274594594256047*G0_0_1 - 0.0274594594256047*G0_0_2 - 0.0537511252572008*G0_1_0 - 0.0328352243499804*G0_1_1 - 0.0328352243499804*G0_1_2 - 0.0537511252572008*G0_2_0 - 0.0328352243499803*G0_2_1 - 0.0328352243499803*G0_2_2;
    A[1372] = 0.0453417065830257*G0_0_0 + 0.0547362472275658*G0_0_1 + 0.0547362472275655*G0_0_2 + 0.054736247227566*G0_1_0 + 0.0641307878721063*G0_1_1 + 0.064130787872106*G0_1_2 + 0.0547362472275661*G0_2_0 + 0.0641307878721062*G0_2_1 + 0.0641307878721062*G0_2_2;
    A[1373] = 0.0226708532915122*G0_0_0 + 0.0259654665036609*G0_0_1 + 0.0259654665036604*G0_0_2 + 0.0259654665036605*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.0292600797158089*G0_1_2 + 0.0259654665036604*G0_2_0 + 0.029260079715809*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[1374] = 0.00939454064453988*G0_0_1 + 0.00939454064454029*G0_0_2 + 0.00939454064454043*G0_1_0 + 0.0187890812890806*G0_1_1 + 0.0187890812890808*G0_1_2 + 0.00939454064454046*G0_2_0 + 0.0187890812890806*G0_2_1 + 0.0187890812890808*G0_2_2;
    A[1375] = 0.0226708532915133*G0_0_0 + 0.0259654665036611*G0_0_1 + 0.0259654665036616*G0_0_2 + 0.025965466503661*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.0292600797158089*G0_1_2 + 0.025965466503661*G0_2_0 + 0.0292600797158089*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[1376] = 0.00329461321214792*G0_0_1 + 0.00329461321214764*G0_0_2 + 0.00329461321214759*G0_1_0 + 0.00658922642429541*G0_1_1 + 0.00658922642429534*G0_1_2 + 0.00329461321214759*G0_2_0 + 0.00658922642429546*G0_2_1 + 0.00658922642429528*G0_2_2;
    A[1377] = 0.00939454064454042*G0_0_1 + 0.00939454064454029*G0_0_2 + 0.00939454064454029*G0_1_0 + 0.0187890812890811*G0_1_1 + 0.018789081289081*G0_1_2 + 0.00939454064454028*G0_2_0 + 0.0187890812890811*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[1378] = -0.0606730749959909*G0_0_0 - 0.0322284936000211*G0_0_1 - 0.0322284936000212*G0_0_2 - 0.032228493600021*G0_1_0 - 0.128522535206562*G0_1_1 - 0.128522535206562*G0_1_2 - 0.032228493600021*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1379] = 0.0226708532915131*G0_0_0 + 0.0219858347028484*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028486*G0_1_0 + 0.064261267603281*G0_1_1 + 0.0642612676032806*G0_1_2 + 0.0219858347028486*G0_2_0 + 0.064261267603281*G0_2_1 + 0.0642612676032807*G0_2_2;
    A[1380] = -0.00939454064454075*G0_0_1 - 0.00939454064454054*G0_0_2 - 0.00939454064454046*G0_1_0 - 0.0306627368259312*G0_1_1 - 0.030662736825931*G0_1_2 - 0.00939454064454049*G0_2_0 - 0.0306627368259312*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[1381] = 0.022670853291513*G0_0_0 + 0.0219858347028487*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028478*G0_1_0 + 0.0642612676032808*G0_1_1 + 0.064261267603281*G0_1_2 + 0.0219858347028477*G0_2_0 + 0.0642612676032807*G0_2_1 + 0.0642612676032809*G0_2_2;
    A[1382] = -0.00329461321214808*G0_0_1 - 0.00329461321214813*G0_0_2 - 0.00329461321214753*G0_1_0 - 0.0091335811821922*G0_1_1 - 0.00913358118219221*G0_1_2 - 0.00329461321214754*G0_2_0 - 0.00913358118219212*G0_2_1 - 0.00913358118219228*G0_2_2;
    A[1383] = -0.00939454064454048*G0_0_1 - 0.00939454064454053*G0_0_2 - 0.00939454064454031*G0_1_0 - 0.0306627368259307*G0_1_1 - 0.0306627368259309*G0_1_2 - 0.0093945406445403*G0_2_0 - 0.0306627368259307*G0_2_1 - 0.0306627368259308*G0_2_2;
    A[1384] = -0.0644569872000422*G0_0_0 - 0.092901568596012*G0_0_1 - 0.160751028806583*G0_0_2 - 0.160751028806583*G0_1_0 - 0.0644569872000419*G0_1_1 - 0.257045070413123*G0_1_2 - 0.160751028806582*G0_2_0 - 0.0644569872000419*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1385] = 0.092086070276173*G0_0_0 + 0.128000616281864*G0_0_1 + 0.134361503176606*G0_0_2 + 0.134361503176606*G0_1_0 + 0.1273155976932*G0_1_1 + 0.176636936077038*G0_1_2 + 0.134361503176606*G0_2_0 + 0.1273155976932*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1386] = -0.0693499771190729*G0_0_0 - 0.0547362472275658*G0_0_1 - 0.0906181733004639*G0_0_2 - 0.0906181733004636*G0_1_0 - 0.0641307878721063*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0906181733004636*G0_2_0 - 0.0641307878721063*G0_2_1 - 0.111886369481854*G0_2_2;
    A[1387] = 0.0439716694056976*G0_0_0 + 0.0446566879943619*G0_0_1 + 0.0862471023061298*G0_0_2 + 0.0862471023061295*G0_1_0 + 0.0439716694056965*G0_1_1 + 0.128522535206562*G0_1_2 + 0.0862471023061295*G0_2_0 + 0.0439716694056965*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1388] = -0.0318696743392927*G0_0_0 - 0.0259654665036611*G0_0_1 - 0.0377086423093362*G0_0_2 - 0.0377086423093365*G0_1_0 - 0.0292600797158087*G0_1_1 - 0.0435476102793806*G0_1_2 - 0.0377086423093365*G0_2_0 - 0.0292600797158087*G0_2_1 - 0.0435476102793806*G0_2_2;
    A[1389] = -0.0187890812890813*G0_0_0 - 0.00939454064454054*G0_0_1 - 0.0400572774704721*G0_0_2 - 0.0400572774704716*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.0613254736518623*G0_1_2 - 0.0400572774704716*G0_2_0 - 0.0187890812890812*G0_2_1 - 0.0613254736518622*G0_2_2;
    A[1390] = -0.0644569872000421*G0_0_0 - 0.160751028806583*G0_0_1 - 0.0929015685960118*G0_0_2 - 0.160751028806582*G0_1_0 - 0.257045070413123*G0_1_1 - 0.0644569872000416*G0_1_2 - 0.160751028806582*G0_2_0 - 0.257045070413123*G0_2_1 - 0.0644569872000417*G0_2_2;
    A[1391] = 0.092086070276173*G0_0_0 + 0.134361503176605*G0_0_1 + 0.128000616281864*G0_0_2 + 0.134361503176605*G0_1_0 + 0.176636936077038*G0_1_1 + 0.1273155976932*G0_1_2 + 0.134361503176605*G0_2_0 + 0.176636936077038*G0_2_1 + 0.1273155976932*G0_2_2;
    A[1392] = -0.0693499771190729*G0_0_0 - 0.0906181733004636*G0_0_1 - 0.0547362472275655*G0_0_2 - 0.0906181733004635*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0641307878721061*G0_1_2 - 0.0906181733004636*G0_2_0 - 0.111886369481854*G0_2_1 - 0.0641307878721062*G0_2_2;
    A[1393] = 0.0439716694056969*G0_0_0 + 0.0862471023061296*G0_0_1 + 0.0446566879943616*G0_0_2 + 0.086247102306129*G0_1_0 + 0.128522535206562*G0_1_1 + 0.043971669405697*G0_1_2 + 0.086247102306129*G0_2_0 + 0.128522535206562*G0_2_1 + 0.043971669405697*G0_2_2;
    A[1394] = -0.0318696743392923*G0_0_0 - 0.0377086423093359*G0_0_1 - 0.0259654665036605*G0_0_2 - 0.0377086423093363*G0_1_0 - 0.0435476102793804*G0_1_1 - 0.0292600797158087*G0_1_2 - 0.0377086423093364*G0_2_0 - 0.0435476102793804*G0_2_1 - 0.0292600797158087*G0_2_2;
    A[1395] = -0.018789081289081*G0_0_0 - 0.040057277470472*G0_0_1 - 0.00939454064454039*G0_0_2 - 0.0400572774704715*G0_1_0 - 0.0613254736518622*G0_1_1 - 0.018789081289081*G0_1_2 - 0.0400572774704714*G0_2_0 - 0.0613254736518622*G0_2_1 - 0.0187890812890811*G0_2_2;
    A[1396] = 0.166687856575008*G0_0_0 + 0.131458329157981*G0_0_1 + 0.131458329157981*G0_0_2 + 0.131458329157981*G0_1_0 + 0.0962288017409536*G0_1_1 + 0.0962288017409528*G0_1_2 + 0.131458329157981*G0_2_0 + 0.0962288017409536*G0_2_1 + 0.0962288017409528*G0_2_2;
    A[1397] = -0.151356488162043*G0_0_0 - 0.153966082785525*G0_0_1 - 0.153966082785526*G0_0_2 - 0.153966082785526*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.153966082785526*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1398] = -0.0453417065830253*G0_0_0 - 0.0479513012065088*G0_0_1 - 0.047951301206509*G0_0_2 - 0.0479513012065091*G0_1_0 - 0.0505608958299923*G0_1_1 - 0.050560895829992*G0_1_2 - 0.047951301206509*G0_2_0 - 0.0505608958299922*G0_2_1 - 0.050560895829992*G0_2_2;
    A[1399] = -0.0453417065830263*G0_0_0 - 0.0479513012065097*G0_0_1 - 0.0479513012065095*G0_0_2 - 0.0479513012065088*G0_1_0 - 0.0505608958299918*G0_1_1 - 0.0505608958299925*G0_1_2 - 0.0479513012065088*G0_2_0 - 0.0505608958299918*G0_2_1 - 0.0505608958299926*G0_2_2;
    A[1400] = 0.0551276864210884*G0_0_0 + 0.0551276864210884*G0_0_1 + 0.0551276864210884*G0_0_2 + 0.0214691349673985*G0_1_0 + 0.0214691349673984*G0_1_1 + 0.0214691349673985*G0_1_2 + 0.0214691349673985*G0_2_0 + 0.0214691349673984*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[1401] = -0.0388177200243167*G0_0_0 - 0.0232632312710432*G0_1_0 - 0.0232632312710432*G0_2_0;
    A[1402] = -0.00336507226698194*G0_1_1 - 0.00336507226698193*G0_2_1;
    A[1403] = -0.00336507226698186*G0_1_2 - 0.00336507226698186*G0_2_2;
    A[1404] = 0.00228991928210685*G0_1_1 + 0.00228991928210686*G0_1_2 + 0.00228991928210677*G0_2_1 + 0.00228991928210684*G0_2_2;
    A[1405] = 0.00178757231708619*G0_1_1 + 0.00178757231708606*G0_1_2 + 0.00178757231708626*G0_2_1 + 0.00178757231708605*G0_2_2;
    A[1406] = 0.00178757231708612*G0_1_1 + 0.00178757231708635*G0_1_2 + 0.00178757231708607*G0_2_1 + 0.00178757231708636*G0_2_2;
    A[1407] = 0.00228991928210686*G0_1_1 + 0.00228991928210663*G0_1_2 + 0.00228991928210691*G0_2_1 + 0.00228991928210666*G0_2_2;
    A[1408] = -0.0315434750113564*G0_0_0 - 0.0599162925551803*G0_0_2 - 0.0599162925551807*G0_1_0 - 0.0788945694544637*G0_1_2 - 0.0599162925551806*G0_2_0 - 0.0788945694544636*G0_2_2;
    A[1409] = -0.00626302709636014*G0_0_0 - 0.0057802520910162*G0_0_2 - 0.00578025209101598*G0_1_0 - 0.00523223722008507*G0_1_2 - 0.00578025209101588*G0_2_0 - 0.00523223722008488*G0_2_2;
    A[1410] = 6.52398655867024e-05*G0_0_0 - 0.0131523569023566*G0_0_2 - 0.0131523569023569*G0_1_0 - 0.0263047138047131*G0_1_2 - 0.013152356902357*G0_2_0 - 0.0263047138047133*G0_2_2;
    A[1411] = 0.00228991928210667*G0_0_2 + 0.00228991928210692*G0_1_0 + 0.0139743792087538*G0_1_2 + 0.00228991928210696*G0_2_0 + 0.0139743792087539*G0_2_2;
    A[1412] = -0.0315434750113561*G0_0_0 - 0.0599162925551804*G0_0_1 - 0.0599162925551804*G0_1_0 - 0.0788945694544638*G0_1_1 - 0.0599162925551804*G0_2_0 - 0.0788945694544637*G0_2_1;
    A[1413] = -0.00626302709636052*G0_0_0 - 0.00578025209101584*G0_0_1 - 0.00578025209101592*G0_1_0 - 0.00523223722008457*G0_1_1 - 0.005780252091016*G0_2_0 - 0.00523223722008474*G0_2_1;
    A[1414] = 6.52398655874223e-05*G0_0_0 - 0.0131523569023566*G0_0_1 - 0.0131523569023567*G0_1_0 - 0.0263047138047135*G0_1_1 - 0.0131523569023566*G0_2_0 - 0.0263047138047134*G0_2_1;
    A[1415] = 0.00228991928210666*G0_0_1 + 0.00228991928210686*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.00228991928210684*G0_2_0 + 0.0139743792087541*G0_2_1;
    A[1416] = 0.0124934342599272*G0_0_0 + 0.0124934342599272*G0_0_1 - 0.0574176057031952*G0_0_2 - 0.0574176057031953*G0_1_0 - 0.0574176057031952*G0_1_1 - 0.0186912214907005*G0_1_2 - 0.0574176057031952*G0_2_0 - 0.0574176057031952*G0_2_1 - 0.0186912214907005*G0_2_2;
    A[1417] = 0.000260959462348325*G0_0_0 + 0.000260959462348257*G0_0_1 + 0.00506261356955768*G0_0_2 + 0.00506261356955787*G0_1_0 + 0.00506261356955775*G0_1_1 + 0.000260959462348352*G0_1_2 + 0.00506261356955781*G0_2_0 + 0.00506261356955769*G0_2_1 + 0.000260959462348312*G0_2_2;
    A[1418] = 6.52398655872819e-05*G0_0_0 + 6.52398655873033e-05*G0_0_1 - 0.000482775005344153*G0_0_2 - 0.000482775005344203*G0_1_0 - 0.000482775005344151*G0_1_1 - 0.00626302709636012*G0_1_2 - 0.000482775005344159*G0_2_0 - 0.000482775005344105*G0_2_1 - 0.00626302709636008*G0_2_2;
    A[1419] = -0.00228991928210699*G0_0_2 - 0.00228991928210695*G0_1_0 - 0.00228991928210695*G0_1_1 + 0.00658922642429553*G0_1_2 - 0.002289919282107*G0_2_0 - 0.002289919282107*G0_2_1 + 0.00658922642429552*G0_2_2;
    A[1420] = 0.0124934342599271*G0_0_0 - 0.0574176057031952*G0_0_1 + 0.0124934342599271*G0_0_2 - 0.0574176057031951*G0_1_0 - 0.0186912214907002*G0_1_1 - 0.0574176057031951*G0_1_2 - 0.0574176057031951*G0_2_0 - 0.0186912214907002*G0_2_1 - 0.0574176057031951*G0_2_2;
    A[1421] = 0.000260959462348437*G0_0_0 + 0.00506261356955795*G0_0_1 + 0.000260959462348377*G0_0_2 + 0.00506261356955793*G0_1_0 + 0.00026095946234807*G0_1_1 + 0.0050626135695579*G0_1_2 + 0.00506261356955787*G0_2_0 + 0.000260959462348115*G0_2_1 + 0.00506261356955784*G0_2_2;
    A[1422] = 6.52398655871212e-05*G0_0_0 - 0.000482775005344263*G0_0_1 + 6.52398655871901e-05*G0_0_2 - 0.000482775005344263*G0_1_0 - 0.00626302709636009*G0_1_1 - 0.000482775005344256*G0_1_2 - 0.000482775005344251*G0_2_0 - 0.00626302709636016*G0_2_1 - 0.000482775005344244*G0_2_2;
    A[1423] = -0.00228991928210689*G0_0_1 - 0.00228991928210687*G0_1_0 + 0.00658922642429564*G0_1_1 - 0.00228991928210686*G0_1_2 - 0.00228991928210684*G0_2_0 + 0.00658922642429568*G0_2_1 - 0.00228991928210683*G0_2_2;
    A[1424] = -0.243357746612952*G0_0_0 - 0.173446706649829*G0_0_1 - 0.17344670664983*G0_0_2 - 0.108337320793916*G0_1_0 - 0.147063705006411*G0_1_1 - 0.147063705006411*G0_1_2 - 0.108337320793916*G0_2_0 - 0.147063705006411*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1425] = 0.428965164208215*G0_0_0 + 0.226095278178608*G0_0_1 + 0.226095278178608*G0_0_2 + 0.226095278178608*G0_1_0 + 0.304226541205703*G0_1_1 + 0.304226541205703*G0_1_2 + 0.226095278178608*G0_2_0 + 0.304226541205703*G0_2_1 + 0.304226541205703*G0_2_2;
    A[1426] = -0.387185554286244*G0_0_0 - 0.167692550505048*G0_0_1 - 0.167692550505048*G0_0_2 - 0.219493003781195*G0_1_0 - 0.204187731314464*G0_1_1 - 0.204187731314464*G0_1_2 - 0.219493003781195*G0_2_0 - 0.204187731314464*G0_2_1 - 0.204187731314464*G0_2_2;
    A[1427] = 0.18526817029421*G0_0_0 + 0.0599162925551803*G0_0_1 + 0.0599162925551803*G0_0_2 + 0.103529142700148*G0_1_0 + 0.0788945694544635*G0_1_1 + 0.0788945694544636*G0_1_2 + 0.103529142700148*G0_2_0 + 0.0788945694544635*G0_2_1 + 0.0788945694544635*G0_2_2;
    A[1428] = -0.0665446628988287*G0_0_0 - 0.0893133759887217*G0_0_1 - 0.0893133759887215*G0_0_2 - 0.0893133759887222*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.0893133759887222*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1429] = -0.000652398655870344*G0_0_0 - 0.00949240044292131*G0_0_1 - 0.00949240044292077*G0_0_2 - 0.00949240044292111*G0_1_0 - 0.0189848008858422*G0_1_1 - 0.0189848008858423*G0_1_2 - 0.00949240044292098*G0_2_0 - 0.0189848008858423*G0_2_1 - 0.0189848008858423*G0_2_2;
    A[1430] = 6.52398655877838e-05*G0_0_1 + 6.52398655874652e-05*G0_0_2 + 6.52398655871719e-05*G0_1_0 - 0.014874689353856*G0_1_1 - 0.0148746893538555*G0_1_2 + 6.52398655870922e-05*G0_2_0 - 0.0148746893538558*G0_2_1 - 0.0148746893538556*G0_2_2;
    A[1431] = -0.000652398655871584*G0_0_0 - 0.00949240044292174*G0_0_1 - 0.0094924004429221*G0_0_2 - 0.00949240044292119*G0_1_0 - 0.0189848008858428*G0_1_1 - 0.0189848008858428*G0_1_2 - 0.00949240044292116*G0_2_0 - 0.0189848008858427*G0_2_1 - 0.0189848008858426*G0_2_2;
    A[1432] = 0.00257697469069007*G0_0_1 + 0.00257697469069018*G0_0_2 + 0.00257697469069007*G0_1_0 + 0.00450155072550932*G0_1_1 + 0.00450155072550887*G0_1_2 + 0.00257697469069008*G0_2_0 + 0.00450155072550928*G0_2_1 + 0.00450155072550892*G0_2_2;
    A[1433] = 6.52398655875692e-05*G0_0_1 + 6.52398655876902e-05*G0_0_2 + 6.52398655872713e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538555*G0_1_2 + 6.52398655872778e-05*G0_2_0 - 0.0148746893538557*G0_2_1 - 0.0148746893538556*G0_2_2;
    A[1434] = 0.0476251018785732*G0_0_0 - 0.0147442096226818*G0_0_1 - 0.0147442096226816*G0_0_2 - 0.0147442096226815*G0_1_0 + 0.0476251018785736*G0_1_1 + 0.0476251018785733*G0_1_2 - 0.0147442096226815*G0_2_0 + 0.0476251018785735*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[1435] = -0.000652398655870844*G0_0_0 + 0.00231601522834165*G0_0_1 + 0.00231601522834137*G0_0_2 + 0.00231601522834186*G0_1_0 + 0.00880738185425668*G0_1_1 + 0.00880738185425674*G0_1_2 + 0.00231601522834183*G0_2_0 + 0.00880738185425676*G0_2_1 + 0.00880738185425686*G0_2_2;
    A[1436] = -6.52398655869029e-05*G0_0_1 - 6.52398655872821e-05*G0_0_2 - 6.52398655870968e-05*G0_1_0 - 0.00130479731174197*G0_1_1 - 0.00130479731174183*G0_1_2 - 6.52398655870199e-05*G0_2_0 - 0.00130479731174192*G0_2_1 - 0.00130479731174183*G0_2_2;
    A[1437] = -0.000652398655870718*G0_0_0 + 0.00231601522834156*G0_0_1 + 0.00231601522834185*G0_0_2 + 0.0023160152283412*G0_1_0 + 0.00880738185425678*G0_1_1 + 0.00880738185425688*G0_1_2 + 0.00231601522834132*G0_2_0 + 0.00880738185425689*G0_2_1 + 0.0088073818542569*G0_2_2;
    A[1438] = -0.00257697469069013*G0_0_1 - 0.00257697469068991*G0_0_2 - 0.00257697469069018*G0_1_0 - 0.0156575677409006*G0_1_1 - 0.015657567740901*G0_1_2 - 0.0025769746906902*G0_2_0 - 0.0156575677409007*G0_2_1 - 0.0156575677409011*G0_2_2;
    A[1439] = -6.52398655872446e-05*G0_0_1 - 6.52398655874737e-05*G0_0_2 - 6.5239865587228e-05*G0_1_0 - 0.00130479731174199*G0_1_1 - 0.00130479731174173*G0_1_2 - 6.52398655872339e-05*G0_2_0 - 0.00130479731174198*G0_2_1 - 0.00130479731174176*G0_2_2;
    A[1440] = -0.00339247301052864*G0_0_0 + 0.0589768384907262*G0_0_1 + 0.194675758911868*G0_0_2 + 0.194675758911868*G0_1_0 + 0.132306447410613*G0_1_1 + 0.121346149991982*G0_1_2 + 0.194675758911868*G0_2_0 + 0.132306447410613*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1441] = -0.0622388317700802*G0_0_0 - 0.160783648739375*G0_0_1 - 0.250814663249556*G0_0_2 - 0.250814663249556*G0_1_0 - 0.166948816037355*G0_1_1 - 0.334680510461756*G0_1_2 - 0.250814663249555*G0_2_0 - 0.166948816037355*G0_2_1 - 0.334680510461756*G0_2_2;
    A[1442] = 0.0846813455320381*G0_0_0 + 0.0893133759887215*G0_0_1 + 0.173472802596064*G0_0_2 + 0.173472802596064*G0_1_0 + 0.127087258163645*G0_1_1 + 0.209419968534549*G0_1_2 + 0.173472802596064*G0_2_0 + 0.127087258163645*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1443] = -0.00678494602105777*G0_0_0 - 0.00975335990526998*G0_0_1 - 0.0371541034518464*G0_0_2 - 0.0371541034518464*G0_1_0 - 0.0436454700777618*G0_1_1 + 0.0371867233846392*G0_1_2 - 0.0371541034518464*G0_2_0 - 0.0436454700777617*G0_2_1 + 0.0371867233846393*G0_2_2;
    A[1444] = 0.0127870136550695*G0_0_0 + 0.00949240044292175*G0_0_1 + 0.0378717419733042*G0_0_2 + 0.0378717419733046*G0_1_0 + 0.0189848008858428*G0_1_1 + 0.056758683060766*G0_1_2 + 0.0378717419733045*G0_2_0 + 0.0189848008858427*G0_2_1 + 0.0567586830607662*G0_2_2;
    A[1445] = -0.000130479731173932*G0_0_0 - 6.52398655874847e-05*G0_0_1 + 0.0136351319077013*G0_0_2 + 0.0136351319077013*G0_1_0 + 0.0148746893538559*G0_1_1 - 0.0254435475789637*G0_1_2 + 0.0136351319077013*G0_2_0 + 0.0148746893538559*G0_2_1 - 0.0254435475789639*G0_2_2;
    A[1446] = -0.00339247301052852*G0_0_0 + 0.194675758911868*G0_0_1 + 0.0589768384907263*G0_0_2 + 0.194675758911867*G0_1_0 + 0.121346149991982*G0_1_1 + 0.132306447410613*G0_1_2 + 0.194675758911867*G0_2_0 + 0.121346149991982*G0_2_1 + 0.132306447410613*G0_2_2;
    A[1447] = -0.0622388317700803*G0_0_0 - 0.250814663249556*G0_0_1 - 0.160783648739375*G0_0_2 - 0.250814663249556*G0_1_0 - 0.334680510461756*G0_1_1 - 0.166948816037355*G0_1_2 - 0.250814663249555*G0_2_0 - 0.334680510461756*G0_2_1 - 0.166948816037355*G0_2_2;
    A[1448] = 0.0846813455320379*G0_0_0 + 0.173472802596064*G0_0_1 + 0.0893133759887214*G0_0_2 + 0.173472802596064*G0_1_0 + 0.209419968534549*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596064*G0_2_0 + 0.209419968534549*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1449] = -0.00678494602105755*G0_0_0 - 0.0371541034518466*G0_0_1 - 0.00975335990526986*G0_0_2 - 0.0371541034518461*G0_1_0 + 0.0371867233846395*G0_1_1 - 0.0436454700777608*G0_1_2 - 0.037154103451846*G0_2_0 + 0.0371867233846396*G0_2_1 - 0.0436454700777609*G0_2_2;
    A[1450] = 0.0127870136550695*G0_0_0 + 0.0378717419733039*G0_0_1 + 0.00949240044292092*G0_0_2 + 0.037871741973304*G0_1_0 + 0.0567586830607658*G0_1_1 + 0.0189848008858423*G0_1_2 + 0.0378717419733041*G0_2_0 + 0.0567586830607658*G0_2_1 + 0.0189848008858423*G0_2_2;
    A[1451] = -0.000130479731174472*G0_0_0 + 0.0136351319077016*G0_0_1 - 6.52398655873635e-05*G0_0_2 + 0.013635131907701*G0_1_0 - 0.0254435475789641*G0_1_1 + 0.0148746893538557*G0_1_2 + 0.013635131907701*G0_2_0 - 0.0254435475789641*G0_2_1 + 0.0148746893538557*G0_2_2;
    A[1452] = -0.161794866655976*G0_0_0 - 0.0662184635708935*G0_0_1 - 0.0662184635708937*G0_0_2 - 0.0662184635708934*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.0662184635708933*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1453] = 0.180714427676232*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182298*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182297*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923198*G0_2_2;
    A[1454] = 0.00130479731174127*G0_0_0 + 0.00717638521457938*G0_0_1 + 0.00717638521457946*G0_0_2 + 0.00717638521457919*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0391439193522517*G0_1_2 + 0.00717638521457909*G0_2_0 + 0.039143919352252*G0_2_1 + 0.0391439193522518*G0_2_2;
    A[1455] = 0.0013047973117423*G0_0_0 + 0.00717638521457976*G0_0_1 + 0.0071763852145798*G0_0_2 + 0.00717638521458007*G0_1_0 + 0.0391439193522524*G0_1_1 + 0.0391439193522522*G0_1_2 + 0.00717638521457992*G0_2_0 + 0.0391439193522523*G0_2_1 + 0.0391439193522523*G0_2_2;
    A[1456] = -0.0388177200243167*G0_0_0 - 0.0388177200243167*G0_0_1 - 0.0388177200243167*G0_0_2 - 0.0155544887532732*G0_1_0 - 0.0155544887532732*G0_1_1 - 0.0155544887532732*G0_1_2 - 0.0155544887532732*G0_2_0 - 0.0155544887532732*G0_2_1 - 0.0155544887532732*G0_2_2;
    A[1457] = 0.0551276864210885*G0_0_0 + 0.0336585514536897*G0_1_0 + 0.0336585514536897*G0_2_0;
    A[1458] = -0.00336507226698199*G0_1_1 - 0.00336507226698198*G0_2_1;
    A[1459] = -0.00336507226698204*G0_1_2 - 0.00336507226698201*G0_2_2;
    A[1460] = 0.00228991928210694*G0_1_1 + 0.00228991928210697*G0_1_2 + 0.00228991928210691*G0_2_1 + 0.00228991928210698*G0_2_2;
    A[1461] = 0.00178757231708605*G0_1_1 + 0.00178757231708599*G0_1_2 + 0.00178757231708608*G0_2_1 + 0.00178757231708594*G0_2_2;
    A[1462] = 0.00178757231708628*G0_1_1 + 0.00178757231708634*G0_1_2 + 0.00178757231708624*G0_2_1 + 0.00178757231708636*G0_2_2;
    A[1463] = 0.00228991928210665*G0_1_1 + 0.00228991928210678*G0_1_2 + 0.00228991928210667*G0_2_1 + 0.00228991928210676*G0_2_2;
    A[1464] = 0.0124934342599269*G0_0_0 + 0.0699110399631218*G0_0_2 + 0.0699110399631222*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0699110399631223*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1465] = 0.000260959462348128*G0_0_0 - 0.00480165410720924*G0_0_2 - 0.00480165410720958*G0_1_0 - 0.00960330821441847*G0_1_2 - 0.00480165410720961*G0_2_0 - 0.00960330821441866*G0_2_2;
    A[1466] = 6.52398655875002e-05*G0_0_0 + 0.000548014870931269*G0_0_2 + 0.000548014870931564*G0_1_0 - 0.00523223722008506*G0_1_2 + 0.000548014870931577*G0_2_0 - 0.0052322372200849*G0_2_2;
    A[1467] = 0.00228991928210686*G0_0_2 + 0.00228991928210661*G0_1_0 + 0.0111690649885097*G0_1_2 + 0.00228991928210663*G0_2_0 + 0.0111690649885096*G0_2_2;
    A[1468] = 0.0124934342599266*G0_0_0 + 0.069911039963122*G0_0_1 + 0.0699110399631221*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0699110399631222*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1469] = 0.000260959462348485*G0_0_0 - 0.00480165410720955*G0_0_1 - 0.00480165410720944*G0_1_0 - 0.00960330821441886*G0_1_1 - 0.00480165410720944*G0_2_0 - 0.00960330821441882*G0_2_1;
    A[1470] = 6.52398655868416e-05*G0_0_0 + 0.000548014870931297*G0_0_1 + 0.000548014870931375*G0_1_0 - 0.00523223722008466*G0_1_1 + 0.000548014870931361*G0_2_0 - 0.0052322372200847*G0_2_1;
    A[1471] = 0.00228991928210685*G0_0_1 + 0.00228991928210695*G0_1_0 + 0.0111690649885096*G0_1_1 + 0.00228991928210696*G0_2_0 + 0.0111690649885096*G0_2_1;
    A[1472] = -0.0315434750113567*G0_0_0 - 0.0315434750113568*G0_0_1 + 0.028372817543824*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.00939454064454067*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.028372817543824*G0_2_1 + 0.00939454064454067*G0_2_2;
    A[1473] = -0.00626302709636023*G0_0_0 - 0.00626302709636013*G0_0_1 - 0.000482775005344197*G0_0_2 - 0.000482775005344234*G0_1_0 - 0.000482775005344159*G0_1_1 + 6.52398655871407e-05*G0_1_2 - 0.000482775005344211*G0_2_0 - 0.000482775005344137*G0_2_1 + 6.52398655871214e-05*G0_2_2;
    A[1474] = 6.52398655868612e-05*G0_0_0 + 6.52398655868297e-05*G0_0_1 + 0.0132175967679436*G0_0_2 + 0.0132175967679437*G0_1_0 + 0.0132175967679436*G0_1_1 + 6.52398655868659e-05*G0_1_2 + 0.0132175967679437*G0_2_0 + 0.0132175967679436*G0_2_1 + 6.52398655868917e-05*G0_2_2;
    A[1475] = -0.00228991928210655*G0_0_2 - 0.00228991928210661*G0_1_0 - 0.00228991928210659*G0_1_1 + 0.0093945406445408*G0_1_2 - 0.00228991928210662*G0_2_0 - 0.0022899192821066*G0_2_1 + 0.00939454064454076*G0_2_2;
    A[1476] = -0.0315434750113566*G0_0_0 + 0.0283728175438241*G0_0_1 - 0.0315434750113566*G0_0_2 + 0.0283728175438242*G0_1_0 + 0.00939454064454062*G0_1_1 + 0.0283728175438243*G0_1_2 + 0.0283728175438242*G0_2_0 + 0.00939454064454063*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1477] = -0.00626302709636047*G0_0_0 - 0.000482775005344442*G0_0_1 - 0.00626302709636042*G0_0_2 - 0.000482775005344787*G0_1_0 + 6.52398655868992e-05*G0_1_1 - 0.000482775005344844*G0_1_2 - 0.000482775005344716*G0_2_0 + 6.52398655869004e-05*G0_2_1 - 0.000482775005344774*G0_2_2;
    A[1478] = 6.52398655870976e-05*G0_0_0 + 0.0132175967679436*G0_0_1 + 6.52398655870279e-05*G0_0_2 + 0.0132175967679441*G0_1_0 + 6.52398655870687e-05*G0_1_1 + 0.0132175967679441*G0_1_2 + 0.013217596767944*G0_2_0 + 6.52398655870654e-05*G0_2_1 + 0.0132175967679441*G0_2_2;
    A[1479] = -0.00228991928210658*G0_0_1 - 0.00228991928210691*G0_1_0 + 0.00939454064454063*G0_1_1 - 0.0022899192821069*G0_1_2 - 0.00228991928210692*G0_2_0 + 0.00939454064454062*G0_2_1 - 0.00228991928210692*G0_2_2;
    A[1480] = 0.18526817029421*G0_0_0 + 0.125351877739029*G0_0_1 + 0.125351877739029*G0_0_2 + 0.081739027594061*G0_1_0 + 0.100717304493344*G0_1_1 + 0.100717304493344*G0_1_2 + 0.081739027594061*G0_2_0 + 0.100717304493344*G0_2_1 + 0.100717304493344*G0_2_2;
    A[1481] = -0.387185554286244*G0_0_0 - 0.219493003781195*G0_0_1 - 0.219493003781195*G0_0_2 - 0.167692550505048*G0_1_0 - 0.204187731314464*G0_1_1 - 0.204187731314464*G0_1_2 - 0.167692550505048*G0_2_0 - 0.204187731314464*G0_2_1 - 0.204187731314464*G0_2_2;
    A[1482] = 0.428965164208214*G0_0_0 + 0.202869886029605*G0_0_1 + 0.202869886029605*G0_0_2 + 0.202869886029605*G0_1_0 + 0.281001149056701*G0_1_1 + 0.281001149056701*G0_1_2 + 0.202869886029605*G0_2_0 + 0.281001149056701*G0_2_1 + 0.281001149056701*G0_2_2;
    A[1483] = -0.243357746612952*G0_0_0 - 0.0699110399631219*G0_0_1 - 0.0699110399631218*G0_0_2 - 0.135020425819035*G0_1_0 - 0.108637424175617*G0_1_1 - 0.108637424175617*G0_1_2 - 0.135020425819035*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1484] = 0.0476251018785731*G0_0_0 + 0.0623693115012544*G0_0_1 + 0.0623693115012543*G0_0_2 + 0.0623693115012557*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012558*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1485] = -0.000652398655871161*G0_0_0 - 0.00296841388421245*G0_0_1 - 0.00296841388421264*G0_0_2 - 0.00296841388421247*G0_1_0 + 0.00352295274170305*G0_1_1 + 0.00352295274170304*G0_1_2 - 0.00296841388421247*G0_2_0 + 0.00352295274170315*G0_2_1 + 0.00352295274170301*G0_2_2;
    A[1486] = 6.52398655865464e-05*G0_0_1 + 6.52398655865926e-05*G0_0_2 + 6.52398655870439e-05*G0_1_0 - 0.00117431758056783*G0_1_1 - 0.00117431758056773*G0_1_2 + 6.52398655870332e-05*G0_2_0 - 0.00117431758056786*G0_2_1 - 0.0011743175805677*G0_2_2;
    A[1487] = -0.000652398655870256*G0_0_0 - 0.00296841388421182*G0_0_1 - 0.00296841388421171*G0_0_2 - 0.00296841388421242*G0_1_0 + 0.00352295274170311*G0_1_1 + 0.00352295274170352*G0_1_2 - 0.00296841388421248*G0_2_0 + 0.00352295274170302*G0_2_1 + 0.00352295274170335*G0_2_2;
    A[1488] = 0.00257697469068952*G0_0_1 + 0.00257697469068964*G0_0_2 + 0.00257697469068975*G0_1_0 - 0.0105036183595212*G0_1_1 - 0.0105036183595213*G0_1_2 + 0.00257697469068969*G0_2_0 - 0.0105036183595213*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[1489] = 6.52398655865184e-05*G0_0_1 + 6.52398655863477e-05*G0_0_2 + 6.52398655868328e-05*G0_1_0 - 0.00117431758056806*G0_1_1 - 0.00117431758056812*G0_1_2 + 6.52398655868519e-05*G0_2_0 - 0.001174317580568*G0_2_1 - 0.00117431758056803*G0_2_2;
    A[1490] = -0.0665446628988284*G0_0_0 + 0.0227687130898934*G0_0_1 + 0.0227687130898934*G0_0_2 + 0.0227687130898934*G0_1_0 - 0.0150051690850301*G0_1_1 - 0.0150051690850299*G0_1_2 + 0.0227687130898934*G0_2_0 - 0.01500516908503*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1491] = -0.000652398655871142*G0_0_0 + 0.00884000178705049*G0_0_1 + 0.0088400017870505*G0_0_2 + 0.00884000178705046*G0_1_0 - 0.00065239865587052*G0_1_1 - 0.000652398655870505*G0_1_2 + 0.00884000178705043*G0_2_0 - 0.000652398655870567*G0_2_1 - 0.000652398655870602*G0_2_2;
    A[1492] = -6.52398655872777e-05*G0_0_1 - 6.52398655870082e-05*G0_0_2 - 6.5239865587093e-05*G0_1_0 - 0.0150051690850303*G0_1_1 - 0.0150051690850301*G0_1_2 - 6.52398655870886e-05*G0_2_0 - 0.0150051690850302*G0_2_1 - 0.01500516908503*G0_2_2;
    A[1493] = -0.000652398655871068*G0_0_0 + 0.00884000178705023*G0_0_1 + 0.0088400017870501*G0_0_2 + 0.00884000178705083*G0_1_0 - 0.000652398655870894*G0_1_1 - 0.000652398655871038*G0_1_2 + 0.00884000178705084*G0_2_0 - 0.000652398655870906*G0_2_1 - 0.000652398655871013*G0_2_2;
    A[1494] = -0.00257697469068973*G0_0_1 - 0.00257697469068976*G0_0_2 - 0.00257697469068965*G0_1_0 - 0.000652398655871091*G0_1_1 - 0.00065239865587098*G0_1_2 - 0.0025769746906896*G0_2_0 - 0.000652398655871132*G0_2_1 - 0.000652398655870971*G0_2_2;
    A[1495] = -6.52398655868883e-05*G0_0_1 - 6.52398655867834e-05*G0_0_2 - 6.52398655868953e-05*G0_1_0 - 0.01500516908503*G0_1_1 - 0.0150051690850302*G0_1_2 - 6.52398655869131e-05*G0_2_0 - 0.01500516908503*G0_2_1 - 0.0150051690850302*G0_2_2;
    A[1496] = 0.0846813455320395*G0_0_0 - 0.0046320304566821*G0_0_1 - 0.088791457064025*G0_0_2 - 0.0887914570640248*G0_1_0 - 0.0510175748891013*G0_1_1 - 0.0528442911255401*G0_1_2 - 0.0887914570640247*G0_2_0 - 0.0510175748891012*G0_2_1 - 0.05284429112554*G0_2_2;
    A[1497] = -0.0622388317700825*G0_0_0 + 0.0985448169692943*G0_0_1 + 0.188575831479474*G0_0_2 + 0.188575831479474*G0_1_0 + 0.182410664181495*G0_1_1 + 0.104709984267273*G0_1_2 + 0.188575831479474*G0_2_0 + 0.182410664181495*G0_2_1 + 0.104709984267273*G0_2_2;
    A[1498] = -0.00339247301052739*G0_0_0 - 0.0623693115012543*G0_0_1 - 0.198068231922396*G0_0_2 - 0.198068231922395*G0_1_0 - 0.12473862300251*G0_1_1 - 0.271397840842282*G0_1_2 - 0.198068231922395*G0_2_0 - 0.12473862300251*G0_2_1 - 0.271397840842282*G0_2_2;
    A[1499] = 0.0127870136550697*G0_0_0 + 0.00329461321214825*G0_0_1 - 0.0250847283182351*G0_0_2 - 0.025084728318235*G0_1_0 - 0.0155923278753135*G0_1_1 - 0.00619778723077305*G0_1_2 - 0.0250847283182351*G0_2_0 - 0.0155923278753135*G0_2_1 - 0.00619778723077316*G0_2_2;
    A[1500] = -0.00678494602105745*G0_0_0 + 0.00296841388421181*G0_0_1 + 0.0303691574307891*G0_0_2 + 0.0303691574307888*G0_1_0 - 0.00352295274170308*G0_1_1 + 0.104709984267276*G0_1_2 + 0.0303691574307888*G0_2_0 - 0.00352295274170299*G0_2_1 + 0.104709984267276*G0_2_2;
    A[1501] = -0.000130479731174369*G0_0_0 - 6.52398655865464e-05*G0_0_1 - 0.0137656116388754*G0_0_2 - 0.0137656116388753*G0_1_0 + 0.00117431758056791*G0_1_1 - 0.052844291125541*G0_1_2 - 0.0137656116388753*G0_2_0 + 0.00117431758056785*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[1502] = 0.0846813455320392*G0_0_0 - 0.0887914570640255*G0_0_1 - 0.00463203045668263*G0_0_2 - 0.0887914570640252*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0510175748891017*G0_1_2 - 0.0887914570640252*G0_2_0 - 0.0528442911255406*G0_2_1 - 0.0510175748891017*G0_2_2;
    A[1503] = -0.0622388317700818*G0_0_0 + 0.188575831479475*G0_0_1 + 0.0985448169692944*G0_0_2 + 0.188575831479474*G0_1_0 + 0.104709984267274*G0_1_1 + 0.182410664181495*G0_1_2 + 0.188575831479474*G0_2_0 + 0.104709984267274*G0_2_1 + 0.182410664181495*G0_2_2;
    A[1504] = -0.00339247301052741*G0_0_0 - 0.198068231922396*G0_0_1 - 0.0623693115012541*G0_0_2 - 0.198068231922396*G0_1_0 - 0.271397840842282*G0_1_1 - 0.12473862300251*G0_1_2 - 0.198068231922396*G0_2_0 - 0.271397840842282*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1505] = 0.0127870136550695*G0_0_0 - 0.0250847283182344*G0_0_1 + 0.00329461321214796*G0_0_2 - 0.0250847283182346*G0_1_0 - 0.00619778723077257*G0_1_1 - 0.0155923278753137*G0_1_2 - 0.0250847283182347*G0_2_0 - 0.00619778723077263*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[1506] = -0.00678494602105743*G0_0_0 + 0.0303691574307891*G0_0_1 + 0.00296841388421255*G0_0_2 + 0.0303691574307889*G0_1_0 + 0.104709984267275*G0_1_1 - 0.00352295274170304*G0_1_2 + 0.0303691574307889*G0_2_0 + 0.104709984267275*G0_2_1 - 0.00352295274170302*G0_2_2;
    A[1507] = -0.000130479731173968*G0_0_0 - 0.0137656116388757*G0_0_1 - 6.52398655866308e-05*G0_0_2 - 0.0137656116388755*G0_1_0 - 0.052844291125541*G0_1_1 + 0.00117431758056753*G0_1_2 - 0.0137656116388754*G0_2_0 - 0.052844291125541*G0_2_1 + 0.0011743175805675*G0_2_2;
    A[1508] = 0.180714427676232*G0_0_0 + 0.0104383784939334*G0_0_1 + 0.0104383784939342*G0_0_2 + 0.0104383784939337*G0_1_0 + 0.0260959462348341*G0_1_1 + 0.0260959462348349*G0_1_2 + 0.0104383784939338*G0_2_0 + 0.0260959462348341*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[1509] = -0.161794866655977*G0_0_0 - 0.0955764030850822*G0_0_1 - 0.0955764030850829*G0_0_2 - 0.0955764030850829*G0_1_0 - 0.185933616923198*G0_1_1 - 0.185933616923199*G0_1_2 - 0.095576403085083*G0_2_0 - 0.185933616923198*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1510] = 0.00130479731174218*G0_0_0 - 0.00587158790283775*G0_0_1 - 0.0058715879028375*G0_0_2 - 0.00587158790283793*G0_1_0 + 0.026095946234835*G0_1_1 + 0.0260959462348353*G0_1_2 - 0.0058715879028379*G0_2_0 + 0.026095946234835*G0_2_1 + 0.0260959462348354*G0_2_2;
    A[1511] = 0.00130479731174132*G0_0_0 - 0.00587158790283762*G0_0_1 - 0.00587158790283796*G0_0_2 - 0.0058715879028384*G0_1_0 + 0.026095946234835*G0_1_1 + 0.0260959462348349*G0_1_2 - 0.00587158790283836*G0_2_0 + 0.0260959462348351*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[1512] = 0.0153313684129655*G0_0_0 + 0.0153313684129654*G0_0_1 + 0.0153313684129654*G0_0_2 + 0.00480034930989776*G0_1_0 + 0.00480034930989774*G0_1_1 + 0.00480034930989775*G0_1_2 + 0.00480034930989776*G0_2_0 + 0.00480034930989774*G0_2_1 + 0.00480034930989775*G0_2_2;
    A[1513] = -0.0551276864210886*G0_0_0 - 0.0372571824394735*G0_1_0 - 0.0372571824394735*G0_2_0;
    A[1514] = -0.00480034930989787*G0_1_1 - 0.00480034930989785*G0_2_1;
    A[1515] = -0.00480034930989772*G0_1_2 - 0.00480034930989774*G0_2_2;
    A[1516] = 0.0053757649243759*G0_1_1 + 0.00537576492437593*G0_1_2 + 0.00537576492437595*G0_2_1 + 0.00537576492437602*G0_2_2;
    A[1517] = 0.00228991928210683*G0_1_1 + 0.00228991928210672*G0_1_2 + 0.00228991928210677*G0_2_1 + 0.00228991928210663*G0_2_2;
    A[1518] = 0.00228991928210663*G0_1_1 + 0.00228991928210689*G0_1_2 + 0.00228991928210669*G0_2_1 + 0.00228991928210697*G0_2_2;
    A[1519] = 0.00537576492437605*G0_1_1 + 0.0053757649243758*G0_1_2 + 0.00537576492437599*G0_2_1 + 0.00537576492437577*G0_2_2;
    A[1520] = 0.0196371995417135*G0_0_0 - 0.0699110399631221*G0_0_2 - 0.0699110399631226*G0_1_0 - 0.139822079926245*G0_1_2 - 0.0699110399631226*G0_2_0 - 0.139822079926245*G0_2_2;
    A[1521] = -0.0186912214907004*G0_0_0 + 0.0387263842124945*G0_0_2 + 0.0387263842124952*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0387263842124951*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1522] = 0.00939454064454037*G0_0_0 - 0.0189782768992834*G0_0_2 - 0.0189782768992838*G0_1_0 - 0.078894569454464*G0_1_2 - 0.0189782768992837*G0_2_0 - 0.078894569454464*G0_2_2;
    A[1523] = 0.00537576492437582*G0_0_2 + 0.00537576492437611*G0_1_0 + 0.0328352243499807*G0_1_2 + 0.00537576492437605*G0_2_0 + 0.0328352243499807*G0_2_2;
    A[1524] = 0.0196371995417135*G0_0_0 - 0.0699110399631222*G0_0_1 - 0.0699110399631223*G0_1_0 - 0.139822079926245*G0_1_1 - 0.0699110399631224*G0_2_0 - 0.139822079926245*G0_2_1;
    A[1525] = -0.0186912214907005*G0_0_0 + 0.0387263842124948*G0_0_1 + 0.0387263842124948*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0387263842124948*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1526] = 0.00939454064454061*G0_0_0 - 0.0189782768992835*G0_0_1 - 0.0189782768992834*G0_1_0 - 0.0788945694544644*G0_1_1 - 0.0189782768992835*G0_2_0 - 0.0788945694544645*G0_2_1;
    A[1527] = 0.00537576492437598*G0_0_1 + 0.00537576492437591*G0_1_0 + 0.032835224349981*G0_1_1 + 0.00537576492437599*G0_2_0 + 0.032835224349981*G0_2_1;
    A[1528] = 0.0220836945012291*G0_0_0 + 0.0220836945012291*G0_0_1 - 0.005375764924376*G0_0_2 - 0.0053757649243755*G0_1_0 - 0.00537576492437545*G0_1_1 - 0.00537576492437552*G0_2_0 - 0.00537576492437547*G0_2_1;
    A[1529] = 0.00658922642429564*G0_0_0 + 0.0065892264242956*G0_0_1 - 0.00228991928210685*G0_0_2 - 0.00228991928210741*G0_1_0 - 0.00228991928210752*G0_1_1 - 0.0022899192821074*G0_2_0 - 0.0022899192821075*G0_2_1;
    A[1530] = 0.00939454064454079*G0_0_0 + 0.00939454064454078*G0_0_1 - 0.00228991928210655*G0_0_2 - 0.00228991928210617*G0_1_0 - 0.00228991928210614*G0_1_1 - 0.00228991928210618*G0_2_0 - 0.00228991928210614*G0_2_1;
    A[1531] = -0.00537576492437617*G0_0_2 - 0.00537576492437609*G0_1_0 - 0.0053757649243761*G0_1_1 - 0.00537576492437604*G0_2_0 - 0.00537576492437604*G0_2_1;
    A[1532] = 0.022083694501229*G0_0_0 - 0.00537576492437607*G0_0_1 + 0.022083694501229*G0_0_2 - 0.00537576492437573*G0_1_0 - 0.00537576492437568*G0_1_2 - 0.00537576492437571*G0_2_0 - 0.00537576492437566*G0_2_2;
    A[1533] = 0.00658922642429582*G0_0_0 - 0.00228991928210665*G0_0_1 + 0.00658922642429579*G0_0_2 - 0.0022899192821068*G0_1_0 - 0.00228991928210687*G0_1_2 - 0.00228991928210683*G0_2_0 - 0.00228991928210691*G0_2_2;
    A[1534] = 0.00939454064454044*G0_0_0 - 0.0022899192821067*G0_0_1 + 0.00939454064454048*G0_0_2 - 0.0022899192821067*G0_1_0 - 0.00228991928210671*G0_1_2 - 0.00228991928210665*G0_2_0 - 0.00228991928210665*G0_2_2;
    A[1535] = -0.00537576492437604*G0_0_1 - 0.00537576492437592*G0_1_0 - 0.00537576492437592*G0_1_2 - 0.00537576492437601*G0_2_0 - 0.005375764924376*G0_2_2;
    A[1536] = -0.081210584682806*G0_0_0 - 0.0537511252572008*G0_0_1 - 0.0537511252572007*G0_0_2 - 0.0274594594256047*G0_1_0 - 0.0328352243499803*G0_1_1 - 0.0328352243499803*G0_1_2 - 0.0274594594256047*G0_2_0 - 0.0328352243499803*G0_2_1 - 0.0328352243499803*G0_2_2;
    A[1537] = 0.18526817029421*G0_0_0 + 0.103529142700148*G0_0_1 + 0.103529142700148*G0_0_2 + 0.0599162925551803*G0_1_0 + 0.0788945694544635*G0_1_1 + 0.0788945694544635*G0_1_2 + 0.0599162925551803*G0_2_0 + 0.0788945694544635*G0_2_1 + 0.0788945694544635*G0_2_2;
    A[1538] = -0.243357746612952*G0_0_0 - 0.135020425819035*G0_0_1 - 0.135020425819035*G0_0_2 - 0.0699110399631219*G0_1_0 - 0.108637424175617*G0_1_1 - 0.108637424175617*G0_1_2 - 0.0699110399631218*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1539] = 0.179096479009672*G0_0_0 + 0.0699110399631221*G0_0_1 + 0.0699110399631221*G0_0_2 + 0.0699110399631221*G0_1_0 + 0.139822079926245*G0_1_1 + 0.139822079926245*G0_1_2 + 0.0699110399631221*G0_2_0 + 0.139822079926245*G0_2_1 + 0.139822079926245*G0_2_2;
    A[1540] = -0.0606730749959907*G0_0_0 - 0.0284445813959695*G0_0_1 - 0.0284445813959694*G0_0_2 - 0.0284445813959699*G0_1_0 - 0.124738623002511*G0_1_1 - 0.12473862300251*G0_1_2 - 0.0284445813959699*G0_2_0 - 0.124738623002511*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1541] = 0.0226708532915128*G0_0_0 + 0.000685018588664529*G0_0_1 + 0.000685018588664404*G0_0_2 + 0.000685018588664643*G0_1_0 + 0.0429604514890973*G0_1_1 + 0.0429604514890971*G0_1_2 + 0.000685018588664594*G0_2_0 + 0.0429604514890973*G0_2_1 + 0.042960451489097*G0_2_2;
    A[1542] = 0.00939454064454078*G0_0_1 + 0.00939454064454093*G0_0_2 + 0.00939454064454038*G0_1_0 - 0.0118736555368499*G0_1_1 - 0.0118736555368498*G0_1_2 + 0.00939454064454042*G0_2_0 - 0.0118736555368499*G0_2_1 - 0.0118736555368497*G0_2_2;
    A[1543] = 0.0226708532915125*G0_0_0 + 0.000685018588664031*G0_0_1 + 0.000685018588664037*G0_0_2 + 0.000685018588664575*G0_1_0 + 0.0429604514890973*G0_1_1 + 0.042960451489097*G0_1_2 + 0.000685018588664577*G0_2_0 + 0.0429604514890973*G0_2_1 + 0.042960451489097*G0_2_2;
    A[1544] = 0.00329461321214827*G0_0_1 + 0.00329461321214809*G0_0_2 + 0.00329461321214803*G0_1_0 - 0.00254435475789659*G0_1_1 - 0.00254435475789666*G0_1_2 + 0.00329461321214802*G0_2_0 - 0.00254435475789659*G0_2_1 - 0.00254435475789672*G0_2_2;
    A[1545] = 0.00939454064454073*G0_0_1 + 0.00939454064454095*G0_0_2 + 0.00939454064454007*G0_1_0 - 0.0118736555368502*G0_1_1 - 0.0118736555368501*G0_1_2 + 0.00939454064454003*G0_2_0 - 0.0118736555368503*G0_2_1 - 0.0118736555368501*G0_2_2;
    A[1546] = 0.0453417065830252*G0_0_0 - 0.00939454064454067*G0_0_1 - 0.00939454064454063*G0_0_2 - 0.00939454064454055*G0_1_0 - 0.0093945406445406*G0_2_0;
    A[1547] = 0.0226708532915129*G0_0_0 - 0.00329461321214809*G0_0_1 - 0.00329461321214787*G0_0_2 - 0.00329461321214818*G0_1_0 - 0.00329461321214815*G0_2_0;
    A[1548] = -0.00939454064454044*G0_0_1 - 0.00939454064454042*G0_0_2 - 0.00939454064454035*G0_1_0 - 0.00939454064454041*G0_2_0;
    A[1549] = 0.022670853291513*G0_0_0 - 0.00329461321214764*G0_0_1 - 0.00329461321214773*G0_0_2 - 0.0032946132121479*G0_1_0 - 0.0032946132121479*G0_2_0;
    A[1550] = -0.00329461321214801*G0_0_1 - 0.00329461321214826*G0_0_2 - 0.00329461321214808*G0_1_0 - 0.00329461321214807*G0_2_0;
    A[1551] = -0.00939454064454097*G0_0_1 - 0.00939454064454066*G0_0_2 - 0.00939454064454003*G0_1_0 - 0.00939454064453998*G0_2_0;
    A[1552] = -0.0693499771190741*G0_0_0 - 0.0146137298915082*G0_0_1 + 0.02126819618139*G0_0_2 + 0.0212681961813897*G0_1_0 + 0.0118736555368496*G0_1_1 + 0.0212681961813897*G0_2_0 + 0.0118736555368496*G0_2_1;
    A[1553] = 0.0920860702761744*G0_0_0 - 0.0359145460056911*G0_0_1 - 0.0422754329004318*G0_0_2 - 0.0422754329004318*G0_1_0 - 0.0429604514890959*G0_1_1 - 0.0422754329004318*G0_2_0 - 0.042960451489096*G0_2_1;
    A[1554] = -0.064456987200043*G0_0_0 + 0.0284445813959693*G0_0_1 + 0.0962940416065404*G0_0_2 + 0.0962940416065401*G0_1_0 + 0.12473862300251*G0_1_1 + 0.0962940416065402*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1555] = -0.0318696743392925*G0_0_0 - 0.00590420783563166*G0_0_1 + 0.00583896797004442*G0_0_2 + 0.00583896797004501*G0_1_0 + 0.00254435475789648*G0_1_1 + 0.00583896797004502*G0_2_0 + 0.00254435475789647*G0_2_1;
    A[1556] = 0.0439716694056971*G0_0_0 - 0.000685018588663995*G0_0_1 - 0.0422754329004326*G0_0_2 - 0.0422754329004327*G0_1_0 - 0.0429604514890973*G0_1_1 - 0.0422754329004327*G0_2_0 - 0.0429604514890973*G0_2_1;
    A[1557] = -0.0187890812890811*G0_0_0 - 0.00939454064454074*G0_0_1 + 0.0212681961813905*G0_0_2 + 0.0212681961813899*G0_1_0 + 0.0118736555368503*G0_1_1 + 0.0212681961813899*G0_2_0 + 0.0118736555368503*G0_2_1;
    A[1558] = -0.0693499771190737*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.0146137298915081*G0_0_2 + 0.0212681961813901*G0_1_0 + 0.0118736555368499*G0_1_2 + 0.0212681961813902*G0_2_0 + 0.0118736555368499*G0_2_2;
    A[1559] = 0.0920860702761738*G0_0_0 - 0.0422754329004319*G0_0_1 - 0.0359145460056911*G0_0_2 - 0.0422754329004321*G0_1_0 - 0.042960451489096*G0_1_2 - 0.0422754329004322*G0_2_0 - 0.042960451489096*G0_2_2;
    A[1560] = -0.0644569872000427*G0_0_0 + 0.0962940416065403*G0_0_1 + 0.0284445813959691*G0_0_2 + 0.0962940416065401*G0_1_0 + 0.12473862300251*G0_1_2 + 0.0962940416065402*G0_2_0 + 0.12473862300251*G0_2_2;
    A[1561] = -0.0318696743392923*G0_0_0 + 0.00583896797004392*G0_0_1 - 0.00590420783563159*G0_0_2 + 0.00583896797004429*G0_1_0 + 0.00254435475789625*G0_1_2 + 0.00583896797004432*G0_2_0 + 0.00254435475789632*G0_2_2;
    A[1562] = 0.0439716694056969*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.000685018588664283*G0_0_2 - 0.0422754329004322*G0_1_0 - 0.0429604514890968*G0_1_2 - 0.0422754329004323*G0_2_0 - 0.0429604514890967*G0_2_2;
    A[1563] = -0.018789081289081*G0_0_0 + 0.0212681961813906*G0_0_1 - 0.00939454064454095*G0_0_2 + 0.0212681961813902*G0_1_0 + 0.0118736555368497*G0_1_2 + 0.0212681961813901*G0_2_0 + 0.0118736555368496*G0_2_2;
    A[1564] = -0.151356488162043*G0_0_0 + 0.00260959462348375*G0_0_1 + 0.00260959462348341*G0_0_2 + 0.00260959462348403*G0_1_0 + 0.00260959462348414*G0_2_0;
    A[1565] = 0.166687856575008*G0_0_0 + 0.0352295274170269*G0_0_1 + 0.0352295274170273*G0_0_2 + 0.0352295274170263*G0_1_0 + 0.0352295274170263*G0_2_0;
    A[1566] = -0.0453417065830256*G0_0_0 + 0.00260959462348367*G0_0_1 + 0.00260959462348345*G0_0_2 + 0.00260959462348355*G0_1_0 + 0.00260959462348358*G0_2_0;
    A[1567] = -0.0453417065830255*G0_0_0 + 0.00260959462348329*G0_0_1 + 0.00260959462348362*G0_0_2 + 0.00260959462348334*G0_1_0 + 0.00260959462348334*G0_2_0;
    A[1568] = -0.013742125287264*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.0137421252872641*G0_0_2 - 0.0137421252872639*G0_1_0 - 0.013742125287264*G0_1_1 - 0.0137421252872639*G0_1_2 - 0.0137421252872637*G0_2_0 - 0.0137421252872638*G0_2_1 - 0.0137421252872638*G0_2_2;
    A[1569] = 0.0479930547204847*G0_0_0 + 0.0143945239431348*G0_1_0 + 0.0143945239431348*G0_2_0;
    A[1570] = -0.00517743573299141*G0_0_1 + 0.013742125287264*G0_1_1 - 0.00517743573299138*G0_2_1;
    A[1571] = -0.00517743573299123*G0_0_2 - 0.00517743573299137*G0_1_2 + 0.0137421252872638*G0_2_2;
    A[1572] = 0.0425363923627809*G0_0_1 + 0.0118736555368499*G0_0_2 - 0.018789081289081*G0_1_1 - 0.00939454064454033*G0_1_2 + 0.0212681961813908*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[1573] = 0.0403182369328201*G0_0_1 + 0.0540186087061082*G0_0_2 + 0.0148746893538559*G0_1_1 + 0.0149399292194427*G0_1_2 + 0.00123955744615454*G0_2_1 + 0.00117431758056792*G0_2_2;
    A[1574] = 0.0540186087061076*G0_0_1 + 0.04031823693282*G0_0_2 + 0.00117431758056759*G0_1_1 + 0.00123955744615496*G0_1_2 + 0.0149399292194435*G0_2_1 + 0.0148746893538557*G0_2_2;
    A[1575] = 0.0118736555368503*G0_0_1 + 0.042536392362781*G0_0_2 + 0.0118736555368502*G0_1_1 + 0.0212681961813908*G0_1_2 - 0.00939454064454049*G0_2_1 - 0.0187890812890805*G0_2_2;
    A[1576] = -0.0644569872000418*G0_0_0 + 0.0284445813959699*G0_0_2 + 0.192588083213082*G0_1_0 + 0.124738623002511*G0_1_2 + 0.0962940416065409*G0_2_0 + 0.12473862300251*G0_2_2;
    A[1577] = 0.132306447410613*G0_0_0 + 0.0733296089198872*G0_0_2 + 0.0109602974186307*G0_1_0 + 0.146659217839773*G0_1_2 - 0.0623693115012549*G0_2_0 - 0.12473862300251*G0_2_2;
    A[1578] = -0.0510175748891028*G0_0_0 - 0.0463855444324205*G0_0_2 + 0.00182671623643803*G0_1_0 - 0.0823327103709051*G0_1_2 + 0.0377738821749231*G0_2_0 + 0.127087258163645*G0_2_2;
    A[1579] = 0.0118736555368503*G0_0_0 + 0.0264873854283582*G0_0_2 + 0.01187365553685*G0_1_0 + 0.0477555816097485*G0_1_2 - 0.00939454064454059*G0_2_0 - 0.0641307878721062*G0_2_2;
    A[1580] = -0.0644569872000417*G0_0_0 + 0.0284445813959699*G0_0_1 + 0.0962940416065411*G0_1_0 + 0.124738623002511*G0_1_1 + 0.192588083213081*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1581] = 0.132306447410612*G0_0_0 + 0.0733296089198859*G0_0_1 - 0.0623693115012559*G0_1_0 - 0.124738623002511*G0_1_1 + 0.01096029741863*G0_2_0 + 0.146659217839771*G0_2_1;
    A[1582] = -0.0510175748891008*G0_0_0 - 0.046385544432419*G0_0_1 + 0.0377738821749238*G0_1_0 + 0.127087258163646*G0_1_1 + 0.00182671623643899*G0_2_0 - 0.0823327103709037*G0_2_1;
    A[1583] = 0.0118736555368499*G0_0_0 + 0.0264873854283581*G0_0_1 - 0.00939454064454035*G0_1_0 - 0.0641307878721065*G0_1_1 + 0.01187365553685*G0_2_0 + 0.047755581609748*G0_2_1;
    A[1584] = 0.0187890812890812*G0_0_0 + 0.0187890812890813*G0_0_1 + 0.00939454064454105*G0_0_2 + 0.018789081289081*G0_1_0 + 0.0187890812890809*G0_1_1 + 0.00939454064454052*G0_1_2 + 0.00939454064454021*G0_2_0 + 0.00939454064454018*G0_2_1;
    A[1585] = -0.0148746893538559*G0_0_0 - 0.014874689353856*G0_0_1 + 6.52398655862736e-05*G0_0_2 - 0.0148746893538559*G0_1_0 - 0.0148746893538558*G0_1_1 + 6.52398655870486e-05*G0_1_2 + 6.52398655875225e-05*G0_2_0 + 6.52398655876715e-05*G0_2_1;
    A[1586] = -0.00117431758056754*G0_0_0 - 0.00117431758056743*G0_0_1 + 6.52398655877476e-05*G0_0_2 - 0.00117431758056761*G0_1_0 - 0.00117431758056769*G0_1_1 + 6.52398655870589e-05*G0_1_2 + 6.52398655867093e-05*G0_2_0 + 6.52398655866578e-05*G0_2_1;
    A[1587] = -0.01187365553685*G0_0_0 - 0.0118736555368502*G0_0_1 + 0.00939454064454044*G0_0_2 - 0.0118736555368499*G0_1_0 - 0.01187365553685*G0_1_1 + 0.00939454064454079*G0_1_2 + 0.00939454064454065*G0_2_0 + 0.00939454064454062*G0_2_1;
    A[1588] = 0.0187890812890812*G0_0_0 + 0.00939454064454088*G0_0_1 + 0.0187890812890812*G0_0_2 + 0.00939454064454039*G0_1_0 + 0.0093945406445404*G0_1_2 + 0.0187890812890806*G0_2_0 + 0.00939454064453989*G0_2_1 + 0.0187890812890806*G0_2_2;
    A[1589] = -0.014874689353856*G0_0_0 + 6.52398655869229e-05*G0_0_1 - 0.0148746893538558*G0_0_2 + 6.52398655871991e-05*G0_1_0 + 6.52398655873374e-05*G0_1_2 - 0.0148746893538558*G0_2_0 + 6.52398655879289e-05*G0_2_1 - 0.0148746893538556*G0_2_2;
    A[1590] = -0.00117431758056652*G0_0_0 + 6.52398655870405e-05*G0_0_1 - 0.00117431758056658*G0_0_2 + 6.52398655874423e-05*G0_1_0 + 6.52398655874019e-05*G0_1_2 - 0.00117431758056702*G0_2_0 + 6.52398655863432e-05*G0_2_1 - 0.00117431758056704*G0_2_2;
    A[1591] = -0.0118736555368498*G0_0_0 + 0.00939454064454079*G0_0_1 - 0.0118736555368499*G0_0_2 + 0.00939454064454042*G0_1_0 + 0.00939454064454034*G0_1_2 - 0.0118736555368499*G0_2_0 + 0.00939454064454101*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1592] = 0.0453417065830257*G0_0_0 + 0.054736247227566*G0_0_1 + 0.0547362472275661*G0_0_2 + 0.0547362472275658*G0_1_0 + 0.0641307878721062*G0_1_1 + 0.0641307878721062*G0_1_2 + 0.0547362472275655*G0_2_0 + 0.064130787872106*G0_2_1 + 0.0641307878721062*G0_2_2;
    A[1593] = -0.0665446628988287*G0_0_0 - 0.0893133759887222*G0_0_1 - 0.0893133759887222*G0_0_2 - 0.0893133759887217*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.0893133759887215*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1594] = 0.0476251018785731*G0_0_0 + 0.0623693115012557*G0_0_1 + 0.0623693115012558*G0_0_2 + 0.0623693115012544*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012543*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1595] = -0.0606730749959907*G0_0_0 - 0.0284445813959699*G0_0_1 - 0.0284445813959699*G0_0_2 - 0.0284445813959695*G0_1_0 - 0.124738623002511*G0_1_1 - 0.124738623002511*G0_1_2 - 0.0284445813959694*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1596] = 0.647179466623906*G0_0_0 + 0.271397840842284*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842284*G0_1_0 + 0.542795681684568*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842284*G0_2_1 + 0.542795681684565*G0_2_2;
    A[1597] = -0.268135847562927*G0_0_0 + 0.180714427676232*G0_0_1 + 0.0225077536275458*G0_0_2 - 0.0737210481134083*G0_1_0 - 0.20941996853455*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0459941052388962*G0_2_0 + 0.0812236326559222*G0_2_1 - 0.0234863516113517*G0_2_2;
    A[1598] = 0.105688582251081*G0_0_0 - 0.0743734467692791*G0_0_1 + 0.000652398655872236*G0_0_2 + 0.0300103381700599*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255412*G0_1_2 + 0.0528442911255414*G0_2_0 + 0.000652398655872279*G0_2_1 + 0.0534966897814126*G0_2_2;
    A[1599] = -0.26813584756293*G0_0_0 + 0.0225077536275446*G0_0_1 + 0.180714427676232*G0_0_2 - 0.0459941052388984*G0_1_0 - 0.0234863516113528*G0_1_1 + 0.0812236326559235*G0_1_2 - 0.0737210481134107*G0_2_0 - 0.104709984267276*G0_2_1 - 0.20941996853455*G0_2_2;
    A[1600] = 0.113517366121532*G0_0_0 - 0.0851380245911496*G0_0_1 - 0.0851380245911495*G0_0_2 + 0.0127217737894826*G0_1_0 - 0.0137003717732884*G0_1_1 - 0.019898159004061*G0_1_2 + 0.0127217737894828*G0_2_0 - 0.0198981590040617*G0_2_1 - 0.0137003717732875*G0_2_2;
    A[1601] = 0.10568858225108*G0_0_0 + 0.000652398655870343*G0_0_1 - 0.0743734467692809*G0_0_2 + 0.0528442911255409*G0_1_0 + 0.053496689781411*G0_1_1 + 0.000652398655869503*G0_1_2 + 0.0300103381700607*G0_2_0 + 0.0528442911255404*G0_2_1 + 0.10568858225108*G0_2_2;
    A[1602] = -0.0600206763401199*G0_0_0 - 0.0300103381700604*G0_0_1 - 0.0300103381700598*G0_0_2 - 0.0300103381700609*G0_1_0 - 0.0150051690850298*G0_1_2 - 0.0300103381700605*G0_2_0 - 0.01500516908503*G0_2_1;
    A[1603] = -0.0254435475789634*G0_0_0 - 0.00130479731174022*G0_0_1 - 0.012721773789483*G0_0_2 - 0.00130479731174093*G0_1_0 - 0.0006523986558714*G0_1_2 - 0.0127217737894811*G0_2_0 - 0.000652398655869802*G0_2_1;
    A[1604] = -0.105688582251081*G0_0_0 - 0.0300103381700614*G0_0_1 - 0.0528442911255412*G0_0_2 - 0.0300103381700601*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0528442911255416*G0_2_0 - 0.0150051690850311*G0_2_1;
    A[1605] = -0.025443547578966*G0_0_0 - 0.0127217737894834*G0_0_1 - 0.00130479731174251*G0_0_2 - 0.0127217737894825*G0_1_0 - 0.000652398655871177*G0_1_2 - 0.00130479731174264*G0_2_0 - 0.000652398655871984*G0_2_1;
    A[1606] = -0.113517366121531*G0_0_0 - 0.012721773789481*G0_0_1 - 0.0127217737894811*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.000652398655870831*G0_1_2 - 0.0127217737894824*G0_2_0 - 0.000652398655870162*G0_2_1;
    A[1607] = -0.105688582251081*G0_0_0 - 0.0528442911255404*G0_0_1 - 0.0300103381700601*G0_0_2 - 0.0528442911255411*G0_1_0 - 0.0150051690850304*G0_1_2 - 0.030010338170061*G0_2_0 - 0.0150051690850304*G0_2_1;
    A[1608] = -0.0456679059109615*G0_0_0 - 0.0756782440810211*G0_0_1 - 0.0378391220405106*G0_0_2 - 0.0756782440810212*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255407*G0_1_2 - 0.0378391220405102*G0_2_0 - 0.0528442911255407*G0_2_1;
    A[1609] = 0.0391439193522527*G0_0_0 + 0.135698920421141*G0_0_1 + 0.0678494602105698*G0_0_2 + 0.135698920421141*G0_1_0 + 0.209419968534548*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0678494602105705*G0_2_0 + 0.104709984267274*G0_2_1;
    A[1610] = 0.0521918924696692*G0_0_0 - 0.271397840842284*G0_0_1 - 0.135698920421143*G0_0_2 - 0.271397840842283*G0_1_0 - 0.542795681684567*G0_1_1 - 0.271397840842283*G0_1_2 - 0.135698920421141*G0_2_0 - 0.271397840842283*G0_2_1;
    A[1611] = 0.0391439193522528*G0_0_0 + 0.0264221455627696*G0_0_1 - 0.00554538857490181*G0_0_2 + 0.0264221455627707*G0_1_0 + 0.0137003717732883*G0_1_1 - 0.00619778723077249*G0_1_2 - 0.00554538857490274*G0_2_0 - 0.00619778723077366*G0_2_1;
    A[1612] = -0.156575677409009*G0_0_0 - 0.0225077536275446*G0_0_1 + 0.0678494602105727*G0_0_2 - 0.0225077536275451*G0_1_0 + 0.0234863516113527*G0_1_1 + 0.104709984267277*G0_1_2 + 0.0678494602105704*G0_2_0 + 0.104709984267276*G0_2_1;
    A[1613] = 0.0521918924696697*G0_0_0 - 0.000652398655870562*G0_0_1 - 0.0378391220405116*G0_0_2 - 0.000652398655870728*G0_1_0 - 0.0534966897814114*G0_1_1 - 0.0528442911255418*G0_1_2 - 0.03783912204051*G0_2_0 - 0.0528442911255405*G0_2_1;
    A[1614] = -0.0456679059109612*G0_0_0 - 0.0378391220405105*G0_0_1 - 0.0756782440810212*G0_0_2 - 0.0378391220405107*G0_1_0 - 0.0528442911255413*G0_1_2 - 0.0756782440810203*G0_2_0 - 0.0528442911255405*G0_2_1 - 0.105688582251082*G0_2_2;
    A[1615] = 0.0391439193522524*G0_0_0 + 0.0678494602105694*G0_0_1 + 0.135698920421141*G0_0_2 + 0.0678494602105706*G0_1_0 + 0.104709984267273*G0_1_2 + 0.13569892042114*G0_2_0 + 0.104709984267273*G0_2_1 + 0.209419968534548*G0_2_2;
    A[1616] = 0.0521918924696691*G0_0_0 - 0.135698920421142*G0_0_1 - 0.271397840842283*G0_0_2 - 0.135698920421141*G0_1_0 - 0.271397840842283*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684565*G0_2_2;
    A[1617] = 0.0391439193522522*G0_0_0 - 0.00554538857490099*G0_0_1 + 0.0264221455627717*G0_0_2 - 0.00554538857490255*G0_1_0 - 0.00619778723077251*G0_1_2 + 0.0264221455627697*G0_2_0 - 0.00619778723077209*G0_2_1 + 0.013700371773289*G0_2_2;
    A[1618] = -0.156575677409008*G0_0_0 + 0.0678494602105718*G0_0_1 - 0.022507753627546*G0_0_2 + 0.0678494602105713*G0_1_0 + 0.104709984267275*G0_1_2 - 0.0225077536275438*G0_2_0 + 0.104709984267276*G0_2_1 + 0.0234863516113514*G0_2_2;
    A[1619] = 0.0521918924696676*G0_0_0 - 0.0378391220405128*G0_0_1 - 0.000652398655872306*G0_0_2 - 0.0378391220405113*G0_1_0 - 0.0528442911255413*G0_1_2 - 0.000652398655871938*G0_2_0 - 0.0528442911255417*G0_2_1 - 0.0534966897814124*G0_2_2;
    A[1620] = 0.19571959676126*G0_0_0 + 0.0750258454251489*G0_0_1 + 0.0750258454251475*G0_0_2 + 0.0750258454251488*G0_1_0 + 0.0260959462348341*G0_1_2 + 0.0750258454251486*G0_2_0 + 0.0260959462348339*G0_2_1;
    A[1621] = -0.782878387045047*G0_0_0 - 0.316413348097373*G0_0_1 - 0.316413348097373*G0_0_2 - 0.316413348097372*G0_1_0 - 0.185933616923198*G0_1_2 - 0.316413348097371*G0_2_0 - 0.185933616923197*G0_2_1;
    A[1622] = 0.293579395141891*G0_0_0 + 0.0750258454251511*G0_0_1 + 0.058715879028378*G0_0_2 + 0.0750258454251492*G0_1_0 + 0.026095946234834*G0_1_2 + 0.0587158790283772*G0_2_0 + 0.0260959462348349*G0_2_1;
    A[1623] = 0.293579395141895*G0_0_0 + 0.0587158790283802*G0_0_1 + 0.0750258454251542*G0_0_2 + 0.0587158790283809*G0_1_0 + 0.0260959462348379*G0_1_2 + 0.0750258454251532*G0_2_0 + 0.0260959462348352*G0_2_1;
    A[1624] = -0.0094649996993745*G0_0_0 - 0.00946499969937443*G0_0_1 - 0.00946499969937444*G0_0_2 - 0.00946499969937474*G0_1_0 - 0.00946499969937467*G0_1_1 - 0.00946499969937473*G0_1_2 - 0.00946499969937458*G0_2_0 - 0.00946499969937456*G0_2_1 - 0.00946499969937454*G0_2_2;
    A[1625] = -0.0345719095719092*G0_0_0 - 0.00472467106581693*G0_1_0 - 0.00472467106581678*G0_2_0;
    A[1626] = -0.00472467106581692*G0_0_1 - 0.0345719095719095*G0_1_1 - 0.00472467106581676*G0_2_1;
    A[1627] = -0.00227817610630131*G0_0_2 - 0.00227817610630131*G0_1_2 + 0.00946499969937473*G0_2_2;
    A[1628] = -0.0845508658008654*G0_0_1 - 0.0429604514890975*G0_0_2 + 0.0439716694056968*G0_1_1 - 0.000685018588664922*G0_1_2 - 0.0422754329004326*G0_2_1 - 0.0429604514890967*G0_2_2;
    A[1629] = -0.0808321934624001*G0_0_1 - 0.108232937008976*G0_0_2 - 0.0436454700777606*G0_1_1 - 0.0338921101724903*G0_1_2 - 0.00649136662591457*G0_2_1 - 0.00352295274170246*G0_2_2;
    A[1630] = -0.0093945406445407*G0_0_1 - 0.0377738821749245*G0_0_2 - 0.0155923278753141*G0_1_1 - 0.0188869410874627*G0_1_2 + 0.00949240044292053*G0_2_1 + 0.0189848008858421*G0_2_2;
    A[1631] = 0.0025443547578967*G0_0_1 + 0.0142875305635731*G0_0_2 + 0.00254435475789674*G0_1_1 + 0.00844856259352872*G0_1_2 - 0.0032946132121474*G0_2_1 - 0.0292600797158084*G0_2_2;
    A[1632] = 0.0439716694056967*G0_0_0 - 0.000685018588664523*G0_0_2 - 0.0845508658008649*G0_1_0 - 0.0429604514890973*G0_1_2 - 0.0422754329004329*G0_2_0 - 0.042960451489097*G0_2_2;
    A[1633] = -0.0436454700777609*G0_0_0 - 0.0338921101724915*G0_0_2 - 0.0808321934624008*G0_1_0 - 0.108232937008976*G0_1_2 - 0.00649136662591446*G0_2_0 - 0.00352295274170262*G0_2_2;
    A[1634] = -0.0155923278753142*G0_0_0 - 0.0188869410874623*G0_0_2 - 0.00939454064454092*G0_1_0 - 0.037773882174925*G0_1_2 + 0.00949240044292071*G0_2_0 + 0.0189848008858423*G0_2_2;
    A[1635] = 0.00254435475789672*G0_0_0 + 0.00844856259352856*G0_0_2 + 0.00254435475789678*G0_1_0 + 0.0142875305635733*G0_1_2 - 0.00329461321214734*G0_2_0 - 0.0292600797158082*G0_2_2;
    A[1636] = 0.1273155976932*G0_0_0 - 0.000685018588664558*G0_0_1 - 0.0070459054834057*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.049321338383838*G0_2_0 - 0.0429604514890971*G0_2_1;
    A[1637] = -0.166948816037355*G0_0_0 - 0.00616516729797926*G0_0_1 + 0.0838658472122003*G0_1_0 + 0.182410664181496*G0_1_1 + 0.167731694424401*G0_2_0 + 0.0777006799142213*G0_2_1;
    A[1638] = 0.182410664181495*G0_0_0 + 0.0838658472122003*G0_0_1 - 0.00616516729797901*G0_1_0 - 0.166948816037356*G0_1_1 + 0.0777006799142208*G0_2_0 + 0.167731694424401*G0_2_1;
    A[1639] = -0.0429604514890978*G0_0_0 - 0.00704590548340633*G0_0_1 - 0.000685018588665085*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0429604514890969*G0_2_0 - 0.0493213383838377*G0_2_1;
    A[1640] = 0.00658922642429591*G0_0_0 + 0.00658922642429591*G0_0_1 + 0.00329461321214764*G0_0_2 + 0.0065892264242963*G0_1_0 + 0.00658922642429639*G0_1_1 + 0.0032946132121483*G0_1_2 + 0.00329461321214844*G0_2_0 + 0.00329461321214854*G0_2_1;
    A[1641] = 0.00450155072550884*G0_0_0 + 0.00450155072550887*G0_0_1 + 0.00257697469069019*G0_0_2 + 0.00450155072550875*G0_1_0 + 0.00450155072550874*G0_1_1 + 0.00257697469068974*G0_1_2 + 0.00257697469068927*G0_2_0 + 0.00257697469068917*G0_2_1;
    A[1642] = -0.0105036183595215*G0_0_0 - 0.0105036183595216*G0_0_1 + 0.00257697469068942*G0_0_2 - 0.0105036183595213*G0_1_0 - 0.0105036183595212*G0_1_1 + 0.00257697469068966*G0_1_2 + 0.00257697469069022*G0_2_0 + 0.0025769746906903*G0_2_1;
    A[1643] = -0.00254435475789674*G0_0_0 - 0.0025443547578967*G0_0_1 + 0.00329461321214841*G0_0_2 - 0.0025443547578967*G0_1_0 - 0.00254435475789668*G0_1_1 + 0.00329461321214846*G0_1_2 + 0.0032946132121474*G0_2_0 + 0.0032946132121474*G0_2_1;
    A[1644] = 0.0292600797158081*G0_0_0 + 0.0259654665036599*G0_0_1 + 0.0292600797158081*G0_0_2 + 0.0259654665036612*G0_1_0 + 0.022670853291513*G0_1_1 + 0.0259654665036611*G0_1_2 + 0.0292600797158088*G0_2_0 + 0.0259654665036605*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[1645] = -0.0189848008858421*G0_0_0 - 0.00949240044292025*G0_0_1 - 0.0189848008858421*G0_0_2 - 0.00949240044292218*G0_1_0 - 0.000652398655871332*G0_1_1 - 0.00949240044292212*G0_1_2 - 0.0189848008858425*G0_2_0 - 0.00949240044292099*G0_2_1 - 0.0189848008858425*G0_2_2;
    A[1646] = 0.00352295274170181*G0_0_0 - 0.00296841388421306*G0_0_1 + 0.00352295274170174*G0_0_2 - 0.00296841388421189*G0_1_0 - 0.000652398655870353*G0_1_1 - 0.00296841388421193*G0_1_2 + 0.00352295274170319*G0_2_0 - 0.00296841388421228*G0_2_1 + 0.00352295274170319*G0_2_2;
    A[1647] = 0.0429604514890975*G0_0_0 + 0.000685018588664801*G0_0_1 + 0.0429604514890976*G0_0_2 + 0.000685018588664926*G0_1_0 + 0.0226708532915129*G0_1_1 + 0.000685018588665031*G0_1_2 + 0.0429604514890967*G0_2_0 + 0.000685018588664296*G0_2_1 + 0.0429604514890969*G0_2_2;
    A[1648] = 0.0226708532915122*G0_0_0 + 0.0259654665036605*G0_0_1 + 0.0259654665036604*G0_0_2 + 0.0259654665036609*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.029260079715809*G0_1_2 + 0.0259654665036604*G0_2_0 + 0.0292600797158089*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[1649] = -0.000652398655870342*G0_0_0 - 0.00949240044292111*G0_0_1 - 0.00949240044292098*G0_0_2 - 0.00949240044292132*G0_1_0 - 0.0189848008858421*G0_1_1 - 0.0189848008858423*G0_1_2 - 0.00949240044292077*G0_2_0 - 0.0189848008858423*G0_2_1 - 0.0189848008858423*G0_2_2;
    A[1650] = -0.000652398655871161*G0_0_0 - 0.00296841388421247*G0_0_1 - 0.00296841388421247*G0_0_2 - 0.00296841388421245*G0_1_0 + 0.00352295274170305*G0_1_1 + 0.00352295274170316*G0_1_2 - 0.00296841388421264*G0_2_0 + 0.00352295274170304*G0_2_1 + 0.00352295274170302*G0_2_2;
    A[1651] = 0.0226708532915128*G0_0_0 + 0.000685018588664643*G0_0_1 + 0.000685018588664595*G0_0_2 + 0.000685018588664525*G0_1_0 + 0.0429604514890973*G0_1_1 + 0.0429604514890973*G0_1_2 + 0.000685018588664403*G0_2_0 + 0.0429604514890971*G0_2_1 + 0.042960451489097*G0_2_2;
    A[1652] = -0.268135847562927*G0_0_0 - 0.0737210481134083*G0_0_1 - 0.0459941052388962*G0_0_2 + 0.180714427676232*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0812236326559222*G0_1_2 + 0.0225077536275458*G0_2_0 - 0.104709984267275*G0_2_1 - 0.0234863516113517*G0_2_2;
    A[1653] = 0.72611970398428*G0_0_0 + 0.270093043530542*G0_0_1 + 0.270093043530542*G0_0_2 + 0.270093043530542*G0_1_0 + 0.726119703984282*G0_1_1 + 0.270093043530543*G0_1_2 + 0.270093043530542*G0_2_0 + 0.270093043530543*G0_2_1 + 0.540186087061083*G0_2_2;
    A[1654] = -0.209419968534549*G0_0_0 + 0.180714427676233*G0_0_1 + 0.0812236326559222*G0_0_2 - 0.0737210481134074*G0_1_0 - 0.268135847562927*G0_1_1 - 0.0459941052388962*G0_1_2 - 0.104709984267275*G0_2_0 + 0.0225077536275452*G0_2_1 - 0.023486351611351*G0_2_2;
    A[1655] = 0.065566064915022*G0_0_0 - 0.0748627457611828*G0_0_1 + 0.0278900425384806*G0_0_2 + 0.0278900425384803*G0_1_0 - 0.149725491522365*G0_1_1 + 0.177126235068944*G0_1_2 - 0.0748627457611826*G0_2_0 - 0.0283793415303823*G0_2_1 - 0.149725491522364*G0_2_2;
    A[1656] = -0.149725491522365*G0_0_0 + 0.0278900425384791*G0_0_1 + 0.177126235068944*G0_0_2 - 0.0748627457611827*G0_1_0 + 0.0655660649150221*G0_1_1 + 0.0278900425384802*G0_1_2 - 0.0283793415303836*G0_2_0 - 0.0748627457611839*G0_2_1 - 0.149725491522366*G0_2_2;
    A[1657] = -0.0137003717732873*G0_0_0 - 0.0198981590040606*G0_0_1 - 0.0851380245911498*G0_0_2 - 0.0198981590040613*G0_1_0 - 0.0137003717732874*G0_1_1 - 0.0851380245911496*G0_1_2 + 0.0127217737894817*G0_2_0 + 0.0127217737894822*G0_2_1 + 0.113517366121531*G0_2_2;
    A[1658] = -0.0254435475789651*G0_0_0 - 0.0241387502672229*G0_0_1 - 0.0127217737894821*G0_0_2 - 0.0241387502672234*G0_1_0 - 0.0228339529554823*G0_1_1 - 0.0120693751336124*G0_1_2 - 0.0127217737894826*G0_2_0 - 0.0120693751336123*G0_2_1;
    A[1659] = 0.0567586830607659*G0_0_0 + 0.028379341530383*G0_0_1 + 0.0283793415303846*G0_0_2 + 0.0283793415303843*G0_1_0 - 0.00489298991902976*G0_1_1 + 0.0141896707651939*G0_1_2 + 0.0283793415303828*G0_2_0 + 0.0141896707651923*G0_2_1;
    A[1660] = 0.209419968534549*G0_0_0 + 0.073721048113409*G0_0_1 + 0.104709984267274*G0_0_2 + 0.0737210481134074*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.036860524056702*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0368605240567041*G0_2_1;
    A[1661] = -0.0166361657247064*G0_0_0 + 0.00146789697571076*G0_0_1 - 0.0181040627004165*G0_0_2 + 0.00146789697571019*G0_1_0 + 0.0195719596761278*G0_1_1 - 0.0102752788299656*G0_1_2 - 0.0181040627004162*G0_2_0 - 0.0102752788299648*G0_2_1;
    A[1662] = 0.149725491522365*G0_0_0 + 0.0748627457611801*G0_0_1 + 0.0283793415303824*G0_0_2 + 0.0748627457611828*G0_1_0 + 0.0195719596761239*G0_1_1 + 0.0141896707651915*G0_1_2 + 0.0283793415303835*G0_2_0 + 0.01418967076519*G0_2_1;
    A[1663] = 0.0137003717732875*G0_0_0 + 0.0198981590040625*G0_0_1 - 0.0127217737894829*G0_0_2 + 0.0198981590040609*G0_1_0 + 0.0260959462348355*G0_1_1 - 0.0120693751336124*G0_1_2 - 0.0127217737894818*G0_2_0 - 0.0120693751336108*G0_2_1;
    A[1664] = -0.0228339529554807*G0_0_0 - 0.024138750267223*G0_0_1 - 0.0120693751336113*G0_0_2 - 0.0241387502672228*G0_1_0 - 0.0254435475789643*G0_1_1 - 0.0127217737894829*G0_1_2 - 0.0120693751336118*G0_2_0 - 0.0127217737894824*G0_2_1;
    A[1665] = -0.00489298991903153*G0_0_0 + 0.0283793415303825*G0_0_1 + 0.0141896707651909*G0_0_2 + 0.0283793415303836*G0_1_0 + 0.0567586830607667*G0_1_1 + 0.0283793415303826*G0_1_2 + 0.0141896707651916*G0_2_0 + 0.0283793415303834*G0_2_1;
    A[1666] = -0.0228339529554802*G0_0_0 + 0.0737210481134081*G0_0_1 + 0.036860524056704*G0_0_2 + 0.0737210481134078*G0_1_0 + 0.20941996853455*G0_1_1 + 0.104709984267273*G0_1_2 + 0.0368605240567046*G0_2_0 + 0.104709984267275*G0_2_1;
    A[1667] = 0.019571959676126*G0_0_0 + 0.00146789697570947*G0_0_1 - 0.0102752788299659*G0_0_2 + 0.00146789697570897*G0_1_0 - 0.016636165724708*G0_1_1 - 0.0181040627004162*G0_1_2 - 0.0102752788299653*G0_2_0 - 0.0181040627004163*G0_2_1;
    A[1668] = 0.0195719596761256*G0_0_0 + 0.0748627457611828*G0_0_1 + 0.0141896707651925*G0_0_2 + 0.0748627457611832*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0283793415303844*G0_1_2 + 0.0141896707651905*G0_2_0 + 0.0283793415303822*G0_2_1;
    A[1669] = 0.0260959462348358*G0_0_0 + 0.0198981590040607*G0_0_1 - 0.0120693751336119*G0_0_2 + 0.0198981590040621*G0_1_0 + 0.0137003717732876*G0_1_1 - 0.0127217737894828*G0_1_2 - 0.012069375133611*G0_2_0 - 0.0127217737894821*G0_2_1;
    A[1670] = -0.0880738185425669*G0_0_0 - 0.0887262171984385*G0_0_1 - 0.10079559233205*G0_0_2 - 0.0887262171984383*G0_1_0 - 0.0880738185425682*G0_1_1 - 0.10079559233205*G0_1_2 - 0.100795592332049*G0_2_0 - 0.10079559233205*G0_2_1 - 0.113517366121531*G0_2_2;
    A[1671] = 0.0195719596761255*G0_0_0 + 0.0606730749959907*G0_0_1 + 0.074862745761182*G0_0_2 + 0.0606730749959902*G0_1_0 + 0.0929668084615983*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0748627457611819*G0_2_0 + 0.121346149991981*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1672] = -0.0880738185425674*G0_0_0 + 0.00913358118219163*G0_0_1 + 0.0459941052388959*G0_0_2 + 0.00913358118219246*G0_1_0 - 0.185933616923199*G0_1_1 - 0.0812236326559225*G0_1_2 + 0.0459941052388965*G0_2_0 - 0.081223632655924*G0_2_1 + 0.0234863516113516*G0_2_2;
    A[1673] = 0.0929668084615987*G0_0_0 + 0.0606730749959902*G0_0_1 + 0.12134614999198*G0_0_2 + 0.0606730749959916*G0_1_0 + 0.0195719596761265*G0_1_1 + 0.0748627457611826*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0748627457611822*G0_2_1 + 0.149725491522364*G0_2_2;
    A[1674] = 0.092966808461599*G0_0_0 - 0.13504652176527*G0_0_1 - 0.270093043530541*G0_0_2 - 0.13504652176527*G0_1_0 + 0.0929668084615997*G0_1_1 - 0.270093043530542*G0_1_2 - 0.27009304353054*G0_2_0 - 0.27009304353054*G0_2_1 - 0.540186087061082*G0_2_2;
    A[1675] = -0.185933616923197*G0_0_0 + 0.00913358118219379*G0_0_1 - 0.0812236326559223*G0_0_2 + 0.00913358118219095*G0_1_0 - 0.0880738185425681*G0_1_1 + 0.0459941052388961*G0_1_2 - 0.081223632655924*G0_2_0 + 0.0459941052388961*G0_2_1 + 0.0234863516113508*G0_2_2;
    A[1676] = 0.146789697570946*G0_0_0 + 0.141896707651915*G0_0_1 + 0.0929668084616*G0_0_2 + 0.141896707651915*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0929668084615989*G0_1_2 + 0.0929668084615999*G0_2_0 + 0.0929668084615987*G0_2_1;
    A[1677] = 0.146789697570947*G0_0_0 - 0.298472385060924*G0_0_1 - 0.102752788299663*G0_0_2 - 0.298472385060923*G0_1_0 - 0.782878387045048*G0_1_1 - 0.298472385060926*G0_1_2 - 0.102752788299663*G0_2_0 - 0.298472385060925*G0_2_1;
    A[1678] = -0.782878387045047*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060924*G0_0_2 - 0.298472385060927*G0_1_0 + 0.146789697570945*G0_1_1 - 0.102752788299663*G0_1_2 - 0.298472385060925*G0_2_0 - 0.102752788299662*G0_2_1;
    A[1679] = -0.0489298991903159*G0_0_0 - 0.0293579395141884*G0_0_1 + 0.0929668084616004*G0_0_2 - 0.0293579395141907*G0_1_0 - 0.0489298991903136*G0_1_1 + 0.0929668084616004*G0_1_2 + 0.0929668084615986*G0_2_0 + 0.0929668084616005*G0_2_1;
    A[1680] = -0.0137421252872638*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872637*G0_1_0 - 0.0137421252872638*G0_1_1 - 0.0137421252872637*G0_1_2 - 0.0137421252872639*G0_2_0 - 0.013742125287264*G0_2_1 - 0.0137421252872639*G0_2_2;
    A[1681] = 0.0137421252872641*G0_0_0 - 0.00517743573299121*G0_1_0 - 0.0051774357329912*G0_2_0;
    A[1682] = 0.014394523943135*G0_0_1 + 0.0479930547204851*G0_1_1 + 0.0143945239431349*G0_2_1;
    A[1683] = -0.0051774357329913*G0_0_2 - 0.00517743573299127*G0_1_2 + 0.0137421252872641*G0_2_2;
    A[1684] = 0.192588083213083*G0_0_1 + 0.124738623002512*G0_0_2 - 0.0644569872000427*G0_1_1 + 0.0284445813959704*G0_1_2 + 0.0962940416065412*G0_2_1 + 0.124738623002512*G0_2_2;
    A[1685] = 0.0109602974186293*G0_0_1 + 0.146659217839771*G0_0_2 + 0.132306447410613*G0_1_1 + 0.0733296089198854*G0_1_2 - 0.0623693115012558*G0_2_1 - 0.124738623002512*G0_2_2;
    A[1686] = 0.00182671623643931*G0_0_1 - 0.082332710370903*G0_0_2 - 0.0510175748891026*G0_1_1 - 0.0463855444324186*G0_1_2 + 0.0377738821749243*G0_2_1 + 0.127087258163647*G0_2_2;
    A[1687] = 0.011873655536849*G0_0_1 + 0.047755581609747*G0_0_2 + 0.0118736555368495*G0_1_1 + 0.0264873854283569*G0_1_2 - 0.00939454064454109*G0_2_1 - 0.0641307878721077*G0_2_2;
    A[1688] = -0.0187890812890808*G0_0_0 - 0.00939454064454038*G0_0_2 + 0.0425363923627809*G0_1_0 + 0.0118736555368501*G0_1_2 + 0.0212681961813905*G0_2_0 + 0.0118736555368498*G0_2_2;
    A[1689] = 0.0148746893538551*G0_0_0 + 0.0149399292194424*G0_0_2 + 0.0403182369328199*G0_1_0 + 0.0540186087061076*G0_1_2 + 0.00123955744615471*G0_2_0 + 0.00117431758056771*G0_2_2;
    A[1690] = 0.00117431758056857*G0_0_0 + 0.00123955744615539*G0_0_2 + 0.0540186087061089*G0_1_0 + 0.0403182369328207*G0_1_2 + 0.0149399292194432*G0_2_0 + 0.0148746893538561*G0_2_2;
    A[1691] = 0.0118736555368489*G0_0_0 + 0.0212681961813897*G0_0_2 + 0.0118736555368494*G0_1_0 + 0.0425363923627806*G0_1_2 - 0.0093945406445412*G0_2_0 - 0.0187890812890817*G0_2_2;
    A[1692] = -0.0641307878721065*G0_0_0 - 0.00939454064454038*G0_0_1 + 0.0264873854283577*G0_1_0 + 0.01187365553685*G0_1_1 + 0.0477555816097477*G0_2_0 + 0.0118736555368497*G0_2_1;
    A[1693] = 0.127087258163646*G0_0_0 + 0.0377738821749238*G0_0_1 - 0.0463855444324183*G0_1_0 - 0.0510175748891015*G0_1_1 - 0.0823327103709028*G0_2_0 + 0.00182671623643948*G0_2_1;
    A[1694] = -0.124738623002512*G0_0_0 - 0.0623693115012559*G0_0_1 + 0.0733296089198849*G0_1_0 + 0.132306447410613*G0_1_1 + 0.146659217839771*G0_2_0 + 0.0109602974186299*G0_2_1;
    A[1695] = 0.124738623002512*G0_0_0 + 0.0962940416065416*G0_0_1 + 0.0284445813959705*G0_1_0 - 0.0644569872000426*G0_1_1 + 0.124738623002512*G0_2_0 + 0.192588083213082*G0_2_1;
    A[1696] = 0.0187890812890807*G0_0_0 + 0.0187890812890808*G0_0_1 + 0.00939454064454029*G0_0_2 + 0.0187890812890806*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454009*G0_1_2 + 0.0093945406445403*G0_2_0 + 0.0093945406445403*G0_2_1;
    A[1697] = -0.0148746893538556*G0_0_0 - 0.0148746893538556*G0_0_1 + 6.52398655874998e-05*G0_0_2 - 0.0148746893538557*G0_1_0 - 0.0148746893538558*G0_1_1 + 6.52398655872548e-05*G0_1_2 + 6.52398655870903e-05*G0_2_0 + 6.52398655871486e-05*G0_2_1;
    A[1698] = -0.00117431758056765*G0_0_0 - 0.00117431758056766*G0_0_1 + 6.52398655869926e-05*G0_0_2 - 0.00117431758056771*G0_1_0 - 0.00117431758056778*G0_1_1 + 6.52398655872507e-05*G0_1_2 + 6.52398655870412e-05*G0_2_0 + 6.52398655870224e-05*G0_2_1;
    A[1699] = -0.011873655536849*G0_0_0 - 0.0118736555368491*G0_0_1 + 0.00939454064454058*G0_0_2 - 0.0118736555368495*G0_1_0 - 0.0118736555368496*G0_1_1 + 0.00939454064454039*G0_1_2 + 0.00939454064454112*G0_2_0 + 0.00939454064454108*G0_2_1;
    A[1700] = 0.0641307878721064*G0_0_0 + 0.054736247227566*G0_0_1 + 0.0641307878721066*G0_0_2 + 0.054736247227565*G0_1_0 + 0.0453417065830246*G0_1_1 + 0.0547362472275651*G0_1_2 + 0.064130787872106*G0_2_0 + 0.0547362472275657*G0_2_1 + 0.0641307878721062*G0_2_2;
    A[1701] = -0.127087258163646*G0_0_0 - 0.0893133759887223*G0_0_1 - 0.127087258163646*G0_0_2 - 0.0893133759887216*G0_1_0 - 0.0665446628988276*G0_1_1 - 0.0893133759887216*G0_1_2 - 0.127087258163645*G0_2_0 - 0.0893133759887215*G0_2_1 - 0.127087258163646*G0_2_2;
    A[1702] = 0.124738623002512*G0_0_0 + 0.0623693115012554*G0_0_1 + 0.124738623002512*G0_0_2 + 0.0623693115012547*G0_1_0 + 0.0476251018785727*G0_1_1 + 0.0623693115012547*G0_1_2 + 0.124738623002511*G0_2_0 + 0.0623693115012546*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1703] = -0.124738623002512*G0_0_0 - 0.0284445813959702*G0_0_1 - 0.124738623002512*G0_0_2 - 0.0284445813959704*G0_1_0 - 0.060673074995991*G0_1_1 - 0.0284445813959705*G0_1_2 - 0.124738623002511*G0_2_0 - 0.0284445813959697*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1704] = 0.00939454064454044*G0_0_1 + 0.00939454064454047*G0_0_2 + 0.00939454064453988*G0_1_0 + 0.0187890812890806*G0_1_1 + 0.0187890812890806*G0_1_2 + 0.00939454064454029*G0_2_0 + 0.0187890812890808*G0_2_1 + 0.0187890812890808*G0_2_2;
    A[1705] = 6.52398655871729e-05*G0_0_1 + 6.5239865587092e-05*G0_0_2 + 6.52398655877819e-05*G0_1_0 - 0.014874689353856*G0_1_1 - 0.0148746893538558*G0_1_2 + 6.5239865587469e-05*G0_2_0 - 0.0148746893538555*G0_2_1 - 0.0148746893538556*G0_2_2;
    A[1706] = 6.52398655870437e-05*G0_0_1 + 6.52398655870332e-05*G0_0_2 + 6.52398655865419e-05*G0_1_0 - 0.00117431758056783*G0_1_1 - 0.00117431758056786*G0_1_2 + 6.52398655865935e-05*G0_2_0 - 0.00117431758056773*G0_2_1 - 0.0011743175805677*G0_2_2;
    A[1707] = 0.00939454064454038*G0_0_1 + 0.00939454064454042*G0_0_2 + 0.00939454064454078*G0_1_0 - 0.0118736555368499*G0_1_1 - 0.0118736555368499*G0_1_2 + 0.00939454064454093*G0_2_0 - 0.0118736555368498*G0_2_1 - 0.0118736555368497*G0_2_2;
    A[1708] = 0.105688582251081*G0_0_0 + 0.0300103381700599*G0_0_1 + 0.0528442911255414*G0_0_2 - 0.0743734467692791*G0_1_0 + 0.105688582251081*G0_1_1 + 0.000652398655872279*G0_1_2 + 0.000652398655872235*G0_2_0 + 0.0528442911255412*G0_2_1 + 0.0534966897814126*G0_2_2;
    A[1709] = -0.209419968534549*G0_0_0 - 0.0737210481134074*G0_0_1 - 0.104709984267275*G0_0_2 + 0.180714427676233*G0_1_0 - 0.268135847562927*G0_1_1 + 0.0225077536275453*G0_1_2 + 0.0812236326559222*G0_2_0 - 0.0459941052388962*G0_2_1 - 0.023486351611351*G0_2_2;
    A[1710] = 0.542795681684567*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842284*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842284*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684565*G0_2_2;
    A[1711] = -0.0137003717732884*G0_0_0 + 0.0127217737894817*G0_0_1 - 0.0198981590040624*G0_0_2 - 0.0851380245911504*G0_1_0 + 0.113517366121531*G0_1_1 - 0.0851380245911501*G0_1_2 - 0.0198981590040629*G0_2_0 + 0.0127217737894819*G0_2_1 - 0.0137003717732892*G0_2_2;
    A[1712] = -0.0234863516113513*G0_0_0 - 0.0459941052388961*G0_0_1 + 0.0812236326559239*G0_0_2 + 0.0225077536275451*G0_1_0 - 0.26813584756293*G0_1_1 + 0.180714427676233*G0_1_2 - 0.104709984267274*G0_2_0 - 0.0737210481134085*G0_2_1 - 0.209419968534551*G0_2_2;
    A[1713] = 0.0534966897814108*G0_0_0 + 0.0528442911255399*G0_0_1 + 0.000652398655870584*G0_0_2 + 0.000652398655869948*G0_1_0 + 0.105688582251081*G0_1_1 - 0.0743734467692812*G0_1_2 + 0.05284429112554*G0_2_0 + 0.0300103381700589*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1714] = -0.105688582251081*G0_0_0 - 0.0756782440810209*G0_0_1 - 0.0528442911255399*G0_0_2 - 0.0756782440810211*G0_1_0 - 0.0456679059109602*G0_1_1 - 0.037839122040509*G0_1_2 - 0.0528442911255417*G0_2_0 - 0.0378391220405106*G0_2_1;
    A[1715] = 0.209419968534551*G0_0_0 + 0.135698920421143*G0_0_1 + 0.104709984267275*G0_0_2 + 0.13569892042114*G0_1_0 + 0.0391439193522525*G0_1_1 + 0.0678494602105696*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0678494602105717*G0_2_1;
    A[1716] = -0.542795681684567*G0_0_0 - 0.271397840842285*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842283*G0_1_0 + 0.0521918924696697*G0_1_1 - 0.13569892042114*G0_1_2 - 0.271397840842284*G0_2_0 - 0.135698920421143*G0_2_1;
    A[1717] = 0.0137003717732866*G0_0_0 + 0.0264221455627687*G0_0_1 - 0.00619778723077426*G0_0_2 + 0.0264221455627691*G0_1_0 + 0.0391439193522508*G0_1_1 - 0.0055453885749035*G0_1_2 - 0.00619778723077414*G0_2_0 - 0.00554538857490298*G0_2_1;
    A[1718] = 0.0234863516113517*G0_0_0 - 0.0225077536275431*G0_0_1 + 0.104709984267275*G0_0_2 - 0.0225077536275452*G0_1_0 - 0.156575677409009*G0_1_1 + 0.0678494602105703*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0678494602105711*G0_2_1;
    A[1719] = -0.053496689781411*G0_0_0 - 0.000652398655871654*G0_0_1 - 0.0528442911255396*G0_0_2 - 0.000652398655869826*G0_1_0 + 0.0521918924696703*G0_1_1 - 0.0378391220405097*G0_1_2 - 0.0528442911255402*G0_2_0 - 0.0378391220405113*G0_2_1;
    A[1720] = -0.0300103381700609*G0_0_1 - 0.0150051690850302*G0_0_2 - 0.0300103381700604*G0_1_0 - 0.0600206763401215*G0_1_1 - 0.0300103381700598*G0_1_2 - 0.0150051690850302*G0_2_0 - 0.0300103381700614*G0_2_1;
    A[1721] = -0.00130479731174112*G0_0_1 - 0.000652398655870774*G0_0_2 - 0.00130479731174136*G0_1_0 - 0.0254435475789642*G0_1_1 - 0.0127217737894822*G0_1_2 - 0.000652398655869969*G0_2_0 - 0.0127217737894824*G0_2_1;
    A[1722] = -0.0300103381700599*G0_0_1 - 0.0150051690850293*G0_0_2 - 0.0300103381700598*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255397*G0_1_2 - 0.0150051690850306*G0_2_0 - 0.0528442911255413*G0_2_1;
    A[1723] = -0.0127217737894822*G0_0_1 - 0.000652398655870253*G0_0_2 - 0.012721773789481*G0_1_0 - 0.0254435475789631*G0_1_1 - 0.00130479731174074*G0_1_2 - 0.000652398655870882*G0_2_0 - 0.00130479731174222*G0_2_1;
    A[1724] = -0.0127217737894818*G0_0_1 - 0.000652398655871931*G0_0_2 - 0.0127217737894833*G0_1_0 - 0.113517366121532*G0_1_1 - 0.0127217737894828*G0_1_2 - 0.000652398655870958*G0_2_0 - 0.0127217737894818*G0_2_1;
    A[1725] = -0.0528442911255399*G0_0_1 - 0.0150051690850296*G0_0_2 - 0.0528442911255411*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0300103381700606*G0_1_2 - 0.0150051690850302*G0_2_0 - 0.030010338170059*G0_2_1;
    A[1726] = -0.0378391220405108*G0_0_1 - 0.0528442911255415*G0_0_2 - 0.0378391220405098*G0_1_0 - 0.0456679059109606*G0_1_1 - 0.0756782440810221*G0_1_2 - 0.0528442911255398*G0_2_0 - 0.0756782440810211*G0_2_1 - 0.105688582251082*G0_2_2;
    A[1727] = -0.0055453885749035*G0_0_1 - 0.00619778723077384*G0_0_2 - 0.00554538857490242*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0264221455627695*G0_1_2 - 0.00619778723077364*G0_2_0 + 0.0264221455627691*G0_2_1 + 0.0137003717732875*G0_2_2;
    A[1728] = -0.0378391220405107*G0_0_1 - 0.0528442911255415*G0_0_2 - 0.0378391220405106*G0_1_0 + 0.0521918924696692*G0_1_1 - 0.000652398655872344*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.000652398655871384*G0_2_1 - 0.0534966897814128*G0_2_2;
    A[1729] = 0.0678494602105719*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0678494602105693*G0_1_0 + 0.0391439193522519*G0_1_1 + 0.13569892042114*G0_1_2 + 0.104709984267274*G0_2_0 + 0.135698920421142*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1730] = 0.067849460210572*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105708*G0_1_0 - 0.156575677409008*G0_1_1 - 0.022507753627545*G0_1_2 + 0.104709984267274*G0_2_0 - 0.0225077536275437*G0_2_1 + 0.0234863516113516*G0_2_2;
    A[1731] = -0.135698920421143*G0_0_1 - 0.271397840842284*G0_0_2 - 0.13569892042114*G0_1_0 + 0.0521918924696703*G0_1_1 - 0.271397840842282*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842284*G0_2_1 - 0.542795681684565*G0_2_2;
    A[1732] = 0.0750258454251494*G0_0_1 + 0.0260959462348339*G0_0_2 + 0.0750258454251493*G0_1_0 + 0.195719596761261*G0_1_1 + 0.0750258454251505*G0_1_2 + 0.0260959462348342*G0_2_0 + 0.0750258454251509*G0_2_1;
    A[1733] = 0.0750258454251487*G0_0_1 + 0.0260959462348359*G0_0_2 + 0.0750258454251508*G0_1_0 + 0.293579395141892*G0_1_1 + 0.0587158790283802*G0_1_2 + 0.0260959462348351*G0_2_0 + 0.0587158790283788*G0_2_1;
    A[1734] = -0.316413348097371*G0_0_1 - 0.185933616923199*G0_0_2 - 0.316413348097373*G0_1_0 - 0.782878387045046*G0_1_1 - 0.316413348097373*G0_1_2 - 0.185933616923198*G0_2_0 - 0.316413348097371*G0_2_1;
    A[1735] = 0.0587158790283783*G0_0_1 + 0.0260959462348369*G0_0_2 + 0.0587158790283809*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0750258454251528*G0_1_2 + 0.0260959462348368*G0_2_0 + 0.0750258454251507*G0_2_1;
    A[1736] = -0.00946499969937482*G0_0_0 - 0.00946499969937479*G0_0_1 - 0.00946499969937479*G0_0_2 - 0.00946499969937481*G0_1_0 - 0.00946499969937485*G0_1_1 - 0.00946499969937479*G0_1_2 - 0.00946499969937499*G0_2_0 - 0.00946499969937499*G0_2_1 - 0.00946499969937497*G0_2_2;
    A[1737] = -0.0345719095719093*G0_0_0 - 0.00472467106581691*G0_1_0 - 0.0047246710658169*G0_2_0;
    A[1738] = -0.00227817610630133*G0_0_1 + 0.00946499969937454*G0_1_1 - 0.00227817610630133*G0_2_1;
    A[1739] = -0.00472467106581711*G0_0_2 - 0.00472467106581675*G0_1_2 - 0.0345719095719094*G0_2_2;
    A[1740] = 0.0142875305635725*G0_0_1 + 0.00254435475789609*G0_0_2 - 0.0292600797158086*G0_1_1 - 0.00329461321214794*G0_1_2 + 0.00844856259352771*G0_2_1 + 0.00254435475789607*G0_2_2;
    A[1741] = -0.0377738821749241*G0_0_1 - 0.00939454064454014*G0_0_2 + 0.0189848008858424*G0_1_1 + 0.00949240044292109*G0_1_2 - 0.0188869410874617*G0_2_1 - 0.0155923278753137*G0_2_2;
    A[1742] = -0.108232937008977*G0_0_1 - 0.0808321934624012*G0_0_2 - 0.00352295274170266*G0_1_1 - 0.0064913666259148*G0_1_2 - 0.033892110172492*G0_2_1 - 0.0436454700777614*G0_2_2;
    A[1743] = -0.0429604514890973*G0_0_1 - 0.0845508658008648*G0_0_2 - 0.0429604514890969*G0_1_1 - 0.0422754329004329*G0_1_2 - 0.000685018588664183*G0_2_1 + 0.043971669405697*G0_2_2;
    A[1744] = 0.127315597693201*G0_0_0 - 0.000685018588664388*G0_0_2 - 0.0493213383838383*G0_1_0 - 0.0429604514890973*G0_1_2 - 0.007045905483405*G0_2_0 - 0.0429604514890968*G0_2_2;
    A[1745] = -0.166948816037357*G0_0_0 - 0.00616516729798015*G0_0_2 + 0.167731694424401*G0_1_0 + 0.077700679914221*G0_1_2 + 0.0838658472122*G0_2_0 + 0.182410664181496*G0_2_2;
    A[1746] = 0.182410664181497*G0_0_0 + 0.0838658472122006*G0_0_2 + 0.0777006799142212*G0_1_0 + 0.167731694424401*G0_1_2 - 0.00616516729797981*G0_2_0 - 0.166948816037357*G0_2_2;
    A[1747] = -0.0429604514890975*G0_0_0 - 0.00704590548340547*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.049321338383838*G0_1_2 - 0.000685018588664239*G0_2_0 + 0.127315597693202*G0_2_2;
    A[1748] = 0.0439716694056961*G0_0_0 - 0.000685018588664448*G0_0_1 - 0.0422754329004329*G0_1_0 - 0.0429604514890972*G0_1_1 - 0.0845508658008649*G0_2_0 - 0.0429604514890968*G0_2_1;
    A[1749] = -0.0436454700777611*G0_0_0 - 0.0338921101724922*G0_0_1 - 0.00649136662591466*G0_1_0 - 0.003522952741702*G0_1_1 - 0.0808321934624008*G0_2_0 - 0.108232937008978*G0_2_1;
    A[1750] = -0.0155923278753143*G0_0_0 - 0.018886941087462*G0_0_1 + 0.00949240044292106*G0_1_0 + 0.0189848008858419*G0_1_1 - 0.00939454064454027*G0_2_0 - 0.0377738821749237*G0_2_1;
    A[1751] = 0.00254435475789601*G0_0_0 + 0.00844856259352758*G0_0_1 - 0.003294613212148*G0_1_0 - 0.0292600797158084*G0_1_1 + 0.00254435475789586*G0_2_0 + 0.0142875305635721*G0_2_1;
    A[1752] = 0.0292600797158088*G0_0_0 + 0.0292600797158087*G0_0_1 + 0.025965466503661*G0_0_2 + 0.0292600797158089*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.0259654665036613*G0_1_2 + 0.025965466503661*G0_2_0 + 0.025965466503661*G0_2_1 + 0.0226708532915138*G0_2_2;
    A[1753] = -0.0189848008858424*G0_0_0 - 0.0189848008858421*G0_0_1 - 0.00949240044292097*G0_0_2 - 0.0189848008858427*G0_1_0 - 0.0189848008858427*G0_1_1 - 0.00949240044292197*G0_1_2 - 0.00949240044292145*G0_2_0 - 0.00949240044292146*G0_2_1 - 0.000652398655871745*G0_2_2;
    A[1754] = 0.00352295274170268*G0_0_0 + 0.00352295274170243*G0_0_1 - 0.00296841388421318*G0_0_2 + 0.00352295274170318*G0_1_0 + 0.00352295274170309*G0_1_1 - 0.00296841388421199*G0_1_2 - 0.00296841388421224*G0_2_0 - 0.00296841388421231*G0_2_1 - 0.000652398655870298*G0_2_2;
    A[1755] = 0.0429604514890974*G0_0_0 + 0.0429604514890976*G0_0_1 + 0.000685018588665028*G0_0_2 + 0.042960451489097*G0_1_0 + 0.0429604514890971*G0_1_1 + 0.000685018588664182*G0_1_2 + 0.000685018588664393*G0_2_0 + 0.000685018588664357*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[1756] = 0.00658922642429611*G0_0_0 + 0.00329461321214826*G0_0_1 + 0.00658922642429607*G0_0_2 + 0.00329461321214801*G0_1_0 + 0.003294613212148*G0_1_2 + 0.00658922642429614*G0_2_0 + 0.00329461321214881*G0_2_1 + 0.00658922642429619*G0_2_2;
    A[1757] = 0.00450155072550883*G0_0_0 + 0.00257697469068948*G0_0_1 + 0.00450155072550878*G0_0_2 + 0.00257697469069014*G0_1_0 + 0.00257697469069006*G0_1_2 + 0.00450155072550868*G0_2_0 + 0.00257697469068902*G0_2_1 + 0.00450155072550857*G0_2_2;
    A[1758] = -0.0105036183595214*G0_0_0 + 0.00257697469069032*G0_0_1 - 0.0105036183595213*G0_0_2 + 0.00257697469068966*G0_1_0 + 0.00257697469068971*G0_1_2 - 0.0105036183595213*G0_2_0 + 0.00257697469069047*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[1759] = -0.00254435475789606*G0_0_0 + 0.00329461321214805*G0_0_1 - 0.00254435475789599*G0_0_2 + 0.00329461321214803*G0_1_0 + 0.00329461321214802*G0_1_2 - 0.0025443547578959*G0_2_0 + 0.00329461321214804*G0_2_1 - 0.00254435475789584*G0_2_2;
    A[1760] = 0.0226708532915132*G0_0_0 + 0.025965466503661*G0_0_1 + 0.025965466503661*G0_0_2 + 0.0259654665036611*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.0292600797158089*G0_1_2 + 0.0259654665036616*G0_2_0 + 0.0292600797158089*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[1761] = -0.000652398655871585*G0_0_0 - 0.00949240044292119*G0_0_1 - 0.00949240044292116*G0_0_2 - 0.00949240044292175*G0_1_0 - 0.0189848008858428*G0_1_1 - 0.0189848008858427*G0_1_2 - 0.00949240044292211*G0_2_0 - 0.0189848008858428*G0_2_1 - 0.0189848008858426*G0_2_2;
    A[1762] = -0.000652398655870256*G0_0_0 - 0.00296841388421242*G0_0_1 - 0.00296841388421248*G0_0_2 - 0.00296841388421183*G0_1_0 + 0.00352295274170311*G0_1_1 + 0.00352295274170303*G0_1_2 - 0.00296841388421171*G0_2_0 + 0.00352295274170352*G0_2_1 + 0.00352295274170336*G0_2_2;
    A[1763] = 0.0226708532915125*G0_0_0 + 0.000685018588664574*G0_0_1 + 0.000685018588664576*G0_0_2 + 0.000685018588664039*G0_1_0 + 0.0429604514890973*G0_1_1 + 0.0429604514890973*G0_1_2 + 0.00068501858866404*G0_2_0 + 0.042960451489097*G0_2_1 + 0.042960451489097*G0_2_2;
    A[1764] = -0.26813584756293*G0_0_0 - 0.0459941052388984*G0_0_1 - 0.0737210481134107*G0_0_2 + 0.0225077536275446*G0_1_0 - 0.0234863516113528*G0_1_1 - 0.104709984267276*G0_1_2 + 0.180714427676232*G0_2_0 + 0.0812236326559235*G0_2_1 - 0.20941996853455*G0_2_2;
    A[1765] = 0.065566064915022*G0_0_0 + 0.0278900425384802*G0_0_1 - 0.0748627457611826*G0_0_2 - 0.0748627457611828*G0_1_0 - 0.149725491522365*G0_1_1 - 0.0283793415303823*G0_1_2 + 0.0278900425384806*G0_2_0 + 0.177126235068944*G0_2_1 - 0.149725491522364*G0_2_2;
    A[1766] = -0.0137003717732884*G0_0_0 - 0.0851380245911504*G0_0_1 - 0.0198981590040629*G0_0_2 + 0.0127217737894817*G0_1_0 + 0.113517366121531*G0_1_1 + 0.0127217737894819*G0_1_2 - 0.0198981590040624*G0_2_0 - 0.0851380245911501*G0_2_1 - 0.0137003717732892*G0_2_2;
    A[1767] = 0.726119703984281*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530542*G0_0_2 + 0.270093043530541*G0_1_0 + 0.540186087061083*G0_1_1 + 0.270093043530541*G0_1_2 + 0.270093043530542*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984283*G0_2_2;
    A[1768] = -0.149725491522365*G0_0_0 + 0.177126235068943*G0_0_1 + 0.0278900425384795*G0_0_2 - 0.0283793415303829*G0_1_0 - 0.149725491522365*G0_1_1 - 0.0748627457611836*G0_1_2 - 0.0748627457611828*G0_2_0 + 0.0278900425384803*G0_2_1 + 0.0655660649150224*G0_2_2;
    A[1769] = -0.209419968534549*G0_0_0 + 0.0812236326559238*G0_0_1 + 0.180714427676233*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0234863516113512*G0_1_1 + 0.0225077536275462*G0_1_2 - 0.0737210481134084*G0_2_0 - 0.0459941052388962*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1770] = -0.0254435475789644*G0_0_0 - 0.0127217737894823*G0_0_1 - 0.0241387502672229*G0_0_2 - 0.0127217737894816*G0_1_0 - 0.012069375133611*G0_1_2 - 0.0241387502672226*G0_2_0 - 0.0120693751336115*G0_2_1 - 0.0228339529554809*G0_2_2;
    A[1771] = -0.0166361657247059*G0_0_0 - 0.0181040627004166*G0_0_1 + 0.00146789697571138*G0_0_2 - 0.0181040627004164*G0_1_0 - 0.0102752788299661*G0_1_2 + 0.00146789697571032*G0_2_0 - 0.0102752788299659*G0_2_1 + 0.0195719596761279*G0_2_2;
    A[1772] = 0.0137003717732886*G0_0_0 - 0.0127217737894823*G0_0_1 + 0.0198981590040613*G0_0_2 - 0.0127217737894817*G0_1_0 - 0.0120693751336106*G0_1_2 + 0.0198981590040624*G0_2_0 - 0.0120693751336113*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[1773] = 0.0567586830607689*G0_0_0 + 0.0283793415303841*G0_0_1 + 0.0283793415303836*G0_0_2 + 0.0283793415303847*G0_1_0 + 0.0141896707651916*G0_1_2 + 0.0283793415303846*G0_2_0 + 0.0141896707651921*G0_2_1 - 0.00489298991903144*G0_2_2;
    A[1774] = 0.149725491522365*G0_0_0 + 0.0283793415303819*G0_0_1 + 0.0748627457611808*G0_0_2 + 0.0283793415303827*G0_1_0 + 0.0141896707651903*G0_1_2 + 0.0748627457611829*G0_2_0 + 0.0141896707651904*G0_2_1 + 0.019571959676125*G0_2_2;
    A[1775] = 0.209419968534549*G0_0_0 + 0.104709984267274*G0_0_1 + 0.0737210481134081*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0368605240567046*G0_1_2 + 0.0737210481134084*G0_2_0 + 0.0368605240567043*G0_2_1 - 0.0228339529554807*G0_2_2;
    A[1776] = -0.0880738185425677*G0_0_0 - 0.100795592332049*G0_0_1 - 0.0887262171984389*G0_0_2 - 0.10079559233205*G0_1_0 - 0.113517366121532*G0_1_1 - 0.10079559233205*G0_1_2 - 0.0887262171984389*G0_2_0 - 0.10079559233205*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[1777] = 0.0195719596761261*G0_0_0 + 0.0748627457611813*G0_0_1 + 0.060673074995991*G0_0_2 + 0.0748627457611823*G0_1_0 + 0.149725491522364*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0606730749959912*G0_2_0 + 0.121346149991981*G0_2_1 + 0.0929668084615982*G0_2_2;
    A[1778] = -0.0880738185425675*G0_0_0 + 0.0459941052388982*G0_0_1 + 0.00913358118219251*G0_0_2 + 0.0459941052388972*G0_1_0 + 0.0234863516113524*G0_1_1 - 0.0812236326559239*G0_1_2 + 0.00913358118219185*G0_2_0 - 0.0812236326559239*G0_2_1 - 0.1859336169232*G0_2_2;
    A[1779] = 0.0929668084615986*G0_0_0 + 0.121346149991983*G0_0_1 + 0.0606730749959912*G0_0_2 + 0.121346149991983*G0_1_0 + 0.149725491522367*G0_1_1 + 0.0748627457611835*G0_1_2 + 0.0606730749959907*G0_2_0 + 0.074862745761183*G0_2_1 + 0.0195719596761264*G0_2_2;
    A[1780] = 0.0929668084616*G0_0_0 - 0.270093043530541*G0_0_1 - 0.13504652176527*G0_0_2 - 0.270093043530542*G0_1_0 - 0.540186087061083*G0_1_1 - 0.270093043530541*G0_1_2 - 0.135046521765269*G0_2_0 - 0.270093043530541*G0_2_1 + 0.092966808461602*G0_2_2;
    A[1781] = -0.185933616923199*G0_0_0 - 0.0812236326559238*G0_0_1 + 0.00913358118219205*G0_0_2 - 0.0812236326559244*G0_1_0 + 0.023486351611351*G0_1_1 + 0.0459941052388957*G0_1_2 + 0.00913358118219186*G0_2_0 + 0.045994105238896*G0_2_1 - 0.0880738185425688*G0_2_2;
    A[1782] = -0.0228339529554804*G0_0_0 - 0.0120693751336115*G0_0_1 - 0.0241387502672219*G0_0_2 - 0.0120693751336113*G0_1_0 - 0.0127217737894819*G0_1_2 - 0.0241387502672228*G0_2_0 - 0.0127217737894824*G0_2_1 - 0.0254435475789645*G0_2_2;
    A[1783] = -0.00489298991903232*G0_0_0 + 0.0141896707651912*G0_0_1 + 0.0283793415303817*G0_0_2 + 0.0141896707651915*G0_1_0 + 0.028379341530383*G0_1_2 + 0.0283793415303834*G0_2_0 + 0.028379341530383*G0_2_1 + 0.0567586830607666*G0_2_2;
    A[1784] = -0.0228339529554795*G0_0_0 + 0.0368605240567044*G0_0_1 + 0.0737210481134101*G0_0_2 + 0.0368605240567046*G0_1_0 + 0.104709984267275*G0_1_2 + 0.0737210481134082*G0_2_0 + 0.104709984267273*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1785] = 0.0195719596761263*G0_0_0 - 0.0102752788299661*G0_0_1 + 0.00146789697570941*G0_0_2 - 0.0102752788299659*G0_1_0 - 0.0181040627004163*G0_1_2 + 0.00146789697571051*G0_2_0 - 0.0181040627004163*G0_2_1 - 0.0166361657247069*G0_2_2;
    A[1786] = 0.0195719596761259*G0_0_0 + 0.0141896707651919*G0_0_1 + 0.0748627457611828*G0_0_2 + 0.0141896707651905*G0_1_0 + 0.0283793415303824*G0_1_2 + 0.0748627457611817*G0_2_0 + 0.0283793415303834*G0_2_1 + 0.149725491522364*G0_2_2;
    A[1787] = 0.0260959462348355*G0_0_0 - 0.0120693751336109*G0_0_1 + 0.0198981590040629*G0_0_2 - 0.0120693751336108*G0_1_0 - 0.0127217737894819*G0_1_2 + 0.0198981590040616*G0_2_0 - 0.0127217737894824*G0_2_1 + 0.0137003717732889*G0_2_2;
    A[1788] = 0.146789697570947*G0_0_0 + 0.0929668084616009*G0_0_1 + 0.141896707651918*G0_0_2 + 0.0929668084616*G0_1_0 + 0.0929668084616014*G0_1_2 + 0.141896707651914*G0_2_0 + 0.0929668084616004*G0_2_1 + 0.146789697570947*G0_2_2;
    A[1789] = 0.146789697570948*G0_0_0 - 0.102752788299662*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299663*G0_1_0 - 0.298472385060924*G0_1_2 - 0.298472385060924*G0_2_0 - 0.298472385060925*G0_2_1 - 0.782878387045049*G0_2_2;
    A[1790] = -0.0489298991903158*G0_0_0 + 0.0929668084615991*G0_0_1 - 0.0293579395141889*G0_0_2 + 0.0929668084615992*G0_1_0 + 0.0929668084615999*G0_1_2 - 0.029357939514191*G0_2_0 + 0.0929668084615994*G0_2_1 - 0.0489298991903151*G0_2_2;
    A[1791] = -0.78287838704505*G0_0_0 - 0.298472385060926*G0_0_1 - 0.298472385060928*G0_0_2 - 0.298472385060926*G0_1_0 - 0.102752788299666*G0_1_2 - 0.298472385060927*G0_2_0 - 0.102752788299663*G0_2_1 + 0.146789697570946*G0_2_2;
    A[1792] = -0.00946499969937458*G0_0_0 - 0.00946499969937462*G0_0_1 - 0.00946499969937461*G0_0_2 - 0.00946499969937467*G0_1_0 - 0.00946499969937463*G0_1_1 - 0.00946499969937468*G0_1_2 - 0.00946499969937445*G0_2_0 - 0.00946499969937448*G0_2_1 - 0.00946499969937452*G0_2_2;
    A[1793] = 0.00946499969937459*G0_0_0 - 0.00227817610630111*G0_1_0 - 0.00227817610630112*G0_2_0;
    A[1794] = -0.00472467106581693*G0_0_1 - 0.0345719095719097*G0_1_1 - 0.00472467106581698*G0_2_1;
    A[1795] = -0.00472467106581676*G0_0_2 - 0.00472467106581703*G0_1_2 - 0.0345719095719096*G0_2_2;
    A[1796] = -0.0493213383838377*G0_0_1 - 0.0429604514890968*G0_0_2 + 0.127315597693202*G0_1_1 - 0.000685018588664325*G0_1_2 - 0.00704590548340559*G0_2_1 - 0.0429604514890977*G0_2_2;
    A[1797] = 0.167731694424402*G0_0_1 + 0.0777006799142209*G0_0_2 - 0.166948816037358*G0_1_1 - 0.00616516729797969*G0_1_2 + 0.0838658472122008*G0_2_1 + 0.182410664181497*G0_2_2;
    A[1798] = 0.0777006799142204*G0_0_1 + 0.167731694424402*G0_0_2 + 0.182410664181497*G0_1_1 + 0.0838658472122003*G0_1_2 - 0.00616516729797992*G0_2_1 - 0.166948816037357*G0_2_2;
    A[1799] = -0.0429604514890966*G0_0_1 - 0.049321338383838*G0_0_2 - 0.0429604514890972*G0_1_1 - 0.00704590548340517*G0_1_2 - 0.000685018588664485*G0_2_1 + 0.127315597693202*G0_2_2;
    A[1800] = -0.0292600797158086*G0_0_0 - 0.00329461321214802*G0_0_2 + 0.0142875305635726*G0_1_0 + 0.00254435475789661*G0_1_2 + 0.00844856259352842*G0_2_0 + 0.00254435475789674*G0_2_2;
    A[1801] = 0.0189848008858425*G0_0_0 + 0.00949240044292149*G0_0_2 - 0.0377738821749243*G0_1_0 - 0.00939454064454089*G0_1_2 - 0.0188869410874625*G0_2_0 - 0.0155923278753142*G0_2_2;
    A[1802] = -0.00352295274170297*G0_0_0 - 0.00649136662591537*G0_0_2 - 0.108232937008978*G0_1_0 - 0.0808321934624011*G0_1_2 - 0.0338921101724914*G0_2_0 - 0.0436454700777616*G0_2_2;
    A[1803] = -0.0429604514890968*G0_0_0 - 0.0422754329004325*G0_0_2 - 0.0429604514890975*G0_1_0 - 0.0845508658008656*G0_1_2 - 0.000685018588664616*G0_2_0 + 0.0439716694056973*G0_2_2;
    A[1804] = -0.0292600797158084*G0_0_0 - 0.00329461321214796*G0_0_1 + 0.00844856259352805*G0_1_0 + 0.00254435475789659*G0_1_1 + 0.0142875305635728*G0_2_0 + 0.00254435475789686*G0_2_1;
    A[1805] = 0.0189848008858421*G0_0_0 + 0.00949240044292132*G0_0_1 - 0.0188869410874622*G0_1_0 - 0.0155923278753142*G0_1_1 - 0.0377738821749243*G0_2_0 - 0.00939454064454119*G0_2_1;
    A[1806] = -0.00352295274170224*G0_0_0 - 0.00649136662591532*G0_0_1 - 0.0338921101724919*G0_1_0 - 0.0436454700777616*G0_1_1 - 0.108232937008977*G0_2_0 - 0.0808321934624004*G0_2_1;
    A[1807] = -0.0429604514890968*G0_0_0 - 0.042275432900432*G0_0_1 - 0.000685018588664362*G0_1_0 + 0.0439716694056973*G0_1_1 - 0.0429604514890976*G0_2_0 - 0.0845508658008658*G0_2_1;
    A[1808] = 0.0292600797158083*G0_0_0 + 0.0292600797158083*G0_0_1 + 0.0259654665036606*G0_0_2 + 0.0292600797158081*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0259654665036605*G0_1_2 + 0.02596546650366*G0_2_0 + 0.02596546650366*G0_2_1 + 0.0226708532915122*G0_2_2;
    A[1809] = -0.0189848008858416*G0_0_0 - 0.0189848008858417*G0_0_1 - 0.00949240044292107*G0_0_2 - 0.018984800885841*G0_1_0 - 0.018984800885841*G0_1_1 - 0.00949240044292039*G0_1_2 - 0.00949240044292008*G0_2_0 - 0.00949240044292014*G0_2_1 - 0.000652398655870076*G0_2_2;
    A[1810] = 0.00352295274170223*G0_0_0 + 0.0035229527417024*G0_0_1 - 0.00296841388421239*G0_0_2 + 0.00352295274170153*G0_1_0 + 0.00352295274170172*G0_1_1 - 0.00296841388421328*G0_1_2 - 0.00296841388421338*G0_2_0 - 0.00296841388421334*G0_2_1 - 0.00065239865587126*G0_2_2;
    A[1811] = 0.0429604514890969*G0_0_0 + 0.0429604514890968*G0_0_1 + 0.000685018588664245*G0_0_2 + 0.0429604514890976*G0_1_0 + 0.0429604514890975*G0_1_1 + 0.000685018588664848*G0_1_2 + 0.000685018588664636*G0_2_0 + 0.000685018588664562*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[1812] = 0.0292600797158084*G0_0_0 + 0.0259654665036607*G0_0_1 + 0.0292600797158083*G0_0_2 + 0.0259654665036606*G0_1_0 + 0.0226708532915129*G0_1_1 + 0.0259654665036606*G0_1_2 + 0.029260079715808*G0_2_0 + 0.0259654665036603*G0_2_1 + 0.0292600797158079*G0_2_2;
    A[1813] = -0.0189848008858423*G0_0_0 - 0.00949240044292112*G0_0_1 - 0.0189848008858423*G0_0_2 - 0.00949240044292138*G0_1_0 - 0.000652398655871013*G0_1_1 - 0.0094924004429215*G0_1_2 - 0.0189848008858424*G0_2_0 - 0.00949240044292098*G0_2_1 - 0.0189848008858425*G0_2_2;
    A[1814] = 0.00352295274170313*G0_0_0 - 0.00296841388421274*G0_0_1 + 0.00352295274170323*G0_0_2 - 0.00296841388421201*G0_1_0 - 0.000652398655870963*G0_1_1 - 0.00296841388421196*G0_1_2 + 0.00352295274170253*G0_2_0 - 0.00296841388421253*G0_2_1 + 0.0035229527417027*G0_2_2;
    A[1815] = 0.0429604514890969*G0_0_0 + 0.000685018588664692*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.000685018588664383*G0_1_0 + 0.0226708532915133*G0_1_1 + 0.000685018588664232*G0_1_2 + 0.0429604514890976*G0_2_0 + 0.00068501858866458*G0_2_1 + 0.0429604514890974*G0_2_2;
    A[1816] = 0.00329461321214759*G0_0_1 + 0.00329461321214759*G0_0_2 + 0.00329461321214792*G0_1_0 + 0.00658922642429542*G0_1_1 + 0.00658922642429545*G0_1_2 + 0.00329461321214764*G0_2_0 + 0.00658922642429534*G0_2_1 + 0.00658922642429528*G0_2_2;
    A[1817] = 0.00257697469069007*G0_0_1 + 0.00257697469069008*G0_0_2 + 0.00257697469069007*G0_1_0 + 0.00450155072550932*G0_1_1 + 0.00450155072550928*G0_1_2 + 0.00257697469069018*G0_2_0 + 0.00450155072550887*G0_2_1 + 0.00450155072550892*G0_2_2;
    A[1818] = 0.00257697469068975*G0_0_1 + 0.00257697469068969*G0_0_2 + 0.00257697469068952*G0_1_0 - 0.0105036183595212*G0_1_1 - 0.0105036183595213*G0_1_2 + 0.00257697469068965*G0_2_0 - 0.0105036183595213*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[1819] = 0.00329461321214803*G0_0_1 + 0.00329461321214802*G0_0_2 + 0.00329461321214827*G0_1_0 - 0.00254435475789659*G0_1_1 - 0.00254435475789659*G0_1_2 + 0.00329461321214809*G0_2_0 - 0.00254435475789666*G0_2_1 - 0.00254435475789672*G0_2_2;
    A[1820] = 0.113517366121532*G0_0_0 + 0.0127217737894826*G0_0_1 + 0.0127217737894828*G0_0_2 - 0.0851380245911496*G0_1_0 - 0.0137003717732884*G0_1_1 - 0.0198981590040617*G0_1_2 - 0.0851380245911495*G0_2_0 - 0.019898159004061*G0_2_1 - 0.0137003717732875*G0_2_2;
    A[1821] = -0.149725491522365*G0_0_0 - 0.0748627457611827*G0_0_1 - 0.0283793415303836*G0_0_2 + 0.0278900425384791*G0_1_0 + 0.0655660649150221*G0_1_1 - 0.0748627457611839*G0_1_2 + 0.177126235068944*G0_2_0 + 0.0278900425384802*G0_2_1 - 0.149725491522366*G0_2_2;
    A[1822] = -0.0234863516113513*G0_0_0 + 0.0225077536275452*G0_0_1 - 0.104709984267274*G0_0_2 - 0.0459941052388961*G0_1_0 - 0.26813584756293*G0_1_1 - 0.0737210481134085*G0_1_2 + 0.0812236326559239*G0_2_0 + 0.180714427676233*G0_2_1 - 0.209419968534551*G0_2_2;
    A[1823] = -0.149725491522365*G0_0_0 - 0.0283793415303829*G0_0_1 - 0.0748627457611828*G0_0_2 + 0.177126235068943*G0_1_0 - 0.149725491522365*G0_1_1 + 0.0278900425384803*G0_1_2 + 0.0278900425384795*G0_2_0 - 0.0748627457611836*G0_2_1 + 0.0655660649150224*G0_2_2;
    A[1824] = 0.540186087061083*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530541*G0_1_0 + 0.726119703984283*G0_1_1 + 0.270093043530541*G0_1_2 + 0.270093043530541*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984284*G0_2_2;
    A[1825] = -0.0234863516113514*G0_0_0 - 0.104709984267275*G0_0_1 + 0.0225077536275452*G0_0_2 + 0.0812236326559241*G0_1_0 - 0.209419968534551*G0_1_1 + 0.180714427676232*G0_1_2 - 0.0459941052388957*G0_2_0 - 0.0737210481134074*G0_2_1 - 0.268135847562929*G0_2_2;
    A[1826] = -0.113517366121531*G0_0_0 - 0.10079559233205*G0_0_1 - 0.100795592332049*G0_0_2 - 0.10079559233205*G0_1_0 - 0.088073818542568*G0_1_1 - 0.0887262171984381*G0_1_2 - 0.100795592332049*G0_2_0 - 0.0887262171984379*G0_2_1 - 0.0880738185425656*G0_2_2;
    A[1827] = 0.149725491522364*G0_0_0 + 0.0748627457611834*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0748627457611828*G0_1_0 + 0.0195719596761275*G0_1_1 + 0.0606730749959913*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0606730749959914*G0_2_1 + 0.0929668084615985*G0_2_2;
    A[1828] = 0.0234863516113509*G0_0_0 + 0.0459941052388955*G0_0_1 - 0.0812236326559235*G0_0_2 + 0.0459941052388959*G0_1_0 - 0.0880738185425699*G0_1_1 + 0.00913358118219155*G0_1_2 - 0.0812236326559242*G0_2_0 + 0.00913358118219199*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1829] = 0.149725491522362*G0_0_0 + 0.12134614999198*G0_0_1 + 0.0748627457611811*G0_0_2 + 0.12134614999198*G0_1_0 + 0.0929668084615973*G0_1_1 + 0.0606730749959897*G0_1_2 + 0.0748627457611807*G0_2_0 + 0.060673074995989*G0_2_1 + 0.0195719596761233*G0_2_2;
    A[1830] = -0.540186087061083*G0_0_0 - 0.27009304353054*G0_0_1 - 0.27009304353054*G0_0_2 - 0.270093043530541*G0_1_0 + 0.0929668084616015*G0_1_1 - 0.13504652176527*G0_1_2 - 0.270093043530541*G0_2_0 - 0.13504652176527*G0_2_1 + 0.0929668084616011*G0_2_2;
    A[1831] = 0.0234863516113513*G0_0_0 - 0.081223632655923*G0_0_1 + 0.0459941052388973*G0_0_2 - 0.0812236326559242*G0_1_0 - 0.185933616923199*G0_1_1 + 0.00913358118219296*G0_1_2 + 0.0459941052388958*G0_2_0 + 0.00913358118219331*G0_2_1 - 0.0880738185425674*G0_2_2;
    A[1832] = -0.0127217737894816*G0_0_1 - 0.0120693751336102*G0_0_2 - 0.0127217737894813*G0_1_0 - 0.0254435475789632*G0_1_1 - 0.024138750267221*G0_1_2 - 0.0120693751336105*G0_2_0 - 0.0241387502672221*G0_2_1 - 0.022833952955479*G0_2_2;
    A[1833] = -0.018104062700416*G0_0_1 - 0.0102752788299661*G0_0_2 - 0.0181040627004161*G0_1_0 - 0.016636165724707*G0_1_1 + 0.00146789697570965*G0_1_2 - 0.0102752788299661*G0_2_0 + 0.00146789697570983*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[1834] = -0.0127217737894829*G0_0_1 - 0.0120693751336114*G0_0_2 - 0.0127217737894833*G0_1_0 + 0.0137003717732883*G0_1_1 + 0.019898159004062*G0_1_2 - 0.0120693751336118*G0_2_0 + 0.0198981590040609*G0_2_1 + 0.0260959462348354*G0_2_2;
    A[1835] = 0.0283793415303819*G0_0_1 + 0.0141896707651901*G0_0_2 + 0.0283793415303809*G0_1_0 + 0.056758683060764*G0_1_1 + 0.0283793415303805*G0_1_2 + 0.0141896707651904*G0_2_0 + 0.0283793415303824*G0_2_1 - 0.00489298991903351*G0_2_2;
    A[1836] = 0.0283793415303834*G0_0_1 + 0.0141896707651913*G0_0_2 + 0.0283793415303845*G0_1_0 + 0.149725491522366*G0_1_1 + 0.0748627457611823*G0_1_2 + 0.0141896707651921*G0_2_0 + 0.0748627457611838*G0_2_1 + 0.019571959676126*G0_2_2;
    A[1837] = 0.104709984267275*G0_0_1 + 0.036860524056705*G0_0_2 + 0.104709984267276*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0737210481134105*G0_1_2 + 0.0368605240567048*G0_2_0 + 0.0737210481134075*G0_2_1 - 0.0228339529554801*G0_2_2;
    A[1838] = -0.012069375133611*G0_0_1 - 0.0127217737894826*G0_0_2 - 0.0120693751336113*G0_1_0 - 0.0228339529554808*G0_1_1 - 0.0241387502672229*G0_1_2 - 0.0127217737894817*G0_2_0 - 0.0241387502672219*G0_2_1 - 0.0254435475789652*G0_2_2;
    A[1839] = -0.0102752788299664*G0_0_1 - 0.0181040627004164*G0_0_2 - 0.0102752788299658*G0_1_0 + 0.0195719596761254*G0_1_1 + 0.00146789697570932*G0_1_2 - 0.0181040627004164*G0_2_0 + 0.0014678969757095*G0_2_1 - 0.0166361657247072*G0_2_2;
    A[1840] = -0.0120693751336111*G0_0_1 - 0.0127217737894826*G0_0_2 - 0.0120693751336116*G0_1_0 + 0.0260959462348355*G0_1_1 + 0.0198981590040616*G0_1_2 - 0.0127217737894828*G0_2_0 + 0.0198981590040621*G0_2_1 + 0.0137003717732875*G0_2_2;
    A[1841] = 0.0141896707651924*G0_0_1 + 0.0283793415303834*G0_0_2 + 0.0141896707651918*G0_1_0 - 0.00489298991902996*G0_1_1 + 0.0283793415303831*G0_1_2 + 0.0283793415303829*G0_2_0 + 0.028379341530383*G0_2_1 + 0.0567586830607654*G0_2_2;
    A[1842] = 0.0141896707651918*G0_0_1 + 0.0283793415303833*G0_0_2 + 0.0141896707651919*G0_1_0 + 0.0195719596761267*G0_1_1 + 0.0748627457611838*G0_1_2 + 0.0283793415303839*G0_2_0 + 0.0748627457611823*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1843] = 0.036860524056703*G0_0_1 + 0.104709984267274*G0_0_2 + 0.036860524056704*G0_1_0 - 0.0228339529554818*G0_1_1 + 0.0737210481134087*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0737210481134093*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1844] = 0.0929668084615964*G0_0_1 + 0.0929668084615976*G0_0_2 + 0.092966808461598*G0_1_0 + 0.146789697570943*G0_1_1 + 0.141896707651913*G0_1_2 + 0.0929668084615972*G0_2_0 + 0.141896707651912*G0_2_1 + 0.146789697570946*G0_2_2;
    A[1845] = 0.092966808461599*G0_0_1 + 0.0929668084615996*G0_0_2 + 0.0929668084616014*G0_1_0 - 0.0489298991903159*G0_1_1 - 0.0293579395141891*G0_1_2 + 0.0929668084616015*G0_2_0 - 0.0293579395141899*G0_2_1 - 0.048929899190315*G0_2_2;
    A[1846] = -0.10275278829966*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299662*G0_1_0 + 0.14678969757095*G0_1_1 - 0.298472385060924*G0_1_2 - 0.298472385060925*G0_2_0 - 0.298472385060923*G0_2_1 - 0.782878387045049*G0_2_2;
    A[1847] = -0.298472385060923*G0_0_1 - 0.102752788299661*G0_0_2 - 0.298472385060923*G0_1_0 - 0.782878387045047*G0_1_1 - 0.298472385060923*G0_1_2 - 0.10275278829966*G0_2_0 - 0.298472385060924*G0_2_1 + 0.146789697570948*G0_2_2;
    A[1848] = -0.013742125287264*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.013742125287264*G0_2_0 - 0.013742125287264*G0_2_1 - 0.013742125287264*G0_2_2;
    A[1849] = 0.013742125287264*G0_0_0 - 0.00517743573299123*G0_1_0 - 0.00517743573299125*G0_2_0;
    A[1850] = -0.00517743573299142*G0_0_1 + 0.013742125287264*G0_1_1 - 0.00517743573299146*G0_2_1;
    A[1851] = 0.0143945239431349*G0_0_2 + 0.0143945239431349*G0_1_2 + 0.0479930547204849*G0_2_2;
    A[1852] = 0.0477555816097483*G0_0_1 + 0.0118736555368499*G0_0_2 - 0.064130787872107*G0_1_1 - 0.00939454064454091*G0_1_2 + 0.0264873854283582*G0_2_1 + 0.0118736555368501*G0_2_2;
    A[1853] = -0.082332710370905*G0_0_1 + 0.00182671623643819*G0_0_2 + 0.127087258163646*G0_1_1 + 0.0377738821749238*G0_1_2 - 0.0463855444324201*G0_2_1 - 0.0510175748891025*G0_2_2;
    A[1854] = 0.146659217839772*G0_0_1 + 0.0109602974186305*G0_0_2 - 0.124738623002511*G0_1_1 - 0.0623693115012557*G0_1_2 + 0.0733296089198868*G0_2_1 + 0.132306447410613*G0_2_2;
    A[1855] = 0.12473862300251*G0_0_1 + 0.192588083213082*G0_0_2 + 0.12473862300251*G0_1_1 + 0.0962940416065408*G0_1_2 + 0.0284445813959694*G0_2_1 - 0.0644569872000427*G0_2_2;
    A[1856] = -0.0641307878721063*G0_0_0 - 0.00939454064454001*G0_0_2 + 0.0477555816097478*G0_1_0 + 0.0118736555368504*G0_1_2 + 0.0264873854283572*G0_2_0 + 0.0118736555368501*G0_2_2;
    A[1857] = 0.127087258163645*G0_0_0 + 0.0377738821749225*G0_0_2 - 0.0823327103709043*G0_1_0 + 0.00182671623643729*G0_1_2 - 0.0463855444324192*G0_2_0 - 0.0510175748891032*G0_2_2;
    A[1858] = -0.12473862300251*G0_0_0 - 0.0623693115012544*G0_0_2 + 0.146659217839773*G0_1_0 + 0.0109602974186318*G0_1_2 + 0.0733296089198868*G0_2_0 + 0.132306447410615*G0_2_2;
    A[1859] = 0.124738623002511*G0_0_0 + 0.0962940416065404*G0_0_2 + 0.124738623002511*G0_1_0 + 0.192588083213081*G0_1_2 + 0.0284445813959694*G0_2_0 - 0.0644569872000433*G0_2_2;
    A[1860] = -0.0187890812890804*G0_0_0 - 0.0093945406445401*G0_0_1 + 0.0212681961813906*G0_1_0 + 0.0118736555368503*G0_1_1 + 0.0425363923627808*G0_2_0 + 0.01187365553685*G0_2_1;
    A[1861] = 0.0148746893538553*G0_0_0 + 0.0149399292194426*G0_0_1 + 0.00123955744615432*G0_1_0 + 0.00117431758056698*G0_1_1 + 0.0403182369328196*G0_2_0 + 0.0540186087061077*G0_2_1;
    A[1862] = 0.0011743175805675*G0_0_0 + 0.00123955744615446*G0_0_1 + 0.0149399292194429*G0_1_0 + 0.0148746893538559*G0_1_1 + 0.0540186087061079*G0_2_0 + 0.0403182369328195*G0_2_1;
    A[1863] = 0.01187365553685*G0_0_0 + 0.0212681961813905*G0_0_1 - 0.00939454064454082*G0_1_0 - 0.0187890812890815*G0_1_1 + 0.0118736555368501*G0_2_0 + 0.0425363923627812*G0_2_1;
    A[1864] = 0.0641307878721067*G0_0_0 + 0.0641307878721067*G0_0_1 + 0.0547362472275662*G0_0_2 + 0.064130787872107*G0_1_0 + 0.064130787872107*G0_1_1 + 0.0547362472275664*G0_1_2 + 0.0547362472275663*G0_2_0 + 0.0547362472275662*G0_2_1 + 0.0453417065830255*G0_2_2;
    A[1865] = -0.127087258163646*G0_0_0 - 0.127087258163646*G0_0_1 - 0.0893133759887227*G0_0_2 - 0.127087258163647*G0_1_0 - 0.127087258163647*G0_1_1 - 0.089313375988723*G0_1_2 - 0.0893133759887222*G0_2_0 - 0.0893133759887223*G0_2_1 - 0.0665446628988288*G0_2_2;
    A[1866] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012556*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0623693115012557*G0_1_2 + 0.0623693115012551*G0_2_0 + 0.0623693115012551*G0_2_1 + 0.0476251018785731*G0_2_2;
    A[1867] = -0.12473862300251*G0_0_0 - 0.12473862300251*G0_0_1 - 0.0284445813959702*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002511*G0_1_1 - 0.0284445813959701*G0_1_2 - 0.0284445813959693*G0_2_0 - 0.0284445813959693*G0_2_1 - 0.0606730749959909*G0_2_2;
    A[1868] = 0.0187890812890807*G0_0_0 + 0.00939454064454022*G0_0_1 + 0.0187890812890808*G0_0_2 + 0.00939454064454029*G0_1_0 + 0.00939454064454031*G0_1_2 + 0.018789081289081*G0_2_0 + 0.00939454064454031*G0_2_1 + 0.018789081289081*G0_2_2;
    A[1869] = -0.0148746893538552*G0_0_0 + 6.52398655879897e-05*G0_0_1 - 0.0148746893538554*G0_0_2 + 6.52398655878096e-05*G0_1_0 + 6.52398655877579e-05*G0_1_2 - 0.0148746893538551*G0_2_0 + 6.52398655878467e-05*G0_2_1 - 0.0148746893538551*G0_2_2;
    A[1870] = -0.00117431758056744*G0_0_0 + 6.52398655862292e-05*G0_0_1 - 0.00117431758056746*G0_0_2 + 6.52398655867422e-05*G0_1_0 + 6.52398655867903e-05*G0_1_2 - 0.00117431758056732*G0_2_0 + 6.52398655863227e-05*G0_2_1 - 0.00117431758056729*G0_2_2;
    A[1871] = -0.0118736555368498*G0_0_0 + 0.00939454064454114*G0_0_1 - 0.0118736555368499*G0_0_2 + 0.00939454064454089*G0_1_0 + 0.00939454064454091*G0_1_2 - 0.01187365553685*G0_2_0 + 0.00939454064454116*G0_2_1 - 0.0118736555368501*G0_2_2;
    A[1872] = 0.00939454064454029*G0_0_1 + 0.00939454064454028*G0_0_2 + 0.00939454064454042*G0_1_0 + 0.0187890812890811*G0_1_1 + 0.0187890812890811*G0_1_2 + 0.0093945406445403*G0_2_0 + 0.018789081289081*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[1873] = 6.52398655872713e-05*G0_0_1 + 6.52398655872772e-05*G0_0_2 + 6.5239865587569e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538557*G0_1_2 + 6.52398655876919e-05*G0_2_0 - 0.0148746893538555*G0_2_1 - 0.0148746893538556*G0_2_2;
    A[1874] = 6.52398655868328e-05*G0_0_1 + 6.5239865586852e-05*G0_0_2 + 6.52398655865175e-05*G0_1_0 - 0.00117431758056806*G0_1_1 - 0.001174317580568*G0_1_2 + 6.52398655863493e-05*G0_2_0 - 0.00117431758056811*G0_2_1 - 0.00117431758056803*G0_2_2;
    A[1875] = 0.00939454064454007*G0_0_1 + 0.00939454064454003*G0_0_2 + 0.00939454064454073*G0_1_0 - 0.0118736555368502*G0_1_1 - 0.0118736555368503*G0_1_2 + 0.00939454064454095*G0_2_0 - 0.0118736555368501*G0_2_1 - 0.0118736555368501*G0_2_2;
    A[1876] = 0.10568858225108*G0_0_0 + 0.0528442911255409*G0_0_1 + 0.0300103381700607*G0_0_2 + 0.000652398655870347*G0_1_0 + 0.053496689781411*G0_1_1 + 0.0528442911255404*G0_1_2 - 0.0743734467692809*G0_2_0 + 0.000652398655869498*G0_2_1 + 0.10568858225108*G0_2_2;
    A[1877] = -0.0137003717732873*G0_0_0 - 0.0198981590040613*G0_0_1 + 0.0127217737894817*G0_0_2 - 0.0198981590040606*G0_1_0 - 0.0137003717732874*G0_1_1 + 0.0127217737894822*G0_1_2 - 0.0851380245911498*G0_2_0 - 0.0851380245911497*G0_2_1 + 0.113517366121531*G0_2_2;
    A[1878] = 0.0534966897814108*G0_0_0 + 0.000652398655869947*G0_0_1 + 0.05284429112554*G0_0_2 + 0.0528442911255399*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0300103381700589*G0_1_2 + 0.000652398655870584*G0_2_0 - 0.0743734467692811*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1879] = -0.209419968534549*G0_0_0 - 0.104709984267275*G0_0_1 - 0.0737210481134084*G0_0_2 + 0.0812236326559238*G0_1_0 - 0.0234863516113511*G0_1_1 - 0.0459941052388962*G0_1_2 + 0.180714427676233*G0_2_0 + 0.0225077536275462*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1880] = -0.0234863516113514*G0_0_0 + 0.0812236326559241*G0_0_1 - 0.0459941052388957*G0_0_2 - 0.104709984267275*G0_1_0 - 0.209419968534551*G0_1_1 - 0.0737210481134074*G0_1_2 + 0.0225077536275452*G0_2_0 + 0.180714427676232*G0_2_1 - 0.268135847562929*G0_2_2;
    A[1881] = 0.542795681684566*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842282*G0_2_1 + 0.647179466623906*G0_2_2;
    A[1882] = -0.105688582251083*G0_0_0 - 0.0528442911255412*G0_0_1 - 0.0756782440810215*G0_0_2 - 0.0528442911255423*G0_1_0 - 0.0378391220405113*G0_1_2 - 0.0756782440810223*G0_2_0 - 0.0378391220405112*G0_2_1 - 0.0456679059109621*G0_2_2;
    A[1883] = 0.0137003717732888*G0_0_0 - 0.00619778723077155*G0_0_1 + 0.0264221455627701*G0_0_2 - 0.00619778723077246*G0_1_0 - 0.00554538857490224*G0_1_2 + 0.0264221455627714*G0_2_0 - 0.00554538857490082*G0_2_1 + 0.0391439193522521*G0_2_2;
    A[1884] = -0.0534966897814109*G0_0_0 - 0.0528442911255422*G0_0_1 - 0.000652398655870757*G0_0_2 - 0.0528442911255398*G0_1_0 - 0.0378391220405105*G0_1_2 - 0.000652398655870544*G0_2_0 - 0.0378391220405123*G0_2_1 + 0.0521918924696699*G0_2_2;
    A[1885] = 0.209419968534548*G0_0_0 + 0.104709984267274*G0_0_1 + 0.135698920421142*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105716*G0_1_2 + 0.135698920421141*G0_2_0 + 0.0678494602105702*G0_2_1 + 0.0391439193522534*G0_2_2;
    A[1886] = 0.0234863516113518*G0_0_0 + 0.104709984267277*G0_0_1 - 0.0225077536275449*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105703*G0_1_2 - 0.0225077536275452*G0_2_0 + 0.0678494602105721*G0_2_1 - 0.15657567740901*G0_2_2;
    A[1887] = -0.542795681684566*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842283*G0_1_0 - 0.135698920421141*G0_1_2 - 0.271397840842282*G0_2_0 - 0.135698920421142*G0_2_1 + 0.0521918924696702*G0_2_2;
    A[1888] = -0.0528442911255423*G0_0_1 - 0.037839122040511*G0_0_2 - 0.0528442911255419*G0_1_0 - 0.105688582251083*G0_1_1 - 0.0756782440810225*G0_1_2 - 0.0378391220405112*G0_2_0 - 0.075678244081022*G0_2_1 - 0.0456679059109619*G0_2_2;
    A[1889] = -0.00619778723077247*G0_0_1 - 0.0055453885749023*G0_0_2 - 0.00619778723077252*G0_1_0 + 0.0137003717732885*G0_1_1 + 0.0264221455627707*G0_1_2 - 0.00554538857490158*G0_2_0 + 0.0264221455627704*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[1890] = -0.052844291125541*G0_0_1 - 0.0378391220405094*G0_0_2 - 0.0528442911255407*G0_1_0 - 0.0534966897814111*G0_1_1 - 0.000652398655869466*G0_1_2 - 0.0378391220405109*G0_2_0 - 0.00065239865586956*G0_2_1 + 0.0521918924696712*G0_2_2;
    A[1891] = 0.104709984267274*G0_0_1 + 0.0678494602105717*G0_0_2 + 0.104709984267277*G0_1_0 + 0.20941996853455*G0_1_1 + 0.135698920421144*G0_1_2 + 0.0678494602105711*G0_2_0 + 0.135698920421141*G0_2_1 + 0.0391439193522533*G0_2_2;
    A[1892] = 0.104709984267275*G0_0_1 + 0.0678494602105691*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0234863516113511*G0_1_1 - 0.0225077536275469*G0_1_2 + 0.0678494602105706*G0_2_0 - 0.0225077536275459*G0_2_1 - 0.156575677409012*G0_2_2;
    A[1893] = -0.271397840842283*G0_0_1 - 0.135698920421141*G0_0_2 - 0.271397840842284*G0_1_0 - 0.542795681684566*G0_1_1 - 0.271397840842284*G0_1_2 - 0.135698920421142*G0_2_0 - 0.271397840842282*G0_2_1 + 0.0521918924696707*G0_2_2;
    A[1894] = -0.0150051690850301*G0_0_1 - 0.0300103381700614*G0_0_2 - 0.0150051690850301*G0_1_0 - 0.0300103381700611*G0_1_2 - 0.0300103381700603*G0_2_0 - 0.0300103381700603*G0_2_1 - 0.0600206763401204*G0_2_2;
    A[1895] = -0.000652398655871742*G0_0_1 - 0.00130479731174192*G0_0_2 - 0.000652398655870919*G0_1_0 - 0.0127217737894831*G0_1_2 - 0.00130479731174181*G0_2_0 - 0.012721773789483*G0_2_1 - 0.0254435475789657*G0_2_2;
    A[1896] = -0.0150051690850294*G0_0_1 - 0.030010338170061*G0_0_2 - 0.01500516908503*G0_1_0 - 0.0528442911255407*G0_1_2 - 0.0300103381700597*G0_2_0 - 0.0528442911255394*G0_2_1 - 0.10568858225108*G0_2_2;
    A[1897] = -0.00065239865586961*G0_0_1 - 0.0127217737894814*G0_0_2 - 0.000652398655871563*G0_1_0 - 0.00130479731174132*G0_1_2 - 0.0127217737894828*G0_2_0 - 0.0013047973117405*G0_2_1 - 0.0254435475789633*G0_2_2;
    A[1898] = -0.000652398655870146*G0_0_1 - 0.0127217737894813*G0_0_2 - 0.000652398655870335*G0_1_0 - 0.0127217737894817*G0_1_2 - 0.0127217737894813*G0_2_0 - 0.0127217737894811*G0_2_1 - 0.11351736612153*G0_2_2;
    A[1899] = -0.0150051690850311*G0_0_1 - 0.0528442911255402*G0_0_2 - 0.0150051690850297*G0_1_0 - 0.0300103381700593*G0_1_2 - 0.0528442911255406*G0_2_0 - 0.0300103381700615*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1900] = 0.0260959462348357*G0_0_1 + 0.0750258454251499*G0_0_2 + 0.0260959462348363*G0_1_0 + 0.0750258454251521*G0_1_2 + 0.0750258454251515*G0_2_0 + 0.0750258454251518*G0_2_1 + 0.195719596761263*G0_2_2;
    A[1901] = 0.0260959462348336*G0_0_1 + 0.0750258454251509*G0_0_2 + 0.0260959462348355*G0_1_0 + 0.0587158790283797*G0_1_2 + 0.0750258454251514*G0_2_0 + 0.0587158790283787*G0_2_1 + 0.293579395141895*G0_2_2;
    A[1902] = 0.0260959462348369*G0_0_1 + 0.0587158790283767*G0_0_2 + 0.0260959462348332*G0_1_0 + 0.0750258454251487*G0_1_2 + 0.0587158790283784*G0_2_0 + 0.0750258454251519*G0_2_1 + 0.293579395141892*G0_2_2;
    A[1903] = -0.185933616923198*G0_0_1 - 0.316413348097371*G0_0_2 - 0.185933616923199*G0_1_0 - 0.316413348097374*G0_1_2 - 0.316413348097373*G0_2_0 - 0.316413348097374*G0_2_1 - 0.782878387045048*G0_2_2;
    A[1904] = 0.0479930547204848*G0_0_0 + 0.0479930547204848*G0_0_1 + 0.0479930547204848*G0_0_2 + 0.0335985307773499*G0_1_0 + 0.0335985307773498*G0_1_1 + 0.0335985307773499*G0_1_2 + 0.0335985307773499*G0_2_0 + 0.0335985307773499*G0_2_1 + 0.0335985307773499*G0_2_2;
    A[1905] = -0.0137421252872639*G0_0_0;
    A[1906] = 0.00517743573299131*G0_0_1 + 0.0189195610202553*G0_1_1;
    A[1907] = 0.00517743573299118*G0_0_2 + 0.0189195610202551*G0_2_2;
    A[1908] = -0.042536392362781*G0_0_1 - 0.01187365553685*G0_0_2 - 0.0613254736518619*G0_1_1 - 0.0212681961813906*G0_1_2 - 0.0212681961813905*G0_2_1;
    A[1909] = -0.0403182369328198*G0_0_1 - 0.0540186087061079*G0_0_2 - 0.0254435475789642*G0_1_1 - 0.0390786794866648*G0_1_2 - 0.0390786794866651*G0_2_1 - 0.0528442911255396*G0_2_2;
    A[1910] = -0.0540186087061083*G0_0_1 - 0.0403182369328201*G0_0_2 - 0.0528442911255406*G0_1_1 - 0.0390786794866657*G0_1_2 - 0.0390786794866648*G0_2_1 - 0.0254435475789651*G0_2_2;
    A[1911] = -0.0118736555368506*G0_0_1 - 0.0425363923627815*G0_0_2 - 0.0212681961813907*G0_1_2 - 0.0212681961813909*G0_2_1 - 0.0613254736518619*G0_2_2;
    A[1912] = 0.0187890812890807*G0_0_0 + 0.00939454064454041*G0_0_2 + 0.00939454064454066*G0_2_0;
    A[1913] = -0.014874689353856*G0_0_0 - 0.0149399292194433*G0_0_2 - 0.014939929219443*G0_2_0 - 0.0150051690850308*G0_2_2;
    A[1914] = -0.00117431758056711*G0_0_0 - 0.00123955744615417*G0_0_2 - 0.0012395574461542*G0_2_0 - 0.00130479731174049*G0_2_2;
    A[1915] = -0.0118736555368506*G0_0_0 - 0.0212681961813912*G0_0_2 - 0.021268196181391*G0_2_0 - 0.0306627368259315*G0_2_2;
    A[1916] = 0.0187890812890806*G0_0_0 + 0.00939454064454062*G0_0_1 + 0.00939454064454058*G0_1_0;
    A[1917] = -0.0148746893538554*G0_0_0 - 0.0149399292194432*G0_0_1 - 0.014939929219443*G0_1_0 - 0.015005169085031*G0_1_1;
    A[1918] = -0.00117431758056818*G0_0_0 - 0.00123955744615448*G0_0_1 - 0.00123955744615477*G0_1_0 - 0.00130479731174111*G0_1_1;
    A[1919] = -0.01187365553685*G0_0_0 - 0.0212681961813911*G0_0_1 - 0.0212681961813906*G0_1_0 - 0.0306627368259316*G0_1_1;
    A[1920] = -0.0644569872000421*G0_0_0 - 0.0644569872000421*G0_0_1 - 0.092901568596012*G0_0_2 - 0.257045070413124*G0_1_0 - 0.257045070413124*G0_1_1 - 0.160751028806583*G0_1_2 - 0.160751028806583*G0_2_0 - 0.160751028806583*G0_2_1 - 0.064456987200042*G0_2_2;
    A[1921] = 0.132306447410614*G0_0_0 + 0.132306447410614*G0_0_1 + 0.0589768384907269*G0_0_2 + 0.121346149991983*G0_1_0 + 0.121346149991983*G0_1_1 + 0.194675758911869*G0_1_2 + 0.194675758911869*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052866*G0_2_2;
    A[1922] = -0.0510175748891024*G0_0_0 - 0.0510175748891024*G0_0_1 - 0.00463203045668275*G0_0_2 - 0.0528442911255408*G0_1_0 - 0.0528442911255409*G0_1_1 - 0.0887914570640261*G0_1_2 - 0.0887914570640256*G0_2_0 - 0.0887914570640256*G0_2_1 + 0.0846813455320394*G0_2_2;
    A[1923] = 0.0118736555368505*G0_0_0 + 0.0118736555368505*G0_0_1 - 0.014613729891508*G0_0_2 + 0.0212681961813904*G0_1_2 + 0.0212681961813909*G0_2_0 + 0.0212681961813909*G0_2_1 - 0.0693499771190741*G0_2_2;
    A[1924] = -0.0644569872000419*G0_0_0 - 0.092901568596012*G0_0_1 - 0.064456987200042*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000419*G0_1_1 - 0.160751028806583*G0_1_2 - 0.257045070413124*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413124*G0_2_2;
    A[1925] = 0.132306447410612*G0_0_0 + 0.0589768384907262*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911867*G0_1_0 - 0.00339247301052879*G0_1_1 + 0.194675758911868*G0_1_2 + 0.121346149991982*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1926] = -0.0510175748891024*G0_0_0 - 0.00463203045668275*G0_0_1 - 0.0510175748891024*G0_0_2 - 0.0887914570640259*G0_1_0 + 0.0846813455320391*G0_1_1 - 0.0887914570640259*G0_1_2 - 0.0528442911255408*G0_2_0 - 0.0887914570640254*G0_2_1 - 0.0528442911255408*G0_2_2;
    A[1927] = 0.0118736555368499*G0_0_0 - 0.0146137298915078*G0_0_1 + 0.0118736555368499*G0_0_2 + 0.0212681961813905*G0_1_0 - 0.0693499771190735*G0_1_1 + 0.0212681961813905*G0_1_2 + 0.0212681961813902*G0_2_1;
    A[1928] = -0.0606730749959909*G0_0_0 - 0.032228493600021*G0_0_1 - 0.032228493600021*G0_0_2 - 0.0322284936000211*G0_1_0 - 0.128522535206562*G0_1_1 - 0.128522535206562*G0_1_2 - 0.0322284936000212*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1929] = 0.0476251018785732*G0_0_0 - 0.0147442096226815*G0_0_1 - 0.0147442096226815*G0_0_2 - 0.0147442096226818*G0_1_0 + 0.0476251018785736*G0_1_1 + 0.0476251018785735*G0_1_2 - 0.0147442096226816*G0_2_0 + 0.0476251018785733*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[1930] = -0.0665446628988284*G0_0_0 + 0.0227687130898934*G0_0_1 + 0.0227687130898934*G0_0_2 + 0.0227687130898934*G0_1_0 - 0.0150051690850301*G0_1_1 - 0.01500516908503*G0_1_2 + 0.0227687130898934*G0_2_0 - 0.0150051690850299*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1931] = 0.0453417065830252*G0_0_0 - 0.00939454064454055*G0_0_1 - 0.0093945406445406*G0_0_2 - 0.00939454064454067*G0_1_0 - 0.00939454064454064*G0_2_0;
    A[1932] = -0.0600206763401199*G0_0_0 - 0.0300103381700609*G0_0_1 - 0.0300103381700605*G0_0_2 - 0.0300103381700604*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0300103381700598*G0_2_0 - 0.0150051690850297*G0_2_1;
    A[1933] = -0.0254435475789651*G0_0_0 - 0.0241387502672234*G0_0_1 - 0.0127217737894826*G0_0_2 - 0.0241387502672229*G0_1_0 - 0.0228339529554823*G0_1_1 - 0.0120693751336123*G0_1_2 - 0.0127217737894821*G0_2_0 - 0.0120693751336124*G0_2_1;
    A[1934] = -0.105688582251081*G0_0_0 - 0.0756782440810211*G0_0_1 - 0.0528442911255417*G0_0_2 - 0.0756782440810209*G0_1_0 - 0.0456679059109602*G0_1_1 - 0.0378391220405106*G0_1_2 - 0.0528442911255399*G0_2_0 - 0.037839122040509*G0_2_1;
    A[1935] = -0.0254435475789644*G0_0_0 - 0.0127217737894816*G0_0_1 - 0.0241387502672226*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.0120693751336115*G0_1_2 - 0.0241387502672229*G0_2_0 - 0.012069375133611*G0_2_1 - 0.0228339529554809*G0_2_2;
    A[1936] = -0.113517366121531*G0_0_0 - 0.10079559233205*G0_0_1 - 0.100795592332049*G0_0_2 - 0.10079559233205*G0_1_0 - 0.088073818542568*G0_1_1 - 0.0887262171984379*G0_1_2 - 0.100795592332049*G0_2_0 - 0.0887262171984381*G0_2_1 - 0.0880738185425656*G0_2_2;
    A[1937] = -0.105688582251083*G0_0_0 - 0.0528442911255423*G0_0_1 - 0.0756782440810223*G0_0_2 - 0.0528442911255412*G0_1_0 - 0.0378391220405111*G0_1_2 - 0.0756782440810215*G0_2_0 - 0.0378391220405113*G0_2_1 - 0.0456679059109622*G0_2_2;
    A[1938] = 0.647179466623907*G0_0_0 + 0.375781625781622*G0_0_1 + 0.375781625781623*G0_0_2 + 0.375781625781622*G0_1_0 + 0.647179466623905*G0_1_1 + 0.375781625781622*G0_1_2 + 0.375781625781623*G0_2_0 + 0.375781625781622*G0_2_1 + 0.647179466623906*G0_2_2;
    A[1939] = -0.268135847562929*G0_0_0 - 0.44885027523916*G0_0_1 - 0.290643601190472*G0_0_2 - 0.194414799449519*G0_1_0 - 0.5845491956603*G0_1_1 - 0.321632537344338*G0_1_2 - 0.222141742324033*G0_2_0 - 0.321632537344341*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1940] = 0.105688582251082*G0_0_0 + 0.180062029020361*G0_0_1 + 0.10503618359521*G0_0_2 + 0.075678244081021*G0_1_0 + 0.255740273101381*G0_1_1 + 0.127870136550691*G0_1_2 + 0.0528442911255401*G0_2_0 + 0.127870136550692*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1941] = -0.268135847562927*G0_0_0 - 0.290643601190473*G0_0_1 - 0.448850275239161*G0_0_2 - 0.222141742324031*G0_1_0 - 0.268135847562929*G0_1_1 - 0.321632537344341*G0_1_2 - 0.194414799449518*G0_2_0 - 0.321632537344339*G0_2_1 - 0.584549195660303*G0_2_2;
    A[1942] = 0.113517366121531*G0_0_0 + 0.19865539071268*G0_0_1 + 0.19865539071268*G0_0_2 + 0.10079559233205*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919136*G0_1_2 + 0.100795592332049*G0_2_0 + 0.166035457919136*G0_2_1 + 0.172233245149911*G0_2_2;
    A[1943] = 0.105688582251083*G0_0_0 + 0.10503618359521*G0_0_1 + 0.180062029020362*G0_0_2 + 0.0528442911255412*G0_1_0 + 0.105688582251081*G0_1_1 + 0.127870136550692*G0_1_2 + 0.0756782440810216*G0_2_0 + 0.12787013655069*G0_2_1 + 0.255740273101382*G0_2_2;
    A[1944] = 0.0521918924696705*G0_0_0 + 0.323589733311955*G0_0_1 + 0.187890812890812*G0_0_2 + 0.323589733311952*G0_1_0 + 0.05219189246967*G0_1_1 + 0.187890812890811*G0_1_2 + 0.187890812890811*G0_2_0 + 0.187890812890812*G0_2_1 + 0.323589733311953*G0_2_2;
    A[1945] = 0.0391439193522511*G0_0_0 - 0.0965550010688895*G0_0_1 - 0.0287055408583187*G0_0_2 - 0.0965550010688891*G0_1_0 - 0.0228339529554814*G0_1_1 - 0.0596944770121851*G0_1_2 - 0.0287055408583184*G0_2_0 - 0.0596944770121856*G0_2_1 - 0.0965550010688892*G0_2_2;
    A[1946] = -0.0456679059109602*G0_0_0 + 0.0300103381700614*G0_0_1 - 0.00782878387044975*G0_0_2 + 0.0300103381700602*G0_1_0 + 0.0150051690850312*G0_1_2 - 0.00782878387045057*G0_2_0 + 0.0150051690850302*G0_2_1 + 0.0300103381700613*G0_2_2;
    A[1947] = -0.15657567740901*G0_0_0 - 0.134067923781464*G0_0_1 - 0.224425137619582*G0_0_2 - 0.134067923781465*G0_1_0 - 0.0880738185425673*G0_1_1 - 0.0972073997247605*G0_1_2 - 0.22442513761958*G0_2_0 - 0.0972073997247589*G0_2_1 - 0.292274597830151*G0_2_2;
    A[1948] = 0.0391439193522523*G0_0_0 + 0.0127217737894811*G0_0_1 + 0.0446893079271543*G0_0_2 + 0.0127217737894825*G0_1_0 + 0.0120693751336105*G0_1_2 + 0.0446893079271548*G0_2_0 + 0.0120693751336104*G0_2_1 + 0.0502346965020565*G0_2_2;
    A[1949] = 0.0521918924696698*G0_0_0 + 0.0528442911255427*G0_0_1 + 0.0900310145101818*G0_0_2 + 0.0528442911255403*G0_1_0 + 0.0378391220405114*G0_1_2 + 0.09003101451018*G0_2_0 + 0.0378391220405115*G0_2_1 + 0.127870136550692*G0_2_2;
    A[1950] = 0.0521918924696704*G0_0_0 + 0.187890812890811*G0_0_1 + 0.323589733311954*G0_0_2 + 0.18789081289081*G0_1_0 + 0.323589733311952*G0_1_1 + 0.18789081289081*G0_1_2 + 0.323589733311953*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696695*G0_2_2;
    A[1951] = 0.0391439193522515*G0_0_0 - 0.0287055408583178*G0_0_1 - 0.0965550010688886*G0_0_2 - 0.028705540858318*G0_1_0 - 0.0965550010688887*G0_1_1 - 0.0596944770121848*G0_1_2 - 0.0965550010688888*G0_2_0 - 0.0596944770121841*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[1952] = -0.0456679059109602*G0_0_0 - 0.00782878387045039*G0_0_1 + 0.0300103381700609*G0_0_2 - 0.00782878387045041*G0_1_0 + 0.0300103381700606*G0_1_1 + 0.0150051690850301*G0_1_2 + 0.0300103381700595*G0_2_0 + 0.0150051690850305*G0_2_1;
    A[1953] = -0.156575677409008*G0_0_0 - 0.22442513761958*G0_0_1 - 0.134067923781465*G0_0_2 - 0.224425137619578*G0_1_0 - 0.292274597830149*G0_1_1 - 0.0972073997247598*G0_1_2 - 0.134067923781463*G0_2_0 - 0.0972073997247601*G0_2_1 - 0.0880738185425687*G0_2_2;
    A[1954] = 0.0391439193522513*G0_0_0 + 0.044689307927154*G0_0_1 + 0.0127217737894825*G0_0_2 + 0.0446893079271541*G0_1_0 + 0.0502346965020566*G0_1_1 + 0.0120693751336119*G0_1_2 + 0.0127217737894822*G0_2_0 + 0.0120693751336101*G0_2_1;
    A[1955] = 0.0521918924696706*G0_0_0 + 0.0900310145101814*G0_0_1 + 0.0528442911255418*G0_0_2 + 0.0900310145101807*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0378391220405108*G0_1_2 + 0.0528442911255408*G0_2_0 + 0.0378391220405119*G0_2_1;
    A[1956] = -0.782878387045047*G0_0_0 - 0.466465038947674*G0_0_1 - 0.466465038947672*G0_0_2 - 0.466465038947674*G0_1_0 - 0.150051690850301*G0_1_1 - 0.335985307773497*G0_1_2 - 0.466465038947675*G0_2_0 - 0.3359853077735*G0_2_1 - 0.150051690850299*G0_2_2;
    A[1957] = 0.19571959676126*G0_0_0 + 0.120693751336113*G0_0_1 + 0.120693751336111*G0_0_2 + 0.120693751336112*G0_1_0 + 0.0456679059109638*G0_1_1 + 0.0717638521457964*G0_1_2 + 0.120693751336112*G0_2_0 + 0.0717638521457977*G0_2_1 + 0.0456679059109611*G0_2_2;
    A[1958] = 0.293579395141894*G0_0_0 + 0.21855354971674*G0_0_1 + 0.234863516113514*G0_0_2 + 0.218553549716742*G0_1_0 + 0.143527704291591*G0_1_1 + 0.185933616923198*G0_1_2 + 0.234863516113515*G0_2_0 + 0.185933616923197*G0_2_1 + 0.176147637085134*G0_2_2;
    A[1959] = 0.293579395141891*G0_0_0 + 0.234863516113514*G0_0_1 + 0.218553549716741*G0_0_2 + 0.234863516113513*G0_1_0 + 0.176147637085135*G0_1_1 + 0.185933616923198*G0_1_2 + 0.218553549716741*G0_2_0 + 0.185933616923197*G0_2_1 + 0.143527704291592*G0_2_2;
    A[1960] = -0.0345719095719093*G0_0_0 - 0.0345719095719093*G0_0_1 - 0.0345719095719093*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060922*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0298472385060923*G0_2_0 - 0.0298472385060923*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[1961] = -0.00946499969937464*G0_0_0;
    A[1962] = 0.00472467106581688*G0_0_1 - 0.0298472385060924*G0_1_1;
    A[1963] = 0.00227817610630121*G0_0_2 + 0.0117431758056759*G0_2_2;
    A[1964] = 0.0845508658008655*G0_0_1 + 0.0429604514890976*G0_0_2 + 0.128522535206562*G0_1_1 + 0.0422754329004329*G0_1_2 + 0.0422754329004327*G0_2_1;
    A[1965] = 0.080832193462401*G0_0_1 + 0.108232937008977*G0_0_2 + 0.0371867233846402*G0_1_1 + 0.0743408268364858*G0_1_2 + 0.0743408268364854*G0_2_1 + 0.104709984267274*G0_2_2;
    A[1966] = 0.00939454064454054*G0_0_1 + 0.0377738821749244*G0_0_2 - 0.00619778723077307*G0_1_1 + 0.0188869410874625*G0_1_2 + 0.0188869410874612*G0_2_1 + 0.0567586830607662*G0_2_2;
    A[1967] = -0.00254435475789622*G0_0_1 - 0.0142875305635724*G0_0_2 - 0.00583896797004379*G0_1_2 - 0.0058389679700438*G0_2_1 - 0.0435476102793808*G0_2_2;
    A[1968] = 0.00658922642429615*G0_0_0 + 0.00329461321214822*G0_0_2 + 0.00329461321214818*G0_2_0;
    A[1969] = 0.0045015507255088*G0_0_0 + 0.0019245760348188*G0_0_2 + 0.00192457603481889*G0_2_0 - 0.000652398655870704*G0_2_2;
    A[1970] = -0.0105036183595208*G0_0_0 - 0.0130805930502108*G0_0_2 - 0.0130805930502114*G0_2_0 - 0.0156575677409015*G0_2_2;
    A[1971] = -0.00254435475789629*G0_0_0 - 0.00583896797004422*G0_0_2 - 0.00583896797004384*G0_2_0 - 0.00913358118219194*G0_2_2;
    A[1972] = 0.0292600797158091*G0_0_0 + 0.00329461321214821*G0_0_1 + 0.00329461321214802*G0_1_0;
    A[1973] = -0.0189848008858428*G0_0_0 - 0.00949240044292121*G0_0_1 - 0.0094924004429211*G0_1_0 - 0.000652398655870053*G0_1_1;
    A[1974] = 0.00352295274170252*G0_0_0 + 0.00649136662591454*G0_0_1 + 0.00649136662591522*G0_1_0 + 0.00880738185425594*G0_1_1;
    A[1975] = 0.0429604514890978*G0_0_0 + 0.0422754329004337*G0_0_1 + 0.042275432900433*G0_1_0 + 0.0642612676032819*G0_1_1;
    A[1976] = 0.0439716694056967*G0_0_0 + 0.0439716694056966*G0_0_1 + 0.0446566879943612*G0_0_2 + 0.128522535206562*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0862471023061292*G0_1_2 + 0.0862471023061289*G0_2_0 + 0.0862471023061289*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[1977] = -0.0436454700777619*G0_0_0 - 0.0436454700777619*G0_0_1 - 0.00975335990526929*G0_0_2 + 0.0371867233846383*G0_1_0 + 0.0371867233846383*G0_1_1 - 0.0371541034518464*G0_1_2 - 0.0371541034518465*G0_2_0 - 0.0371541034518465*G0_2_1 - 0.00678494602105741*G0_2_2;
    A[1978] = -0.0155923278753139*G0_0_0 - 0.0155923278753139*G0_0_1 + 0.0032946132121473*G0_0_2 - 0.00619778723077272*G0_1_0 - 0.00619778723077276*G0_1_1 - 0.0250847283182352*G0_1_2 - 0.025084728318235*G0_2_0 - 0.025084728318235*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[1979] = 0.00254435475789621*G0_0_0 + 0.00254435475789616*G0_0_1 - 0.00590420783563106*G0_0_2 + 0.00583896797004451*G0_1_2 + 0.00583896797004378*G0_2_0 + 0.00583896797004375*G0_2_1 - 0.0318696743392921*G0_2_2;
    A[1980] = 0.127315597693201*G0_0_0 + 0.128000616281866*G0_0_1 + 0.127315597693201*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761736*G0_1_1 + 0.134361503176606*G0_1_2 + 0.176636936077038*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1981] = -0.166948816037356*G0_0_0 - 0.160783648739377*G0_0_1 - 0.166948816037356*G0_0_2 - 0.250814663249556*G0_1_0 - 0.062238831770081*G0_1_1 - 0.250814663249556*G0_1_2 - 0.334680510461756*G0_2_0 - 0.250814663249556*G0_2_1 - 0.334680510461756*G0_2_2;
    A[1982] = 0.182410664181496*G0_0_0 + 0.0985448169692953*G0_0_1 + 0.182410664181496*G0_0_2 + 0.188575831479476*G0_1_0 - 0.0622388317700809*G0_1_1 + 0.188575831479476*G0_1_2 + 0.104709984267275*G0_2_0 + 0.188575831479475*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1983] = -0.0429604514890975*G0_0_0 - 0.0359145460056914*G0_0_1 - 0.0429604514890976*G0_0_2 - 0.0422754329004329*G0_1_0 + 0.0920860702761734*G0_1_1 - 0.0422754329004328*G0_1_2 - 0.0422754329004324*G0_2_1;
    A[1984] = 0.0226708532915131*G0_0_0 + 0.0219858347028486*G0_0_1 + 0.0219858347028486*G0_0_2 + 0.0219858347028484*G0_1_0 + 0.064261267603281*G0_1_1 + 0.064261267603281*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032806*G0_2_1 + 0.0642612676032807*G0_2_2;
    A[1985] = -0.000652398655870845*G0_0_0 + 0.00231601522834186*G0_0_1 + 0.00231601522834183*G0_0_2 + 0.00231601522834165*G0_1_0 + 0.00880738185425668*G0_1_1 + 0.00880738185425676*G0_1_2 + 0.00231601522834137*G0_2_0 + 0.00880738185425674*G0_2_1 + 0.00880738185425686*G0_2_2;
    A[1986] = -0.000652398655871142*G0_0_0 + 0.00884000178705046*G0_0_1 + 0.00884000178705043*G0_0_2 + 0.00884000178705048*G0_1_0 - 0.000652398655870518*G0_1_1 - 0.000652398655870568*G0_1_2 + 0.0088400017870505*G0_2_0 - 0.000652398655870505*G0_2_1 - 0.000652398655870602*G0_2_2;
    A[1987] = 0.0226708532915129*G0_0_0 - 0.00329461321214818*G0_0_1 - 0.00329461321214815*G0_0_2 - 0.00329461321214808*G0_1_0 - 0.00329461321214787*G0_2_0;
    A[1988] = -0.0254435475789634*G0_0_0 - 0.00130479731174094*G0_0_1 - 0.0127217737894811*G0_0_2 - 0.00130479731174025*G0_1_0 - 0.0006523986558698*G0_1_2 - 0.012721773789483*G0_2_0 - 0.0006523986558714*G0_2_1;
    A[1989] = 0.0567586830607659*G0_0_0 + 0.0283793415303843*G0_0_1 + 0.0283793415303828*G0_0_2 + 0.0283793415303831*G0_1_0 - 0.00489298991902976*G0_1_1 + 0.0141896707651924*G0_1_2 + 0.0283793415303846*G0_2_0 + 0.0141896707651939*G0_2_1;
    A[1990] = 0.209419968534551*G0_0_0 + 0.13569892042114*G0_0_1 + 0.104709984267276*G0_0_2 + 0.135698920421143*G0_1_0 + 0.0391439193522525*G0_1_1 + 0.0678494602105717*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0678494602105696*G0_2_1;
    A[1991] = -0.0166361657247059*G0_0_0 - 0.0181040627004164*G0_0_1 + 0.00146789697571032*G0_0_2 - 0.0181040627004166*G0_1_0 - 0.0102752788299659*G0_1_2 + 0.00146789697571138*G0_2_0 - 0.0102752788299661*G0_2_1 + 0.0195719596761279*G0_2_2;
    A[1992] = 0.149725491522364*G0_0_0 + 0.0748627457611828*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0748627457611834*G0_1_0 + 0.0195719596761275*G0_1_1 + 0.0606730749959914*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0606730749959913*G0_2_1 + 0.0929668084615985*G0_2_2;
    A[1993] = 0.0137003717732888*G0_0_0 - 0.00619778723077246*G0_0_1 + 0.0264221455627714*G0_0_2 - 0.00619778723077155*G0_1_0 - 0.00554538857490082*G0_1_2 + 0.0264221455627701*G0_2_0 - 0.00554538857490224*G0_2_1 + 0.0391439193522521*G0_2_2;
    A[1994] = -0.268135847562929*G0_0_0 - 0.194414799449519*G0_0_1 - 0.222141742324033*G0_0_2 - 0.44885027523916*G0_1_0 - 0.5845491956603*G0_1_1 - 0.321632537344341*G0_1_2 - 0.290643601190472*G0_2_0 - 0.321632537344338*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1995] = 0.726119703984281*G0_0_0 + 0.45602666045374*G0_0_1 + 0.456026660453739*G0_0_2 + 0.45602666045374*G0_1_0 + 0.912053320907477*G0_1_1 + 0.456026660453738*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453738*G0_2_1 + 0.726119703984277*G0_2_2;
    A[1996] = -0.209419968534551*G0_0_0 - 0.390134396210783*G0_0_1 - 0.290643601190474*G0_0_2 - 0.135698920421143*G0_1_0 - 0.584549195660301*G0_1_1 - 0.262916658315962*G0_1_2 - 0.104709984267275*G0_2_0 - 0.262916658315961*G0_2_1 - 0.209419968534549*G0_2_2;
    A[1997] = 0.0655660649150221*G0_0_0 + 0.140428810676205*G0_0_1 + 0.0376760223765437*G0_0_2 + 0.0376760223765423*G0_1_0 - 0.0371867233846387*G0_1_1 + 0.186912214907007*G0_1_2 + 0.140428810676204*G0_2_0 + 0.186912214907004*G0_2_1 - 0.0371867233846396*G0_2_2;
    A[1998] = -0.149725491522365*G0_0_0 - 0.177615534060845*G0_0_1 - 0.326851726591308*G0_0_2 - 0.0748627457611836*G0_1_0 - 0.0371867233846407*G0_1_1 - 0.224098938291645*G0_1_2 - 0.121346149991981*G0_2_0 - 0.224098938291644*G0_2_1 - 0.448197876583288*G0_2_2;
    A[1999] = -0.0137003717732892*G0_0_0 + 0.00619778723077355*G0_0_1 + 0.0714376528178602*G0_0_2 + 0.00619778723077143*G0_1_0 + 0.0123955744615462*G0_1_1 + 0.00619778723077173*G0_1_2 - 0.0264221455627702*G0_2_0 + 0.00619778723077413*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2000] = -0.0228339529554803*G0_0_0 - 0.0965550010688897*G0_0_1 - 0.0596944770121856*G0_0_2 - 0.0965550010688885*G0_1_0 + 0.0391439193522517*G0_1_1 - 0.028705540858319*G0_1_2 - 0.0596944770121841*G0_2_0 - 0.028705540858318*G0_2_1 - 0.0965550010688885*G0_2_2;
    A[2001] = -0.00489298991903175*G0_0_0 - 0.0332723314494145*G0_0_1 - 0.0190826606842235*G0_0_2 - 0.0332723314494148*G0_1_0 - 0.00489298991903104*G0_1_1 - 0.0190826606842231*G0_1_2 - 0.0190826606842232*G0_2_0 - 0.0190826606842219*G0_2_1 - 0.0332723314494141*G0_2_2;
    A[2002] = -0.0228339529554808*G0_0_0 + 0.001304797311741*G0_0_1 - 0.0107645778218693*G0_0_2 + 0.00130479731174174*G0_1_0 + 0.000652398655869958*G0_1_2 - 0.0107645778218699*G0_2_0 + 0.00065239865587089*G0_2_1 + 0.00130479731174125*G0_2_2;
    A[2003] = 0.0195719596761264*G0_0_0 - 0.0552907860850563*G0_0_1 + 0.00538228891093574*G0_0_2 - 0.0552907860850559*G0_1_0 + 0.0195719596761253*G0_1_1 - 0.0411011153198641*G0_1_2 + 0.00538228891093483*G0_2_0 - 0.0411011153198655*G0_2_1 - 0.00880738185425726*G0_2_2;
    A[2004] = 0.0195719596761262*G0_0_0 + 0.0181040627004167*G0_0_1 + 0.0298472385060926*G0_0_2 + 0.0181040627004168*G0_1_0 + 0.0102752788299672*G0_1_2 + 0.0298472385060925*G0_2_0 + 0.0102752788299664*G0_2_1 + 0.040122517336059*G0_2_2;
    A[2005] = 0.0260959462348347*G0_0_0 + 0.0061977872307723*G0_0_1 + 0.0381653213684454*G0_0_2 + 0.00619778723077374*G0_1_0 + 0.00554538857490165*G0_1_2 + 0.0381653213684465*G0_2_0 + 0.00554538857490212*G0_2_1 + 0.050234696502057*G0_2_2;
    A[2006] = -0.0880738185425682*G0_0_0 - 0.0972073997247603*G0_0_1 - 0.134067923781464*G0_0_2 - 0.0972073997247596*G0_1_0 - 0.29227459783015*G0_1_1 - 0.224425137619579*G0_1_2 - 0.134067923781463*G0_2_0 - 0.224425137619578*G0_2_1 - 0.156575677409008*G0_2_2;
    A[2007] = 0.0195719596761261*G0_0_0 - 0.0411011153198671*G0_0_1 - 0.0552907860850583*G0_0_2 - 0.0411011153198653*G0_1_0 - 0.00880738185425761*G0_1_1 + 0.00538228891093449*G0_1_2 - 0.0552907860850566*G0_2_0 + 0.00538228891093366*G0_2_1 + 0.0195719596761256*G0_2_2;
    A[2008] = -0.0880738185425678*G0_0_0 + 0.000652398655870936*G0_0_1 + 0.0127217737894812*G0_0_2 + 0.000652398655870664*G0_1_0 + 0.00130479731174116*G0_1_1 + 0.000652398655869771*G0_1_2 + 0.0127217737894816*G0_2_0 + 0.000652398655870722*G0_2_1;
    A[2009] = 0.0929668084615979*G0_0_0 + 0.228013330226871*G0_0_1 + 0.363059851992141*G0_0_2 + 0.228013330226868*G0_1_0 + 0.456026660453738*G0_1_1 + 0.22801333022687*G0_1_2 + 0.363059851992138*G0_2_0 + 0.228013330226868*G0_2_1 + 0.092966808461599*G0_2_2;
    A[2010] = 0.0929668084616004*G0_0_0 + 0.0322937334656107*G0_0_1 - 0.0283793415303822*G0_0_2 + 0.0322937334656091*G0_1_0 - 0.00880738185425556*G0_1_1 - 0.0141896707651917*G0_1_2 - 0.0283793415303811*G0_2_0 - 0.0141896707651897*G0_2_1;
    A[2011] = -0.185933616923199*G0_0_0 - 0.195067198105393*G0_0_1 - 0.104709984267276*G0_0_2 - 0.195067198105391*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105718*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105717*G0_2_1;
    A[2012] = 0.146789697570944*G0_0_0 + 0.445262082631869*G0_0_1 + 0.249542485870608*G0_0_2 + 0.44526208263187*G0_1_0 - 0.0391439193522514*G0_1_1 + 0.249542485870606*G0_1_2 + 0.249542485870607*G0_2_0 + 0.249542485870607*G0_2_1 + 0.352295274170268*G0_2_2;
    A[2013] = 0.146789697570947*G0_0_0 + 0.00489298991903024*G0_0_1 + 0.0538228891093478*G0_0_2 + 0.00489298991903033*G0_1_0 + 0.00978597983806011*G0_1_1 + 0.00489298991903111*G0_1_2 + 0.0538228891093479*G0_2_0 + 0.00489298991902858*G0_2_1 - 0.0391439193522536*G0_2_2;
    A[2014] = -0.782878387045048*G0_0_0 - 0.48440600198412*G0_0_1 - 0.484406001984122*G0_0_2 - 0.484406001984123*G0_1_0 - 0.0391439193522528*G0_1_1 - 0.288686405222862*G0_1_2 - 0.484406001984124*G0_2_0 - 0.28868640522286*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2015] = -0.0489298991903162*G0_0_0 - 0.0195719596761266*G0_0_1 - 0.141896707651915*G0_0_2 - 0.0195719596761259*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0195719596761267*G0_1_2 - 0.141896707651915*G0_2_0 - 0.0195719596761258*G0_2_1 - 0.234863516113513*G0_2_2;
    A[2016] = 0.0137421252872639*G0_0_0 + 0.013742125287264*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202554*G0_1_0 + 0.0189195610202553*G0_1_1 + 0.0189195610202554*G0_1_2 + 0.0189195610202552*G0_2_0 + 0.0189195610202552*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[2017] = -0.0137421252872641*G0_0_0;
    A[2018] = -0.014394523943135*G0_0_1 + 0.0335985307773501*G0_1_1;
    A[2019] = 0.0051774357329913*G0_0_2 + 0.0189195610202553*G0_2_2;
    A[2020] = -0.192588083213083*G0_0_1 - 0.124738623002512*G0_0_2 - 0.257045070413125*G0_1_1 - 0.0962940416065415*G0_1_2 - 0.0962940416065412*G0_2_1;
    A[2021] = -0.0109602974186295*G0_0_1 - 0.146659217839771*G0_0_2 + 0.121346149991983*G0_1_1 - 0.0733296089198858*G0_1_2 - 0.0733296089198855*G0_2_1 - 0.271397840842282*G0_2_2;
    A[2022] = -0.00182671623643939*G0_0_1 + 0.082332710370903*G0_0_2 - 0.052844291125542*G0_1_1 + 0.0359471659384841*G0_1_2 + 0.0359471659384848*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2023] = -0.011873655536849*G0_0_1 - 0.0477555816097471*G0_0_2 - 0.0212681961813906*G0_1_2 - 0.0212681961813901*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2024] = 0.0187890812890808*G0_0_0 + 0.00939454064454036*G0_0_2 + 0.00939454064454057*G0_2_0;
    A[2025] = -0.0148746893538552*G0_0_0 - 0.0149399292194424*G0_0_2 - 0.0149399292194424*G0_2_0 - 0.0150051690850296*G0_2_2;
    A[2026] = -0.00117431758056849*G0_0_0 - 0.00123955744615534*G0_0_2 - 0.00123955744615499*G0_2_0 - 0.00130479731174186*G0_2_2;
    A[2027] = -0.0118736555368489*G0_0_0 - 0.0212681961813898*G0_0_2 - 0.0212681961813901*G0_2_0 - 0.0306627368259308*G0_2_2;
    A[2028] = 0.0641307878721066*G0_0_0 + 0.00939454064454036*G0_0_1 + 0.00939454064454048*G0_1_0;
    A[2029] = -0.127087258163647*G0_0_0 - 0.0377738821749239*G0_0_1 - 0.0377738821749237*G0_1_0 - 0.0150051690850307*G0_1_1;
    A[2030] = 0.124738623002512*G0_0_0 + 0.0623693115012559*G0_0_1 + 0.0623693115012557*G0_1_0 + 0.0476251018785746*G0_1_1;
    A[2031] = -0.124738623002512*G0_0_0 - 0.0962940416065416*G0_0_1 - 0.0962940416065417*G0_1_0 - 0.128522535206563*G0_1_1;
    A[2032] = -0.0187890812890808*G0_0_0 - 0.0187890812890809*G0_0_1 - 0.00939454064454035*G0_0_2 - 0.061325473651862*G0_1_0 - 0.061325473651862*G0_1_1 - 0.0400572774704716*G0_1_2 - 0.0400572774704713*G0_2_0 - 0.0400572774704714*G0_2_1 - 0.018789081289081*G0_2_2;
    A[2033] = 0.0148746893538557*G0_0_0 + 0.0148746893538557*G0_0_1 - 6.5239865587421e-05*G0_0_2 - 0.0254435475789631*G0_1_0 - 0.0254435475789632*G0_1_1 + 0.0136351319077012*G0_1_2 + 0.013635131907701*G0_2_0 + 0.0136351319077011*G0_2_1 - 0.000130479731174349*G0_2_2;
    A[2034] = 0.00117431758056755*G0_0_0 + 0.00117431758056755*G0_0_1 - 6.5239865587071e-05*G0_0_2 - 0.0528442911255414*G0_1_0 - 0.0528442911255414*G0_1_1 - 0.0137656116388754*G0_1_2 - 0.0137656116388752*G0_2_0 - 0.0137656116388752*G0_2_1 - 0.000130479731174145*G0_2_2;
    A[2035] = 0.011873655536849*G0_0_0 + 0.0118736555368491*G0_0_1 - 0.00939454064454054*G0_0_2 + 0.0212681961813905*G0_1_2 + 0.0212681961813901*G0_2_0 + 0.0212681961813902*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2036] = -0.0641307878721064*G0_0_0 - 0.054736247227566*G0_0_1 - 0.0641307878721066*G0_0_2 - 0.0906181733004643*G0_1_0 - 0.069349977119074*G0_1_1 - 0.0906181733004646*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004643*G0_2_1 - 0.111886369481855*G0_2_2;
    A[2037] = 0.127087258163646*G0_0_0 + 0.0893133759887222*G0_0_1 + 0.127087258163646*G0_0_2 + 0.173472802596065*G0_1_0 + 0.0846813455320394*G0_1_1 + 0.173472802596065*G0_1_2 + 0.20941996853455*G0_2_0 + 0.173472802596065*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2038] = -0.124738623002512*G0_0_0 - 0.0623693115012553*G0_0_1 - 0.124738623002512*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052867*G0_1_1 - 0.198068231922397*G0_1_2 - 0.271397840842283*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842284*G0_2_2;
    A[2039] = 0.124738623002512*G0_0_0 + 0.0284445813959701*G0_0_1 + 0.124738623002512*G0_0_2 + 0.0962940416065415*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.0962940416065414*G0_1_2 + 0.0962940416065409*G0_2_1;
    A[2040] = -0.00939454064454046*G0_0_1 - 0.00939454064454049*G0_0_2 - 0.00939454064454075*G0_1_0 - 0.0306627368259312*G0_1_1 - 0.0306627368259312*G0_1_2 - 0.00939454064454054*G0_2_0 - 0.030662736825931*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[2041] = -6.5239865587097e-05*G0_0_1 - 6.52398655870204e-05*G0_0_2 - 6.52398655869052e-05*G0_1_0 - 0.00130479731174197*G0_1_1 - 0.00130479731174192*G0_1_2 - 6.52398655872793e-05*G0_2_0 - 0.00130479731174183*G0_2_1 - 0.00130479731174183*G0_2_2;
    A[2042] = -6.52398655870937e-05*G0_0_1 - 6.52398655870888e-05*G0_0_2 - 6.52398655872783e-05*G0_1_0 - 0.0150051690850303*G0_1_1 - 0.0150051690850302*G0_1_2 - 6.52398655870046e-05*G0_2_0 - 0.0150051690850301*G0_2_1 - 0.01500516908503*G0_2_2;
    A[2043] = -0.00939454064454035*G0_0_1 - 0.0093945406445404*G0_0_2 - 0.00939454064454044*G0_1_0 - 0.00939454064454042*G0_2_0;
    A[2044] = -0.105688582251081*G0_0_0 - 0.0300103381700601*G0_0_1 - 0.0528442911255416*G0_0_2 - 0.0300103381700614*G0_1_0 - 0.0150051690850311*G0_1_2 - 0.0528442911255413*G0_2_0 - 0.01500516908503*G0_2_1;
    A[2045] = 0.209419968534549*G0_0_0 + 0.0737210481134074*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0737210481134089*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.0368605240567041*G0_1_2 + 0.104709984267274*G0_2_0 + 0.036860524056702*G0_2_1;
    A[2046] = -0.542795681684567*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842284*G0_0_2 - 0.271397840842285*G0_1_0 + 0.0521918924696697*G0_1_1 - 0.135698920421143*G0_1_2 - 0.271397840842283*G0_2_0 - 0.13569892042114*G0_2_1;
    A[2047] = 0.0137003717732886*G0_0_0 - 0.0127217737894817*G0_0_1 + 0.0198981590040624*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.0120693751336113*G0_1_2 + 0.0198981590040613*G0_2_0 - 0.0120693751336106*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[2048] = 0.0234863516113509*G0_0_0 + 0.0459941052388959*G0_0_1 - 0.0812236326559242*G0_0_2 + 0.0459941052388955*G0_1_0 - 0.0880738185425699*G0_1_1 + 0.00913358118219199*G0_1_2 - 0.0812236326559235*G0_2_0 + 0.00913358118219155*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2049] = -0.0534966897814109*G0_0_0 - 0.0528442911255398*G0_0_1 - 0.000652398655870538*G0_0_2 - 0.0528442911255422*G0_1_0 - 0.0378391220405123*G0_1_2 - 0.000652398655870752*G0_2_0 - 0.0378391220405105*G0_2_1 + 0.0521918924696699*G0_2_2;
    A[2050] = 0.105688582251082*G0_0_0 + 0.075678244081021*G0_0_1 + 0.0528442911255401*G0_0_2 + 0.180062029020361*G0_1_0 + 0.255740273101381*G0_1_1 + 0.127870136550692*G0_1_2 + 0.10503618359521*G0_2_0 + 0.127870136550691*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2051] = -0.209419968534551*G0_0_0 - 0.135698920421143*G0_0_1 - 0.104709984267275*G0_0_2 - 0.390134396210783*G0_1_0 - 0.584549195660301*G0_1_1 - 0.262916658315961*G0_1_2 - 0.290643601190474*G0_2_0 - 0.262916658315962*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2052] = 0.542795681684567*G0_0_0 + 0.271397840842285*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842285*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684565*G0_2_2;
    A[2053] = -0.0137003717732868*G0_0_0 - 0.0264221455627689*G0_0_1 + 0.00619778723077405*G0_0_2 + 0.0714376528178608*G0_1_0 + 0.17223324514991*G0_1_1 + 0.00619778723077245*G0_1_2 + 0.0061977872307743*G0_2_0 + 0.00619778723077408*G0_2_1 + 0.012395574461547*G0_2_2;
    A[2054] = -0.0234863516113512*G0_0_0 + 0.0225077536275433*G0_0_1 - 0.104709984267274*G0_0_2 - 0.0459941052388955*G0_1_0 - 0.268135847562929*G0_1_1 + 0.0534966897814122*G0_1_2 + 0.0812236326559233*G0_2_0 + 0.0534966897814108*G0_2_1 - 0.20941996853455*G0_2_2;
    A[2055] = 0.0534966897814111*G0_0_0 + 0.000652398655871834*G0_0_1 + 0.0528442911255397*G0_0_2 + 0.0528442911255425*G0_1_0 + 0.105688582251083*G0_1_1 - 0.0221815542996091*G0_1_2 + 0.000652398655870933*G0_2_0 - 0.0221815542996096*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2056] = 0.030010338170061*G0_0_1 + 0.0150051690850302*G0_0_2 + 0.0300103381700593*G0_1_0 - 0.0456679059109608*G0_1_1 - 0.00782878387044983*G0_1_2 + 0.0150051690850297*G0_2_0 - 0.00782878387045053*G0_2_1 + 0.0300103381700597*G0_2_2;
    A[2057] = 0.00130479731174116*G0_0_1 + 0.000652398655870743*G0_0_2 + 0.00130479731174212*G0_1_0 - 0.0228339529554806*G0_1_1 - 0.0107645778218689*G0_1_2 + 0.000652398655871087*G0_2_0 - 0.0107645778218701*G0_2_1 + 0.0013047973117417*G0_2_2;
    A[2058] = 0.0300103381700601*G0_0_1 + 0.0150051690850294*G0_0_2 + 0.0300103381700605*G0_1_0 + 0.0150051690850303*G0_1_2 + 0.01500516908503*G0_2_0 + 0.0150051690850299*G0_2_1 + 0.0300103381700596*G0_2_2;
    A[2059] = 0.0127217737894821*G0_0_1 + 0.000652398655870141*G0_0_2 + 0.0127217737894822*G0_1_0 - 0.0880738185425671*G0_1_1 + 0.000652398655870127*G0_1_2 + 0.000652398655870628*G0_2_0 + 0.000652398655870769*G0_2_1 + 0.00130479731174192*G0_2_2;
    A[2060] = 0.0127217737894817*G0_0_1 + 0.000652398655871827*G0_0_2 + 0.0127217737894811*G0_1_0 + 0.0120693751336107*G0_1_2 + 0.000652398655870852*G0_2_0 + 0.0120693751336106*G0_2_1 + 0.00130479731174203*G0_2_2;
    A[2061] = 0.0528442911255398*G0_0_1 + 0.0150051690850297*G0_0_2 + 0.0528442911255412*G0_1_0 + 0.0378391220405108*G0_1_2 + 0.0150051690850301*G0_2_0 + 0.0378391220405104*G0_2_1 + 0.03001033817006*G0_2_2;
    A[2062] = 0.0378391220405108*G0_0_1 + 0.0528442911255415*G0_0_2 + 0.0378391220405101*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0900310145101801*G0_1_2 + 0.0528442911255405*G0_2_0 + 0.0900310145101804*G0_2_1 + 0.0521918924696703*G0_2_2;
    A[2063] = 0.00554538857490348*G0_0_1 + 0.00619778723077389*G0_0_2 + 0.00554538857490306*G0_1_0 + 0.0502346965020585*G0_1_1 + 0.0381653213684464*G0_1_2 + 0.0061977872307735*G0_2_0 + 0.0381653213684468*G0_2_1 + 0.0260959462348351*G0_2_2;
    A[2064] = 0.0378391220405108*G0_0_1 + 0.0528442911255417*G0_0_2 + 0.0378391220405107*G0_1_0 + 0.0300103381700605*G0_1_1 + 0.0150051690850312*G0_1_2 + 0.0528442911255402*G0_2_0 + 0.0150051690850303*G0_2_1;
    A[2065] = -0.0678494602105718*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105697*G0_1_0 - 0.292274597830151*G0_1_1 - 0.195067198105392*G0_1_2 - 0.104709984267275*G0_2_0 - 0.195067198105391*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2066] = -0.0678494602105721*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105715*G0_1_0 - 0.0965550010688906*G0_1_1 - 0.0368605240567048*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0368605240567054*G0_2_1;
    A[2067] = 0.135698920421143*G0_0_1 + 0.271397840842284*G0_0_2 + 0.135698920421142*G0_1_0 + 0.323589733311954*G0_1_1 + 0.135698920421143*G0_1_2 + 0.271397840842283*G0_2_0 + 0.135698920421143*G0_2_1;
    A[2068] = -0.0750258454251495*G0_0_1 - 0.026095946234834*G0_0_2 - 0.0750258454251496*G0_1_0 + 0.143527704291592*G0_1_1 - 0.042405912631605*G0_1_2 - 0.0260959462348353*G0_2_0 - 0.0424059126316059*G0_2_1 - 0.05219189246967*G0_2_2;
    A[2069] = -0.0750258454251487*G0_0_1 - 0.026095946234836*G0_0_2 - 0.0750258454251501*G0_1_0 + 0.0456679059109629*G0_1_1 - 0.0260959462348347*G0_1_2 - 0.0260959462348337*G0_2_0 - 0.0260959462348318*G0_2_1 - 0.0521918924696692*G0_2_2;
    A[2070] = 0.316413348097371*G0_0_1 + 0.1859336169232*G0_0_2 + 0.316413348097374*G0_1_0 - 0.150051690850301*G0_1_1 + 0.1859336169232*G0_1_2 + 0.185933616923199*G0_2_0 + 0.185933616923197*G0_2_1 + 0.371867233846398*G0_2_2;
    A[2071] = -0.058715879028378*G0_0_1 - 0.0260959462348369*G0_0_2 - 0.0587158790283784*G0_1_0 + 0.176147637085136*G0_1_1 - 0.00978597983806252*G0_1_2 - 0.0260959462348355*G0_2_0 - 0.00978597983806252*G0_2_1 - 0.0521918924696695*G0_2_2;
    A[2072] = -0.0345719095719094*G0_0_0 - 0.0345719095719094*G0_0_1 - 0.0345719095719094*G0_0_2 - 0.0298472385060926*G0_1_0 - 0.0298472385060926*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0298472385060925*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[2073] = -0.00946499969937492*G0_0_0;
    A[2074] = 0.00227817610630123*G0_0_1 + 0.0117431758056758*G0_1_1;
    A[2075] = 0.00472467106581674*G0_0_2 - 0.0298472385060924*G0_2_2;
    A[2076] = -0.0142875305635725*G0_0_1 - 0.00254435475789642*G0_0_2 - 0.0435476102793811*G0_1_1 - 0.00583896797004417*G0_1_2 - 0.00583896797004432*G0_2_1;
    A[2077] = 0.0377738821749231*G0_0_1 + 0.00939454064453976*G0_0_2 + 0.056758683060766*G0_1_1 + 0.0188869410874609*G0_1_2 + 0.0188869410874616*G0_2_1 - 0.00619778723077428*G0_2_2;
    A[2078] = 0.108232937008977*G0_0_1 + 0.0808321934624005*G0_0_2 + 0.104709984267274*G0_1_1 + 0.0743408268364859*G0_1_2 + 0.0743408268364856*G0_2_1 + 0.0371867233846406*G0_2_2;
    A[2079] = 0.0429604514890963*G0_0_1 + 0.0845508658008645*G0_0_2 + 0.0422754329004323*G0_1_2 + 0.0422754329004326*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2080] = 0.0292600797158095*G0_0_0 + 0.00329461321214793*G0_0_2 + 0.00329461321214792*G0_2_0;
    A[2081] = -0.0189848008858432*G0_0_0 - 0.00949240044292091*G0_0_2 - 0.00949240044292138*G0_2_0 - 0.000652398655869983*G0_2_2;
    A[2082] = 0.0035229527417036*G0_0_0 + 0.00649136662591534*G0_0_2 + 0.00649136662591518*G0_2_0 + 0.00880738185425565*G0_2_2;
    A[2083] = 0.0429604514890965*G0_0_0 + 0.0422754329004326*G0_0_2 + 0.0422754329004328*G0_2_0 + 0.0642612676032815*G0_2_2;
    A[2084] = 0.00658922642429613*G0_0_0 + 0.00329461321214794*G0_0_1 + 0.00329461321214776*G0_1_0;
    A[2085] = 0.0045015507255088*G0_0_0 + 0.00192457603481895*G0_0_1 + 0.0019245760348192*G0_1_0 - 0.000652398655870326*G0_1_1;
    A[2086] = -0.0105036183595211*G0_0_0 - 0.0130805930502108*G0_0_1 - 0.0130805930502109*G0_1_0 - 0.0156575677409011*G0_1_1;
    A[2087] = -0.00254435475789638*G0_0_0 - 0.00583896797004456*G0_0_1 - 0.00583896797004428*G0_1_0 - 0.00913358118219195*G0_1_1;
    A[2088] = 0.1273155976932*G0_0_0 + 0.1273155976932*G0_0_1 + 0.128000616281865*G0_0_2 + 0.176636936077039*G0_1_0 + 0.176636936077039*G0_1_1 + 0.134361503176607*G0_1_2 + 0.134361503176606*G0_2_0 + 0.134361503176606*G0_2_1 + 0.0920860702761735*G0_2_2;
    A[2089] = -0.166948816037355*G0_0_0 - 0.166948816037355*G0_0_1 - 0.160783648739377*G0_0_2 - 0.334680510461758*G0_1_0 - 0.334680510461758*G0_1_1 - 0.250814663249558*G0_1_2 - 0.250814663249557*G0_2_0 - 0.250814663249557*G0_2_1 - 0.062238831770081*G0_2_2;
    A[2090] = 0.182410664181495*G0_0_0 + 0.182410664181495*G0_0_1 + 0.0985448169692953*G0_0_2 + 0.104709984267274*G0_1_0 + 0.104709984267274*G0_1_1 + 0.188575831479476*G0_1_2 + 0.188575831479475*G0_2_0 + 0.188575831479475*G0_2_1 - 0.0622388317700816*G0_2_2;
    A[2091] = -0.0429604514890964*G0_0_0 - 0.0429604514890963*G0_0_1 - 0.0359145460056913*G0_0_2 - 0.0422754329004324*G0_1_2 - 0.0422754329004326*G0_2_0 - 0.0422754329004326*G0_2_1 + 0.0920860702761742*G0_2_2;
    A[2092] = 0.043971669405696*G0_0_0 + 0.0446566879943613*G0_0_1 + 0.043971669405696*G0_0_2 + 0.0862471023061292*G0_1_0 + 0.043971669405697*G0_1_1 + 0.0862471023061291*G0_1_2 + 0.128522535206562*G0_2_0 + 0.0862471023061296*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2093] = -0.0436454700777606*G0_0_0 - 0.00975335990526954*G0_0_1 - 0.0436454700777605*G0_0_2 - 0.0371541034518459*G0_1_0 - 0.00678494602105709*G0_1_1 - 0.0371541034518459*G0_1_2 + 0.0371867233846396*G0_2_0 - 0.0371541034518463*G0_2_1 + 0.0371867233846395*G0_2_2;
    A[2094] = -0.0155923278753139*G0_0_0 + 0.00329461321214813*G0_0_1 - 0.0155923278753139*G0_0_2 - 0.025084728318235*G0_1_0 + 0.0127870136550689*G0_1_1 - 0.025084728318235*G0_1_2 - 0.00619778723077306*G0_2_0 - 0.0250847283182351*G0_2_1 - 0.00619778723077313*G0_2_2;
    A[2095] = 0.0025443547578965*G0_0_0 - 0.00590420783563159*G0_0_1 + 0.00254435475789644*G0_0_2 + 0.00583896797004434*G0_1_0 - 0.031869674339292*G0_1_1 + 0.00583896797004433*G0_1_2 + 0.00583896797004443*G0_2_1;
    A[2096] = 0.022670853291513*G0_0_0 + 0.0219858347028478*G0_0_1 + 0.0219858347028477*G0_0_2 + 0.0219858347028487*G0_1_0 + 0.0642612676032808*G0_1_1 + 0.0642612676032807*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.064261267603281*G0_2_1 + 0.0642612676032809*G0_2_2;
    A[2097] = -0.000652398655870716*G0_0_0 + 0.0023160152283412*G0_0_1 + 0.00231601522834132*G0_0_2 + 0.00231601522834156*G0_1_0 + 0.00880738185425679*G0_1_1 + 0.00880738185425689*G0_1_2 + 0.00231601522834186*G0_2_0 + 0.00880738185425688*G0_2_1 + 0.00880738185425689*G0_2_2;
    A[2098] = -0.000652398655871065*G0_0_0 + 0.00884000178705083*G0_0_1 + 0.00884000178705084*G0_0_2 + 0.00884000178705023*G0_1_0 - 0.000652398655870893*G0_1_1 - 0.000652398655870905*G0_1_2 + 0.0088400017870501*G0_2_0 - 0.000652398655871037*G0_2_1 - 0.000652398655871012*G0_2_2;
    A[2099] = 0.022670853291513*G0_0_0 - 0.0032946132121479*G0_0_1 - 0.00329461321214791*G0_0_2 - 0.00329461321214764*G0_1_0 - 0.00329461321214774*G0_2_0;
    A[2100] = -0.025443547578966*G0_0_0 - 0.0127217737894825*G0_0_1 - 0.00130479731174264*G0_0_2 - 0.0127217737894834*G0_1_0 - 0.000652398655871983*G0_1_2 - 0.0013047973117425*G0_2_0 - 0.000652398655871176*G0_2_1;
    A[2101] = -0.0166361657247064*G0_0_0 + 0.00146789697571019*G0_0_1 - 0.0181040627004162*G0_0_2 + 0.00146789697571076*G0_1_0 + 0.0195719596761278*G0_1_1 - 0.0102752788299648*G0_1_2 - 0.0181040627004165*G0_2_0 - 0.0102752788299656*G0_2_1;
    A[2102] = 0.0137003717732866*G0_0_0 + 0.0264221455627691*G0_0_1 - 0.00619778723077414*G0_0_2 + 0.0264221455627687*G0_1_0 + 0.0391439193522508*G0_1_1 - 0.00554538857490298*G0_1_2 - 0.00619778723077426*G0_2_0 - 0.00554538857490349*G0_2_1;
    A[2103] = 0.0567586830607689*G0_0_0 + 0.0283793415303847*G0_0_1 + 0.0283793415303846*G0_0_2 + 0.0283793415303841*G0_1_0 + 0.0141896707651921*G0_1_2 + 0.0283793415303837*G0_2_0 + 0.0141896707651916*G0_2_1 - 0.00489298991903143*G0_2_2;
    A[2104] = 0.149725491522362*G0_0_0 + 0.12134614999198*G0_0_1 + 0.0748627457611807*G0_0_2 + 0.12134614999198*G0_1_0 + 0.0929668084615973*G0_1_1 + 0.060673074995989*G0_1_2 + 0.0748627457611811*G0_2_0 + 0.0606730749959897*G0_2_1 + 0.0195719596761234*G0_2_2;
    A[2105] = 0.209419968534548*G0_0_0 + 0.104709984267275*G0_0_1 + 0.135698920421141*G0_0_2 + 0.104709984267274*G0_1_0 + 0.0678494602105702*G0_1_2 + 0.135698920421142*G0_2_0 + 0.0678494602105716*G0_2_1 + 0.0391439193522535*G0_2_2;
    A[2106] = -0.268135847562927*G0_0_0 - 0.222141742324031*G0_0_1 - 0.194414799449518*G0_0_2 - 0.290643601190473*G0_1_0 - 0.268135847562929*G0_1_1 - 0.321632537344339*G0_1_2 - 0.448850275239161*G0_2_0 - 0.321632537344341*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2107] = 0.0655660649150221*G0_0_0 + 0.0376760223765423*G0_0_1 + 0.140428810676204*G0_0_2 + 0.140428810676205*G0_1_0 - 0.0371867233846387*G0_1_1 + 0.186912214907004*G0_1_2 + 0.0376760223765437*G0_2_0 + 0.186912214907007*G0_2_1 - 0.0371867233846396*G0_2_2;
    A[2108] = -0.0137003717732868*G0_0_0 + 0.0714376528178608*G0_0_1 + 0.00619778723077429*G0_0_2 - 0.0264221455627689*G0_1_0 + 0.17223324514991*G0_1_1 + 0.00619778723077408*G0_1_2 + 0.00619778723077405*G0_2_0 + 0.00619778723077246*G0_2_1 + 0.012395574461547*G0_2_2;
    A[2109] = 0.72611970398428*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453738*G0_0_2 + 0.456026660453738*G0_1_0 + 0.72611970398428*G0_1_1 + 0.456026660453739*G0_1_2 + 0.456026660453738*G0_2_0 + 0.45602666045374*G0_2_1 + 0.912053320907479*G0_2_2;
    A[2110] = -0.149725491522363*G0_0_0 - 0.326851726591307*G0_0_1 - 0.177615534060844*G0_0_2 - 0.12134614999198*G0_1_0 - 0.448197876583289*G0_1_1 - 0.224098938291644*G0_1_2 - 0.0748627457611811*G0_2_0 - 0.224098938291644*G0_2_1 - 0.03718672338464*G0_2_2;
    A[2111] = -0.209419968534548*G0_0_0 - 0.290643601190472*G0_0_1 - 0.390134396210781*G0_0_2 - 0.104709984267274*G0_1_0 - 0.209419968534548*G0_1_1 - 0.262916658315961*G0_1_2 - 0.135698920421142*G0_2_0 - 0.262916658315961*G0_2_1 - 0.584549195660301*G0_2_2;
    A[2112] = -0.088073818542568*G0_0_0 - 0.134067923781464*G0_0_1 - 0.0972073997247581*G0_0_2 - 0.134067923781465*G0_1_0 - 0.156575677409009*G0_1_1 - 0.22442513761958*G0_1_2 - 0.0972073997247605*G0_2_0 - 0.224425137619581*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2113] = 0.0195719596761273*G0_0_0 - 0.0552907860850555*G0_0_1 - 0.0411011153198647*G0_0_2 - 0.0552907860850559*G0_1_0 + 0.0195719596761268*G0_1_1 + 0.00538228891093486*G0_1_2 - 0.0411011153198643*G0_2_0 + 0.00538228891093525*G0_2_1 - 0.00880738185425661*G0_2_2;
    A[2114] = -0.0880738185425691*G0_0_0 + 0.0127217737894819*G0_0_1 + 0.000652398655869781*G0_0_2 + 0.0127217737894818*G0_1_0 + 0.000652398655869837*G0_1_2 + 0.000652398655870747*G0_2_0 + 0.000652398655870715*G0_2_1 + 0.00130479731174019*G0_2_2;
    A[2115] = 0.0929668084615987*G0_0_0 + 0.363059851992141*G0_0_1 + 0.228013330226869*G0_0_2 + 0.363059851992141*G0_1_0 + 0.0929668084615995*G0_1_1 + 0.22801333022687*G0_1_2 + 0.22801333022687*G0_2_0 + 0.228013330226869*G0_2_1 + 0.456026660453741*G0_2_2;
    A[2116] = 0.0929668084615992*G0_0_0 - 0.028379341530384*G0_0_1 + 0.032293733465608*G0_0_2 - 0.0283793415303837*G0_1_0 - 0.0141896707651909*G0_1_2 + 0.032293733465608*G0_2_0 - 0.0141896707651909*G0_2_1 - 0.00880738185425597*G0_2_2;
    A[2117] = -0.185933616923198*G0_0_0 - 0.104709984267275*G0_0_1 - 0.195067198105391*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105716*G0_1_2 - 0.195067198105391*G0_2_0 - 0.0678494602105718*G0_2_1 - 0.292274597830152*G0_2_2;
    A[2118] = -0.0228339529554804*G0_0_0 - 0.0596944770121833*G0_0_1 - 0.0965550010688892*G0_0_2 - 0.0596944770121853*G0_1_0 - 0.0965550010688891*G0_1_1 - 0.0287055408583187*G0_1_2 - 0.0965550010688899*G0_2_0 - 0.0287055408583188*G0_2_1 + 0.0391439193522514*G0_2_2;
    A[2119] = -0.0048929899190307*G0_0_0 - 0.0190826606842233*G0_0_1 - 0.0332723314494143*G0_0_2 - 0.0190826606842226*G0_1_0 - 0.0332723314494146*G0_1_1 - 0.0190826606842233*G0_1_2 - 0.0332723314494141*G0_2_0 - 0.0190826606842229*G0_2_1 - 0.00489298991903102*G0_2_2;
    A[2120] = -0.0228339529554813*G0_0_0 - 0.0107645778218697*G0_0_1 + 0.00130479731174243*G0_0_2 - 0.0107645778218691*G0_1_0 + 0.00130479731174131*G0_1_1 + 0.000652398655871905*G0_1_2 + 0.00130479731174231*G0_2_0 + 0.000652398655870442*G0_2_1;
    A[2121] = 0.0195719596761257*G0_0_0 + 0.00538228891093403*G0_0_1 - 0.0552907860850567*G0_0_2 + 0.00538228891093487*G0_1_0 - 0.00880738185425618*G0_1_1 - 0.0411011153198649*G0_1_2 - 0.0552907860850557*G0_2_0 - 0.0411011153198641*G0_2_1 + 0.0195719596761275*G0_2_2;
    A[2122] = 0.0195719596761262*G0_0_0 + 0.0298472385060926*G0_0_1 + 0.0181040627004162*G0_0_2 + 0.0298472385060918*G0_1_0 + 0.0401225173360589*G0_1_1 + 0.0102752788299649*G0_1_2 + 0.0181040627004154*G0_2_0 + 0.0102752788299658*G0_2_1;
    A[2123] = 0.0260959462348349*G0_0_0 + 0.038165321368446*G0_0_1 + 0.00619778723077398*G0_0_2 + 0.0381653213684459*G0_1_0 + 0.0502346965020563*G0_1_1 + 0.00554538857490293*G0_1_2 + 0.0061977872307728*G0_2_0 + 0.00554538857490143*G0_2_1;
    A[2124] = 0.146789697570946*G0_0_0 + 0.249542485870608*G0_0_1 + 0.445262082631871*G0_0_2 + 0.249542485870609*G0_1_0 + 0.352295274170271*G0_1_1 + 0.249542485870609*G0_1_2 + 0.445262082631873*G0_2_0 + 0.24954248587061*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[2125] = 0.146789697570947*G0_0_0 + 0.0538228891093456*G0_0_1 + 0.00489298991902994*G0_0_2 + 0.0538228891093475*G0_1_0 - 0.0391439193522539*G0_1_1 + 0.00489298991903118*G0_1_2 + 0.00489298991903056*G0_2_0 + 0.00489298991903037*G0_2_1 + 0.00978597983806263*G0_2_2;
    A[2126] = -0.0489298991903154*G0_0_0 - 0.141896707651913*G0_0_1 - 0.0195719596761244*G0_0_2 - 0.141896707651916*G0_1_0 - 0.234863516113512*G0_1_1 - 0.0195719596761243*G0_1_2 - 0.0195719596761269*G0_2_0 - 0.0195719596761256*G0_2_1 - 0.0391439193522502*G0_2_2;
    A[2127] = -0.782878387045049*G0_0_0 - 0.484406001984121*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984123*G0_1_0 - 0.185933616923197*G0_1_1 - 0.28868640522286*G0_1_2 - 0.484406001984121*G0_2_0 - 0.288686405222859*G0_2_1 - 0.0391439193522531*G0_2_2;
    A[2128] = 0.00946499969937457*G0_0_0 + 0.00946499969937461*G0_0_1 + 0.0094649996993746*G0_0_2 + 0.0117431758056757*G0_1_0 + 0.0117431758056758*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.0117431758056758*G0_2_0 + 0.0117431758056758*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[2129] = -0.0094649996993745*G0_0_0;
    A[2130] = 0.00472467106581691*G0_0_1 - 0.0298472385060927*G0_1_1;
    A[2131] = 0.00472467106581677*G0_0_2 - 0.0298472385060927*G0_2_2;
    A[2132] = 0.0493213383838378*G0_0_1 + 0.0429604514890968*G0_0_2 + 0.17663693607704*G0_1_1 + 0.0422754329004326*G0_1_2 + 0.0422754329004324*G0_2_1;
    A[2133] = -0.167731694424402*G0_0_1 - 0.0777006799142207*G0_0_2 - 0.334680510461759*G0_1_1 - 0.0838658472122*G0_1_2 - 0.083865847212201*G0_2_1 + 0.104709984267276*G0_2_2;
    A[2134] = -0.0777006799142203*G0_0_1 - 0.167731694424402*G0_0_2 + 0.104709984267277*G0_1_1 - 0.0838658472122006*G0_1_2 - 0.0838658472121997*G0_2_1 - 0.334680510461758*G0_2_2;
    A[2135] = 0.0429604514890967*G0_0_1 + 0.0493213383838382*G0_0_2 + 0.042275432900433*G0_1_2 + 0.0422754329004318*G0_2_1 + 0.176636936077039*G0_2_2;
    A[2136] = 0.0292600797158085*G0_0_0 + 0.00329461321214806*G0_0_2 + 0.00329461321214774*G0_2_0;
    A[2137] = -0.0189848008858423*G0_0_0 - 0.00949240044292138*G0_0_2 - 0.0094924004429216*G0_2_0 - 0.000652398655871498*G0_2_2;
    A[2138] = 0.00352295274170271*G0_0_0 + 0.00649136662591519*G0_0_2 + 0.00649136662591558*G0_2_0 + 0.00880738185425708*G0_2_2;
    A[2139] = 0.0429604514890969*G0_0_0 + 0.0422754329004326*G0_0_2 + 0.0422754329004319*G0_2_0 + 0.0642612676032807*G0_2_2;
    A[2140] = 0.0292600797158083*G0_0_0 + 0.00329461321214801*G0_0_1 + 0.00329461321214771*G0_1_0;
    A[2141] = -0.0189848008858419*G0_0_0 - 0.00949240044292122*G0_0_1 - 0.00949240044292082*G0_1_0 - 0.000652398655870543*G0_1_1;
    A[2142] = 0.00352295274170211*G0_0_0 + 0.00649136662591524*G0_0_1 + 0.00649136662591457*G0_1_0 + 0.00880738185425626*G0_1_1;
    A[2143] = 0.0429604514890968*G0_0_0 + 0.042275432900432*G0_0_1 + 0.0422754329004326*G0_1_0 + 0.0642612676032811*G0_1_1;
    A[2144] = -0.0292600797158083*G0_0_0 - 0.0292600797158083*G0_0_1 - 0.0259654665036606*G0_0_2 - 0.0435476102793807*G0_1_0 - 0.0435476102793808*G0_1_1 - 0.0377086423093366*G0_1_2 - 0.0377086423093364*G0_2_0 - 0.0377086423093364*G0_2_1 - 0.0318696743392923*G0_2_2;
    A[2145] = 0.0189848008858415*G0_0_0 + 0.0189848008858417*G0_0_1 + 0.00949240044292105*G0_0_2 + 0.0567586830607655*G0_1_0 + 0.0567586830607655*G0_1_1 + 0.0378717419733045*G0_1_2 + 0.0378717419733045*G0_2_0 + 0.0378717419733045*G0_2_1 + 0.0127870136550695*G0_2_2;
    A[2146] = -0.0035229527417021*G0_0_0 - 0.00352295274170228*G0_0_1 + 0.00296841388421245*G0_0_2 + 0.104709984267276*G0_1_0 + 0.104709984267276*G0_1_1 + 0.0303691574307893*G0_1_2 + 0.0303691574307892*G0_2_0 + 0.0303691574307891*G0_2_1 - 0.00678494602105711*G0_2_2;
    A[2147] = -0.042960451489097*G0_0_0 - 0.0429604514890969*G0_0_1 - 0.000685018588664298*G0_0_2 - 0.0422754329004328*G0_1_2 - 0.0422754329004319*G0_2_0 - 0.0422754329004319*G0_2_1 + 0.043971669405697*G0_2_2;
    A[2148] = -0.0292600797158084*G0_0_0 - 0.0259654665036607*G0_0_1 - 0.0292600797158084*G0_0_2 - 0.0377086423093364*G0_1_0 - 0.0318696743392924*G0_1_1 - 0.0377086423093364*G0_1_2 - 0.0435476102793808*G0_2_0 - 0.0377086423093367*G0_2_1 - 0.0435476102793809*G0_2_2;
    A[2149] = 0.0189848008858423*G0_0_0 + 0.00949240044292114*G0_0_1 + 0.0189848008858424*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.0127870136550695*G0_1_1 + 0.037871741973304*G0_1_2 + 0.0567586830607654*G0_2_0 + 0.0378717419733042*G0_2_1 + 0.0567586830607654*G0_2_2;
    A[2150] = -0.00352295274170319*G0_0_0 + 0.00296841388421273*G0_0_1 - 0.00352295274170329*G0_0_2 + 0.0303691574307889*G0_1_0 - 0.00678494602105733*G0_1_1 + 0.0303691574307888*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0303691574307894*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2151] = -0.0429604514890969*G0_0_0 - 0.000685018588664665*G0_0_1 - 0.0429604514890967*G0_0_2 - 0.0422754329004327*G0_1_0 + 0.0439716694056971*G0_1_1 - 0.0422754329004326*G0_1_2 - 0.0422754329004327*G0_2_1;
    A[2152] = -0.00329461321214753*G0_0_1 - 0.00329461321214754*G0_0_2 - 0.00329461321214808*G0_1_0 - 0.0091335811821922*G0_1_1 - 0.00913358118219212*G0_1_2 - 0.00329461321214813*G0_2_0 - 0.00913358118219221*G0_2_1 - 0.00913358118219228*G0_2_2;
    A[2153] = -0.00257697469069018*G0_0_1 - 0.0025769746906902*G0_0_2 - 0.00257697469069013*G0_1_0 - 0.0156575677409006*G0_1_1 - 0.0156575677409007*G0_1_2 - 0.00257697469068991*G0_2_0 - 0.015657567740901*G0_2_1 - 0.0156575677409011*G0_2_2;
    A[2154] = -0.00257697469068965*G0_0_1 - 0.0025769746906896*G0_0_2 - 0.00257697469068973*G0_1_0 - 0.000652398655871092*G0_1_1 - 0.000652398655871132*G0_1_2 - 0.00257697469068976*G0_2_0 - 0.000652398655870981*G0_2_1 - 0.000652398655870972*G0_2_2;
    A[2155] = -0.00329461321214808*G0_0_1 - 0.00329461321214807*G0_0_2 - 0.00329461321214801*G0_1_0 - 0.00329461321214826*G0_2_0;
    A[2156] = -0.113517366121531*G0_0_0 - 0.0127217737894822*G0_0_1 - 0.0127217737894824*G0_0_2 - 0.012721773789481*G0_1_0 - 0.000652398655870163*G0_1_2 - 0.0127217737894811*G0_2_0 - 0.00065239865587083*G0_2_1;
    A[2157] = 0.149725491522365*G0_0_0 + 0.0748627457611828*G0_0_1 + 0.0283793415303835*G0_0_2 + 0.0748627457611802*G0_1_0 + 0.0195719596761239*G0_1_1 + 0.01418967076519*G0_1_2 + 0.0283793415303824*G0_2_0 + 0.0141896707651915*G0_2_1;
    A[2158] = 0.0234863516113516*G0_0_0 - 0.0225077536275452*G0_0_1 + 0.104709984267274*G0_0_2 - 0.0225077536275431*G0_1_0 - 0.156575677409009*G0_1_1 + 0.0678494602105711*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0678494602105703*G0_2_1;
    A[2159] = 0.149725491522365*G0_0_0 + 0.0283793415303827*G0_0_1 + 0.0748627457611829*G0_0_2 + 0.0283793415303819*G0_1_0 + 0.0141896707651904*G0_1_2 + 0.0748627457611809*G0_2_0 + 0.0141896707651903*G0_2_1 + 0.019571959676125*G0_2_2;
    A[2160] = -0.540186087061083*G0_0_0 - 0.270093043530541*G0_0_1 - 0.270093043530541*G0_0_2 - 0.27009304353054*G0_1_0 + 0.0929668084616014*G0_1_1 - 0.13504652176527*G0_1_2 - 0.27009304353054*G0_2_0 - 0.13504652176527*G0_2_1 + 0.0929668084616011*G0_2_2;
    A[2161] = 0.0234863516113518*G0_0_0 + 0.104709984267275*G0_0_1 - 0.0225077536275452*G0_0_2 + 0.104709984267277*G0_1_0 + 0.0678494602105721*G0_1_2 - 0.0225077536275449*G0_2_0 + 0.0678494602105703*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2162] = 0.113517366121531*G0_0_0 + 0.10079559233205*G0_0_1 + 0.100795592332049*G0_0_2 + 0.19865539071268*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919136*G0_1_2 + 0.19865539071268*G0_2_0 + 0.166035457919136*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2163] = -0.149725491522365*G0_0_0 - 0.0748627457611836*G0_0_1 - 0.121346149991981*G0_0_2 - 0.177615534060845*G0_1_0 - 0.0371867233846407*G0_1_1 - 0.224098938291644*G0_1_2 - 0.326851726591308*G0_2_0 - 0.224098938291645*G0_2_1 - 0.448197876583288*G0_2_2;
    A[2164] = -0.0234863516113512*G0_0_0 - 0.0459941052388955*G0_0_1 + 0.0812236326559233*G0_0_2 + 0.0225077536275433*G0_1_0 - 0.268135847562929*G0_1_1 + 0.0534966897814108*G0_1_2 - 0.104709984267274*G0_2_0 + 0.0534966897814122*G0_2_1 - 0.20941996853455*G0_2_2;
    A[2165] = -0.149725491522363*G0_0_0 - 0.12134614999198*G0_0_1 - 0.0748627457611811*G0_0_2 - 0.326851726591307*G0_1_0 - 0.448197876583289*G0_1_1 - 0.224098938291644*G0_1_2 - 0.177615534060844*G0_2_0 - 0.224098938291644*G0_2_1 - 0.03718672338464*G0_2_2;
    A[2166] = 0.540186087061082*G0_0_0 + 0.27009304353054*G0_0_1 + 0.27009304353054*G0_0_2 + 0.27009304353054*G0_1_0 + 0.726119703984284*G0_1_1 + 0.270093043530541*G0_1_2 + 0.27009304353054*G0_2_0 + 0.270093043530541*G0_2_1 + 0.72611970398428*G0_2_2;
    A[2167] = -0.0234863516113516*G0_0_0 + 0.0812236326559228*G0_0_1 - 0.0459941052388975*G0_0_2 - 0.104709984267277*G0_1_0 - 0.209419968534553*G0_1_1 + 0.0534966897814113*G0_1_2 + 0.022507753627545*G0_2_0 + 0.0534966897814106*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2168] = 0.0127217737894816*G0_0_1 + 0.0120693751336102*G0_0_2 + 0.012721773789483*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.0446893079271544*G0_1_2 + 0.0120693751336111*G0_2_0 + 0.0446893079271549*G0_2_1 + 0.0502346965020575*G0_2_2;
    A[2169] = 0.0181040627004161*G0_0_1 + 0.0102752788299661*G0_0_2 + 0.018104062700416*G0_1_0 + 0.0195719596761253*G0_1_1 + 0.0298472385060919*G0_1_2 + 0.0102752788299661*G0_2_0 + 0.029847238506092*G0_2_1 + 0.0401225173360585*G0_2_2;
    A[2170] = 0.0127217737894825*G0_0_1 + 0.0120693751336112*G0_0_2 + 0.012721773789482*G0_1_0 + 0.000652398655871521*G0_1_2 + 0.0120693751336116*G0_2_0 + 0.000652398655871265*G0_2_1 + 0.00130479731174263*G0_2_2;
    A[2171] = -0.0283793415303821*G0_0_1 - 0.0141896707651902*G0_0_2 - 0.0283793415303836*G0_1_0 + 0.092966808461599*G0_1_1 + 0.0322937334656086*G0_1_2 - 0.0141896707651914*G0_2_0 + 0.0322937334656089*G0_2_1 - 0.0088073818542566*G0_2_2;
    A[2172] = -0.0283793415303832*G0_0_1 - 0.014189670765191*G0_0_2 - 0.0283793415303815*G0_1_0 - 0.0141896707651917*G0_1_2 - 0.0141896707651913*G0_2_0 - 0.0141896707651908*G0_2_1 - 0.0332723314494146*G0_2_2;
    A[2173] = -0.104709984267275*G0_0_1 - 0.036860524056705*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0678494602105706*G0_1_2 - 0.0368605240567046*G0_2_0 - 0.0678494602105701*G0_2_1 - 0.0965550010688891*G0_2_2;
    A[2174] = 0.012069375133611*G0_0_1 + 0.0127217737894826*G0_0_2 + 0.012069375133612*G0_1_0 + 0.0502346965020571*G0_1_1 + 0.0446893079271553*G0_1_2 + 0.0127217737894824*G0_2_0 + 0.0446893079271544*G0_2_1 + 0.0391439193522517*G0_2_2;
    A[2175] = 0.0102752788299666*G0_0_1 + 0.0181040627004166*G0_0_2 + 0.0102752788299656*G0_1_0 + 0.0401225173360584*G0_1_1 + 0.0298472385060926*G0_1_2 + 0.0181040627004163*G0_2_0 + 0.029847238506093*G0_2_1 + 0.0195719596761267*G0_2_2;
    A[2176] = 0.0120693751336109*G0_0_1 + 0.0127217737894822*G0_0_2 + 0.0120693751336111*G0_1_0 + 0.00130479731174189*G0_1_1 + 0.00065239865587002*G0_1_2 + 0.0127217737894827*G0_2_0 + 0.000652398655870783*G0_2_1;
    A[2177] = -0.0141896707651925*G0_0_1 - 0.0283793415303834*G0_0_2 - 0.0141896707651916*G0_1_0 - 0.00880738185425719*G0_1_1 + 0.0322937334656076*G0_1_2 - 0.0283793415303821*G0_2_0 + 0.0322937334656078*G0_2_1 + 0.0929668084615986*G0_2_2;
    A[2178] = -0.0141896707651917*G0_0_1 - 0.0283793415303832*G0_0_2 - 0.0141896707651916*G0_1_0 - 0.0332723314494143*G0_1_1 - 0.01418967076519*G0_1_2 - 0.0283793415303844*G0_2_0 - 0.0141896707651922*G0_2_1;
    A[2179] = -0.036860524056703*G0_0_1 - 0.104709984267274*G0_0_2 - 0.0368605240567035*G0_1_0 - 0.0965550010688885*G0_1_1 - 0.067849460210571*G0_1_2 - 0.104709984267274*G0_2_0 - 0.067849460210569*G0_2_1;
    A[2180] = -0.0929668084615965*G0_0_1 - 0.0929668084615979*G0_0_2 - 0.0929668084615994*G0_1_0 - 0.234863516113515*G0_1_1 - 0.21529155643739*G0_1_2 - 0.0929668084615979*G0_2_0 - 0.215291556437388*G0_2_1 - 0.234863516113512*G0_2_2;
    A[2181] = -0.0929668084615991*G0_0_1 - 0.0929668084615999*G0_0_2 - 0.0929668084615993*G0_1_0 - 0.0391439193522504*G0_1_1 - 0.0440369092712831*G0_1_2 - 0.0929668084616007*G0_2_0 - 0.0440369092712838*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[2182] = 0.10275278829966*G0_0_1 + 0.298472385060924*G0_0_2 + 0.102752788299665*G0_1_0 + 0.352295274170272*G0_1_1 + 0.102752788299662*G0_1_2 + 0.298472385060926*G0_2_0 + 0.102752788299663*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2183] = 0.298472385060923*G0_0_1 + 0.102752788299661*G0_0_2 + 0.298472385060925*G0_1_0 - 0.1859336169232*G0_1_1 + 0.102752788299662*G0_1_2 + 0.102752788299663*G0_2_0 + 0.102752788299661*G0_2_1 + 0.35229527417027*G0_2_2;
    A[2184] = 0.013742125287264*G0_0_0 + 0.013742125287264*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202554*G0_1_0 + 0.0189195610202554*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0189195610202554*G0_2_0 + 0.0189195610202554*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[2185] = -0.0137421252872641*G0_0_0;
    A[2186] = 0.00517743573299144*G0_0_1 + 0.0189195610202555*G0_1_1;
    A[2187] = -0.0143945239431349*G0_0_2 + 0.0335985307773502*G0_2_2;
    A[2188] = -0.0477555816097486*G0_0_1 - 0.01187365553685*G0_0_2 - 0.111886369481856*G0_1_1 - 0.0212681961813909*G0_1_2 - 0.0212681961813905*G0_2_1;
    A[2189] = 0.0823327103709051*G0_0_1 - 0.00182671623643823*G0_0_2 + 0.209419968534551*G0_1_1 + 0.0359471659384854*G0_1_2 + 0.0359471659384855*G0_2_1 - 0.0528442911255402*G0_2_2;
    A[2190] = -0.146659217839773*G0_0_1 - 0.0109602974186306*G0_0_2 - 0.271397840842284*G0_1_1 - 0.073329608919886*G0_1_2 - 0.0733296089198863*G0_2_1 + 0.121346149991982*G0_2_2;
    A[2191] = -0.12473862300251*G0_0_1 - 0.192588083213082*G0_0_2 - 0.0962940416065408*G0_1_2 - 0.0962940416065402*G0_2_1 - 0.257045070413124*G0_2_2;
    A[2192] = 0.0641307878721063*G0_0_0 + 0.00939454064453996*G0_0_2 + 0.00939454064454016*G0_2_0;
    A[2193] = -0.127087258163645*G0_0_0 - 0.0377738821749225*G0_0_2 - 0.0377738821749228*G0_2_0 - 0.01500516908503*G0_2_2;
    A[2194] = 0.12473862300251*G0_0_0 + 0.0623693115012546*G0_0_2 + 0.062369311501255*G0_2_0 + 0.047625101878574*G0_2_2;
    A[2195] = -0.124738623002511*G0_0_0 - 0.0962940416065403*G0_0_2 - 0.0962940416065405*G0_2_0 - 0.128522535206562*G0_2_2;
    A[2196] = 0.0187890812890804*G0_0_0 + 0.00939454064454005*G0_0_1 + 0.00939454064454056*G0_1_0;
    A[2197] = -0.0148746893538555*G0_0_0 - 0.0149399292194427*G0_0_1 - 0.0149399292194432*G0_1_0 - 0.015005169085031*G0_1_1;
    A[2198] = -0.00117431758056745*G0_0_0 - 0.00123955744615441*G0_0_1 - 0.00123955744615434*G0_1_0 - 0.00130479731174106*G0_1_1;
    A[2199] = -0.01187365553685*G0_0_0 - 0.0212681961813907*G0_0_1 - 0.0212681961813909*G0_1_0 - 0.0306627368259318*G0_1_1;
    A[2200] = -0.0641307878721067*G0_0_0 - 0.0641307878721067*G0_0_1 - 0.0547362472275662*G0_0_2 - 0.111886369481854*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0906181733004642*G0_1_2 - 0.090618173300464*G0_2_0 - 0.0906181733004642*G0_2_1 - 0.0693499771190738*G0_2_2;
    A[2201] = 0.127087258163646*G0_0_0 + 0.127087258163646*G0_0_1 + 0.0893133759887228*G0_0_2 + 0.20941996853455*G0_1_0 + 0.20941996853455*G0_1_1 + 0.173472802596065*G0_1_2 + 0.173472802596064*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320392*G0_2_2;
    A[2202] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0623693115012558*G0_0_2 - 0.271397840842283*G0_1_0 - 0.271397840842283*G0_1_1 - 0.198068231922397*G0_1_2 - 0.198068231922396*G0_2_0 - 0.198068231922396*G0_2_1 - 0.00339247301052848*G0_2_2;
    A[2203] = 0.12473862300251*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0284445813959703*G0_0_2 + 0.0962940416065412*G0_1_2 + 0.0962940416065403*G0_2_0 + 0.0962940416065403*G0_2_1 - 0.0644569872000425*G0_2_2;
    A[2204] = -0.0187890812890808*G0_0_0 - 0.00939454064454032*G0_0_1 - 0.0187890812890809*G0_0_2 - 0.0400572774704712*G0_1_0 - 0.018789081289081*G0_1_1 - 0.0400572774704712*G0_1_2 - 0.061325473651862*G0_2_0 - 0.0400572774704718*G0_2_1 - 0.061325473651862*G0_2_2;
    A[2205] = 0.0148746893538553*G0_0_0 - 6.52398655878204e-05*G0_0_1 + 0.0148746893538555*G0_0_2 + 0.013635131907701*G0_1_0 - 0.000130479731174503*G0_1_1 + 0.0136351319077011*G0_1_2 - 0.0254435475789637*G0_2_0 + 0.0136351319077014*G0_2_1 - 0.0254435475789636*G0_2_2;
    A[2206] = 0.0011743175805673*G0_0_0 - 6.5239865586366e-05*G0_0_1 + 0.00117431758056732*G0_0_2 - 0.0137656116388757*G0_1_0 - 0.000130479731173825*G0_1_1 - 0.0137656116388757*G0_1_2 - 0.0528442911255407*G0_2_0 - 0.0137656116388756*G0_2_1 - 0.0528442911255407*G0_2_2;
    A[2207] = 0.0118736555368499*G0_0_0 - 0.0093945406445411*G0_0_1 + 0.01187365553685*G0_0_2 + 0.0212681961813908*G0_1_0 - 0.0187890812890816*G0_1_1 + 0.0212681961813909*G0_1_2 + 0.0212681961813904*G0_2_1;
    A[2208] = -0.00939454064454031*G0_0_1 - 0.0093945406445403*G0_0_2 - 0.00939454064454048*G0_1_0 - 0.0306627368259307*G0_1_1 - 0.0306627368259307*G0_1_2 - 0.00939454064454054*G0_2_0 - 0.0306627368259309*G0_2_1 - 0.0306627368259308*G0_2_2;
    A[2209] = -6.5239865587228e-05*G0_0_1 - 6.52398655872339e-05*G0_0_2 - 6.52398655872432e-05*G0_1_0 - 0.00130479731174199*G0_1_1 - 0.00130479731174198*G0_1_2 - 6.52398655874747e-05*G0_2_0 - 0.00130479731174173*G0_2_1 - 0.00130479731174176*G0_2_2;
    A[2210] = -6.52398655868953e-05*G0_0_1 - 6.52398655869124e-05*G0_0_2 - 6.52398655868854e-05*G0_1_0 - 0.01500516908503*G0_1_1 - 0.01500516908503*G0_1_2 - 6.52398655867815e-05*G0_2_0 - 0.0150051690850302*G0_2_1 - 0.0150051690850302*G0_2_2;
    A[2211] = -0.00939454064454003*G0_0_1 - 0.00939454064453998*G0_0_2 - 0.00939454064454097*G0_1_0 - 0.00939454064454066*G0_2_0;
    A[2212] = -0.105688582251081*G0_0_0 - 0.0528442911255411*G0_0_1 - 0.030010338170061*G0_0_2 - 0.0528442911255404*G0_1_0 - 0.0150051690850304*G0_1_2 - 0.0300103381700601*G0_2_0 - 0.0150051690850304*G0_2_1;
    A[2213] = 0.0137003717732875*G0_0_0 + 0.0198981590040609*G0_0_1 - 0.0127217737894818*G0_0_2 + 0.0198981590040625*G0_1_0 + 0.0260959462348355*G0_1_1 - 0.0120693751336108*G0_1_2 - 0.0127217737894829*G0_2_0 - 0.0120693751336124*G0_2_1;
    A[2214] = -0.053496689781411*G0_0_0 - 0.000652398655869825*G0_0_1 - 0.0528442911255402*G0_0_2 - 0.000652398655871653*G0_1_0 + 0.0521918924696703*G0_1_1 - 0.0378391220405113*G0_1_2 - 0.0528442911255396*G0_2_0 - 0.0378391220405097*G0_2_1;
    A[2215] = 0.209419968534549*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0737210481134084*G0_0_2 + 0.104709984267274*G0_1_0 + 0.0368605240567043*G0_1_2 + 0.0737210481134081*G0_2_0 + 0.0368605240567046*G0_2_1 - 0.0228339529554807*G0_2_2;
    A[2216] = 0.0234863516113513*G0_0_0 - 0.0812236326559242*G0_0_1 + 0.0459941052388958*G0_0_2 - 0.081223632655923*G0_1_0 - 0.185933616923199*G0_1_1 + 0.00913358118219332*G0_1_2 + 0.0459941052388973*G0_2_0 + 0.00913358118219296*G0_2_1 - 0.0880738185425674*G0_2_2;
    A[2217] = -0.542795681684566*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842282*G0_0_2 - 0.271397840842283*G0_1_0 - 0.135698920421142*G0_1_2 - 0.271397840842283*G0_2_0 - 0.135698920421141*G0_2_1 + 0.0521918924696702*G0_2_2;
    A[2218] = 0.105688582251083*G0_0_0 + 0.0528442911255412*G0_0_1 + 0.0756782440810216*G0_0_2 + 0.10503618359521*G0_1_0 + 0.105688582251081*G0_1_1 + 0.12787013655069*G0_1_2 + 0.180062029020362*G0_2_0 + 0.127870136550692*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2219] = -0.0137003717732892*G0_0_0 + 0.00619778723077144*G0_0_1 - 0.0264221455627703*G0_0_2 + 0.00619778723077355*G0_1_0 + 0.0123955744615462*G0_1_1 + 0.00619778723077412*G0_1_2 + 0.0714376528178602*G0_2_0 + 0.00619778723077174*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2220] = 0.0534966897814111*G0_0_0 + 0.0528442911255425*G0_0_1 + 0.000652398655870931*G0_0_2 + 0.000652398655871834*G0_1_0 + 0.105688582251083*G0_1_1 - 0.0221815542996096*G0_1_2 + 0.0528442911255397*G0_2_0 - 0.0221815542996091*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2221] = -0.209419968534548*G0_0_0 - 0.104709984267274*G0_0_1 - 0.135698920421142*G0_0_2 - 0.290643601190472*G0_1_0 - 0.209419968534548*G0_1_1 - 0.262916658315961*G0_1_2 - 0.390134396210781*G0_2_0 - 0.262916658315961*G0_2_1 - 0.584549195660301*G0_2_2;
    A[2222] = -0.0234863516113516*G0_0_0 - 0.104709984267277*G0_0_1 + 0.022507753627545*G0_0_2 + 0.0812236326559228*G0_1_0 - 0.209419968534553*G0_1_1 + 0.0534966897814106*G0_1_2 - 0.0459941052388975*G0_2_0 + 0.0534966897814113*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2223] = 0.542795681684566*G0_0_0 + 0.271397840842284*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684567*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842283*G0_2_1 + 0.647179466623905*G0_2_2;
    A[2224] = 0.0528442911255422*G0_0_1 + 0.0378391220405109*G0_0_2 + 0.0528442911255399*G0_1_0 + 0.0521918924696691*G0_1_1 + 0.0900310145101796*G0_1_2 + 0.0378391220405109*G0_2_0 + 0.090031014510181*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2225] = 0.00619778723077263*G0_0_1 + 0.00554538857490249*G0_0_2 + 0.00619778723077321*G0_1_0 + 0.0260959462348346*G0_1_1 + 0.0381653213684463*G0_1_2 + 0.00554538857490185*G0_2_0 + 0.0381653213684455*G0_2_1 + 0.0502346965020575*G0_2_2;
    A[2226] = 0.0528442911255412*G0_0_1 + 0.0378391220405095*G0_0_2 + 0.0528442911255417*G0_1_0 + 0.0150051690850308*G0_1_2 + 0.0378391220405112*G0_2_0 + 0.0150051690850306*G0_2_1 + 0.0300103381700606*G0_2_2;
    A[2227] = -0.104709984267274*G0_0_1 - 0.0678494602105716*G0_0_2 - 0.104709984267274*G0_1_0 - 0.185933616923198*G0_1_1 - 0.19506719810539*G0_1_2 - 0.0678494602105706*G0_2_0 - 0.19506719810539*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2228] = -0.104709984267275*G0_0_1 - 0.0678494602105696*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0368605240567045*G0_1_2 - 0.0678494602105712*G0_2_0 - 0.036860524056705*G0_2_1 - 0.0965550010688896*G0_2_2;
    A[2229] = 0.271397840842283*G0_0_1 + 0.135698920421141*G0_0_2 + 0.271397840842284*G0_1_0 + 0.135698920421141*G0_1_2 + 0.135698920421142*G0_2_0 + 0.135698920421142*G0_2_1 + 0.323589733311953*G0_2_2;
    A[2230] = 0.0150051690850302*G0_0_1 + 0.0300103381700615*G0_0_2 + 0.0150051690850297*G0_1_0 + 0.0300103381700598*G0_1_1 - 0.00782878387045045*G0_1_2 + 0.0300103381700605*G0_2_0 - 0.00782878387045019*G0_2_1 - 0.0456679059109595*G0_2_2;
    A[2231] = 0.000652398655871693*G0_0_1 + 0.00130479731174182*G0_0_2 + 0.000652398655870636*G0_1_0 + 0.00130479731174202*G0_1_1 - 0.01076457782187*G0_1_2 + 0.00130479731174128*G0_2_0 - 0.0107645778218693*G0_2_1 - 0.0228339529554815*G0_2_2;
    A[2232] = 0.0150051690850296*G0_0_1 + 0.0300103381700613*G0_0_2 + 0.0150051690850305*G0_1_0 + 0.0300103381700606*G0_1_1 + 0.0150051690850304*G0_1_2 + 0.0300103381700603*G0_2_0 + 0.0150051690850305*G0_2_1;
    A[2233] = 0.000652398655869475*G0_0_1 + 0.0127217737894811*G0_0_2 + 0.000652398655870843*G0_1_0 + 0.00130479731174135*G0_1_1 + 0.000652398655870698*G0_1_2 + 0.0127217737894809*G0_2_0 + 0.000652398655870033*G0_2_1 - 0.0880738185425685*G0_2_2;
    A[2234] = 0.000652398655870107*G0_0_1 + 0.0127217737894814*G0_0_2 + 0.000652398655871203*G0_1_0 + 0.00130479731174134*G0_1_1 + 0.0120693751336109*G0_1_2 + 0.0127217737894832*G0_2_0 + 0.0120693751336115*G0_2_1;
    A[2235] = 0.0150051690850313*G0_0_1 + 0.0528442911255403*G0_0_2 + 0.0150051690850301*G0_1_0 + 0.0300103381700611*G0_1_1 + 0.0378391220405113*G0_1_2 + 0.0528442911255406*G0_2_0 + 0.0378391220405109*G0_2_1;
    A[2236] = -0.026095946234836*G0_0_1 - 0.0750258454251498*G0_0_2 - 0.0260959462348345*G0_1_0 - 0.0521918924696675*G0_1_1 - 0.0424059126316045*G0_1_2 - 0.0750258454251523*G0_2_0 - 0.0424059126316074*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2237] = -0.0260959462348334*G0_0_1 - 0.0750258454251508*G0_0_2 - 0.0260959462348355*G0_1_0 - 0.0521918924696696*G0_1_1 - 0.0260959462348345*G0_1_2 - 0.0750258454251496*G0_2_0 - 0.0260959462348331*G0_2_1 + 0.0456679059109621*G0_2_2;
    A[2238] = -0.0260959462348367*G0_0_1 - 0.0587158790283764*G0_0_2 - 0.0260959462348361*G0_1_0 - 0.0521918924696721*G0_1_1 - 0.00978597983806376*G0_1_2 - 0.0587158790283776*G0_2_0 - 0.00978597983806401*G0_2_1 + 0.176147637085135*G0_2_2;
    A[2239] = 0.185933616923198*G0_0_1 + 0.316413348097371*G0_0_2 + 0.185933616923197*G0_1_0 + 0.371867233846396*G0_1_1 + 0.185933616923197*G0_1_2 + 0.316413348097373*G0_2_0 + 0.185933616923197*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2240] = 0.03359853077735*G0_0_0 + 0.03359853077735*G0_0_1 + 0.03359853077735*G0_0_2 + 0.0479930547204848*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.0479930547204849*G0_1_2 + 0.0335985307773499*G0_2_0 + 0.03359853077735*G0_2_1 + 0.03359853077735*G0_2_2;
    A[2241] = 0.0189195610202555*G0_0_0 + 0.0051774357329915*G0_1_0;
    A[2242] = -0.0137421252872641*G0_1_1;
    A[2243] = 0.00517743573299124*G0_1_2 + 0.0189195610202551*G0_2_2;
    A[2244] = 0.0187890812890812*G0_1_1 + 0.00939454064454046*G0_1_2 + 0.00939454064454053*G0_2_1;
    A[2245] = -0.0148746893538562*G0_1_1 - 0.014939929219443*G0_1_2 - 0.0149399292194432*G0_2_1 - 0.0150051690850294*G0_2_2;
    A[2246] = -0.00117431758056745*G0_1_1 - 0.00123955744615476*G0_1_2 - 0.00123955744615409*G0_2_1 - 0.00130479731174235*G0_2_2;
    A[2247] = -0.0118736555368504*G0_1_1 - 0.021268196181391*G0_1_2 - 0.0212681961813909*G0_2_1 - 0.030662736825931*G0_2_2;
    A[2248] = -0.0613254736518617*G0_0_0 - 0.0212681961813898*G0_0_2 - 0.0425363923627808*G0_1_0 - 0.0118736555368498*G0_1_2 - 0.0212681961813904*G0_2_0;
    A[2249] = -0.0254435475789644*G0_0_0 - 0.039078679486666*G0_0_2 - 0.0403182369328204*G0_1_0 - 0.0540186087061084*G0_1_2 - 0.0390786794866652*G0_2_0 - 0.0528442911255409*G0_2_2;
    A[2250] = -0.0528442911255406*G0_0_0 - 0.0390786794866646*G0_0_2 - 0.0540186087061082*G0_1_0 - 0.0403182369328199*G0_1_2 - 0.0390786794866654*G0_2_0 - 0.0254435475789635*G0_2_2;
    A[2251] = -0.0212681961813913*G0_0_2 - 0.0118736555368504*G0_1_0 - 0.0425363923627817*G0_1_2 - 0.0212681961813909*G0_2_0 - 0.0613254736518626*G0_2_2;
    A[2252] = -0.0306627368259308*G0_0_0 - 0.0212681961813898*G0_0_1 - 0.0212681961813906*G0_1_0 - 0.0118736555368497*G0_1_1;
    A[2253] = -0.00130479731174176*G0_0_0 - 0.00123955744615517*G0_0_1 - 0.00123955744615451*G0_1_0 - 0.00117431758056765*G0_1_1;
    A[2254] = -0.0150051690850303*G0_0_0 - 0.0149399292194429*G0_0_1 - 0.0149399292194434*G0_1_0 - 0.0148746893538562*G0_1_1;
    A[2255] = 0.00939454064454058*G0_0_1 + 0.00939454064454057*G0_1_0 + 0.018789081289081*G0_1_1;
    A[2256] = -0.257045070413124*G0_0_0 - 0.257045070413124*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0644569872000421*G0_1_0 - 0.0644569872000421*G0_1_1 - 0.0929015685960123*G0_1_2 - 0.160751028806583*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000422*G0_2_2;
    A[2257] = 0.121346149991983*G0_0_0 + 0.121346149991983*G0_0_1 + 0.194675758911869*G0_0_2 + 0.132306447410614*G0_1_0 + 0.132306447410614*G0_1_1 + 0.0589768384907271*G0_1_2 + 0.194675758911869*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052828*G0_2_2;
    A[2258] = -0.0528442911255406*G0_0_0 - 0.0528442911255406*G0_0_1 - 0.0887914570640261*G0_0_2 - 0.051017574889102*G0_1_0 - 0.051017574889102*G0_1_1 - 0.00463203045668274*G0_1_2 - 0.0887914570640257*G0_2_0 - 0.0887914570640257*G0_2_1 + 0.0846813455320391*G0_2_2;
    A[2259] = 0.0212681961813903*G0_0_2 + 0.0118736555368502*G0_1_0 + 0.0118736555368502*G0_1_1 - 0.0146137298915081*G0_1_2 + 0.0212681961813907*G0_2_0 + 0.0212681961813908*G0_2_1 - 0.069349977119074*G0_2_2;
    A[2260] = -0.128522535206561*G0_0_0 - 0.032228493600021*G0_0_1 - 0.128522535206561*G0_0_2 - 0.0322284936000209*G0_1_0 - 0.060673074995991*G0_1_1 - 0.0322284936000208*G0_1_2 - 0.128522535206562*G0_2_0 - 0.0322284936000211*G0_2_1 - 0.128522535206562*G0_2_2;
    A[2261] = 0.0476251018785732*G0_0_0 - 0.0147442096226819*G0_0_1 + 0.0476251018785733*G0_0_2 - 0.014744209622682*G0_1_0 + 0.0476251018785732*G0_1_1 - 0.0147442096226821*G0_1_2 + 0.0476251018785732*G0_2_0 - 0.0147442096226818*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[2262] = -0.0150051690850298*G0_0_0 + 0.0227687130898933*G0_0_1 - 0.0150051690850298*G0_0_2 + 0.0227687130898936*G0_1_0 - 0.0665446628988286*G0_1_1 + 0.0227687130898936*G0_1_2 - 0.01500516908503*G0_2_0 + 0.0227687130898936*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[2263] = -0.00939454064454039*G0_0_1 - 0.00939454064454056*G0_1_0 + 0.0453417065830254*G0_1_1 - 0.00939454064454052*G0_1_2 - 0.00939454064454075*G0_2_1;
    A[2264] = -0.0644569872000422*G0_0_0 - 0.160751028806583*G0_0_1 - 0.160751028806582*G0_0_2 - 0.092901568596012*G0_1_0 - 0.0644569872000418*G0_1_1 - 0.0644569872000418*G0_1_2 - 0.160751028806583*G0_2_0 - 0.257045070413123*G0_2_1 - 0.257045070413123*G0_2_2;
    A[2265] = -0.00339247301052865*G0_0_0 + 0.194675758911868*G0_0_1 + 0.194675758911868*G0_0_2 + 0.0589768384907262*G0_1_0 + 0.132306447410613*G0_1_1 + 0.132306447410613*G0_1_2 + 0.194675758911868*G0_2_0 + 0.121346149991982*G0_2_1 + 0.121346149991982*G0_2_2;
    A[2266] = 0.0846813455320395*G0_0_0 - 0.0887914570640248*G0_0_1 - 0.0887914570640247*G0_0_2 - 0.00463203045668213*G0_1_0 - 0.0510175748891013*G0_1_1 - 0.0510175748891012*G0_1_2 - 0.088791457064025*G0_2_0 - 0.0528442911255401*G0_2_1 - 0.05284429112554*G0_2_2;
    A[2267] = -0.0693499771190741*G0_0_0 + 0.0212681961813896*G0_0_1 + 0.0212681961813897*G0_0_2 - 0.0146137298915082*G0_1_0 + 0.0118736555368496*G0_1_1 + 0.0118736555368496*G0_1_2 + 0.02126819618139*G0_2_0;
    A[2268] = -0.0456679059109615*G0_0_0 - 0.0756782440810212*G0_0_1 - 0.0378391220405102*G0_0_2 - 0.0756782440810211*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255407*G0_1_2 - 0.0378391220405106*G0_2_0 - 0.0528442911255407*G0_2_1;
    A[2269] = -0.0228339529554808*G0_0_0 - 0.0241387502672228*G0_0_1 - 0.0120693751336118*G0_0_2 - 0.024138750267223*G0_1_0 - 0.0254435475789643*G0_1_1 - 0.0127217737894824*G0_1_2 - 0.0120693751336113*G0_2_0 - 0.0127217737894829*G0_2_1;
    A[2270] = -0.0300103381700604*G0_0_1 - 0.0150051690850302*G0_0_2 - 0.0300103381700609*G0_1_0 - 0.0600206763401215*G0_1_1 - 0.0300103381700614*G0_1_2 - 0.0150051690850302*G0_2_0 - 0.0300103381700598*G0_2_1;
    A[2271] = -0.0880738185425677*G0_0_0 - 0.10079559233205*G0_0_1 - 0.0887262171984389*G0_0_2 - 0.100795592332049*G0_1_0 - 0.113517366121532*G0_1_1 - 0.10079559233205*G0_1_2 - 0.0887262171984389*G0_2_0 - 0.10079559233205*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2272] = -0.0127217737894813*G0_0_1 - 0.0120693751336105*G0_0_2 - 0.0127217737894816*G0_1_0 - 0.0254435475789631*G0_1_1 - 0.0241387502672221*G0_1_2 - 0.0120693751336102*G0_2_0 - 0.024138750267221*G0_2_1 - 0.022833952955479*G0_2_2;
    A[2273] = -0.0528442911255419*G0_0_1 - 0.0378391220405112*G0_0_2 - 0.0528442911255423*G0_1_0 - 0.105688582251083*G0_1_1 - 0.075678244081022*G0_1_2 - 0.037839122040511*G0_2_0 - 0.0756782440810226*G0_2_1 - 0.0456679059109619*G0_2_2;
    A[2274] = 0.0521918924696705*G0_0_0 + 0.323589733311952*G0_0_1 + 0.187890812890811*G0_0_2 + 0.323589733311955*G0_1_0 + 0.05219189246967*G0_1_1 + 0.187890812890812*G0_1_2 + 0.187890812890812*G0_2_0 + 0.187890812890811*G0_2_1 + 0.323589733311953*G0_2_2;
    A[2275] = -0.0228339529554803*G0_0_0 - 0.0965550010688884*G0_0_1 - 0.0596944770121841*G0_0_2 - 0.0965550010688897*G0_1_0 + 0.0391439193522517*G0_1_1 - 0.028705540858318*G0_1_2 - 0.0596944770121856*G0_2_0 - 0.028705540858319*G0_2_1 - 0.0965550010688885*G0_2_2;
    A[2276] = 0.0300103381700593*G0_0_1 + 0.0150051690850297*G0_0_2 + 0.030010338170061*G0_1_0 - 0.0456679059109608*G0_1_1 - 0.00782878387045053*G0_1_2 + 0.0150051690850302*G0_2_0 - 0.00782878387044982*G0_2_1 + 0.0300103381700597*G0_2_2;
    A[2277] = -0.088073818542568*G0_0_0 - 0.134067923781465*G0_0_1 - 0.0972073997247605*G0_0_2 - 0.134067923781464*G0_1_0 - 0.156575677409009*G0_1_1 - 0.224425137619581*G0_1_2 - 0.0972073997247581*G0_2_0 - 0.22442513761958*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2278] = 0.012721773789483*G0_0_1 + 0.0120693751336111*G0_0_2 + 0.0127217737894816*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.0446893079271549*G0_1_2 + 0.0120693751336102*G0_2_0 + 0.0446893079271544*G0_2_1 + 0.0502346965020575*G0_2_2;
    A[2279] = 0.0528442911255399*G0_0_1 + 0.0378391220405109*G0_0_2 + 0.0528442911255422*G0_1_0 + 0.0521918924696691*G0_1_1 + 0.090031014510181*G0_1_2 + 0.0378391220405109*G0_2_0 + 0.0900310145101796*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2280] = 0.647179466623905*G0_0_0 + 0.375781625781623*G0_0_1 + 0.375781625781623*G0_0_2 + 0.375781625781623*G0_1_0 + 0.647179466623908*G0_1_1 + 0.375781625781624*G0_1_2 + 0.375781625781623*G0_2_0 + 0.375781625781624*G0_2_1 + 0.647179466623906*G0_2_2;
    A[2281] = -0.5845491956603*G0_0_0 - 0.194414799449519*G0_0_1 - 0.321632537344339*G0_0_2 - 0.44885027523916*G0_1_0 - 0.268135847562928*G0_1_1 - 0.290643601190474*G0_1_2 - 0.32163253734434*G0_2_0 - 0.222141742324032*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2282] = 0.25574027310138*G0_0_0 + 0.0756782440810214*G0_0_1 + 0.127870136550691*G0_0_2 + 0.18006202902036*G0_1_0 + 0.105688582251082*G0_1_1 + 0.10503618359521*G0_1_2 + 0.127870136550691*G0_2_0 + 0.0528442911255411*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2283] = -0.26813584756293*G0_0_0 - 0.222141742324033*G0_0_1 - 0.321632537344342*G0_0_2 - 0.290643601190475*G0_1_0 - 0.26813584756293*G0_1_1 - 0.448850275239163*G0_1_2 - 0.321632537344341*G0_2_0 - 0.194414799449519*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2284] = 0.172233245149912*G0_0_0 + 0.10079559233205*G0_0_1 + 0.166035457919137*G0_0_2 + 0.198655390712682*G0_1_0 + 0.113517366121531*G0_1_1 + 0.198655390712681*G0_1_2 + 0.166035457919137*G0_2_0 + 0.100795592332049*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2285] = 0.105688582251082*G0_0_0 + 0.0528442911255422*G0_0_1 + 0.127870136550693*G0_0_2 + 0.105036183595211*G0_1_0 + 0.105688582251084*G0_1_1 + 0.180062029020363*G0_1_2 + 0.127870136550691*G0_2_0 + 0.0756782440810228*G0_2_1 + 0.255740273101383*G0_2_2;
    A[2286] = 0.323589733311952*G0_0_0 + 0.187890812890811*G0_0_1 + 0.187890812890811*G0_0_2 + 0.187890812890812*G0_1_0 + 0.05219189246967*G0_1_1 + 0.323589733311955*G0_1_2 + 0.187890812890811*G0_2_0 + 0.323589733311952*G0_2_1 + 0.0521918924696703*G0_2_2;
    A[2287] = -0.29227459783015*G0_0_0 - 0.224425137619579*G0_0_1 - 0.0972073997247592*G0_0_2 - 0.22442513761958*G0_1_0 - 0.156575677409009*G0_1_1 - 0.134067923781463*G0_1_2 - 0.09720739972476*G0_2_0 - 0.134067923781464*G0_2_1 - 0.0880738185425674*G0_2_2;
    A[2288] = 0.12787013655069*G0_0_0 + 0.0900310145101802*G0_0_1 + 0.0378391220405105*G0_0_2 + 0.0900310145101803*G0_1_0 + 0.0521918924696694*G0_1_1 + 0.0528442911255413*G0_1_2 + 0.0378391220405104*G0_2_0 + 0.0528442911255408*G0_2_1;
    A[2289] = -0.0965550010688884*G0_0_0 - 0.0287055408583175*G0_0_1 - 0.0596944770121844*G0_0_2 - 0.0287055408583178*G0_1_0 + 0.0391439193522518*G0_1_1 - 0.0965550010688892*G0_1_2 - 0.0596944770121847*G0_2_0 - 0.0965550010688894*G0_2_1 - 0.0228339529554816*G0_2_2;
    A[2290] = 0.0502346965020568*G0_0_0 + 0.0446893079271546*G0_0_1 + 0.0120693751336117*G0_0_2 + 0.0446893079271542*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0127217737894826*G0_1_2 + 0.0120693751336115*G0_2_0 + 0.0127217737894816*G0_2_1;
    A[2291] = 0.0300103381700602*G0_0_0 - 0.00782878387045084*G0_0_1 + 0.0150051690850306*G0_0_2 - 0.0078287838704501*G0_1_0 - 0.0456679059109603*G0_1_1 + 0.0300103381700617*G0_1_2 + 0.0150051690850302*G0_2_0 + 0.0300103381700612*G0_2_1;
    A[2292] = -0.150051690850301*G0_0_0 - 0.466465038947674*G0_0_1 - 0.335985307773498*G0_0_2 - 0.466465038947673*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947673*G0_1_2 - 0.335985307773499*G0_2_0 - 0.466465038947674*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2293] = 0.143527704291592*G0_0_0 + 0.218553549716742*G0_0_1 + 0.185933616923198*G0_0_2 + 0.21855354971674*G0_1_0 + 0.293579395141892*G0_1_1 + 0.234863516113512*G0_1_2 + 0.185933616923198*G0_2_0 + 0.234863516113515*G0_2_1 + 0.176147637085135*G0_2_2;
    A[2294] = 0.0456679059109608*G0_0_0 + 0.120693751336111*G0_0_1 + 0.0717638521457945*G0_0_2 + 0.120693751336113*G0_1_0 + 0.195719596761261*G0_1_1 + 0.120693751336111*G0_1_2 + 0.0717638521457968*G0_2_0 + 0.12069375133611*G0_2_1 + 0.0456679059109599*G0_2_2;
    A[2295] = 0.176147637085136*G0_0_0 + 0.234863516113514*G0_0_1 + 0.185933616923199*G0_0_2 + 0.234863516113513*G0_1_0 + 0.293579395141893*G0_1_1 + 0.218553549716741*G0_1_2 + 0.185933616923197*G0_2_0 + 0.218553549716741*G0_2_1 + 0.143527704291592*G0_2_2;
    A[2296] = -0.0298472385060921*G0_0_0 - 0.0298472385060921*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.034571909571909*G0_1_0 - 0.034571909571909*G0_1_1 - 0.0345719095719091*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060921*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[2297] = -0.0298472385060926*G0_0_0 + 0.00472467106581666*G0_1_0;
    A[2298] = -0.00946499969937482*G0_1_1;
    A[2299] = 0.00227817610630128*G0_1_2 + 0.0117431758056757*G0_2_2;
    A[2300] = 0.00658922642429604*G0_1_1 + 0.00329461321214786*G0_1_2 + 0.00329461321214782*G0_2_1;
    A[2301] = 0.00450155072550945*G0_1_1 + 0.00192457603481957*G0_1_2 + 0.00192457603481913*G0_2_1 - 0.000652398655870257*G0_2_2;
    A[2302] = -0.0105036183595213*G0_1_1 - 0.013080593050211*G0_1_2 - 0.013080593050211*G0_2_1 - 0.0156575677409013*G0_2_2;
    A[2303] = -0.00254435475789568*G0_1_1 - 0.00583896797004383*G0_1_2 - 0.00583896797004432*G0_2_1 - 0.0091335811821921*G0_2_2;
    A[2304] = 0.128522535206561*G0_0_0 + 0.042275432900432*G0_0_2 + 0.084550865800864*G0_1_0 + 0.0429604514890962*G0_1_2 + 0.0422754329004323*G0_2_0;
    A[2305] = 0.0371867233846397*G0_0_0 + 0.0743408268364858*G0_0_2 + 0.0808321934624013*G0_1_0 + 0.108232937008978*G0_1_2 + 0.0743408268364854*G0_2_0 + 0.104709984267275*G0_2_2;
    A[2306] = -0.00619778723077332*G0_0_0 + 0.0188869410874618*G0_0_2 + 0.0093945406445398*G0_1_0 + 0.0377738821749229*G0_1_2 + 0.018886941087462*G0_2_0 + 0.056758683060766*G0_2_2;
    A[2307] = -0.00583896797004399*G0_0_2 - 0.00254435475789572*G0_1_0 - 0.0142875305635717*G0_1_2 - 0.00583896797004435*G0_2_0 - 0.0435476102793808*G0_2_2;
    A[2308] = 0.0642612676032806*G0_0_0 + 0.0422754329004321*G0_0_1 + 0.0422754329004318*G0_1_0 + 0.0429604514890962*G0_1_1;
    A[2309] = 0.0088073818542569*G0_0_0 + 0.00649136662591535*G0_0_1 + 0.00649136662591558*G0_1_0 + 0.00352295274170345*G0_1_1;
    A[2310] = -0.000652398655870866*G0_0_0 - 0.00949240044292123*G0_0_1 - 0.00949240044292136*G0_1_0 - 0.0189848008858431*G0_1_1;
    A[2311] = 0.00329461321214822*G0_0_1 + 0.00329461321214777*G0_1_0 + 0.0292600797158089*G0_1_1;
    A[2312] = 0.128522535206562*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061293*G0_0_2 + 0.0439716694056971*G0_1_0 + 0.0439716694056971*G0_1_1 + 0.0446566879943612*G0_1_2 + 0.0862471023061291*G0_2_0 + 0.0862471023061291*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[2313] = 0.0371867233846393*G0_0_0 + 0.0371867233846393*G0_0_1 - 0.0371541034518462*G0_0_2 - 0.0436454700777619*G0_1_0 - 0.0436454700777617*G0_1_1 - 0.0097533599052697*G0_1_2 - 0.0371541034518463*G0_2_0 - 0.0371541034518461*G0_2_1 - 0.00678494602105704*G0_2_2;
    A[2314] = -0.00619778723077307*G0_0_0 - 0.00619778723077314*G0_0_1 - 0.0250847283182353*G0_0_2 - 0.0155923278753132*G0_1_0 - 0.0155923278753133*G0_1_1 + 0.00329461321214798*G0_1_2 - 0.0250847283182351*G0_2_0 - 0.0250847283182352*G0_2_1 + 0.0127870136550688*G0_2_2;
    A[2315] = 0.00583896797004454*G0_0_2 + 0.00254435475789586*G0_1_0 + 0.00254435475789589*G0_1_1 - 0.00590420783563157*G0_1_2 + 0.00583896797004444*G0_2_0 + 0.00583896797004445*G0_2_1 - 0.031869674339292*G0_2_2;
    A[2316] = 0.0642612676032807*G0_0_0 + 0.021985834702848*G0_0_1 + 0.0642612676032807*G0_0_2 + 0.0219858347028488*G0_1_0 + 0.0226708532915128*G0_1_1 + 0.0219858347028488*G0_1_2 + 0.0642612676032806*G0_2_0 + 0.0219858347028481*G0_2_1 + 0.0642612676032806*G0_2_2;
    A[2317] = 0.00880738185425651*G0_0_0 + 0.00231601522834189*G0_0_1 + 0.00880738185425647*G0_0_2 + 0.00231601522834095*G0_1_0 - 0.000652398655871101*G0_1_1 + 0.00231601522834089*G0_1_2 + 0.0088073818542568*G0_2_0 + 0.00231601522834173*G0_2_1 + 0.00880738185425683*G0_2_2;
    A[2318] = -0.000652398655870913*G0_0_0 + 0.00884000178705024*G0_0_1 - 0.000652398655870866*G0_0_2 + 0.00884000178705045*G0_1_0 - 0.000652398655870832*G0_1_1 + 0.00884000178705038*G0_1_2 - 0.00065239865587136*G0_2_0 + 0.0088400017870502*G0_2_1 - 0.000652398655871416*G0_2_2;
    A[2319] = -0.00329461321214795*G0_0_1 - 0.00329461321214779*G0_1_0 + 0.022670853291513*G0_1_1 - 0.00329461321214773*G0_1_2 - 0.00329461321214771*G0_2_1;
    A[2320] = 0.092086070276173*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176606*G0_0_2 + 0.128000616281864*G0_1_0 + 0.1273155976932*G0_1_1 + 0.1273155976932*G0_1_2 + 0.134361503176606*G0_2_0 + 0.176636936077038*G0_2_1 + 0.176636936077038*G0_2_2;
    A[2321] = -0.0622388317700802*G0_0_0 - 0.250814663249556*G0_0_1 - 0.250814663249555*G0_0_2 - 0.160783648739375*G0_1_0 - 0.166948816037355*G0_1_1 - 0.166948816037355*G0_1_2 - 0.250814663249556*G0_2_0 - 0.334680510461756*G0_2_1 - 0.334680510461756*G0_2_2;
    A[2322] = -0.0622388317700825*G0_0_0 + 0.188575831479474*G0_0_1 + 0.188575831479474*G0_0_2 + 0.0985448169692943*G0_1_0 + 0.182410664181495*G0_1_1 + 0.182410664181495*G0_1_2 + 0.188575831479474*G0_2_0 + 0.104709984267273*G0_2_1 + 0.104709984267273*G0_2_2;
    A[2323] = 0.0920860702761744*G0_0_0 - 0.0422754329004318*G0_0_1 - 0.0422754329004318*G0_0_2 - 0.0359145460056911*G0_1_0 - 0.0429604514890959*G0_1_1 - 0.042960451489096*G0_1_2 - 0.0422754329004318*G0_2_0;
    A[2324] = 0.0391439193522527*G0_0_0 + 0.135698920421141*G0_0_1 + 0.0678494602105705*G0_0_2 + 0.135698920421141*G0_1_0 + 0.209419968534548*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0678494602105698*G0_2_0 + 0.104709984267274*G0_2_1;
    A[2325] = -0.00489298991903153*G0_0_0 + 0.0283793415303836*G0_0_1 + 0.0141896707651916*G0_0_2 + 0.0283793415303826*G0_1_0 + 0.0567586830607667*G0_1_1 + 0.0283793415303834*G0_1_2 + 0.0141896707651909*G0_2_0 + 0.0283793415303826*G0_2_1;
    A[2326] = -0.00130479731174136*G0_0_1 - 0.000652398655869972*G0_0_2 - 0.00130479731174113*G0_1_0 - 0.0254435475789642*G0_1_1 - 0.0127217737894824*G0_1_2 - 0.000652398655870776*G0_2_0 - 0.0127217737894822*G0_2_1;
    A[2327] = 0.019571959676126*G0_0_0 + 0.0748627457611823*G0_0_1 + 0.0606730749959912*G0_0_2 + 0.0748627457611813*G0_1_0 + 0.149725491522364*G0_1_1 + 0.121346149991981*G0_1_2 + 0.060673074995991*G0_2_0 + 0.121346149991982*G0_2_1 + 0.0929668084615982*G0_2_2;
    A[2328] = -0.0181040627004161*G0_0_1 - 0.0102752788299661*G0_0_2 - 0.018104062700416*G0_1_0 - 0.016636165724707*G0_1_1 + 0.00146789697570982*G0_1_2 - 0.0102752788299661*G0_2_0 + 0.00146789697570966*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2329] = -0.00619778723077252*G0_0_1 - 0.00554538857490158*G0_0_2 - 0.00619778723077247*G0_1_0 + 0.0137003717732885*G0_1_1 + 0.0264221455627704*G0_1_2 - 0.0055453885749023*G0_2_0 + 0.0264221455627707*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[2330] = 0.0391439193522511*G0_0_0 - 0.0965550010688891*G0_0_1 - 0.0287055408583184*G0_0_2 - 0.0965550010688895*G0_1_0 - 0.0228339529554814*G0_1_1 - 0.0596944770121856*G0_1_2 - 0.0287055408583187*G0_2_0 - 0.0596944770121851*G0_2_1 - 0.0965550010688892*G0_2_2;
    A[2331] = -0.00489298991903175*G0_0_0 - 0.0332723314494148*G0_0_1 - 0.0190826606842232*G0_0_2 - 0.0332723314494145*G0_1_0 - 0.00489298991903104*G0_1_1 - 0.0190826606842219*G0_1_2 - 0.0190826606842235*G0_2_0 - 0.0190826606842231*G0_2_1 - 0.0332723314494141*G0_2_2;
    A[2332] = 0.0013047973117421*G0_0_1 + 0.000652398655871094*G0_0_2 + 0.00130479731174115*G0_1_0 - 0.0228339529554806*G0_1_1 - 0.0107645778218701*G0_1_2 + 0.000652398655870743*G0_2_0 - 0.0107645778218689*G0_2_1 + 0.0013047973117417*G0_2_2;
    A[2333] = 0.0195719596761273*G0_0_0 - 0.0552907860850559*G0_0_1 - 0.0411011153198643*G0_0_2 - 0.0552907860850555*G0_1_0 + 0.0195719596761268*G0_1_1 + 0.00538228891093525*G0_1_2 - 0.0411011153198647*G0_2_0 + 0.00538228891093487*G0_2_1 - 0.00880738185425662*G0_2_2;
    A[2334] = 0.018104062700416*G0_0_1 + 0.0102752788299661*G0_0_2 + 0.0181040627004161*G0_1_0 + 0.0195719596761253*G0_1_1 + 0.029847238506092*G0_1_2 + 0.0102752788299661*G0_2_0 + 0.0298472385060919*G0_2_1 + 0.0401225173360585*G0_2_2;
    A[2335] = 0.00619778723077321*G0_0_1 + 0.00554538857490184*G0_0_2 + 0.00619778723077263*G0_1_0 + 0.0260959462348346*G0_1_1 + 0.0381653213684455*G0_1_2 + 0.00554538857490249*G0_2_0 + 0.0381653213684463*G0_2_1 + 0.0502346965020575*G0_2_2;
    A[2336] = -0.5845491956603*G0_0_0 - 0.44885027523916*G0_0_1 - 0.32163253734434*G0_0_2 - 0.194414799449519*G0_1_0 - 0.268135847562928*G0_1_1 - 0.222141742324032*G0_1_2 - 0.321632537344339*G0_2_0 - 0.290643601190474*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2337] = 0.912053320907477*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453738*G0_1_0 + 0.726119703984278*G0_1_1 + 0.456026660453738*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453738*G0_2_1 + 0.726119703984279*G0_2_2;
    A[2338] = -0.5845491956603*G0_0_0 - 0.135698920421141*G0_0_1 - 0.26291665831596*G0_0_2 - 0.390134396210779*G0_1_0 - 0.209419968534548*G0_1_1 - 0.290643601190472*G0_1_2 - 0.26291665831596*G0_2_0 - 0.104709984267274*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2339] = -0.0371867233846393*G0_0_0 + 0.0376760223765438*G0_0_1 + 0.186912214907006*G0_0_2 + 0.140428810676205*G0_1_0 + 0.0655660649150229*G0_1_1 + 0.0376760223765435*G0_1_2 + 0.186912214907005*G0_2_0 + 0.140428810676205*G0_2_1 - 0.037186723384639*G0_2_2;
    A[2340] = -0.0371867233846401*G0_0_0 - 0.0748627457611823*G0_0_1 - 0.224098938291645*G0_0_2 - 0.177615534060844*G0_1_0 - 0.149725491522364*G0_1_1 - 0.326851726591306*G0_1_2 - 0.224098938291644*G0_2_0 - 0.121346149991982*G0_2_1 - 0.44819787658329*G0_2_2;
    A[2341] = 0.0123955744615461*G0_0_0 + 0.00619778723077206*G0_0_1 + 0.00619778723077224*G0_0_2 + 0.00619778723077303*G0_1_0 - 0.0137003717732889*G0_1_1 + 0.0714376528178598*G0_1_2 + 0.00619778723077282*G0_2_0 - 0.0264221455627709*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2342] = -0.29227459783015*G0_0_0 - 0.0972073997247601*G0_0_1 - 0.22442513761958*G0_0_2 - 0.0972073997247602*G0_1_0 - 0.0880738185425686*G0_1_1 - 0.134067923781464*G0_1_2 - 0.22442513761958*G0_2_0 - 0.134067923781464*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2343] = 0.456026660453738*G0_0_0 + 0.228013330226869*G0_0_1 + 0.228013330226869*G0_0_2 + 0.228013330226869*G0_1_0 + 0.0929668084615992*G0_1_1 + 0.363059851992139*G0_1_2 + 0.228013330226869*G0_2_0 + 0.36305985199214*G0_2_1 + 0.0929668084615984*G0_2_2;
    A[2344] = -0.29227459783015*G0_0_0 - 0.19506719810539*G0_0_1 - 0.0678494602105708*G0_0_2 - 0.19506719810539*G0_1_0 - 0.185933616923199*G0_1_1 - 0.104709984267274*G0_1_2 - 0.0678494602105698*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2345] = -0.00880738185425694*G0_0_0 - 0.0411011153198649*G0_0_1 + 0.00538228891093427*G0_0_2 - 0.0411011153198642*G0_1_0 + 0.0195719596761268*G0_1_1 - 0.0552907860850565*G0_1_2 + 0.00538228891093491*G0_2_0 - 0.0552907860850565*G0_2_1 + 0.0195719596761255*G0_2_2;
    A[2346] = -0.00880738185425642*G0_0_0 + 0.032293733465608*G0_0_1 - 0.0141896707651916*G0_0_2 + 0.0322937334656078*G0_1_0 + 0.0929668084615995*G0_1_1 - 0.0283793415303832*G0_1_2 - 0.0141896707651918*G0_2_0 - 0.0283793415303831*G0_2_1;
    A[2347] = 0.00130479731174165*G0_0_0 + 0.00065239865587089*G0_0_1 + 0.000652398655869585*G0_0_2 + 0.000652398655870853*G0_1_0 - 0.0880738185425677*G0_1_1 + 0.0127217737894821*G0_1_2 + 0.000652398655871405*G0_2_0 + 0.0127217737894827*G0_2_1;
    A[2348] = -0.0391439193522533*G0_0_0 + 0.445262082631869*G0_0_1 + 0.249542485870608*G0_0_2 + 0.44526208263187*G0_1_0 + 0.146789697570946*G0_1_1 + 0.249542485870609*G0_1_2 + 0.249542485870608*G0_2_0 + 0.249542485870609*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2349] = -0.0391439193522507*G0_0_0 - 0.484406001984122*G0_0_1 - 0.28868640522286*G0_0_2 - 0.484406001984121*G0_1_0 - 0.782878387045046*G0_1_1 - 0.48440600198412*G0_1_2 - 0.28868640522286*G0_2_0 - 0.484406001984121*G0_2_1 - 0.185933616923196*G0_2_2;
    A[2350] = 0.00978597983806362*G0_0_0 + 0.00489298991903231*G0_0_1 + 0.00489298991903221*G0_0_2 + 0.00489298991903205*G0_1_0 + 0.146789697570946*G0_1_1 + 0.053822889109347*G0_1_2 + 0.00489298991903276*G0_2_0 + 0.0538228891093469*G0_2_1 - 0.0391439193522519*G0_2_2;
    A[2351] = -0.0391439193522535*G0_0_0 - 0.0195719596761273*G0_0_1 - 0.019571959676126*G0_0_2 - 0.0195719596761259*G0_1_0 - 0.0489298991903151*G0_1_1 - 0.141896707651915*G0_1_2 - 0.0195719596761265*G0_2_0 - 0.141896707651915*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2352] = 0.018919561020255*G0_0_0 + 0.018919561020255*G0_0_1 + 0.018919561020255*G0_0_2 + 0.0137421252872639*G0_1_0 + 0.013742125287264*G0_1_1 + 0.0137421252872639*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[2353] = 0.0335985307773501*G0_0_0 - 0.0143945239431346*G0_1_0;
    A[2354] = -0.013742125287264*G0_1_1;
    A[2355] = 0.00517743573299139*G0_1_2 + 0.0189195610202552*G0_2_2;
    A[2356] = 0.0187890812890812*G0_1_1 + 0.00939454064454041*G0_1_2 + 0.00939454064454049*G0_2_1;
    A[2357] = -0.0148746893538561*G0_1_1 - 0.0149399292194427*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0150051690850292*G0_2_2;
    A[2358] = -0.00117431758056766*G0_1_1 - 0.00123955744615516*G0_1_2 - 0.00123955744615456*G0_2_1 - 0.00130479731174267*G0_2_2;
    A[2359] = -0.0118736555368502*G0_1_1 - 0.0212681961813907*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0306627368259307*G0_2_2;
    A[2360] = -0.257045070413123*G0_0_0 - 0.0962940416065404*G0_0_2 - 0.192588083213081*G0_1_0 - 0.12473862300251*G0_1_2 - 0.0962940416065406*G0_2_0;
    A[2361] = 0.121346149991983*G0_0_0 - 0.0733296089198859*G0_0_2 - 0.0109602974186309*G0_1_0 - 0.146659217839773*G0_1_2 - 0.073329608919886*G0_2_0 - 0.271397840842283*G0_2_2;
    A[2362] = -0.0528442911255406*G0_0_0 + 0.0359471659384854*G0_0_2 - 0.00182671623643807*G0_1_0 + 0.0823327103709055*G0_1_2 + 0.0359471659384852*G0_2_0 + 0.20941996853455*G0_2_2;
    A[2363] = -0.0212681961813907*G0_0_2 - 0.0118736555368501*G0_1_0 - 0.0477555816097487*G0_1_2 - 0.0212681961813904*G0_2_0 - 0.111886369481855*G0_2_2;
    A[2364] = -0.128522535206561*G0_0_0 - 0.0962940416065405*G0_0_1 - 0.0962940416065407*G0_1_0 - 0.12473862300251*G0_1_1;
    A[2365] = 0.0476251018785734*G0_0_0 + 0.0623693115012553*G0_0_1 + 0.0623693115012557*G0_1_0 + 0.124738623002511*G0_1_1;
    A[2366] = -0.0150051690850295*G0_0_0 - 0.0377738821749233*G0_0_1 - 0.0377738821749239*G0_1_0 - 0.127087258163646*G0_1_1;
    A[2367] = 0.0093945406445401*G0_0_1 + 0.00939454064454043*G0_1_0 + 0.0641307878721064*G0_1_1;
    A[2368] = -0.0613254736518618*G0_0_0 - 0.0613254736518618*G0_0_1 - 0.0400572774704712*G0_0_2 - 0.0187890812890812*G0_1_0 - 0.0187890812890811*G0_1_1 - 0.00939454064454062*G0_1_2 - 0.0400572774704722*G0_2_0 - 0.0400572774704721*G0_2_1 - 0.0187890812890818*G0_2_2;
    A[2369] = -0.025443547578964*G0_0_0 - 0.0254435475789641*G0_0_1 + 0.0136351319077007*G0_0_2 + 0.0148746893538563*G0_1_0 + 0.0148746893538562*G0_1_1 - 6.52398655869318e-05*G0_1_2 + 0.013635131907702*G0_2_0 + 0.0136351319077021*G0_2_1 - 0.00013047973117328*G0_2_2;
    A[2370] = -0.052844291125541*G0_0_0 - 0.0528442911255408*G0_0_1 - 0.0137656116388751*G0_0_2 + 0.0011743175805674*G0_1_0 + 0.00117431758056749*G0_1_1 - 6.5239865587156e-05*G0_1_2 - 0.0137656116388762*G0_2_0 - 0.0137656116388761*G0_2_1 - 0.000130479731175018*G0_2_2;
    A[2371] = 0.0212681961813903*G0_0_2 + 0.0118736555368499*G0_1_0 + 0.01187365553685*G0_1_1 - 0.00939454064454075*G0_1_2 + 0.0212681961813903*G0_2_0 + 0.0212681961813904*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[2372] = -0.030662736825931*G0_0_0 - 0.0093945406445402*G0_0_1 - 0.030662736825931*G0_0_2 - 0.00939454064454058*G0_1_0 - 0.00939454064454059*G0_1_2 - 0.0306627368259314*G0_2_0 - 0.00939454064454098*G0_2_1 - 0.0306627368259316*G0_2_2;
    A[2373] = -0.00130479731174122*G0_0_0 - 6.52398655872781e-05*G0_0_1 - 0.00130479731174117*G0_0_2 - 6.52398655869302e-05*G0_1_0 - 6.52398655870692e-05*G0_1_2 - 0.00130479731174128*G0_2_0 - 6.52398655866675e-05*G0_2_1 - 0.00130479731174113*G0_2_2;
    A[2374] = -0.0150051690850302*G0_0_0 - 6.52398655868958e-05*G0_0_1 - 0.0150051690850302*G0_0_2 - 6.52398655872887e-05*G0_1_0 - 6.52398655872486e-05*G0_1_2 - 0.0150051690850304*G0_2_0 - 6.52398655871313e-05*G0_2_1 - 0.0150051690850304*G0_2_2;
    A[2375] = -0.00939454064454068*G0_0_1 - 0.00939454064454051*G0_1_0 - 0.00939454064454043*G0_1_2 - 0.00939454064454064*G0_2_1;
    A[2376] = -0.0693499771190729*G0_0_0 - 0.0906181733004636*G0_0_1 - 0.0906181733004636*G0_0_2 - 0.0547362472275658*G0_1_0 - 0.0641307878721063*G0_1_1 - 0.0641307878721063*G0_1_2 - 0.0906181733004639*G0_2_0 - 0.111886369481854*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2377] = 0.084681345532038*G0_0_0 + 0.173472802596064*G0_0_1 + 0.173472802596064*G0_0_2 + 0.0893133759887215*G0_1_0 + 0.127087258163645*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596064*G0_2_0 + 0.209419968534549*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2378] = -0.00339247301052742*G0_0_0 - 0.198068231922395*G0_0_1 - 0.198068231922395*G0_0_2 - 0.0623693115012542*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.198068231922396*G0_2_0 - 0.271397840842282*G0_2_1 - 0.271397840842282*G0_2_2;
    A[2379] = -0.064456987200043*G0_0_0 + 0.0962940416065401*G0_0_1 + 0.0962940416065402*G0_0_2 + 0.0284445813959693*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0962940416065404*G0_2_0;
    A[2380] = 0.0521918924696691*G0_0_0 - 0.271397840842283*G0_0_1 - 0.135698920421142*G0_0_2 - 0.271397840842284*G0_1_0 - 0.542795681684567*G0_1_1 - 0.271397840842283*G0_1_2 - 0.135698920421143*G0_2_0 - 0.271397840842283*G0_2_1;
    A[2381] = -0.0228339529554801*G0_0_0 + 0.0737210481134078*G0_0_1 + 0.0368605240567047*G0_0_2 + 0.0737210481134081*G0_1_0 + 0.20941996853455*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0368605240567039*G0_2_0 + 0.104709984267273*G0_2_1;
    A[2382] = -0.0300103381700598*G0_0_1 - 0.0150051690850306*G0_0_2 - 0.0300103381700599*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255413*G0_1_2 - 0.0150051690850293*G0_2_0 - 0.0528442911255397*G0_2_1;
    A[2383] = -0.0880738185425675*G0_0_0 + 0.0459941052388972*G0_0_1 + 0.00913358118219185*G0_0_2 + 0.0459941052388982*G0_1_0 + 0.0234863516113524*G0_1_1 - 0.0812236326559239*G0_1_2 + 0.00913358118219251*G0_2_0 - 0.0812236326559239*G0_2_1 - 0.1859336169232*G0_2_2;
    A[2384] = -0.0127217737894833*G0_0_1 - 0.0120693751336118*G0_0_2 - 0.0127217737894829*G0_1_0 + 0.0137003717732883*G0_1_1 + 0.0198981590040609*G0_1_2 - 0.0120693751336114*G0_2_0 + 0.019898159004062*G0_2_1 + 0.0260959462348354*G0_2_2;
    A[2385] = -0.0528442911255407*G0_0_1 - 0.0378391220405109*G0_0_2 - 0.052844291125541*G0_1_0 - 0.0534966897814111*G0_1_1 - 0.000652398655869567*G0_1_2 - 0.0378391220405094*G0_2_0 - 0.000652398655869468*G0_2_1 + 0.0521918924696713*G0_2_2;
    A[2386] = -0.0456679059109602*G0_0_0 + 0.0300103381700602*G0_0_1 - 0.00782878387045058*G0_0_2 + 0.0300103381700614*G0_1_0 + 0.0150051690850302*G0_1_2 - 0.00782878387044975*G0_2_0 + 0.0150051690850312*G0_2_1 + 0.0300103381700613*G0_2_2;
    A[2387] = -0.0228339529554808*G0_0_0 + 0.00130479731174174*G0_0_1 - 0.0107645778218699*G0_0_2 + 0.001304797311741*G0_1_0 + 0.000652398655870889*G0_1_2 - 0.0107645778218693*G0_2_0 + 0.000652398655869957*G0_2_1 + 0.00130479731174125*G0_2_2;
    A[2388] = 0.0300103381700605*G0_0_1 + 0.01500516908503*G0_0_2 + 0.0300103381700601*G0_1_0 + 0.0150051690850298*G0_1_2 + 0.0150051690850294*G0_2_0 + 0.0150051690850303*G0_2_1 + 0.0300103381700596*G0_2_2;
    A[2389] = -0.0880738185425691*G0_0_0 + 0.0127217737894818*G0_0_1 + 0.000652398655870742*G0_0_2 + 0.0127217737894819*G0_1_0 + 0.000652398655870706*G0_1_2 + 0.000652398655869783*G0_2_0 + 0.000652398655869838*G0_2_1 + 0.00130479731174019*G0_2_2;
    A[2390] = 0.012721773789482*G0_0_1 + 0.0120693751336116*G0_0_2 + 0.0127217737894825*G0_1_0 + 0.000652398655871265*G0_1_2 + 0.0120693751336112*G0_2_0 + 0.000652398655871522*G0_2_1 + 0.00130479731174263*G0_2_2;
    A[2391] = 0.0528442911255417*G0_0_1 + 0.0378391220405112*G0_0_2 + 0.0528442911255412*G0_1_0 + 0.0150051690850306*G0_1_2 + 0.0378391220405095*G0_2_0 + 0.0150051690850308*G0_2_1 + 0.0300103381700606*G0_2_2;
    A[2392] = 0.25574027310138*G0_0_0 + 0.18006202902036*G0_0_1 + 0.127870136550691*G0_0_2 + 0.0756782440810214*G0_1_0 + 0.105688582251082*G0_1_1 + 0.0528442911255411*G0_1_2 + 0.127870136550691*G0_2_0 + 0.10503618359521*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2393] = -0.5845491956603*G0_0_0 - 0.390134396210779*G0_0_1 - 0.26291665831596*G0_0_2 - 0.135698920421141*G0_1_0 - 0.209419968534548*G0_1_1 - 0.104709984267274*G0_1_2 - 0.26291665831596*G0_2_0 - 0.290643601190472*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2394] = 0.647179466623904*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842281*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842281*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684565*G0_2_2;
    A[2395] = 0.172233245149911*G0_0_0 + 0.0714376528178604*G0_0_1 + 0.00619778723077319*G0_0_2 - 0.026422145562771*G0_1_0 - 0.0137003717732886*G0_1_1 + 0.00619778723077208*G0_1_2 + 0.00619778723077198*G0_2_0 + 0.00619778723077234*G0_2_1 + 0.012395574461545*G0_2_2;
    A[2396] = -0.26813584756293*G0_0_0 - 0.0459941052388974*G0_0_1 + 0.0534966897814109*G0_0_2 + 0.0225077536275452*G0_1_0 - 0.0234863516113523*G0_1_1 - 0.104709984267277*G0_1_2 + 0.0534966897814119*G0_2_0 + 0.0812236326559238*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2397] = 0.105688582251081*G0_0_0 + 0.0528442911255411*G0_0_1 - 0.0221815542996093*G0_0_2 + 0.000652398655870967*G0_1_0 + 0.0534966897814115*G0_1_1 + 0.0528442911255421*G0_1_2 - 0.0221815542996088*G0_2_0 + 0.000652398655869816*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2398] = 0.12787013655069*G0_0_0 + 0.0378391220405112*G0_0_1 + 0.0900310145101805*G0_0_2 + 0.037839122040511*G0_1_0 + 0.0528442911255416*G0_1_2 + 0.0900310145101805*G0_2_0 + 0.0528442911255412*G0_2_1 + 0.0521918924696696*G0_2_2;
    A[2399] = -0.292274597830149*G0_0_0 - 0.0678494602105702*G0_0_1 - 0.195067198105389*G0_0_2 - 0.0678494602105706*G0_1_0 - 0.104709984267274*G0_1_2 - 0.19506719810539*G0_2_0 - 0.104709984267274*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2400] = 0.323589733311951*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421141*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842283*G0_1_2 + 0.135698920421141*G0_2_0 + 0.271397840842283*G0_2_1;
    A[2401] = 0.0502346965020569*G0_0_0 + 0.00554538857490156*G0_0_1 + 0.0381653213684461*G0_0_2 + 0.00554538857490199*G0_1_0 + 0.00619778723077251*G0_1_2 + 0.0381653213684454*G0_2_0 + 0.00619778723077222*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[2402] = -0.0965550010688892*G0_0_0 - 0.0678494602105708*G0_0_1 - 0.0368605240567046*G0_0_2 - 0.0678494602105709*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0368605240567043*G0_2_0 - 0.104709984267276*G0_2_1;
    A[2403] = 0.0300103381700599*G0_0_0 + 0.037839122040511*G0_0_1 + 0.0150051690850308*G0_0_2 + 0.0378391220405112*G0_1_0 + 0.0528442911255413*G0_1_2 + 0.0150051690850306*G0_2_0 + 0.0528442911255417*G0_2_1;
    A[2404] = 0.143527704291593*G0_0_0 - 0.0750258454251494*G0_0_1 - 0.0424059126316066*G0_0_2 - 0.0750258454251496*G0_1_0 - 0.0260959462348348*G0_1_2 - 0.0424059126316071*G0_2_0 - 0.0260959462348344*G0_2_1 - 0.0521918924696691*G0_2_2;
    A[2405] = -0.150051690850302*G0_0_0 + 0.316413348097372*G0_0_1 + 0.185933616923198*G0_0_2 + 0.316413348097372*G0_1_0 + 0.185933616923198*G0_1_2 + 0.1859336169232*G0_2_0 + 0.185933616923199*G0_2_1 + 0.371867233846398*G0_2_2;
    A[2406] = 0.0456679059109607*G0_0_0 - 0.0750258454251511*G0_0_1 - 0.0260959462348346*G0_0_2 - 0.075025845425149*G0_1_0 - 0.0260959462348338*G0_1_2 - 0.0260959462348346*G0_2_0 - 0.0260959462348357*G0_2_1 - 0.0521918924696706*G0_2_2;
    A[2407] = 0.176147637085137*G0_0_0 - 0.0587158790283775*G0_0_1 - 0.00978597983806308*G0_0_2 - 0.05871587902838*G0_1_0 - 0.0260959462348374*G0_1_2 - 0.00978597983806225*G0_2_0 - 0.0260959462348349*G0_2_1 - 0.0521918924696713*G0_2_2;
    A[2408] = -0.0298472385060929*G0_0_0 - 0.0298472385060929*G0_0_1 - 0.0298472385060928*G0_0_2 - 0.0345719095719097*G0_1_0 - 0.0345719095719097*G0_1_1 - 0.0345719095719097*G0_1_2 - 0.0298472385060927*G0_2_0 - 0.0298472385060928*G0_2_1 - 0.0298472385060927*G0_2_2;
    A[2409] = 0.0117431758056756*G0_0_0 + 0.00227817610630093*G0_1_0;
    A[2410] = -0.00946499969937458*G0_1_1;
    A[2411] = 0.00472467106581671*G0_1_2 - 0.0298472385060923*G0_2_2;
    A[2412] = 0.0292600797158083*G0_1_1 + 0.00329461321214749*G0_1_2 + 0.00329461321214781*G0_2_1;
    A[2413] = -0.0189848008858424*G0_1_1 - 0.00949240044292084*G0_1_2 - 0.00949240044292074*G0_2_1 - 0.000652398655871591*G0_2_2;
    A[2414] = 0.00352295274170231*G0_1_1 + 0.00649136662591446*G0_1_2 + 0.00649136662591435*G0_2_1 + 0.00880738185425734*G0_2_2;
    A[2415] = 0.0429604514890966*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004332*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[2416] = -0.0435476102793811*G0_0_0 - 0.00583896797004491*G0_0_2 - 0.0142875305635717*G0_1_0 - 0.00254435475789641*G0_1_2 - 0.00583896797004438*G0_2_0;
    A[2417] = 0.0567586830607669*G0_0_0 + 0.018886941087463*G0_0_2 + 0.0377738821749237*G0_1_0 + 0.00939454064454107*G0_1_2 + 0.0188869410874624*G0_2_0 - 0.00619778723077244*G0_2_2;
    A[2418] = 0.104709984267275*G0_0_0 + 0.074340826836485*G0_0_2 + 0.108232937008978*G0_1_0 + 0.0808321934624013*G0_1_2 + 0.074340826836486*G0_2_0 + 0.0371867233846392*G0_2_2;
    A[2419] = 0.0422754329004337*G0_0_2 + 0.0429604514890968*G0_1_0 + 0.0845508658008655*G0_1_2 + 0.0422754329004335*G0_2_0 + 0.128522535206563*G0_2_2;
    A[2420] = -0.00913358118219256*G0_0_0 - 0.00583896797004504*G0_0_1 - 0.00583896797004408*G0_1_0 - 0.00254435475789649*G0_1_1;
    A[2421] = -0.0156575677409005*G0_0_0 - 0.0130805930502101*G0_0_1 - 0.013080593050211*G0_1_0 - 0.0105036183595208*G0_1_1;
    A[2422] = -0.0006523986558707*G0_0_0 + 0.00192457603481902*G0_0_1 + 0.00192457603481928*G0_1_0 + 0.00450155072550929*G0_1_1;
    A[2423] = 0.00329461321214777*G0_0_1 + 0.00329461321214755*G0_1_0 + 0.00658922642429526*G0_1_1;
    A[2424] = 0.176636936077039*G0_0_0 + 0.176636936077039*G0_0_1 + 0.134361503176607*G0_0_2 + 0.127315597693201*G0_1_0 + 0.127315597693201*G0_1_1 + 0.128000616281866*G0_1_2 + 0.134361503176607*G0_2_0 + 0.134361503176607*G0_2_1 + 0.0920860702761741*G0_2_2;
    A[2425] = -0.334680510461759*G0_0_0 - 0.334680510461759*G0_0_1 - 0.250814663249558*G0_0_2 - 0.166948816037356*G0_1_0 - 0.166948816037356*G0_1_1 - 0.160783648739378*G0_1_2 - 0.250814663249558*G0_2_0 - 0.250814663249558*G0_2_1 - 0.0622388317700818*G0_2_2;
    A[2426] = 0.104709984267275*G0_0_0 + 0.104709984267275*G0_0_1 + 0.188575831479476*G0_0_2 + 0.182410664181496*G0_1_0 + 0.182410664181495*G0_1_1 + 0.0985448169692957*G0_1_2 + 0.188575831479476*G0_2_0 + 0.188575831479476*G0_2_1 - 0.0622388317700811*G0_2_2;
    A[2427] = -0.0422754329004325*G0_0_2 - 0.0429604514890965*G0_1_0 - 0.0429604514890965*G0_1_1 - 0.0359145460056914*G0_1_2 - 0.0422754329004332*G0_2_0 - 0.0422754329004332*G0_2_1 + 0.0920860702761741*G0_2_2;
    A[2428] = 0.0642612676032809*G0_0_0 + 0.0219858347028489*G0_0_1 + 0.0642612676032808*G0_0_2 + 0.0219858347028479*G0_1_0 + 0.0226708532915132*G0_1_1 + 0.0219858347028479*G0_1_2 + 0.0642612676032812*G0_2_0 + 0.0219858347028487*G0_2_1 + 0.0642612676032812*G0_2_2;
    A[2429] = 0.00880738185425683*G0_0_0 + 0.0023160152283414*G0_0_1 + 0.0088073818542569*G0_0_2 + 0.00231601522834213*G0_1_0 - 0.000652398655871117*G0_1_1 + 0.00231601522834224*G0_1_2 + 0.00880738185425695*G0_2_0 + 0.00231601522834166*G0_2_1 + 0.0088073818542569*G0_2_2;
    A[2430] = -0.000652398655871095*G0_0_0 + 0.00884000178705037*G0_0_1 - 0.000652398655871162*G0_0_2 + 0.0088400017870501*G0_1_0 - 0.000652398655870409*G0_1_1 + 0.00884000178705005*G0_1_2 - 0.000652398655870762*G0_2_0 + 0.00884000178705011*G0_2_1 - 0.000652398655870821*G0_2_2;
    A[2431] = -0.00329461321214782*G0_0_1 - 0.0032946132121475*G0_1_0 + 0.0226708532915126*G0_1_1 - 0.00329461321214752*G0_1_2 - 0.00329461321214781*G0_2_1;
    A[2432] = 0.0439716694056976*G0_0_0 + 0.0862471023061295*G0_0_1 + 0.0862471023061295*G0_0_2 + 0.0446566879943619*G0_1_0 + 0.0439716694056965*G0_1_1 + 0.0439716694056965*G0_1_2 + 0.0862471023061298*G0_2_0 + 0.128522535206562*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2433] = -0.00678494602105777*G0_0_0 - 0.0371541034518464*G0_0_1 - 0.0371541034518464*G0_0_2 - 0.00975335990526997*G0_1_0 - 0.0436454700777618*G0_1_1 - 0.0436454700777617*G0_1_2 - 0.0371541034518464*G0_2_0 + 0.0371867233846392*G0_2_1 + 0.0371867233846393*G0_2_2;
    A[2434] = 0.0127870136550697*G0_0_0 - 0.025084728318235*G0_0_1 - 0.0250847283182351*G0_0_2 + 0.00329461321214825*G0_1_0 - 0.0155923278753135*G0_1_1 - 0.0155923278753135*G0_1_2 - 0.0250847283182351*G0_2_0 - 0.00619778723077305*G0_2_1 - 0.00619778723077316*G0_2_2;
    A[2435] = -0.0318696743392925*G0_0_0 + 0.00583896797004501*G0_0_1 + 0.00583896797004502*G0_0_2 - 0.00590420783563165*G0_1_0 + 0.00254435475789648*G0_1_1 + 0.00254435475789647*G0_1_2 + 0.00583896797004443*G0_2_0;
    A[2436] = 0.0391439193522528*G0_0_0 + 0.0264221455627707*G0_0_1 - 0.00554538857490274*G0_0_2 + 0.0264221455627696*G0_1_0 + 0.0137003717732883*G0_1_1 - 0.00619778723077366*G0_1_2 - 0.00554538857490182*G0_2_0 - 0.00619778723077249*G0_2_1;
    A[2437] = 0.019571959676126*G0_0_0 + 0.00146789697570897*G0_0_1 - 0.0102752788299653*G0_0_2 + 0.00146789697570946*G0_1_0 - 0.016636165724708*G0_1_1 - 0.0181040627004163*G0_1_2 - 0.0102752788299659*G0_2_0 - 0.0181040627004162*G0_2_1;
    A[2438] = -0.012721773789481*G0_0_1 - 0.000652398655870879*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.0254435475789631*G0_1_1 - 0.00130479731174222*G0_1_2 - 0.000652398655870261*G0_2_0 - 0.00130479731174075*G0_2_1;
    A[2439] = 0.0929668084615986*G0_0_0 + 0.121346149991983*G0_0_1 + 0.0606730749959907*G0_0_2 + 0.121346149991983*G0_1_0 + 0.149725491522367*G0_1_1 + 0.074862745761183*G0_1_2 + 0.0606730749959912*G0_2_0 + 0.0748627457611835*G0_2_1 + 0.0195719596761264*G0_2_2;
    A[2440] = 0.0283793415303809*G0_0_1 + 0.0141896707651904*G0_0_2 + 0.028379341530382*G0_1_0 + 0.056758683060764*G0_1_1 + 0.0283793415303824*G0_1_2 + 0.0141896707651901*G0_2_0 + 0.0283793415303805*G0_2_1 - 0.00489298991903351*G0_2_2;
    A[2441] = 0.104709984267277*G0_0_1 + 0.0678494602105711*G0_0_2 + 0.104709984267274*G0_1_0 + 0.20941996853455*G0_1_1 + 0.135698920421141*G0_1_2 + 0.0678494602105717*G0_2_0 + 0.135698920421144*G0_2_1 + 0.0391439193522533*G0_2_2;
    A[2442] = -0.15657567740901*G0_0_0 - 0.134067923781465*G0_0_1 - 0.22442513761958*G0_0_2 - 0.134067923781464*G0_1_0 - 0.0880738185425672*G0_1_1 - 0.0972073997247589*G0_1_2 - 0.224425137619582*G0_2_0 - 0.0972073997247606*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2443] = 0.0195719596761264*G0_0_0 - 0.0552907860850559*G0_0_1 + 0.00538228891093483*G0_0_2 - 0.0552907860850563*G0_1_0 + 0.0195719596761253*G0_1_1 - 0.0411011153198655*G0_1_2 + 0.00538228891093574*G0_2_0 - 0.0411011153198641*G0_2_1 - 0.00880738185425725*G0_2_2;
    A[2444] = 0.0127217737894822*G0_0_1 + 0.000652398655870627*G0_0_2 + 0.0127217737894821*G0_1_0 - 0.0880738185425671*G0_1_1 + 0.000652398655870767*G0_1_2 + 0.000652398655870144*G0_2_0 + 0.000652398655870131*G0_2_1 + 0.00130479731174192*G0_2_2;
    A[2445] = 0.0929668084615987*G0_0_0 + 0.363059851992141*G0_0_1 + 0.22801333022687*G0_0_2 + 0.363059851992141*G0_1_0 + 0.0929668084615995*G0_1_1 + 0.228013330226869*G0_1_2 + 0.228013330226869*G0_2_0 + 0.22801333022687*G0_2_1 + 0.456026660453741*G0_2_2;
    A[2446] = -0.0283793415303836*G0_0_1 - 0.0141896707651914*G0_0_2 - 0.0283793415303821*G0_1_0 + 0.092966808461599*G0_1_1 + 0.0322937334656089*G0_1_2 - 0.0141896707651902*G0_2_0 + 0.0322937334656086*G0_2_1 - 0.0088073818542566*G0_2_2;
    A[2447] = -0.104709984267274*G0_0_1 - 0.0678494602105706*G0_0_2 - 0.104709984267274*G0_1_0 - 0.185933616923198*G0_1_1 - 0.19506719810539*G0_1_2 - 0.0678494602105716*G0_2_0 - 0.19506719810539*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2448] = -0.26813584756293*G0_0_0 - 0.290643601190475*G0_0_1 - 0.321632537344341*G0_0_2 - 0.222141742324033*G0_1_0 - 0.26813584756293*G0_1_1 - 0.194414799449519*G0_1_2 - 0.321632537344342*G0_2_0 - 0.448850275239163*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2449] = -0.0371867233846393*G0_0_0 + 0.140428810676205*G0_0_1 + 0.186912214907005*G0_0_2 + 0.0376760223765438*G0_1_0 + 0.0655660649150229*G0_1_1 + 0.140428810676205*G0_1_2 + 0.186912214907006*G0_2_0 + 0.0376760223765435*G0_2_1 - 0.037186723384639*G0_2_2;
    A[2450] = 0.172233245149911*G0_0_0 - 0.026422145562771*G0_0_1 + 0.00619778723077198*G0_0_2 + 0.0714376528178604*G0_1_0 - 0.0137003717732886*G0_1_1 + 0.00619778723077234*G0_1_2 + 0.00619778723077319*G0_2_0 + 0.00619778723077208*G0_2_1 + 0.012395574461545*G0_2_2;
    A[2451] = 0.726119703984283*G0_0_0 + 0.456026660453741*G0_0_1 + 0.456026660453742*G0_0_2 + 0.456026660453741*G0_1_0 + 0.726119703984283*G0_1_1 + 0.456026660453741*G0_1_2 + 0.456026660453742*G0_2_0 + 0.456026660453741*G0_2_1 + 0.912053320907483*G0_2_2;
    A[2452] = -0.448197876583292*G0_0_0 - 0.121346149991983*G0_0_1 - 0.224098938291644*G0_0_2 - 0.326851726591309*G0_1_0 - 0.149725491522367*G0_1_1 - 0.177615534060845*G0_1_2 - 0.224098938291646*G0_2_0 - 0.0748627457611829*G0_2_1 - 0.0371867233846394*G0_2_2;
    A[2453] = -0.20941996853455*G0_0_0 - 0.104709984267277*G0_0_1 - 0.262916658315964*G0_0_2 - 0.290643601190474*G0_1_0 - 0.209419968534551*G0_1_1 - 0.390134396210783*G0_1_2 - 0.262916658315963*G0_2_0 - 0.135698920421144*G0_2_1 - 0.584549195660304*G0_2_2;
    A[2454] = -0.0965550010688896*G0_0_0 - 0.0596944770121853*G0_0_1 - 0.0287055408583195*G0_0_2 - 0.0596944770121848*G0_1_0 - 0.0228339529554795*G0_1_1 - 0.0965550010688899*G0_1_2 - 0.0287055408583195*G0_2_0 - 0.0965550010688895*G0_2_1 + 0.0391439193522507*G0_2_2;
    A[2455] = -0.00880738185425672*G0_0_0 + 0.00538228891093496*G0_0_1 - 0.041101115319865*G0_0_2 + 0.00538228891093522*G0_1_0 + 0.0195719596761263*G0_1_1 - 0.055290786085056*G0_1_2 - 0.0411011153198637*G0_2_0 - 0.0552907860850561*G0_2_1 + 0.0195719596761269*G0_2_2;
    A[2456] = 0.0502346965020579*G0_0_0 + 0.0381653213684452*G0_0_1 + 0.00554538857490237*G0_0_2 + 0.0381653213684455*G0_1_0 + 0.0260959462348343*G0_1_1 + 0.00619778723077315*G0_1_2 + 0.00554538857490187*G0_2_0 + 0.00619778723077225*G0_2_1;
    A[2457] = -0.033272331449414*G0_0_0 - 0.0190826606842227*G0_0_1 - 0.0190826606842226*G0_0_2 - 0.0190826606842233*G0_1_0 - 0.00489298991903249*G0_1_1 - 0.0332723314494143*G0_1_2 - 0.0190826606842228*G0_2_0 - 0.033272331449414*G0_2_1 - 0.00489298991903002*G0_2_2;
    A[2458] = 0.0401225173360576*G0_0_0 + 0.0298472385060923*G0_0_1 + 0.0102752788299653*G0_0_2 + 0.0298472385060922*G0_1_0 + 0.019571959676126*G0_1_1 + 0.0181040627004163*G0_1_2 + 0.0102752788299654*G0_2_0 + 0.0181040627004166*G0_2_1;
    A[2459] = 0.00130479731174175*G0_0_0 - 0.0107645778218694*G0_0_1 + 0.000652398655870647*G0_0_2 - 0.0107645778218695*G0_1_0 - 0.0228339529554797*G0_1_1 + 0.00130479731174187*G0_1_2 + 0.000652398655870289*G0_2_0 + 0.00130479731174061*G0_2_1;
    A[2460] = 0.352295274170273*G0_0_0 + 0.249542485870611*G0_0_1 + 0.24954248587061*G0_0_2 + 0.24954248587061*G0_1_0 + 0.146789697570949*G0_1_1 + 0.445262082631872*G0_1_2 + 0.24954248587061*G0_2_0 + 0.445262082631872*G0_2_1 - 0.039143919352253*G0_2_2;
    A[2461] = -0.234863516113516*G0_0_0 - 0.141896707651914*G0_0_1 - 0.0195719596761257*G0_0_2 - 0.141896707651916*G0_1_0 - 0.0489298991903153*G0_1_1 - 0.0195719596761268*G0_1_2 - 0.0195719596761268*G0_2_0 - 0.0195719596761272*G0_2_1 - 0.0391439193522532*G0_2_2;
    A[2462] = -0.0391439193522524*G0_0_0 + 0.0538228891093458*G0_0_1 + 0.00489298991903259*G0_0_2 + 0.0538228891093471*G0_1_0 + 0.146789697570945*G0_1_1 + 0.00489298991903214*G0_1_2 + 0.00489298991903029*G0_2_0 + 0.00489298991903153*G0_2_1 + 0.00978597983806458*G0_2_2;
    A[2463] = -0.185933616923197*G0_0_0 - 0.484406001984122*G0_0_1 - 0.288686405222862*G0_0_2 - 0.484406001984124*G0_1_0 - 0.782878387045047*G0_1_1 - 0.484406001984124*G0_1_2 - 0.28868640522286*G0_2_0 - 0.484406001984122*G0_2_1 - 0.0391439193522522*G0_2_2;
    A[2464] = 0.011743175805676*G0_0_0 + 0.011743175805676*G0_0_1 + 0.011743175805676*G0_0_2 + 0.00946499969937476*G0_1_0 + 0.0094649996993748*G0_1_1 + 0.00946499969937475*G0_1_2 + 0.0117431758056756*G0_2_0 + 0.0117431758056756*G0_2_1 + 0.0117431758056756*G0_2_2;
    A[2465] = -0.0298472385060923*G0_0_0 + 0.00472467106581688*G0_1_0;
    A[2466] = -0.00946499969937459*G0_1_1;
    A[2467] = 0.00472467106581669*G0_1_2 - 0.0298472385060926*G0_2_2;
    A[2468] = 0.0292600797158087*G0_1_1 + 0.00329461321214798*G0_1_2 + 0.00329461321214786*G0_2_1;
    A[2469] = -0.0189848008858425*G0_1_1 - 0.00949240044292116*G0_1_2 - 0.00949240044292138*G0_2_1 - 0.000652398655871956*G0_2_2;
    A[2470] = 0.00352295274170306*G0_1_1 + 0.00649136662591508*G0_1_2 + 0.00649136662591511*G0_2_1 + 0.00880738185425746*G0_2_2;
    A[2471] = 0.0429604514890968*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0642612676032807*G0_2_2;
    A[2472] = 0.176636936077039*G0_0_0 + 0.0422754329004327*G0_0_2 + 0.0493213383838384*G0_1_0 + 0.0429604514890972*G0_1_2 + 0.0422754329004332*G0_2_0;
    A[2473] = -0.334680510461758*G0_0_0 - 0.083865847212201*G0_0_2 - 0.167731694424402*G0_1_0 - 0.0777006799142209*G0_1_2 - 0.0838658472122007*G0_2_0 + 0.104709984267276*G0_2_2;
    A[2474] = 0.104709984267275*G0_0_0 - 0.0838658472122006*G0_0_2 - 0.0777006799142209*G0_1_0 - 0.167731694424401*G0_1_2 - 0.0838658472122008*G0_2_0 - 0.334680510461758*G0_2_2;
    A[2475] = 0.0422754329004323*G0_0_2 + 0.0429604514890968*G0_1_0 + 0.049321338383838*G0_1_2 + 0.0422754329004323*G0_2_0 + 0.176636936077039*G0_2_2;
    A[2476] = 0.0642612676032805*G0_0_0 + 0.0422754329004327*G0_0_1 + 0.0422754329004329*G0_1_0 + 0.0429604514890972*G0_1_1;
    A[2477] = 0.00880738185425676*G0_0_0 + 0.00649136662591455*G0_0_1 + 0.00649136662591465*G0_1_0 + 0.00352295274170206*G0_1_1;
    A[2478] = -0.000652398655871545*G0_0_0 - 0.00949240044292143*G0_0_1 - 0.00949240044292107*G0_1_0 - 0.018984800885842*G0_1_1;
    A[2479] = 0.00329461321214822*G0_0_1 + 0.00329461321214804*G0_1_0 + 0.0292600797158085*G0_1_1;
    A[2480] = -0.0435476102793812*G0_0_0 - 0.0435476102793812*G0_0_1 - 0.0377086423093373*G0_0_2 - 0.0292600797158087*G0_1_0 - 0.0292600797158087*G0_1_1 - 0.0259654665036612*G0_1_2 - 0.0377086423093361*G0_2_0 - 0.0377086423093361*G0_2_1 - 0.0318696743392918*G0_2_2;
    A[2481] = 0.0567586830607667*G0_0_0 + 0.0567586830607667*G0_0_1 + 0.0378717419733055*G0_0_2 + 0.0189848008858425*G0_1_0 + 0.0189848008858425*G0_1_1 + 0.00949240044292197*G0_1_2 + 0.037871741973304*G0_2_0 + 0.0378717419733037*G0_2_1 + 0.0127870136550687*G0_2_2;
    A[2482] = 0.104709984267275*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0303691574307885*G0_0_2 - 0.00352295274170297*G0_1_0 - 0.00352295274170288*G0_1_1 + 0.00296841388421191*G0_1_2 + 0.0303691574307892*G0_2_0 + 0.0303691574307893*G0_2_1 - 0.00678494602105647*G0_2_2;
    A[2483] = -0.0422754329004324*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.0429604514890969*G0_1_1 - 0.000685018588664084*G0_1_2 - 0.0422754329004323*G0_2_0 - 0.0422754329004324*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[2484] = -0.00913358118219224*G0_0_0 - 0.00329461321214843*G0_0_1 - 0.00913358118219221*G0_0_2 - 0.00329461321214778*G0_1_0 - 0.00329461321214778*G0_1_2 - 0.00913358118219191*G0_2_0 - 0.00329461321214753*G0_2_1 - 0.00913358118219182*G0_2_2;
    A[2485] = -0.0156575677409011*G0_0_0 - 0.00257697469068968*G0_0_1 - 0.0156575677409011*G0_0_2 - 0.00257697469069027*G0_1_0 - 0.00257697469069018*G0_1_2 - 0.0156575677409013*G0_2_0 - 0.00257697469069028*G0_2_1 - 0.0156575677409015*G0_2_2;
    A[2486] = -0.000652398655870707*G0_0_0 - 0.00257697469069009*G0_0_1 - 0.000652398655870633*G0_0_2 - 0.0025769746906896*G0_1_0 - 0.00257697469068965*G0_1_2 - 0.000652398655870287*G0_2_0 - 0.00257697469068979*G0_2_1 - 0.000652398655870285*G0_2_2;
    A[2487] = -0.00329461321214779*G0_0_1 - 0.00329461321214807*G0_1_0 - 0.00329461321214806*G0_1_2 - 0.00329461321214808*G0_2_1;
    A[2488] = -0.0318696743392927*G0_0_0 - 0.0377086423093365*G0_0_1 - 0.0377086423093365*G0_0_2 - 0.0259654665036611*G0_1_0 - 0.0292600797158087*G0_1_1 - 0.0292600797158087*G0_1_2 - 0.0377086423093362*G0_2_0 - 0.0435476102793806*G0_2_1 - 0.0435476102793806*G0_2_2;
    A[2489] = 0.0127870136550695*G0_0_0 + 0.0378717419733046*G0_0_1 + 0.0378717419733045*G0_0_2 + 0.00949240044292176*G0_1_0 + 0.0189848008858428*G0_1_1 + 0.0189848008858427*G0_1_2 + 0.0378717419733042*G0_2_0 + 0.056758683060766*G0_2_1 + 0.0567586830607662*G0_2_2;
    A[2490] = -0.00678494602105746*G0_0_0 + 0.0303691574307888*G0_0_1 + 0.0303691574307888*G0_0_2 + 0.00296841388421181*G0_1_0 - 0.00352295274170308*G0_1_1 - 0.00352295274170299*G0_1_2 + 0.0303691574307891*G0_2_0 + 0.104709984267276*G0_2_1 + 0.104709984267276*G0_2_2;
    A[2491] = 0.0439716694056971*G0_0_0 - 0.0422754329004327*G0_0_1 - 0.0422754329004327*G0_0_2 - 0.000685018588663995*G0_1_0 - 0.0429604514890973*G0_1_1 - 0.0429604514890973*G0_1_2 - 0.0422754329004326*G0_2_0;
    A[2492] = -0.156575677409009*G0_0_0 - 0.0225077536275451*G0_0_1 + 0.0678494602105704*G0_0_2 - 0.0225077536275446*G0_1_0 + 0.0234863516113528*G0_1_1 + 0.104709984267276*G0_1_2 + 0.0678494602105727*G0_2_0 + 0.104709984267277*G0_2_1;
    A[2493] = 0.0195719596761256*G0_0_0 + 0.0748627457611832*G0_0_1 + 0.0141896707651905*G0_0_2 + 0.0748627457611828*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0283793415303822*G0_1_2 + 0.0141896707651925*G0_2_0 + 0.0283793415303844*G0_2_1;
    A[2494] = -0.0127217737894833*G0_0_1 - 0.000652398655870961*G0_0_2 - 0.0127217737894818*G0_1_0 - 0.113517366121532*G0_1_1 - 0.0127217737894818*G0_1_2 - 0.000652398655871931*G0_2_0 - 0.0127217737894827*G0_2_1;
    A[2495] = 0.0929668084616*G0_0_0 - 0.270093043530542*G0_0_1 - 0.135046521765269*G0_0_2 - 0.270093043530541*G0_1_0 - 0.540186087061083*G0_1_1 - 0.270093043530541*G0_1_2 - 0.13504652176527*G0_2_0 - 0.270093043530541*G0_2_1 + 0.092966808461602*G0_2_2;
    A[2496] = 0.0283793415303844*G0_0_1 + 0.0141896707651921*G0_0_2 + 0.0283793415303834*G0_1_0 + 0.149725491522366*G0_1_1 + 0.0748627457611838*G0_1_2 + 0.0141896707651913*G0_2_0 + 0.0748627457611823*G0_2_1 + 0.019571959676126*G0_2_2;
    A[2497] = 0.104709984267275*G0_0_1 + 0.0678494602105706*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0234863516113512*G0_1_1 - 0.0225077536275459*G0_1_2 + 0.0678494602105691*G0_2_0 - 0.0225077536275469*G0_2_1 - 0.156575677409012*G0_2_2;
    A[2498] = 0.0391439193522524*G0_0_0 + 0.0127217737894825*G0_0_1 + 0.0446893079271548*G0_0_2 + 0.0127217737894811*G0_1_0 + 0.0120693751336104*G0_1_2 + 0.0446893079271543*G0_2_0 + 0.0120693751336105*G0_2_1 + 0.0502346965020565*G0_2_2;
    A[2499] = 0.0195719596761262*G0_0_0 + 0.0181040627004168*G0_0_1 + 0.0298472385060925*G0_0_2 + 0.0181040627004167*G0_1_0 + 0.0102752788299664*G0_1_2 + 0.0298472385060926*G0_2_0 + 0.0102752788299672*G0_2_1 + 0.040122517336059*G0_2_2;
    A[2500] = 0.0127217737894811*G0_0_1 + 0.000652398655870852*G0_0_2 + 0.0127217737894817*G0_1_0 + 0.0120693751336106*G0_1_2 + 0.000652398655871833*G0_2_0 + 0.0120693751336107*G0_2_1 + 0.00130479731174203*G0_2_2;
    A[2501] = 0.0929668084615992*G0_0_0 - 0.0283793415303837*G0_0_1 + 0.032293733465608*G0_0_2 - 0.028379341530384*G0_1_0 - 0.0141896707651909*G0_1_2 + 0.032293733465608*G0_2_0 - 0.0141896707651909*G0_2_1 - 0.00880738185425596*G0_2_2;
    A[2502] = -0.0283793415303815*G0_0_1 - 0.0141896707651913*G0_0_2 - 0.0283793415303832*G0_1_0 - 0.0141896707651908*G0_1_2 - 0.014189670765191*G0_2_0 - 0.0141896707651917*G0_2_1 - 0.0332723314494146*G0_2_2;
    A[2503] = -0.104709984267276*G0_0_1 - 0.0678494602105713*G0_0_2 - 0.104709984267275*G0_1_0 - 0.036860524056705*G0_1_2 - 0.0678494602105696*G0_2_0 - 0.0368605240567045*G0_2_1 - 0.0965550010688896*G0_2_2;
    A[2504] = 0.172233245149912*G0_0_0 + 0.198655390712682*G0_0_1 + 0.166035457919137*G0_0_2 + 0.10079559233205*G0_1_0 + 0.113517366121531*G0_1_1 + 0.100795592332049*G0_1_2 + 0.166035457919137*G0_2_0 + 0.198655390712681*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2505] = -0.0371867233846401*G0_0_0 - 0.177615534060844*G0_0_1 - 0.224098938291644*G0_0_2 - 0.0748627457611823*G0_1_0 - 0.149725491522364*G0_1_1 - 0.121346149991982*G0_1_2 - 0.224098938291645*G0_2_0 - 0.326851726591306*G0_2_1 - 0.44819787658329*G0_2_2;
    A[2506] = -0.26813584756293*G0_0_0 + 0.0225077536275452*G0_0_1 + 0.0534966897814119*G0_0_2 - 0.0459941052388973*G0_1_0 - 0.0234863516113523*G0_1_1 + 0.0812236326559238*G0_1_2 + 0.0534966897814109*G0_2_0 - 0.104709984267277*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2507] = -0.448197876583292*G0_0_0 - 0.326851726591309*G0_0_1 - 0.224098938291646*G0_0_2 - 0.121346149991983*G0_1_0 - 0.149725491522367*G0_1_1 - 0.074862745761183*G0_1_2 - 0.224098938291644*G0_2_0 - 0.177615534060845*G0_2_1 - 0.0371867233846394*G0_2_2;
    A[2508] = 0.726119703984283*G0_0_0 + 0.270093043530542*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530542*G0_1_0 + 0.540186087061083*G0_1_1 + 0.270093043530541*G0_1_2 + 0.270093043530541*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984283*G0_2_2;
    A[2509] = -0.20941996853455*G0_0_0 - 0.104709984267274*G0_0_1 + 0.0534966897814129*G0_0_2 + 0.081223632655924*G0_1_0 - 0.0234863516113509*G0_1_1 - 0.045994105238896*G0_1_2 + 0.0534966897814117*G0_2_0 + 0.0225077536275469*G0_2_1 - 0.268135847562929*G0_2_2;
    A[2510] = 0.0502346965020579*G0_0_0 + 0.0120693751336112*G0_0_1 + 0.0446893079271555*G0_0_2 + 0.0120693751336111*G0_1_0 + 0.0127217737894817*G0_1_2 + 0.0446893079271548*G0_2_0 + 0.0127217737894818*G0_2_1 + 0.0391439193522525*G0_2_2;
    A[2511] = -0.00880738185425746*G0_0_0 - 0.014189670765192*G0_0_1 + 0.0322937334656078*G0_0_2 - 0.0141896707651915*G0_1_0 - 0.028379341530383*G0_1_2 + 0.0322937334656083*G0_2_0 - 0.028379341530384*G0_2_1 + 0.0929668084616004*G0_2_2;
    A[2512] = -0.0965550010688887*G0_0_0 - 0.0368605240567033*G0_0_1 - 0.06784946021057*G0_0_2 - 0.0368605240567047*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0678494602105715*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2513] = 0.0401225173360582*G0_0_0 + 0.0102752788299664*G0_0_1 + 0.0298472385060921*G0_0_2 + 0.010275278829966*G0_1_0 + 0.0181040627004164*G0_1_2 + 0.0298472385060926*G0_2_0 + 0.0181040627004176*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2514] = -0.0332723314494139*G0_0_0 - 0.0141896707651911*G0_0_1 - 0.0141896707651905*G0_0_2 - 0.0141896707651905*G0_1_0 - 0.0283793415303824*G0_1_2 - 0.0141896707651906*G0_2_0 - 0.0283793415303822*G0_2_1;
    A[2515] = 0.00130479731174221*G0_0_0 + 0.0120693751336107*G0_0_1 + 0.00065239865587112*G0_0_2 + 0.0120693751336107*G0_1_0 + 0.0127217737894819*G0_1_2 + 0.000652398655870028*G0_2_0 + 0.0127217737894809*G0_2_1;
    A[2516] = -0.234863516113515*G0_0_0 - 0.0929668084616017*G0_0_1 - 0.215291556437389*G0_0_2 - 0.0929668084615995*G0_1_0 - 0.0929668084616009*G0_1_2 - 0.215291556437388*G0_2_0 - 0.0929668084616011*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2517] = 0.352295274170272*G0_0_0 + 0.102752788299661*G0_0_1 + 0.102752788299661*G0_0_2 + 0.102752788299663*G0_1_0 + 0.298472385060923*G0_1_2 + 0.102752788299661*G0_2_0 + 0.298472385060922*G0_2_1 - 0.185933616923202*G0_2_2;
    A[2518] = -0.0391439193522517*G0_0_0 - 0.0929668084615972*G0_0_1 - 0.0440369092712843*G0_0_2 - 0.0929668084615995*G0_1_0 - 0.0929668084616001*G0_1_2 - 0.0440369092712852*G0_2_0 - 0.0929668084615975*G0_2_1 - 0.0391439193522517*G0_2_2;
    A[2519] = -0.185933616923199*G0_0_0 + 0.298472385060924*G0_0_1 + 0.102752788299663*G0_0_2 + 0.298472385060925*G0_1_0 + 0.102752788299665*G0_1_2 + 0.102752788299662*G0_2_0 + 0.102752788299663*G0_2_1 + 0.352295274170275*G0_2_2;
    A[2520] = 0.0189195610202555*G0_0_0 + 0.0189195610202555*G0_0_1 + 0.0189195610202555*G0_0_2 + 0.0137421252872641*G0_1_0 + 0.0137421252872641*G0_1_1 + 0.0137421252872641*G0_1_2 + 0.0189195610202556*G0_2_0 + 0.0189195610202557*G0_2_1 + 0.0189195610202556*G0_2_2;
    A[2521] = 0.0189195610202555*G0_0_0 + 0.00517743573299124*G0_1_0;
    A[2522] = -0.0137421252872639*G0_1_1;
    A[2523] = -0.0143945239431349*G0_1_2 + 0.03359853077735*G0_2_2;
    A[2524] = 0.0641307878721068*G0_1_1 + 0.00939454064454087*G0_1_2 + 0.00939454064454087*G0_2_1;
    A[2525] = -0.127087258163646*G0_1_1 - 0.0377738821749238*G0_1_2 - 0.0377738821749239*G0_2_1 - 0.0150051690850296*G0_2_2;
    A[2526] = 0.124738623002511*G0_1_1 + 0.0623693115012556*G0_1_2 + 0.062369311501256*G0_2_1 + 0.0476251018785734*G0_2_2;
    A[2527] = -0.12473862300251*G0_1_1 - 0.0962940416065408*G0_1_2 - 0.0962940416065409*G0_2_1 - 0.128522535206562*G0_2_2;
    A[2528] = -0.111886369481854*G0_0_0 - 0.0212681961813901*G0_0_2 - 0.047755581609748*G0_1_0 - 0.0118736555368505*G0_1_2 - 0.0212681961813907*G0_2_0;
    A[2529] = 0.20941996853455*G0_0_0 + 0.0359471659384844*G0_0_2 + 0.0823327103709042*G0_1_0 - 0.0018267162364375*G0_1_2 + 0.0359471659384847*G0_2_0 - 0.052844291125541*G0_2_2;
    A[2530] = -0.271397840842284*G0_0_0 - 0.0733296089198858*G0_0_2 - 0.146659217839773*G0_1_0 - 0.0109602974186317*G0_1_2 - 0.0733296089198864*G0_2_0 + 0.121346149991983*G0_2_2;
    A[2531] = -0.0962940416065416*G0_0_2 - 0.124738623002511*G0_1_0 - 0.192588083213081*G0_1_2 - 0.0962940416065412*G0_2_0 - 0.257045070413125*G0_2_2;
    A[2532] = -0.0306627368259305*G0_0_0 - 0.0212681961813899*G0_0_1 - 0.0212681961813907*G0_1_0 - 0.0118736555368504*G0_1_1;
    A[2533] = -0.00130479731174214*G0_0_0 - 0.00123955744615517*G0_0_1 - 0.00123955744615428*G0_1_0 - 0.00117431758056688*G0_1_1;
    A[2534] = -0.0150051690850297*G0_0_0 - 0.0149399292194426*G0_0_1 - 0.0149399292194429*G0_1_0 - 0.014874689353856*G0_1_1;
    A[2535] = 0.00939454064454042*G0_0_1 + 0.00939454064454077*G0_1_0 + 0.0187890812890815*G0_1_1;
    A[2536] = -0.111886369481855*G0_0_0 - 0.111886369481855*G0_0_1 - 0.0906181733004646*G0_0_2 - 0.0641307878721072*G0_1_0 - 0.0641307878721071*G0_1_1 - 0.0547362472275665*G0_1_2 - 0.0906181733004648*G0_2_0 - 0.0906181733004649*G0_2_1 - 0.0693499771190745*G0_2_2;
    A[2537] = 0.20941996853455*G0_0_0 + 0.20941996853455*G0_0_1 + 0.173472802596066*G0_0_2 + 0.127087258163647*G0_1_0 + 0.127087258163647*G0_1_1 + 0.0893133759887231*G0_1_2 + 0.173472802596066*G0_2_0 + 0.173472802596066*G0_2_1 + 0.0846813455320403*G0_2_2;
    A[2538] = -0.271397840842284*G0_0_0 - 0.271397840842283*G0_0_1 - 0.198068231922398*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002511*G0_1_1 - 0.0623693115012559*G0_1_2 - 0.198068231922397*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052944*G0_2_2;
    A[2539] = 0.096294041606541*G0_0_2 + 0.124738623002511*G0_1_0 + 0.12473862300251*G0_1_1 + 0.0284445813959702*G0_1_2 + 0.096294041606541*G0_2_0 + 0.096294041606541*G0_2_1 - 0.0644569872000422*G0_2_2;
    A[2540] = -0.0306627368259308*G0_0_0 - 0.00939454064454045*G0_0_1 - 0.0306627368259308*G0_0_2 - 0.00939454064454035*G0_1_0 - 0.00939454064454038*G0_1_2 - 0.0306627368259312*G0_2_0 - 0.0093945406445409*G0_2_1 - 0.0306627368259312*G0_2_2;
    A[2541] = -0.00130479731174206*G0_0_0 - 6.52398655873559e-05*G0_0_1 - 0.00130479731174208*G0_0_2 - 6.52398655877536e-05*G0_1_0 - 6.52398655877005e-05*G0_1_2 - 0.00130479731174184*G0_2_0 - 6.52398655870559e-05*G0_2_1 - 0.00130479731174173*G0_2_2;
    A[2542] = -0.0150051690850299*G0_0_0 - 6.52398655869633e-05*G0_0_1 - 0.0150051690850299*G0_0_2 - 6.52398655867698e-05*G0_1_0 - 6.52398655868178e-05*G0_1_2 - 0.0150051690850301*G0_2_0 - 6.5239865587085e-05*G0_2_1 - 0.01500516908503*G0_2_2;
    A[2543] = -0.00939454064454078*G0_0_1 - 0.00939454064454085*G0_1_0 - 0.00939454064454087*G0_1_2 - 0.00939454064454056*G0_2_1;
    A[2544] = -0.0187890812890813*G0_0_0 - 0.0400572774704716*G0_0_1 - 0.0400572774704716*G0_0_2 - 0.00939454064454054*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.0187890812890812*G0_1_2 - 0.0400572774704721*G0_2_0 - 0.0613254736518623*G0_2_1 - 0.0613254736518622*G0_2_2;
    A[2545] = -0.000130479731173935*G0_0_0 + 0.0136351319077013*G0_0_1 + 0.0136351319077013*G0_0_2 - 6.52398655874846e-05*G0_1_0 + 0.0148746893538559*G0_1_1 + 0.0148746893538559*G0_1_2 + 0.0136351319077014*G0_2_0 - 0.0254435475789637*G0_2_1 - 0.0254435475789639*G0_2_2;
    A[2546] = -0.000130479731174371*G0_0_0 - 0.0137656116388753*G0_0_1 - 0.0137656116388753*G0_0_2 - 6.52398655865449e-05*G0_1_0 + 0.00117431758056791*G0_1_1 + 0.00117431758056785*G0_1_2 - 0.0137656116388754*G0_2_0 - 0.052844291125541*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[2547] = -0.0187890812890811*G0_0_0 + 0.0212681961813899*G0_0_1 + 0.0212681961813899*G0_0_2 - 0.00939454064454074*G0_1_0 + 0.0118736555368503*G0_1_1 + 0.0118736555368503*G0_1_2 + 0.0212681961813905*G0_2_0;
    A[2548] = 0.0521918924696698*G0_0_0 - 0.000652398655870723*G0_0_1 - 0.03783912204051*G0_0_2 - 0.000652398655870567*G0_1_0 - 0.0534966897814114*G0_1_1 - 0.0528442911255405*G0_1_2 - 0.0378391220405116*G0_2_0 - 0.0528442911255418*G0_2_1;
    A[2549] = 0.0260959462348358*G0_0_0 + 0.0198981590040621*G0_0_1 - 0.012069375133611*G0_0_2 + 0.0198981590040607*G0_1_0 + 0.0137003717732876*G0_1_1 - 0.0127217737894821*G0_1_2 - 0.0120693751336119*G0_2_0 - 0.0127217737894828*G0_2_1;
    A[2550] = -0.0528442911255411*G0_0_1 - 0.0150051690850302*G0_0_2 - 0.0528442911255399*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0300103381700591*G0_1_2 - 0.0150051690850296*G0_2_0 - 0.0300103381700606*G0_2_1;
    A[2551] = -0.185933616923199*G0_0_0 - 0.0812236326559245*G0_0_1 + 0.00913358118219187*G0_0_2 - 0.0812236326559238*G0_1_0 + 0.023486351611351*G0_1_1 + 0.045994105238896*G0_1_2 + 0.00913358118219203*G0_2_0 + 0.0459941052388957*G0_2_1 - 0.0880738185425688*G0_2_2;
    A[2552] = 0.104709984267276*G0_0_1 + 0.0368605240567048*G0_0_2 + 0.104709984267275*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0737210481134075*G0_1_2 + 0.036860524056705*G0_2_0 + 0.0737210481134105*G0_2_1 - 0.0228339529554801*G0_2_2;
    A[2553] = -0.271397840842284*G0_0_1 - 0.135698920421142*G0_0_2 - 0.271397840842283*G0_1_0 - 0.542795681684566*G0_1_1 - 0.271397840842282*G0_1_2 - 0.135698920421141*G0_2_0 - 0.271397840842284*G0_2_1 + 0.0521918924696707*G0_2_2;
    A[2554] = 0.0521918924696698*G0_0_0 + 0.0528442911255403*G0_0_1 + 0.09003101451018*G0_0_2 + 0.0528442911255427*G0_1_0 + 0.0378391220405115*G0_1_2 + 0.0900310145101818*G0_2_0 + 0.0378391220405114*G0_2_1 + 0.127870136550692*G0_2_2;
    A[2555] = 0.0260959462348347*G0_0_0 + 0.00619778723077374*G0_0_1 + 0.0381653213684465*G0_0_2 + 0.0061977872307723*G0_1_0 + 0.00554538857490212*G0_1_2 + 0.0381653213684454*G0_2_0 + 0.00554538857490165*G0_2_1 + 0.050234696502057*G0_2_2;
    A[2556] = 0.0528442911255412*G0_0_1 + 0.0150051690850301*G0_0_2 + 0.0528442911255398*G0_1_0 + 0.0378391220405104*G0_1_2 + 0.0150051690850297*G0_2_0 + 0.0378391220405108*G0_2_1 + 0.03001033817006*G0_2_2;
    A[2557] = -0.185933616923198*G0_0_0 - 0.104709984267275*G0_0_1 - 0.195067198105391*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105719*G0_1_2 - 0.195067198105391*G0_2_0 - 0.0678494602105716*G0_2_1 - 0.292274597830152*G0_2_2;
    A[2558] = -0.104709984267276*G0_0_1 - 0.0368605240567046*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105701*G0_1_2 - 0.036860524056705*G0_2_0 - 0.0678494602105706*G0_2_1 - 0.0965550010688891*G0_2_2;
    A[2559] = 0.271397840842284*G0_0_1 + 0.135698920421142*G0_0_2 + 0.271397840842283*G0_1_0 + 0.135698920421142*G0_1_2 + 0.135698920421141*G0_2_0 + 0.135698920421141*G0_2_1 + 0.323589733311953*G0_2_2;
    A[2560] = 0.105688582251082*G0_0_0 + 0.105036183595211*G0_0_1 + 0.127870136550691*G0_0_2 + 0.0528442911255422*G0_1_0 + 0.105688582251084*G0_1_1 + 0.0756782440810228*G0_1_2 + 0.127870136550693*G0_2_0 + 0.180062029020363*G0_2_1 + 0.255740273101383*G0_2_2;
    A[2561] = 0.0123955744615461*G0_0_0 + 0.00619778723077303*G0_0_1 + 0.00619778723077282*G0_0_2 + 0.00619778723077206*G0_1_0 - 0.0137003717732889*G0_1_1 - 0.0264221455627709*G0_1_2 + 0.00619778723077222*G0_2_0 + 0.0714376528178598*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2562] = 0.105688582251081*G0_0_0 + 0.000652398655870961*G0_0_1 - 0.0221815542996088*G0_0_2 + 0.0528442911255411*G0_1_0 + 0.0534966897814115*G0_1_1 + 0.000652398655869814*G0_1_2 - 0.0221815542996093*G0_2_0 + 0.0528442911255421*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2563] = -0.20941996853455*G0_0_0 - 0.290643601190474*G0_0_1 - 0.262916658315963*G0_0_2 - 0.104709984267277*G0_1_0 - 0.209419968534551*G0_1_1 - 0.135698920421144*G0_1_2 - 0.262916658315964*G0_2_0 - 0.390134396210783*G0_2_1 - 0.584549195660304*G0_2_2;
    A[2564] = -0.20941996853455*G0_0_0 + 0.081223632655924*G0_0_1 + 0.0534966897814117*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0234863516113509*G0_1_1 + 0.0225077536275469*G0_1_2 + 0.0534966897814129*G0_2_0 - 0.045994105238896*G0_2_1 - 0.268135847562929*G0_2_2;
    A[2565] = 0.542795681684567*G0_0_0 + 0.271397840842285*G0_0_1 + 0.271397840842284*G0_0_2 + 0.271397840842284*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842284*G0_2_0 + 0.271397840842284*G0_2_1 + 0.647179466623907*G0_2_2;
    A[2566] = 0.0300103381700597*G0_0_0 + 0.0150051690850298*G0_0_1 - 0.00782878387045048*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0300103381700615*G0_1_2 - 0.00782878387045003*G0_2_0 + 0.0300103381700602*G0_2_1 - 0.04566790591096*G0_2_2;
    A[2567] = 0.00130479731174203*G0_0_0 + 0.000652398655870868*G0_0_1 + 0.000652398655871122*G0_0_2 + 0.000652398655870682*G0_1_0 + 0.0127217737894829*G0_1_2 + 0.00065239865587033*G0_2_0 + 0.0127217737894823*G0_2_1 - 0.0880738185425685*G0_2_2;
    A[2568] = 0.0300103381700597*G0_0_0 + 0.0150051690850304*G0_0_1 + 0.0378391220405101*G0_0_2 + 0.0150051690850301*G0_1_0 + 0.0528442911255408*G0_1_2 + 0.0378391220405112*G0_2_0 + 0.0528442911255412*G0_2_1;
    A[2569] = 0.00130479731174218*G0_0_0 + 0.000652398655871319*G0_0_1 - 0.0107645778218695*G0_0_2 + 0.00065239865587145*G0_1_0 + 0.00130479731174117*G0_1_2 - 0.0107645778218695*G0_2_0 + 0.00130479731174154*G0_2_1 - 0.0228339529554813*G0_2_2;
    A[2570] = 0.00130479731174202*G0_0_0 + 0.000652398655870563*G0_0_1 + 0.0120693751336111*G0_0_2 + 0.000652398655870346*G0_1_0 + 0.0127217737894816*G0_1_2 + 0.0120693751336113*G0_2_0 + 0.0127217737894819*G0_2_1;
    A[2571] = 0.0300103381700597*G0_0_0 + 0.0150051690850301*G0_0_1 + 0.0150051690850303*G0_0_2 + 0.0150051690850298*G0_1_0 + 0.0300103381700594*G0_1_2 + 0.0150051690850304*G0_2_0 + 0.0300103381700605*G0_2_1;
    A[2572] = -0.0521918924696705*G0_0_0 - 0.0260959462348352*G0_0_1 - 0.0424059126316074*G0_0_2 - 0.026095946234837*G0_1_0 - 0.0750258454251528*G0_1_2 - 0.0424059126316084*G0_2_0 - 0.075025845425152*G0_2_1 + 0.143527704291592*G0_2_2;
    A[2573] = -0.0521918924696693*G0_0_0 - 0.0260959462348349*G0_0_1 - 0.00978597983806296*G0_0_2 - 0.026095946234835*G0_1_0 - 0.0587158790283792*G0_1_2 - 0.00978597983806181*G0_2_0 - 0.0587158790283771*G0_2_1 + 0.176147637085137*G0_2_2;
    A[2574] = -0.0521918924696707*G0_0_0 - 0.026095946234835*G0_0_1 - 0.0260959462348356*G0_0_2 - 0.0260959462348331*G0_1_0 - 0.0750258454251487*G0_1_2 - 0.0260959462348336*G0_2_0 - 0.0750258454251505*G0_2_1 + 0.0456679059109603*G0_2_2;
    A[2575] = 0.371867233846397*G0_0_0 + 0.185933616923199*G0_0_1 + 0.1859336169232*G0_0_2 + 0.185933616923199*G0_1_0 + 0.316413348097374*G0_1_2 + 0.185933616923199*G0_2_0 + 0.316413348097373*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2576] = 0.0335985307773498*G0_0_0 + 0.0335985307773498*G0_0_1 + 0.0335985307773499*G0_0_2 + 0.0335985307773497*G0_1_0 + 0.0335985307773497*G0_1_1 + 0.0335985307773498*G0_1_2 + 0.0479930547204847*G0_2_0 + 0.0479930547204847*G0_2_1 + 0.0479930547204847*G0_2_2;
    A[2577] = 0.0189195610202553*G0_0_0 + 0.00517743573299134*G0_2_0;
    A[2578] = 0.0189195610202553*G0_1_1 + 0.00517743573299124*G0_2_1;
    A[2579] = -0.013742125287264*G0_2_2;
    A[2580] = -0.0306627368259311*G0_1_1 - 0.0212681961813906*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0118736555368497*G0_2_2;
    A[2581] = -0.00130479731174165*G0_1_1 - 0.0012395574461544*G0_1_2 - 0.00123955744615481*G0_2_1 - 0.00117431758056795*G0_2_2;
    A[2582] = -0.0150051690850301*G0_1_1 - 0.0149399292194433*G0_1_2 - 0.0149399292194434*G0_2_1 - 0.0148746893538557*G0_2_2;
    A[2583] = 0.00939454064454042*G0_1_2 + 0.00939454064454017*G0_2_1 + 0.0187890812890804*G0_2_2;
    A[2584] = -0.0306627368259309*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0212681961813907*G0_2_0 - 0.01187365553685*G0_2_2;
    A[2585] = -0.00130479731174216*G0_0_0 - 0.00123955744615501*G0_0_2 - 0.00123955744615504*G0_2_0 - 0.00117431758056783*G0_2_2;
    A[2586] = -0.0150051690850296*G0_0_0 - 0.0149399292194425*G0_0_2 - 0.0149399292194427*G0_2_0 - 0.0148746893538559*G0_2_2;
    A[2587] = 0.00939454064454032*G0_0_2 + 0.00939454064454023*G0_2_0 + 0.0187890812890806*G0_2_2;
    A[2588] = -0.0613254736518618*G0_0_0 - 0.0212681961813903*G0_0_1 - 0.0212681961813906*G0_1_0 - 0.0425363923627813*G0_2_0 - 0.01187365553685*G0_2_1;
    A[2589] = -0.0254435475789637*G0_0_0 - 0.0390786794866649*G0_0_1 - 0.0390786794866649*G0_1_0 - 0.0528442911255406*G0_1_1 - 0.0403182369328194*G0_2_0 - 0.054018608706108*G0_2_1;
    A[2590] = -0.0528442911255405*G0_0_0 - 0.0390786794866652*G0_0_1 - 0.0390786794866651*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0540186087061085*G0_2_0 - 0.04031823693282*G0_2_1;
    A[2591] = -0.0212681961813902*G0_0_1 - 0.0212681961813905*G0_1_0 - 0.0613254736518622*G0_1_1 - 0.0118736555368498*G0_2_0 - 0.0425363923627809*G0_2_1;
    A[2592] = -0.128522535206561*G0_0_0 - 0.128522535206562*G0_0_1 - 0.0322284936000213*G0_0_2 - 0.128522535206561*G0_1_0 - 0.128522535206562*G0_1_1 - 0.032228493600021*G0_1_2 - 0.0322284936000209*G0_2_0 - 0.0322284936000208*G0_2_1 - 0.0606730749959912*G0_2_2;
    A[2593] = 0.0476251018785742*G0_0_0 + 0.0476251018785742*G0_0_1 - 0.0147442096226813*G0_0_2 + 0.0476251018785742*G0_1_0 + 0.0476251018785743*G0_1_1 - 0.0147442096226814*G0_1_2 - 0.0147442096226811*G0_2_0 - 0.0147442096226811*G0_2_1 + 0.0476251018785741*G0_2_2;
    A[2594] = -0.01500516908503*G0_0_0 - 0.01500516908503*G0_0_1 + 0.0227687130898934*G0_0_2 - 0.01500516908503*G0_1_0 - 0.0150051690850301*G0_1_1 + 0.0227687130898934*G0_1_2 + 0.0227687130898934*G0_2_0 + 0.0227687130898935*G0_2_1 - 0.0665446628988287*G0_2_2;
    A[2595] = -0.00939454064454077*G0_0_2 - 0.0093945406445407*G0_1_2 - 0.00939454064454031*G0_2_0 - 0.00939454064454028*G0_2_1 + 0.0453417065830253*G0_2_2;
    A[2596] = -0.257045070413123*G0_0_0 - 0.160751028806583*G0_0_1 - 0.257045070413123*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000416*G0_1_1 - 0.160751028806582*G0_1_2 - 0.0644569872000413*G0_2_0 - 0.0929015685960116*G0_2_1 - 0.0644569872000414*G0_2_2;
    A[2597] = 0.121346149991981*G0_0_0 + 0.194675758911868*G0_0_1 + 0.121346149991981*G0_0_2 + 0.194675758911867*G0_1_0 - 0.00339247301052916*G0_1_1 + 0.194675758911867*G0_1_2 + 0.132306447410612*G0_2_0 + 0.0589768384907258*G0_2_1 + 0.132306447410612*G0_2_2;
    A[2598] = -0.0528442911255404*G0_0_0 - 0.0887914570640255*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0887914570640257*G0_1_0 + 0.0846813455320394*G0_1_1 - 0.0887914570640258*G0_1_2 - 0.0510175748891023*G0_2_0 - 0.00463203045668289*G0_2_1 - 0.0510175748891024*G0_2_2;
    A[2599] = 0.0212681961813905*G0_0_1 + 0.0212681961813905*G0_1_0 - 0.0693499771190735*G0_1_1 + 0.0212681961813906*G0_1_2 + 0.0118736555368498*G0_2_0 - 0.0146137298915075*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[2600] = -0.0644569872000421*G0_0_0 - 0.160751028806582*G0_0_1 - 0.160751028806582*G0_0_2 - 0.160751028806583*G0_1_0 - 0.257045070413123*G0_1_1 - 0.257045070413123*G0_1_2 - 0.0929015685960118*G0_2_0 - 0.0644569872000416*G0_2_1 - 0.0644569872000417*G0_2_2;
    A[2601] = -0.0033924730105285*G0_0_0 + 0.194675758911867*G0_0_1 + 0.194675758911867*G0_0_2 + 0.194675758911868*G0_1_0 + 0.121346149991982*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0589768384907263*G0_2_0 + 0.132306447410613*G0_2_1 + 0.132306447410613*G0_2_2;
    A[2602] = 0.0846813455320392*G0_0_0 - 0.0887914570640252*G0_0_1 - 0.0887914570640252*G0_0_2 - 0.0887914570640255*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.00463203045668265*G0_2_0 - 0.0510175748891017*G0_2_1 - 0.0510175748891017*G0_2_2;
    A[2603] = -0.0693499771190737*G0_0_0 + 0.0212681961813901*G0_0_1 + 0.0212681961813902*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0146137298915081*G0_2_0 + 0.0118736555368499*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[2604] = -0.0456679059109612*G0_0_0 - 0.0378391220405108*G0_0_1 - 0.0756782440810203*G0_0_2 - 0.0378391220405105*G0_1_0 - 0.0528442911255405*G0_1_2 - 0.0756782440810212*G0_2_0 - 0.0528442911255413*G0_2_1 - 0.105688582251082*G0_2_2;
    A[2605] = -0.0880738185425669*G0_0_0 - 0.0887262171984383*G0_0_1 - 0.100795592332049*G0_0_2 - 0.0887262171984384*G0_1_0 - 0.0880738185425682*G0_1_1 - 0.10079559233205*G0_1_2 - 0.10079559233205*G0_2_0 - 0.10079559233205*G0_2_1 - 0.113517366121531*G0_2_2;
    A[2606] = -0.0378391220405098*G0_0_1 - 0.0528442911255398*G0_0_2 - 0.0378391220405108*G0_1_0 - 0.0456679059109606*G0_1_1 - 0.0756782440810211*G0_1_2 - 0.0528442911255415*G0_2_0 - 0.075678244081022*G0_2_1 - 0.105688582251082*G0_2_2;
    A[2607] = -0.0228339529554804*G0_0_0 - 0.0120693751336113*G0_0_1 - 0.0241387502672228*G0_0_2 - 0.0120693751336115*G0_1_0 - 0.0127217737894824*G0_1_2 - 0.024138750267222*G0_2_0 - 0.0127217737894819*G0_2_1 - 0.0254435475789645*G0_2_2;
    A[2608] = -0.0120693751336113*G0_0_1 - 0.0127217737894817*G0_0_2 - 0.012069375133611*G0_1_0 - 0.0228339529554808*G0_1_1 - 0.0241387502672219*G0_1_2 - 0.0127217737894826*G0_2_0 - 0.0241387502672229*G0_2_1 - 0.0254435475789652*G0_2_2;
    A[2609] = -0.0150051690850301*G0_0_1 - 0.0300103381700603*G0_0_2 - 0.0150051690850301*G0_1_0 - 0.0300103381700603*G0_1_2 - 0.0300103381700614*G0_2_0 - 0.0300103381700611*G0_2_1 - 0.0600206763401204*G0_2_2;
    A[2610] = 0.0521918924696704*G0_0_0 + 0.18789081289081*G0_0_1 + 0.323589733311953*G0_0_2 + 0.187890812890811*G0_1_0 + 0.323589733311952*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311954*G0_2_0 + 0.18789081289081*G0_2_1 + 0.0521918924696695*G0_2_2;
    A[2611] = -0.0880738185425682*G0_0_0 - 0.0972073997247596*G0_0_1 - 0.134067923781463*G0_0_2 - 0.0972073997247602*G0_1_0 - 0.29227459783015*G0_1_1 - 0.224425137619578*G0_1_2 - 0.134067923781464*G0_2_0 - 0.224425137619579*G0_2_1 - 0.156575677409008*G0_2_2;
    A[2612] = 0.0378391220405101*G0_0_1 + 0.0528442911255405*G0_0_2 + 0.0378391220405108*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0900310145101804*G0_1_2 + 0.0528442911255415*G0_2_0 + 0.0900310145101801*G0_2_1 + 0.0521918924696703*G0_2_2;
    A[2613] = -0.0228339529554804*G0_0_0 - 0.0596944770121853*G0_0_1 - 0.0965550010688899*G0_0_2 - 0.0596944770121833*G0_1_0 - 0.0965550010688891*G0_1_1 - 0.0287055408583188*G0_1_2 - 0.0965550010688892*G0_2_0 - 0.0287055408583187*G0_2_1 + 0.0391439193522514*G0_2_2;
    A[2614] = 0.012069375133612*G0_0_1 + 0.0127217737894824*G0_0_2 + 0.012069375133611*G0_1_0 + 0.0502346965020571*G0_1_1 + 0.0446893079271544*G0_1_2 + 0.0127217737894826*G0_2_0 + 0.0446893079271553*G0_2_1 + 0.0391439193522517*G0_2_2;
    A[2615] = 0.0150051690850297*G0_0_1 + 0.0300103381700605*G0_0_2 + 0.0150051690850302*G0_1_0 + 0.0300103381700598*G0_1_1 - 0.00782878387045018*G0_1_2 + 0.0300103381700615*G0_2_0 - 0.00782878387045045*G0_2_1 - 0.0456679059109595*G0_2_2;
    A[2616] = 0.323589733311952*G0_0_0 + 0.187890812890812*G0_0_1 + 0.187890812890811*G0_0_2 + 0.187890812890811*G0_1_0 + 0.05219189246967*G0_1_1 + 0.323589733311952*G0_1_2 + 0.187890812890811*G0_2_0 + 0.323589733311955*G0_2_1 + 0.0521918924696703*G0_2_2;
    A[2617] = -0.29227459783015*G0_0_0 - 0.0972073997247602*G0_0_1 - 0.22442513761958*G0_0_2 - 0.0972073997247601*G0_1_0 - 0.0880738185425686*G0_1_1 - 0.134067923781464*G0_1_2 - 0.22442513761958*G0_2_0 - 0.134067923781464*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2618] = 0.12787013655069*G0_0_0 + 0.037839122040511*G0_0_1 + 0.0900310145101805*G0_0_2 + 0.0378391220405112*G0_1_0 + 0.0528442911255412*G0_1_2 + 0.0900310145101805*G0_2_0 + 0.0528442911255416*G0_2_1 + 0.0521918924696696*G0_2_2;
    A[2619] = -0.0965550010688896*G0_0_0 - 0.0596944770121848*G0_0_1 - 0.0287055408583195*G0_0_2 - 0.0596944770121853*G0_1_0 - 0.0228339529554795*G0_1_1 - 0.0965550010688895*G0_1_2 - 0.0287055408583195*G0_2_0 - 0.0965550010688899*G0_2_1 + 0.0391439193522507*G0_2_2;
    A[2620] = 0.0502346965020579*G0_0_0 + 0.0120693751336111*G0_0_1 + 0.0446893079271548*G0_0_2 + 0.0120693751336112*G0_1_0 + 0.0127217737894818*G0_1_2 + 0.0446893079271555*G0_2_0 + 0.0127217737894817*G0_2_1 + 0.0391439193522525*G0_2_2;
    A[2621] = 0.0300103381700597*G0_0_0 + 0.0150051690850303*G0_0_1 - 0.00782878387045001*G0_0_2 + 0.0150051690850298*G0_1_0 + 0.0300103381700602*G0_1_2 - 0.00782878387045048*G0_2_0 + 0.0300103381700614*G0_2_1 - 0.0456679059109601*G0_2_2;
    A[2622] = 0.647179466623903*G0_0_0 + 0.375781625781621*G0_0_1 + 0.375781625781621*G0_0_2 + 0.375781625781621*G0_1_0 + 0.647179466623903*G0_1_1 + 0.375781625781621*G0_1_2 + 0.375781625781621*G0_2_0 + 0.375781625781621*G0_2_1 + 0.647179466623906*G0_2_2;
    A[2623] = -0.584549195660299*G0_0_0 - 0.321632537344338*G0_0_1 - 0.194414799449518*G0_0_2 - 0.321632537344339*G0_1_0 - 0.268135847562927*G0_1_1 - 0.222141742324031*G0_1_2 - 0.448850275239159*G0_2_0 - 0.290643601190472*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2624] = 0.255740273101381*G0_0_0 + 0.12787013655069*G0_0_1 + 0.0756782440810207*G0_0_2 + 0.127870136550691*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255407*G0_1_2 + 0.180062029020361*G0_2_0 + 0.10503618359521*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2625] = -0.268135847562927*G0_0_0 - 0.321632537344339*G0_0_1 - 0.222141742324031*G0_0_2 - 0.321632537344338*G0_1_0 - 0.5845491956603*G0_1_1 - 0.19441479944952*G0_1_2 - 0.290643601190471*G0_2_0 - 0.448850275239159*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2626] = 0.172233245149909*G0_0_0 + 0.166035457919136*G0_0_1 + 0.100795592332049*G0_0_2 + 0.166035457919135*G0_1_0 + 0.172233245149909*G0_1_1 + 0.100795592332049*G0_1_2 + 0.198655390712678*G0_2_0 + 0.19865539071268*G0_2_1 + 0.113517366121532*G0_2_2;
    A[2627] = 0.105688582251081*G0_0_0 + 0.127870136550691*G0_0_1 + 0.0528442911255401*G0_0_2 + 0.127870136550691*G0_1_0 + 0.255740273101382*G0_1_1 + 0.0756782440810213*G0_1_2 + 0.105036183595211*G0_2_0 + 0.180062029020361*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2628] = -0.150051690850301*G0_0_0 - 0.335985307773499*G0_0_1 - 0.466465038947673*G0_0_2 - 0.335985307773498*G0_1_0 - 0.1500516908503*G0_1_1 - 0.466465038947671*G0_1_2 - 0.466465038947672*G0_2_0 - 0.466465038947673*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2629] = 0.143527704291592*G0_0_0 + 0.185933616923198*G0_0_1 + 0.218553549716741*G0_0_2 + 0.185933616923198*G0_1_0 + 0.176147637085137*G0_1_1 + 0.234863516113514*G0_1_2 + 0.21855354971674*G0_2_0 + 0.234863516113514*G0_2_1 + 0.293579395141891*G0_2_2;
    A[2630] = 0.176147637085135*G0_0_0 + 0.185933616923197*G0_0_1 + 0.234863516113513*G0_0_2 + 0.185933616923199*G0_1_0 + 0.143527704291591*G0_1_1 + 0.218553549716742*G0_1_2 + 0.234863516113514*G0_2_0 + 0.218553549716741*G0_2_1 + 0.293579395141893*G0_2_2;
    A[2631] = 0.045667905910961*G0_0_0 + 0.071763852145796*G0_0_1 + 0.120693751336112*G0_0_2 + 0.071763852145795*G0_1_0 + 0.0456679059109601*G0_1_1 + 0.120693751336111*G0_1_2 + 0.120693751336111*G0_2_0 + 0.120693751336113*G0_2_1 + 0.19571959676126*G0_2_2;
    A[2632] = -0.0298472385060921*G0_0_0 - 0.0298472385060921*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0298472385060919*G0_1_0 - 0.0298472385060919*G0_1_1 - 0.0298472385060919*G0_1_2 - 0.0345719095719088*G0_2_0 - 0.0345719095719088*G0_2_1 - 0.0345719095719088*G0_2_2;
    A[2633] = -0.0298472385060925*G0_0_0 + 0.00472467106581673*G0_2_0;
    A[2634] = 0.0117431758056758*G0_1_1 + 0.00227817610630102*G0_2_1;
    A[2635] = -0.00946499969937467*G0_2_2;
    A[2636] = -0.0091335811821928*G0_1_1 - 0.00583896797004477*G0_1_2 - 0.00583896797004427*G0_2_1 - 0.00254435475789639*G0_2_2;
    A[2637] = -0.0156575677409011*G0_1_1 - 0.013080593050211*G0_1_2 - 0.0130805930502113*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[2638] = -0.000652398655870701*G0_1_1 + 0.0019245760348187*G0_1_2 + 0.00192457603481927*G0_2_1 + 0.00450155072550905*G0_2_2;
    A[2639] = 0.003294613212148*G0_1_2 + 0.0032946132121482*G0_2_1 + 0.00658922642429634*G0_2_2;
    A[2640] = 0.0642612676032805*G0_0_0 + 0.0422754329004324*G0_0_2 + 0.0422754329004317*G0_2_0 + 0.0429604514890962*G0_2_2;
    A[2641] = 0.00880738185425737*G0_0_0 + 0.00649136662591535*G0_0_2 + 0.00649136662591579*G0_2_0 + 0.00352295274170324*G0_2_2;
    A[2642] = -0.000652398655871396*G0_0_0 - 0.00949240044292144*G0_0_2 - 0.00949240044292202*G0_2_0 - 0.0189848008858434*G0_2_2;
    A[2643] = 0.00329461321214819*G0_0_2 + 0.00329461321214819*G0_2_0 + 0.0292600797158091*G0_2_2;
    A[2644] = 0.128522535206561*G0_0_0 + 0.0422754329004325*G0_0_1 + 0.0422754329004322*G0_1_0 + 0.0845508658008642*G0_2_0 + 0.0429604514890961*G0_2_1;
    A[2645] = 0.0371867233846394*G0_0_0 + 0.0743408268364851*G0_0_1 + 0.0743408268364855*G0_1_0 + 0.104709984267274*G0_1_1 + 0.0808321934624009*G0_2_0 + 0.108232937008978*G0_2_1;
    A[2646] = -0.00619778723077336*G0_0_0 + 0.0188869410874622*G0_0_1 + 0.0188869410874617*G0_1_0 + 0.0567586830607662*G0_1_1 + 0.00939454064454042*G0_2_0 + 0.0377738821749231*G0_2_1;
    A[2647] = -0.00583896797004473*G0_0_1 - 0.00583896797004485*G0_1_0 - 0.0435476102793816*G0_1_1 - 0.00254435475789657*G0_2_0 - 0.0142875305635722*G0_2_1;
    A[2648] = 0.0642612676032806*G0_0_0 + 0.0642612676032806*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0642612676032806*G0_1_0 + 0.0642612676032806*G0_1_1 + 0.0219858347028482*G0_1_2 + 0.0219858347028484*G0_2_0 + 0.0219858347028484*G0_2_1 + 0.0226708532915127*G0_2_2;
    A[2649] = 0.00880738185425655*G0_0_0 + 0.00880738185425657*G0_0_1 + 0.00231601522834145*G0_0_2 + 0.00880738185425695*G0_1_0 + 0.00880738185425701*G0_1_1 + 0.0023160152283416*G0_1_2 + 0.0023160152283416*G0_2_0 + 0.00231601522834163*G0_2_1 - 0.000652398655870858*G0_2_2;
    A[2650] = -0.000652398655870754*G0_0_0 - 0.000652398655870788*G0_0_1 + 0.00884000178705026*G0_0_2 - 0.000652398655871122*G0_1_0 - 0.000652398655871138*G0_1_1 + 0.00884000178705016*G0_1_2 + 0.00884000178705075*G0_2_0 + 0.00884000178705074*G0_2_1 - 0.000652398655870952*G0_2_2;
    A[2651] = -0.00329461321214786*G0_0_2 - 0.00329461321214771*G0_1_2 - 0.00329461321214808*G0_2_0 - 0.00329461321214808*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[2652] = 0.128522535206561*G0_0_0 + 0.0862471023061288*G0_0_1 + 0.128522535206561*G0_0_2 + 0.0862471023061283*G0_1_0 + 0.0439716694056957*G0_1_1 + 0.0862471023061283*G0_1_2 + 0.0439716694056963*G0_2_0 + 0.0446566879943604*G0_2_1 + 0.0439716694056963*G0_2_2;
    A[2653] = 0.0371867233846399*G0_0_0 - 0.0371541034518456*G0_0_1 + 0.0371867233846399*G0_0_2 - 0.0371541034518453*G0_1_0 - 0.0067849460210558*G0_1_1 - 0.0371541034518453*G0_1_2 - 0.0436454700777611*G0_2_0 - 0.00975335990526873*G0_2_1 - 0.0436454700777612*G0_2_2;
    A[2654] = -0.0061977872307736*G0_0_0 - 0.0250847283182352*G0_0_1 - 0.0061977872307735*G0_0_2 - 0.0250847283182359*G0_1_0 + 0.0127870136550684*G0_1_1 - 0.0250847283182359*G0_1_2 - 0.0155923278753141*G0_2_0 + 0.00329461321214743*G0_2_1 - 0.0155923278753142*G0_2_2;
    A[2655] = 0.00583896797004429*G0_0_1 + 0.00583896797004478*G0_1_0 - 0.0318696743392919*G0_1_1 + 0.0058389679700448*G0_1_2 + 0.00254435475789653*G0_2_0 - 0.00590420783563113*G0_2_1 + 0.00254435475789658*G0_2_2;
    A[2656] = 0.092086070276173*G0_0_0 + 0.134361503176605*G0_0_1 + 0.134361503176605*G0_0_2 + 0.134361503176605*G0_1_0 + 0.176636936077038*G0_1_1 + 0.176636936077038*G0_1_2 + 0.128000616281864*G0_2_0 + 0.1273155976932*G0_2_1 + 0.1273155976932*G0_2_2;
    A[2657] = -0.0622388317700803*G0_0_0 - 0.250814663249556*G0_0_1 - 0.250814663249555*G0_0_2 - 0.250814663249556*G0_1_0 - 0.334680510461756*G0_1_1 - 0.334680510461756*G0_1_2 - 0.160783648739375*G0_2_0 - 0.166948816037355*G0_2_1 - 0.166948816037355*G0_2_2;
    A[2658] = -0.0622388317700818*G0_0_0 + 0.188575831479474*G0_0_1 + 0.188575831479474*G0_0_2 + 0.188575831479474*G0_1_0 + 0.104709984267274*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0985448169692944*G0_2_0 + 0.182410664181495*G0_2_1 + 0.182410664181495*G0_2_2;
    A[2659] = 0.0920860702761738*G0_0_0 - 0.0422754329004321*G0_0_1 - 0.0422754329004322*G0_0_2 - 0.0422754329004319*G0_1_0 - 0.0359145460056911*G0_2_0 - 0.042960451489096*G0_2_1 - 0.042960451489096*G0_2_2;
    A[2660] = 0.0391439193522524*G0_0_0 + 0.0678494602105706*G0_0_1 + 0.13569892042114*G0_0_2 + 0.0678494602105694*G0_1_0 + 0.104709984267273*G0_1_2 + 0.135698920421141*G0_2_0 + 0.104709984267273*G0_2_1 + 0.209419968534548*G0_2_2;
    A[2661] = 0.0195719596761255*G0_0_0 + 0.0606730749959902*G0_0_1 + 0.0748627457611819*G0_0_2 + 0.0606730749959907*G0_1_0 + 0.0929668084615983*G0_1_1 + 0.121346149991982*G0_1_2 + 0.074862745761182*G0_2_0 + 0.121346149991982*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2662] = -0.00554538857490242*G0_0_1 - 0.00619778723077363*G0_0_2 - 0.0055453885749035*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0264221455627691*G0_1_2 - 0.00619778723077384*G0_2_0 + 0.0264221455627695*G0_2_1 + 0.0137003717732875*G0_2_2;
    A[2663] = -0.00489298991903233*G0_0_0 + 0.0141896707651915*G0_0_1 + 0.0283793415303834*G0_0_2 + 0.0141896707651912*G0_1_0 + 0.028379341530383*G0_1_2 + 0.0283793415303817*G0_2_0 + 0.028379341530383*G0_2_1 + 0.0567586830607666*G0_2_2;
    A[2664] = -0.0102752788299658*G0_0_1 - 0.0181040627004164*G0_0_2 - 0.0102752788299664*G0_1_0 + 0.0195719596761255*G0_1_1 + 0.0014678969757095*G0_1_2 - 0.0181040627004164*G0_2_0 + 0.00146789697570932*G0_2_1 - 0.0166361657247072*G0_2_2;
    A[2665] = -0.000652398655870921*G0_0_1 - 0.00130479731174181*G0_0_2 - 0.000652398655871741*G0_1_0 - 0.012721773789483*G0_1_2 - 0.00130479731174192*G0_2_0 - 0.0127217737894831*G0_2_1 - 0.0254435475789657*G0_2_2;
    A[2666] = 0.0391439193522515*G0_0_0 - 0.028705540858318*G0_0_1 - 0.0965550010688888*G0_0_2 - 0.0287055408583178*G0_1_0 - 0.0965550010688887*G0_1_1 - 0.0596944770121841*G0_1_2 - 0.0965550010688886*G0_2_0 - 0.0596944770121848*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[2667] = 0.0195719596761261*G0_0_0 - 0.0411011153198653*G0_0_1 - 0.0552907860850566*G0_0_2 - 0.0411011153198671*G0_1_0 - 0.0088073818542576*G0_1_1 + 0.00538228891093366*G0_1_2 - 0.0552907860850583*G0_2_0 + 0.00538228891093448*G0_2_1 + 0.0195719596761256*G0_2_2;
    A[2668] = 0.00554538857490306*G0_0_1 + 0.00619778723077351*G0_0_2 + 0.00554538857490348*G0_1_0 + 0.0502346965020585*G0_1_1 + 0.0381653213684468*G0_1_2 + 0.00619778723077389*G0_2_0 + 0.0381653213684464*G0_2_1 + 0.0260959462348351*G0_2_2;
    A[2669] = -0.00489298991903069*G0_0_0 - 0.0190826606842226*G0_0_1 - 0.0332723314494141*G0_0_2 - 0.0190826606842233*G0_1_0 - 0.0332723314494146*G0_1_1 - 0.0190826606842229*G0_1_2 - 0.0332723314494143*G0_2_0 - 0.0190826606842233*G0_2_1 - 0.00489298991903103*G0_2_2;
    A[2670] = 0.0102752788299656*G0_0_1 + 0.0181040627004163*G0_0_2 + 0.0102752788299665*G0_1_0 + 0.0401225173360584*G0_1_1 + 0.029847238506093*G0_1_2 + 0.0181040627004166*G0_2_0 + 0.0298472385060926*G0_2_1 + 0.0195719596761267*G0_2_2;
    A[2671] = 0.000652398655870642*G0_0_1 + 0.00130479731174128*G0_0_2 + 0.000652398655871691*G0_1_0 + 0.00130479731174202*G0_1_1 - 0.0107645778218693*G0_1_2 + 0.00130479731174183*G0_2_0 - 0.01076457782187*G0_2_1 - 0.0228339529554815*G0_2_2;
    A[2672] = -0.29227459783015*G0_0_0 - 0.22442513761958*G0_0_1 - 0.09720739972476*G0_0_2 - 0.224425137619579*G0_1_0 - 0.156575677409009*G0_1_1 - 0.134067923781464*G0_1_2 - 0.0972073997247592*G0_2_0 - 0.134067923781463*G0_2_1 - 0.0880738185425673*G0_2_2;
    A[2673] = 0.456026660453738*G0_0_0 + 0.228013330226869*G0_0_1 + 0.228013330226869*G0_0_2 + 0.228013330226869*G0_1_0 + 0.0929668084615992*G0_1_1 + 0.36305985199214*G0_1_2 + 0.228013330226869*G0_2_0 + 0.363059851992139*G0_2_1 + 0.0929668084615984*G0_2_2;
    A[2674] = -0.292274597830149*G0_0_0 - 0.0678494602105706*G0_0_1 - 0.19506719810539*G0_0_2 - 0.0678494602105702*G0_1_0 - 0.104709984267274*G0_1_2 - 0.195067198105389*G0_2_0 - 0.104709984267274*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2675] = -0.00880738185425671*G0_0_0 + 0.00538228891093522*G0_0_1 - 0.0411011153198637*G0_0_2 + 0.00538228891093496*G0_1_0 + 0.0195719596761263*G0_1_1 - 0.0552907860850561*G0_1_2 - 0.041101115319865*G0_2_0 - 0.055290786085056*G0_2_1 + 0.0195719596761269*G0_2_2;
    A[2676] = -0.00880738185425746*G0_0_0 - 0.0141896707651914*G0_0_1 + 0.0322937334656083*G0_0_2 - 0.014189670765192*G0_1_0 - 0.028379341530384*G0_1_2 + 0.0322937334656078*G0_2_0 - 0.028379341530383*G0_2_1 + 0.0929668084616004*G0_2_2;
    A[2677] = 0.00130479731174202*G0_0_0 + 0.000652398655870683*G0_0_1 + 0.000652398655870337*G0_0_2 + 0.000652398655870866*G0_1_0 + 0.0127217737894823*G0_1_2 + 0.000652398655871123*G0_2_0 + 0.0127217737894829*G0_2_1 - 0.0880738185425685*G0_2_2;
    A[2678] = -0.584549195660299*G0_0_0 - 0.321632537344339*G0_0_1 - 0.448850275239159*G0_0_2 - 0.321632537344338*G0_1_0 - 0.268135847562927*G0_1_1 - 0.290643601190472*G0_1_2 - 0.194414799449519*G0_2_0 - 0.222141742324031*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2679] = 0.912053320907477*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453738*G0_0_2 + 0.456026660453738*G0_1_0 + 0.72611970398428*G0_1_1 + 0.456026660453739*G0_1_2 + 0.456026660453738*G0_2_0 + 0.456026660453739*G0_2_1 + 0.72611970398428*G0_2_2;
    A[2680] = -0.5845491956603*G0_0_0 - 0.26291665831596*G0_0_1 - 0.135698920421141*G0_0_2 - 0.262916658315961*G0_1_0 - 0.209419968534549*G0_1_1 - 0.104709984267274*G0_1_2 - 0.39013439621078*G0_2_0 - 0.290643601190473*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2681] = -0.0371867233846405*G0_0_0 + 0.186912214907004*G0_0_1 + 0.0376760223765421*G0_0_2 + 0.186912214907003*G0_1_0 - 0.0371867233846416*G0_1_1 + 0.140428810676203*G0_1_2 + 0.140428810676204*G0_2_0 + 0.0376760223765416*G0_2_1 + 0.065566064915021*G0_2_2;
    A[2682] = -0.037186723384639*G0_0_0 - 0.224098938291645*G0_0_1 - 0.0748627457611819*G0_0_2 - 0.224098938291644*G0_1_0 - 0.448197876583289*G0_1_1 - 0.121346149991981*G0_1_2 - 0.177615534060844*G0_2_0 - 0.326851726591307*G0_2_1 - 0.149725491522365*G0_2_2;
    A[2683] = 0.0123955744615469*G0_0_0 + 0.00619778723077385*G0_0_1 + 0.00619778723077321*G0_0_2 + 0.00619778723077413*G0_1_0 + 0.172233245149912*G0_1_1 - 0.0264221455627693*G0_1_2 + 0.00619778723077364*G0_2_0 + 0.0714376528178614*G0_2_1 - 0.0137003717732879*G0_2_2;
    A[2684] = -0.0391439193522528*G0_0_0 + 0.249542485870608*G0_0_1 + 0.44526208263187*G0_0_2 + 0.249542485870608*G0_1_0 + 0.352295274170269*G0_1_1 + 0.249542485870607*G0_1_2 + 0.445262082631869*G0_2_0 + 0.249542485870606*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2685] = -0.0391439193522515*G0_0_0 - 0.288686405222859*G0_0_1 - 0.484406001984122*G0_0_2 - 0.28868640522286*G0_1_0 - 0.185933616923197*G0_1_1 - 0.484406001984121*G0_1_2 - 0.484406001984121*G0_2_0 - 0.484406001984121*G0_2_1 - 0.782878387045046*G0_2_2;
    A[2686] = -0.0391439193522512*G0_0_0 - 0.0195719596761255*G0_0_1 - 0.0195719596761252*G0_0_2 - 0.0195719596761235*G0_1_0 - 0.234863516113513*G0_1_1 - 0.141896707651912*G0_1_2 - 0.0195719596761238*G0_2_0 - 0.141896707651913*G0_2_1 - 0.0489298991903133*G0_2_2;
    A[2687] = 0.00978597983806301*G0_0_0 + 0.0048929899190308*G0_0_1 + 0.00489298991903148*G0_0_2 + 0.00489298991903197*G0_1_0 - 0.0391439193522516*G0_1_1 + 0.0538228891093479*G0_1_2 + 0.00489298991903254*G0_2_0 + 0.0538228891093474*G0_2_1 + 0.146789697570948*G0_2_2;
    A[2688] = 0.0189195610202549*G0_0_0 + 0.0189195610202549*G0_0_1 + 0.0189195610202549*G0_0_2 + 0.0189195610202551*G0_1_0 + 0.0189195610202551*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.0137421252872638*G0_2_0 + 0.0137421252872638*G0_2_1 + 0.0137421252872638*G0_2_2;
    A[2689] = 0.0335985307773501*G0_0_0 - 0.0143945239431346*G0_2_0;
    A[2690] = 0.0189195610202553*G0_1_1 + 0.00517743573299135*G0_2_1;
    A[2691] = -0.0137421252872638*G0_2_2;
    A[2692] = -0.0306627368259312*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.0212681961813909*G0_2_1 - 0.01187365553685*G0_2_2;
    A[2693] = -0.00130479731174155*G0_1_1 - 0.0012395574461545*G0_1_2 - 0.00123955744615444*G0_2_1 - 0.00117431758056773*G0_2_2;
    A[2694] = -0.0150051690850302*G0_1_1 - 0.0149399292194432*G0_1_2 - 0.0149399292194435*G0_2_1 - 0.0148746893538559*G0_2_2;
    A[2695] = 0.00939454064454052*G0_1_2 + 0.00939454064454044*G0_2_1 + 0.0187890812890805*G0_2_2;
    A[2696] = -0.128522535206561*G0_0_0 - 0.0962940416065402*G0_0_2 - 0.0962940416065407*G0_2_0 - 0.12473862300251*G0_2_2;
    A[2697] = 0.0476251018785734*G0_0_0 + 0.0623693115012549*G0_0_2 + 0.0623693115012547*G0_2_0 + 0.124738623002509*G0_2_2;
    A[2698] = -0.0150051690850297*G0_0_0 - 0.0377738821749228*G0_0_2 - 0.0377738821749232*G0_2_0 - 0.127087258163644*G0_2_2;
    A[2699] = 0.00939454064454019*G0_0_2 + 0.00939454064454054*G0_2_0 + 0.0641307878721059*G0_2_2;
    A[2700] = -0.257045070413123*G0_0_0 - 0.0962940416065403*G0_0_1 - 0.0962940416065403*G0_1_0 - 0.192588083213081*G0_2_0 - 0.12473862300251*G0_2_1;
    A[2701] = 0.121346149991982*G0_0_0 - 0.0733296089198857*G0_0_1 - 0.0733296089198856*G0_1_0 - 0.271397840842282*G0_1_1 - 0.0109602974186302*G0_2_0 - 0.146659217839772*G0_2_1;
    A[2702] = -0.0528442911255401*G0_0_0 + 0.0359471659384849*G0_0_1 + 0.0359471659384846*G0_1_0 + 0.20941996853455*G0_1_1 - 0.00182671623643884*G0_2_0 + 0.0823327103709041*G0_2_1;
    A[2703] = -0.0212681961813903*G0_0_1 - 0.0212681961813905*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0118736555368501*G0_2_0 - 0.0477555816097483*G0_2_1;
    A[2704] = -0.0306627368259308*G0_0_0 - 0.0306627368259307*G0_0_1 - 0.00939454064454022*G0_0_2 - 0.0306627368259313*G0_1_0 - 0.0306627368259313*G0_1_1 - 0.0093945406445408*G0_1_2 - 0.00939454064454031*G0_2_0 - 0.00939454064454026*G0_2_1;
    A[2705] = -0.00130479731174213*G0_0_0 - 0.00130479731174223*G0_0_1 - 6.5239865587674e-05*G0_0_2 - 0.00130479731174141*G0_1_0 - 0.00130479731174143*G0_1_1 - 6.52398655867706e-05*G0_1_2 - 6.52398655873991e-05*G0_2_0 - 6.523986558755e-05*G0_2_1;
    A[2706] = -0.0150051690850299*G0_0_0 - 0.0150051690850299*G0_0_1 - 6.52398655865114e-05*G0_0_2 - 0.0150051690850304*G0_1_0 - 0.0150051690850304*G0_1_1 - 6.52398655873643e-05*G0_1_2 - 6.52398655867258e-05*G0_2_0 - 6.5239865586677e-05*G0_2_1;
    A[2707] = -0.00939454064454084*G0_0_2 - 0.00939454064454052*G0_1_2 - 0.00939454064454061*G0_2_0 - 0.00939454064454057*G0_2_1;
    A[2708] = -0.0613254736518615*G0_0_0 - 0.0400572774704709*G0_0_1 - 0.0613254736518616*G0_0_2 - 0.0400572774704717*G0_1_0 - 0.018789081289081*G0_1_1 - 0.0400572774704717*G0_1_2 - 0.0187890812890808*G0_2_0 - 0.00939454064454003*G0_2_1 - 0.0187890812890807*G0_2_2;
    A[2709] = -0.0254435475789639*G0_0_0 + 0.0136351319077006*G0_0_1 - 0.0254435475789638*G0_0_2 + 0.0136351319077015*G0_1_0 - 0.000130479731174103*G0_1_1 + 0.0136351319077016*G0_1_2 + 0.0148746893538559*G0_2_0 - 6.52398655877838e-05*G0_2_1 + 0.0148746893538557*G0_2_2;
    A[2710] = -0.0528442911255402*G0_0_0 - 0.0137656116388748*G0_0_1 - 0.0528442911255403*G0_0_2 - 0.0137656116388758*G0_1_0 - 0.000130479731174212*G0_1_1 - 0.0137656116388758*G0_1_2 + 0.00117431758056708*G0_2_0 - 6.52398655864219e-05*G0_2_1 + 0.00117431758056708*G0_2_2;
    A[2711] = 0.0212681961813901*G0_0_1 + 0.0212681961813905*G0_1_0 - 0.0187890812890812*G0_1_1 + 0.0212681961813905*G0_1_2 + 0.0118736555368501*G0_2_0 - 0.00939454064454095*G0_2_1 + 0.0118736555368502*G0_2_2;
    A[2712] = -0.0693499771190729*G0_0_0 - 0.0906181733004635*G0_0_1 - 0.0906181733004636*G0_0_2 - 0.0906181733004636*G0_1_0 - 0.111886369481854*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0547362472275655*G0_2_0 - 0.0641307878721061*G0_2_1 - 0.0641307878721062*G0_2_2;
    A[2713] = 0.0846813455320379*G0_0_0 + 0.173472802596064*G0_0_1 + 0.173472802596064*G0_0_2 + 0.173472802596064*G0_1_0 + 0.209419968534549*G0_1_1 + 0.209419968534549*G0_1_2 + 0.0893133759887214*G0_2_0 + 0.127087258163645*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2714] = -0.00339247301052739*G0_0_0 - 0.198068231922396*G0_0_1 - 0.198068231922396*G0_0_2 - 0.198068231922396*G0_1_0 - 0.271397840842282*G0_1_1 - 0.271397840842282*G0_1_2 - 0.0623693115012541*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2715] = -0.0644569872000427*G0_0_0 + 0.0962940416065401*G0_0_1 + 0.0962940416065402*G0_0_2 + 0.0962940416065403*G0_1_0 + 0.0284445813959691*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[2716] = 0.0521918924696691*G0_0_0 - 0.135698920421141*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421142*G0_1_0 - 0.271397840842282*G0_1_2 - 0.271397840842283*G0_2_0 - 0.271397840842283*G0_2_1 - 0.542795681684565*G0_2_2;
    A[2717] = -0.0880738185425674*G0_0_0 + 0.00913358118219246*G0_0_1 + 0.0459941052388966*G0_0_2 + 0.00913358118219163*G0_1_0 - 0.185933616923199*G0_1_1 - 0.081223632655924*G0_1_2 + 0.0459941052388959*G0_2_0 - 0.0812236326559225*G0_2_1 + 0.0234863516113516*G0_2_2;
    A[2718] = -0.0378391220405106*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0378391220405107*G0_1_0 + 0.0521918924696692*G0_1_1 - 0.00065239865587139*G0_1_2 - 0.0528442911255415*G0_2_0 - 0.00065239865587235*G0_2_1 - 0.0534966897814128*G0_2_2;
    A[2719] = -0.0228339529554796*G0_0_0 + 0.0368605240567046*G0_0_1 + 0.0737210481134082*G0_0_2 + 0.0368605240567044*G0_1_0 + 0.104709984267273*G0_1_2 + 0.0737210481134101*G0_2_0 + 0.104709984267275*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2720] = -0.0120693751336116*G0_0_1 - 0.0127217737894828*G0_0_2 - 0.0120693751336111*G0_1_0 + 0.0260959462348355*G0_1_1 + 0.0198981590040621*G0_1_2 - 0.0127217737894826*G0_2_0 + 0.0198981590040616*G0_2_1 + 0.0137003717732875*G0_2_2;
    A[2721] = -0.01500516908503*G0_0_1 - 0.0300103381700597*G0_0_2 - 0.0150051690850294*G0_1_0 - 0.0528442911255394*G0_1_2 - 0.030010338170061*G0_2_0 - 0.0528442911255407*G0_2_1 - 0.10568858225108*G0_2_2;
    A[2722] = -0.0456679059109602*G0_0_0 - 0.00782878387045041*G0_0_1 + 0.0300103381700595*G0_0_2 - 0.00782878387045039*G0_1_0 + 0.0300103381700606*G0_1_1 + 0.0150051690850305*G0_1_2 + 0.0300103381700609*G0_2_0 + 0.0150051690850301*G0_2_1;
    A[2723] = -0.0880738185425678*G0_0_0 + 0.000652398655870678*G0_0_1 + 0.0127217737894816*G0_0_2 + 0.000652398655870933*G0_1_0 + 0.00130479731174116*G0_1_1 + 0.000652398655870722*G0_1_2 + 0.0127217737894812*G0_2_0 + 0.000652398655869767*G0_2_1;
    A[2724] = 0.0378391220405107*G0_0_1 + 0.0528442911255402*G0_0_2 + 0.0378391220405108*G0_1_0 + 0.0300103381700605*G0_1_1 + 0.0150051690850303*G0_1_2 + 0.0528442911255417*G0_2_0 + 0.0150051690850312*G0_2_1;
    A[2725] = -0.0228339529554813*G0_0_0 - 0.0107645778218691*G0_0_1 + 0.0013047973117423*G0_0_2 - 0.0107645778218697*G0_1_0 + 0.00130479731174131*G0_1_1 + 0.000652398655870444*G0_1_2 + 0.00130479731174243*G0_2_0 + 0.000652398655871905*G0_2_1;
    A[2726] = 0.0120693751336111*G0_0_1 + 0.0127217737894827*G0_0_2 + 0.0120693751336109*G0_1_0 + 0.00130479731174189*G0_1_1 + 0.00065239865587078*G0_1_2 + 0.0127217737894822*G0_2_0 + 0.000652398655870022*G0_2_1;
    A[2727] = 0.0150051690850305*G0_0_1 + 0.0300103381700603*G0_0_2 + 0.0150051690850296*G0_1_0 + 0.0300103381700606*G0_1_1 + 0.0150051690850305*G0_1_2 + 0.0300103381700613*G0_2_0 + 0.0150051690850304*G0_2_1;
    A[2728] = 0.12787013655069*G0_0_0 + 0.0900310145101804*G0_0_1 + 0.0378391220405104*G0_0_2 + 0.0900310145101802*G0_1_0 + 0.0521918924696694*G0_1_1 + 0.0528442911255408*G0_1_2 + 0.0378391220405105*G0_2_0 + 0.0528442911255413*G0_2_1;
    A[2729] = -0.29227459783015*G0_0_0 - 0.19506719810539*G0_0_1 - 0.0678494602105698*G0_0_2 - 0.19506719810539*G0_1_0 - 0.185933616923199*G0_1_1 - 0.104709984267274*G0_1_2 - 0.0678494602105708*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2730] = 0.323589733311951*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421141*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842283*G0_1_2 + 0.135698920421141*G0_2_0 + 0.271397840842283*G0_2_1;
    A[2731] = 0.0502346965020579*G0_0_0 + 0.0381653213684455*G0_0_1 + 0.00554538857490187*G0_0_2 + 0.0381653213684452*G0_1_0 + 0.0260959462348343*G0_1_1 + 0.00619778723077225*G0_1_2 + 0.00554538857490237*G0_2_0 + 0.00619778723077316*G0_2_1;
    A[2732] = -0.0965550010688887*G0_0_0 - 0.0368605240567047*G0_0_1 - 0.0678494602105715*G0_0_2 - 0.0368605240567033*G0_1_0 - 0.104709984267275*G0_1_2 - 0.06784946021057*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2733] = 0.0300103381700596*G0_0_0 + 0.0150051690850301*G0_0_1 + 0.0378391220405112*G0_0_2 + 0.0150051690850304*G0_1_0 + 0.0528442911255412*G0_1_2 + 0.0378391220405101*G0_2_0 + 0.0528442911255408*G0_2_1;
    A[2734] = 0.255740273101381*G0_0_0 + 0.127870136550691*G0_0_1 + 0.180062029020361*G0_0_2 + 0.12787013655069*G0_1_0 + 0.105688582251081*G0_1_1 + 0.10503618359521*G0_1_2 + 0.0756782440810207*G0_2_0 + 0.0528442911255407*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2735] = -0.5845491956603*G0_0_0 - 0.262916658315961*G0_0_1 - 0.39013439621078*G0_0_2 - 0.26291665831596*G0_1_0 - 0.209419968534549*G0_1_1 - 0.290643601190473*G0_1_2 - 0.135698920421141*G0_2_0 - 0.104709984267274*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2736] = 0.647179466623903*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684565*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684564*G0_2_2;
    A[2737] = 0.17223324514991*G0_0_0 + 0.00619778723077282*G0_0_1 + 0.0714376528178608*G0_0_2 + 0.00619778723077336*G0_1_0 + 0.0123955744615467*G0_1_1 + 0.00619778723077338*G0_1_2 - 0.0264221455627698*G0_2_0 + 0.00619778723077215*G0_2_1 - 0.0137003717732889*G0_2_2;
    A[2738] = -0.268135847562928*G0_0_0 + 0.0534966897814118*G0_0_1 - 0.0459941052388966*G0_0_2 + 0.0534966897814106*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0812236326559238*G0_1_2 + 0.0225077536275439*G0_2_0 - 0.104709984267275*G0_2_1 - 0.0234863516113512*G0_2_2;
    A[2739] = 0.10568858225108*G0_0_0 - 0.0221815542996099*G0_0_1 + 0.0528442911255406*G0_0_2 - 0.0221815542996088*G0_1_0 + 0.105688582251082*G0_1_1 + 0.000652398655871575*G0_1_2 + 0.000652398655871752*G0_2_0 + 0.0528442911255416*G0_2_1 + 0.0534966897814126*G0_2_2;
    A[2740] = 0.143527704291592*G0_0_0 - 0.0424059126316064*G0_0_1 - 0.075025845425151*G0_0_2 - 0.042405912631607*G0_1_0 - 0.0521918924696685*G0_1_1 - 0.0260959462348345*G0_1_2 - 0.0750258454251495*G0_2_0 - 0.0260959462348346*G0_2_1;
    A[2741] = -0.150051690850301*G0_0_0 + 0.185933616923198*G0_0_1 + 0.316413348097373*G0_0_2 + 0.185933616923199*G0_1_0 + 0.371867233846397*G0_1_1 + 0.185933616923199*G0_1_2 + 0.316413348097372*G0_2_0 + 0.185933616923198*G0_2_1;
    A[2742] = 0.176147637085135*G0_0_0 - 0.0097859798380635*G0_0_1 - 0.0587158790283786*G0_0_2 - 0.00978597983806262*G0_1_0 - 0.0521918924696698*G0_1_1 - 0.0260959462348357*G0_1_2 - 0.058715879028377*G0_2_0 - 0.0260959462348348*G0_2_1;
    A[2743] = 0.0456679059109608*G0_0_0 - 0.026095946234834*G0_0_1 - 0.0750258454251503*G0_0_2 - 0.0260959462348346*G0_1_0 - 0.0521918924696703*G0_1_1 - 0.0260959462348354*G0_1_2 - 0.0750258454251524*G0_2_0 - 0.0260959462348346*G0_2_1;
    A[2744] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060923*G0_0_2 - 0.0298472385060925*G0_1_0 - 0.0298472385060925*G0_1_1 - 0.0298472385060926*G0_1_2 - 0.0345719095719093*G0_2_0 - 0.0345719095719094*G0_2_1 - 0.0345719095719094*G0_2_2;
    A[2745] = 0.0117431758056758*G0_0_0 + 0.00227817610630125*G0_2_0;
    A[2746] = -0.0298472385060924*G0_1_1 + 0.00472467106581685*G0_2_1;
    A[2747] = -0.00946499969937474*G0_2_2;
    A[2748] = 0.0642612676032814*G0_1_1 + 0.0422754329004329*G0_1_2 + 0.0422754329004327*G0_2_1 + 0.0429604514890971*G0_2_2;
    A[2749] = 0.00880738185425689*G0_1_1 + 0.00649136662591501*G0_1_2 + 0.00649136662591545*G0_2_1 + 0.00352295274170279*G0_2_2;
    A[2750] = -0.000652398655870402*G0_1_1 - 0.00949240044292028*G0_1_2 - 0.00949240044292083*G0_2_1 - 0.0189848008858418*G0_2_2;
    A[2751] = 0.00329461321214812*G0_1_2 + 0.00329461321214796*G0_2_1 + 0.0292600797158088*G0_2_2;
    A[2752] = -0.00913358118219213*G0_0_0 - 0.0058389679700442*G0_0_2 - 0.00583896797004434*G0_2_0 - 0.00254435475789614*G0_2_2;
    A[2753] = -0.0156575677409006*G0_0_0 - 0.0130805930502107*G0_0_2 - 0.0130805930502107*G0_2_0 - 0.0105036183595211*G0_2_2;
    A[2754] = -0.000652398655871467*G0_0_0 + 0.00192457603481872*G0_0_2 + 0.00192457603481913*G0_2_0 + 0.00450155072550923*G0_2_2;
    A[2755] = 0.00329461321214794*G0_0_2 + 0.00329461321214795*G0_2_0 + 0.00658922642429594*G0_2_2;
    A[2756] = -0.0435476102793806*G0_0_0 - 0.0058389679700444*G0_0_1 - 0.00583896797004396*G0_1_0 - 0.0142875305635721*G0_2_0 - 0.00254435475789634*G0_2_1;
    A[2757] = 0.0567586830607653*G0_0_0 + 0.0188869410874617*G0_0_1 + 0.0188869410874616*G0_1_0 - 0.00619778723077341*G0_1_1 + 0.0377738821749237*G0_2_0 + 0.00939454064454083*G0_2_1;
    A[2758] = 0.104709984267275*G0_0_0 + 0.0743408268364852*G0_0_1 + 0.0743408268364855*G0_1_0 + 0.0371867233846393*G0_1_1 + 0.108232937008977*G0_2_0 + 0.0808321934624003*G0_2_1;
    A[2759] = 0.0422754329004325*G0_0_1 + 0.042275432900433*G0_1_0 + 0.128522535206563*G0_1_1 + 0.0429604514890973*G0_2_0 + 0.0845508658008655*G0_2_1;
    A[2760] = 0.0642612676032806*G0_0_0 + 0.0642612676032806*G0_0_1 + 0.0219858347028486*G0_0_2 + 0.0642612676032805*G0_1_0 + 0.0642612676032806*G0_1_1 + 0.0219858347028485*G0_1_2 + 0.0219858347028484*G0_2_0 + 0.0219858347028483*G0_2_1 + 0.022670853291513*G0_2_2;
    A[2761] = 0.00880738185425629*G0_0_0 + 0.00880738185425627*G0_0_1 + 0.00231601522834096*G0_0_2 + 0.00880738185425587*G0_1_0 + 0.00880738185425586*G0_1_1 + 0.00231601522834128*G0_1_2 + 0.00231601522834103*G0_2_0 + 0.00231601522834107*G0_2_1 - 0.000652398655870863*G0_2_2;
    A[2762] = -0.000652398655870691*G0_0_0 - 0.000652398655870672*G0_0_1 + 0.00884000178705056*G0_0_2 - 0.00065239865587064*G0_1_0 - 0.00065239865587064*G0_1_1 + 0.00884000178705032*G0_1_2 + 0.00884000178705032*G0_2_0 + 0.00884000178705032*G0_2_1 - 0.000652398655871493*G0_2_2;
    A[2763] = -0.0032946132121478*G0_0_2 - 0.00329461321214784*G0_1_2 - 0.00329461321214795*G0_2_0 - 0.003294613212148*G0_2_1 + 0.0226708532915134*G0_2_2;
    A[2764] = 0.176636936077038*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077038*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761738*G0_1_1 + 0.134361503176606*G0_1_2 + 0.127315597693201*G0_2_0 + 0.128000616281865*G0_2_1 + 0.127315597693201*G0_2_2;
    A[2765] = -0.334680510461755*G0_0_0 - 0.250814663249556*G0_0_1 - 0.334680510461756*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700814*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037356*G0_2_0 - 0.160783648739377*G0_2_1 - 0.166948816037356*G0_2_2;
    A[2766] = 0.104709984267275*G0_0_0 + 0.188575831479475*G0_0_1 + 0.104709984267275*G0_0_2 + 0.188575831479476*G0_1_0 - 0.0622388317700806*G0_1_1 + 0.188575831479476*G0_1_2 + 0.182410664181496*G0_2_0 + 0.0985448169692955*G0_2_1 + 0.182410664181496*G0_2_2;
    A[2767] = -0.0422754329004325*G0_0_1 - 0.042275432900433*G0_1_0 + 0.0920860702761732*G0_1_1 - 0.0422754329004331*G0_1_2 - 0.0429604514890972*G0_2_0 - 0.0359145460056916*G0_2_1 - 0.0429604514890973*G0_2_2;
    A[2768] = 0.0439716694056969*G0_0_0 + 0.086247102306129*G0_0_1 + 0.086247102306129*G0_0_2 + 0.0862471023061295*G0_1_0 + 0.128522535206562*G0_1_1 + 0.128522535206562*G0_1_2 + 0.0446566879943616*G0_2_0 + 0.043971669405697*G0_2_1 + 0.043971669405697*G0_2_2;
    A[2769] = -0.00678494602105755*G0_0_0 - 0.0371541034518461*G0_0_1 - 0.037154103451846*G0_0_2 - 0.0371541034518466*G0_1_0 + 0.0371867233846395*G0_1_1 + 0.0371867233846396*G0_1_2 - 0.00975335990526986*G0_2_0 - 0.0436454700777608*G0_2_1 - 0.0436454700777609*G0_2_2;
    A[2770] = 0.0127870136550695*G0_0_0 - 0.0250847283182346*G0_0_1 - 0.0250847283182347*G0_0_2 - 0.0250847283182344*G0_1_0 - 0.00619778723077256*G0_1_1 - 0.00619778723077263*G0_1_2 + 0.00329461321214796*G0_2_0 - 0.0155923278753137*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[2771] = -0.0318696743392923*G0_0_0 + 0.00583896797004429*G0_0_1 + 0.00583896797004432*G0_0_2 + 0.00583896797004392*G0_1_0 - 0.00590420783563159*G0_2_0 + 0.00254435475789625*G0_2_1 + 0.00254435475789633*G0_2_2;
    A[2772] = 0.0391439193522522*G0_0_0 - 0.00554538857490255*G0_0_1 + 0.0264221455627697*G0_0_2 - 0.00554538857490098*G0_1_0 - 0.00619778723077209*G0_1_2 + 0.0264221455627717*G0_2_0 - 0.00619778723077251*G0_2_1 + 0.013700371773289*G0_2_2;
    A[2773] = 0.0929668084615987*G0_0_0 + 0.0606730749959916*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0606730749959901*G0_1_0 + 0.0195719596761265*G0_1_1 + 0.0748627457611822*G0_1_2 + 0.12134614999198*G0_2_0 + 0.0748627457611826*G0_2_1 + 0.149725491522364*G0_2_2;
    A[2774] = 0.0678494602105693*G0_0_1 + 0.104709984267274*G0_0_2 + 0.0678494602105719*G0_1_0 + 0.0391439193522519*G0_1_1 + 0.135698920421142*G0_1_2 + 0.104709984267276*G0_2_0 + 0.13569892042114*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2775] = 0.0195719596761263*G0_0_0 - 0.0102752788299659*G0_0_1 + 0.00146789697571051*G0_0_2 - 0.0102752788299661*G0_1_0 - 0.0181040627004162*G0_1_2 + 0.00146789697570941*G0_2_0 - 0.0181040627004163*G0_2_1 - 0.0166361657247069*G0_2_2;
    A[2776] = 0.0141896707651918*G0_0_1 + 0.0283793415303829*G0_0_2 + 0.0141896707651924*G0_1_0 - 0.00489298991902996*G0_1_1 + 0.028379341530383*G0_1_2 + 0.0283793415303834*G0_2_0 + 0.0283793415303831*G0_2_1 + 0.0567586830607654*G0_2_2;
    A[2777] = -0.000652398655871565*G0_0_1 - 0.0127217737894829*G0_0_2 - 0.000652398655869612*G0_1_0 - 0.00130479731174051*G0_1_2 - 0.0127217737894814*G0_2_0 - 0.00130479731174132*G0_2_1 - 0.0254435475789634*G0_2_2;
    A[2778] = -0.156575677409008*G0_0_0 - 0.224425137619578*G0_0_1 - 0.134067923781463*G0_0_2 - 0.22442513761958*G0_1_0 - 0.292274597830149*G0_1_1 - 0.0972073997247601*G0_1_2 - 0.134067923781465*G0_2_0 - 0.0972073997247598*G0_2_1 - 0.0880738185425687*G0_2_2;
    A[2779] = 0.0929668084615979*G0_0_0 + 0.228013330226868*G0_0_1 + 0.363059851992138*G0_0_2 + 0.228013330226871*G0_1_0 + 0.456026660453738*G0_1_1 + 0.228013330226868*G0_1_2 + 0.363059851992141*G0_2_0 + 0.22801333022687*G0_2_1 + 0.092966808461599*G0_2_2;
    A[2780] = -0.0678494602105697*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105718*G0_1_0 - 0.292274597830151*G0_1_1 - 0.195067198105391*G0_1_2 - 0.104709984267275*G0_2_0 - 0.195067198105392*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2781] = 0.0195719596761257*G0_0_0 + 0.00538228891093487*G0_0_1 - 0.0552907860850557*G0_0_2 + 0.00538228891093403*G0_1_0 - 0.00880738185425618*G0_1_1 - 0.0411011153198641*G0_1_2 - 0.0552907860850567*G0_2_0 - 0.0411011153198649*G0_2_1 + 0.0195719596761275*G0_2_2;
    A[2782] = -0.0141896707651916*G0_0_1 - 0.0283793415303821*G0_0_2 - 0.0141896707651925*G0_1_0 - 0.00880738185425719*G0_1_1 + 0.0322937334656078*G0_1_2 - 0.0283793415303834*G0_2_0 + 0.0322937334656076*G0_2_1 + 0.0929668084615986*G0_2_2;
    A[2783] = 0.000652398655870844*G0_0_1 + 0.0127217737894809*G0_0_2 + 0.00065239865586947*G0_1_0 + 0.00130479731174135*G0_1_1 + 0.000652398655870026*G0_1_2 + 0.0127217737894811*G0_2_0 + 0.000652398655870695*G0_2_1 - 0.0880738185425685*G0_2_2;
    A[2784] = -0.0965550010688884*G0_0_0 - 0.0287055408583178*G0_0_1 - 0.0596944770121847*G0_0_2 - 0.0287055408583175*G0_1_0 + 0.0391439193522518*G0_1_1 - 0.0965550010688894*G0_1_2 - 0.0596944770121844*G0_2_0 - 0.0965550010688892*G0_2_1 - 0.0228339529554816*G0_2_2;
    A[2785] = -0.00880738185425694*G0_0_0 - 0.0411011153198642*G0_0_1 + 0.00538228891093492*G0_0_2 - 0.041101115319865*G0_1_0 + 0.0195719596761268*G0_1_1 - 0.0552907860850565*G0_1_2 + 0.00538228891093427*G0_2_0 - 0.0552907860850565*G0_2_1 + 0.0195719596761255*G0_2_2;
    A[2786] = 0.0502346965020569*G0_0_0 + 0.00554538857490199*G0_0_1 + 0.0381653213684454*G0_0_2 + 0.00554538857490156*G0_1_0 + 0.00619778723077222*G0_1_2 + 0.0381653213684461*G0_2_0 + 0.00619778723077251*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[2787] = -0.033272331449414*G0_0_0 - 0.0190826606842233*G0_0_1 - 0.0190826606842228*G0_0_2 - 0.0190826606842227*G0_1_0 - 0.00489298991903249*G0_1_1 - 0.033272331449414*G0_1_2 - 0.0190826606842226*G0_2_0 - 0.0332723314494143*G0_2_1 - 0.00489298991903002*G0_2_2;
    A[2788] = 0.0401225173360582*G0_0_0 + 0.010275278829966*G0_0_1 + 0.0298472385060926*G0_0_2 + 0.0102752788299664*G0_1_0 + 0.0181040627004176*G0_1_2 + 0.0298472385060921*G0_2_0 + 0.0181040627004164*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2789] = 0.00130479731174218*G0_0_0 + 0.00065239865587145*G0_0_1 - 0.0107645778218695*G0_0_2 + 0.000652398655871321*G0_1_0 + 0.00130479731174155*G0_1_2 - 0.0107645778218695*G0_2_0 + 0.00130479731174117*G0_2_1 - 0.0228339529554813*G0_2_2;
    A[2790] = -0.268135847562927*G0_0_0 - 0.321632537344338*G0_0_1 - 0.290643601190471*G0_0_2 - 0.321632537344339*G0_1_0 - 0.5845491956603*G0_1_1 - 0.448850275239159*G0_1_2 - 0.222141742324031*G0_2_0 - 0.19441479944952*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2791] = -0.0371867233846405*G0_0_0 + 0.186912214907003*G0_0_1 + 0.140428810676204*G0_0_2 + 0.186912214907004*G0_1_0 - 0.0371867233846415*G0_1_1 + 0.0376760223765416*G0_1_2 + 0.0376760223765421*G0_2_0 + 0.140428810676203*G0_2_1 + 0.065566064915021*G0_2_2;
    A[2792] = 0.17223324514991*G0_0_0 + 0.00619778723077336*G0_0_1 - 0.0264221455627698*G0_0_2 + 0.00619778723077282*G0_1_0 + 0.0123955744615467*G0_1_1 + 0.00619778723077214*G0_1_2 + 0.0714376528178608*G0_2_0 + 0.00619778723077339*G0_2_1 - 0.0137003717732889*G0_2_2;
    A[2793] = 0.726119703984278*G0_0_0 + 0.456026660453737*G0_0_1 + 0.456026660453738*G0_0_2 + 0.456026660453737*G0_1_0 + 0.912053320907478*G0_1_1 + 0.45602666045374*G0_1_2 + 0.456026660453738*G0_2_0 + 0.45602666045374*G0_2_1 + 0.72611970398428*G0_2_2;
    A[2794] = -0.448197876583287*G0_0_0 - 0.224098938291643*G0_0_1 - 0.121346149991981*G0_0_2 - 0.224098938291643*G0_1_0 - 0.037186723384638*G0_1_1 - 0.074862745761182*G0_1_2 - 0.326851726591306*G0_2_0 - 0.177615534060842*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2795] = -0.20941996853455*G0_0_0 - 0.262916658315961*G0_0_1 - 0.104709984267274*G0_0_2 - 0.262916658315962*G0_1_0 - 0.584549195660302*G0_1_1 - 0.135698920421142*G0_1_2 - 0.290643601190473*G0_2_0 - 0.390134396210783*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2796] = 0.352295274170269*G0_0_0 + 0.249542485870607*G0_0_1 + 0.249542485870606*G0_0_2 + 0.249542485870607*G0_1_0 - 0.0391439193522525*G0_1_1 + 0.445262082631868*G0_1_2 + 0.249542485870607*G0_2_0 + 0.445262082631868*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2797] = -0.234863516113513*G0_0_0 - 0.0195719596761267*G0_0_1 - 0.141896707651915*G0_0_2 - 0.0195719596761259*G0_1_0 - 0.0391439193522538*G0_1_1 - 0.0195719596761258*G0_1_2 - 0.141896707651914*G0_2_0 - 0.0195719596761261*G0_2_1 - 0.0489298991903143*G0_2_2;
    A[2798] = -0.185933616923197*G0_0_0 - 0.288686405222859*G0_0_1 - 0.484406001984121*G0_0_2 - 0.288686405222861*G0_1_0 - 0.0391439193522512*G0_1_1 - 0.484406001984123*G0_1_2 - 0.484406001984121*G0_2_0 - 0.484406001984119*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2799] = -0.0391439193522522*G0_0_0 + 0.00489298991903157*G0_0_1 + 0.0538228891093481*G0_0_2 + 0.00489298991903198*G0_1_0 + 0.00978597983806272*G0_1_1 + 0.00489298991903108*G0_1_2 + 0.0538228891093473*G0_2_0 + 0.0048929899190312*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2800] = 0.0117431758056757*G0_0_0 + 0.0117431758056757*G0_0_1 + 0.0117431758056757*G0_0_2 + 0.0117431758056754*G0_1_0 + 0.0117431758056754*G0_1_1 + 0.0117431758056754*G0_1_2 + 0.00946499969937461*G0_2_0 + 0.00946499969937459*G0_2_1 + 0.00946499969937458*G0_2_2;
    A[2801] = -0.0298472385060925*G0_0_0 + 0.0047246710658167*G0_2_0;
    A[2802] = -0.0298472385060926*G0_1_1 + 0.00472467106581684*G0_2_1;
    A[2803] = -0.0094649996993747*G0_2_2;
    A[2804] = 0.0642612676032816*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0422754329004329*G0_2_1 + 0.0429604514890971*G0_2_2;
    A[2805] = 0.00880738185425669*G0_1_1 + 0.00649136662591533*G0_1_2 + 0.0064913666259146*G0_2_1 + 0.00352295274170229*G0_2_2;
    A[2806] = -0.000652398655870637*G0_1_1 - 0.00949240044292077*G0_1_2 - 0.00949240044292064*G0_2_1 - 0.0189848008858419*G0_2_2;
    A[2807] = 0.00329461321214783*G0_1_2 + 0.00329461321214755*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[2808] = 0.0642612676032809*G0_0_0 + 0.0422754329004322*G0_0_2 + 0.0422754329004327*G0_2_0 + 0.0429604514890967*G0_2_2;
    A[2809] = 0.00880738185425628*G0_0_0 + 0.00649136662591539*G0_0_2 + 0.00649136662591455*G0_2_0 + 0.00352295274170292*G0_2_2;
    A[2810] = -0.000652398655870625*G0_0_0 - 0.0094924004429215*G0_0_2 - 0.00949240044292072*G0_2_0 - 0.0189848008858425*G0_2_2;
    A[2811] = 0.00329461321214798*G0_0_2 + 0.00329461321214749*G0_2_0 + 0.0292600797158084*G0_2_2;
    A[2812] = 0.176636936077038*G0_0_0 + 0.0422754329004323*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.0493213383838378*G0_2_0 + 0.0429604514890968*G0_2_1;
    A[2813] = -0.334680510461756*G0_0_0 - 0.0838658472121998*G0_0_1 - 0.0838658472122004*G0_1_0 + 0.104709984267275*G0_1_1 - 0.167731694424401*G0_2_0 - 0.077700679914221*G0_2_1;
    A[2814] = 0.104709984267274*G0_0_0 - 0.0838658472122007*G0_0_1 - 0.0838658472122002*G0_1_0 - 0.334680510461758*G0_1_1 - 0.0777006799142211*G0_2_0 - 0.167731694424401*G0_2_1;
    A[2815] = 0.0422754329004325*G0_0_1 + 0.042275432900433*G0_1_0 + 0.17663693607704*G0_1_1 + 0.0429604514890974*G0_2_0 + 0.0493213383838381*G0_2_1;
    A[2816] = -0.00913358118219219*G0_0_0 - 0.00913358118219224*G0_0_1 - 0.00329461321214825*G0_0_2 - 0.00913358118219194*G0_1_0 - 0.00913358118219194*G0_1_1 - 0.00329461321214764*G0_1_2 - 0.00329461321214821*G0_2_0 - 0.00329461321214831*G0_2_1;
    A[2817] = -0.0156575677409003*G0_0_0 - 0.0156575677409002*G0_0_1 - 0.00257697469068916*G0_0_2 - 0.0156575677409011*G0_1_0 - 0.0156575677409011*G0_1_1 - 0.00257697469068983*G0_1_2 - 0.00257697469068936*G0_2_0 - 0.00257697469068926*G0_2_1;
    A[2818] = -0.000652398655871332*G0_0_0 - 0.000652398655871394*G0_0_1 - 0.00257697469069041*G0_0_2 - 0.000652398655870727*G0_1_0 - 0.000652398655870758*G0_1_1 - 0.00257697469068992*G0_1_2 - 0.00257697469069022*G0_2_0 - 0.0025769746906903*G0_2_1;
    A[2819] = -0.00329461321214789*G0_0_2 - 0.00329461321214793*G0_1_2 - 0.00329461321214754*G0_2_0 - 0.00329461321214754*G0_2_1;
    A[2820] = -0.0435476102793805*G0_0_0 - 0.0377086423093364*G0_0_1 - 0.0435476102793804*G0_0_2 - 0.0377086423093354*G0_1_0 - 0.031869674339291*G0_1_1 - 0.0377086423093353*G0_1_2 - 0.0292600797158084*G0_2_0 - 0.0259654665036604*G0_2_1 - 0.0292600797158084*G0_2_2;
    A[2821] = 0.0567586830607651*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.056758683060765*G0_0_2 + 0.0378717419733028*G0_1_0 + 0.0127870136550674*G0_1_1 + 0.0378717419733027*G0_1_2 + 0.0189848008858419*G0_2_0 + 0.0094924004429207*G0_2_1 + 0.0189848008858418*G0_2_2;
    A[2822] = 0.104709984267275*G0_0_0 + 0.0303691574307887*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0303691574307901*G0_1_0 - 0.00678494602105611*G0_1_1 + 0.0303691574307901*G0_1_2 - 0.0035229527417027*G0_2_0 + 0.00296841388421255*G0_2_1 - 0.00352295274170271*G0_2_2;
    A[2823] = -0.0422754329004323*G0_0_1 - 0.0422754329004328*G0_1_0 + 0.0439716694056966*G0_1_1 - 0.0422754329004329*G0_1_2 - 0.0429604514890972*G0_2_0 - 0.000685018588664468*G0_2_1 - 0.0429604514890973*G0_2_2;
    A[2824] = -0.0318696743392923*G0_0_0 - 0.0377086423093363*G0_0_1 - 0.0377086423093364*G0_0_2 - 0.037708642309336*G0_1_0 - 0.0435476102793804*G0_1_1 - 0.0435476102793804*G0_1_2 - 0.0259654665036605*G0_2_0 - 0.0292600797158087*G0_2_1 - 0.0292600797158087*G0_2_2;
    A[2825] = 0.0127870136550695*G0_0_0 + 0.037871741973304*G0_0_1 + 0.0378717419733041*G0_0_2 + 0.0378717419733039*G0_1_0 + 0.0567586830607658*G0_1_1 + 0.0567586830607658*G0_1_2 + 0.00949240044292092*G0_2_0 + 0.0189848008858423*G0_2_1 + 0.0189848008858423*G0_2_2;
    A[2826] = -0.00678494602105744*G0_0_0 + 0.0303691574307889*G0_0_1 + 0.0303691574307889*G0_0_2 + 0.0303691574307891*G0_1_0 + 0.104709984267275*G0_1_1 + 0.104709984267275*G0_1_2 + 0.00296841388421255*G0_2_0 - 0.00352295274170304*G0_2_1 - 0.00352295274170302*G0_2_2;
    A[2827] = 0.0439716694056969*G0_0_0 - 0.0422754329004322*G0_0_1 - 0.0422754329004323*G0_0_2 - 0.0422754329004324*G0_1_0 - 0.000685018588664284*G0_2_0 - 0.0429604514890968*G0_2_1 - 0.0429604514890967*G0_2_2;
    A[2828] = -0.156575677409008*G0_0_0 + 0.0678494602105713*G0_0_1 - 0.0225077536275438*G0_0_2 + 0.0678494602105718*G0_1_0 + 0.104709984267276*G0_1_2 - 0.022507753627546*G0_2_0 + 0.104709984267275*G0_2_1 + 0.0234863516113513*G0_2_2;
    A[2829] = 0.092966808461599*G0_0_0 - 0.13504652176527*G0_0_1 - 0.27009304353054*G0_0_2 - 0.13504652176527*G0_1_0 + 0.0929668084615997*G0_1_1 - 0.27009304353054*G0_1_2 - 0.270093043530541*G0_2_0 - 0.270093043530542*G0_2_1 - 0.540186087061083*G0_2_2;
    A[2830] = 0.0678494602105708*G0_0_1 + 0.104709984267274*G0_0_2 + 0.067849460210572*G0_1_0 - 0.156575677409008*G0_1_1 - 0.0225077536275437*G0_1_2 + 0.104709984267275*G0_2_0 - 0.022507753627545*G0_2_1 + 0.0234863516113516*G0_2_2;
    A[2831] = 0.0195719596761259*G0_0_0 + 0.0141896707651905*G0_0_1 + 0.0748627457611817*G0_0_2 + 0.0141896707651919*G0_1_0 + 0.0283793415303835*G0_1_2 + 0.0748627457611828*G0_2_0 + 0.0283793415303824*G0_2_1 + 0.149725491522364*G0_2_2;
    A[2832] = 0.0141896707651919*G0_0_1 + 0.0283793415303839*G0_0_2 + 0.0141896707651918*G0_1_0 + 0.0195719596761267*G0_1_1 + 0.0748627457611823*G0_1_2 + 0.0283793415303833*G0_2_0 + 0.0748627457611838*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2833] = -0.000652398655870336*G0_0_1 - 0.0127217737894813*G0_0_2 - 0.000652398655870146*G0_1_0 - 0.0127217737894811*G0_1_2 - 0.0127217737894813*G0_2_0 - 0.0127217737894817*G0_2_1 - 0.11351736612153*G0_2_2;
    A[2834] = 0.0391439193522513*G0_0_0 + 0.0446893079271541*G0_0_1 + 0.0127217737894822*G0_0_2 + 0.044689307927154*G0_1_0 + 0.0502346965020566*G0_1_1 + 0.0120693751336101*G0_1_2 + 0.0127217737894825*G0_2_0 + 0.0120693751336119*G0_2_1;
    A[2835] = 0.0929668084616004*G0_0_0 + 0.0322937334656091*G0_0_1 - 0.0283793415303811*G0_0_2 + 0.0322937334656107*G0_1_0 - 0.00880738185425557*G0_1_1 - 0.0141896707651897*G0_1_2 - 0.0283793415303822*G0_2_0 - 0.0141896707651917*G0_2_1;
    A[2836] = -0.0678494602105714*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105721*G0_1_0 - 0.0965550010688906*G0_1_1 - 0.0368605240567054*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0368605240567047*G0_2_1;
    A[2837] = 0.0195719596761263*G0_0_0 + 0.0298472385060918*G0_0_1 + 0.0181040627004154*G0_0_2 + 0.0298472385060926*G0_1_0 + 0.0401225173360589*G0_1_1 + 0.0102752788299658*G0_1_2 + 0.0181040627004162*G0_2_0 + 0.0102752788299649*G0_2_1;
    A[2838] = -0.0141896707651915*G0_0_1 - 0.0283793415303844*G0_0_2 - 0.0141896707651917*G0_1_0 - 0.0332723314494143*G0_1_1 - 0.0141896707651922*G0_1_2 - 0.0283793415303832*G0_2_0 - 0.01418967076519*G0_2_1;
    A[2839] = 0.000652398655871199*G0_0_1 + 0.0127217737894833*G0_0_2 + 0.000652398655870105*G0_1_0 + 0.00130479731174134*G0_1_1 + 0.0120693751336115*G0_1_2 + 0.0127217737894814*G0_2_0 + 0.0120693751336109*G0_2_1;
    A[2840] = 0.0502346965020568*G0_0_0 + 0.0446893079271542*G0_0_1 + 0.0120693751336115*G0_0_2 + 0.0446893079271546*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0127217737894816*G0_1_2 + 0.0120693751336117*G0_2_0 + 0.0127217737894826*G0_2_1;
    A[2841] = -0.00880738185425642*G0_0_0 + 0.0322937334656077*G0_0_1 - 0.0141896707651918*G0_0_2 + 0.032293733465608*G0_1_0 + 0.0929668084615995*G0_1_1 - 0.0283793415303831*G0_1_2 - 0.0141896707651916*G0_2_0 - 0.0283793415303832*G0_2_1;
    A[2842] = -0.0965550010688892*G0_0_0 - 0.0678494602105709*G0_0_1 - 0.0368605240567043*G0_0_2 - 0.0678494602105708*G0_1_0 - 0.104709984267276*G0_1_2 - 0.0368605240567046*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2843] = 0.0401225173360576*G0_0_0 + 0.0298472385060922*G0_0_1 + 0.0102752788299654*G0_0_2 + 0.0298472385060923*G0_1_0 + 0.019571959676126*G0_1_1 + 0.0181040627004166*G0_1_2 + 0.0102752788299653*G0_2_0 + 0.0181040627004163*G0_2_1;
    A[2844] = -0.0332723314494139*G0_0_0 - 0.0141896707651905*G0_0_1 - 0.0141896707651906*G0_0_2 - 0.0141896707651911*G0_1_0 - 0.0283793415303822*G0_1_2 - 0.0141896707651905*G0_2_0 - 0.0283793415303824*G0_2_1;
    A[2845] = 0.00130479731174202*G0_0_0 + 0.000652398655870344*G0_0_1 + 0.0120693751336113*G0_0_2 + 0.000652398655870563*G0_1_0 + 0.0127217737894819*G0_1_2 + 0.0120693751336111*G0_2_0 + 0.0127217737894816*G0_2_1;
    A[2846] = 0.172233245149909*G0_0_0 + 0.166035457919135*G0_0_1 + 0.198655390712678*G0_0_2 + 0.166035457919136*G0_1_0 + 0.172233245149909*G0_1_1 + 0.19865539071268*G0_1_2 + 0.100795592332049*G0_2_0 + 0.100795592332049*G0_2_1 + 0.113517366121532*G0_2_2;
    A[2847] = -0.037186723384639*G0_0_0 - 0.224098938291644*G0_0_1 - 0.177615534060844*G0_0_2 - 0.224098938291644*G0_1_0 - 0.448197876583289*G0_1_1 - 0.326851726591307*G0_1_2 - 0.0748627457611819*G0_2_0 - 0.121346149991981*G0_2_1 - 0.149725491522365*G0_2_2;
    A[2848] = -0.268135847562928*G0_0_0 + 0.0534966897814106*G0_0_1 + 0.0225077536275439*G0_0_2 + 0.0534966897814118*G0_1_0 - 0.20941996853455*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0459941052388965*G0_2_0 + 0.0812236326559238*G0_2_1 - 0.0234863516113512*G0_2_2;
    A[2849] = -0.448197876583287*G0_0_0 - 0.224098938291643*G0_0_1 - 0.326851726591306*G0_0_2 - 0.224098938291643*G0_1_0 - 0.037186723384638*G0_1_1 - 0.177615534060842*G0_1_2 - 0.121346149991981*G0_2_0 - 0.074862745761182*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2850] = 0.726119703984277*G0_0_0 + 0.27009304353054*G0_0_1 + 0.27009304353054*G0_0_2 + 0.27009304353054*G0_1_0 + 0.726119703984281*G0_1_1 + 0.27009304353054*G0_1_2 + 0.27009304353054*G0_2_0 + 0.27009304353054*G0_2_1 + 0.540186087061082*G0_2_2;
    A[2851] = -0.209419968534549*G0_0_0 + 0.0534966897814114*G0_0_1 - 0.104709984267275*G0_0_2 + 0.0534966897814102*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0225077536275439*G0_1_2 + 0.0812236326559238*G0_2_0 - 0.0459941052388965*G0_2_1 - 0.0234863516113514*G0_2_2;
    A[2852] = -0.234863516113513*G0_0_0 - 0.215291556437386*G0_0_1 - 0.0929668084615961*G0_0_2 - 0.215291556437387*G0_1_0 - 0.234863516113512*G0_1_1 - 0.0929668084615981*G0_1_2 - 0.0929668084615998*G0_2_0 - 0.0929668084615988*G0_2_1;
    A[2853] = 0.352295274170269*G0_0_0 + 0.102752788299661*G0_0_1 + 0.102752788299662*G0_0_2 + 0.102752788299661*G0_1_0 - 0.1859336169232*G0_1_1 + 0.298472385060923*G0_1_2 + 0.102752788299663*G0_2_0 + 0.298472385060925*G0_2_1;
    A[2854] = -0.185933616923199*G0_0_0 + 0.102752788299661*G0_0_1 + 0.298472385060922*G0_0_2 + 0.102752788299659*G0_1_0 + 0.352295274170269*G0_1_1 + 0.10275278829966*G0_1_2 + 0.298472385060924*G0_2_0 + 0.102752788299662*G0_2_1;
    A[2855] = -0.0391439193522521*G0_0_0 - 0.0440369092712843*G0_0_1 - 0.0929668084616004*G0_0_2 - 0.0440369092712846*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0929668084616006*G0_1_2 - 0.092966808461599*G0_2_0 - 0.0929668084616003*G0_2_1;
    A[2856] = 0.0189195610202553*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0189195610202555*G0_1_0 + 0.0189195610202555*G0_1_1 + 0.0189195610202556*G0_1_2 + 0.0137421252872639*G0_2_0 + 0.013742125287264*G0_2_1 + 0.013742125287264*G0_2_2;
    A[2857] = 0.0189195610202553*G0_0_0 + 0.00517743573299123*G0_2_0;
    A[2858] = 0.0335985307773501*G0_1_1 - 0.0143945239431348*G0_2_1;
    A[2859] = -0.013742125287264*G0_2_2;
    A[2860] = -0.128522535206563*G0_1_1 - 0.0962940416065412*G0_1_2 - 0.0962940416065411*G0_2_1 - 0.124738623002511*G0_2_2;
    A[2861] = 0.0476251018785736*G0_1_1 + 0.0623693115012555*G0_1_2 + 0.0623693115012555*G0_2_1 + 0.124738623002511*G0_2_2;
    A[2862] = -0.0150051690850308*G0_1_1 - 0.0377738821749248*G0_1_2 - 0.0377738821749241*G0_2_1 - 0.127087258163647*G0_2_2;
    A[2863] = 0.00939454064454073*G0_1_2 + 0.00939454064454088*G0_2_1 + 0.0641307878721074*G0_2_2;
    A[2864] = -0.0306627368259308*G0_0_0 - 0.0212681961813902*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.0118736555368497*G0_2_2;
    A[2865] = -0.00130479731174196*G0_0_0 - 0.00123955744615503*G0_0_2 - 0.00123955744615481*G0_2_0 - 0.00117431758056795*G0_2_2;
    A[2866] = -0.0150051690850298*G0_0_0 - 0.0149399292194426*G0_0_2 - 0.0149399292194432*G0_2_0 - 0.0148746893538559*G0_2_2;
    A[2867] = 0.00939454064454057*G0_0_2 + 0.009394540644541*G0_2_0 + 0.0187890812890814*G0_2_2;
    A[2868] = -0.111886369481854*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.0212681961813902*G0_1_0 - 0.0477555816097477*G0_2_0 - 0.0118736555368496*G0_2_1;
    A[2869] = 0.209419968534549*G0_0_0 + 0.035947165938484*G0_0_1 + 0.0359471659384844*G0_1_0 - 0.0528442911255412*G0_1_1 + 0.0823327103709029*G0_2_0 - 0.00182671623643949*G0_2_1;
    A[2870] = -0.271397840842282*G0_0_0 - 0.0733296089198851*G0_0_1 - 0.0733296089198856*G0_1_0 + 0.121346149991983*G0_1_1 - 0.146659217839771*G0_2_0 - 0.0109602974186299*G0_2_1;
    A[2871] = -0.096294041606541*G0_0_1 - 0.0962940416065414*G0_1_0 - 0.257045070413125*G0_1_1 - 0.124738623002511*G0_2_0 - 0.192588083213082*G0_2_1;
    A[2872] = -0.030662736825931*G0_0_0 - 0.0306627368259311*G0_0_1 - 0.00939454064454062*G0_0_2 - 0.0306627368259311*G0_1_0 - 0.0306627368259311*G0_1_1 - 0.00939454064454095*G0_1_2 - 0.00939454064454049*G0_2_0 - 0.00939454064454049*G0_2_1;
    A[2873] = -0.00130479731174171*G0_0_0 - 0.00130479731174172*G0_0_1 - 6.52398655870597e-05*G0_0_2 - 0.00130479731174085*G0_1_0 - 0.00130479731174095*G0_1_1 - 6.52398655869114e-05*G0_1_2 - 6.52398655869125e-05*G0_2_0 - 6.5239865586971e-05*G0_2_1;
    A[2874] = -0.0150051690850299*G0_0_0 - 0.0150051690850299*G0_0_1 - 6.52398655868787e-05*G0_0_2 - 0.0150051690850304*G0_1_0 - 0.0150051690850304*G0_1_1 - 6.52398655869788e-05*G0_1_2 - 6.52398655872227e-05*G0_2_0 - 6.52398655872021e-05*G0_2_1;
    A[2875] = -0.00939454064454068*G0_0_2 - 0.00939454064454066*G0_1_2 - 0.00939454064454091*G0_2_0 - 0.00939454064454087*G0_2_1;
    A[2876] = -0.111886369481854*G0_0_0 - 0.090618173300464*G0_0_1 - 0.111886369481854*G0_0_2 - 0.0906181733004645*G0_1_0 - 0.0693499771190744*G0_1_1 - 0.0906181733004648*G0_1_2 - 0.0641307878721061*G0_2_0 - 0.0547362472275657*G0_2_1 - 0.0641307878721063*G0_2_2;
    A[2877] = 0.209419968534549*G0_0_0 + 0.173472802596064*G0_0_1 + 0.20941996853455*G0_0_2 + 0.173472802596065*G0_1_0 + 0.0846813455320401*G0_1_1 + 0.173472802596065*G0_1_2 + 0.127087258163645*G0_2_0 + 0.0893133759887215*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2878] = -0.271397840842282*G0_0_0 - 0.198068231922396*G0_0_1 - 0.271397840842283*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052918*G0_1_1 - 0.198068231922398*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0623693115012544*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2879] = 0.0962940416065405*G0_0_1 + 0.0962940416065413*G0_1_0 - 0.064456987200042*G0_1_1 + 0.0962940416065413*G0_1_2 + 0.124738623002511*G0_2_0 + 0.0284445813959695*G0_2_1 + 0.124738623002511*G0_2_2;
    A[2880] = -0.018789081289081*G0_0_0 - 0.0400572774704715*G0_0_1 - 0.0400572774704714*G0_0_2 - 0.040057277470472*G0_1_0 - 0.0613254736518622*G0_1_1 - 0.0613254736518622*G0_1_2 - 0.00939454064454039*G0_2_0 - 0.018789081289081*G0_2_1 - 0.0187890812890811*G0_2_2;
    A[2881] = -0.000130479731174469*G0_0_0 + 0.013635131907701*G0_0_1 + 0.013635131907701*G0_0_2 + 0.0136351319077016*G0_1_0 - 0.0254435475789641*G0_1_1 - 0.0254435475789641*G0_1_2 - 6.523986558736e-05*G0_2_0 + 0.0148746893538557*G0_2_1 + 0.0148746893538557*G0_2_2;
    A[2882] = -0.000130479731173968*G0_0_0 - 0.0137656116388755*G0_0_1 - 0.0137656116388754*G0_0_2 - 0.0137656116388757*G0_1_0 - 0.052844291125541*G0_1_1 - 0.052844291125541*G0_1_2 - 6.52398655866348e-05*G0_2_0 + 0.00117431758056753*G0_2_1 + 0.0011743175805675*G0_2_2;
    A[2883] = -0.018789081289081*G0_0_0 + 0.0212681961813902*G0_0_1 + 0.0212681961813901*G0_0_2 + 0.0212681961813906*G0_1_0 - 0.00939454064454094*G0_2_0 + 0.0118736555368497*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[2884] = 0.0521918924696676*G0_0_0 - 0.0378391220405113*G0_0_1 - 0.000652398655871947*G0_0_2 - 0.0378391220405128*G0_1_0 - 0.0528442911255417*G0_1_2 - 0.000652398655872306*G0_2_0 - 0.0528442911255413*G0_2_1 - 0.0534966897814124*G0_2_2;
    A[2885] = -0.185933616923197*G0_0_0 + 0.00913358118219095*G0_0_1 - 0.081223632655924*G0_0_2 + 0.00913358118219381*G0_1_0 - 0.0880738185425681*G0_1_1 + 0.0459941052388961*G0_1_2 - 0.0812236326559223*G0_2_0 + 0.0459941052388961*G0_2_1 + 0.0234863516113508*G0_2_2;
    A[2886] = -0.13569892042114*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421143*G0_1_0 + 0.0521918924696703*G0_1_1 - 0.271397840842284*G0_1_2 - 0.271397840842284*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684565*G0_2_2;
    A[2887] = 0.0260959462348355*G0_0_0 - 0.0120693751336108*G0_0_1 + 0.0198981590040616*G0_0_2 - 0.0120693751336109*G0_1_0 - 0.0127217737894824*G0_1_2 + 0.0198981590040629*G0_2_0 - 0.0127217737894819*G0_2_1 + 0.0137003717732889*G0_2_2;
    A[2888] = 0.036860524056704*G0_0_1 + 0.104709984267275*G0_0_2 + 0.036860524056703*G0_1_0 - 0.0228339529554818*G0_1_1 + 0.0737210481134093*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0737210481134087*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2889] = -0.0150051690850297*G0_0_1 - 0.0528442911255406*G0_0_2 - 0.0150051690850311*G0_1_0 - 0.0300103381700615*G0_1_2 - 0.0528442911255402*G0_2_0 - 0.0300103381700593*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2890] = 0.0521918924696706*G0_0_0 + 0.0900310145101807*G0_0_1 + 0.0528442911255408*G0_0_2 + 0.0900310145101814*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0378391220405119*G0_1_2 + 0.0528442911255418*G0_2_0 + 0.0378391220405107*G0_2_1;
    A[2891] = -0.185933616923199*G0_0_0 - 0.195067198105391*G0_0_1 - 0.104709984267275*G0_0_2 - 0.195067198105393*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105717*G0_1_2 - 0.104709984267276*G0_2_0 - 0.0678494602105717*G0_2_1;
    A[2892] = 0.135698920421142*G0_0_1 + 0.271397840842283*G0_0_2 + 0.135698920421143*G0_1_0 + 0.323589733311954*G0_1_1 + 0.135698920421143*G0_1_2 + 0.271397840842284*G0_2_0 + 0.135698920421143*G0_2_1;
    A[2893] = 0.0260959462348349*G0_0_0 + 0.0381653213684459*G0_0_1 + 0.0061977872307728*G0_0_2 + 0.0381653213684461*G0_1_0 + 0.0502346965020563*G0_1_1 + 0.00554538857490142*G0_1_2 + 0.00619778723077398*G0_2_0 + 0.00554538857490293*G0_2_1;
    A[2894] = -0.0368605240567035*G0_0_1 - 0.104709984267274*G0_0_2 - 0.036860524056703*G0_1_0 - 0.0965550010688885*G0_1_1 - 0.067849460210569*G0_1_2 - 0.104709984267274*G0_2_0 - 0.067849460210571*G0_2_1;
    A[2895] = 0.0150051690850301*G0_0_1 + 0.0528442911255406*G0_0_2 + 0.0150051690850314*G0_1_0 + 0.0300103381700611*G0_1_1 + 0.0378391220405109*G0_1_2 + 0.0528442911255403*G0_2_0 + 0.0378391220405113*G0_2_1;
    A[2896] = 0.0300103381700602*G0_0_0 - 0.0078287838704501*G0_0_1 + 0.0150051690850302*G0_0_2 - 0.00782878387045082*G0_1_0 - 0.0456679059109603*G0_1_1 + 0.0300103381700612*G0_1_2 + 0.0150051690850306*G0_2_0 + 0.0300103381700617*G0_2_1;
    A[2897] = 0.00130479731174165*G0_0_0 + 0.000652398655870853*G0_0_1 + 0.000652398655871405*G0_0_2 + 0.00065239865587089*G0_1_0 - 0.0880738185425677*G0_1_1 + 0.0127217737894827*G0_1_2 + 0.000652398655869585*G0_2_0 + 0.0127217737894821*G0_2_1;
    A[2898] = 0.0300103381700599*G0_0_0 + 0.0378391220405112*G0_0_1 + 0.0150051690850306*G0_0_2 + 0.037839122040511*G0_1_0 + 0.0528442911255417*G0_1_2 + 0.0150051690850308*G0_2_0 + 0.0528442911255413*G0_2_1;
    A[2899] = 0.00130479731174175*G0_0_0 - 0.0107645778218695*G0_0_1 + 0.000652398655870289*G0_0_2 - 0.0107645778218694*G0_1_0 - 0.0228339529554797*G0_1_1 + 0.00130479731174061*G0_1_2 + 0.000652398655870649*G0_2_0 + 0.00130479731174188*G0_2_1;
    A[2900] = 0.00130479731174221*G0_0_0 + 0.0120693751336107*G0_0_1 + 0.000652398655870024*G0_0_2 + 0.0120693751336107*G0_1_0 + 0.0127217737894809*G0_1_2 + 0.00065239865587112*G0_2_0 + 0.0127217737894819*G0_2_1;
    A[2901] = 0.0300103381700597*G0_0_0 + 0.0150051690850298*G0_0_1 + 0.0150051690850304*G0_0_2 + 0.0150051690850301*G0_1_0 + 0.0300103381700605*G0_1_2 + 0.0150051690850303*G0_2_0 + 0.0300103381700594*G0_2_1;
    A[2902] = 0.105688582251081*G0_0_0 + 0.127870136550691*G0_0_1 + 0.105036183595211*G0_0_2 + 0.127870136550691*G0_1_0 + 0.255740273101382*G0_1_1 + 0.180062029020361*G0_1_2 + 0.0528442911255401*G0_2_0 + 0.0756782440810213*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2903] = 0.0123955744615469*G0_0_0 + 0.00619778723077413*G0_0_1 + 0.00619778723077364*G0_0_2 + 0.00619778723077385*G0_1_0 + 0.172233245149912*G0_1_1 + 0.0714376528178614*G0_1_2 + 0.00619778723077321*G0_2_0 - 0.0264221455627693*G0_2_1 - 0.0137003717732879*G0_2_2;
    A[2904] = 0.10568858225108*G0_0_0 - 0.0221815542996088*G0_0_1 + 0.000652398655871747*G0_0_2 - 0.0221815542996099*G0_1_0 + 0.105688582251082*G0_1_1 + 0.0528442911255416*G0_1_2 + 0.0528442911255406*G0_2_0 + 0.000652398655871573*G0_2_1 + 0.0534966897814126*G0_2_2;
    A[2905] = -0.20941996853455*G0_0_0 - 0.262916658315962*G0_0_1 - 0.290643601190473*G0_0_2 - 0.262916658315961*G0_1_0 - 0.584549195660302*G0_1_1 - 0.390134396210783*G0_1_2 - 0.104709984267274*G0_2_0 - 0.135698920421142*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2906] = -0.209419968534549*G0_0_0 + 0.0534966897814102*G0_0_1 + 0.0812236326559238*G0_0_2 + 0.0534966897814114*G0_1_0 - 0.26813584756293*G0_1_1 - 0.0459941052388965*G0_1_2 - 0.104709984267274*G0_2_0 + 0.0225077536275439*G0_2_1 - 0.0234863516113514*G0_2_2;
    A[2907] = 0.542795681684564*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623907*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684565*G0_2_2;
    A[2908] = -0.0521918924696709*G0_0_0 - 0.0424059126316073*G0_0_1 - 0.0260959462348359*G0_0_2 - 0.0424059126316062*G0_1_0 + 0.143527704291591*G0_1_1 - 0.0750258454251492*G0_1_2 - 0.0260959462348344*G0_2_0 - 0.0750258454251509*G0_2_1;
    A[2909] = -0.0521918924696673*G0_0_0 - 0.00978597983806169*G0_0_1 - 0.0260959462348345*G0_0_2 - 0.00978597983806231*G0_1_0 + 0.176147637085137*G0_1_1 - 0.0587158790283786*G0_1_2 - 0.0260959462348349*G0_2_0 - 0.0587158790283783*G0_2_1;
    A[2910] = 0.371867233846396*G0_0_0 + 0.185933616923197*G0_0_1 + 0.185933616923198*G0_0_2 + 0.185933616923199*G0_1_0 - 0.150051690850301*G0_1_1 + 0.316413348097373*G0_1_2 + 0.185933616923198*G0_2_0 + 0.316413348097371*G0_2_1;
    A[2911] = -0.0521918924696704*G0_0_0 - 0.0260959462348345*G0_0_1 - 0.0260959462348345*G0_0_2 - 0.0260959462348351*G0_1_0 + 0.0456679059109611*G0_1_1 - 0.0750258454251497*G0_1_2 - 0.0260959462348367*G0_2_0 - 0.0750258454251505*G0_2_1;
    A[2912] = -0.0782878387045046*G0_0_0 - 0.0782878387045046*G0_0_1 - 0.0782878387045046*G0_0_2 - 0.0782878387045046*G0_1_0 - 0.0782878387045049*G0_1_1 - 0.0782878387045048*G0_1_2 - 0.0782878387045047*G0_2_0 - 0.0782878387045048*G0_2_1 - 0.0782878387045048*G0_2_2;
    A[2913] = 0.0440369092712837*G0_0_0;
    A[2914] = 0.0440369092712843*G0_1_1;
    A[2915] = 0.0440369092712838*G0_2_2;
    A[2916] = -0.0505608958299934*G0_1_1 - 0.00260959462348357*G0_1_2 - 0.00260959462348447*G0_2_1;
    A[2917] = 0.0391439193522526*G0_1_1 + 0.0319675341376721*G0_1_2 + 0.0319675341376729*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[2918] = 0.0260959462348343*G0_1_1 + 0.0319675341376727*G0_1_2 + 0.0319675341376724*G0_2_1 + 0.0391439193522508*G0_2_2;
    A[2919] = -0.00260959462348325*G0_1_2 - 0.0026095946234837*G0_2_1 - 0.0505608958299924*G0_2_2;
    A[2920] = -0.050560895829993*G0_0_0 - 0.00260959462348375*G0_0_2 - 0.00260959462348271*G0_2_0;
    A[2921] = 0.0391439193522527*G0_0_0 + 0.031967534137673*G0_0_2 + 0.0319675341376723*G0_2_0 + 0.0260959462348343*G0_2_2;
    A[2922] = 0.0260959462348346*G0_0_0 + 0.0319675341376725*G0_0_2 + 0.0319675341376739*G0_2_0 + 0.0391439193522539*G0_2_2;
    A[2923] = -0.00260959462348279*G0_0_2 - 0.00260959462348357*G0_2_0 - 0.0505608958299926*G0_2_2;
    A[2924] = -0.0505608958299927*G0_0_0 - 0.00260959462348422*G0_0_1 - 0.00260959462348302*G0_1_0;
    A[2925] = 0.0391439193522516*G0_0_0 + 0.0319675341376733*G0_0_1 + 0.0319675341376715*G0_1_0 + 0.0260959462348341*G0_1_1;
    A[2926] = 0.0260959462348359*G0_0_0 + 0.0319675341376724*G0_0_1 + 0.0319675341376735*G0_1_0 + 0.039143919352253*G0_1_1;
    A[2927] = -0.00260959462348349*G0_0_1 - 0.00260959462348361*G0_1_0 - 0.0505608958299929*G0_1_1;
    A[2928] = 0.0962288017409539*G0_0_0 + 0.0962288017409539*G0_0_1 + 0.131458329157981*G0_0_2 + 0.0962288017409537*G0_1_0 + 0.0962288017409536*G0_1_1 + 0.131458329157981*G0_1_2 + 0.13145832915798*G0_2_0 + 0.13145832915798*G0_2_1 + 0.166687856575008*G0_2_2;
    A[2929] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.0662184635708941*G0_0_2 - 0.15657567740901*G0_1_0 - 0.15657567740901*G0_1_1 - 0.0662184635708939*G0_1_2 - 0.0662184635708937*G0_2_0 - 0.0662184635708935*G0_2_1 - 0.161794866655977*G0_2_2;
    A[2930] = 0.0260959462348353*G0_0_0 + 0.0260959462348353*G0_0_1 + 0.0104383784939334*G0_0_2 + 0.026095946234834*G0_1_0 + 0.026095946234834*G0_1_1 + 0.0104383784939329*G0_1_2 + 0.0104383784939331*G0_2_0 + 0.010438378493933*G0_2_1 + 0.180714427676231*G0_2_2;
    A[2931] = 0.00260959462348416*G0_0_2 + 0.00260959462348455*G0_1_2 + 0.00260959462348375*G0_2_0 + 0.00260959462348379*G0_2_1 - 0.151356488162042*G0_2_2;
    A[2932] = 0.0962288017409538*G0_0_0 + 0.131458329157981*G0_0_1 + 0.0962288017409537*G0_0_2 + 0.131458329157981*G0_1_0 + 0.166687856575008*G0_1_1 + 0.131458329157981*G0_1_2 + 0.0962288017409526*G0_2_0 + 0.131458329157981*G0_2_1 + 0.0962288017409527*G0_2_2;
    A[2933] = -0.156575677409008*G0_0_0 - 0.0662184635708935*G0_0_1 - 0.156575677409008*G0_0_2 - 0.0662184635708928*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0662184635708928*G0_1_2 - 0.156575677409008*G0_2_0 - 0.0662184635708934*G0_2_1 - 0.156575677409007*G0_2_2;
    A[2934] = 0.0260959462348349*G0_0_0 + 0.0104383784939341*G0_0_1 + 0.0260959462348348*G0_0_2 + 0.0104383784939338*G0_1_0 + 0.180714427676232*G0_1_1 + 0.0104383784939338*G0_1_2 + 0.0260959462348348*G0_2_0 + 0.0104383784939345*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[2935] = 0.00260959462348357*G0_0_1 + 0.00260959462348362*G0_1_0 - 0.151356488162042*G0_1_1 + 0.00260959462348355*G0_1_2 + 0.00260959462348297*G0_2_1;
    A[2936] = 0.166687856575008*G0_0_0 + 0.131458329157981*G0_0_1 + 0.131458329157981*G0_0_2 + 0.131458329157981*G0_1_0 + 0.0962288017409536*G0_1_1 + 0.0962288017409536*G0_1_2 + 0.131458329157981*G0_2_0 + 0.0962288017409528*G0_2_1 + 0.0962288017409528*G0_2_2;
    A[2937] = -0.161794866655976*G0_0_0 - 0.0662184635708934*G0_0_1 - 0.0662184635708933*G0_0_2 - 0.0662184635708935*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.0662184635708937*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2938] = 0.180714427676232*G0_0_0 + 0.0104383784939338*G0_0_1 + 0.0104383784939338*G0_0_2 + 0.0104383784939334*G0_1_0 + 0.0260959462348341*G0_1_1 + 0.0260959462348341*G0_1_2 + 0.0104383784939342*G0_2_0 + 0.0260959462348349*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[2939] = -0.151356488162043*G0_0_0 + 0.00260959462348403*G0_0_1 + 0.00260959462348413*G0_0_2 + 0.00260959462348376*G0_1_0 + 0.00260959462348341*G0_2_0;
    A[2940] = 0.19571959676126*G0_0_0 + 0.0750258454251488*G0_0_1 + 0.0750258454251486*G0_0_2 + 0.075025845425149*G0_1_0 + 0.0260959462348339*G0_1_2 + 0.0750258454251475*G0_2_0 + 0.0260959462348341*G0_2_1;
    A[2941] = 0.146789697570946*G0_0_0 + 0.141896707651915*G0_0_1 + 0.0929668084615999*G0_0_2 + 0.141896707651915*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0929668084615987*G0_1_2 + 0.0929668084616*G0_2_0 + 0.0929668084615989*G0_2_1;
    A[2942] = 0.0750258454251492*G0_0_1 + 0.0260959462348342*G0_0_2 + 0.0750258454251494*G0_1_0 + 0.195719596761261*G0_1_1 + 0.0750258454251509*G0_1_2 + 0.0260959462348339*G0_2_0 + 0.0750258454251505*G0_2_1;
    A[2943] = 0.146789697570947*G0_0_0 + 0.0929668084616*G0_0_1 + 0.141896707651914*G0_0_2 + 0.092966808461601*G0_1_0 + 0.0929668084616004*G0_1_2 + 0.141896707651918*G0_2_0 + 0.0929668084616014*G0_2_1 + 0.146789697570947*G0_2_2;
    A[2944] = 0.092966808461598*G0_0_1 + 0.0929668084615972*G0_0_2 + 0.0929668084615964*G0_1_0 + 0.146789697570943*G0_1_1 + 0.141896707651912*G0_1_2 + 0.0929668084615977*G0_2_0 + 0.141896707651913*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2945] = 0.0260959462348363*G0_0_1 + 0.0750258454251515*G0_0_2 + 0.0260959462348357*G0_1_0 + 0.0750258454251518*G0_1_2 + 0.0750258454251499*G0_2_0 + 0.0750258454251521*G0_2_1 + 0.195719596761263*G0_2_2;
    A[2946] = -0.782878387045046*G0_0_0 - 0.466465038947674*G0_0_1 - 0.466465038947675*G0_0_2 - 0.466465038947674*G0_1_0 - 0.150051690850301*G0_1_1 - 0.3359853077735*G0_1_2 - 0.466465038947672*G0_2_0 - 0.335985307773497*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2947] = 0.146789697570944*G0_0_0 + 0.44526208263187*G0_0_1 + 0.249542485870607*G0_0_2 + 0.445262082631869*G0_1_0 - 0.0391439193522514*G0_1_1 + 0.249542485870607*G0_1_2 + 0.249542485870608*G0_2_0 + 0.249542485870606*G0_2_1 + 0.352295274170268*G0_2_2;
    A[2948] = -0.0750258454251497*G0_0_1 - 0.0260959462348353*G0_0_2 - 0.0750258454251495*G0_1_0 + 0.143527704291592*G0_1_1 - 0.0424059126316059*G0_1_2 - 0.026095946234834*G0_2_0 - 0.042405912631605*G0_2_1 - 0.05219189246967*G0_2_2;
    A[2949] = 0.146789697570946*G0_0_0 + 0.249542485870609*G0_0_1 + 0.445262082631873*G0_0_2 + 0.249542485870608*G0_1_0 + 0.352295274170271*G0_1_1 + 0.24954248587061*G0_1_2 + 0.445262082631871*G0_2_0 + 0.249542485870609*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[2950] = -0.0929668084615994*G0_0_1 - 0.0929668084615979*G0_0_2 - 0.0929668084615965*G0_1_0 - 0.234863516113515*G0_1_1 - 0.215291556437388*G0_1_2 - 0.0929668084615978*G0_2_0 - 0.21529155643739*G0_2_1 - 0.234863516113512*G0_2_2;
    A[2951] = -0.0260959462348345*G0_0_1 - 0.0750258454251523*G0_0_2 - 0.026095946234836*G0_1_0 - 0.0521918924696675*G0_1_1 - 0.0424059126316074*G0_1_2 - 0.0750258454251498*G0_2_0 - 0.0424059126316045*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2952] = -0.150051690850301*G0_0_0 - 0.466465038947673*G0_0_1 - 0.335985307773499*G0_0_2 - 0.466465038947674*G0_1_0 - 0.782878387045048*G0_1_1 - 0.466465038947674*G0_1_2 - 0.335985307773498*G0_2_0 - 0.466465038947673*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2953] = -0.0391439193522533*G0_0_0 + 0.44526208263187*G0_0_1 + 0.249542485870608*G0_0_2 + 0.445262082631869*G0_1_0 + 0.146789697570946*G0_1_1 + 0.249542485870609*G0_1_2 + 0.249542485870608*G0_2_0 + 0.249542485870609*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2954] = 0.143527704291593*G0_0_0 - 0.0750258454251496*G0_0_1 - 0.0424059126316071*G0_0_2 - 0.0750258454251494*G0_1_0 - 0.0260959462348344*G0_1_2 - 0.0424059126316066*G0_2_0 - 0.0260959462348348*G0_2_1 - 0.0521918924696691*G0_2_2;
    A[2955] = 0.352295274170273*G0_0_0 + 0.24954248587061*G0_0_1 + 0.24954248587061*G0_0_2 + 0.249542485870611*G0_1_0 + 0.146789697570948*G0_1_1 + 0.445262082631872*G0_1_2 + 0.24954248587061*G0_2_0 + 0.445262082631872*G0_2_1 - 0.039143919352253*G0_2_2;
    A[2956] = -0.234863516113515*G0_0_0 - 0.0929668084615995*G0_0_1 - 0.215291556437388*G0_0_2 - 0.0929668084616017*G0_1_0 - 0.092966808461601*G0_1_2 - 0.215291556437389*G0_2_0 - 0.0929668084616009*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2957] = -0.0521918924696705*G0_0_0 - 0.026095946234837*G0_0_1 - 0.0424059126316084*G0_0_2 - 0.0260959462348352*G0_1_0 - 0.075025845425152*G0_1_2 - 0.0424059126316074*G0_2_0 - 0.0750258454251528*G0_2_1 + 0.143527704291592*G0_2_2;
    A[2958] = -0.150051690850301*G0_0_0 - 0.335985307773498*G0_0_1 - 0.466465038947672*G0_0_2 - 0.335985307773499*G0_1_0 - 0.1500516908503*G0_1_1 - 0.466465038947673*G0_1_2 - 0.466465038947673*G0_2_0 - 0.466465038947671*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2959] = -0.0391439193522528*G0_0_0 + 0.249542485870608*G0_0_1 + 0.445262082631869*G0_0_2 + 0.249542485870608*G0_1_0 + 0.352295274170269*G0_1_1 + 0.249542485870606*G0_1_2 + 0.44526208263187*G0_2_0 + 0.249542485870607*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2960] = 0.143527704291592*G0_0_0 - 0.042405912631607*G0_0_1 - 0.0750258454251495*G0_0_2 - 0.0424059126316064*G0_1_0 - 0.0521918924696685*G0_1_1 - 0.0260959462348346*G0_1_2 - 0.0750258454251511*G0_2_0 - 0.0260959462348345*G0_2_1;
    A[2961] = 0.352295274170269*G0_0_0 + 0.249542485870607*G0_0_1 + 0.249542485870607*G0_0_2 + 0.249542485870607*G0_1_0 - 0.0391439193522525*G0_1_1 + 0.445262082631868*G0_1_2 + 0.249542485870606*G0_2_0 + 0.445262082631868*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2962] = -0.234863516113513*G0_0_0 - 0.215291556437387*G0_0_1 - 0.0929668084615998*G0_0_2 - 0.215291556437386*G0_1_0 - 0.234863516113512*G0_1_1 - 0.0929668084615989*G0_1_2 - 0.0929668084615961*G0_2_0 - 0.092966808461598*G0_2_1;
    A[2963] = -0.0521918924696709*G0_0_0 - 0.0424059126316062*G0_0_1 - 0.0260959462348343*G0_0_2 - 0.0424059126316073*G0_1_0 + 0.143527704291591*G0_1_1 - 0.0750258454251509*G0_1_2 - 0.0260959462348359*G0_2_0 - 0.0750258454251492*G0_2_1;
    A[2964] = 1.76147637085136*G0_0_0 + 0.978597983806307*G0_0_1 + 0.978597983806307*G0_0_2 + 0.978597983806307*G0_1_0 + 1.76147637085135*G0_1_1 + 0.978597983806307*G0_1_2 + 0.978597983806307*G0_2_0 + 0.978597983806307*G0_2_1 + 1.76147637085136*G0_2_2;
    A[2965] = -1.17431758056757*G0_0_0 - 0.587158790283784*G0_0_1 - 0.587158790283783*G0_0_2 - 0.587158790283782*G0_1_0 - 0.293579395141891*G0_1_1 - 0.342509294332206*G0_1_2 - 0.587158790283783*G0_2_0 - 0.342509294332208*G0_2_1 - 0.29357939514189*G0_2_2;
    A[2966] = -0.29357939514189*G0_0_0 - 0.587158790283783*G0_0_1 - 0.342509294332204*G0_0_2 - 0.587158790283784*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283779*G0_1_2 - 0.342509294332207*G0_2_0 - 0.587158790283784*G0_2_1 - 0.293579395141891*G0_2_2;
    A[2967] = -0.293579395141892*G0_0_0 - 0.342509294332208*G0_0_1 - 0.587158790283786*G0_0_2 - 0.342509294332209*G0_1_0 - 0.293579395141892*G0_1_1 - 0.587158790283787*G0_1_2 - 0.587158790283789*G0_2_0 - 0.587158790283786*G0_2_1 - 1.17431758056757*G0_2_2;
    A[2968] = 0.0440369092712839*G0_0_0 + 0.044036909271284*G0_0_1 + 0.044036909271284*G0_0_2 + 0.0440369092712838*G0_1_0 + 0.0440369092712837*G0_1_1 + 0.0440369092712838*G0_1_2 + 0.0440369092712837*G0_2_0 + 0.0440369092712837*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[2969] = -0.0782878387045046*G0_0_0;
    A[2970] = 0.0440369092712844*G0_1_1;
    A[2971] = 0.044036909271284*G0_2_2;
    A[2972] = -0.0505608958299934*G0_1_1 - 0.0026095946234831*G0_1_2 - 0.00260959462348356*G0_2_1;
    A[2973] = 0.039143919352252*G0_1_1 + 0.0319675341376709*G0_1_2 + 0.031967534137673*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[2974] = 0.0260959462348348*G0_1_1 + 0.0319675341376737*G0_1_2 + 0.0319675341376728*G0_2_1 + 0.0391439193522525*G0_2_2;
    A[2975] = -0.00260959462348488*G0_1_2 - 0.00260959462348391*G0_2_1 - 0.0505608958299933*G0_2_2;
    A[2976] = 0.0962288017409542*G0_0_0 - 0.0352295274170265*G0_0_2 - 0.035229527417027*G0_2_0;
    A[2977] = -0.15657567740901*G0_0_0 - 0.090357213838117*G0_0_2 - 0.0903572138381156*G0_2_0 - 0.185933616923199*G0_2_2;
    A[2978] = 0.0260959462348356*G0_0_0 + 0.0156575677409022*G0_0_2 + 0.0156575677409018*G0_2_0 + 0.1859336169232*G0_2_2;
    A[2979] = -0.00260959462348428*G0_0_2 - 0.00260959462348384*G0_2_0 - 0.15657567740901*G0_2_2;
    A[2980] = 0.0962288017409538*G0_0_0 - 0.0352295274170262*G0_0_1 - 0.0352295274170267*G0_1_0;
    A[2981] = -0.156575677409008*G0_0_0 - 0.0903572138381161*G0_0_1 - 0.0903572138381156*G0_1_0 - 0.185933616923198*G0_1_1;
    A[2982] = 0.0260959462348332*G0_0_0 + 0.0156575677409012*G0_0_1 + 0.0156575677409005*G0_1_0 + 0.185933616923199*G0_1_1;
    A[2983] = -0.00260959462348364*G0_0_1 - 0.00260959462348285*G0_1_0 - 0.15657567740901*G0_1_1;
    A[2984] = -0.0505608958299931*G0_0_0 - 0.0505608958299931*G0_0_1 - 0.04795130120651*G0_0_2 - 0.0505608958299934*G0_1_0 - 0.0505608958299935*G0_1_1 - 0.0479513012065096*G0_1_2 - 0.0479513012065094*G0_2_0 - 0.0479513012065094*G0_2_1 - 0.0453417065830263*G0_2_2;
    A[2985] = 0.0391439193522528*G0_0_0 + 0.0391439193522528*G0_0_1 + 0.007176385214581*G0_0_2 + 0.0391439193522528*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.00717638521457996*G0_1_2 + 0.00717638521457974*G0_2_0 + 0.00717638521457974*G0_2_1 + 0.00130479731174257*G0_2_2;
    A[2986] = 0.0260959462348346*G0_0_0 + 0.0260959462348345*G0_0_1 - 0.00587158790283843*G0_0_2 + 0.0260959462348346*G0_1_0 + 0.0260959462348346*G0_1_1 - 0.00587158790283754*G0_1_2 - 0.00587158790283847*G0_2_0 - 0.00587158790283838*G0_2_1 + 0.00130479731174122*G0_2_2;
    A[2987] = 0.00260959462348341*G0_0_2 + 0.00260959462348302*G0_1_2 + 0.00260959462348358*G0_2_0 + 0.00260959462348362*G0_2_1 - 0.0453417065830257*G0_2_2;
    A[2988] = -0.0505608958299931*G0_0_0 - 0.0479513012065098*G0_0_1 - 0.050560895829993*G0_0_2 - 0.0479513012065098*G0_1_0 - 0.0453417065830257*G0_1_1 - 0.0479513012065098*G0_1_2 - 0.0505608958299926*G0_2_0 - 0.0479513012065092*G0_2_1 - 0.0505608958299926*G0_2_2;
    A[2989] = 0.0391439193522524*G0_0_0 + 0.00717638521458054*G0_0_1 + 0.0391439193522522*G0_0_2 + 0.00717638521458098*G0_1_0 + 0.00130479731174216*G0_1_1 + 0.00717638521458099*G0_1_2 + 0.0391439193522528*G0_2_0 + 0.00717638521457979*G0_2_1 + 0.039143919352253*G0_2_2;
    A[2990] = 0.0260959462348339*G0_0_0 - 0.00587158790283851*G0_0_1 + 0.0260959462348341*G0_0_2 - 0.00587158790283884*G0_1_0 + 0.00130479731174155*G0_1_1 - 0.00587158790283871*G0_1_2 + 0.0260959462348349*G0_2_0 - 0.00587158790283811*G0_2_1 + 0.026095946234835*G0_2_2;
    A[2991] = 0.0026095946234835*G0_0_1 + 0.00260959462348302*G0_1_0 - 0.0453417065830263*G0_1_1 + 0.00260959462348287*G0_1_2 + 0.00260959462348322*G0_2_1;
    A[2992] = -0.151356488162043*G0_0_0 - 0.153966082785526*G0_0_1 - 0.153966082785526*G0_0_2 - 0.153966082785525*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.153966082785526*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2993] = 0.180714427676232*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182297*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182298*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923198*G0_2_2;
    A[2994] = -0.161794866655977*G0_0_0 - 0.0955764030850829*G0_0_1 - 0.095576403085083*G0_0_2 - 0.0955764030850822*G0_1_0 - 0.185933616923198*G0_1_1 - 0.185933616923199*G0_1_2 - 0.0955764030850829*G0_2_0 - 0.185933616923199*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2995] = 0.166687856575008*G0_0_0 + 0.0352295274170263*G0_0_1 + 0.0352295274170263*G0_0_2 + 0.0352295274170269*G0_1_0 + 0.0352295274170273*G0_2_0;
    A[2996] = -0.782878387045047*G0_0_0 - 0.316413348097372*G0_0_1 - 0.316413348097371*G0_0_2 - 0.316413348097373*G0_1_0 - 0.185933616923197*G0_1_2 - 0.316413348097373*G0_2_0 - 0.185933616923198*G0_2_1;
    A[2997] = 0.146789697570947*G0_0_0 - 0.298472385060923*G0_0_1 - 0.102752788299663*G0_0_2 - 0.298472385060924*G0_1_0 - 0.782878387045048*G0_1_1 - 0.298472385060925*G0_1_2 - 0.102752788299663*G0_2_0 - 0.298472385060926*G0_2_1;
    A[2998] = 0.0750258454251508*G0_0_1 + 0.0260959462348351*G0_0_2 + 0.0750258454251487*G0_1_0 + 0.293579395141892*G0_1_1 + 0.0587158790283788*G0_1_2 + 0.0260959462348359*G0_2_0 + 0.0587158790283802*G0_2_1;
    A[2999] = 0.146789697570948*G0_0_0 - 0.102752788299663*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299662*G0_1_0 - 0.298472385060925*G0_1_2 - 0.298472385060923*G0_2_0 - 0.298472385060924*G0_2_1 - 0.782878387045049*G0_2_2;
    A[3000] = 0.0929668084616014*G0_0_1 + 0.0929668084616015*G0_0_2 + 0.092966808461599*G0_1_0 - 0.0489298991903159*G0_1_1 - 0.0293579395141899*G0_1_2 + 0.0929668084615996*G0_2_0 - 0.0293579395141891*G0_2_1 - 0.048929899190315*G0_2_2;
    A[3001] = 0.0260959462348355*G0_0_1 + 0.0750258454251514*G0_0_2 + 0.0260959462348336*G0_1_0 + 0.0587158790283787*G0_1_2 + 0.0750258454251509*G0_2_0 + 0.0587158790283797*G0_2_1 + 0.293579395141895*G0_2_2;
    A[3002] = 0.19571959676126*G0_0_0 + 0.120693751336112*G0_0_1 + 0.120693751336112*G0_0_2 + 0.120693751336113*G0_1_0 + 0.0456679059109638*G0_1_1 + 0.0717638521457977*G0_1_2 + 0.120693751336111*G0_2_0 + 0.0717638521457964*G0_2_1 + 0.0456679059109611*G0_2_2;
    A[3003] = 0.146789697570947*G0_0_0 + 0.00489298991903031*G0_0_1 + 0.0538228891093479*G0_0_2 + 0.00489298991903024*G0_1_0 + 0.00978597983806009*G0_1_1 + 0.00489298991902857*G0_1_2 + 0.0538228891093478*G0_2_0 + 0.0048929899190311*G0_2_1 - 0.0391439193522536*G0_2_2;
    A[3004] = -0.0750258454251501*G0_0_1 - 0.0260959462348337*G0_0_2 - 0.0750258454251487*G0_1_0 + 0.0456679059109629*G0_1_1 - 0.0260959462348318*G0_1_2 - 0.026095946234836*G0_2_0 - 0.0260959462348347*G0_2_1 - 0.0521918924696692*G0_2_2;
    A[3005] = 0.146789697570947*G0_0_0 + 0.0538228891093475*G0_0_1 + 0.00489298991903059*G0_0_2 + 0.0538228891093456*G0_1_0 - 0.0391439193522539*G0_1_1 + 0.00489298991903037*G0_1_2 + 0.00489298991902996*G0_2_0 + 0.00489298991903118*G0_2_1 + 0.00978597983806265*G0_2_2;
    A[3006] = -0.0929668084615993*G0_0_1 - 0.0929668084616008*G0_0_2 - 0.0929668084615991*G0_1_0 - 0.0391439193522504*G0_1_1 - 0.0440369092712838*G0_1_2 - 0.0929668084615999*G0_2_0 - 0.0440369092712831*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[3007] = -0.0260959462348355*G0_0_1 - 0.0750258454251496*G0_0_2 - 0.0260959462348334*G0_1_0 - 0.0521918924696696*G0_1_1 - 0.0260959462348331*G0_1_2 - 0.0750258454251508*G0_2_0 - 0.0260959462348345*G0_2_1 + 0.0456679059109621*G0_2_2;
    A[3008] = 0.143527704291592*G0_0_0 + 0.21855354971674*G0_0_1 + 0.185933616923198*G0_0_2 + 0.218553549716742*G0_1_0 + 0.293579395141892*G0_1_1 + 0.234863516113515*G0_1_2 + 0.185933616923198*G0_2_0 + 0.234863516113512*G0_2_1 + 0.176147637085135*G0_2_2;
    A[3009] = -0.0391439193522507*G0_0_0 - 0.484406001984122*G0_0_1 - 0.28868640522286*G0_0_2 - 0.484406001984122*G0_1_0 - 0.782878387045046*G0_1_1 - 0.484406001984121*G0_1_2 - 0.28868640522286*G0_2_0 - 0.48440600198412*G0_2_1 - 0.185933616923196*G0_2_2;
    A[3010] = -0.150051690850302*G0_0_0 + 0.316413348097372*G0_0_1 + 0.1859336169232*G0_0_2 + 0.316413348097372*G0_1_0 + 0.185933616923199*G0_1_2 + 0.185933616923198*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846398*G0_2_2;
    A[3011] = -0.234863516113516*G0_0_0 - 0.141896707651916*G0_0_1 - 0.0195719596761267*G0_0_2 - 0.141896707651914*G0_1_0 - 0.0489298991903153*G0_1_1 - 0.0195719596761272*G0_1_2 - 0.0195719596761257*G0_2_0 - 0.0195719596761268*G0_2_1 - 0.0391439193522532*G0_2_2;
    A[3012] = 0.352295274170272*G0_0_0 + 0.102752788299663*G0_0_1 + 0.102752788299661*G0_0_2 + 0.102752788299661*G0_1_0 + 0.298472385060922*G0_1_2 + 0.102752788299661*G0_2_0 + 0.298472385060923*G0_2_1 - 0.185933616923202*G0_2_2;
    A[3013] = -0.0521918924696693*G0_0_0 - 0.026095946234835*G0_0_1 - 0.00978597983806181*G0_0_2 - 0.0260959462348349*G0_1_0 - 0.0587158790283771*G0_1_2 - 0.00978597983806296*G0_2_0 - 0.0587158790283793*G0_2_1 + 0.176147637085137*G0_2_2;
    A[3014] = 0.143527704291592*G0_0_0 + 0.185933616923198*G0_0_1 + 0.21855354971674*G0_0_2 + 0.185933616923198*G0_1_0 + 0.176147637085137*G0_1_1 + 0.234863516113514*G0_1_2 + 0.218553549716741*G0_2_0 + 0.234863516113514*G0_2_1 + 0.293579395141891*G0_2_2;
    A[3015] = -0.0391439193522515*G0_0_0 - 0.28868640522286*G0_0_1 - 0.484406001984121*G0_0_2 - 0.288686405222859*G0_1_0 - 0.185933616923197*G0_1_1 - 0.484406001984121*G0_1_2 - 0.484406001984122*G0_2_0 - 0.484406001984121*G0_2_1 - 0.782878387045046*G0_2_2;
    A[3016] = -0.150051690850301*G0_0_0 + 0.185933616923199*G0_0_1 + 0.316413348097372*G0_0_2 + 0.185933616923198*G0_1_0 + 0.371867233846397*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097373*G0_2_0 + 0.185933616923199*G0_2_1;
    A[3017] = -0.234863516113513*G0_0_0 - 0.0195719596761259*G0_0_1 - 0.141896707651914*G0_0_2 - 0.0195719596761267*G0_1_0 - 0.0391439193522538*G0_1_1 - 0.0195719596761261*G0_1_2 - 0.141896707651915*G0_2_0 - 0.0195719596761258*G0_2_1 - 0.0489298991903143*G0_2_2;
    A[3018] = 0.352295274170269*G0_0_0 + 0.102752788299661*G0_0_1 + 0.102752788299663*G0_0_2 + 0.102752788299661*G0_1_0 - 0.1859336169232*G0_1_1 + 0.298472385060925*G0_1_2 + 0.102752788299662*G0_2_0 + 0.298472385060923*G0_2_1;
    A[3019] = -0.0521918924696673*G0_0_0 - 0.00978597983806231*G0_0_1 - 0.0260959462348349*G0_0_2 - 0.00978597983806169*G0_1_0 + 0.176147637085137*G0_1_1 - 0.0587158790283783*G0_1_2 - 0.0260959462348345*G0_2_0 - 0.0587158790283786*G0_2_1;
    A[3020] = -1.17431758056757*G0_0_0 - 0.587158790283782*G0_0_1 - 0.587158790283783*G0_0_2 - 0.587158790283784*G0_1_0 - 0.293579395141891*G0_1_1 - 0.342509294332208*G0_1_2 - 0.587158790283783*G0_2_0 - 0.342509294332206*G0_2_1 - 0.29357939514189*G0_2_2;
    A[3021] = 1.76147637085136*G0_0_0 + 0.782878387045045*G0_0_1 + 0.782878387045045*G0_0_2 + 0.782878387045045*G0_1_0 + 1.56575677409009*G0_1_1 + 0.782878387045045*G0_1_2 + 0.782878387045045*G0_2_0 + 0.782878387045045*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3022] = -0.293579395141893*G0_0_0 + 0.293579395141892*G0_0_1 + 0.0489298991903125*G0_0_2 + 0.293579395141894*G0_1_0 - 0.293579395141893*G0_1_1 + 0.0489298991903157*G0_1_2 + 0.048929899190315*G0_2_0 + 0.0489298991903139*G0_2_1 + 0.0978597983806283*G0_2_2;
    A[3023] = -0.293579395141894*G0_0_0 + 0.0489298991903136*G0_0_1 + 0.293579395141892*G0_0_2 + 0.0489298991903165*G0_1_0 + 0.09785979838063*G0_1_1 + 0.048929899190315*G0_1_2 + 0.293579395141894*G0_2_0 + 0.0489298991903146*G0_2_1 - 0.293579395141895*G0_2_2;
    A[3024] = 0.0440369092712838*G0_0_0 + 0.0440369092712838*G0_0_1 + 0.0440369092712838*G0_0_2 + 0.0440369092712835*G0_1_0 + 0.0440369092712838*G0_1_1 + 0.0440369092712836*G0_1_2 + 0.0440369092712838*G0_2_0 + 0.0440369092712838*G0_2_1 + 0.0440369092712839*G0_2_2;
    A[3025] = 0.0440369092712839*G0_0_0;
    A[3026] = -0.0782878387045052*G0_1_1;
    A[3027] = 0.0440369092712842*G0_2_2;
    A[3028] = 0.0962288017409548*G0_1_1 - 0.0352295274170272*G0_1_2 - 0.0352295274170274*G0_2_1;
    A[3029] = -0.15657567740901*G0_1_1 - 0.0903572138381149*G0_1_2 - 0.0903572138381159*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3030] = 0.0260959462348358*G0_1_1 + 0.0156575677409008*G0_1_2 + 0.0156575677409003*G0_2_1 + 0.185933616923198*G0_2_2;
    A[3031] = -0.00260959462348276*G0_1_2 - 0.00260959462348333*G0_2_1 - 0.15657567740901*G0_2_2;
    A[3032] = -0.0505608958299927*G0_0_0 - 0.00260959462348371*G0_0_2 - 0.0026095946234839*G0_2_0;
    A[3033] = 0.0391439193522519*G0_0_0 + 0.0319675341376727*G0_0_2 + 0.0319675341376732*G0_2_0 + 0.0260959462348356*G0_2_2;
    A[3034] = 0.0260959462348352*G0_0_0 + 0.0319675341376731*G0_0_2 + 0.0319675341376723*G0_2_0 + 0.039143919352252*G0_2_2;
    A[3035] = -0.00260959462348431*G0_0_2 - 0.00260959462348319*G0_2_0 - 0.0505608958299927*G0_2_2;
    A[3036] = -0.15657567740901*G0_0_0 - 0.00260959462348367*G0_0_1 - 0.00260959462348361*G0_1_0;
    A[3037] = 0.185933616923199*G0_0_0 + 0.0156575677409005*G0_0_1 + 0.0156575677409008*G0_1_0 + 0.0260959462348347*G0_1_1;
    A[3038] = -0.185933616923198*G0_0_0 - 0.090357213838115*G0_0_1 - 0.0903572138381148*G0_1_0 - 0.156575677409009*G0_1_1;
    A[3039] = -0.0352295274170273*G0_0_1 - 0.0352295274170274*G0_1_0 + 0.0962288017409544*G0_1_1;
    A[3040] = -0.0505608958299925*G0_0_0 - 0.0505608958299925*G0_0_1 - 0.0479513012065088*G0_0_2 - 0.0505608958299921*G0_1_0 - 0.0505608958299921*G0_1_1 - 0.0479513012065087*G0_1_2 - 0.0479513012065083*G0_2_0 - 0.0479513012065083*G0_2_1 - 0.0453417065830252*G0_2_2;
    A[3041] = 0.0391439193522531*G0_0_0 + 0.039143919352253*G0_0_1 + 0.00717638521457903*G0_0_2 + 0.0391439193522519*G0_1_0 + 0.039143919352252*G0_1_1 + 0.00717638521457937*G0_1_2 + 0.00717638521457888*G0_2_0 + 0.00717638521457889*G0_2_1 + 0.00130479731174105*G0_2_2;
    A[3042] = 0.0260959462348355*G0_0_0 + 0.0260959462348355*G0_0_1 - 0.00587158790283672*G0_0_2 + 0.0260959462348357*G0_1_0 + 0.0260959462348357*G0_1_1 - 0.00587158790283738*G0_1_2 - 0.00587158790283676*G0_2_0 - 0.0058715879028368*G0_2_1 + 0.00130479731174245*G0_2_2;
    A[3043] = 0.0026095946234827*G0_0_2 + 0.00260959462348328*G0_1_2 + 0.00260959462348317*G0_2_0 + 0.0026095946234833*G0_2_1 - 0.0453417065830262*G0_2_2;
    A[3044] = -0.156575677409009*G0_0_0 - 0.153966082785526*G0_0_1 - 0.156575677409009*G0_0_2 - 0.153966082785525*G0_1_0 - 0.151356488162042*G0_1_1 - 0.153966082785525*G0_1_2 - 0.156575677409009*G0_2_0 - 0.153966082785526*G0_2_1 - 0.156575677409009*G0_2_2;
    A[3045] = 0.185933616923198*G0_0_0 + 0.170276049182297*G0_0_1 + 0.185933616923198*G0_0_2 + 0.170276049182297*G0_1_0 + 0.180714427676231*G0_1_1 + 0.170276049182297*G0_1_2 + 0.185933616923199*G0_2_0 + 0.170276049182298*G0_2_1 + 0.185933616923199*G0_2_2;
    A[3046] = -0.185933616923198*G0_0_0 - 0.0955764030850824*G0_0_1 - 0.185933616923198*G0_0_2 - 0.0955764030850821*G0_1_0 - 0.161794866655976*G0_1_1 - 0.095576403085082*G0_1_2 - 0.185933616923199*G0_2_0 - 0.0955764030850833*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3047] = 0.0352295274170267*G0_0_1 + 0.0352295274170273*G0_1_0 + 0.166687856575008*G0_1_1 + 0.0352295274170274*G0_1_2 + 0.0352295274170272*G0_2_1;
    A[3048] = -0.0453417065830253*G0_0_0 - 0.0479513012065091*G0_0_1 - 0.047951301206509*G0_0_2 - 0.0479513012065088*G0_1_0 - 0.0505608958299923*G0_1_1 - 0.0505608958299922*G0_1_2 - 0.047951301206509*G0_2_0 - 0.0505608958299921*G0_2_1 - 0.050560895829992*G0_2_2;
    A[3049] = 0.00130479731174126*G0_0_0 + 0.00717638521457919*G0_0_1 + 0.0071763852145791*G0_0_2 + 0.00717638521457939*G0_1_0 + 0.039143919352252*G0_1_1 + 0.039143919352252*G0_1_2 + 0.00717638521457946*G0_2_0 + 0.0391439193522518*G0_2_1 + 0.0391439193522518*G0_2_2;
    A[3050] = 0.00130479731174218*G0_0_0 - 0.00587158790283793*G0_0_1 - 0.0058715879028379*G0_0_2 - 0.00587158790283775*G0_1_0 + 0.026095946234835*G0_1_1 + 0.026095946234835*G0_1_2 - 0.0058715879028375*G0_2_0 + 0.0260959462348353*G0_2_1 + 0.0260959462348354*G0_2_2;
    A[3051] = -0.0453417065830256*G0_0_0 + 0.00260959462348356*G0_0_1 + 0.00260959462348358*G0_0_2 + 0.00260959462348367*G0_1_0 + 0.00260959462348345*G0_2_0;
    A[3052] = 0.293579395141891*G0_0_0 + 0.0750258454251492*G0_0_1 + 0.0587158790283772*G0_0_2 + 0.0750258454251511*G0_1_0 + 0.0260959462348349*G0_1_2 + 0.058715879028378*G0_2_0 + 0.026095946234834*G0_2_1;
    A[3053] = -0.782878387045047*G0_0_0 - 0.298472385060927*G0_0_1 - 0.298472385060925*G0_0_2 - 0.298472385060924*G0_1_0 + 0.146789697570945*G0_1_1 - 0.102752788299662*G0_1_2 - 0.298472385060924*G0_2_0 - 0.102752788299663*G0_2_1;
    A[3054] = -0.316413348097373*G0_0_1 - 0.185933616923198*G0_0_2 - 0.316413348097371*G0_1_0 - 0.782878387045046*G0_1_1 - 0.316413348097371*G0_1_2 - 0.185933616923199*G0_2_0 - 0.316413348097373*G0_2_1;
    A[3055] = -0.0489298991903158*G0_0_0 + 0.0929668084615992*G0_0_1 - 0.029357939514191*G0_0_2 + 0.0929668084615991*G0_1_0 + 0.0929668084615994*G0_1_2 - 0.0293579395141889*G0_2_0 + 0.0929668084615999*G0_2_1 - 0.0489298991903151*G0_2_2;
    A[3056] = -0.102752788299662*G0_0_1 - 0.298472385060925*G0_0_2 - 0.10275278829966*G0_1_0 + 0.14678969757095*G0_1_1 - 0.298472385060923*G0_1_2 - 0.298472385060924*G0_2_0 - 0.298472385060924*G0_2_1 - 0.782878387045049*G0_2_2;
    A[3057] = 0.0260959462348332*G0_0_1 + 0.0587158790283784*G0_0_2 + 0.0260959462348369*G0_1_0 + 0.0750258454251519*G0_1_2 + 0.0587158790283767*G0_2_0 + 0.0750258454251487*G0_2_1 + 0.293579395141892*G0_2_2;
    A[3058] = 0.293579395141894*G0_0_0 + 0.218553549716742*G0_0_1 + 0.234863516113515*G0_0_2 + 0.21855354971674*G0_1_0 + 0.143527704291591*G0_1_1 + 0.185933616923197*G0_1_2 + 0.234863516113514*G0_2_0 + 0.185933616923198*G0_2_1 + 0.176147637085134*G0_2_2;
    A[3059] = -0.782878387045048*G0_0_0 - 0.484406001984123*G0_0_1 - 0.484406001984124*G0_0_2 - 0.48440600198412*G0_1_0 - 0.0391439193522528*G0_1_1 - 0.28868640522286*G0_1_2 - 0.484406001984122*G0_2_0 - 0.288686405222862*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3060] = 0.316413348097374*G0_0_1 + 0.185933616923199*G0_0_2 + 0.316413348097371*G0_1_0 - 0.150051690850301*G0_1_1 + 0.185933616923197*G0_1_2 + 0.1859336169232*G0_2_0 + 0.1859336169232*G0_2_1 + 0.371867233846398*G0_2_2;
    A[3061] = -0.0489298991903154*G0_0_0 - 0.141896707651916*G0_0_1 - 0.0195719596761269*G0_0_2 - 0.141896707651913*G0_1_0 - 0.234863516113512*G0_1_1 - 0.0195719596761256*G0_1_2 - 0.0195719596761244*G0_2_0 - 0.0195719596761243*G0_2_1 - 0.0391439193522501*G0_2_2;
    A[3062] = 0.102752788299665*G0_0_1 + 0.298472385060926*G0_0_2 + 0.10275278829966*G0_1_0 + 0.352295274170272*G0_1_1 + 0.102752788299663*G0_1_2 + 0.298472385060924*G0_2_0 + 0.102752788299662*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3063] = -0.0260959462348361*G0_0_1 - 0.0587158790283776*G0_0_2 - 0.0260959462348367*G0_1_0 - 0.0521918924696721*G0_1_1 - 0.00978597983806401*G0_1_2 - 0.0587158790283764*G0_2_0 - 0.00978597983806375*G0_2_1 + 0.176147637085135*G0_2_2;
    A[3064] = 0.0456679059109608*G0_0_0 + 0.120693751336113*G0_0_1 + 0.0717638521457969*G0_0_2 + 0.120693751336111*G0_1_0 + 0.195719596761261*G0_1_1 + 0.12069375133611*G0_1_2 + 0.0717638521457945*G0_2_0 + 0.120693751336111*G0_2_1 + 0.0456679059109599*G0_2_2;
    A[3065] = 0.00978597983806362*G0_0_0 + 0.00489298991903205*G0_0_1 + 0.00489298991903278*G0_0_2 + 0.00489298991903231*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0538228891093469*G0_1_2 + 0.00489298991903221*G0_2_0 + 0.053822889109347*G0_2_1 - 0.0391439193522519*G0_2_2;
    A[3066] = 0.0456679059109607*G0_0_0 - 0.075025845425149*G0_0_1 - 0.0260959462348346*G0_0_2 - 0.0750258454251511*G0_1_0 - 0.0260959462348356*G0_1_2 - 0.0260959462348346*G0_2_0 - 0.0260959462348338*G0_2_1 - 0.0521918924696706*G0_2_2;
    A[3067] = -0.0391439193522524*G0_0_0 + 0.0538228891093471*G0_0_1 + 0.00489298991903029*G0_0_2 + 0.0538228891093458*G0_1_0 + 0.146789697570945*G0_1_1 + 0.00489298991903155*G0_1_2 + 0.0048929899190326*G0_2_0 + 0.00489298991903214*G0_2_1 + 0.00978597983806459*G0_2_2;
    A[3068] = -0.0391439193522517*G0_0_0 - 0.0929668084615995*G0_0_1 - 0.0440369092712852*G0_0_2 - 0.0929668084615971*G0_1_0 - 0.0929668084615975*G0_1_2 - 0.0440369092712843*G0_2_0 - 0.0929668084616001*G0_2_1 - 0.0391439193522517*G0_2_2;
    A[3069] = -0.0521918924696707*G0_0_0 - 0.026095946234833*G0_0_1 - 0.0260959462348336*G0_0_2 - 0.026095946234835*G0_1_0 - 0.0750258454251505*G0_1_2 - 0.0260959462348356*G0_2_0 - 0.0750258454251487*G0_2_1 + 0.0456679059109603*G0_2_2;
    A[3070] = 0.176147637085135*G0_0_0 + 0.185933616923199*G0_0_1 + 0.234863516113514*G0_0_2 + 0.185933616923197*G0_1_0 + 0.143527704291591*G0_1_1 + 0.218553549716741*G0_1_2 + 0.234863516113513*G0_2_0 + 0.218553549716742*G0_2_1 + 0.293579395141893*G0_2_2;
    A[3071] = -0.0391439193522512*G0_0_0 - 0.0195719596761236*G0_0_1 - 0.0195719596761238*G0_0_2 - 0.0195719596761255*G0_1_0 - 0.234863516113513*G0_1_1 - 0.141896707651913*G0_1_2 - 0.0195719596761252*G0_2_0 - 0.141896707651912*G0_2_1 - 0.0489298991903133*G0_2_2;
    A[3072] = 0.176147637085135*G0_0_0 - 0.00978597983806262*G0_0_1 - 0.058715879028377*G0_0_2 - 0.0097859798380635*G0_1_0 - 0.0521918924696698*G0_1_1 - 0.0260959462348348*G0_1_2 - 0.0587158790283786*G0_2_0 - 0.0260959462348357*G0_2_1;
    A[3073] = -0.185933616923197*G0_0_0 - 0.288686405222861*G0_0_1 - 0.484406001984121*G0_0_2 - 0.288686405222859*G0_1_0 - 0.0391439193522513*G0_1_1 - 0.484406001984119*G0_1_2 - 0.484406001984121*G0_2_0 - 0.484406001984123*G0_2_1 - 0.782878387045045*G0_2_2;
    A[3074] = -0.185933616923199*G0_0_0 + 0.102752788299659*G0_0_1 + 0.298472385060924*G0_0_2 + 0.102752788299661*G0_1_0 + 0.352295274170269*G0_1_1 + 0.102752788299662*G0_1_2 + 0.298472385060922*G0_2_0 + 0.10275278829966*G0_2_1;
    A[3075] = 0.371867233846396*G0_0_0 + 0.185933616923199*G0_0_1 + 0.185933616923198*G0_0_2 + 0.185933616923197*G0_1_0 - 0.150051690850301*G0_1_1 + 0.316413348097371*G0_1_2 + 0.185933616923198*G0_2_0 + 0.316413348097373*G0_2_1;
    A[3076] = -0.29357939514189*G0_0_0 - 0.587158790283784*G0_0_1 - 0.342509294332208*G0_0_2 - 0.587158790283783*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283784*G0_1_2 - 0.342509294332204*G0_2_0 - 0.587158790283779*G0_2_1 - 0.293579395141891*G0_2_2;
    A[3077] = -0.293579395141893*G0_0_0 + 0.293579395141894*G0_0_1 + 0.048929899190315*G0_0_2 + 0.293579395141892*G0_1_0 - 0.293579395141893*G0_1_1 + 0.0489298991903139*G0_1_2 + 0.0489298991903124*G0_2_0 + 0.0489298991903157*G0_2_1 + 0.0978597983806284*G0_2_2;
    A[3078] = 1.5657567740901*G0_0_0 + 0.782878387045045*G0_0_1 + 0.782878387045046*G0_0_2 + 0.782878387045045*G0_1_0 + 1.76147637085135*G0_1_1 + 0.782878387045042*G0_1_2 + 0.782878387045046*G0_2_0 + 0.782878387045042*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3079] = 0.0978597983806316*G0_0_0 + 0.0489298991903147*G0_0_1 + 0.0489298991903149*G0_0_2 + 0.0489298991903146*G0_1_0 - 0.293579395141894*G0_1_1 + 0.293579395141892*G0_1_2 + 0.0489298991903135*G0_2_0 + 0.293579395141892*G0_2_1 - 0.293579395141893*G0_2_2;
    A[3080] = 0.0440369092712843*G0_0_0 + 0.0440369092712842*G0_0_1 + 0.0440369092712842*G0_0_2 + 0.0440369092712844*G0_1_0 + 0.0440369092712843*G0_1_1 + 0.0440369092712843*G0_1_2 + 0.0440369092712841*G0_2_0 + 0.0440369092712841*G0_2_1 + 0.0440369092712841*G0_2_2;
    A[3081] = 0.0440369092712842*G0_0_0;
    A[3082] = 0.0440369092712841*G0_1_1;
    A[3083] = -0.0782878387045046*G0_2_2;
    A[3084] = -0.15657567740901*G0_1_1 - 0.00260959462348317*G0_1_2 - 0.0026095946234833*G0_2_1;
    A[3085] = 0.1859336169232*G0_1_1 + 0.0156575677409007*G0_1_2 + 0.0156575677409015*G0_2_1 + 0.0260959462348343*G0_2_2;
    A[3086] = -0.185933616923199*G0_1_1 - 0.0903572138381151*G0_1_2 - 0.0903572138381156*G0_2_1 - 0.156575677409008*G0_2_2;
    A[3087] = -0.035229527417027*G0_1_2 - 0.035229527417026*G0_2_1 + 0.0962288017409544*G0_2_2;
    A[3088] = -0.15657567740901*G0_0_0 - 0.00260959462348357*G0_0_2 - 0.00260959462348372*G0_2_0;
    A[3089] = 0.1859336169232*G0_0_0 + 0.0156575677409011*G0_0_2 + 0.015657567740902*G0_2_0 + 0.0260959462348363*G0_2_2;
    A[3090] = -0.1859336169232*G0_0_0 - 0.090357213838116*G0_0_2 - 0.090357213838118*G0_2_0 - 0.156575677409012*G0_2_2;
    A[3091] = -0.0352295274170272*G0_0_2 - 0.0352295274170262*G0_2_0 + 0.0962288017409546*G0_2_2;
    A[3092] = -0.0505608958299923*G0_0_0 - 0.0026095946234835*G0_0_1 - 0.0026095946234841*G0_1_0;
    A[3093] = 0.0391439193522523*G0_0_0 + 0.0319675341376732*G0_0_1 + 0.0319675341376733*G0_1_0 + 0.0260959462348352*G0_1_1;
    A[3094] = 0.0260959462348354*G0_0_0 + 0.0319675341376728*G0_0_1 + 0.0319675341376726*G0_1_0 + 0.0391439193522522*G0_1_1;
    A[3095] = -0.00260959462348306*G0_0_1 - 0.00260959462348318*G0_1_0 - 0.0505608958299923*G0_1_1;
    A[3096] = -0.156575677409009*G0_0_0 - 0.156575677409009*G0_0_1 - 0.153966082785527*G0_0_2 - 0.156575677409009*G0_1_0 - 0.156575677409009*G0_1_1 - 0.153966082785526*G0_1_2 - 0.153966082785526*G0_2_0 - 0.153966082785526*G0_2_1 - 0.151356488162043*G0_2_2;
    A[3097] = 0.185933616923197*G0_0_0 + 0.185933616923197*G0_0_1 + 0.170276049182298*G0_0_2 + 0.185933616923197*G0_1_0 + 0.185933616923197*G0_1_1 + 0.170276049182298*G0_1_2 + 0.170276049182297*G0_2_0 + 0.170276049182297*G0_2_1 + 0.180714427676232*G0_2_2;
    A[3098] = -0.185933616923197*G0_0_0 - 0.185933616923197*G0_0_1 - 0.0955764030850821*G0_0_2 - 0.185933616923197*G0_1_0 - 0.185933616923197*G0_1_1 - 0.0955764030850822*G0_1_2 - 0.0955764030850818*G0_2_0 - 0.0955764030850817*G0_2_1 - 0.161794866655975*G0_2_2;
    A[3099] = 0.0352295274170263*G0_0_2 + 0.0352295274170263*G0_1_2 + 0.035229527417026*G0_2_0 + 0.0352295274170259*G0_2_1 + 0.166687856575007*G0_2_2;
    A[3100] = -0.0505608958299922*G0_0_0 - 0.0479513012065095*G0_0_1 - 0.0505608958299922*G0_0_2 - 0.0479513012065086*G0_1_0 - 0.0453417065830262*G0_1_1 - 0.0479513012065084*G0_1_2 - 0.0505608958299925*G0_2_0 - 0.0479513012065094*G0_2_1 - 0.0505608958299924*G0_2_2;
    A[3101] = 0.0391439193522518*G0_0_0 + 0.00717638521458003*G0_0_1 + 0.0391439193522518*G0_0_2 + 0.00717638521457916*G0_1_0 + 0.0013047973117422*G0_1_1 + 0.00717638521457916*G0_1_2 + 0.039143919352251*G0_2_0 + 0.00717638521457957*G0_2_1 + 0.0391439193522509*G0_2_2;
    A[3102] = 0.0260959462348353*G0_0_0 - 0.00587158790283838*G0_0_1 + 0.0260959462348352*G0_0_2 - 0.005871587902838*G0_1_0 + 0.00130479731174126*G0_1_1 - 0.005871587902838*G0_1_2 + 0.0260959462348346*G0_2_0 - 0.00587158790283773*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[3103] = 0.00260959462348347*G0_0_1 + 0.00260959462348311*G0_1_0 - 0.0453417065830257*G0_1_1 + 0.00260959462348328*G0_1_2 + 0.0026095946234832*G0_2_1;
    A[3104] = -0.0453417065830263*G0_0_0 - 0.0479513012065088*G0_0_1 - 0.0479513012065088*G0_0_2 - 0.0479513012065097*G0_1_0 - 0.0505608958299918*G0_1_1 - 0.0505608958299918*G0_1_2 - 0.0479513012065095*G0_2_0 - 0.0505608958299925*G0_2_1 - 0.0505608958299925*G0_2_2;
    A[3105] = 0.0013047973117423*G0_0_0 + 0.00717638521458006*G0_0_1 + 0.00717638521457992*G0_0_2 + 0.00717638521457977*G0_1_0 + 0.0391439193522524*G0_1_1 + 0.0391439193522523*G0_1_2 + 0.0071763852145798*G0_2_0 + 0.0391439193522522*G0_2_1 + 0.0391439193522523*G0_2_2;
    A[3106] = 0.00130479731174132*G0_0_0 - 0.00587158790283841*G0_0_1 - 0.00587158790283837*G0_0_2 - 0.00587158790283762*G0_1_0 + 0.026095946234835*G0_1_1 + 0.0260959462348351*G0_1_2 - 0.00587158790283797*G0_2_0 + 0.0260959462348349*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[3107] = -0.0453417065830255*G0_0_0 + 0.00260959462348334*G0_0_1 + 0.00260959462348334*G0_0_2 + 0.00260959462348329*G0_1_0 + 0.00260959462348362*G0_2_0;
    A[3108] = 0.293579395141895*G0_0_0 + 0.0587158790283809*G0_0_1 + 0.0750258454251532*G0_0_2 + 0.0587158790283802*G0_1_0 + 0.0260959462348352*G0_1_2 + 0.0750258454251542*G0_2_0 + 0.0260959462348379*G0_2_1;
    A[3109] = -0.0489298991903159*G0_0_0 - 0.0293579395141907*G0_0_1 + 0.0929668084615986*G0_0_2 - 0.0293579395141884*G0_1_0 - 0.0489298991903136*G0_1_1 + 0.0929668084616005*G0_1_2 + 0.0929668084616004*G0_2_0 + 0.0929668084616004*G0_2_1;
    A[3110] = 0.0587158790283809*G0_0_1 + 0.0260959462348368*G0_0_2 + 0.0587158790283783*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0750258454251507*G0_1_2 + 0.0260959462348369*G0_2_0 + 0.0750258454251528*G0_2_1;
    A[3111] = -0.78287838704505*G0_0_0 - 0.298472385060926*G0_0_1 - 0.298472385060927*G0_0_2 - 0.298472385060926*G0_1_0 - 0.102752788299663*G0_1_2 - 0.298472385060928*G0_2_0 - 0.102752788299666*G0_2_1 + 0.146789697570946*G0_2_2;
    A[3112] = -0.298472385060923*G0_0_1 - 0.10275278829966*G0_0_2 - 0.298472385060923*G0_1_0 - 0.782878387045047*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299661*G0_2_0 - 0.298472385060923*G0_2_1 + 0.146789697570948*G0_2_2;
    A[3113] = -0.185933616923199*G0_0_1 - 0.316413348097373*G0_0_2 - 0.185933616923198*G0_1_0 - 0.316413348097374*G0_1_2 - 0.316413348097371*G0_2_0 - 0.316413348097374*G0_2_1 - 0.782878387045048*G0_2_2;
    A[3114] = 0.293579395141891*G0_0_0 + 0.234863516113513*G0_0_1 + 0.218553549716741*G0_0_2 + 0.234863516113514*G0_1_0 + 0.176147637085135*G0_1_1 + 0.185933616923197*G0_1_2 + 0.218553549716741*G0_2_0 + 0.185933616923198*G0_2_1 + 0.143527704291592*G0_2_2;
    A[3115] = -0.0489298991903162*G0_0_0 - 0.0195719596761259*G0_0_1 - 0.141896707651915*G0_0_2 - 0.0195719596761266*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0195719596761258*G0_1_2 - 0.141896707651915*G0_2_0 - 0.0195719596761267*G0_2_1 - 0.234863516113513*G0_2_2;
    A[3116] = -0.0587158790283784*G0_0_1 - 0.0260959462348355*G0_0_2 - 0.058715879028378*G0_1_0 + 0.176147637085136*G0_1_1 - 0.00978597983806252*G0_1_2 - 0.0260959462348369*G0_2_0 - 0.00978597983806252*G0_2_1 - 0.0521918924696696*G0_2_2;
    A[3117] = -0.78287838704505*G0_0_0 - 0.484406001984123*G0_0_1 - 0.484406001984121*G0_0_2 - 0.484406001984121*G0_1_0 - 0.185933616923197*G0_1_1 - 0.288686405222859*G0_1_2 - 0.484406001984123*G0_2_0 - 0.28868640522286*G0_2_1 - 0.0391439193522531*G0_2_2;
    A[3118] = 0.298472385060925*G0_0_1 + 0.102752788299663*G0_0_2 + 0.298472385060923*G0_1_0 - 0.1859336169232*G0_1_1 + 0.102752788299661*G0_1_2 + 0.102752788299661*G0_2_0 + 0.102752788299662*G0_2_1 + 0.35229527417027*G0_2_2;
    A[3119] = 0.185933616923197*G0_0_1 + 0.316413348097373*G0_0_2 + 0.185933616923198*G0_1_0 + 0.371867233846396*G0_1_1 + 0.185933616923197*G0_1_2 + 0.316413348097371*G0_2_0 + 0.185933616923197*G0_2_1 - 0.150051690850301*G0_2_2;
    A[3120] = 0.176147637085136*G0_0_0 + 0.234863516113513*G0_0_1 + 0.185933616923197*G0_0_2 + 0.234863516113514*G0_1_0 + 0.293579395141893*G0_1_1 + 0.218553549716741*G0_1_2 + 0.185933616923199*G0_2_0 + 0.218553549716741*G0_2_1 + 0.143527704291592*G0_2_2;
    A[3121] = -0.0391439193522535*G0_0_0 - 0.0195719596761259*G0_0_1 - 0.0195719596761265*G0_0_2 - 0.0195719596761272*G0_1_0 - 0.0489298991903151*G0_1_1 - 0.141896707651915*G0_1_2 - 0.019571959676126*G0_2_0 - 0.141896707651915*G0_2_1 - 0.234863516113514*G0_2_2;
    A[3122] = 0.176147637085137*G0_0_0 - 0.05871587902838*G0_0_1 - 0.00978597983806224*G0_0_2 - 0.0587158790283775*G0_1_0 - 0.0260959462348349*G0_1_2 - 0.00978597983806307*G0_2_0 - 0.0260959462348374*G0_2_1 - 0.0521918924696714*G0_2_2;
    A[3123] = -0.185933616923197*G0_0_0 - 0.484406001984124*G0_0_1 - 0.28868640522286*G0_0_2 - 0.484406001984122*G0_1_0 - 0.782878387045047*G0_1_1 - 0.484406001984122*G0_1_2 - 0.288686405222862*G0_2_0 - 0.484406001984124*G0_2_1 - 0.0391439193522522*G0_2_2;
    A[3124] = -0.185933616923199*G0_0_0 + 0.298472385060925*G0_0_1 + 0.102752788299662*G0_0_2 + 0.298472385060924*G0_1_0 + 0.102752788299663*G0_1_2 + 0.102752788299663*G0_2_0 + 0.102752788299665*G0_2_1 + 0.352295274170275*G0_2_2;
    A[3125] = 0.371867233846397*G0_0_0 + 0.185933616923199*G0_0_1 + 0.185933616923199*G0_0_2 + 0.185933616923199*G0_1_0 + 0.316413348097373*G0_1_2 + 0.1859336169232*G0_2_0 + 0.316413348097374*G0_2_1 - 0.150051690850302*G0_2_2;
    A[3126] = 0.045667905910961*G0_0_0 + 0.071763852145795*G0_0_1 + 0.120693751336111*G0_0_2 + 0.071763852145796*G0_1_0 + 0.0456679059109601*G0_1_1 + 0.120693751336113*G0_1_2 + 0.120693751336112*G0_2_0 + 0.120693751336111*G0_2_1 + 0.19571959676126*G0_2_2;
    A[3127] = 0.00978597983806301*G0_0_0 + 0.00489298991903197*G0_0_1 + 0.00489298991903254*G0_0_2 + 0.0048929899190308*G0_1_0 - 0.0391439193522516*G0_1_1 + 0.0538228891093473*G0_1_2 + 0.00489298991903147*G0_2_0 + 0.0538228891093479*G0_2_1 + 0.146789697570948*G0_2_2;
    A[3128] = 0.0456679059109608*G0_0_0 - 0.0260959462348346*G0_0_1 - 0.0750258454251524*G0_0_2 - 0.026095946234834*G0_1_0 - 0.0521918924696703*G0_1_1 - 0.0260959462348346*G0_1_2 - 0.0750258454251503*G0_2_0 - 0.0260959462348354*G0_2_1;
    A[3129] = -0.0391439193522522*G0_0_0 + 0.00489298991903199*G0_0_1 + 0.0538228891093473*G0_0_2 + 0.00489298991903156*G0_1_0 + 0.00978597983806273*G0_1_1 + 0.0048929899190312*G0_1_2 + 0.0538228891093481*G0_2_0 + 0.00489298991903105*G0_2_1 + 0.146789697570946*G0_2_2;
    A[3130] = -0.0391439193522521*G0_0_0 - 0.0440369092712846*G0_0_1 - 0.092966808461599*G0_0_2 - 0.0440369092712843*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0929668084616003*G0_1_2 - 0.0929668084616004*G0_2_0 - 0.0929668084616006*G0_2_1;
    A[3131] = -0.0521918924696704*G0_0_0 - 0.0260959462348351*G0_0_1 - 0.0260959462348367*G0_0_2 - 0.0260959462348345*G0_1_0 + 0.0456679059109611*G0_1_1 - 0.0750258454251505*G0_1_2 - 0.0260959462348345*G0_2_0 - 0.0750258454251496*G0_2_1;
    A[3132] = -0.293579395141892*G0_0_0 - 0.342509294332209*G0_0_1 - 0.587158790283789*G0_0_2 - 0.342509294332208*G0_1_0 - 0.293579395141892*G0_1_1 - 0.587158790283786*G0_1_2 - 0.587158790283786*G0_2_0 - 0.587158790283787*G0_2_1 - 1.17431758056757*G0_2_2;
    A[3133] = -0.293579395141894*G0_0_0 + 0.0489298991903165*G0_0_1 + 0.293579395141894*G0_0_2 + 0.0489298991903136*G0_1_0 + 0.0978597983806299*G0_1_1 + 0.0489298991903146*G0_1_2 + 0.293579395141892*G0_2_0 + 0.048929899190315*G0_2_1 - 0.293579395141895*G0_2_2;
    A[3134] = 0.0978597983806316*G0_0_0 + 0.0489298991903146*G0_0_1 + 0.0489298991903135*G0_0_2 + 0.0489298991903147*G0_1_0 - 0.293579395141894*G0_1_1 + 0.293579395141892*G0_1_2 + 0.0489298991903149*G0_2_0 + 0.293579395141892*G0_2_1 - 0.293579395141893*G0_2_2;
    A[3135] = 1.5657567740901*G0_0_0 + 0.782878387045047*G0_0_1 + 0.782878387045051*G0_0_2 + 0.782878387045047*G0_1_0 + 1.56575677409009*G0_1_1 + 0.782878387045047*G0_1_2 + 0.782878387045051*G0_2_0 + 0.782878387045047*G0_2_1 + 1.76147637085136*G0_2_2;
  }

  /// Tabulate the tensor for the contribution from a local cell
  /// using the specified reference cell quadrature points/weights
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int num_quadrature_points,
                               const double * const * quadrature_points,
                               const double* quadrature_weights) const
  {
    throw std::runtime_error("Quadrature version of tabulate_tensor not available when using the FFC tensor representation.");
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_cell_integral_1_0: public ufc::cell_integral
{
public:

  /// Constructor
  poisson3d_5_cell_integral_1_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_cell_integral_1_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      18
    // Number of operations (multiply-add pairs) for geometry tensor:    56
    // Number of operations (multiply-add pairs) for tensor contraction: 3108
    // Total number of operations (multiply-add pairs):                  3182
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_20 = J_01*J_12 - J_02*J_11;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0 = det*w[0][0]*(1.0);
    const double G0_1 = det*w[0][1]*(1.0);
    const double G0_2 = det*w[0][2]*(1.0);
    const double G0_3 = det*w[0][3]*(1.0);
    const double G0_4 = det*w[0][4]*(1.0);
    const double G0_5 = det*w[0][5]*(1.0);
    const double G0_6 = det*w[0][6]*(1.0);
    const double G0_7 = det*w[0][7]*(1.0);
    const double G0_8 = det*w[0][8]*(1.0);
    const double G0_9 = det*w[0][9]*(1.0);
    const double G0_10 = det*w[0][10]*(1.0);
    const double G0_11 = det*w[0][11]*(1.0);
    const double G0_12 = det*w[0][12]*(1.0);
    const double G0_13 = det*w[0][13]*(1.0);
    const double G0_14 = det*w[0][14]*(1.0);
    const double G0_15 = det*w[0][15]*(1.0);
    const double G0_16 = det*w[0][16]*(1.0);
    const double G0_17 = det*w[0][17]*(1.0);
    const double G0_18 = det*w[0][18]*(1.0);
    const double G0_19 = det*w[0][19]*(1.0);
    const double G0_20 = det*w[0][20]*(1.0);
    const double G0_21 = det*w[0][21]*(1.0);
    const double G0_22 = det*w[0][22]*(1.0);
    const double G0_23 = det*w[0][23]*(1.0);
    const double G0_24 = det*w[0][24]*(1.0);
    const double G0_25 = det*w[0][25]*(1.0);
    const double G0_26 = det*w[0][26]*(1.0);
    const double G0_27 = det*w[0][27]*(1.0);
    const double G0_28 = det*w[0][28]*(1.0);
    const double G0_29 = det*w[0][29]*(1.0);
    const double G0_30 = det*w[0][30]*(1.0);
    const double G0_31 = det*w[0][31]*(1.0);
    const double G0_32 = det*w[0][32]*(1.0);
    const double G0_33 = det*w[0][33]*(1.0);
    const double G0_34 = det*w[0][34]*(1.0);
    const double G0_35 = det*w[0][35]*(1.0);
    const double G0_36 = det*w[0][36]*(1.0);
    const double G0_37 = det*w[0][37]*(1.0);
    const double G0_38 = det*w[0][38]*(1.0);
    const double G0_39 = det*w[0][39]*(1.0);
    const double G0_40 = det*w[0][40]*(1.0);
    const double G0_41 = det*w[0][41]*(1.0);
    const double G0_42 = det*w[0][42]*(1.0);
    const double G0_43 = det*w[0][43]*(1.0);
    const double G0_44 = det*w[0][44]*(1.0);
    const double G0_45 = det*w[0][45]*(1.0);
    const double G0_46 = det*w[0][46]*(1.0);
    const double G0_47 = det*w[0][47]*(1.0);
    const double G0_48 = det*w[0][48]*(1.0);
    const double G0_49 = det*w[0][49]*(1.0);
    const double G0_50 = det*w[0][50]*(1.0);
    const double G0_51 = det*w[0][51]*(1.0);
    const double G0_52 = det*w[0][52]*(1.0);
    const double G0_53 = det*w[0][53]*(1.0);
    const double G0_54 = det*w[0][54]*(1.0);
    const double G0_55 = det*w[0][55]*(1.0);
    
    // Compute element tensor
    A[0] = 8.34957866207912e-05*G0_0 + 9.27851405282019e-06*G0_1 + 9.2785140528201e-06*G0_2 + 9.27851405282014e-06*G0_3 + 4.07498237359356e-06*G0_4 + 1.52962392545736e-05*G0_5 + 1.52962392545731e-05*G0_6 + 4.07498237359354e-06*G0_7 + 4.07498237359382e-06*G0_8 + 1.52962392545743e-05*G0_9 + 1.52962392545737e-05*G0_10 + 4.07498237359347e-06*G0_11 + 4.07498237359384e-06*G0_12 + 1.52962392545734e-05*G0_13 + 1.52962392545738e-05*G0_14 + 4.07498237359319e-06*G0_15 - 6.04020786312489e-05*G0_16 + 6.04020786312484e-05*G0_17 - 4.43831824040176e-05*G0_18 + 1.81868671452013e-05*G0_19 - 6.04020786312478e-05*G0_20 + 6.04020786312477e-05*G0_21 - 4.43831824040171e-05*G0_22 + 1.8186867145201e-05*G0_23 - 6.04020786312489e-05*G0_24 + 6.04020786312488e-05*G0_25 - 4.43831824040181e-05*G0_26 + 1.81868671452016e-05*G0_27 + 0.000173437673437683*G0_28 - 1.77151327324947e-05*G0_29 + 0.000173437673437683*G0_30 - 1.77151327324945e-05*G0_31 - 1.77151327324955e-05*G0_32 + 0.000173437673437684*G0_33 + 0.000149650214754389*G0_34 - 0.000121496703527958*G0_35 + 9.5952786925014e-05*G0_36 - 0.000121496703527961*G0_37 + 3.9846502520116e-05*G0_38 + 9.59527869250149e-05*G0_39 + 0.00014965021475439*G0_40 - 0.00012149670352796*G0_41 + 9.59527869250148e-05*G0_42 - 0.000121496703527961*G0_43 + 3.98465025201163e-05*G0_44 + 9.59527869250154e-05*G0_45 + 0.00014965021475439*G0_46 - 0.000121496703527959*G0_47 + 9.59527869250147e-05*G0_48 - 0.00012149670352796*G0_49 + 3.98465025201151e-05*G0_50 + 9.59527869250149e-05*G0_51 - 0.00028002957690459*G0_52 + 0.000204250963799588*G0_53 + 0.000204250963799588*G0_54 + 0.000204250963799587*G0_55;
    A[1] = 9.27851405282019e-06*G0_0 + 8.34957866207909e-05*G0_1 + 9.27851405282e-06*G0_2 + 9.27851405282001e-06*G0_3 + 4.07498237359389e-06*G0_4 + 1.52962392545731e-05*G0_5 + 1.52962392545732e-05*G0_6 + 4.07498237359362e-06*G0_7 - 6.04020786312483e-05*G0_8 + 6.04020786312489e-05*G0_9 - 4.4383182404018e-05*G0_10 + 1.81868671452014e-05*G0_11 - 6.04020786312484e-05*G0_12 + 6.04020786312484e-05*G0_13 - 4.43831824040178e-05*G0_14 + 1.81868671452014e-05*G0_15 + 4.07498237359336e-06*G0_16 + 1.52962392545733e-05*G0_17 + 1.52962392545737e-05*G0_18 + 4.07498237359372e-06*G0_19 + 4.07498237359406e-06*G0_20 + 1.52962392545729e-05*G0_21 + 1.52962392545739e-05*G0_22 + 4.0749823735936e-06*G0_23 + 1.81868671452013e-05*G0_24 - 4.43831824040181e-05*G0_25 + 6.04020786312483e-05*G0_26 - 6.04020786312483e-05*G0_27 + 0.00014965021475439*G0_28 - 0.000121496703527959*G0_29 + 9.5952786925014e-05*G0_30 - 0.00012149670352796*G0_31 + 3.98465025201156e-05*G0_32 + 9.59527869250139e-05*G0_33 + 0.000173437673437682*G0_34 - 1.77151327324942e-05*G0_35 + 0.000173437673437682*G0_36 - 1.77151327324953e-05*G0_37 - 1.77151327324941e-05*G0_38 + 0.000173437673437682*G0_39 + 9.59527869250139e-05*G0_40 - 0.00012149670352796*G0_41 + 0.000149650214754389*G0_42 + 3.98465025201155e-05*G0_43 - 0.000121496703527959*G0_44 + 9.59527869250138e-05*G0_45 + 9.5952786925014e-05*G0_46 - 0.00012149670352796*G0_47 + 0.000149650214754389*G0_48 + 3.98465025201154e-05*G0_49 - 0.00012149670352796*G0_50 + 9.5952786925014e-05*G0_51 + 0.000204250963799584*G0_52 - 0.000280029576904591*G0_53 + 0.000204250963799586*G0_54 + 0.000204250963799585*G0_55;
    A[2] = 9.2785140528201e-06*G0_0 + 9.27851405282e-06*G0_1 + 8.3495786620791e-05*G0_2 + 9.27851405281999e-06*G0_3 - 6.0402078631249e-05*G0_4 + 6.04020786312487e-05*G0_5 - 4.43831824040182e-05*G0_6 + 1.81868671452012e-05*G0_7 + 4.07498237359398e-06*G0_8 + 1.52962392545738e-05*G0_9 + 1.52962392545738e-05*G0_10 + 4.07498237359351e-06*G0_11 + 1.81868671452018e-05*G0_12 - 4.43831824040184e-05*G0_13 + 6.04020786312493e-05*G0_14 - 6.04020786312494e-05*G0_15 + 4.07498237359372e-06*G0_16 + 1.52962392545731e-05*G0_17 + 1.52962392545742e-05*G0_18 + 4.07498237359346e-06*G0_19 + 1.81868671452025e-05*G0_20 - 4.43831824040197e-05*G0_21 + 6.04020786312503e-05*G0_22 - 6.04020786312495e-05*G0_23 + 4.07498237359386e-06*G0_24 + 1.52962392545733e-05*G0_25 + 1.52962392545736e-05*G0_26 + 4.07498237359378e-06*G0_27 + 9.59527869250137e-05*G0_28 - 0.00012149670352796*G0_29 + 0.000149650214754389*G0_30 + 3.98465025201167e-05*G0_31 - 0.00012149670352796*G0_32 + 9.59527869250134e-05*G0_33 + 9.59527869250138e-05*G0_34 - 0.000121496703527959*G0_35 + 0.000149650214754389*G0_36 + 3.98465025201153e-05*G0_37 - 0.00012149670352796*G0_38 + 9.59527869250134e-05*G0_39 + 0.000173437673437683*G0_40 - 1.77151327324943e-05*G0_41 + 0.000173437673437682*G0_42 - 1.77151327324949e-05*G0_43 - 1.77151327324941e-05*G0_44 + 0.000173437673437682*G0_45 + 9.59527869250144e-05*G0_46 + 3.98465025201169e-05*G0_47 + 9.59527869250141e-05*G0_48 - 0.000121496703527961*G0_49 - 0.000121496703527961*G0_50 + 0.00014965021475439*G0_51 + 0.000204250963799588*G0_52 + 0.000204250963799587*G0_53 - 0.000280029576904595*G0_54 + 0.000204250963799584*G0_55;
    A[3] = 9.27851405282014e-06*G0_0 + 9.27851405282001e-06*G0_1 + 9.27851405281999e-06*G0_2 + 8.34957866207912e-05*G0_3 + 1.81868671452017e-05*G0_4 - 4.43831824040186e-05*G0_5 + 6.04020786312487e-05*G0_6 - 6.0402078631249e-05*G0_7 + 1.81868671452018e-05*G0_8 - 4.43831824040178e-05*G0_9 + 6.04020786312488e-05*G0_10 - 6.04020786312489e-05*G0_11 + 4.07498237359388e-06*G0_12 + 1.52962392545735e-05*G0_13 + 1.52962392545735e-05*G0_14 + 4.07498237359384e-06*G0_15 + 1.81868671452012e-05*G0_16 - 4.43831824040186e-05*G0_17 + 6.04020786312493e-05*G0_18 - 6.04020786312489e-05*G0_19 + 4.07498237359436e-06*G0_20 + 1.5296239254573e-05*G0_21 + 1.52962392545737e-05*G0_22 + 4.07498237359382e-06*G0_23 + 4.07498237359371e-06*G0_24 + 1.52962392545734e-05*G0_25 + 1.52962392545734e-05*G0_26 + 4.07498237359401e-06*G0_27 + 9.59527869250132e-05*G0_28 + 3.98465025201171e-05*G0_29 + 9.59527869250137e-05*G0_30 - 0.00012149670352796*G0_31 - 0.000121496703527961*G0_32 + 0.00014965021475439*G0_33 + 9.59527869250131e-05*G0_34 + 3.98465025201175e-05*G0_35 + 9.59527869250135e-05*G0_36 - 0.000121496703527961*G0_37 - 0.00012149670352796*G0_38 + 0.000149650214754389*G0_39 + 9.59527869250131e-05*G0_40 + 3.98465025201165e-05*G0_41 + 9.59527869250131e-05*G0_42 - 0.00012149670352796*G0_43 - 0.00012149670352796*G0_44 + 0.000149650214754389*G0_45 + 0.000173437673437682*G0_46 - 1.77151327324943e-05*G0_47 + 0.000173437673437682*G0_48 - 1.77151327324944e-05*G0_49 - 1.77151327324942e-05*G0_50 + 0.000173437673437682*G0_51 + 0.000204250963799585*G0_52 + 0.000204250963799586*G0_53 + 0.000204250963799587*G0_54 - 0.000280029576904594*G0_55;
    A[4] = 4.07498237359356e-06*G0_0 + 4.07498237359389e-06*G0_1 - 6.0402078631249e-05*G0_2 + 1.81868671452017e-05*G0_3 + 0.000886258481744639*G0_4 - 0.000858657000150102*G0_5 + 0.000659424487549521*G0_6 - 0.000299350614020768*G0_7 + 5.50356814824437e-05*G0_8 - 0.000178405940124699*G0_9 + 9.0499403288759e-05*G0_10 - 0.000144029549411501*G0_11 - 0.000144029549411502*G0_12 + 0.0003573137253693*G0_13 - 0.000456929981669589*G0_14 + 0.00044312924087232*G0_15 + 5.50356814824431e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887583e-05*G0_18 - 0.000144029549411501*G0_19 - 0.000144029549411503*G0_20 + 0.000357313725369301*G0_21 - 0.00045692998166959*G0_22 + 0.000443129240872321*G0_23 + 2.75178407412213e-05*G0_24 - 2.24993895422144e-05*G0_25 - 2.2499389542215e-05*G0_26 + 2.75178407412218e-05*G0_27 - 4.01476095920555e-05*G0_28 - 5.52029631890772e-05*G0_29 + 0.000110405926378155*G0_30 - 9.28413471816306e-05*G0_31 + 0.000308634748738931*G0_32 - 0.000341254681532476*G0_33 - 4.01476095920553e-05*G0_34 - 5.52029631890778e-05*G0_35 + 0.000110405926378156*G0_36 - 9.28413471816288e-05*G0_37 + 0.00030863474873893*G0_38 - 0.000341254681532475*G0_39 - 0.000266814322080541*G0_40 + 0.000135498182373188*G0_41 - 0.000266814322080539*G0_42 - 8.78228959826194e-06*G0_43 - 8.78228959826302e-06*G0_44 - 0.000555375266023442*G0_45 - 2.00738047960285e-05*G0_46 + 2.88560943942882e-05*G0_47 - 2.00738047960275e-05*G0_48 - 2.76014815945374e-05*G0_49 - 2.76014815945381e-05*G0_50 + 5.52029631890776e-05*G0_51 - 0.000301107071940422*G0_52 - 0.000301107071940422*G0_53 + 0.000727675423856023*G0_54 - 0.000777859935846086*G0_55;
    A[5] = 1.52962392545736e-05*G0_0 + 1.52962392545731e-05*G0_1 + 6.04020786312487e-05*G0_2 - 4.43831824040186e-05*G0_3 - 0.000858657000150102*G0_4 + 0.00168419222238676*G0_5 - 0.00130178624102242*G0_6 + 0.000659424487549523*G0_7 - 0.0001784059401247*G0_8 + 0.000347778668091187*G0_9 - 0.000261963152588166*G0_10 + 0.0003573137253693*G0_11 + 9.04994032887608e-05*G0_12 - 0.000261963152588167*G0_13 + 0.000328541271828326*G0_14 - 0.00045692998166959*G0_15 - 0.000178405940124699*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588165*G0_18 + 0.0003573137253693*G0_19 + 9.04994032887606e-05*G0_20 - 0.000261963152588168*G0_21 + 0.000328541271828327*G0_22 - 0.00045692998166959*G0_23 - 2.24993895422148e-05*G0_24 + 1.12078743444488e-05*G0_25 + 1.12078743444489e-05*G0_26 - 2.24993895422147e-05*G0_27 + 0.000127970505574679*G0_28 - 0.000120442828776168*G0_29 - 0.000110405926378154*G0_30 - 7.02583167860962e-05*G0_31 - 4.51660607910637e-05*G0_32 + 0.000491808217502687*G0_33 + 0.000127970505574678*G0_34 - 0.000120442828776169*G0_35 - 0.000110405926378156*G0_36 - 7.02583167860986e-05*G0_37 - 4.51660607910627e-05*G0_38 + 0.000491808217502686*G0_39 + 0.00017062734076624*G0_40 - 0.000122952054375672*G0_41 + 0.000170627340766238*G0_42 - 0.000148044310370707*G0_43 - 0.000148044310370705*G0_44 + 0.000873210508627221*G0_45 - 0.000109569517844986*G0_46 - 2.25830303955297e-05*G0_47 - 0.000109569517844988*G0_48 + 0.000253431785549852*G0_49 + 0.000253431785549854*G0_50 - 0.000623960765743206*G0_51 + 6.27306399875843e-05*G0_52 + 6.27306399875896e-05*G0_53 - 0.000903321215821267*G0_54 + 0.00110405926378155*G0_55;
    A[6] = 1.52962392545731e-05*G0_0 + 1.52962392545732e-05*G0_1 - 4.43831824040182e-05*G0_2 + 6.04020786312487e-05*G0_3 + 0.000659424487549521*G0_4 - 0.00130178624102242*G0_5 + 0.00168419222238675*G0_6 - 0.000858657000150101*G0_7 + 9.04994032887608e-05*G0_8 - 0.000261963152588165*G0_9 + 0.000328541271828326*G0_10 - 0.000456929981669589*G0_11 - 0.000178405940124699*G0_12 + 0.000347778668091185*G0_13 - 0.000261963152588165*G0_14 + 0.000357313725369299*G0_15 + 9.04994032887589e-05*G0_16 - 0.000261963152588166*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669589*G0_19 - 0.000178405940124698*G0_20 + 0.000347778668091185*G0_21 - 0.000261963152588165*G0_22 + 0.0003573137253693*G0_23 - 2.24993895422153e-05*G0_24 + 1.12078743444496e-05*G0_25 + 1.12078743444484e-05*G0_26 - 2.2499389542214e-05*G0_27 + 0.000127970505574678*G0_28 - 7.02583167860976e-05*G0_29 + 0.000491808217502685*G0_30 - 0.000120442828776168*G0_31 - 4.5166060791065e-05*G0_32 - 0.000110405926378154*G0_33 + 0.000127970505574675*G0_34 - 7.02583167860951e-05*G0_35 + 0.000491808217502685*G0_36 - 0.000120442828776168*G0_37 - 4.51660607910642e-05*G0_38 - 0.000110405926378154*G0_39 - 0.000109569517844989*G0_40 - 2.25830303955307e-05*G0_41 - 0.000109569517844987*G0_42 + 0.000253431785549854*G0_43 + 0.000253431785549855*G0_44 - 0.000623960765743204*G0_45 + 0.000170627340766236*G0_46 - 0.000122952054375671*G0_47 + 0.000170627340766238*G0_48 - 0.000148044310370705*G0_49 - 0.000148044310370708*G0_50 + 0.000873210508627219*G0_51 + 6.27306399875892e-05*G0_52 + 6.2730639987591e-05*G0_53 + 0.00110405926378154*G0_54 - 0.000903321215821265*G0_55;
    A[7] = 4.07498237359354e-06*G0_0 + 4.07498237359362e-06*G0_1 + 1.81868671452012e-05*G0_2 - 6.0402078631249e-05*G0_3 - 0.000299350614020768*G0_4 + 0.000659424487549523*G0_5 - 0.000858657000150101*G0_6 + 0.00088625848174464*G0_7 - 0.000144029549411502*G0_8 + 0.000357313725369299*G0_9 - 0.00045692998166959*G0_10 + 0.000443129240872321*G0_11 + 5.50356814824439e-05*G0_12 - 0.000178405940124699*G0_13 + 9.04994032887594e-05*G0_14 - 0.000144029549411501*G0_15 - 0.000144029549411501*G0_16 + 0.000357313725369299*G0_17 - 0.000456929981669589*G0_18 + 0.000443129240872321*G0_19 + 5.50356814824429e-05*G0_20 - 0.000178405940124698*G0_21 + 9.04994032887587e-05*G0_22 - 0.000144029549411501*G0_23 + 2.75178407412221e-05*G0_24 - 2.24993895422149e-05*G0_25 - 2.24993895422153e-05*G0_26 + 2.75178407412218e-05*G0_27 - 4.01476095920561e-05*G0_28 - 9.28413471816302e-05*G0_29 - 0.000341254681532478*G0_30 - 5.52029631890768e-05*G0_31 + 0.000308634748738935*G0_32 + 0.000110405926378151*G0_33 - 4.01476095920574e-05*G0_34 - 9.28413471816302e-05*G0_35 - 0.000341254681532478*G0_36 - 5.52029631890761e-05*G0_37 + 0.000308634748738933*G0_38 + 0.000110405926378151*G0_39 - 2.00738047960297e-05*G0_40 + 2.88560943942906e-05*G0_41 - 2.0073804796029e-05*G0_42 - 2.76014815945377e-05*G0_43 - 2.76014815945383e-05*G0_44 + 5.5202963189075e-05*G0_45 - 0.000266814322080541*G0_46 + 0.00013549818237319*G0_47 - 0.000266814322080541*G0_48 - 8.7822895982617e-06*G0_49 - 8.78228959826222e-06*G0_50 - 0.000555375266023444*G0_51 - 0.000301107071940424*G0_52 - 0.000301107071940425*G0_53 - 0.000777859935846087*G0_54 + 0.000727675423856017*G0_55;
    A[8] = 4.07498237359382e-06*G0_0 - 6.04020786312483e-05*G0_1 + 4.07498237359398e-06*G0_2 + 1.81868671452018e-05*G0_3 + 5.50356814824437e-05*G0_4 - 0.0001784059401247*G0_5 + 9.04994032887608e-05*G0_6 - 0.000144029549411502*G0_7 + 0.000886258481744639*G0_8 - 0.000858657000150101*G0_9 + 0.000659424487549522*G0_10 - 0.000299350614020769*G0_11 + 0.000443129240872319*G0_12 - 0.000456929981669588*G0_13 + 0.0003573137253693*G0_14 - 0.000144029549411501*G0_15 + 5.50356814824433e-05*G0_16 - 0.000178405940124699*G0_17 + 9.0499403288759e-05*G0_18 - 0.000144029549411501*G0_19 + 2.75178407412209e-05*G0_20 - 2.24993895422134e-05*G0_21 - 2.2499389542216e-05*G0_22 + 2.75178407412222e-05*G0_23 - 0.000144029549411502*G0_24 + 0.000357313725369301*G0_25 - 0.00045692998166959*G0_26 + 0.000443129240872321*G0_27 + 0.000110405926378155*G0_28 - 5.52029631890789e-05*G0_29 - 4.01476095920541e-05*G0_30 + 0.00030863474873893*G0_31 - 9.28413471816294e-05*G0_32 - 0.000341254681532476*G0_33 - 0.000266814322080539*G0_34 + 0.000135498182373188*G0_35 - 0.000266814322080538*G0_36 - 8.78228959826106e-06*G0_37 - 8.78228959826251e-06*G0_38 - 0.000555375266023442*G0_39 - 4.01476095920552e-05*G0_40 - 5.52029631890796e-05*G0_41 + 0.000110405926378159*G0_42 - 9.28413471816292e-05*G0_43 + 0.000308634748738928*G0_44 - 0.000341254681532477*G0_45 - 2.00738047960273e-05*G0_46 - 2.76014815945401e-05*G0_47 + 5.52029631890795e-05*G0_48 + 2.88560943942908e-05*G0_49 - 2.76014815945396e-05*G0_50 - 2.00738047960262e-05*G0_51 - 0.000301107071940419*G0_52 + 0.000727675423856017*G0_53 - 0.000301107071940419*G0_54 - 0.000777859935846087*G0_55;
    A[9] = 1.52962392545743e-05*G0_0 + 6.04020786312489e-05*G0_1 + 1.52962392545738e-05*G0_2 - 4.43831824040178e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091187*G0_5 - 0.000261963152588165*G0_6 + 0.000357313725369299*G0_7 - 0.000858657000150101*G0_8 + 0.00168419222238675*G0_9 - 0.00130178624102242*G0_10 + 0.000659424487549522*G0_11 - 0.000456929981669589*G0_12 + 0.000328541271828326*G0_13 - 0.000261963152588166*G0_14 + 9.04994032887592e-05*G0_15 - 0.0001784059401247*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588164*G0_18 + 0.000357313725369299*G0_19 - 2.24993895422153e-05*G0_20 + 1.12078743444492e-05*G0_21 + 1.12078743444504e-05*G0_22 - 2.24993895422155e-05*G0_23 + 9.04994032887588e-05*G0_24 - 0.000261963152588166*G0_25 + 0.000328541271828327*G0_26 - 0.000456929981669591*G0_27 - 0.000110405926378147*G0_28 - 0.000120442828776168*G0_29 + 0.000127970505574684*G0_30 - 4.51660607910648e-05*G0_31 - 7.02583167860995e-05*G0_32 + 0.000491808217502692*G0_33 + 0.000170627340766244*G0_34 - 0.000122952054375672*G0_35 + 0.000170627340766243*G0_36 - 0.000148044310370709*G0_37 - 0.000148044310370708*G0_38 + 0.000873210508627223*G0_39 + 0.000127970505574685*G0_40 - 0.00012044282877617*G0_41 - 0.000110405926378149*G0_42 - 7.02583167861001e-05*G0_43 - 4.51660607910608e-05*G0_44 + 0.000491808217502692*G0_45 - 0.000109569517844981*G0_46 + 0.000253431785549855*G0_47 - 0.000623960765743201*G0_48 - 2.25830303955338e-05*G0_49 + 0.000253431785549853*G0_50 - 0.000109569517844981*G0_51 + 6.27306399875933e-05*G0_52 - 0.000903321215821253*G0_53 + 6.27306399875969e-05*G0_54 + 0.00110405926378155*G0_55;
    A[10] = 1.52962392545737e-05*G0_0 - 4.43831824040179e-05*G0_1 + 1.52962392545738e-05*G0_2 + 6.04020786312488e-05*G0_3 + 9.0499403288759e-05*G0_4 - 0.000261963152588166*G0_5 + 0.000328541271828326*G0_6 - 0.00045692998166959*G0_7 + 0.000659424487549522*G0_8 - 0.00130178624102242*G0_9 + 0.00168419222238676*G0_10 - 0.000858657000150103*G0_11 + 0.000357313725369299*G0_12 - 0.000261963152588166*G0_13 + 0.000347778668091187*G0_14 - 0.0001784059401247*G0_15 + 9.04994032887591e-05*G0_16 - 0.000261963152588166*G0_17 + 0.000328541271828326*G0_18 - 0.00045692998166959*G0_19 - 2.24993895422139e-05*G0_20 + 1.12078743444481e-05*G0_21 + 1.12078743444499e-05*G0_22 - 2.24993895422156e-05*G0_23 - 0.0001784059401247*G0_24 + 0.000347778668091187*G0_25 - 0.000261963152588166*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502687*G0_28 - 7.02583167861e-05*G0_29 + 0.000127970505574681*G0_30 - 4.51660607910627e-05*G0_31 - 0.00012044282877617*G0_32 - 0.000110405926378152*G0_33 - 0.000109569517844984*G0_34 - 2.25830303955319e-05*G0_35 - 0.000109569517844986*G0_36 + 0.000253431785549852*G0_37 + 0.000253431785549854*G0_38 - 0.000623960765743204*G0_39 + 0.000127970505574682*G0_40 - 7.02583167860992e-05*G0_41 + 0.000491808217502689*G0_42 - 0.000120442828776171*G0_43 - 4.51660607910636e-05*G0_44 - 0.000110405926378153*G0_45 + 0.00017062734076624*G0_46 - 0.000148044310370708*G0_47 + 0.000873210508627222*G0_48 - 0.000122952054375673*G0_49 - 0.000148044310370708*G0_50 + 0.00017062734076624*G0_51 + 6.27306399875902e-05*G0_52 + 0.00110405926378155*G0_53 + 6.27306399875853e-05*G0_54 - 0.000903321215821261*G0_55;
    A[11] = 4.07498237359346e-06*G0_0 + 1.81868671452014e-05*G0_1 + 4.07498237359351e-06*G0_2 - 6.04020786312489e-05*G0_3 - 0.000144029549411501*G0_4 + 0.0003573137253693*G0_5 - 0.000456929981669589*G0_6 + 0.000443129240872321*G0_7 - 0.000299350614020769*G0_8 + 0.000659424487549522*G0_9 - 0.000858657000150103*G0_10 + 0.000886258481744641*G0_11 - 0.000144029549411501*G0_12 + 9.04994032887595e-05*G0_13 - 0.000178405940124701*G0_14 + 5.50356814824442e-05*G0_15 - 0.0001440295494115*G0_16 + 0.000357313725369299*G0_17 - 0.000456929981669589*G0_18 + 0.00044312924087232*G0_19 + 2.75178407412216e-05*G0_20 - 2.24993895422144e-05*G0_21 - 2.2499389542215e-05*G0_22 + 2.7517840741222e-05*G0_23 + 5.50356814824443e-05*G0_24 - 0.000178405940124701*G0_25 + 9.04994032887605e-05*G0_26 - 0.000144029549411503*G0_27 - 0.000341254681532478*G0_28 - 9.28413471816287e-05*G0_29 - 4.01476095920577e-05*G0_30 + 0.000308634748738933*G0_31 - 5.52029631890754e-05*G0_32 + 0.000110405926378151*G0_33 - 2.00738047960284e-05*G0_34 + 2.88560943942902e-05*G0_35 - 2.00738047960287e-05*G0_36 - 2.76014815945371e-05*G0_37 - 2.76014815945382e-05*G0_38 + 5.52029631890752e-05*G0_39 - 4.0147609592057e-05*G0_40 - 9.284134718163e-05*G0_41 - 0.000341254681532478*G0_42 - 5.52029631890752e-05*G0_43 + 0.000308634748738933*G0_44 + 0.000110405926378152*G0_45 - 0.00026681432208054*G0_46 - 8.78228959826156e-06*G0_47 - 0.000555375266023445*G0_48 + 0.00013549818237319*G0_49 - 8.78228959826212e-06*G0_50 - 0.000266814322080541*G0_51 - 0.000301107071940421*G0_52 - 0.000777859935846092*G0_53 - 0.000301107071940421*G0_54 + 0.000727675423856018*G0_55;
    A[12] = 4.07498237359384e-06*G0_0 - 6.04020786312484e-05*G0_1 + 1.81868671452018e-05*G0_2 + 4.07498237359388e-06*G0_3 - 0.000144029549411502*G0_4 + 9.04994032887608e-05*G0_5 - 0.000178405940124699*G0_6 + 5.50356814824439e-05*G0_7 + 0.000443129240872319*G0_8 - 0.000456929981669588*G0_9 + 0.000357313725369299*G0_10 - 0.000144029549411501*G0_11 + 0.000886258481744637*G0_12 - 0.000858657000150098*G0_13 + 0.000659424487549522*G0_14 - 0.000299350614020769*G0_15 + 2.75178407412217e-05*G0_16 - 2.24993895422142e-05*G0_17 - 2.24993895422154e-05*G0_18 + 2.75178407412222e-05*G0_19 + 5.50356814824428e-05*G0_20 - 0.000178405940124698*G0_21 + 9.04994032887586e-05*G0_22 - 0.000144029549411501*G0_23 - 0.000144029549411502*G0_24 + 0.000357313725369301*G0_25 - 0.00045692998166959*G0_26 + 0.00044312924087232*G0_27 + 0.000110405926378155*G0_28 + 0.000308634748738929*G0_29 - 0.000341254681532475*G0_30 - 5.52029631890771e-05*G0_31 - 9.28413471816296e-05*G0_32 - 4.0147609592055e-05*G0_33 - 0.000266814322080538*G0_34 - 8.78228959826363e-06*G0_35 - 0.000555375266023442*G0_36 + 0.000135498182373189*G0_37 - 8.78228959826333e-06*G0_38 - 0.000266814322080538*G0_39 - 2.00738047960266e-05*G0_40 - 2.76014815945402e-05*G0_41 + 5.52029631890796e-05*G0_42 + 2.88560943942906e-05*G0_43 - 2.76014815945404e-05*G0_44 - 2.00738047960268e-05*G0_45 - 4.01476095920554e-05*G0_46 - 5.52029631890795e-05*G0_47 + 0.000110405926378156*G0_48 - 9.28413471816289e-05*G0_49 + 0.00030863474873893*G0_50 - 0.000341254681532475*G0_51 - 0.000301107071940421*G0_52 + 0.000727675423856017*G0_53 - 0.000777859935846088*G0_54 - 0.000301107071940417*G0_55;
    A[13] = 1.52962392545734e-05*G0_0 + 6.04020786312484e-05*G0_1 - 4.43831824040184e-05*G0_2 + 1.52962392545735e-05*G0_3 + 0.0003573137253693*G0_4 - 0.000261963152588167*G0_5 + 0.000347778668091185*G0_6 - 0.000178405940124699*G0_7 - 0.000456929981669588*G0_8 + 0.000328541271828326*G0_9 - 0.000261963152588166*G0_10 + 9.04994032887596e-05*G0_11 - 0.000858657000150098*G0_12 + 0.00168419222238675*G0_13 - 0.00130178624102242*G0_14 + 0.000659424487549521*G0_15 - 2.24993895422152e-05*G0_16 + 1.12078743444491e-05*G0_17 + 1.1207874344449e-05*G0_18 - 2.2499389542215e-05*G0_19 - 0.000178405940124699*G0_20 + 0.000347778668091184*G0_21 - 0.000261963152588164*G0_22 + 0.000357313725369299*G0_23 + 9.04994032887592e-05*G0_24 - 0.000261963152588166*G0_25 + 0.000328541271828326*G0_26 - 0.000456929981669589*G0_27 - 0.000110405926378152*G0_28 - 4.51660607910601e-05*G0_29 + 0.000491808217502685*G0_30 - 0.000120442828776169*G0_31 - 7.02583167860996e-05*G0_32 + 0.00012797050557468*G0_33 + 0.000170627340766237*G0_34 - 0.000148044310370707*G0_35 + 0.000873210508627219*G0_36 - 0.000122952054375671*G0_37 - 0.000148044310370706*G0_38 + 0.000170627340766238*G0_39 - 0.000109569517844986*G0_40 + 0.000253431785549855*G0_41 - 0.000623960765743204*G0_42 - 2.25830303955316e-05*G0_43 + 0.000253431785549855*G0_44 - 0.000109569517844986*G0_45 + 0.000127970505574679*G0_46 - 0.000120442828776167*G0_47 - 0.000110405926378155*G0_48 - 7.02583167861005e-05*G0_49 - 4.51660607910608e-05*G0_50 + 0.000491808217502686*G0_51 + 6.27306399875873e-05*G0_52 - 0.000903321215821258*G0_53 + 0.00110405926378154*G0_54 + 6.27306399875867e-05*G0_55;
    A[14] = 1.52962392545738e-05*G0_0 - 4.43831824040178e-05*G0_1 + 6.04020786312493e-05*G0_2 + 1.52962392545735e-05*G0_3 - 0.000456929981669589*G0_4 + 0.000328541271828326*G0_5 - 0.000261963152588165*G0_6 + 9.04994032887594e-05*G0_7 + 0.0003573137253693*G0_8 - 0.000261963152588166*G0_9 + 0.000347778668091187*G0_10 - 0.000178405940124701*G0_11 + 0.000659424487549522*G0_12 - 0.00130178624102242*G0_13 + 0.00168419222238676*G0_14 - 0.000858657000150101*G0_15 - 2.24993895422155e-05*G0_16 + 1.1207874344449e-05*G0_17 + 1.12078743444501e-05*G0_18 - 2.2499389542215e-05*G0_19 + 9.04994032887604e-05*G0_20 - 0.000261963152588166*G0_21 + 0.000328541271828326*G0_22 - 0.00045692998166959*G0_23 - 0.0001784059401247*G0_24 + 0.000347778668091187*G0_25 - 0.000261963152588167*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502689*G0_28 - 4.5166060791066e-05*G0_29 - 0.00011040592637815*G0_30 - 7.02583167860993e-05*G0_31 - 0.000120442828776169*G0_32 + 0.00012797050557468*G0_33 - 0.000109569517844985*G0_34 + 0.000253431785549854*G0_35 - 0.000623960765743204*G0_36 - 2.25830303955332e-05*G0_37 + 0.000253431785549854*G0_38 - 0.000109569517844986*G0_39 + 0.00017062734076624*G0_40 - 0.000148044310370707*G0_41 + 0.000873210508627224*G0_42 - 0.000122952054375672*G0_43 - 0.000148044310370709*G0_44 + 0.000170627340766239*G0_45 + 0.00012797050557468*G0_46 - 7.02583167860972e-05*G0_47 + 0.00049180821750269*G0_48 - 0.000120442828776168*G0_49 - 4.51660607910667e-05*G0_50 - 0.000110405926378151*G0_51 + 6.27306399875913e-05*G0_52 + 0.00110405926378155*G0_53 - 0.000903321215821266*G0_54 + 6.27306399875886e-05*G0_55;
    A[15] = 4.07498237359318e-06*G0_0 + 1.81868671452014e-05*G0_1 - 6.04020786312494e-05*G0_2 + 4.07498237359384e-06*G0_3 + 0.00044312924087232*G0_4 - 0.00045692998166959*G0_5 + 0.000357313725369299*G0_6 - 0.000144029549411501*G0_7 - 0.000144029549411501*G0_8 + 9.04994032887592e-05*G0_9 - 0.0001784059401247*G0_10 + 5.50356814824442e-05*G0_11 - 0.000299350614020769*G0_12 + 0.000659424487549521*G0_13 - 0.000858657000150101*G0_14 + 0.000886258481744639*G0_15 + 2.75178407412224e-05*G0_16 - 2.24993895422155e-05*G0_17 - 2.24993895422152e-05*G0_18 + 2.75178407412215e-05*G0_19 - 0.000144029549411501*G0_20 + 0.000357313725369299*G0_21 - 0.000456929981669589*G0_22 + 0.00044312924087232*G0_23 + 5.50356814824442e-05*G0_24 - 0.0001784059401247*G0_25 + 9.04994032887605e-05*G0_26 - 0.000144029549411502*G0_27 - 0.000341254681532478*G0_28 + 0.000308634748738932*G0_29 + 0.000110405926378152*G0_30 - 9.28413471816286e-05*G0_31 - 5.52029631890774e-05*G0_32 - 4.01476095920573e-05*G0_33 - 2.00738047960284e-05*G0_34 - 2.76014815945372e-05*G0_35 + 5.52029631890758e-05*G0_36 + 2.88560943942913e-05*G0_37 - 2.76014815945391e-05*G0_38 - 2.00738047960289e-05*G0_39 - 0.000266814322080541*G0_40 - 8.78228959826251e-06*G0_41 - 0.000555375266023444*G0_42 + 0.000135498182373189*G0_43 - 8.78228959826086e-06*G0_44 - 0.00026681432208054*G0_45 - 4.0147609592057e-05*G0_46 - 9.28413471816308e-05*G0_47 - 0.000341254681532478*G0_48 - 5.52029631890751e-05*G0_49 + 0.000308634748738932*G0_50 + 0.000110405926378151*G0_51 - 0.000301107071940421*G0_52 - 0.000777859935846091*G0_53 + 0.00072767542385602*G0_54 - 0.000301107071940422*G0_55;
    A[16] = -6.04020786312489e-05*G0_0 + 4.07498237359336e-06*G0_1 + 4.07498237359372e-06*G0_2 + 1.81868671452012e-05*G0_3 + 5.50356814824431e-05*G0_4 - 0.000178405940124699*G0_5 + 9.04994032887589e-05*G0_6 - 0.000144029549411501*G0_7 + 5.50356814824433e-05*G0_8 - 0.0001784059401247*G0_9 + 9.04994032887591e-05*G0_10 - 0.0001440295494115*G0_11 + 2.75178407412217e-05*G0_12 - 2.24993895422152e-05*G0_13 - 2.24993895422155e-05*G0_14 + 2.75178407412224e-05*G0_15 + 0.000886258481744641*G0_16 - 0.000858657000150101*G0_17 + 0.000659424487549521*G0_18 - 0.000299350614020769*G0_19 + 0.00044312924087232*G0_20 - 0.000456929981669589*G0_21 + 0.000357313725369299*G0_22 - 0.000144029549411501*G0_23 + 0.000443129240872321*G0_24 - 0.00045692998166959*G0_25 + 0.0003573137253693*G0_26 - 0.000144029549411502*G0_27 - 0.000266814322080544*G0_28 + 0.00013549818237319*G0_29 - 0.000266814322080543*G0_30 - 8.78228959826203e-06*G0_31 - 8.78228959826053e-06*G0_32 - 0.000555375266023445*G0_33 + 0.000110405926378154*G0_34 - 5.52029631890786e-05*G0_35 - 4.01476095920571e-05*G0_36 + 0.000308634748738932*G0_37 - 9.28413471816301e-05*G0_38 - 0.000341254681532478*G0_39 + 0.000110405926378153*G0_40 - 5.52029631890754e-05*G0_41 - 4.01476095920599e-05*G0_42 + 0.000308634748738931*G0_43 - 9.28413471816307e-05*G0_44 - 0.000341254681532478*G0_45 + 5.5202963189075e-05*G0_46 - 2.76014815945387e-05*G0_47 - 2.00738047960319e-05*G0_48 - 2.76014815945379e-05*G0_49 + 2.88560943942924e-05*G0_50 - 2.00738047960316e-05*G0_51 + 0.000727675423856014*G0_52 - 0.000301107071940427*G0_53 - 0.000301107071940427*G0_54 - 0.000777859935846089*G0_55;
    A[17] = 6.04020786312484e-05*G0_0 + 1.52962392545733e-05*G0_1 + 1.52962392545731e-05*G0_2 - 4.43831824040186e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091186*G0_5 - 0.000261963152588166*G0_6 + 0.000357313725369299*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091186*G0_9 - 0.000261963152588166*G0_10 + 0.000357313725369299*G0_11 - 2.24993895422142e-05*G0_12 + 1.12078743444491e-05*G0_13 + 1.1207874344449e-05*G0_14 - 2.24993895422155e-05*G0_15 - 0.000858657000150101*G0_16 + 0.00168419222238675*G0_17 - 0.00130178624102242*G0_18 + 0.000659424487549523*G0_19 - 0.000456929981669588*G0_20 + 0.000328541271828326*G0_21 - 0.000261963152588166*G0_22 + 9.04994032887598e-05*G0_23 - 0.00045692998166959*G0_24 + 0.000328541271828326*G0_25 - 0.000261963152588167*G0_26 + 9.04994032887604e-05*G0_27 + 0.000170627340766238*G0_28 - 0.000122952054375672*G0_29 + 0.00017062734076624*G0_30 - 0.000148044310370704*G0_31 - 0.000148044310370707*G0_32 + 0.000873210508627218*G0_33 - 0.000110405926378155*G0_34 - 0.000120442828776167*G0_35 + 0.000127970505574677*G0_36 - 4.51660607910634e-05*G0_37 - 7.02583167860971e-05*G0_38 + 0.000491808217502684*G0_39 - 0.000110405926378156*G0_40 - 0.000120442828776169*G0_41 + 0.000127970505574679*G0_42 - 4.51660607910615e-05*G0_43 - 7.02583167860935e-05*G0_44 + 0.000491808217502684*G0_45 - 0.000623960765743206*G0_46 + 0.000253431785549856*G0_47 - 0.000109569517844987*G0_48 + 0.000253431785549854*G0_49 - 2.25830303955331e-05*G0_50 - 0.000109569517844986*G0_51 - 0.000903321215821265*G0_52 + 6.27306399875908e-05*G0_53 + 6.27306399875907e-05*G0_54 + 0.00110405926378154*G0_55;
    A[18] = -4.43831824040176e-05*G0_0 + 1.52962392545737e-05*G0_1 + 1.52962392545742e-05*G0_2 + 6.04020786312493e-05*G0_3 + 9.04994032887583e-05*G0_4 - 0.000261963152588165*G0_5 + 0.000328541271828325*G0_6 - 0.000456929981669589*G0_7 + 9.04994032887589e-05*G0_8 - 0.000261963152588164*G0_9 + 0.000328541271828326*G0_10 - 0.000456929981669589*G0_11 - 2.24993895422154e-05*G0_12 + 1.1207874344449e-05*G0_13 + 1.12078743444501e-05*G0_14 - 2.24993895422152e-05*G0_15 + 0.000659424487549521*G0_16 - 0.00130178624102242*G0_17 + 0.00168419222238675*G0_18 - 0.000858657000150102*G0_19 + 0.0003573137253693*G0_20 - 0.000261963152588167*G0_21 + 0.000347778668091187*G0_22 - 0.0001784059401247*G0_23 + 0.0003573137253693*G0_24 - 0.000261963152588165*G0_25 + 0.000347778668091187*G0_26 - 0.0001784059401247*G0_27 - 0.000109569517844984*G0_28 - 2.25830303955328e-05*G0_29 - 0.000109569517844983*G0_30 + 0.000253431785549852*G0_31 + 0.000253431785549852*G0_32 - 0.0006239607657432*G0_33 + 0.00049180821750269*G0_34 - 7.02583167860999e-05*G0_35 + 0.000127970505574683*G0_36 - 4.51660607910641e-05*G0_37 - 0.00012044282877617*G0_38 - 0.000110405926378148*G0_39 + 0.00049180821750269*G0_40 - 7.02583167860969e-05*G0_41 + 0.000127970505574682*G0_42 - 4.51660607910661e-05*G0_43 - 0.000120442828776171*G0_44 - 0.000110405926378147*G0_45 + 0.000873210508627224*G0_46 - 0.000148044310370707*G0_47 + 0.000170627340766241*G0_48 - 0.000148044310370708*G0_49 - 0.000122952054375672*G0_50 + 0.000170627340766242*G0_51 + 0.00110405926378155*G0_52 + 6.2730639987592e-05*G0_53 + 6.27306399875871e-05*G0_54 - 0.000903321215821257*G0_55;
    A[19] = 1.81868671452013e-05*G0_0 + 4.07498237359372e-06*G0_1 + 4.07498237359345e-06*G0_2 - 6.04020786312489e-05*G0_3 - 0.000144029549411501*G0_4 + 0.0003573137253693*G0_5 - 0.000456929981669589*G0_6 + 0.000443129240872321*G0_7 - 0.000144029549411501*G0_8 + 0.000357313725369299*G0_9 - 0.00045692998166959*G0_10 + 0.00044312924087232*G0_11 + 2.75178407412222e-05*G0_12 - 2.2499389542215e-05*G0_13 - 2.2499389542215e-05*G0_14 + 2.75178407412215e-05*G0_15 - 0.000299350614020769*G0_16 + 0.000659424487549523*G0_17 - 0.000858657000150102*G0_18 + 0.000886258481744641*G0_19 - 0.000144029549411502*G0_20 + 9.04994032887606e-05*G0_21 - 0.000178405940124701*G0_22 + 5.5035681482444e-05*G0_23 - 0.000144029549411502*G0_24 + 9.049940328876e-05*G0_25 - 0.0001784059401247*G0_26 + 5.50356814824438e-05*G0_27 - 2.00738047960272e-05*G0_28 + 2.88560943942905e-05*G0_29 - 2.00738047960283e-05*G0_30 - 2.76014815945376e-05*G0_31 - 2.7601481594538e-05*G0_32 + 5.52029631890764e-05*G0_33 - 0.000341254681532479*G0_34 - 9.28413471816293e-05*G0_35 - 4.01476095920575e-05*G0_36 + 0.000308634748738933*G0_37 - 5.52029631890773e-05*G0_38 + 0.000110405926378153*G0_39 - 0.000341254681532479*G0_40 - 9.28413471816305e-05*G0_41 - 4.01476095920553e-05*G0_42 + 0.000308634748738934*G0_43 - 5.52029631890757e-05*G0_44 + 0.000110405926378153*G0_45 - 0.000555375266023446*G0_46 - 8.78228959826223e-06*G0_47 - 0.00026681432208054*G0_48 - 8.78228959826143e-06*G0_49 + 0.000135498182373189*G0_50 - 0.000266814322080541*G0_51 - 0.000777859935846091*G0_52 - 0.000301107071940421*G0_53 - 0.000301107071940421*G0_54 + 0.000727675423856022*G0_55;
    A[20] = -6.04020786312478e-05*G0_0 + 4.07498237359406e-06*G0_1 + 1.81868671452025e-05*G0_2 + 4.07498237359436e-06*G0_3 - 0.000144029549411503*G0_4 + 9.04994032887606e-05*G0_5 - 0.000178405940124698*G0_6 + 5.50356814824429e-05*G0_7 + 2.75178407412209e-05*G0_8 - 2.24993895422153e-05*G0_9 - 2.24993895422139e-05*G0_10 + 2.75178407412216e-05*G0_11 + 5.50356814824429e-05*G0_12 - 0.000178405940124699*G0_13 + 9.04994032887604e-05*G0_14 - 0.000144029549411501*G0_15 + 0.000443129240872319*G0_16 - 0.000456929981669588*G0_17 + 0.0003573137253693*G0_18 - 0.000144029549411502*G0_19 + 0.000886258481744636*G0_20 - 0.000858657000150097*G0_21 + 0.000659424487549521*G0_22 - 0.000299350614020769*G0_23 + 0.000443129240872319*G0_24 - 0.000456929981669588*G0_25 + 0.000357313725369299*G0_26 - 0.000144029549411502*G0_27 - 0.000266814322080535*G0_28 - 8.78228959826491e-06*G0_29 - 0.000555375266023436*G0_30 + 0.000135498182373189*G0_31 - 8.78228959826363e-06*G0_32 - 0.000266814322080535*G0_33 + 0.00011040592637816*G0_34 + 0.000308634748738928*G0_35 - 0.00034125468153247*G0_36 - 5.52029631890779e-05*G0_37 - 9.28413471816323e-05*G0_38 - 4.01476095920509e-05*G0_39 + 5.52029631890818e-05*G0_40 - 2.7601481594539e-05*G0_41 - 2.00738047960239e-05*G0_42 - 2.76014815945403e-05*G0_43 + 2.88560943942889e-05*G0_44 - 2.00738047960233e-05*G0_45 + 0.000110405926378159*G0_46 - 5.52029631890795e-05*G0_47 - 4.01476095920518e-05*G0_48 + 0.000308634748738929*G0_49 - 9.28413471816301e-05*G0_50 - 0.000341254681532471*G0_51 + 0.000727675423856021*G0_52 - 0.000301107071940415*G0_53 - 0.000777859935846082*G0_54 - 0.000301107071940413*G0_55;
    A[21] = 6.04020786312477e-05*G0_0 + 1.52962392545729e-05*G0_1 - 4.43831824040197e-05*G0_2 + 1.5296239254573e-05*G0_3 + 0.000357313725369301*G0_4 - 0.000261963152588168*G0_5 + 0.000347778668091185*G0_6 - 0.000178405940124698*G0_7 - 2.24993895422134e-05*G0_8 + 1.12078743444492e-05*G0_9 + 1.12078743444481e-05*G0_10 - 2.24993895422144e-05*G0_11 - 0.000178405940124698*G0_12 + 0.000347778668091184*G0_13 - 0.000261963152588166*G0_14 + 0.000357313725369299*G0_15 - 0.000456929981669589*G0_16 + 0.000328541271828326*G0_17 - 0.000261963152588167*G0_18 + 9.04994032887606e-05*G0_19 - 0.000858657000150097*G0_20 + 0.00168419222238675*G0_21 - 0.00130178624102242*G0_22 + 0.000659424487549522*G0_23 - 0.000456929981669589*G0_24 + 0.000328541271828325*G0_25 - 0.000261963152588167*G0_26 + 9.04994032887607e-05*G0_27 + 0.000170627340766234*G0_28 - 0.000148044310370704*G0_29 + 0.000873210508627214*G0_30 - 0.00012295205437567*G0_31 - 0.000148044310370705*G0_32 + 0.000170627340766234*G0_33 - 0.000110405926378158*G0_34 - 4.51660607910588e-05*G0_35 + 0.000491808217502679*G0_36 - 0.000120442828776168*G0_37 - 7.02583167860956e-05*G0_38 + 0.000127970505574674*G0_39 - 0.00062396076574321*G0_40 + 0.000253431785549854*G0_41 - 0.000109569517844989*G0_42 + 0.000253431785549855*G0_43 - 2.25830303955286e-05*G0_44 - 0.000109569517844991*G0_45 - 0.000110405926378161*G0_46 - 0.000120442828776166*G0_47 + 0.000127970505574675*G0_48 - 4.51660607910593e-05*G0_49 - 7.02583167860996e-05*G0_50 + 0.00049180821750268*G0_51 - 0.000903321215821267*G0_52 + 6.27306399875843e-05*G0_53 + 0.00110405926378154*G0_54 + 6.2730639987581e-05*G0_55;
    A[22] = -4.43831824040171e-05*G0_0 + 1.52962392545739e-05*G0_1 + 6.04020786312503e-05*G0_2 + 1.52962392545737e-05*G0_3 - 0.00045692998166959*G0_4 + 0.000328541271828327*G0_5 - 0.000261963152588165*G0_6 + 9.04994032887587e-05*G0_7 - 2.2499389542216e-05*G0_8 + 1.12078743444504e-05*G0_9 + 1.12078743444499e-05*G0_10 - 2.2499389542215e-05*G0_11 + 9.04994032887587e-05*G0_12 - 0.000261963152588164*G0_13 + 0.000328541271828326*G0_14 - 0.000456929981669589*G0_15 + 0.000357313725369299*G0_16 - 0.000261963152588166*G0_17 + 0.000347778668091187*G0_18 - 0.000178405940124701*G0_19 + 0.000659424487549521*G0_20 - 0.00130178624102242*G0_21 + 0.00168419222238675*G0_22 - 0.000858657000150101*G0_23 + 0.000357313725369298*G0_24 - 0.000261963152588165*G0_25 + 0.000347778668091186*G0_26 - 0.000178405940124701*G0_27 - 0.000109569517844981*G0_28 + 0.000253431785549852*G0_29 - 0.000623960765743198*G0_30 - 2.25830303955321e-05*G0_31 + 0.000253431785549853*G0_32 - 0.000109569517844981*G0_33 + 0.000491808217502693*G0_34 - 4.51660607910651e-05*G0_35 - 0.000110405926378146*G0_36 - 7.02583167861008e-05*G0_37 - 0.00012044282877617*G0_38 + 0.000127970505574685*G0_39 + 0.000873210508627227*G0_40 - 0.000148044310370707*G0_41 + 0.000170627340766242*G0_42 - 0.000148044310370709*G0_43 - 0.000122952054375673*G0_44 + 0.000170627340766243*G0_45 + 0.000491808217502693*G0_46 - 7.02583167860998e-05*G0_47 + 0.000127970505574683*G0_48 - 4.51660607910671e-05*G0_49 - 0.000120442828776168*G0_50 - 0.000110405926378147*G0_51 + 0.00110405926378155*G0_52 + 6.27306399875927e-05*G0_53 - 0.000903321215821257*G0_54 + 6.27306399875954e-05*G0_55;
    A[23] = 1.8186867145201e-05*G0_0 + 4.0749823735936e-06*G0_1 - 6.04020786312495e-05*G0_2 + 4.07498237359382e-06*G0_3 + 0.000443129240872321*G0_4 - 0.00045692998166959*G0_5 + 0.0003573137253693*G0_6 - 0.000144029549411501*G0_7 + 2.75178407412222e-05*G0_8 - 2.24993895422155e-05*G0_9 - 2.24993895422156e-05*G0_10 + 2.7517840741222e-05*G0_11 - 0.000144029549411501*G0_12 + 0.000357313725369299*G0_13 - 0.00045692998166959*G0_14 + 0.00044312924087232*G0_15 - 0.000144029549411501*G0_16 + 9.04994032887598e-05*G0_17 - 0.0001784059401247*G0_18 + 5.5035681482444e-05*G0_19 - 0.000299350614020769*G0_20 + 0.000659424487549522*G0_21 - 0.000858657000150101*G0_22 + 0.000886258481744639*G0_23 - 0.000144029549411501*G0_24 + 9.04994032887595e-05*G0_25 - 0.0001784059401247*G0_26 + 5.50356814824437e-05*G0_27 - 2.007380479603e-05*G0_28 - 2.76014815945373e-05*G0_29 + 5.52029631890751e-05*G0_30 + 2.88560943942904e-05*G0_31 - 2.76014815945388e-05*G0_32 - 2.00738047960294e-05*G0_33 - 0.000341254681532478*G0_34 + 0.000308634748738932*G0_35 + 0.000110405926378152*G0_36 - 9.2841347181628e-05*G0_37 - 5.52029631890777e-05*G0_38 - 4.01476095920575e-05*G0_39 - 0.000555375266023445*G0_40 - 8.78228959826315e-06*G0_41 - 0.00026681432208054*G0_42 - 8.78228959826143e-06*G0_43 + 0.00013549818237319*G0_44 - 0.000266814322080541*G0_45 - 0.000341254681532478*G0_46 - 9.28413471816303e-05*G0_47 - 4.01476095920575e-05*G0_48 + 0.000308634748738933*G0_49 - 5.52029631890769e-05*G0_50 + 0.000110405926378152*G0_51 - 0.00077785993584609*G0_52 - 0.000301107071940425*G0_53 + 0.000727675423856019*G0_54 - 0.000301107071940424*G0_55;
    A[24] = -6.04020786312489e-05*G0_0 + 1.81868671452013e-05*G0_1 + 4.07498237359387e-06*G0_2 + 4.0749823735937e-06*G0_3 + 2.75178407412213e-05*G0_4 - 2.24993895422148e-05*G0_5 - 2.24993895422153e-05*G0_6 + 2.75178407412221e-05*G0_7 - 0.000144029549411502*G0_8 + 9.04994032887588e-05*G0_9 - 0.0001784059401247*G0_10 + 5.50356814824444e-05*G0_11 - 0.000144029549411502*G0_12 + 9.04994032887592e-05*G0_13 - 0.0001784059401247*G0_14 + 5.50356814824442e-05*G0_15 + 0.000443129240872321*G0_16 - 0.00045692998166959*G0_17 + 0.0003573137253693*G0_18 - 0.000144029549411502*G0_19 + 0.000443129240872319*G0_20 - 0.000456929981669589*G0_21 + 0.000357313725369298*G0_22 - 0.000144029549411501*G0_23 + 0.000886258481744641*G0_24 - 0.000858657000150101*G0_25 + 0.000659424487549523*G0_26 - 0.000299350614020769*G0_27 - 0.000555375266023444*G0_28 - 8.78228959826186e-06*G0_29 - 0.000266814322080541*G0_30 - 8.78228959826234e-06*G0_31 + 0.00013549818237319*G0_32 - 0.00026681432208054*G0_33 + 5.52029631890766e-05*G0_34 - 2.76014815945394e-05*G0_35 - 2.00738047960277e-05*G0_36 - 2.76014815945368e-05*G0_37 + 2.88560943942893e-05*G0_38 - 2.0073804796028e-05*G0_39 + 0.000110405926378154*G0_40 + 0.000308634748738933*G0_41 - 0.000341254681532478*G0_42 - 5.52029631890778e-05*G0_43 - 9.2841347181631e-05*G0_44 - 4.01476095920557e-05*G0_45 + 0.000110405926378153*G0_46 + 0.000308634748738932*G0_47 - 0.000341254681532478*G0_48 - 5.52029631890776e-05*G0_49 - 9.28413471816283e-05*G0_50 - 4.01476095920573e-05*G0_51 + 0.000727675423856017*G0_52 - 0.00077785993584609*G0_53 - 0.000301107071940424*G0_54 - 0.000301107071940422*G0_55;
    A[25] = 6.04020786312488e-05*G0_0 - 4.43831824040181e-05*G0_1 + 1.52962392545733e-05*G0_2 + 1.52962392545734e-05*G0_3 - 2.24993895422144e-05*G0_4 + 1.12078743444487e-05*G0_5 + 1.12078743444496e-05*G0_6 - 2.24993895422149e-05*G0_7 + 0.000357313725369301*G0_8 - 0.000261963152588166*G0_9 + 0.000347778668091187*G0_10 - 0.0001784059401247*G0_11 + 0.000357313725369301*G0_12 - 0.000261963152588166*G0_13 + 0.000347778668091187*G0_14 - 0.0001784059401247*G0_15 - 0.00045692998166959*G0_16 + 0.000328541271828326*G0_17 - 0.000261963152588165*G0_18 + 9.049940328876e-05*G0_19 - 0.000456929981669588*G0_20 + 0.000328541271828325*G0_21 - 0.000261963152588165*G0_22 + 9.04994032887595e-05*G0_23 - 0.000858657000150101*G0_24 + 0.00168419222238676*G0_25 - 0.00130178624102242*G0_26 + 0.000659424487549525*G0_27 + 0.000873210508627219*G0_28 - 0.000148044310370707*G0_29 + 0.000170627340766238*G0_30 - 0.000148044310370706*G0_31 - 0.000122952054375672*G0_32 + 0.000170627340766237*G0_33 - 0.000623960765743205*G0_34 + 0.000253431785549855*G0_35 - 0.000109569517844989*G0_36 + 0.000253431785549852*G0_37 - 2.25830303955305e-05*G0_38 - 0.000109569517844988*G0_39 - 0.000110405926378153*G0_40 - 4.5166060791063e-05*G0_41 + 0.000491808217502687*G0_42 - 0.00012044282877617*G0_43 - 7.02583167860968e-05*G0_44 + 0.000127970505574678*G0_45 - 0.000110405926378154*G0_46 - 4.51660607910618e-05*G0_47 + 0.000491808217502687*G0_48 - 0.000120442828776169*G0_49 - 7.02583167860998e-05*G0_50 + 0.00012797050557468*G0_51 - 0.000903321215821261*G0_52 + 0.00110405926378155*G0_53 + 6.2730639987586e-05*G0_54 + 6.27306399875853e-05*G0_55;
    A[26] = -4.43831824040181e-05*G0_0 + 6.04020786312483e-05*G0_1 + 1.52962392545736e-05*G0_2 + 1.52962392545734e-05*G0_3 - 2.2499389542215e-05*G0_4 + 1.12078743444489e-05*G0_5 + 1.12078743444484e-05*G0_6 - 2.24993895422153e-05*G0_7 - 0.00045692998166959*G0_8 + 0.000328541271828327*G0_9 - 0.000261963152588166*G0_10 + 9.04994032887606e-05*G0_11 - 0.00045692998166959*G0_12 + 0.000328541271828326*G0_13 - 0.000261963152588167*G0_14 + 9.04994032887605e-05*G0_15 + 0.0003573137253693*G0_16 - 0.000261963152588167*G0_17 + 0.000347778668091187*G0_18 - 0.0001784059401247*G0_19 + 0.000357313725369299*G0_20 - 0.000261963152588167*G0_21 + 0.000347778668091186*G0_22 - 0.0001784059401247*G0_23 + 0.000659424487549523*G0_24 - 0.00130178624102242*G0_25 + 0.00168419222238676*G0_26 - 0.000858657000150103*G0_27 - 0.000623960765743205*G0_28 + 0.000253431785549855*G0_29 - 0.000109569517844987*G0_30 + 0.000253431785549855*G0_31 - 2.25830303955321e-05*G0_32 - 0.000109569517844986*G0_33 + 0.000873210508627221*G0_34 - 0.000148044310370707*G0_35 + 0.00017062734076624*G0_36 - 0.000148044310370706*G0_37 - 0.000122952054375672*G0_38 + 0.000170627340766239*G0_39 + 0.000491808217502688*G0_40 - 4.51660607910612e-05*G0_41 - 0.000110405926378155*G0_42 - 7.02583167860979e-05*G0_43 - 0.000120442828776169*G0_44 + 0.00012797050557468*G0_45 + 0.000491808217502688*G0_46 - 4.51660607910617e-05*G0_47 - 0.000110405926378156*G0_48 - 7.02583167860995e-05*G0_49 - 0.000120442828776166*G0_50 + 0.000127970505574679*G0_51 + 0.00110405926378154*G0_52 - 0.000903321215821261*G0_53 + 6.27306399875872e-05*G0_54 + 6.2730639987588e-05*G0_55;
    A[27] = 1.81868671452016e-05*G0_0 - 6.04020786312483e-05*G0_1 + 4.07498237359378e-06*G0_2 + 4.07498237359401e-06*G0_3 + 2.75178407412218e-05*G0_4 - 2.24993895422147e-05*G0_5 - 2.2499389542214e-05*G0_6 + 2.75178407412218e-05*G0_7 + 0.000443129240872321*G0_8 - 0.000456929981669591*G0_9 + 0.000357313725369301*G0_10 - 0.000144029549411503*G0_11 + 0.00044312924087232*G0_12 - 0.000456929981669589*G0_13 + 0.000357313725369301*G0_14 - 0.000144029549411502*G0_15 - 0.000144029549411502*G0_16 + 9.04994032887604e-05*G0_17 - 0.0001784059401247*G0_18 + 5.50356814824438e-05*G0_19 - 0.000144029549411502*G0_20 + 9.04994032887607e-05*G0_21 - 0.000178405940124701*G0_22 + 5.50356814824438e-05*G0_23 - 0.000299350614020769*G0_24 + 0.000659424487549525*G0_25 - 0.000858657000150103*G0_26 + 0.000886258481744641*G0_27 + 5.52029631890781e-05*G0_28 - 2.760148159454e-05*G0_29 - 2.00738047960265e-05*G0_30 - 2.76014815945393e-05*G0_31 + 2.88560943942907e-05*G0_32 - 2.0073804796027e-05*G0_33 - 0.000555375266023443*G0_34 - 8.78228959826283e-06*G0_35 - 0.00026681432208054*G0_36 - 8.78228959826206e-06*G0_37 + 0.000135498182373189*G0_38 - 0.000266814322080539*G0_39 - 0.000341254681532477*G0_40 + 0.00030863474873893*G0_41 + 0.000110405926378157*G0_42 - 9.28413471816304e-05*G0_43 - 5.52029631890792e-05*G0_44 - 4.01476095920555e-05*G0_45 - 0.000341254681532477*G0_46 + 0.000308634748738931*G0_47 + 0.000110405926378156*G0_48 - 9.28413471816293e-05*G0_49 - 5.52029631890785e-05*G0_50 - 4.01476095920548e-05*G0_51 - 0.000777859935846089*G0_52 + 0.000727675423856019*G0_53 - 0.00030110707194042*G0_54 - 0.000301107071940421*G0_55;
    A[28] = 0.000173437673437683*G0_0 + 0.00014965021475439*G0_1 + 9.59527869250137e-05*G0_2 + 9.59527869250132e-05*G0_3 - 4.01476095920554e-05*G0_4 + 0.000127970505574679*G0_5 + 0.000127970505574678*G0_6 - 4.01476095920561e-05*G0_7 + 0.000110405926378155*G0_8 - 0.000110405926378147*G0_9 + 0.000491808217502687*G0_10 - 0.000341254681532478*G0_11 + 0.000110405926378155*G0_12 - 0.000110405926378152*G0_13 + 0.000491808217502689*G0_14 - 0.000341254681532478*G0_15 - 0.000266814322080544*G0_16 + 0.000170627340766238*G0_17 - 0.000109569517844984*G0_18 - 2.00738047960272e-05*G0_19 - 0.000266814322080535*G0_20 + 0.000170627340766234*G0_21 - 0.000109569517844981*G0_22 - 2.007380479603e-05*G0_23 - 0.000555375266023444*G0_24 + 0.000873210508627219*G0_25 - 0.000623960765743205*G0_26 + 5.52029631890781e-05*G0_27 + 0.00455006242043303*G0_28 - 0.00180664243164252*G0_29 + 0.00103714658112811*G0_30 - 0.00180664243164253*G0_31 + 0.00051439124789822*G0_32 + 0.00103714658112811*G0_33 + 0.000618942314544201*G0_34 - 0.000213284175957794*G0_35 + 0.000694219082529302*G0_36 - 0.000213284175957805*G0_37 + 0.000577121887885812*G0_38 + 0.000694219082529303*G0_39 + 0.000694219082529305*G0_40 - 0.00125461279975176*G0_41 + 0.00227503121021651*G0_42 + 6.27306399875876e-05*G0_43 - 0.000903321215821259*G0_44 + 0.000518573290564058*G0_45 + 0.000694219082529305*G0_46 - 0.00125461279975175*G0_47 + 0.00227503121021652*G0_48 + 6.27306399875889e-05*G0_49 - 0.000903321215821266*G0_50 + 0.00051857329056406*G0_51 - 0.000351291583930488*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846086*G0_54 + 0.000777859935846087*G0_55;
    A[29] = -1.77151327324947e-05*G0_0 - 0.000121496703527959*G0_1 - 0.00012149670352796*G0_2 + 3.98465025201171e-05*G0_3 - 5.52029631890772e-05*G0_4 - 0.000120442828776168*G0_5 - 7.02583167860976e-05*G0_6 - 9.28413471816302e-05*G0_7 - 5.52029631890789e-05*G0_8 - 0.000120442828776168*G0_9 - 7.02583167861e-05*G0_10 - 9.28413471816287e-05*G0_11 + 0.00030863474873893*G0_12 - 4.51660607910601e-05*G0_13 - 4.51660607910661e-05*G0_14 + 0.000308634748738932*G0_15 + 0.00013549818237319*G0_16 - 0.000122952054375672*G0_17 - 2.25830303955328e-05*G0_18 + 2.88560943942905e-05*G0_19 - 8.78228959826487e-06*G0_20 - 0.000148044310370704*G0_21 + 0.000253431785549852*G0_22 - 2.76014815945373e-05*G0_23 - 8.78228959826188e-06*G0_24 - 0.000148044310370707*G0_25 + 0.000253431785549855*G0_26 - 2.760148159454e-05*G0_27 - 0.00180664243164252*G0_28 + 0.00526937375895737*G0_29 - 0.00180664243164252*G0_30 - 0.000940959599813814*G0_31 - 0.00094095959981382*G0_32 + 0.000514391247898223*G0_33 - 0.000213284175957796*G0_34 + 0.000225830303955314*G0_35 - 0.00125461279975175*G0_36 + 0.000414022223918077*G0_37 - 0.000639852527873401*G0_38 + 6.27306399875916e-05*G0_39 - 0.000213284175957796*G0_40 + 0.000225830303955316*G0_41 - 0.00125461279975175*G0_42 + 0.000414022223918078*G0_43 - 0.000639852527873396*G0_44 + 6.27306399875903e-05*G0_45 + 0.000577121887885808*G0_46 - 0.000639852527873396*G0_47 - 0.000903321215821263*G0_48 - 0.000639852527873396*G0_49 + 0.00263468687947868*G0_50 - 0.000903321215821261*G0_51 - 0.000639852527873396*G0_52 + 0.000677490911865943*G0_53 + 0.000677490911865949*G0_54 - 0.00120442828776168*G0_55;
    A[30] = 0.000173437673437683*G0_0 + 9.5952786925014e-05*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250137e-05*G0_3 + 0.000110405926378155*G0_4 - 0.000110405926378154*G0_5 + 0.000491808217502685*G0_6 - 0.000341254681532478*G0_7 - 4.01476095920541e-05*G0_8 + 0.000127970505574684*G0_9 + 0.000127970505574681*G0_10 - 4.01476095920577e-05*G0_11 - 0.000341254681532475*G0_12 + 0.000491808217502685*G0_13 - 0.00011040592637815*G0_14 + 0.000110405926378152*G0_15 - 0.000266814322080543*G0_16 + 0.000170627340766239*G0_17 - 0.000109569517844983*G0_18 - 2.00738047960283e-05*G0_19 - 0.000555375266023436*G0_20 + 0.000873210508627214*G0_21 - 0.000623960765743198*G0_22 + 5.52029631890751e-05*G0_23 - 0.000266814322080541*G0_24 + 0.000170627340766238*G0_25 - 0.000109569517844986*G0_26 - 2.00738047960265e-05*G0_27 + 0.00103714658112811*G0_28 - 0.00180664243164252*G0_29 + 0.00455006242043303*G0_30 + 0.00051439124789822*G0_31 - 0.00180664243164253*G0_32 + 0.00103714658112812*G0_33 + 0.000694219082529302*G0_34 - 0.00125461279975174*G0_35 + 0.00227503121021651*G0_36 + 6.27306399875827e-05*G0_37 - 0.000903321215821257*G0_38 + 0.000518573290564058*G0_39 + 0.000618942314544198*G0_40 - 0.000213284175957798*G0_41 + 0.000694219082529305*G0_42 - 0.000213284175957799*G0_43 + 0.00057712188788581*G0_44 + 0.000694219082529304*G0_45 + 0.000694219082529303*G0_46 + 6.27306399875923e-05*G0_47 + 0.000518573290564057*G0_48 - 0.00125461279975176*G0_49 - 0.000903321215821264*G0_50 + 0.00227503121021651*G0_51 - 0.000351291583930488*G0_52 + 0.000777859935846095*G0_53 + 0.00140516633572196*G0_54 + 0.000777859935846087*G0_55;
    A[31] = -1.77151327324945e-05*G0_0 - 0.00012149670352796*G0_1 + 3.98465025201167e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816306e-05*G0_4 - 7.02583167860963e-05*G0_5 - 0.000120442828776168*G0_6 - 5.52029631890768e-05*G0_7 + 0.00030863474873893*G0_8 - 4.51660607910647e-05*G0_9 - 4.51660607910628e-05*G0_10 + 0.000308634748738933*G0_11 - 5.52029631890771e-05*G0_12 - 0.000120442828776169*G0_13 - 7.02583167860993e-05*G0_14 - 9.28413471816286e-05*G0_15 - 8.78228959826196e-06*G0_16 - 0.000148044310370704*G0_17 + 0.000253431785549852*G0_18 - 2.76014815945376e-05*G0_19 + 0.000135498182373189*G0_20 - 0.00012295205437567*G0_21 - 2.25830303955321e-05*G0_22 + 2.88560943942904e-05*G0_23 - 8.78228959826236e-06*G0_24 - 0.000148044310370706*G0_25 + 0.000253431785549855*G0_26 - 2.76014815945393e-05*G0_27 - 0.00180664243164253*G0_28 - 0.000940959599813814*G0_29 + 0.00051439124789822*G0_30 + 0.00526937375895737*G0_31 - 0.000940959599813814*G0_32 - 0.00180664243164252*G0_33 - 0.000213284175957798*G0_34 + 0.000414022223918077*G0_35 + 6.27306399875858e-05*G0_36 + 0.000225830303955316*G0_37 - 0.000639852527873394*G0_38 - 0.00125461279975175*G0_39 + 0.000577121887885808*G0_40 - 0.000639852527873396*G0_41 - 0.000903321215821262*G0_42 - 0.000639852527873395*G0_43 + 0.00263468687947868*G0_44 - 0.000903321215821262*G0_45 - 0.000213284175957797*G0_46 + 0.000225830303955317*G0_47 - 0.00125461279975176*G0_48 + 0.000414022223918078*G0_49 - 0.000639852527873394*G0_50 + 6.27306399875873e-05*G0_51 - 0.000639852527873398*G0_52 + 0.000677490911865948*G0_53 - 0.00120442828776169*G0_54 + 0.000677490911865955*G0_55;
    A[32] = -1.77151327324955e-05*G0_0 + 3.98465025201156e-05*G0_1 - 0.00012149670352796*G0_2 - 0.000121496703527961*G0_3 + 0.000308634748738931*G0_4 - 4.51660607910637e-05*G0_5 - 4.5166060791065e-05*G0_6 + 0.000308634748738935*G0_7 - 9.28413471816295e-05*G0_8 - 7.02583167860995e-05*G0_9 - 0.00012044282877617*G0_10 - 5.52029631890754e-05*G0_11 - 9.28413471816296e-05*G0_12 - 7.02583167860996e-05*G0_13 - 0.000120442828776169*G0_14 - 5.52029631890774e-05*G0_15 - 8.78228959826054e-06*G0_16 - 0.000148044310370707*G0_17 + 0.000253431785549852*G0_18 - 2.7601481594538e-05*G0_19 - 8.78228959826362e-06*G0_20 - 0.000148044310370705*G0_21 + 0.000253431785549853*G0_22 - 2.76014815945388e-05*G0_23 + 0.00013549818237319*G0_24 - 0.000122952054375672*G0_25 - 2.25830303955321e-05*G0_26 + 2.88560943942907e-05*G0_27 + 0.00051439124789822*G0_28 - 0.00094095959981382*G0_29 - 0.00180664243164253*G0_30 - 0.000940959599813814*G0_31 + 0.00526937375895737*G0_32 - 0.00180664243164253*G0_33 + 0.000577121887885807*G0_34 - 0.000639852527873398*G0_35 - 0.000903321215821262*G0_36 - 0.000639852527873391*G0_37 + 0.00263468687947869*G0_38 - 0.000903321215821268*G0_39 - 0.000213284175957798*G0_40 + 0.000414022223918078*G0_41 + 6.27306399875886e-05*G0_42 + 0.000225830303955316*G0_43 - 0.000639852527873393*G0_44 - 0.00125461279975175*G0_45 - 0.0002132841759578*G0_46 + 0.000414022223918077*G0_47 + 6.2730639987587e-05*G0_48 + 0.000225830303955318*G0_49 - 0.000639852527873396*G0_50 - 0.00125461279975175*G0_51 - 0.000639852527873394*G0_52 - 0.00120442828776169*G0_53 + 0.000677490911865942*G0_54 + 0.000677490911865947*G0_55;
    A[33] = 0.000173437673437684*G0_0 + 9.59527869250139e-05*G0_1 + 9.59527869250134e-05*G0_2 + 0.00014965021475439*G0_3 - 0.000341254681532476*G0_4 + 0.000491808217502687*G0_5 - 0.000110405926378154*G0_6 + 0.000110405926378151*G0_7 - 0.000341254681532476*G0_8 + 0.000491808217502692*G0_9 - 0.000110405926378152*G0_10 + 0.000110405926378151*G0_11 - 4.0147609592055e-05*G0_12 + 0.00012797050557468*G0_13 + 0.00012797050557468*G0_14 - 4.01476095920573e-05*G0_15 - 0.000555375266023445*G0_16 + 0.000873210508627218*G0_17 - 0.0006239607657432*G0_18 + 5.52029631890764e-05*G0_19 - 0.000266814322080535*G0_20 + 0.000170627340766234*G0_21 - 0.000109569517844981*G0_22 - 2.00738047960294e-05*G0_23 - 0.00026681432208054*G0_24 + 0.000170627340766237*G0_25 - 0.000109569517844986*G0_26 - 2.0073804796027e-05*G0_27 + 0.00103714658112811*G0_28 + 0.000514391247898223*G0_29 + 0.00103714658112812*G0_30 - 0.00180664243164252*G0_31 - 0.00180664243164253*G0_32 + 0.00455006242043303*G0_33 + 0.000694219082529302*G0_34 + 6.27306399875941e-05*G0_35 + 0.000518573290564057*G0_36 - 0.00125461279975176*G0_37 - 0.000903321215821263*G0_38 + 0.00227503121021651*G0_39 + 0.000694219082529302*G0_40 + 6.27306399875888e-05*G0_41 + 0.000518573290564057*G0_42 - 0.00125461279975175*G0_43 - 0.000903321215821259*G0_44 + 0.00227503121021651*G0_45 + 0.000618942314544198*G0_46 - 0.000213284175957792*G0_47 + 0.000694219082529302*G0_48 - 0.0002132841759578*G0_49 + 0.000577121887885805*G0_50 + 0.000694219082529305*G0_51 - 0.00035129158393049*G0_52 + 0.000777859935846089*G0_53 + 0.00077785993584609*G0_54 + 0.00140516633572196*G0_55;
    A[34] = 0.000149650214754389*G0_0 + 0.000173437673437682*G0_1 + 9.59527869250138e-05*G0_2 + 9.59527869250132e-05*G0_3 - 4.01476095920553e-05*G0_4 + 0.000127970505574678*G0_5 + 0.000127970505574676*G0_6 - 4.01476095920574e-05*G0_7 - 0.000266814322080539*G0_8 + 0.000170627340766244*G0_9 - 0.000109569517844984*G0_10 - 2.00738047960284e-05*G0_11 - 0.000266814322080538*G0_12 + 0.000170627340766237*G0_13 - 0.000109569517844985*G0_14 - 2.00738047960284e-05*G0_15 + 0.000110405926378154*G0_16 - 0.000110405926378155*G0_17 + 0.00049180821750269*G0_18 - 0.000341254681532479*G0_19 + 0.00011040592637816*G0_20 - 0.000110405926378158*G0_21 + 0.000491808217502693*G0_22 - 0.000341254681532478*G0_23 + 5.52029631890766e-05*G0_24 - 0.000623960765743205*G0_25 + 0.000873210508627221*G0_26 - 0.000555375266023443*G0_27 + 0.000618942314544201*G0_28 - 0.000213284175957796*G0_29 + 0.000694219082529302*G0_30 - 0.000213284175957798*G0_31 + 0.000577121887885807*G0_32 + 0.000694219082529302*G0_33 + 0.00455006242043302*G0_34 - 0.00180664243164252*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164253*G0_37 + 0.000514391247898224*G0_38 + 0.00103714658112811*G0_39 + 0.00227503121021651*G0_40 - 0.00125461279975175*G0_41 + 0.000694219082529302*G0_42 - 0.000903321215821261*G0_43 + 6.2730639987589e-05*G0_44 + 0.000518573290564057*G0_45 + 0.00227503121021651*G0_46 - 0.00125461279975175*G0_47 + 0.000694219082529303*G0_48 - 0.000903321215821263*G0_49 + 6.27306399875884e-05*G0_50 + 0.000518573290564057*G0_51 + 0.00140516633572196*G0_52 - 0.00035129158393049*G0_53 + 0.000777859935846082*G0_54 + 0.000777859935846082*G0_55;
    A[35] = -0.000121496703527958*G0_0 - 1.77151327324942e-05*G0_1 - 0.000121496703527959*G0_2 + 3.98465025201175e-05*G0_3 - 5.52029631890778e-05*G0_4 - 0.000120442828776169*G0_5 - 7.0258316786095e-05*G0_6 - 9.28413471816303e-05*G0_7 + 0.000135498182373188*G0_8 - 0.000122952054375672*G0_9 - 2.25830303955319e-05*G0_10 + 2.88560943942902e-05*G0_11 - 8.78228959826357e-06*G0_12 - 0.000148044310370707*G0_13 + 0.000253431785549854*G0_14 - 2.76014815945372e-05*G0_15 - 5.52029631890786e-05*G0_16 - 0.000120442828776167*G0_17 - 7.02583167860999e-05*G0_18 - 9.28413471816293e-05*G0_19 + 0.000308634748738928*G0_20 - 4.51660607910587e-05*G0_21 - 4.5166060791065e-05*G0_22 + 0.000308634748738932*G0_23 - 2.76014815945394e-05*G0_24 + 0.000253431785549855*G0_25 - 0.000148044310370707*G0_26 - 8.78228959826281e-06*G0_27 - 0.000213284175957794*G0_28 + 0.000225830303955314*G0_29 - 0.00125461279975174*G0_30 + 0.000414022223918077*G0_31 - 0.000639852527873398*G0_32 + 6.27306399875941e-05*G0_33 - 0.00180664243164252*G0_34 + 0.00526937375895736*G0_35 - 0.00180664243164252*G0_36 - 0.000940959599813812*G0_37 - 0.000940959599813819*G0_38 + 0.000514391247898225*G0_39 - 0.00125461279975175*G0_40 + 0.000225830303955315*G0_41 - 0.000213284175957792*G0_42 - 0.000639852527873395*G0_43 + 0.000414022223918076*G0_44 + 6.27306399875928e-05*G0_45 - 0.000903321215821259*G0_46 - 0.000639852527873393*G0_47 + 0.000577121887885812*G0_48 + 0.00263468687947868*G0_49 - 0.000639852527873395*G0_50 - 0.000903321215821255*G0_51 + 0.000677490911865948*G0_52 - 0.000639852527873391*G0_53 + 0.000677490911865951*G0_54 - 0.00120442828776167*G0_55;
    A[36] = 9.5952786925014e-05*G0_0 + 0.000173437673437682*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250135e-05*G0_3 + 0.000110405926378156*G0_4 - 0.000110405926378156*G0_5 + 0.000491808217502685*G0_6 - 0.000341254681532478*G0_7 - 0.000266814322080538*G0_8 + 0.000170627340766243*G0_9 - 0.000109569517844986*G0_10 - 2.00738047960287e-05*G0_11 - 0.000555375266023442*G0_12 + 0.000873210508627219*G0_13 - 0.000623960765743205*G0_14 + 5.52029631890758e-05*G0_15 - 4.01476095920571e-05*G0_16 + 0.000127970505574677*G0_17 + 0.000127970505574683*G0_18 - 4.01476095920575e-05*G0_19 - 0.00034125468153247*G0_20 + 0.000491808217502679*G0_21 - 0.000110405926378146*G0_22 + 0.000110405926378152*G0_23 - 2.00738047960277e-05*G0_24 - 0.000109569517844989*G0_25 + 0.00017062734076624*G0_26 - 0.00026681432208054*G0_27 + 0.000694219082529302*G0_28 - 0.00125461279975175*G0_29 + 0.00227503121021651*G0_30 + 6.27306399875858e-05*G0_31 - 0.000903321215821262*G0_32 + 0.000518573290564057*G0_33 + 0.00103714658112812*G0_34 - 0.00180664243164252*G0_35 + 0.00455006242043302*G0_36 + 0.000514391247898213*G0_37 - 0.00180664243164252*G0_38 + 0.00103714658112812*G0_39 + 0.000694219082529305*G0_40 - 0.000213284175957794*G0_41 + 0.000618942314544195*G0_42 + 0.000577121887885804*G0_43 - 0.000213284175957796*G0_44 + 0.000694219082529302*G0_45 + 0.000518573290564058*G0_46 + 6.27306399875923e-05*G0_47 + 0.000694219082529303*G0_48 - 0.000903321215821264*G0_49 - 0.00125461279975176*G0_50 + 0.00227503121021651*G0_51 + 0.000777859935846097*G0_52 - 0.00035129158393049*G0_53 + 0.00140516633572196*G0_54 + 0.000777859935846081*G0_55;
    A[37] = -0.000121496703527961*G0_0 - 1.77151327324953e-05*G0_1 + 3.98465025201153e-05*G0_2 - 0.000121496703527961*G0_3 - 9.28413471816289e-05*G0_4 - 7.02583167860986e-05*G0_5 - 0.000120442828776168*G0_6 - 5.52029631890761e-05*G0_7 - 8.78228959826103e-06*G0_8 - 0.000148044310370709*G0_9 + 0.000253431785549852*G0_10 - 2.76014815945371e-05*G0_11 + 0.000135498182373189*G0_12 - 0.000122952054375671*G0_13 - 2.25830303955333e-05*G0_14 + 2.88560943942913e-05*G0_15 + 0.000308634748738932*G0_16 - 4.51660607910634e-05*G0_17 - 4.51660607910641e-05*G0_18 + 0.000308634748738933*G0_19 - 5.5202963189078e-05*G0_20 - 0.000120442828776168*G0_21 - 7.02583167861008e-05*G0_22 - 9.2841347181628e-05*G0_23 - 2.76014815945368e-05*G0_24 + 0.000253431785549852*G0_25 - 0.000148044310370706*G0_26 - 8.78228959826206e-06*G0_27 - 0.000213284175957805*G0_28 + 0.000414022223918077*G0_29 + 6.27306399875827e-05*G0_30 + 0.000225830303955316*G0_31 - 0.000639852527873391*G0_32 - 0.00125461279975176*G0_33 - 0.00180664243164253*G0_34 - 0.000940959599813813*G0_35 + 0.000514391247898213*G0_36 + 0.00526937375895736*G0_37 - 0.000940959599813812*G0_38 - 0.00180664243164253*G0_39 - 0.000903321215821267*G0_40 - 0.000639852527873388*G0_41 + 0.000577121887885799*G0_42 + 0.00263468687947868*G0_43 - 0.000639852527873392*G0_44 - 0.000903321215821268*G0_45 - 0.00125461279975176*G0_46 + 0.000225830303955317*G0_47 - 0.000213284175957802*G0_48 - 0.000639852527873392*G0_49 + 0.000414022223918077*G0_50 + 6.27306399875836e-05*G0_51 + 0.000677490911865939*G0_52 - 0.000639852527873405*G0_53 - 0.00120442828776168*G0_54 + 0.00067749091186594*G0_55;
    A[38] = 3.9846502520116e-05*G0_0 - 1.77151327324941e-05*G0_1 - 0.00012149670352796*G0_2 - 0.00012149670352796*G0_3 + 0.00030863474873893*G0_4 - 4.51660607910627e-05*G0_5 - 4.51660607910642e-05*G0_6 + 0.000308634748738933*G0_7 - 8.78228959826255e-06*G0_8 - 0.000148044310370708*G0_9 + 0.000253431785549854*G0_10 - 2.76014815945382e-05*G0_11 - 8.7822895982633e-06*G0_12 - 0.000148044310370706*G0_13 + 0.000253431785549854*G0_14 - 2.76014815945391e-05*G0_15 - 9.28413471816301e-05*G0_16 - 7.02583167860971e-05*G0_17 - 0.00012044282877617*G0_18 - 5.52029631890773e-05*G0_19 - 9.28413471816323e-05*G0_20 - 7.02583167860956e-05*G0_21 - 0.00012044282877617*G0_22 - 5.52029631890777e-05*G0_23 + 2.88560943942893e-05*G0_24 - 2.25830303955305e-05*G0_25 - 0.000122952054375672*G0_26 + 0.000135498182373189*G0_27 + 0.000577121887885812*G0_28 - 0.0006398525278734*G0_29 - 0.000903321215821257*G0_30 - 0.000639852527873394*G0_31 + 0.00263468687947869*G0_32 - 0.000903321215821263*G0_33 + 0.000514391247898224*G0_34 - 0.000940959599813819*G0_35 - 0.00180664243164252*G0_36 - 0.000940959599813812*G0_37 + 0.00526937375895736*G0_38 - 0.00180664243164253*G0_39 + 6.27306399875922e-05*G0_40 + 0.000414022223918074*G0_41 - 0.000213284175957793*G0_42 - 0.000639852527873395*G0_43 + 0.000225830303955316*G0_44 - 0.00125461279975175*G0_45 + 6.27306399875902e-05*G0_46 + 0.000414022223918075*G0_47 - 0.000213284175957795*G0_48 - 0.000639852527873394*G0_49 + 0.000225830303955315*G0_50 - 0.00125461279975175*G0_51 - 0.00120442828776168*G0_52 - 0.000639852527873393*G0_53 + 0.000677490911865953*G0_54 + 0.000677490911865954*G0_55;
    A[39] = 9.59527869250149e-05*G0_0 + 0.000173437673437682*G0_1 + 9.59527869250134e-05*G0_2 + 0.000149650214754389*G0_3 - 0.000341254681532475*G0_4 + 0.000491808217502686*G0_5 - 0.000110405926378153*G0_6 + 0.000110405926378151*G0_7 - 0.000555375266023442*G0_8 + 0.000873210508627223*G0_9 - 0.000623960765743204*G0_10 + 5.52029631890752e-05*G0_11 - 0.000266814322080538*G0_12 + 0.000170627340766238*G0_13 - 0.000109569517844986*G0_14 - 2.00738047960289e-05*G0_15 - 0.000341254681532478*G0_16 + 0.000491808217502684*G0_17 - 0.000110405926378148*G0_18 + 0.000110405926378153*G0_19 - 4.0147609592051e-05*G0_20 + 0.000127970505574674*G0_21 + 0.000127970505574685*G0_22 - 4.01476095920575e-05*G0_23 - 2.0073804796028e-05*G0_24 - 0.000109569517844988*G0_25 + 0.000170627340766239*G0_26 - 0.000266814322080539*G0_27 + 0.000694219082529303*G0_28 + 6.27306399875916e-05*G0_29 + 0.000518573290564058*G0_30 - 0.00125461279975175*G0_31 - 0.000903321215821268*G0_32 + 0.00227503121021651*G0_33 + 0.00103714658112811*G0_34 + 0.000514391247898225*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164253*G0_37 - 0.00180664243164253*G0_38 + 0.00455006242043303*G0_39 + 0.000518573290564055*G0_40 + 6.27306399875912e-05*G0_41 + 0.000694219082529301*G0_42 - 0.000903321215821263*G0_43 - 0.00125461279975175*G0_44 + 0.00227503121021651*G0_45 + 0.000694219082529302*G0_46 - 0.000213284175957792*G0_47 + 0.000618942314544196*G0_48 + 0.000577121887885806*G0_49 - 0.0002132841759578*G0_50 + 0.000694219082529304*G0_51 + 0.000777859935846089*G0_52 - 0.000351291583930489*G0_53 + 0.000777859935846086*G0_54 + 0.00140516633572195*G0_55;
    A[40] = 0.00014965021475439*G0_0 + 9.59527869250139e-05*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250131e-05*G0_3 - 0.000266814322080541*G0_4 + 0.00017062734076624*G0_5 - 0.000109569517844989*G0_6 - 2.00738047960297e-05*G0_7 - 4.01476095920551e-05*G0_8 + 0.000127970505574685*G0_9 + 0.000127970505574682*G0_10 - 4.0147609592057e-05*G0_11 - 2.00738047960266e-05*G0_12 - 0.000109569517844986*G0_13 + 0.00017062734076624*G0_14 - 0.000266814322080541*G0_15 + 0.000110405926378153*G0_16 - 0.000110405926378156*G0_17 + 0.00049180821750269*G0_18 - 0.000341254681532479*G0_19 + 5.52029631890818e-05*G0_20 - 0.00062396076574321*G0_21 + 0.000873210508627227*G0_22 - 0.000555375266023445*G0_23 + 0.000110405926378154*G0_24 - 0.000110405926378153*G0_25 + 0.000491808217502688*G0_26 - 0.000341254681532477*G0_27 + 0.000694219082529305*G0_28 - 0.000213284175957796*G0_29 + 0.000618942314544198*G0_30 + 0.000577121887885808*G0_31 - 0.000213284175957798*G0_32 + 0.000694219082529302*G0_33 + 0.00227503121021651*G0_34 - 0.00125461279975175*G0_35 + 0.000694219082529305*G0_36 - 0.000903321215821266*G0_37 + 6.27306399875922e-05*G0_38 + 0.000518573290564055*G0_39 + 0.00455006242043303*G0_40 - 0.00180664243164252*G0_41 + 0.00103714658112811*G0_42 - 0.00180664243164253*G0_43 + 0.000514391247898222*G0_44 + 0.00103714658112811*G0_45 + 0.00227503121021651*G0_46 - 0.00090332121582126*G0_47 + 0.000518573290564057*G0_48 - 0.00125461279975176*G0_49 + 6.273063998759e-05*G0_50 + 0.000694219082529305*G0_51 + 0.00140516633572197*G0_52 + 0.000777859935846091*G0_53 - 0.000351291583930495*G0_54 + 0.000777859935846079*G0_55;
    A[41] = -0.00012149670352796*G0_0 - 0.00012149670352796*G0_1 - 1.77151327324943e-05*G0_2 + 3.98465025201165e-05*G0_3 + 0.000135498182373188*G0_4 - 0.000122952054375672*G0_5 - 2.25830303955307e-05*G0_6 + 2.88560943942906e-05*G0_7 - 5.52029631890795e-05*G0_8 - 0.00012044282877617*G0_9 - 7.02583167860992e-05*G0_10 - 9.28413471816299e-05*G0_11 - 2.76014815945402e-05*G0_12 + 0.000253431785549855*G0_13 - 0.000148044310370707*G0_14 - 8.78228959826251e-06*G0_15 - 5.52029631890755e-05*G0_16 - 0.000120442828776169*G0_17 - 7.02583167860969e-05*G0_18 - 9.28413471816305e-05*G0_19 - 2.7601481594539e-05*G0_20 + 0.000253431785549854*G0_21 - 0.000148044310370707*G0_22 - 8.7822895982631e-06*G0_23 + 0.000308634748738933*G0_24 - 4.51660607910631e-05*G0_25 - 4.51660607910611e-05*G0_26 + 0.00030863474873893*G0_27 - 0.00125461279975176*G0_28 + 0.000225830303955316*G0_29 - 0.000213284175957798*G0_30 - 0.000639852527873396*G0_31 + 0.000414022223918078*G0_32 + 6.27306399875887e-05*G0_33 - 0.00125461279975175*G0_34 + 0.000225830303955315*G0_35 - 0.000213284175957794*G0_36 - 0.000639852527873388*G0_37 + 0.000414022223918074*G0_38 + 6.27306399875911e-05*G0_39 - 0.00180664243164252*G0_40 + 0.00526937375895737*G0_41 - 0.00180664243164252*G0_42 - 0.000940959599813813*G0_43 - 0.000940959599813821*G0_44 + 0.000514391247898219*G0_45 - 0.000903321215821264*G0_46 + 0.00263468687947868*G0_47 - 0.000903321215821262*G0_48 - 0.000639852527873394*G0_49 - 0.000639852527873393*G0_50 + 0.000577121887885808*G0_51 + 0.000677490911865955*G0_52 + 0.00067749091186594*G0_53 - 0.000639852527873397*G0_54 - 0.00120442828776168*G0_55;
    A[42] = 9.59527869250148e-05*G0_0 + 0.000149650214754389*G0_1 + 0.000173437673437682*G0_2 + 9.59527869250132e-05*G0_3 - 0.000266814322080539*G0_4 + 0.000170627340766238*G0_5 - 0.000109569517844987*G0_6 - 2.0073804796029e-05*G0_7 + 0.000110405926378159*G0_8 - 0.000110405926378149*G0_9 + 0.000491808217502689*G0_10 - 0.000341254681532478*G0_11 + 5.52029631890797e-05*G0_12 - 0.000623960765743205*G0_13 + 0.000873210508627224*G0_14 - 0.000555375266023444*G0_15 - 4.01476095920599e-05*G0_16 + 0.000127970505574679*G0_17 + 0.000127970505574682*G0_18 - 4.01476095920553e-05*G0_19 - 2.00738047960239e-05*G0_20 - 0.00010956951784499*G0_21 + 0.000170627340766242*G0_22 - 0.00026681432208054*G0_23 - 0.000341254681532478*G0_24 + 0.000491808217502687*G0_25 - 0.000110405926378155*G0_26 + 0.000110405926378157*G0_27 + 0.00227503121021651*G0_28 - 0.00125461279975175*G0_29 + 0.000694219082529305*G0_30 - 0.000903321215821262*G0_31 + 6.27306399875886e-05*G0_32 + 0.000518573290564057*G0_33 + 0.000694219082529302*G0_34 - 0.000213284175957792*G0_35 + 0.000618942314544195*G0_36 + 0.000577121887885799*G0_37 - 0.000213284175957793*G0_38 + 0.000694219082529301*G0_39 + 0.00103714658112811*G0_40 - 0.00180664243164252*G0_41 + 0.00455006242043303*G0_42 + 0.000514391247898217*G0_43 - 0.00180664243164252*G0_44 + 0.00103714658112812*G0_45 + 0.000518573290564056*G0_46 - 0.000903321215821259*G0_47 + 0.00227503121021651*G0_48 + 6.27306399875884e-05*G0_49 - 0.00125461279975176*G0_50 + 0.000694219082529305*G0_51 + 0.000777859935846083*G0_52 + 0.00140516633572197*G0_53 - 0.000351291583930491*G0_54 + 0.000777859935846081*G0_55;
    A[43] = -0.000121496703527961*G0_0 + 3.98465025201155e-05*G0_1 - 1.77151327324949e-05*G0_2 - 0.00012149670352796*G0_3 - 8.782289598262e-06*G0_4 - 0.000148044310370707*G0_5 + 0.000253431785549854*G0_6 - 2.76014815945377e-05*G0_7 - 9.28413471816292e-05*G0_8 - 7.02583167861001e-05*G0_9 - 0.000120442828776171*G0_10 - 5.52029631890752e-05*G0_11 + 2.88560943942906e-05*G0_12 - 2.25830303955315e-05*G0_13 - 0.000122952054375672*G0_14 + 0.000135498182373189*G0_15 + 0.000308634748738931*G0_16 - 4.51660607910613e-05*G0_17 - 4.51660607910662e-05*G0_18 + 0.000308634748738934*G0_19 - 2.76014815945403e-05*G0_20 + 0.000253431785549855*G0_21 - 0.000148044310370709*G0_22 - 8.78228959826142e-06*G0_23 - 5.52029631890779e-05*G0_24 - 0.00012044282877617*G0_25 - 7.02583167860979e-05*G0_26 - 9.28413471816305e-05*G0_27 + 6.27306399875876e-05*G0_28 + 0.000414022223918078*G0_29 - 0.000213284175957799*G0_30 - 0.000639852527873395*G0_31 + 0.000225830303955316*G0_32 - 0.00125461279975175*G0_33 - 0.000903321215821261*G0_34 - 0.000639852527873395*G0_35 + 0.000577121887885804*G0_36 + 0.00263468687947868*G0_37 - 0.000639852527873395*G0_38 - 0.000903321215821262*G0_39 - 0.00180664243164253*G0_40 - 0.000940959599813813*G0_41 + 0.000514391247898217*G0_42 + 0.00526937375895736*G0_43 - 0.000940959599813815*G0_44 - 0.00180664243164252*G0_45 - 0.00125461279975176*G0_46 - 0.000639852527873395*G0_47 + 6.27306399875886e-05*G0_48 + 0.000225830303955317*G0_49 + 0.000414022223918077*G0_50 - 0.000213284175957799*G0_51 + 0.000677490911865946*G0_52 - 0.00120442828776168*G0_53 - 0.000639852527873392*G0_54 + 0.000677490911865948*G0_55;
    A[44] = 3.98465025201163e-05*G0_0 - 0.000121496703527959*G0_1 - 1.77151327324942e-05*G0_2 - 0.00012149670352796*G0_3 - 8.78228959826302e-06*G0_4 - 0.000148044310370705*G0_5 + 0.000253431785549855*G0_6 - 2.76014815945383e-05*G0_7 + 0.000308634748738928*G0_8 - 4.51660607910608e-05*G0_9 - 4.51660607910636e-05*G0_10 + 0.000308634748738933*G0_11 - 2.76014815945403e-05*G0_12 + 0.000253431785549855*G0_13 - 0.000148044310370709*G0_14 - 8.78228959826086e-06*G0_15 - 9.28413471816307e-05*G0_16 - 7.02583167860934e-05*G0_17 - 0.000120442828776171*G0_18 - 5.52029631890757e-05*G0_19 + 2.88560943942889e-05*G0_20 - 2.25830303955286e-05*G0_21 - 0.000122952054375673*G0_22 + 0.00013549818237319*G0_23 - 9.2841347181631e-05*G0_24 - 7.02583167860968e-05*G0_25 - 0.000120442828776169*G0_26 - 5.52029631890792e-05*G0_27 - 0.000903321215821259*G0_28 - 0.000639852527873396*G0_29 + 0.00057712188788581*G0_30 + 0.00263468687947868*G0_31 - 0.000639852527873393*G0_32 - 0.000903321215821259*G0_33 + 6.2730639987589e-05*G0_34 + 0.000414022223918076*G0_35 - 0.000213284175957796*G0_36 - 0.000639852527873392*G0_37 + 0.000225830303955316*G0_38 - 0.00125461279975175*G0_39 + 0.000514391247898222*G0_40 - 0.000940959599813822*G0_41 - 0.00180664243164252*G0_42 - 0.000940959599813815*G0_43 + 0.00526937375895737*G0_44 - 0.00180664243164252*G0_45 + 6.27306399875901e-05*G0_46 - 0.000639852527873397*G0_47 - 0.00125461279975175*G0_48 + 0.000414022223918078*G0_49 + 0.000225830303955316*G0_50 - 0.000213284175957797*G0_51 - 0.00120442828776168*G0_52 + 0.000677490911865944*G0_53 - 0.00063985252787339*G0_54 + 0.000677490911865957*G0_55;
    A[45] = 9.59527869250154e-05*G0_0 + 9.59527869250138e-05*G0_1 + 0.000173437673437682*G0_2 + 0.000149650214754389*G0_3 - 0.000555375266023442*G0_4 + 0.000873210508627221*G0_5 - 0.000623960765743204*G0_6 + 5.5202963189075e-05*G0_7 - 0.000341254681532476*G0_8 + 0.000491808217502692*G0_9 - 0.000110405926378153*G0_10 + 0.000110405926378152*G0_11 - 2.00738047960268e-05*G0_12 - 0.000109569517844986*G0_13 + 0.000170627340766239*G0_14 - 0.00026681432208054*G0_15 - 0.000341254681532478*G0_16 + 0.000491808217502684*G0_17 - 0.000110405926378147*G0_18 + 0.000110405926378153*G0_19 - 2.00738047960233e-05*G0_20 - 0.000109569517844991*G0_21 + 0.000170627340766243*G0_22 - 0.000266814322080541*G0_23 - 4.01476095920557e-05*G0_24 + 0.000127970505574678*G0_25 + 0.00012797050557468*G0_26 - 4.01476095920555e-05*G0_27 + 0.000518573290564058*G0_28 + 6.27306399875904e-05*G0_29 + 0.000694219082529304*G0_30 - 0.000903321215821262*G0_31 - 0.00125461279975175*G0_32 + 0.00227503121021651*G0_33 + 0.000518573290564057*G0_34 + 6.27306399875928e-05*G0_35 + 0.000694219082529302*G0_36 - 0.000903321215821268*G0_37 - 0.00125461279975175*G0_38 + 0.00227503121021651*G0_39 + 0.00103714658112811*G0_40 + 0.000514391247898219*G0_41 + 0.00103714658112812*G0_42 - 0.00180664243164252*G0_43 - 0.00180664243164252*G0_44 + 0.00455006242043303*G0_45 + 0.000694219082529303*G0_46 + 0.000577121887885811*G0_47 + 0.000694219082529303*G0_48 - 0.0002132841759578*G0_49 - 0.000213284175957799*G0_50 + 0.000618942314544198*G0_51 + 0.000777859935846084*G0_52 + 0.000777859935846085*G0_53 - 0.000351291583930489*G0_54 + 0.00140516633572196*G0_55;
    A[46] = 0.00014965021475439*G0_0 + 9.5952786925014e-05*G0_1 + 9.59527869250144e-05*G0_2 + 0.000173437673437682*G0_3 - 2.00738047960285e-05*G0_4 - 0.000109569517844986*G0_5 + 0.000170627340766236*G0_6 - 0.000266814322080541*G0_7 - 2.00738047960273e-05*G0_8 - 0.000109569517844981*G0_9 + 0.00017062734076624*G0_10 - 0.00026681432208054*G0_11 - 4.01476095920554e-05*G0_12 + 0.000127970505574679*G0_13 + 0.00012797050557468*G0_14 - 4.0147609592057e-05*G0_15 + 5.52029631890751e-05*G0_16 - 0.000623960765743206*G0_17 + 0.000873210508627224*G0_18 - 0.000555375266023446*G0_19 + 0.000110405926378159*G0_20 - 0.000110405926378161*G0_21 + 0.000491808217502693*G0_22 - 0.000341254681532478*G0_23 + 0.000110405926378153*G0_24 - 0.000110405926378154*G0_25 + 0.000491808217502687*G0_26 - 0.000341254681532477*G0_27 + 0.000694219082529305*G0_28 + 0.000577121887885808*G0_29 + 0.000694219082529303*G0_30 - 0.000213284175957797*G0_31 - 0.0002132841759578*G0_32 + 0.000618942314544198*G0_33 + 0.00227503121021651*G0_34 - 0.000903321215821259*G0_35 + 0.000518573290564058*G0_36 - 0.00125461279975176*G0_37 + 6.27306399875902e-05*G0_38 + 0.000694219082529302*G0_39 + 0.00227503121021651*G0_40 - 0.000903321215821264*G0_41 + 0.000518573290564056*G0_42 - 0.00125461279975176*G0_43 + 6.27306399875901e-05*G0_44 + 0.000694219082529303*G0_45 + 0.00455006242043303*G0_46 - 0.00180664243164252*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164253*G0_49 + 0.00051439124789822*G0_50 + 0.00103714658112812*G0_51 + 0.00140516633572197*G0_52 + 0.00077785993584609*G0_53 + 0.000777859935846082*G0_54 - 0.000351291583930496*G0_55;
    A[47] = -0.000121496703527959*G0_0 - 0.00012149670352796*G0_1 + 3.98465025201169e-05*G0_2 - 1.77151327324943e-05*G0_3 + 2.88560943942882e-05*G0_4 - 2.25830303955297e-05*G0_5 - 0.000122952054375671*G0_6 + 0.00013549818237319*G0_7 - 2.76014815945401e-05*G0_8 + 0.000253431785549855*G0_9 - 0.000148044310370709*G0_10 - 8.78228959826156e-06*G0_11 - 5.52029631890794e-05*G0_12 - 0.000120442828776167*G0_13 - 7.02583167860972e-05*G0_14 - 9.28413471816308e-05*G0_15 - 2.76014815945387e-05*G0_16 + 0.000253431785549856*G0_17 - 0.000148044310370707*G0_18 - 8.78228959826232e-06*G0_19 - 5.52029631890796e-05*G0_20 - 0.000120442828776166*G0_21 - 7.02583167860999e-05*G0_22 - 9.28413471816303e-05*G0_23 + 0.000308634748738932*G0_24 - 4.51660607910618e-05*G0_25 - 4.51660607910617e-05*G0_26 + 0.000308634748738931*G0_27 - 0.00125461279975175*G0_28 - 0.000639852527873396*G0_29 + 6.27306399875923e-05*G0_30 + 0.000225830303955317*G0_31 + 0.000414022223918077*G0_32 - 0.000213284175957792*G0_33 - 0.00125461279975175*G0_34 - 0.000639852527873393*G0_35 + 6.27306399875922e-05*G0_36 + 0.000225830303955317*G0_37 + 0.000414022223918075*G0_38 - 0.000213284175957792*G0_39 - 0.00090332121582126*G0_40 + 0.00263468687947868*G0_41 - 0.000903321215821259*G0_42 - 0.000639852527873395*G0_43 - 0.000639852527873397*G0_44 + 0.000577121887885811*G0_45 - 0.00180664243164252*G0_46 + 0.00526937375895736*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813814*G0_49 - 0.000940959599813816*G0_50 + 0.000514391247898223*G0_51 + 0.000677490911865951*G0_52 + 0.000677490911865953*G0_53 - 0.00120442828776168*G0_54 - 0.000639852527873385*G0_55;
    A[48] = 9.59527869250146e-05*G0_0 + 0.00014965021475439*G0_1 + 9.59527869250141e-05*G0_2 + 0.000173437673437682*G0_3 - 2.00738047960275e-05*G0_4 - 0.000109569517844988*G0_5 + 0.000170627340766238*G0_6 - 0.000266814322080541*G0_7 + 5.52029631890796e-05*G0_8 - 0.000623960765743201*G0_9 + 0.000873210508627222*G0_10 - 0.000555375266023445*G0_11 + 0.000110405926378156*G0_12 - 0.000110405926378155*G0_13 + 0.00049180821750269*G0_14 - 0.000341254681532478*G0_15 - 2.00738047960319e-05*G0_16 - 0.000109569517844987*G0_17 + 0.000170627340766241*G0_18 - 0.00026681432208054*G0_19 - 4.01476095920517e-05*G0_20 + 0.000127970505574675*G0_21 + 0.000127970505574683*G0_22 - 4.01476095920575e-05*G0_23 - 0.000341254681532479*G0_24 + 0.000491808217502687*G0_25 - 0.000110405926378156*G0_26 + 0.000110405926378156*G0_27 + 0.00227503121021652*G0_28 - 0.000903321215821263*G0_29 + 0.000518573290564057*G0_30 - 0.00125461279975176*G0_31 + 6.2730639987587e-05*G0_32 + 0.000694219082529302*G0_33 + 0.000694219082529303*G0_34 + 0.000577121887885812*G0_35 + 0.000694219082529303*G0_36 - 0.000213284175957802*G0_37 - 0.000213284175957795*G0_38 + 0.000618942314544196*G0_39 + 0.000518573290564057*G0_40 - 0.000903321215821262*G0_41 + 0.00227503121021651*G0_42 + 6.27306399875886e-05*G0_43 - 0.00125461279975175*G0_44 + 0.000694219082529303*G0_45 + 0.00103714658112812*G0_46 - 0.00180664243164253*G0_47 + 0.00455006242043302*G0_48 + 0.000514391247898219*G0_49 - 0.00180664243164253*G0_50 + 0.00103714658112812*G0_51 + 0.00077785993584609*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846084*G0_54 - 0.000351291583930496*G0_55;
    A[49] = -0.00012149670352796*G0_0 + 3.98465025201154e-05*G0_1 - 0.000121496703527961*G0_2 - 1.77151327324944e-05*G0_3 - 2.76014815945374e-05*G0_4 + 0.000253431785549852*G0_5 - 0.000148044310370705*G0_6 - 8.78228959826165e-06*G0_7 + 2.88560943942908e-05*G0_8 - 2.25830303955338e-05*G0_9 - 0.000122952054375673*G0_10 + 0.00013549818237319*G0_11 - 9.2841347181629e-05*G0_12 - 7.02583167861006e-05*G0_13 - 0.000120442828776168*G0_14 - 5.52029631890751e-05*G0_15 - 2.76014815945379e-05*G0_16 + 0.000253431785549854*G0_17 - 0.000148044310370708*G0_18 - 8.78228959826146e-06*G0_19 + 0.000308634748738929*G0_20 - 4.51660607910591e-05*G0_21 - 4.51660607910671e-05*G0_22 + 0.000308634748738933*G0_23 - 5.52029631890776e-05*G0_24 - 0.000120442828776169*G0_25 - 7.02583167860994e-05*G0_26 - 9.28413471816293e-05*G0_27 + 6.27306399875889e-05*G0_28 - 0.000639852527873397*G0_29 - 0.00125461279975176*G0_30 + 0.000414022223918078*G0_31 + 0.000225830303955318*G0_32 - 0.0002132841759578*G0_33 - 0.000903321215821263*G0_34 + 0.00263468687947868*G0_35 - 0.000903321215821264*G0_36 - 0.000639852527873392*G0_37 - 0.000639852527873394*G0_38 + 0.000577121887885806*G0_39 - 0.00125461279975176*G0_40 - 0.000639852527873395*G0_41 + 6.27306399875884e-05*G0_42 + 0.000225830303955317*G0_43 + 0.000414022223918078*G0_44 - 0.0002132841759578*G0_45 - 0.00180664243164253*G0_46 - 0.000940959599813815*G0_47 + 0.000514391247898219*G0_48 + 0.00526937375895736*G0_49 - 0.000940959599813815*G0_50 - 0.00180664243164253*G0_51 + 0.00067749091186594*G0_52 - 0.00120442828776169*G0_53 + 0.000677490911865945*G0_54 - 0.000639852527873396*G0_55;
    A[50] = 3.98465025201151e-05*G0_0 - 0.00012149670352796*G0_1 - 0.000121496703527961*G0_2 - 1.77151327324943e-05*G0_3 - 2.76014815945381e-05*G0_4 + 0.000253431785549854*G0_5 - 0.000148044310370708*G0_6 - 8.78228959826218e-06*G0_7 - 2.76014815945396e-05*G0_8 + 0.000253431785549853*G0_9 - 0.000148044310370708*G0_10 - 8.78228959826212e-06*G0_11 + 0.00030863474873893*G0_12 - 4.51660607910607e-05*G0_13 - 4.51660607910667e-05*G0_14 + 0.000308634748738932*G0_15 + 2.88560943942924e-05*G0_16 - 2.25830303955331e-05*G0_17 - 0.000122952054375672*G0_18 + 0.000135498182373189*G0_19 - 9.28413471816301e-05*G0_20 - 7.02583167860996e-05*G0_21 - 0.000120442828776169*G0_22 - 5.52029631890768e-05*G0_23 - 9.28413471816283e-05*G0_24 - 7.02583167860997e-05*G0_25 - 0.000120442828776166*G0_26 - 5.52029631890785e-05*G0_27 - 0.000903321215821266*G0_28 + 0.00263468687947868*G0_29 - 0.000903321215821264*G0_30 - 0.000639852527873394*G0_31 - 0.000639852527873396*G0_32 + 0.000577121887885805*G0_33 + 6.27306399875883e-05*G0_34 - 0.000639852527873395*G0_35 - 0.00125461279975176*G0_36 + 0.000414022223918077*G0_37 + 0.000225830303955315*G0_38 - 0.0002132841759578*G0_39 + 6.273063998759e-05*G0_40 - 0.000639852527873393*G0_41 - 0.00125461279975176*G0_42 + 0.000414022223918077*G0_43 + 0.000225830303955316*G0_44 - 0.000213284175957799*G0_45 + 0.00051439124789822*G0_46 - 0.000940959599813816*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813815*G0_49 + 0.00526937375895737*G0_50 - 0.00180664243164253*G0_51 - 0.00120442828776168*G0_52 + 0.000677490911865945*G0_53 + 0.000677490911865944*G0_54 - 0.0006398525278734*G0_55;
    A[51] = 9.59527869250149e-05*G0_0 + 9.5952786925014e-05*G0_1 + 0.00014965021475439*G0_2 + 0.000173437673437682*G0_3 + 5.52029631890776e-05*G0_4 - 0.000623960765743206*G0_5 + 0.000873210508627219*G0_6 - 0.000555375266023444*G0_7 - 2.00738047960262e-05*G0_8 - 0.000109569517844981*G0_9 + 0.00017062734076624*G0_10 - 0.000266814322080541*G0_11 - 0.000341254681532475*G0_12 + 0.000491808217502686*G0_13 - 0.000110405926378151*G0_14 + 0.000110405926378151*G0_15 - 2.00738047960316e-05*G0_16 - 0.000109569517844986*G0_17 + 0.000170627340766242*G0_18 - 0.000266814322080541*G0_19 - 0.000341254681532471*G0_20 + 0.00049180821750268*G0_21 - 0.000110405926378147*G0_22 + 0.000110405926378152*G0_23 - 4.01476095920573e-05*G0_24 + 0.00012797050557468*G0_25 + 0.000127970505574679*G0_26 - 4.01476095920548e-05*G0_27 + 0.00051857329056406*G0_28 - 0.000903321215821261*G0_29 + 0.00227503121021651*G0_30 + 6.27306399875874e-05*G0_31 - 0.00125461279975176*G0_32 + 0.000694219082529305*G0_33 + 0.000518573290564057*G0_34 - 0.000903321215821255*G0_35 + 0.00227503121021651*G0_36 + 6.27306399875837e-05*G0_37 - 0.00125461279975175*G0_38 + 0.000694219082529304*G0_39 + 0.000694219082529305*G0_40 + 0.000577121887885808*G0_41 + 0.000694219082529305*G0_42 - 0.000213284175957799*G0_43 - 0.000213284175957797*G0_44 + 0.000618942314544198*G0_45 + 0.00103714658112812*G0_46 + 0.000514391247898223*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164253*G0_49 - 0.00180664243164253*G0_50 + 0.00455006242043303*G0_51 + 0.000777859935846091*G0_52 + 0.000777859935846096*G0_53 + 0.00140516633572196*G0_54 - 0.000351291583930492*G0_55;
    A[52] = -0.00028002957690459*G0_0 + 0.000204250963799584*G0_1 + 0.000204250963799588*G0_2 + 0.000204250963799585*G0_3 - 0.000301107071940423*G0_4 + 6.27306399875843e-05*G0_5 + 6.27306399875892e-05*G0_6 - 0.000301107071940424*G0_7 - 0.000301107071940419*G0_8 + 6.27306399875933e-05*G0_9 + 6.27306399875902e-05*G0_10 - 0.000301107071940421*G0_11 - 0.000301107071940421*G0_12 + 6.27306399875873e-05*G0_13 + 6.27306399875914e-05*G0_14 - 0.000301107071940421*G0_15 + 0.000727675423856014*G0_16 - 0.000903321215821265*G0_17 + 0.00110405926378155*G0_18 - 0.000777859935846091*G0_19 + 0.000727675423856021*G0_20 - 0.000903321215821267*G0_21 + 0.00110405926378155*G0_22 - 0.00077785993584609*G0_23 + 0.000727675423856016*G0_24 - 0.000903321215821261*G0_25 + 0.00110405926378154*G0_26 - 0.000777859935846089*G0_27 - 0.000351291583930488*G0_28 - 0.000639852527873396*G0_29 - 0.000351291583930489*G0_30 - 0.000639852527873398*G0_31 - 0.000639852527873394*G0_32 - 0.00035129158393049*G0_33 + 0.00140516633572196*G0_34 + 0.000677490911865948*G0_35 + 0.000777859935846097*G0_36 + 0.000677490911865939*G0_37 - 0.00120442828776168*G0_38 + 0.000777859935846089*G0_39 + 0.00140516633572197*G0_40 + 0.000677490911865954*G0_41 + 0.000777859935846083*G0_42 + 0.000677490911865946*G0_43 - 0.00120442828776168*G0_44 + 0.000777859935846084*G0_45 + 0.00140516633572197*G0_46 + 0.000677490911865951*G0_47 + 0.000777859935846089*G0_48 + 0.000677490911865939*G0_49 - 0.00120442828776168*G0_50 + 0.000777859935846091*G0_51 + 0.0165608889567232*G0_52 - 0.00225830303955316*G0_53 - 0.00225830303955316*G0_54 - 0.00225830303955316*G0_55;
    A[53] = 0.000204250963799588*G0_0 - 0.000280029576904591*G0_1 + 0.000204250963799587*G0_2 + 0.000204250963799586*G0_3 - 0.000301107071940422*G0_4 + 6.27306399875896e-05*G0_5 + 6.2730639987591e-05*G0_6 - 0.000301107071940425*G0_7 + 0.000727675423856017*G0_8 - 0.000903321215821253*G0_9 + 0.00110405926378155*G0_10 - 0.000777859935846092*G0_11 + 0.000727675423856017*G0_12 - 0.000903321215821258*G0_13 + 0.00110405926378155*G0_14 - 0.000777859935846091*G0_15 - 0.000301107071940427*G0_16 + 6.27306399875908e-05*G0_17 + 6.27306399875919e-05*G0_18 - 0.000301107071940421*G0_19 - 0.000301107071940415*G0_20 + 6.27306399875842e-05*G0_21 + 6.27306399875927e-05*G0_22 - 0.000301107071940425*G0_23 - 0.000777859935846091*G0_24 + 0.00110405926378155*G0_25 - 0.000903321215821261*G0_26 + 0.000727675423856019*G0_27 + 0.00140516633572197*G0_28 + 0.000677490911865943*G0_29 + 0.000777859935846095*G0_30 + 0.000677490911865948*G0_31 - 0.00120442828776169*G0_32 + 0.000777859935846089*G0_33 - 0.00035129158393049*G0_34 - 0.000639852527873391*G0_35 - 0.00035129158393049*G0_36 - 0.000639852527873405*G0_37 - 0.000639852527873392*G0_38 - 0.000351291583930489*G0_39 + 0.000777859935846091*G0_40 + 0.00067749091186594*G0_41 + 0.00140516633572197*G0_42 - 0.00120442828776168*G0_43 + 0.000677490911865944*G0_44 + 0.000777859935846085*G0_45 + 0.00077785993584609*G0_46 + 0.000677490911865953*G0_47 + 0.00140516633572197*G0_48 - 0.00120442828776169*G0_49 + 0.000677490911865945*G0_50 + 0.000777859935846096*G0_51 - 0.00225830303955316*G0_52 + 0.0165608889567232*G0_53 - 0.00225830303955316*G0_54 - 0.00225830303955316*G0_55;
    A[54] = 0.000204250963799588*G0_0 + 0.000204250963799586*G0_1 - 0.000280029576904595*G0_2 + 0.000204250963799587*G0_3 + 0.000727675423856023*G0_4 - 0.000903321215821267*G0_5 + 0.00110405926378154*G0_6 - 0.000777859935846087*G0_7 - 0.000301107071940419*G0_8 + 6.27306399875969e-05*G0_9 + 6.27306399875853e-05*G0_10 - 0.000301107071940421*G0_11 - 0.000777859935846088*G0_12 + 0.00110405926378154*G0_13 - 0.000903321215821266*G0_14 + 0.00072767542385602*G0_15 - 0.000301107071940427*G0_16 + 6.27306399875907e-05*G0_17 + 6.27306399875872e-05*G0_18 - 0.000301107071940421*G0_19 - 0.000777859935846082*G0_20 + 0.00110405926378154*G0_21 - 0.000903321215821257*G0_22 + 0.000727675423856019*G0_23 - 0.000301107071940424*G0_24 + 6.2730639987586e-05*G0_25 + 6.27306399875872e-05*G0_26 - 0.00030110707194042*G0_27 + 0.000777859935846086*G0_28 + 0.000677490911865949*G0_29 + 0.00140516633572196*G0_30 - 0.00120442828776169*G0_31 + 0.000677490911865942*G0_32 + 0.00077785993584609*G0_33 + 0.000777859935846082*G0_34 + 0.000677490911865951*G0_35 + 0.00140516633572196*G0_36 - 0.00120442828776168*G0_37 + 0.000677490911865953*G0_38 + 0.000777859935846086*G0_39 - 0.000351291583930495*G0_40 - 0.000639852527873397*G0_41 - 0.000351291583930491*G0_42 - 0.000639852527873392*G0_43 - 0.00063985252787339*G0_44 - 0.000351291583930489*G0_45 + 0.000777859935846082*G0_46 - 0.00120442828776168*G0_47 + 0.000777859935846084*G0_48 + 0.000677490911865945*G0_49 + 0.000677490911865944*G0_50 + 0.00140516633572196*G0_51 - 0.00225830303955316*G0_52 - 0.00225830303955316*G0_53 + 0.0165608889567232*G0_54 - 0.00225830303955316*G0_55;
    A[55] = 0.000204250963799587*G0_0 + 0.000204250963799585*G0_1 + 0.000204250963799584*G0_2 - 0.000280029576904594*G0_3 - 0.000777859935846086*G0_4 + 0.00110405926378155*G0_5 - 0.000903321215821265*G0_6 + 0.000727675423856017*G0_7 - 0.000777859935846087*G0_8 + 0.00110405926378155*G0_9 - 0.000903321215821261*G0_10 + 0.000727675423856019*G0_11 - 0.000301107071940417*G0_12 + 6.27306399875867e-05*G0_13 + 6.27306399875886e-05*G0_14 - 0.000301107071940422*G0_15 - 0.000777859935846089*G0_16 + 0.00110405926378154*G0_17 - 0.000903321215821256*G0_18 + 0.000727675423856022*G0_19 - 0.000301107071940413*G0_20 + 6.2730639987581e-05*G0_21 + 6.27306399875953e-05*G0_22 - 0.000301107071940424*G0_23 - 0.000301107071940422*G0_24 + 6.27306399875853e-05*G0_25 + 6.2730639987588e-05*G0_26 - 0.000301107071940421*G0_27 + 0.000777859935846087*G0_28 - 0.00120442828776168*G0_29 + 0.000777859935846087*G0_30 + 0.000677490911865955*G0_31 + 0.000677490911865947*G0_32 + 0.00140516633572196*G0_33 + 0.000777859935846082*G0_34 - 0.00120442828776167*G0_35 + 0.000777859935846082*G0_36 + 0.00067749091186594*G0_37 + 0.000677490911865953*G0_38 + 0.00140516633572195*G0_39 + 0.000777859935846079*G0_40 - 0.00120442828776168*G0_41 + 0.000777859935846081*G0_42 + 0.000677490911865948*G0_43 + 0.000677490911865957*G0_44 + 0.00140516633572196*G0_45 - 0.000351291583930496*G0_46 - 0.000639852527873385*G0_47 - 0.000351291583930496*G0_48 - 0.000639852527873396*G0_49 - 0.0006398525278734*G0_50 - 0.000351291583930492*G0_51 - 0.00225830303955316*G0_52 - 0.00225830303955315*G0_53 - 0.00225830303955316*G0_54 + 0.0165608889567231*G0_55;
  }

  /// Tabulate the tensor for the contribution from a local cell
  /// using the specified reference cell quadrature points/weights
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int num_quadrature_points,
                               const double * const * quadrature_points,
                               const double* quadrature_weights) const
  {
    throw std::runtime_error("Quadrature version of tabulate_tensor not available when using the FFC tensor representation.");
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson3d_5_form_0: public ufc::form
{
public:

  /// Constructor
  poisson3d_5_form_0() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_form_0()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(IndexSum(Product(SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 0), MultiIndex((Index(0),), {Index(0): 3})), SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 1), MultiIndex((Index(0),), {Index(0): 3}))), MultiIndex((Index(0),), {Index(0): 3})), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell domains
  virtual unsigned int num_cell_domains() const
  {
    return 1;
  }

  /// Return the number of exterior facet domains
  virtual unsigned int num_exterior_facet_domains() const
  {
    return 0;
  }

  /// Return the number of interior facet domains
  virtual unsigned int num_interior_facet_domains() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dofmap for argument function i
  virtual ufc::dofmap* create_dofmap(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_cell_integral_0_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson3d_5_form_1: public ufc::form
{
public:

  /// Constructor
  poisson3d_5_form_1() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_form_1()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(Product(Argument(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 0), Coefficient(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 0)), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell domains
  virtual unsigned int num_cell_domains() const
  {
    return 1;
  }

  /// Return the number of exterior facet domains
  virtual unsigned int num_exterior_facet_domains() const
  {
    return 0;
  }

  /// Return the number of interior facet domains
  virtual unsigned int num_interior_facet_domains() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dofmap for argument function i
  virtual ufc::dofmap* create_dofmap(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_cell_integral_1_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

// Standard library includes
#include <string>

// DOLFIN includes
#include <dolfin/common/NoDeleter.h>
#include <dolfin/fem/FiniteElement.h>
#include <dolfin/fem/DofMap.h>
#include <dolfin/fem/Form.h>
#include <dolfin/function/FunctionSpace.h>
#include <dolfin/function/GenericFunction.h>
#include <dolfin/function/CoefficientAssigner.h>
#include <dolfin/adaptivity/ErrorControl.h>
#include <dolfin/adaptivity/GoalFunctional.h>

namespace Poisson3D_5
{

class CoefficientSpace_f: public dolfin::FunctionSpace
{
public:

  CoefficientSpace_f(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  CoefficientSpace_f(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~CoefficientSpace_f()
  {
  }

};

class Form_0_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  Form_0_FunctionSpace_0(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_0_FunctionSpace_0()
  {
  }

};

class Form_0_FunctionSpace_1: public dolfin::FunctionSpace
{
public:

  Form_0_FunctionSpace_1(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_0_FunctionSpace_1()
  {
  }

};

class Form_0: public dolfin::Form
{
public:

  // Constructor
  Form_0(const dolfin::FunctionSpace& V1, const dolfin::FunctionSpace& V0):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);
    _function_spaces[1] = reference_to_no_delete_pointer(V1);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Constructor
  Form_0(boost::shared_ptr<const dolfin::FunctionSpace> V1, boost::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Destructor
  ~Form_0()
  {}

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string& name) const
  {

    dolfin::error("No coefficients.");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {

    dolfin::error("No coefficients.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_0_FunctionSpace_0 TestSpace;
  typedef Form_0_FunctionSpace_1 TrialSpace;

  // Coefficients
};

class Form_1_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  Form_1_FunctionSpace_0(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_1_FunctionSpace_0()
  {
  }

};

typedef CoefficientSpace_f Form_1_FunctionSpace_1;

class Form_1: public dolfin::Form
{
public:

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0, const dolfin::GenericFunction& f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    this->f = f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0, boost::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    this->f = *f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0, const dolfin::GenericFunction& f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0, boost::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = *f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Destructor
  ~Form_1()
  {}

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::error("Invalid coefficient.");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::error("Invalid coefficient.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_1_FunctionSpace_0 TestSpace;
  typedef Form_1_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::CoefficientAssigner f;
};

// Class typedefs
typedef Form_0 BilinearForm;
typedef Form_0 JacobianForm;
typedef Form_1 LinearForm;
typedef Form_1 ResidualForm;
typedef Form_0::TestSpace FunctionSpace;

}

#endif
